package net.minecraft.advancements.critereon;

import com.google.gson.JsonElement;
import javax.annotation.Nullable;
import net.minecraft.advancements.CriterionTriggerInstance;
import net.minecraft.server.level.ServerPlayer;
import net.minecraft.world.level.dimension.DimensionType;
import net.minecraft.util.GsonHelper;
import com.google.gson.JsonDeserializationContext;
import com.google.gson.JsonObject;
import net.minecraft.resources.ResourceLocation;

public class ChangeDimensionTrigger extends SimpleCriterionTrigger<TriggerInstance> {
    private static final ResourceLocation ID;
    
    @Override
    public ResourceLocation getId() {
        return ChangeDimensionTrigger.ID;
    }
    
    @Override
    public TriggerInstance createInstance(final JsonObject jsonObject, final JsonDeserializationContext jsonDeserializationContext) {
        final DimensionType cbf4 = jsonObject.has("from") ? DimensionType.getByName(new ResourceLocation(GsonHelper.getAsString(jsonObject, "from"))) : null;
        final DimensionType cbf5 = jsonObject.has("to") ? DimensionType.getByName(new ResourceLocation(GsonHelper.getAsString(jsonObject, "to"))) : null;
        return new TriggerInstance(cbf4, cbf5);
    }
    
    public void trigger(final ServerPlayer xe, final DimensionType cbf2, final DimensionType cbf3) {
        this.trigger(xe.getAdvancements(), a -> a.matches(cbf2, cbf3));
    }
    
    static {
        ID = new ResourceLocation("changed_dimension");
    }
    
    public static class TriggerInstance extends AbstractCriterionTriggerInstance {
        @Nullable
        private final DimensionType from;
        @Nullable
        private final DimensionType to;
        
        public TriggerInstance(@Nullable final DimensionType cbf1, @Nullable final DimensionType cbf2) {
            super(ChangeDimensionTrigger.ID);
            this.from = cbf1;
            this.to = cbf2;
        }
        
        public static TriggerInstance changedDimensionTo(final DimensionType cbf) {
            return new TriggerInstance(null, cbf);
        }
        
        public boolean matches(final DimensionType cbf1, final DimensionType cbf2) {
            return (this.from == null || this.from == cbf1) && (this.to == null || this.to == cbf2);
        }
        
        @Override
        public JsonElement serializeToJson() {
            final JsonObject jsonObject2 = new JsonObject();
            if (this.from != null) {
                jsonObject2.addProperty("from", DimensionType.getName(this.from).toString());
            }
            if (this.to != null) {
                jsonObject2.addProperty("to", DimensionType.getName(this.to).toString());
            }
            return jsonObject2;
        }
    }
}
