package net.minecraft.client;

import java.util.Collections;
import net.minecraft.world.inventory.SmokerMenu;
import net.minecraft.world.inventory.BlastFurnaceMenu;
import net.minecraft.world.inventory.FurnaceMenu;
import net.minecraft.world.inventory.InventoryMenu;
import net.minecraft.world.inventory.CraftingMenu;
import net.minecraft.world.inventory.RecipeBookMenu;
import net.minecraft.world.item.ItemStack;
import net.minecraft.world.item.CreativeModeTab;
import net.minecraft.world.item.BlockItem;
import net.minecraft.world.item.crafting.RecipeType;
import java.util.Iterator;
import com.google.common.collect.Table;
import net.minecraft.world.item.crafting.Recipe;
import com.google.common.collect.HashBasedTable;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import net.minecraft.client.gui.screens.recipebook.RecipeCollection;
import java.util.List;
import java.util.Map;
import net.minecraft.world.item.crafting.RecipeManager;
import net.minecraft.stats.RecipeBook;

public class ClientRecipeBook extends RecipeBook {
    private final RecipeManager recipes;
    private final Map<RecipeBookCategories, List<RecipeCollection>> collectionsByTab;
    private final List<RecipeCollection> collections;
    
    public ClientRecipeBook(final RecipeManager bgu) {
        this.collectionsByTab = Maps.newHashMap();
        this.collections = Lists.newArrayList();
        this.recipes = bgu;
    }
    
    public void setupCollections() {
        this.collections.clear();
        this.collectionsByTab.clear();
        final Table<RecipeBookCategories, String, RecipeCollection> table2 = HashBasedTable.create();
        for (final Recipe<?> bgt4 : this.recipes.getRecipes()) {
            if (bgt4.isSpecial()) {
                continue;
            }
            final RecipeBookCategories dbs5 = getCategory(bgt4);
            final String string6 = bgt4.getGroup();
            RecipeCollection diq7;
            if (string6.isEmpty()) {
                diq7 = this.createCollection(dbs5);
            }
            else {
                diq7 = table2.get(dbs5, string6);
                if (diq7 == null) {
                    diq7 = this.createCollection(dbs5);
                    table2.put(dbs5, string6, diq7);
                }
            }
            diq7.add(bgt4);
        }
    }
    
    private RecipeCollection createCollection(final RecipeBookCategories dbs) {
        final RecipeCollection diq3 = new RecipeCollection();
        this.collections.add(diq3);
        this.collectionsByTab.computeIfAbsent(dbs, dbs -> Lists.newArrayList()).add(diq3);
        if (dbs == RecipeBookCategories.FURNACE_BLOCKS || dbs == RecipeBookCategories.FURNACE_FOOD || dbs == RecipeBookCategories.FURNACE_MISC) {
            this.addToCollection(RecipeBookCategories.FURNACE_SEARCH, diq3);
        }
        else if (dbs == RecipeBookCategories.BLAST_FURNACE_BLOCKS || dbs == RecipeBookCategories.BLAST_FURNACE_MISC) {
            this.addToCollection(RecipeBookCategories.BLAST_FURNACE_SEARCH, diq3);
        }
        else if (dbs == RecipeBookCategories.SMOKER_FOOD) {
            this.addToCollection(RecipeBookCategories.SMOKER_SEARCH, diq3);
        }
        else if (dbs == RecipeBookCategories.STONECUTTER) {
            this.addToCollection(RecipeBookCategories.STONECUTTER, diq3);
        }
        else if (dbs == RecipeBookCategories.CAMPFIRE) {
            this.addToCollection(RecipeBookCategories.CAMPFIRE, diq3);
        }
        else {
            this.addToCollection(RecipeBookCategories.SEARCH, diq3);
        }
        return diq3;
    }
    
    private void addToCollection(final RecipeBookCategories dbs, final RecipeCollection diq) {
        this.collectionsByTab.computeIfAbsent(dbs, dbs -> Lists.newArrayList()).add(diq);
    }
    
    private static RecipeBookCategories getCategory(final Recipe<?> bgt) {
        final RecipeType<?> bgw2 = bgt.getType();
        if (bgw2 == RecipeType.SMELTING) {
            if (bgt.getResultItem().getItem().isEdible()) {
                return RecipeBookCategories.FURNACE_FOOD;
            }
            if (bgt.getResultItem().getItem() instanceof BlockItem) {
                return RecipeBookCategories.FURNACE_BLOCKS;
            }
            return RecipeBookCategories.FURNACE_MISC;
        }
        else if (bgw2 == RecipeType.BLASTING) {
            if (bgt.getResultItem().getItem() instanceof BlockItem) {
                return RecipeBookCategories.BLAST_FURNACE_BLOCKS;
            }
            return RecipeBookCategories.BLAST_FURNACE_MISC;
        }
        else {
            if (bgw2 == RecipeType.SMOKING) {
                return RecipeBookCategories.SMOKER_FOOD;
            }
            if (bgw2 == RecipeType.STONECUTTING) {
                return RecipeBookCategories.STONECUTTER;
            }
            if (bgw2 == RecipeType.CAMPFIRE_COOKING) {
                return RecipeBookCategories.CAMPFIRE;
            }
            final ItemStack bek3 = bgt.getResultItem();
            final CreativeModeTab bda4 = bek3.getItem().getItemCategory();
            if (bda4 == CreativeModeTab.TAB_BUILDING_BLOCKS) {
                return RecipeBookCategories.BUILDING_BLOCKS;
            }
            if (bda4 == CreativeModeTab.TAB_TOOLS || bda4 == CreativeModeTab.TAB_COMBAT) {
                return RecipeBookCategories.EQUIPMENT;
            }
            if (bda4 == CreativeModeTab.TAB_REDSTONE) {
                return RecipeBookCategories.REDSTONE;
            }
            return RecipeBookCategories.MISC;
        }
    }
    
    public static List<RecipeBookCategories> getCategories(final RecipeBookMenu<?> bbq) {
        if (bbq instanceof CraftingMenu || bbq instanceof InventoryMenu) {
            return Lists.<RecipeBookCategories>newArrayList(RecipeBookCategories.SEARCH, RecipeBookCategories.EQUIPMENT, RecipeBookCategories.BUILDING_BLOCKS, RecipeBookCategories.MISC, RecipeBookCategories.REDSTONE);
        }
        if (bbq instanceof FurnaceMenu) {
            return Lists.<RecipeBookCategories>newArrayList(RecipeBookCategories.FURNACE_SEARCH, RecipeBookCategories.FURNACE_FOOD, RecipeBookCategories.FURNACE_BLOCKS, RecipeBookCategories.FURNACE_MISC);
        }
        if (bbq instanceof BlastFurnaceMenu) {
            return Lists.<RecipeBookCategories>newArrayList(RecipeBookCategories.BLAST_FURNACE_SEARCH, RecipeBookCategories.BLAST_FURNACE_BLOCKS, RecipeBookCategories.BLAST_FURNACE_MISC);
        }
        if (bbq instanceof SmokerMenu) {
            return Lists.<RecipeBookCategories>newArrayList(RecipeBookCategories.SMOKER_SEARCH, RecipeBookCategories.SMOKER_FOOD);
        }
        return Lists.newArrayList();
    }
    
    public List<RecipeCollection> getCollections() {
        return this.collections;
    }
    
    public List<RecipeCollection> getCollection(final RecipeBookCategories dbs) {
        return this.collectionsByTab.getOrDefault(dbs, Collections.<RecipeCollection>emptyList());
    }
}
