package net.minecraft.client;

import java.util.AbstractList;
import org.apache.logging.log4j.LogManager;
import net.minecraft.world.item.crafting.Recipe;
import java.util.Optional;
import java.util.concurrent.CompletionStage;
import net.minecraft.client.resources.PackAdapterV4;
import net.minecraft.client.resources.LegacyResourcePackAdapter;
import net.minecraft.server.packs.metadata.pack.PackMetadataSection;
import java.util.function.Supplier;
import net.minecraft.util.profiling.ProfilerFiller;
import net.minecraft.world.level.biome.Biome;
import net.minecraft.world.level.dimension.end.TheEndDimension;
import net.minecraft.world.level.dimension.NetherDimension;
import net.minecraft.client.gui.screens.WinScreen;
import com.mojang.authlib.GameProfile;
import com.google.common.collect.Multimap;
import java.nio.ByteOrder;
import com.mojang.blaze3d.platform.GlUtil;
import net.minecraft.nbt.StringTag;
import net.minecraft.nbt.ListTag;
import net.minecraft.nbt.Tag;
import net.minecraft.world.item.PlayerHeadItem;
import net.minecraft.nbt.CompoundTag;
import net.minecraft.world.level.block.entity.BlockEntity;
import net.minecraft.world.entity.player.Inventory;
import net.minecraft.world.item.SpawnEggItem;
import net.minecraft.world.entity.boss.enderdragon.EndCrystal;
import net.minecraft.world.entity.decoration.ArmorStand;
import net.minecraft.world.entity.vehicle.Boat;
import net.minecraft.world.entity.vehicle.AbstractMinecart;
import net.minecraft.world.entity.decoration.ItemFrame;
import net.minecraft.world.entity.decoration.LeashFenceKnotEntity;
import net.minecraft.world.level.ItemLike;
import net.minecraft.world.item.Items;
import net.minecraft.world.entity.decoration.Painting;
import net.minecraft.world.level.BlockGetter;
import net.minecraft.client.renderer.blockentity.BlockEntityRenderDispatcher;
import net.minecraft.client.multiplayer.ClientPacketListener;
import com.mojang.authlib.AuthenticationService;
import net.minecraft.client.gui.screens.ProgressScreen;
import java.net.SocketAddress;
import com.mojang.authlib.GameProfileRepository;
import net.minecraft.world.level.storage.LevelStorage;
import net.minecraft.network.protocol.login.ServerboundHelloPacket;
import net.minecraft.network.protocol.handshake.ClientIntentionPacket;
import net.minecraft.network.ConnectionProtocol;
import net.minecraft.network.PacketListener;
import net.minecraft.client.multiplayer.ClientHandshakePacketListenerImpl;
import net.minecraft.client.gui.screens.LevelLoadingScreen;
import net.minecraft.server.level.progress.ChunkProgressListener;
import net.minecraft.server.level.progress.ProcessorChunkProgressListener;
import net.minecraft.world.level.block.entity.SkullBlockEntity;
import net.minecraft.server.players.GameProfileCache;
import net.minecraft.world.level.storage.LevelData;
import net.minecraft.server.MinecraftServer;
import net.minecraft.world.level.LevelSettings;
import net.minecraft.client.gui.screens.ChatScreen;
import net.minecraft.world.entity.player.ChatVisiblity;
import net.minecraft.network.protocol.Packet;
import net.minecraft.network.protocol.game.ServerboundPlayerActionPacket;
import net.minecraft.client.gui.screens.advancements.AdvancementsScreen;
import net.minecraft.client.gui.screens.inventory.InventoryScreen;
import net.minecraft.client.gui.screens.inventory.CreativeModeInventoryScreen;
import net.minecraft.CrashReportCategory;
import net.minecraft.world.Difficulty;
import net.minecraft.client.gui.screens.InBedChatScreen;
import net.minecraft.world.InteractionResult;
import net.minecraft.world.phys.EntityHitResult;
import net.minecraft.core.Direction;
import net.minecraft.core.BlockPos;
import net.minecraft.world.InteractionHand;
import net.minecraft.world.phys.BlockHitResult;
import net.minecraft.client.gui.screens.PauseScreen;
import com.mojang.blaze3d.vertex.BufferBuilder;
import java.text.DecimalFormatSymbols;
import java.text.DecimalFormat;
import net.minecraft.util.Mth;
import com.mojang.blaze3d.vertex.DefaultVertexFormat;
import com.mojang.blaze3d.vertex.Tesselator;
import net.minecraft.util.profiling.ProfileResults;
import net.minecraft.util.profiling.ResultField;
import net.minecraft.client.gui.screens.GenericDirtMessageScreen;
import net.minecraft.client.renderer.FogRenderer;
import net.minecraft.client.gui.chat.NarratorChatListener;
import net.minecraft.client.gui.screens.multiplayer.JoinMultiplayerScreen;
import net.minecraft.client.gui.screens.DeathScreen;
import net.minecraft.client.renderer.texture.TextureAtlasSprite;
import com.google.common.collect.UnmodifiableIterator;
import net.minecraft.client.resources.model.BakedModel;
import net.minecraft.client.renderer.block.BlockModelShaper;
import net.minecraft.client.gui.screens.MenuScreens;
import java.util.Locale;
import net.minecraft.world.level.block.RenderShape;
import net.minecraft.world.level.block.state.BlockState;
import net.minecraft.world.level.block.Block;
import java.util.Date;
import java.text.SimpleDateFormat;
import net.minecraft.DefaultUncaughtExceptionHandler;
import java.util.Iterator;
import net.minecraft.client.searchtree.MutableSearchTree;
import net.minecraft.client.gui.screens.recipebook.RecipeCollection;
import net.minecraft.world.item.CreativeModeTab;
import net.minecraft.world.item.Item;
import net.minecraft.core.NonNullList;
import net.minecraft.client.searchtree.ReloadableIdSearchTree;
import net.minecraft.tags.ItemTags;
import net.minecraft.world.item.ItemStack;
import net.minecraft.client.searchtree.ReloadableSearchTree;
import java.util.stream.Stream;
import net.minecraft.core.Registry;
import net.minecraft.ChatFormatting;
import net.minecraft.world.entity.player.Player;
import net.minecraft.world.item.TooltipFlag;
import net.minecraft.ReportedException;
import net.minecraft.client.gui.screens.OutOfMemoryScreen;
import net.minecraft.network.chat.Component;
import net.minecraft.network.chat.TranslatableComponent;
import net.minecraft.client.gui.components.toasts.SystemToast;
import java.util.Collection;
import java.util.Collections;
import net.minecraft.network.chat.TextComponent;
import java.io.InputStream;
import java.util.concurrent.Executor;
import java.util.stream.Collector;
import java.util.stream.Collectors;
import java.util.function.Function;
import net.minecraft.server.packs.repository.UnopenedPack;
import net.minecraft.server.packs.Pack;
import java.util.List;
import net.minecraft.client.gui.screens.LoadingOverlay;
import net.minecraft.client.gui.screens.ConnectScreen;
import net.minecraft.client.gui.screens.TitleScreen;
import net.minecraft.world.level.Level;
import net.minecraft.client.resources.FoliageColorReloadListener;
import net.minecraft.client.resources.GrassColorReloadListener;
import net.minecraft.server.packs.resources.ResourceManager;
import net.minecraft.server.packs.resources.PreparableReloadListener;
import net.minecraft.server.packs.resources.SimpleReloadableResourceManager;
import java.io.IOException;
import net.minecraft.server.packs.PackType;
import net.minecraft.SharedConstants;
import com.mojang.blaze3d.platform.DisplayData;
import com.mojang.blaze3d.systems.RenderSystem;
import net.minecraft.util.datafix.DataFixers;
import net.minecraft.network.chat.KeybindComponent;
import net.minecraft.server.Bootstrap;
import com.mojang.authlib.yggdrasil.YggdrasilAuthenticationService;
import java.util.UUID;
import net.minecraft.server.packs.repository.FolderRepositorySource;
import net.minecraft.server.packs.repository.RepositorySource;
import com.google.common.collect.Queues;
import net.minecraft.Util;
import net.minecraft.client.main.GameConfig;
import java.util.Queue;
import net.minecraft.CrashReport;
import net.minecraft.client.gui.screens.Overlay;
import net.minecraft.client.gui.screens.Screen;
import net.minecraft.world.phys.HitResult;
import net.minecraft.world.entity.Entity;
import net.minecraft.network.Connection;
import net.minecraft.client.multiplayer.ServerData;
import net.minecraft.client.server.IntegratedServer;
import net.minecraft.client.player.LocalPlayer;
import net.minecraft.client.multiplayer.ClientLevel;
import javax.annotation.Nullable;
import net.minecraft.client.multiplayer.MultiPlayerGameMode;
import net.minecraft.client.tutorial.Tutorial;
import net.minecraft.client.gui.components.toasts.ToastComponent;
import net.minecraft.client.resources.MobEffectTextureManager;
import net.minecraft.client.resources.PaintingTextureManager;
import net.minecraft.client.renderer.block.BlockRenderDispatcher;
import net.minecraft.client.resources.model.ModelManager;
import net.minecraft.client.resources.SkinManager;
import com.mojang.authlib.minecraft.MinecraftSessionService;
import net.minecraft.client.resources.SplashManager;
import net.minecraft.client.gui.font.FontManager;
import net.minecraft.client.sounds.MusicManager;
import net.minecraft.client.sounds.SoundManager;
import com.mojang.blaze3d.pipeline.RenderTarget;
import net.minecraft.client.color.item.ItemColors;
import net.minecraft.client.color.block.BlockColors;
import net.minecraft.client.resources.language.LanguageManager;
import net.minecraft.client.resources.UnopenedResourcePack;
import net.minecraft.server.packs.repository.PackRepository;
import net.minecraft.client.resources.ClientPackSource;
import net.minecraft.server.packs.resources.ReloadableResourceManager;
import net.minecraft.util.profiling.GameProfiler;
import net.minecraft.util.FrameTimer;
import net.minecraft.world.level.storage.LevelStorageSource;
import java.net.Proxy;
import net.minecraft.client.gui.Gui;
import net.minecraft.server.level.progress.StoringChunkProgressListener;
import java.util.concurrent.atomic.AtomicReference;
import net.minecraft.client.renderer.debug.DebugRenderer;
import net.minecraft.client.renderer.GameRenderer;
import net.minecraft.client.gui.Font;
import net.minecraft.client.searchtree.SearchRegistry;
import net.minecraft.client.particle.ParticleEngine;
import net.minecraft.client.renderer.ItemInHandRenderer;
import net.minecraft.client.renderer.entity.ItemRenderer;
import net.minecraft.client.renderer.entity.EntityRenderDispatcher;
import net.minecraft.client.renderer.LevelRenderer;
import net.minecraft.client.renderer.RenderBuffers;
import net.minecraft.world.Snooper;
import com.mojang.blaze3d.platform.Window;
import net.minecraft.client.renderer.VirtualScreen;
import com.mojang.datafixers.DataFixer;
import net.minecraft.client.renderer.texture.TextureManager;
import com.mojang.authlib.properties.PropertyMap;
import java.io.File;
import net.minecraft.util.Unit;
import java.util.concurrent.CompletableFuture;
import net.minecraft.resources.ResourceLocation;
import org.apache.logging.log4j.Logger;
import com.mojang.blaze3d.platform.WindowEventHandler;
import net.minecraft.world.SnooperPopulator;
import net.minecraft.util.thread.ReentrantBlockableEventLoop;

public class Minecraft extends ReentrantBlockableEventLoop<Runnable> implements SnooperPopulator, WindowEventHandler {
    private static Minecraft instance;
    private static final Logger LOGGER;
    public static final boolean ON_OSX;
    public static final ResourceLocation DEFAULT_FONT;
    public static final ResourceLocation ALT_FONT;
    private static final CompletableFuture<Unit> RESOURCE_RELOAD_INITIAL_TASK;
    private final File resourcePackDirectory;
    private final PropertyMap profileProperties;
    private final TextureManager textureManager;
    private final DataFixer fixerUpper;
    private final VirtualScreen virtualScreen;
    private final Window window;
    private final Timer timer;
    private final Snooper snooper;
    private final RenderBuffers renderBuffers;
    public final LevelRenderer levelRenderer;
    private final EntityRenderDispatcher entityRenderDispatcher;
    private final ItemRenderer itemRenderer;
    private final ItemInHandRenderer itemInHandRenderer;
    public final ParticleEngine particleEngine;
    private final SearchRegistry searchRegistry;
    private final User user;
    public final Font font;
    public final GameRenderer gameRenderer;
    public final DebugRenderer debugRenderer;
    private final AtomicReference<StoringChunkProgressListener> progressListener;
    public final Gui gui;
    public final Options options;
    private final HotbarManager hotbarManager;
    public final MouseHandler mouseHandler;
    public final KeyboardHandler keyboardHandler;
    public final File gameDirectory;
    private final String launchedVersion;
    private final String versionType;
    private final Proxy proxy;
    private final LevelStorageSource levelSource;
    public final FrameTimer frameTimer;
    private final boolean is64bit;
    private final boolean demo;
    private final GameProfiler profiler;
    private final ReloadableResourceManager resourceManager;
    private final ClientPackSource clientPackSource;
    private final PackRepository<UnopenedResourcePack> resourcePackRepository;
    private final LanguageManager languageManager;
    private final BlockColors blockColors;
    private final ItemColors itemColors;
    private final RenderTarget mainRenderTarget;
    private final SoundManager soundManager;
    private final MusicManager musicManager;
    private final FontManager fontManager;
    private final SplashManager splashManager;
    private final MinecraftSessionService minecraftSessionService;
    private final SkinManager skinManager;
    private final ModelManager modelManager;
    private final BlockRenderDispatcher blockRenderer;
    private final PaintingTextureManager paintingTextures;
    private final MobEffectTextureManager mobEffectTextures;
    private final ToastComponent toast;
    private final Game game;
    private final Tutorial tutorial;
    public static byte[] reserve;
    @Nullable
    public MultiPlayerGameMode gameMode;
    @Nullable
    public ClientLevel level;
    @Nullable
    public LocalPlayer player;
    @Nullable
    private IntegratedServer singleplayerServer;
    @Nullable
    private ServerData currentServer;
    @Nullable
    private Connection pendingConnection;
    private boolean isLocalServer;
    @Nullable
    public Entity cameraEntity;
    @Nullable
    public Entity crosshairPickEntity;
    @Nullable
    public HitResult hitResult;
    private int rightClickDelay;
    protected int missTime;
    private boolean pause;
    private float pausePartialTick;
    private long lastNanoTime;
    private long lastTime;
    private int frames;
    public boolean noRender;
    @Nullable
    public Screen screen;
    @Nullable
    public Overlay overlay;
    private boolean connectedToRealms;
    private Thread gameThread;
    private volatile boolean running;
    @Nullable
    private CrashReport delayedCrash;
    private static int fps;
    public String fpsString;
    public boolean chunkPath;
    public boolean chunkVisibility;
    public boolean smartCull;
    private boolean windowActive;
    private final Queue<Runnable> progressTasks;
    @Nullable
    private CompletableFuture<Void> pendingReload;
    private String debugPath;
    
    public Minecraft(final GameConfig djv) {
        super("Client");
        this.timer = new Timer(20.0f, 0L);
        this.snooper = new Snooper("client", this, Util.getMillis());
        this.searchRegistry = new SearchRegistry();
        this.progressListener = new AtomicReference<StoringChunkProgressListener>();
        this.frameTimer = new FrameTimer();
        this.profiler = new GameProfiler(() -> this.timer.ticks);
        this.game = new Game(this);
        this.lastNanoTime = Util.getNanos();
        this.running = true;
        this.fpsString = "";
        this.smartCull = true;
        this.progressTasks = Queues.newConcurrentLinkedQueue();
        this.debugPath = "root";
        Minecraft.instance = this;
        this.gameDirectory = djv.location.gameDirectory;
        final File file3 = djv.location.assetDirectory;
        this.resourcePackDirectory = djv.location.resourcePackDirectory;
        this.launchedVersion = djv.game.launchVersion;
        this.versionType = djv.game.versionType;
        this.profileProperties = djv.user.profileProperties;
        this.clientPackSource = new ClientPackSource(new File(this.gameDirectory, "server-resource-packs"), djv.location.getAssetIndex());
        (this.resourcePackRepository = new PackRepository<UnopenedResourcePack>(Minecraft::createClientPackAdapter)).addSource(this.clientPackSource);
        this.resourcePackRepository.addSource(new FolderRepositorySource(this.resourcePackDirectory));
        this.proxy = djv.user.proxy;
        this.minecraftSessionService = new YggdrasilAuthenticationService(this.proxy, UUID.randomUUID().toString()).createMinecraftSessionService();
        this.user = djv.user.user;
        Minecraft.LOGGER.info("Setting user: {}", this.user.getName());
        Minecraft.LOGGER.debug("(Session ID is {})", this.user.getSessionId());
        this.demo = djv.game.demo;
        this.is64bit = checkIs64Bit();
        this.singleplayerServer = null;
        String string4;
        int integer5;
        if (djv.server.hostname != null) {
            string4 = djv.server.hostname;
            integer5 = djv.server.port;
        }
        else {
            string4 = null;
            integer5 = 0;
        }
        Bootstrap.bootStrap();
        Bootstrap.validate();
        KeybindComponent.keyResolver = KeyMapping::createNameSupplier;
        this.fixerUpper = DataFixers.getDataFixer();
        this.toast = new ToastComponent(this);
        this.tutorial = new Tutorial(this);
        this.gameThread = Thread.currentThread();
        this.options = new Options(this, this.gameDirectory);
        this.hotbarManager = new HotbarManager(this.gameDirectory, this.fixerUpper);
        this.startTimerHackThread();
        Minecraft.LOGGER.info("Backend library: {}", RenderSystem.getBackendDescription());
        DisplayData cxe6;
        if (this.options.overrideHeight > 0 && this.options.overrideWidth > 0) {
            cxe6 = new DisplayData(this.options.overrideWidth, this.options.overrideHeight, djv.display.fullscreenWidth, djv.display.fullscreenHeight, djv.display.isFullscreen);
        }
        else {
            cxe6 = djv.display;
        }
        Util.timeSource = RenderSystem.initBackendSystem();
        this.virtualScreen = new VirtualScreen(this);
        this.window = this.virtualScreen.newWindow(cxe6, this.options.fullscreenVideoModeString, "Minecraft " + SharedConstants.getCurrentVersion().getName());
        this.setWindowActive(true);
        try {
            final InputStream inputStream7 = this.getClientPackSource().getVanillaPack().getResource(PackType.CLIENT_RESOURCES, new ResourceLocation("icons/icon_16x16.png"));
            final InputStream inputStream8 = this.getClientPackSource().getVanillaPack().getResource(PackType.CLIENT_RESOURCES, new ResourceLocation("icons/icon_32x32.png"));
            this.window.setIcon(inputStream7, inputStream8);
        }
        catch (IOException iOException7) {
            Minecraft.LOGGER.error("Couldn't set icon", (Throwable)iOException7);
        }
        this.window.setFramerateLimit(this.options.framerateLimit);
        (this.mouseHandler = new MouseHandler(this)).setup(this.window.getWindow());
        (this.keyboardHandler = new KeyboardHandler(this)).setup(this.window.getWindow());
        RenderSystem.initRenderer(this.options.glDebugVerbosity, false);
        (this.mainRenderTarget = new RenderTarget(this.window.getWidth(), this.window.getHeight(), true, Minecraft.ON_OSX)).setClearColor(0.0f, 0.0f, 0.0f, 0.0f);
        this.resourceManager = new SimpleReloadableResourceManager(PackType.CLIENT_RESOURCES, this.gameThread);
        this.options.loadResourcePacks(this.resourcePackRepository);
        this.resourcePackRepository.reload();
        this.languageManager = new LanguageManager(this.options.languageCode);
        this.resourceManager.registerReloadListener(this.languageManager);
        this.textureManager = new TextureManager(this.resourceManager);
        this.resourceManager.registerReloadListener(this.textureManager);
        this.skinManager = new SkinManager(this.textureManager, new File(file3, "skins"), this.minecraftSessionService);
        this.levelSource = new LevelStorageSource(this.gameDirectory.toPath().resolve("saves"), this.gameDirectory.toPath().resolve("backups"), this.fixerUpper);
        this.soundManager = new SoundManager(this.resourceManager, this.options);
        this.resourceManager.registerReloadListener(this.soundManager);
        this.splashManager = new SplashManager(this.user);
        this.resourceManager.registerReloadListener(this.splashManager);
        this.musicManager = new MusicManager(this);
        this.fontManager = new FontManager(this.textureManager, this.isEnforceUnicode());
        this.resourceManager.registerReloadListener(this.fontManager.getReloadListener());
        final Font dcf7 = this.fontManager.get(Minecraft.DEFAULT_FONT);
        if (dcf7 == null) {
            throw new IllegalStateException("Default font is null");
        }
        (this.font = dcf7).setBidirectional(this.languageManager.isBidirectional());
        this.resourceManager.registerReloadListener(new GrassColorReloadListener());
        this.resourceManager.registerReloadListener(new FoliageColorReloadListener());
        this.window.setErrorSection("Startup");
        RenderSystem.setupDefaultState(0, 0, this.window.getWidth(), this.window.getHeight());
        this.window.setErrorSection("Post startup");
        this.blockColors = BlockColors.createDefault();
        this.itemColors = ItemColors.createDefault(this.blockColors);
        this.modelManager = new ModelManager(this.textureManager, this.blockColors, this.options.mipmapLevels);
        this.resourceManager.registerReloadListener(this.modelManager);
        this.itemRenderer = new ItemRenderer(this.textureManager, this.modelManager, this.itemColors);
        this.entityRenderDispatcher = new EntityRenderDispatcher(this.textureManager, this.itemRenderer, this.resourceManager, this.font, this.options);
        this.itemInHandRenderer = new ItemInHandRenderer(this);
        this.resourceManager.registerReloadListener(this.itemRenderer);
        this.renderBuffers = new RenderBuffers();
        this.gameRenderer = new GameRenderer(this, this.resourceManager, this.renderBuffers);
        this.resourceManager.registerReloadListener(this.gameRenderer);
        this.blockRenderer = new BlockRenderDispatcher(this.modelManager.getBlockModelShaper(), this.blockColors);
        this.resourceManager.registerReloadListener(this.blockRenderer);
        this.levelRenderer = new LevelRenderer(this, this.renderBuffers);
        this.resourceManager.registerReloadListener(this.levelRenderer);
        this.createSearchTrees();
        this.resourceManager.registerReloadListener(this.searchRegistry);
        this.particleEngine = new ParticleEngine(this.level, this.textureManager);
        this.resourceManager.registerReloadListener(this.particleEngine);
        this.paintingTextures = new PaintingTextureManager(this.textureManager);
        this.resourceManager.registerReloadListener(this.paintingTextures);
        this.mobEffectTextures = new MobEffectTextureManager(this.textureManager);
        this.resourceManager.registerReloadListener(this.mobEffectTextures);
        this.gui = new Gui(this);
        this.debugRenderer = new DebugRenderer(this);
        RenderSystem.setErrorCallback(this::onFullscreenError);
        if (this.options.fullscreen && !this.window.isFullscreen()) {
            this.window.toggleFullScreen();
            this.options.fullscreen = this.window.isFullscreen();
        }
        this.window.updateVsync(this.options.enableVsync);
        this.window.updateRawMouseInput(this.options.rawMouseInput);
        this.window.setDefaultErrorCallback();
        this.resizeDisplay();
        if (string4 != null) {
            this.setScreen(new ConnectScreen(new TitleScreen(), this, string4, integer5));
        }
        else {
            this.setScreen(new TitleScreen(true));
        }
        LoadingOverlay.registerTextures(this);
        final List<Pack> list8 = this.resourcePackRepository.getSelected().stream().map(UnopenedPack::open).collect(Collectors.toList());
        this.setOverlay(new LoadingOverlay(this, this.resourceManager.createFullReload(Util.backgroundExecutor(), this, Minecraft.RESOURCE_RELOAD_INITIAL_TASK, list8), optional -> Util.<Throwable>ifElse(optional, this::rollbackResourcePacks, () -> {
            this.languageManager.reload(list8);
            if (SharedConstants.IS_RUNNING_IN_IDE) {
                this.selfTest();
            }
        }), false));
    }
    
    private void rollbackResourcePacks(final Throwable throwable) {
        if (this.resourcePackRepository.getSelected().size() > 1) {
            if (throwable instanceof SimpleReloadableResourceManager.ResourcePackLoadingFailure) {
                final Component lf3 = new TextComponent(((SimpleReloadableResourceManager.ResourcePackLoadingFailure)throwable).getPack().getName());
            }
            else {
                final Component lf3 = null;
            }
            Minecraft.LOGGER.info("Caught error loading resourcepacks, removing all selected resourcepacks", throwable);
            this.resourcePackRepository.setSelected(Collections.emptyList());
            this.options.resourcePacks.clear();
            this.options.incompatibleResourcePacks.clear();
            this.options.save();
            final ToastComponent ddz3;
            final Component lf4;
            this.reloadResourcePacks().thenRun(() -> {
                ddz3 = this.getToasts();
                SystemToast.addOrUpdate(ddz3, SystemToast.SystemToastIds.PACK_LOAD_FAILURE, new TranslatableComponent("resourcePack.load_fail", new Object[0]), lf4);
            });
        }
        else {
            Util.throwAsRuntime(throwable);
        }
    }
    
    public void run() {
        this.gameThread = Thread.currentThread();
        try {
            boolean boolean2 = false;
            while (this.running) {
                if (this.delayedCrash != null) {
                    crash(this.delayedCrash);
                    return;
                }
                try {
                    this.runTick(!boolean2);
                }
                catch (OutOfMemoryError outOfMemoryError3) {
                    if (boolean2) {
                        throw outOfMemoryError3;
                    }
                    this.emergencySave();
                    this.setScreen(new OutOfMemoryScreen());
                    System.gc();
                    Minecraft.LOGGER.fatal("Out of memory", (Throwable)outOfMemoryError3);
                    boolean2 = true;
                }
            }
        }
        catch (ReportedException q2) {
            this.fillReport(q2.getReport());
            this.emergencySave();
            Minecraft.LOGGER.fatal("Reported exception thrown!", (Throwable)q2);
            crash(q2.getReport());
        }
        catch (Throwable throwable2) {
            final CrashReport h3 = this.fillReport(new CrashReport("Unexpected error", throwable2));
            Minecraft.LOGGER.fatal("Unreported exception thrown!", throwable2);
            this.emergencySave();
            crash(h3);
        }
    }
    
    private void createSearchTrees() {
        final ReloadableSearchTree<ItemStack> edd2 = new ReloadableSearchTree<ItemStack>(bek -> bek.getTooltipLines(null, TooltipFlag.Default.NORMAL).stream().map(lf -> ChatFormatting.stripFormatting(lf.getString()).trim()).filter(string -> !string.isEmpty()), bek -> Stream.<ResourceLocation>of(Registry.ITEM.getKey(bek.getItem())));
        final ReloadableIdSearchTree<ItemStack> edc3 = new ReloadableIdSearchTree<ItemStack>(bek -> ItemTags.getAllTags().getMatchingTags(bek.getItem()).stream());
        final NonNullList<ItemStack> fy4 = NonNullList.<ItemStack>create();
        for (final Item bef6 : Registry.ITEM) {
            bef6.fillItemCategory(CreativeModeTab.TAB_SEARCH, fy4);
        }
        final ReloadableIdSearchTree<ItemStack> reloadableIdSearchTree;
        final ReloadableIdSearchTree<ItemStack> reloadableIdSearchTree2;
        fy4.forEach(bek -> {
            reloadableIdSearchTree.add(bek);
            reloadableIdSearchTree2.add(bek);
            return;
        });
        final ReloadableSearchTree<RecipeCollection> edd3 = new ReloadableSearchTree<RecipeCollection>(diq -> diq.getRecipes().stream().flatMap(bgt -> bgt.getResultItem().getTooltipLines(null, TooltipFlag.Default.NORMAL).stream()).map(lf -> ChatFormatting.stripFormatting(lf.getString()).trim()).filter(string -> !string.isEmpty()), diq -> diq.getRecipes().stream().map(bgt -> Registry.ITEM.getKey(bgt.getResultItem().getItem())));
        this.searchRegistry.<ItemStack>register(SearchRegistry.CREATIVE_NAMES, edd2);
        this.searchRegistry.<ItemStack>register(SearchRegistry.CREATIVE_TAGS, edc3);
        this.searchRegistry.<RecipeCollection>register(SearchRegistry.RECIPE_COLLECTIONS, edd3);
    }
    
    private void onFullscreenError(final int integer, final long long2) {
        this.options.enableVsync = false;
        this.options.save();
    }
    
    private static boolean checkIs64Bit() {
        final String[] array;
        final String[] arr1 = array = new String[] { "sun.arch.data.model", "com.ibm.vm.bitmode", "os.arch" };
        for (final String string5 : array) {
            final String string6 = System.getProperty(string5);
            if (string6 != null && string6.contains("64")) {
                return true;
            }
        }
        return false;
    }
    
    public RenderTarget getMainRenderTarget() {
        return this.mainRenderTarget;
    }
    
    public String getLaunchedVersion() {
        return this.launchedVersion;
    }
    
    public String getVersionType() {
        return this.versionType;
    }
    
    private void startTimerHackThread() {
        final Thread thread2 = new Thread("Timer hack thread") {
            @Override
            public void run() {
                while (Minecraft.this.running) {
                    try {
                        Thread.sleep(2147483647L);
                    }
                    catch (InterruptedException ex) {}
                }
            }
        };
        thread2.setDaemon(true);
        thread2.setUncaughtExceptionHandler(new DefaultUncaughtExceptionHandler(Minecraft.LOGGER));
        thread2.start();
    }
    
    public void delayCrash(final CrashReport h) {
        this.delayedCrash = h;
    }
    
    public static void crash(final CrashReport h) {
        final File file2 = new File(getInstance().gameDirectory, "crash-reports");
        final File file3 = new File(file2, "crash-" + new SimpleDateFormat("yyyy-MM-dd_HH.mm.ss").format(new Date()) + "-client.txt");
        Bootstrap.realStdoutPrintln(h.getFriendlyReport());
        if (h.getSaveFile() != null) {
            Bootstrap.realStdoutPrintln("#@!@# Game crashed! Crash report saved to: #@!@# " + h.getSaveFile());
            System.exit(-1);
        }
        else if (h.saveToFile(file3)) {
            Bootstrap.realStdoutPrintln("#@!@# Game crashed! Crash report saved to: #@!@# " + file3.getAbsolutePath());
            System.exit(-1);
        }
        else {
            Bootstrap.realStdoutPrintln("#@?@# Game crashed! Crash report could not be saved. #@?@#");
            System.exit(-2);
        }
    }
    
    public boolean isEnforceUnicode() {
        return this.options.forceUnicodeFont;
    }
    
    public CompletableFuture<Void> reloadResourcePacks() {
        if (this.pendingReload != null) {
            return this.pendingReload;
        }
        final CompletableFuture<Void> completableFuture2 = new CompletableFuture<Void>();
        if (this.overlay instanceof LoadingOverlay) {
            return this.pendingReload = completableFuture2;
        }
        this.resourcePackRepository.reload();
        final List<Pack> list3 = this.resourcePackRepository.getSelected().stream().map(UnopenedPack::open).collect(Collectors.toList());
        final List<Pack> list4;
        final CompletableFuture<Object> completableFuture3;
        this.setOverlay(new LoadingOverlay(this, this.resourceManager.createFullReload(Util.backgroundExecutor(), this, Minecraft.RESOURCE_RELOAD_INITIAL_TASK, list3), optional -> Util.<Throwable>ifElse(optional, this::rollbackResourcePacks, () -> {
            this.languageManager.reload(list4);
            this.levelRenderer.allChanged();
            completableFuture3.complete(null);
        }), true));
        return completableFuture2;
    }
    
    private void selfTest() {
        boolean boolean2 = false;
        final BlockModelShaper drg3 = this.getBlockRenderer().getBlockModelShaper();
        final BakedModel ebs4 = drg3.getModelManager().getMissingModel();
        for (final Block bpe6 : Registry.BLOCK) {
            for (final BlockState byg8 : bpe6.getStateDefinition().getPossibleStates()) {
                if (byg8.getRenderShape() == RenderShape.MODEL) {
                    final BakedModel ebs5 = drg3.getBlockModel(byg8);
                    if (ebs5 != ebs4) {
                        continue;
                    }
                    Minecraft.LOGGER.debug("Missing model for: {}", byg8);
                    boolean2 = true;
                }
            }
        }
        final TextureAtlasSprite eag5 = ebs4.getParticleIcon();
        for (final Block bpe7 : Registry.BLOCK) {
            for (final BlockState byg9 : bpe7.getStateDefinition().getPossibleStates()) {
                final TextureAtlasSprite eag6 = drg3.getParticleIcon(byg9);
                if (!byg9.isAir() && eag6 == eag5) {
                    Minecraft.LOGGER.debug("Missing particle icon for: {}", byg9);
                    boolean2 = true;
                }
            }
        }
        final NonNullList<ItemStack> fy6 = NonNullList.<ItemStack>create();
        for (final Item bef8 : Registry.ITEM) {
            fy6.clear();
            bef8.fillItemCategory(CreativeModeTab.TAB_SEARCH, fy6);
            for (final ItemStack bek10 : fy6) {
                final String string11 = bek10.getDescriptionId();
                final String string12 = new TranslatableComponent(string11, new Object[0]).getString();
                if (string12.toLowerCase(Locale.ROOT).equals(bef8.getDescriptionId())) {
                    Minecraft.LOGGER.debug("Missing translation for: {} {} {}", bek10, string11, bek10.getItem());
                }
            }
        }
        boolean2 |= MenuScreens.selfTest();
        if (boolean2) {
            throw new IllegalStateException("Your game data is foobar, fix the errors above!");
        }
    }
    
    public LevelStorageSource getLevelSource() {
        return this.levelSource;
    }
    
    public void setScreen(@Nullable Screen dfz) {
        if (this.screen != null) {
            this.screen.removed();
        }
        if (dfz == null && this.level == null) {
            dfz = new TitleScreen();
        }
        else if (dfz == null && this.player.getHealth() <= 0.0f) {
            if (this.player.shouldShowDeathScreen()) {
                dfz = new DeathScreen(null, this.level.getLevelData().isHardcore());
            }
            else {
                this.player.respawn();
            }
        }
        if (dfz instanceof TitleScreen || dfz instanceof JoinMultiplayerScreen) {
            this.options.renderDebug = false;
            this.gui.getChat().clearMessages(true);
        }
        if ((this.screen = dfz) != null) {
            this.mouseHandler.releaseMouse();
            KeyMapping.releaseAll();
            dfz.init(this, this.window.getGuiScaledWidth(), this.window.getGuiScaledHeight());
            this.noRender = false;
            NarratorChatListener.INSTANCE.sayNow(dfz.getNarrationMessage());
        }
        else {
            this.soundManager.resume();
            this.mouseHandler.grabMouse();
        }
    }
    
    public void setOverlay(@Nullable final Overlay dfu) {
        this.overlay = dfu;
    }
    
    public void destroy() {
        try {
            Minecraft.LOGGER.info("Stopping!");
            NarratorChatListener.INSTANCE.destroy();
            try {
                if (this.level != null) {
                    this.level.disconnect();
                }
                this.clearLevel();
            }
            catch (Throwable t) {}
            if (this.screen != null) {
                this.screen.removed();
            }
            this.close();
        }
        finally {
            Util.timeSource = System::nanoTime;
            if (this.delayedCrash == null) {
                System.exit(0);
            }
        }
    }
    
    @Override
    public void close() {
        try {
            this.modelManager.close();
            this.font.close();
            this.fontManager.close();
            this.gameRenderer.close();
            this.levelRenderer.close();
            this.soundManager.destroy();
            this.resourcePackRepository.close();
            this.particleEngine.close();
            this.mobEffectTextures.close();
            this.paintingTextures.close();
            Util.shutdownBackgroundExecutor();
        }
        finally {
            this.virtualScreen.close();
            this.window.close();
        }
    }
    
    private void runTick(final boolean boolean1) {
        this.window.setErrorSection("Pre render");
        final long long3 = Util.getNanos();
        this.profiler.startTick();
        if (this.window.shouldClose()) {
            this.stop();
        }
        if (this.pendingReload != null && !(this.overlay instanceof LoadingOverlay)) {
            final CompletableFuture<Void> completableFuture5 = this.pendingReload;
            this.pendingReload = null;
            this.reloadResourcePacks().thenRun(() -> completableFuture5.complete(null));
        }
        Runnable runnable5;
        while ((runnable5 = this.progressTasks.poll()) != null) {
            runnable5.run();
        }
        if (boolean1) {
            this.timer.advanceTime(Util.getMillis());
            this.profiler.push("scheduledExecutables");
            this.runAllTasks();
            this.profiler.pop();
        }
        this.profiler.push("tick");
        if (boolean1) {
            for (int integer6 = 0; integer6 < Math.min(10, this.timer.ticks); ++integer6) {
                this.tick();
            }
        }
        this.mouseHandler.turnPlayer();
        this.window.setErrorSection("Render");
        this.profiler.popPush("sound");
        this.soundManager.updateSource(this.gameRenderer.getMainCamera());
        this.profiler.pop();
        this.profiler.push("render");
        RenderSystem.pushMatrix();
        RenderSystem.clear(16640, Minecraft.ON_OSX);
        this.mainRenderTarget.bindWrite(true);
        FogRenderer.setupNoFog();
        this.profiler.push("display");
        RenderSystem.enableTexture();
        this.profiler.pop();
        if (!this.noRender) {
            this.profiler.popPush("gameRenderer");
            this.gameRenderer.render(this.pause ? this.pausePartialTick : this.timer.partialTick, long3, boolean1);
            this.profiler.popPush("toasts");
            this.toast.render();
            this.profiler.pop();
        }
        this.profiler.endTick();
        if (this.options.renderDebug && this.options.renderDebugCharts && !this.options.hideGui) {
            this.profiler.continuous().enable();
            this.renderFpsMeter();
        }
        else {
            this.profiler.continuous().disable();
        }
        this.mainRenderTarget.unbindWrite();
        RenderSystem.popMatrix();
        RenderSystem.pushMatrix();
        this.mainRenderTarget.blitToScreen(this.window.getWidth(), this.window.getHeight());
        RenderSystem.popMatrix();
        this.profiler.startTick();
        this.profiler.push("updateDisplay");
        this.window.updateDisplay();
        int integer6 = this.getFramerateLimit();
        if (integer6 < Option.FRAMERATE_LIMIT.getMaxValue()) {
            RenderSystem.limitDisplayFPS(integer6);
        }
        this.profiler.popPush("yield");
        Thread.yield();
        this.profiler.pop();
        this.window.setErrorSection("Post render");
        ++this.frames;
        final boolean boolean2 = this.hasSingleplayerServer() && ((this.screen != null && this.screen.isPauseScreen()) || (this.overlay != null && this.overlay.isPauseScreen())) && !this.singleplayerServer.isPublished();
        if (this.pause != boolean2) {
            if (this.pause) {
                this.pausePartialTick = this.timer.partialTick;
            }
            else {
                this.timer.partialTick = this.pausePartialTick;
            }
            this.pause = boolean2;
        }
        final long long4 = Util.getNanos();
        this.frameTimer.logFrameDuration(long4 - this.lastNanoTime);
        this.lastNanoTime = long4;
        while (Util.getMillis() >= this.lastTime + 1000L) {
            Minecraft.fps = this.frames;
            this.fpsString = String.format("%d fps T: %s%s%s%s B: %d", Minecraft.fps, (this.options.framerateLimit == Option.FRAMERATE_LIMIT.getMaxValue()) ? "inf" : Integer.valueOf(this.options.framerateLimit), this.options.enableVsync ? " vsync" : "", this.options.fancyGraphics ? "" : " fast", (this.options.renderClouds == CloudStatus.OFF) ? "" : ((this.options.renderClouds == CloudStatus.FAST) ? " fast-clouds" : " fancy-clouds"), this.options.biomeBlendRadius);
            this.lastTime += 1000L;
            this.frames = 0;
            this.snooper.prepare();
            if (!this.snooper.isStarted()) {
                this.snooper.start();
            }
        }
        this.profiler.endTick();
    }
    
    @Override
    public void resizeDisplay() {
        final int integer2 = this.window.calculateScale(this.options.guiScale, this.isEnforceUnicode());
        this.window.setGuiScale(integer2);
        if (this.screen != null) {
            this.screen.resize(this, this.window.getGuiScaledWidth(), this.window.getGuiScaledHeight());
        }
        final RenderTarget cxb3 = this.getMainRenderTarget();
        cxb3.resize(this.window.getWidth(), this.window.getHeight(), Minecraft.ON_OSX);
        this.gameRenderer.resize(this.window.getWidth(), this.window.getHeight());
        this.mouseHandler.setIgnoreFirstMove();
    }
    
    private int getFramerateLimit() {
        if (this.level == null && (this.screen != null || this.overlay != null)) {
            return 60;
        }
        return this.window.getFramerateLimit();
    }
    
    public void emergencySave() {
        try {
            Minecraft.reserve = new byte[0];
            this.levelRenderer.clear();
        }
        catch (Throwable t) {}
        try {
            System.gc();
            if (this.isLocalServer && this.singleplayerServer != null) {
                this.singleplayerServer.halt(true);
            }
            this.clearLevel(new GenericDirtMessageScreen(new TranslatableComponent("menu.savingLevel", new Object[0])));
        }
        catch (Throwable t2) {}
        System.gc();
    }
    
    void debugFpsMeterKeyPress(int integer) {
        final ProfileResults ail3 = this.profiler.continuous().getResults();
        final List<ResultField> list4 = ail3.getTimes(this.debugPath);
        if (list4.isEmpty()) {
            return;
        }
        final ResultField ain5 = list4.remove(0);
        if (integer == 0) {
            if (!ain5.name.isEmpty()) {
                final int integer2 = this.debugPath.lastIndexOf(30);
                if (integer2 >= 0) {
                    this.debugPath = this.debugPath.substring(0, integer2);
                }
            }
        }
        else if (--integer < list4.size() && !"unspecified".equals(list4.get(integer).name)) {
            if (!this.debugPath.isEmpty()) {
                this.debugPath += '\u001e';
            }
            this.debugPath += list4.get(integer).name;
        }
    }
    
    private void renderFpsMeter() {
        if (!this.profiler.continuous().isEnabled()) {
            return;
        }
        final ProfileResults ail2 = this.profiler.continuous().getResults();
        final List<ResultField> list3 = ail2.getTimes(this.debugPath);
        final ResultField ain4 = list3.remove(0);
        RenderSystem.clear(256, Minecraft.ON_OSX);
        RenderSystem.matrixMode(5889);
        RenderSystem.loadIdentity();
        RenderSystem.ortho(0.0, this.window.getWidth(), this.window.getHeight(), 0.0, 1000.0, 3000.0);
        RenderSystem.matrixMode(5888);
        RenderSystem.loadIdentity();
        RenderSystem.translatef(0.0f, 0.0f, -2000.0f);
        RenderSystem.lineWidth(1.0f);
        RenderSystem.disableTexture();
        final Tesselator cyj5 = Tesselator.getInstance();
        final BufferBuilder cyd6 = cyj5.getBuilder();
        final int integer7 = 160;
        final int integer8 = this.window.getWidth() - 160 - 10;
        final int integer9 = this.window.getHeight() - 320;
        RenderSystem.enableBlend();
        cyd6.begin(7, DefaultVertexFormat.POSITION_COLOR);
        cyd6.vertex(integer8 - 176.0f, integer9 - 96.0f - 16.0f, 0.0).color(200, 0, 0, 0).endVertex();
        cyd6.vertex(integer8 - 176.0f, integer9 + 320, 0.0).color(200, 0, 0, 0).endVertex();
        cyd6.vertex(integer8 + 176.0f, integer9 + 320, 0.0).color(200, 0, 0, 0).endVertex();
        cyd6.vertex(integer8 + 176.0f, integer9 - 96.0f - 16.0f, 0.0).color(200, 0, 0, 0).endVertex();
        cyj5.end();
        RenderSystem.disableBlend();
        double double10 = 0.0;
        for (final ResultField ain5 : list3) {
            final int integer10 = Mth.floor(ain5.percentage / 4.0) + 1;
            cyd6.begin(6, DefaultVertexFormat.POSITION_COLOR);
            final int integer11 = ain5.getColor();
            final int integer12 = integer11 >> 16 & 0xFF;
            final int integer13 = integer11 >> 8 & 0xFF;
            final int integer14 = integer11 & 0xFF;
            cyd6.vertex(integer8, integer9, 0.0).color(integer12, integer13, integer14, 255).endVertex();
            for (int integer15 = integer10; integer15 >= 0; --integer15) {
                final float float20 = (float)((double10 + ain5.percentage * integer15 / integer10) * 6.2831854820251465 / 100.0);
                final float float21 = Mth.sin(float20) * 160.0f;
                final float float22 = Mth.cos(float20) * 160.0f * 0.5f;
                cyd6.vertex(integer8 + float21, integer9 - float22, 0.0).color(integer12, integer13, integer14, 255).endVertex();
            }
            cyj5.end();
            cyd6.begin(5, DefaultVertexFormat.POSITION_COLOR);
            for (int integer15 = integer10; integer15 >= 0; --integer15) {
                final float float20 = (float)((double10 + ain5.percentage * integer15 / integer10) * 6.2831854820251465 / 100.0);
                final float float21 = Mth.sin(float20) * 160.0f;
                final float float22 = Mth.cos(float20) * 160.0f * 0.5f;
                if (float22 <= 0.0f) {
                    cyd6.vertex(integer8 + float21, integer9 - float22, 0.0).color(integer12 >> 1, integer13 >> 1, integer14 >> 1, 255).endVertex();
                    cyd6.vertex(integer8 + float21, integer9 - float22 + 10.0f, 0.0).color(integer12 >> 1, integer13 >> 1, integer14 >> 1, 255).endVertex();
                }
            }
            cyj5.end();
            double10 += ain5.percentage;
        }
        final DecimalFormat decimalFormat12 = new DecimalFormat("##0.00");
        decimalFormat12.setDecimalFormatSymbols(DecimalFormatSymbols.getInstance(Locale.ROOT));
        RenderSystem.enableTexture();
        final String string13 = ProfileResults.demanglePath(ain4.name);
        String string14 = "";
        if (!"unspecified".equals(string13)) {
            string14 += "[0] ";
        }
        if (string13.isEmpty()) {
            string14 += "ROOT ";
        }
        else {
            string14 = string14 + string13 + ' ';
        }
        final int integer11 = 16777215;
        this.font.drawShadow(string14, (float)(integer8 - 160), (float)(integer9 - 80 - 16), 16777215);
        string14 = decimalFormat12.format(ain4.globalPercentage) + "%";
        this.font.drawShadow(string14, (float)(integer8 + 160 - this.font.width(string14)), (float)(integer9 - 80 - 16), 16777215);
        for (int integer16 = 0; integer16 < list3.size(); ++integer16) {
            final ResultField ain6 = list3.get(integer16);
            final StringBuilder stringBuilder15 = new StringBuilder();
            if ("unspecified".equals(ain6.name)) {
                stringBuilder15.append("[?] ");
            }
            else {
                stringBuilder15.append("[").append(integer16 + 1).append("] ");
            }
            String string15 = stringBuilder15.append(ain6.name).toString();
            this.font.drawShadow(string15, (float)(integer8 - 160), (float)(integer9 + 80 + integer16 * 8 + 20), ain6.getColor());
            string15 = decimalFormat12.format(ain6.percentage) + "%";
            this.font.drawShadow(string15, (float)(integer8 + 160 - 50 - this.font.width(string15)), (float)(integer9 + 80 + integer16 * 8 + 20), ain6.getColor());
            string15 = decimalFormat12.format(ain6.globalPercentage) + "%";
            this.font.drawShadow(string15, (float)(integer8 + 160 - this.font.width(string15)), (float)(integer9 + 80 + integer16 * 8 + 20), ain6.getColor());
        }
    }
    
    public void stop() {
        this.running = false;
    }
    
    public boolean isRunning() {
        return this.running;
    }
    
    public void pauseGame(final boolean boolean1) {
        if (this.screen != null) {
            return;
        }
        final boolean boolean2 = this.hasSingleplayerServer() && !this.singleplayerServer.isPublished();
        if (boolean2) {
            this.setScreen(new PauseScreen(!boolean1));
            this.soundManager.pause();
        }
        else {
            this.setScreen(new PauseScreen(true));
        }
    }
    
    private void continueAttack(final boolean boolean1) {
        if (!boolean1) {
            this.missTime = 0;
        }
        if (this.missTime > 0 || this.player.isUsingItem()) {
            return;
        }
        if (boolean1 && this.hitResult != null && this.hitResult.getType() == HitResult.Type.BLOCK) {
            final BlockHitResult cvd3 = (BlockHitResult)this.hitResult;
            final BlockPos fk4 = cvd3.getBlockPos();
            if (!this.level.getBlockState(fk4).isAir()) {
                final Direction fp5 = cvd3.getDirection();
                if (this.gameMode.continueDestroyBlock(fk4, fp5)) {
                    this.particleEngine.crack(fk4, fp5);
                    this.player.swing(InteractionHand.MAIN_HAND);
                }
            }
            return;
        }
        this.gameMode.stopDestroyBlock();
    }
    
    private void startAttack() {
        if (this.missTime > 0) {
            return;
        }
        if (this.hitResult == null) {
            Minecraft.LOGGER.error("Null returned as 'hitResult', this shouldn't happen!");
            if (this.gameMode.hasMissTime()) {
                this.missTime = 10;
            }
            return;
        }
        if (this.player.isHandsBusy()) {
            return;
        }
        switch (this.hitResult.getType()) {
            case ENTITY: {
                this.gameMode.attack(this.player, ((EntityHitResult)this.hitResult).getEntity());
                break;
            }
            case BLOCK: {
                final BlockHitResult cvd2 = (BlockHitResult)this.hitResult;
                final BlockPos fk3 = cvd2.getBlockPos();
                if (!this.level.getBlockState(fk3).isAir()) {
                    this.gameMode.startDestroyBlock(fk3, cvd2.getDirection());
                    break;
                }
            }
            case MISS: {
                if (this.gameMode.hasMissTime()) {
                    this.missTime = 10;
                }
                this.player.resetAttackStrengthTicker();
                break;
            }
        }
        this.player.swing(InteractionHand.MAIN_HAND);
    }
    
    private void startUseItem() {
        if (this.gameMode.isDestroying()) {
            return;
        }
        this.rightClickDelay = 4;
        if (this.player.isHandsBusy()) {
            return;
        }
        if (this.hitResult == null) {
            Minecraft.LOGGER.warn("Null returned as 'hitResult', this shouldn't happen!");
        }
        for (final InteractionHand ajh5 : InteractionHand.values()) {
            final ItemStack bek6 = this.player.getItemInHand(ajh5);
            if (this.hitResult != null) {
                switch (this.hitResult.getType()) {
                    case ENTITY: {
                        final EntityHitResult cve7 = (EntityHitResult)this.hitResult;
                        final Entity akn8 = cve7.getEntity();
                        InteractionResult aji9 = this.gameMode.interactAt(this.player, akn8, cve7, ajh5);
                        if (!aji9.consumesAction()) {
                            aji9 = this.gameMode.interact(this.player, akn8, ajh5);
                        }
                        if (aji9.consumesAction()) {
                            if (aji9.shouldSwing()) {
                                this.player.swing(ajh5);
                            }
                            return;
                        }
                        break;
                    }
                    case BLOCK: {
                        final BlockHitResult cvd10 = (BlockHitResult)this.hitResult;
                        final int integer11 = bek6.getCount();
                        final InteractionResult aji10 = this.gameMode.useItemOn(this.player, this.level, ajh5, cvd10);
                        if (aji10.consumesAction()) {
                            if (aji10.shouldSwing()) {
                                this.player.swing(ajh5);
                                if (!bek6.isEmpty() && (bek6.getCount() != integer11 || this.gameMode.hasInfiniteItems())) {
                                    this.gameRenderer.itemInHandRenderer.itemUsed(ajh5);
                                }
                            }
                            return;
                        }
                        if (aji10 == InteractionResult.FAIL) {
                            return;
                        }
                        break;
                    }
                }
            }
            if (!bek6.isEmpty()) {
                final InteractionResult aji11 = this.gameMode.useItem(this.player, this.level, ajh5);
                if (aji11.consumesAction()) {
                    if (aji11.shouldSwing()) {
                        this.player.swing(ajh5);
                    }
                    this.gameRenderer.itemInHandRenderer.itemUsed(ajh5);
                    return;
                }
            }
        }
    }
    
    public MusicManager getMusicManager() {
        return this.musicManager;
    }
    
    public void tick() {
        if (this.rightClickDelay > 0) {
            --this.rightClickDelay;
        }
        this.profiler.push("gui");
        if (!this.pause) {
            this.gui.tick();
        }
        this.profiler.pop();
        this.gameRenderer.pick(1.0f);
        this.tutorial.onLookAt(this.level, this.hitResult);
        this.profiler.push("gameMode");
        if (!this.pause && this.level != null) {
            this.gameMode.tick();
        }
        this.profiler.popPush("textures");
        if (this.level != null) {
            this.textureManager.tick();
        }
        if (this.screen == null && this.player != null) {
            if (this.player.getHealth() <= 0.0f && !(this.screen instanceof DeathScreen)) {
                this.setScreen(null);
            }
            else if (this.player.isSleeping() && this.level != null) {
                this.setScreen(new InBedChatScreen());
            }
        }
        else if (this.screen != null && this.screen instanceof InBedChatScreen && !this.player.isSleeping()) {
            this.setScreen(null);
        }
        if (this.screen != null) {
            this.missTime = 10000;
        }
        if (this.screen != null) {
            Screen.wrapScreenError(() -> this.screen.tick(), "Ticking screen", this.screen.getClass().getCanonicalName());
        }
        if (!this.options.renderDebug) {
            this.gui.clearCache();
        }
        if (this.overlay == null && (this.screen == null || this.screen.passEvents)) {
            this.profiler.popPush("Keybindings");
            this.handleKeybinds();
            if (this.missTime > 0) {
                --this.missTime;
            }
        }
        if (this.level != null) {
            this.profiler.popPush("gameRenderer");
            if (!this.pause) {
                this.gameRenderer.tick();
            }
            this.profiler.popPush("levelRenderer");
            if (!this.pause) {
                this.levelRenderer.tick();
            }
            this.profiler.popPush("level");
            if (!this.pause) {
                if (this.level.getSkyFlashTime() > 0) {
                    this.level.setSkyFlashTime(this.level.getSkyFlashTime() - 1);
                }
                this.level.tickEntities();
            }
        }
        else if (this.gameRenderer.currentEffect() != null) {
            this.gameRenderer.shutdownEffect();
        }
        if (!this.pause) {
            this.musicManager.tick();
        }
        this.soundManager.tick(this.pause);
        if (this.level != null) {
            if (!this.pause) {
                this.level.setSpawnSettings(this.level.getDifficulty() != Difficulty.PEACEFUL, true);
                this.tutorial.tick();
                try {
                    this.level.tick(() -> true);
                }
                catch (Throwable throwable2) {
                    final CrashReport h3 = CrashReport.forThrowable(throwable2, "Exception in world tick");
                    if (this.level == null) {
                        final CrashReportCategory i4 = h3.addCategory("Affected level");
                        i4.setDetail("Problem", "Level is null!");
                    }
                    else {
                        this.level.fillReportDetails(h3);
                    }
                    throw new ReportedException(h3);
                }
            }
            this.profiler.popPush("animateTick");
            if (!this.pause && this.level != null) {
                this.level.animateTick(Mth.floor(this.player.getX()), Mth.floor(this.player.getY()), Mth.floor(this.player.getZ()));
            }
            this.profiler.popPush("particles");
            if (!this.pause) {
                this.particleEngine.tick();
            }
        }
        else if (this.pendingConnection != null) {
            this.profiler.popPush("pendingConnection");
            this.pendingConnection.tick();
        }
        this.profiler.popPush("keyboard");
        this.keyboardHandler.tick();
        this.profiler.pop();
    }
    
    private void handleKeybinds() {
        while (this.options.keyTogglePerspective.consumeClick()) {
            final Options options = this.options;
            ++options.thirdPersonView;
            if (this.options.thirdPersonView > 2) {
                this.options.thirdPersonView = 0;
            }
            if (this.options.thirdPersonView == 0) {
                this.gameRenderer.checkEntityPostEffect(this.getCameraEntity());
            }
            else if (this.options.thirdPersonView == 1) {
                this.gameRenderer.checkEntityPostEffect(null);
            }
            this.levelRenderer.needsUpdate();
        }
        while (this.options.keySmoothCamera.consumeClick()) {
            this.options.smoothCamera = !this.options.smoothCamera;
        }
        for (int integer2 = 0; integer2 < 9; ++integer2) {
            final boolean boolean3 = this.options.keySaveHotbarActivator.isDown();
            final boolean boolean4 = this.options.keyLoadHotbarActivator.isDown();
            if (this.options.keyHotbarSlots[integer2].consumeClick()) {
                if (this.player.isSpectator()) {
                    this.gui.getSpectatorGui().onHotbarSelected(integer2);
                }
                else if (this.player.isCreative() && this.screen == null && (boolean4 || boolean3)) {
                    CreativeModeInventoryScreen.handleHotbarLoadOrSave(this, integer2, boolean4, boolean3);
                }
                else {
                    this.player.inventory.selected = integer2;
                }
            }
        }
        while (this.options.keyInventory.consumeClick()) {
            if (this.gameMode.isServerControlledInventory()) {
                this.player.sendOpenInventory();
            }
            else {
                this.tutorial.onOpenInventory();
                this.setScreen(new InventoryScreen(this.player));
            }
        }
        while (this.options.keyAdvancements.consumeClick()) {
            this.setScreen(new AdvancementsScreen(this.player.connection.getAdvancements()));
        }
        while (this.options.keySwapHands.consumeClick()) {
            if (!this.player.isSpectator()) {
                this.getConnection().send(new ServerboundPlayerActionPacket(ServerboundPlayerActionPacket.Action.SWAP_HELD_ITEMS, BlockPos.ZERO, Direction.DOWN));
            }
        }
        while (this.options.keyDrop.consumeClick()) {
            if (!this.player.isSpectator() && this.player.drop(Screen.hasControlDown())) {
                this.player.swing(InteractionHand.MAIN_HAND);
            }
        }
        final boolean boolean5 = this.options.chatVisibility != ChatVisiblity.HIDDEN;
        if (boolean5) {
            while (this.options.keyChat.consumeClick()) {
                this.setScreen(new ChatScreen(""));
            }
            if (this.screen == null && this.overlay == null && this.options.keyCommand.consumeClick()) {
                this.setScreen(new ChatScreen("/"));
            }
        }
        if (this.player.isUsingItem()) {
            if (!this.options.keyUse.isDown()) {
                this.gameMode.releaseUsingItem(this.player);
            }
            while (this.options.keyAttack.consumeClick()) {}
            while (this.options.keyUse.consumeClick()) {}
            while (this.options.keyPickItem.consumeClick()) {}
        }
        else {
            while (this.options.keyAttack.consumeClick()) {
                this.startAttack();
            }
            while (this.options.keyUse.consumeClick()) {
                this.startUseItem();
            }
            while (this.options.keyPickItem.consumeClick()) {
                this.pickBlock();
            }
        }
        if (this.options.keyUse.isDown() && this.rightClickDelay == 0 && !this.player.isUsingItem()) {
            this.startUseItem();
        }
        this.continueAttack(this.screen == null && this.options.keyAttack.isDown() && this.mouseHandler.isMouseGrabbed());
    }
    
    public void selectLevel(final String string1, final String string2, @Nullable LevelSettings bjx) {
        this.clearLevel();
        final LevelStorage crk5 = this.levelSource.selectLevel(string1, null);
        LevelData crj6 = crk5.prepareLevel();
        if (crj6 == null && bjx != null) {
            crj6 = new LevelData(bjx, string1);
            crk5.saveLevelData(crj6);
        }
        if (bjx == null) {
            bjx = new LevelSettings(crj6);
        }
        this.progressListener.set(null);
        try {
            final YggdrasilAuthenticationService yggdrasilAuthenticationService7 = new YggdrasilAuthenticationService(this.proxy, UUID.randomUUID().toString());
            final MinecraftSessionService minecraftSessionService8 = yggdrasilAuthenticationService7.createMinecraftSessionService();
            final GameProfileRepository gameProfileRepository9 = yggdrasilAuthenticationService7.createProfileRepository();
            final GameProfileCache zk10 = new GameProfileCache(gameProfileRepository9, new File(this.gameDirectory, MinecraftServer.USERID_CACHE_FILE.getName()));
            SkullBlockEntity.setProfileCache(zk10);
            SkullBlockEntity.setSessionService(minecraftSessionService8);
            GameProfileCache.setUsesAuthentication(false);
            final StoringChunkProgressListener xq3;
            (this.singleplayerServer = new IntegratedServer(this, string1, string2, bjx, yggdrasilAuthenticationService7, minecraftSessionService8, gameProfileRepository9, zk10, integer -> {
                xq3 = new StoringChunkProgressListener(integer + 0);
                xq3.start();
                this.progressListener.set(xq3);
                return new ProcessorChunkProgressListener(xq3, this.progressTasks::add);
            })).forkAndRun();
            this.isLocalServer = true;
        }
        catch (Throwable throwable7) {
            final CrashReport h8 = CrashReport.forThrowable(throwable7, "Starting integrated server");
            final CrashReportCategory i9 = h8.addCategory("Starting integrated server");
            i9.setDetail("Level ID", string1);
            i9.setDetail("Level Name", string2);
            throw new ReportedException(h8);
        }
        while (this.progressListener.get() == null) {
            Thread.yield();
        }
        final LevelLoadingScreen dfn7 = new LevelLoadingScreen(this.progressListener.get());
        this.setScreen(dfn7);
        while (!this.singleplayerServer.isReady()) {
            dfn7.tick();
            this.runTick(false);
            try {
                Thread.sleep(16L);
            }
            catch (InterruptedException ex) {}
            if (this.delayedCrash != null) {
                crash(this.delayedCrash);
                return;
            }
        }
        final SocketAddress socketAddress8 = this.singleplayerServer.getConnection().startMemoryChannel();
        final Connection kt9 = Connection.connectToLocalServer(socketAddress8);
        kt9.setListener(new ClientHandshakePacketListenerImpl(kt9, this, null, lf -> {}));
        kt9.send(new ClientIntentionPacket(socketAddress8.toString(), 0, ConnectionProtocol.LOGIN));
        kt9.send(new ServerboundHelloPacket(this.getUser().getGameProfile()));
        this.pendingConnection = kt9;
    }
    
    public void setLevel(final ClientLevel dnl) {
        final ProgressScreen dfx3 = new ProgressScreen();
        dfx3.progressStartNoAbort(new TranslatableComponent("connect.joining", new Object[0]));
        this.updateScreenAndTick(dfx3);
        this.updateLevelInEngines(this.level = dnl);
        if (!this.isLocalServer) {
            final AuthenticationService authenticationService4 = (AuthenticationService)new YggdrasilAuthenticationService(this.proxy, UUID.randomUUID().toString());
            final MinecraftSessionService minecraftSessionService5 = authenticationService4.createMinecraftSessionService();
            final GameProfileRepository gameProfileRepository6 = authenticationService4.createProfileRepository();
            final GameProfileCache zk7 = new GameProfileCache(gameProfileRepository6, new File(this.gameDirectory, MinecraftServer.USERID_CACHE_FILE.getName()));
            SkullBlockEntity.setProfileCache(zk7);
            SkullBlockEntity.setSessionService(minecraftSessionService5);
            GameProfileCache.setUsesAuthentication(false);
        }
    }
    
    public void clearLevel() {
        this.clearLevel(new ProgressScreen());
    }
    
    public void clearLevel(final Screen dfz) {
        final ClientPacketListener dnm3 = this.getConnection();
        if (dnm3 != null) {
            this.dropAllTasks();
            dnm3.cleanup();
        }
        final IntegratedServer edj4 = this.singleplayerServer;
        this.singleplayerServer = null;
        this.gameRenderer.resetData();
        this.gameMode = null;
        NarratorChatListener.INSTANCE.clear();
        this.updateScreenAndTick(dfz);
        if (this.level != null) {
            if (edj4 != null) {
                while (!edj4.isShutdown()) {
                    this.runTick(false);
                }
            }
            this.clientPackSource.clearServerPack();
            this.gui.onDisconnected();
            this.currentServer = null;
            this.isLocalServer = false;
            this.game.onLeaveGameSession();
        }
        this.updateLevelInEngines(this.level = null);
        this.player = null;
    }
    
    private void updateScreenAndTick(final Screen dfz) {
        this.musicManager.stopPlaying();
        this.soundManager.stop();
        this.cameraEntity = null;
        this.pendingConnection = null;
        this.setScreen(dfz);
        this.runTick(false);
    }
    
    private void updateLevelInEngines(@Nullable final ClientLevel dnl) {
        this.levelRenderer.setLevel(dnl);
        this.particleEngine.setLevel(dnl);
        BlockEntityRenderDispatcher.instance.setLevel(dnl);
    }
    
    public final boolean isDemo() {
        return this.demo;
    }
    
    @Nullable
    public ClientPacketListener getConnection() {
        return (this.player == null) ? null : this.player.connection;
    }
    
    public static boolean renderNames() {
        return !Minecraft.instance.options.hideGui;
    }
    
    public static boolean useFancyGraphics() {
        return Minecraft.instance.options.fancyGraphics;
    }
    
    public static boolean useAmbientOcclusion() {
        return Minecraft.instance.options.ambientOcclusion != AmbientOcclusionStatus.OFF;
    }
    
    private void pickBlock() {
        if (this.hitResult == null || this.hitResult.getType() == HitResult.Type.MISS) {
            return;
        }
        final boolean boolean2 = this.player.abilities.instabuild;
        BlockEntity bwi3 = null;
        final HitResult.Type a5 = this.hitResult.getType();
        ItemStack bek4;
        if (a5 == HitResult.Type.BLOCK) {
            final BlockPos fk6 = ((BlockHitResult)this.hitResult).getBlockPos();
            final BlockState byg7 = this.level.getBlockState(fk6);
            final Block bpe8 = byg7.getBlock();
            if (byg7.isAir()) {
                return;
            }
            bek4 = bpe8.getCloneItemStack(this.level, fk6, byg7);
            if (bek4.isEmpty()) {
                return;
            }
            if (boolean2 && Screen.hasControlDown() && bpe8.isEntityBlock()) {
                bwi3 = this.level.getBlockEntity(fk6);
            }
        }
        else {
            if (a5 != HitResult.Type.ENTITY || !boolean2) {
                return;
            }
            final Entity akn6 = ((EntityHitResult)this.hitResult).getEntity();
            if (akn6 instanceof Painting) {
                bek4 = new ItemStack(Items.PAINTING);
            }
            else if (akn6 instanceof LeashFenceKnotEntity) {
                bek4 = new ItemStack(Items.LEAD);
            }
            else if (akn6 instanceof ItemFrame) {
                final ItemFrame avo7 = (ItemFrame)akn6;
                final ItemStack bek5 = avo7.getItem();
                if (bek5.isEmpty()) {
                    bek4 = new ItemStack(Items.ITEM_FRAME);
                }
                else {
                    bek4 = bek5.copy();
                }
            }
            else if (akn6 instanceof AbstractMinecart) {
                final AbstractMinecart azu7 = (AbstractMinecart)akn6;
                Item bef8 = null;
                switch (azu7.getMinecartType()) {
                    case FURNACE: {
                        bef8 = Items.FURNACE_MINECART;
                        break;
                    }
                    case CHEST: {
                        bef8 = Items.CHEST_MINECART;
                        break;
                    }
                    case TNT: {
                        bef8 = Items.TNT_MINECART;
                        break;
                    }
                    case HOPPER: {
                        bef8 = Items.HOPPER_MINECART;
                        break;
                    }
                    case COMMAND_BLOCK: {
                        bef8 = Items.COMMAND_BLOCK_MINECART;
                        break;
                    }
                    default: {
                        bef8 = Items.MINECART;
                        break;
                    }
                }
                bek4 = new ItemStack(bef8);
            }
            else if (akn6 instanceof Boat) {
                bek4 = new ItemStack(((Boat)akn6).getDropItem());
            }
            else if (akn6 instanceof ArmorStand) {
                bek4 = new ItemStack(Items.ARMOR_STAND);
            }
            else if (akn6 instanceof EndCrystal) {
                bek4 = new ItemStack(Items.END_CRYSTAL);
            }
            else {
                final SpawnEggItem bfi7 = SpawnEggItem.byId(akn6.getType());
                if (bfi7 == null) {
                    return;
                }
                bek4 = new ItemStack(bfi7);
            }
        }
        if (bek4.isEmpty()) {
            String string6 = "";
            if (a5 == HitResult.Type.BLOCK) {
                string6 = Registry.BLOCK.getKey(this.level.getBlockState(((BlockHitResult)this.hitResult).getBlockPos()).getBlock()).toString();
            }
            else if (a5 == HitResult.Type.ENTITY) {
                string6 = Registry.ENTITY_TYPE.getKey(((EntityHitResult)this.hitResult).getEntity().getType()).toString();
            }
            Minecraft.LOGGER.warn("Picking on: [{}] {} gave null item", a5, string6);
            return;
        }
        final Inventory ayf6 = this.player.inventory;
        if (bwi3 != null) {
            this.addCustomNbtData(bek4, bwi3);
        }
        final int integer7 = ayf6.findSlotMatchingItem(bek4);
        if (boolean2) {
            ayf6.setPickedItem(bek4);
            this.gameMode.handleCreativeModeItemAdd(this.player.getItemInHand(InteractionHand.MAIN_HAND), 36 + ayf6.selected);
        }
        else if (integer7 != -1) {
            if (Inventory.isHotbarSlot(integer7)) {
                ayf6.selected = integer7;
            }
            else {
                this.gameMode.handlePickItem(integer7);
            }
        }
    }
    
    private ItemStack addCustomNbtData(final ItemStack bek, final BlockEntity bwi) {
        final CompoundTag jt4 = bwi.save(new CompoundTag());
        if (bek.getItem() instanceof PlayerHeadItem && jt4.contains("Owner")) {
            final CompoundTag jt5 = jt4.getCompound("Owner");
            bek.getOrCreateTag().put("SkullOwner", jt5);
            return bek;
        }
        bek.addTagElement("BlockEntityTag", jt4);
        final CompoundTag jt5 = new CompoundTag();
        final ListTag jz6 = new ListTag();
        ((AbstractList<StringTag>)jz6).add(StringTag.valueOf("\"(+NBT)\""));
        jt5.put("Lore", jz6);
        bek.addTagElement("display", jt5);
        return bek;
    }
    
    public CrashReport fillReport(final CrashReport h) {
        fillReport(this.languageManager, this.launchedVersion, this.options, h);
        if (this.level != null) {
            this.level.fillReportDetails(h);
        }
        return h;
    }
    
    public static void fillReport(@Nullable final LanguageManager ebc, final String string, @Nullable final Options dbp, final CrashReport h) {
        final CrashReportCategory i5 = h.getSystemDetails();
        i5.setDetail("Launched Version", () -> string);
        i5.setDetail("Backend library", RenderSystem::getBackendDescription);
        i5.setDetail("Backend API", RenderSystem::getApiDescription);
        i5.setDetail("GL Caps", RenderSystem::getCapsString);
        i5.setDetail("Using VBOs", () -> "Yes");
        final String string2;
        i5.setDetail("Is Modded", () -> {
            string2 = ClientBrandRetriever.getClientModName();
            if (!"vanilla".equals(string2)) {
                return "Definitely; Client brand changed to '" + string2 + "'";
            }
            else if (Minecraft.class.getSigners() == null) {
                return "Very likely; Jar signature invalidated";
            }
            else {
                return "Probably not. Jar signature remains and client brand is untouched.";
            }
        });
        i5.setDetail("Type", "Client (map_client.txt)");
        if (dbp != null) {
            final StringBuilder stringBuilder2;
            final Iterator<String> iterator;
            String string3;
            i5.setDetail("Resource Packs", () -> {
                stringBuilder2 = new StringBuilder();
                dbp.resourcePacks.iterator();
                while (iterator.hasNext()) {
                    string3 = iterator.next();
                    if (stringBuilder2.length() > 0) {
                        stringBuilder2.append(", ");
                    }
                    stringBuilder2.append(string3);
                    if (dbp.incompatibleResourcePacks.contains(string3)) {
                        stringBuilder2.append(" (incompatible)");
                    }
                }
                return stringBuilder2.toString();
            });
        }
        if (ebc != null) {
            i5.setDetail("Current Language", () -> ebc.getSelected().toString());
        }
        i5.setDetail("CPU", GlUtil::getCpuInfo);
    }
    
    public static Minecraft getInstance() {
        return Minecraft.instance;
    }
    
    public CompletableFuture<Void> delayTextureReload() {
        return this.<CompletableFuture>submit(this::reloadResourcePacks).<Void>thenCompose(completableFuture -> completableFuture);
    }
    
    @Override
    public void populateSnooper(final Snooper ajp) {
        ajp.setDynamicData("fps", Minecraft.fps);
        ajp.setDynamicData("vsync_enabled", this.options.enableVsync);
        ajp.setDynamicData("display_frequency", this.window.getRefreshRate());
        ajp.setDynamicData("display_type", this.window.isFullscreen() ? "fullscreen" : "windowed");
        ajp.setDynamicData("run_time", (Util.getMillis() - ajp.getStartupTime()) / 60L * 1000L);
        ajp.setDynamicData("current_action", this.getCurrentSnooperAction());
        ajp.setDynamicData("language", (this.options.languageCode == null) ? "en_us" : this.options.languageCode);
        final String string3 = (ByteOrder.nativeOrder() == ByteOrder.LITTLE_ENDIAN) ? "little" : "big";
        ajp.setDynamicData("endianness", string3);
        ajp.setDynamicData("subtitles", this.options.showSubtitles);
        ajp.setDynamicData("touch", this.options.touchscreen ? "touch" : "mouse");
        int integer4 = 0;
        for (final UnopenedResourcePack eaz6 : this.resourcePackRepository.getSelected()) {
            if (!eaz6.isRequired() && !eaz6.isFixedPosition()) {
                ajp.setDynamicData("resource_pack[" + integer4++ + "]", eaz6.getId());
            }
        }
        ajp.setDynamicData("resource_packs", integer4);
        if (this.singleplayerServer != null) {
            ajp.setDynamicData("snooper_partner", this.singleplayerServer.getSnooper().getToken());
        }
    }
    
    private String getCurrentSnooperAction() {
        if (this.singleplayerServer != null) {
            if (this.singleplayerServer.isPublished()) {
                return "hosting_lan";
            }
            return "singleplayer";
        }
        else {
            if (this.currentServer == null) {
                return "out_of_game";
            }
            if (this.currentServer.isLan()) {
                return "playing_lan";
            }
            return "multiplayer";
        }
    }
    
    public void setCurrentServer(@Nullable final ServerData dnr) {
        this.currentServer = dnr;
    }
    
    @Nullable
    public ServerData getCurrentServer() {
        return this.currentServer;
    }
    
    public boolean isLocalServer() {
        return this.isLocalServer;
    }
    
    public boolean hasSingleplayerServer() {
        return this.isLocalServer && this.singleplayerServer != null;
    }
    
    @Nullable
    public IntegratedServer getSingleplayerServer() {
        return this.singleplayerServer;
    }
    
    public Snooper getSnooper() {
        return this.snooper;
    }
    
    public User getUser() {
        return this.user;
    }
    
    public PropertyMap getProfileProperties() {
        if (this.profileProperties.isEmpty()) {
            final GameProfile gameProfile2 = this.getMinecraftSessionService().fillProfileProperties(this.user.getGameProfile(), false);
            this.profileProperties.putAll((Multimap)gameProfile2.getProperties());
        }
        return this.profileProperties;
    }
    
    public Proxy getProxy() {
        return this.proxy;
    }
    
    public TextureManager getTextureManager() {
        return this.textureManager;
    }
    
    public ResourceManager getResourceManager() {
        return this.resourceManager;
    }
    
    public PackRepository<UnopenedResourcePack> getResourcePackRepository() {
        return this.resourcePackRepository;
    }
    
    public ClientPackSource getClientPackSource() {
        return this.clientPackSource;
    }
    
    public File getResourcePackDirectory() {
        return this.resourcePackDirectory;
    }
    
    public LanguageManager getLanguageManager() {
        return this.languageManager;
    }
    
    public Function<ResourceLocation, TextureAtlasSprite> getTextureAtlas(final ResourceLocation sm) {
        return this.modelManager.getAtlas(sm)::getSprite;
    }
    
    public boolean is64Bit() {
        return this.is64bit;
    }
    
    public boolean isPaused() {
        return this.pause;
    }
    
    public SoundManager getSoundManager() {
        return this.soundManager;
    }
    
    public MusicManager.Music getSituationalMusic() {
        if (this.screen instanceof WinScreen) {
            return MusicManager.Music.CREDITS;
        }
        if (this.player == null) {
            return MusicManager.Music.MENU;
        }
        if (this.player.level.dimension instanceof NetherDimension) {
            return MusicManager.Music.NETHER;
        }
        if (this.player.level.dimension instanceof TheEndDimension) {
            if (this.gui.getBossOverlay().shouldPlayMusic()) {
                return MusicManager.Music.END_BOSS;
            }
            return MusicManager.Music.END;
        }
        else {
            final Biome.BiomeCategory b2 = this.player.level.getBiome(new BlockPos(this.player)).getBiomeCategory();
            if (this.musicManager.isPlayingMusic(MusicManager.Music.UNDER_WATER) || (this.player.isUnderWater() && !this.musicManager.isPlayingMusic(MusicManager.Music.GAME) && (b2 == Biome.BiomeCategory.OCEAN || b2 == Biome.BiomeCategory.RIVER))) {
                return MusicManager.Music.UNDER_WATER;
            }
            if (this.player.abilities.instabuild && this.player.abilities.mayfly) {
                return MusicManager.Music.CREATIVE;
            }
            return MusicManager.Music.GAME;
        }
    }
    
    public MinecraftSessionService getMinecraftSessionService() {
        return this.minecraftSessionService;
    }
    
    public SkinManager getSkinManager() {
        return this.skinManager;
    }
    
    @Nullable
    public Entity getCameraEntity() {
        return this.cameraEntity;
    }
    
    public void setCameraEntity(final Entity akn) {
        this.cameraEntity = akn;
        this.gameRenderer.checkEntityPostEffect(akn);
    }
    
    @Override
    protected Thread getRunningThread() {
        return this.gameThread;
    }
    
    @Override
    protected Runnable wrapRunnable(final Runnable runnable) {
        return runnable;
    }
    
    @Override
    protected boolean shouldRun(final Runnable runnable) {
        return true;
    }
    
    public BlockRenderDispatcher getBlockRenderer() {
        return this.blockRenderer;
    }
    
    public EntityRenderDispatcher getEntityRenderDispatcher() {
        return this.entityRenderDispatcher;
    }
    
    public ItemRenderer getItemRenderer() {
        return this.itemRenderer;
    }
    
    public ItemInHandRenderer getItemInHandRenderer() {
        return this.itemInHandRenderer;
    }
    
    public <T> MutableSearchTree<T> getSearchTree(final SearchRegistry.Key<T> a) {
        return this.searchRegistry.<T>getTree(a);
    }
    
    public FrameTimer getFrameTimer() {
        return this.frameTimer;
    }
    
    public boolean isConnectedToRealms() {
        return this.connectedToRealms;
    }
    
    public void setConnectedToRealms(final boolean boolean1) {
        this.connectedToRealms = boolean1;
    }
    
    public DataFixer getFixerUpper() {
        return this.fixerUpper;
    }
    
    public float getFrameTime() {
        return this.timer.partialTick;
    }
    
    public float getDeltaFrameTime() {
        return this.timer.tickDelta;
    }
    
    public BlockColors getBlockColors() {
        return this.blockColors;
    }
    
    public boolean showOnlyReducedInfo() {
        return (this.player != null && this.player.isReducedDebugInfo()) || this.options.reducedDebugInfo;
    }
    
    public ToastComponent getToasts() {
        return this.toast;
    }
    
    public Tutorial getTutorial() {
        return this.tutorial;
    }
    
    public boolean isWindowActive() {
        return this.windowActive;
    }
    
    public HotbarManager getHotbarManager() {
        return this.hotbarManager;
    }
    
    public ModelManager getModelManager() {
        return this.modelManager;
    }
    
    public FontManager getFontManager() {
        return this.fontManager;
    }
    
    public PaintingTextureManager getPaintingTextures() {
        return this.paintingTextures;
    }
    
    public MobEffectTextureManager getMobEffectTextures() {
        return this.mobEffectTextures;
    }
    
    @Override
    public void setWindowActive(final boolean boolean1) {
        this.windowActive = boolean1;
    }
    
    public ProfilerFiller getProfiler() {
        return this.profiler;
    }
    
    public Game getGame() {
        return this.game;
    }
    
    public SplashManager getSplashManager() {
        return this.splashManager;
    }
    
    @Nullable
    public Overlay getOverlay() {
        return this.overlay;
    }
    
    public boolean renderOnThread() {
        return false;
    }
    
    public Window getWindow() {
        return this.window;
    }
    
    public RenderBuffers renderBuffers() {
        return this.renderBuffers;
    }
    
    private static UnopenedResourcePack createClientPackAdapter(final String string, final boolean boolean2, final Supplier<Pack> supplier, final Pack ye, final PackMetadataSection yj, final UnopenedPack.Position a) {
        final int integer7 = yj.getPackFormat();
        Supplier<Pack> supplier2 = supplier;
        if (integer7 <= 3) {
            supplier2 = adaptV3(supplier2);
        }
        if (integer7 <= 4) {
            supplier2 = adaptV4(supplier2);
        }
        return new UnopenedResourcePack(string, boolean2, supplier2, ye, yj, a);
    }
    
    private static Supplier<Pack> adaptV3(final Supplier<Pack> supplier) {
        return (Supplier<Pack>)(() -> new LegacyResourcePackAdapter(supplier.get(), LegacyResourcePackAdapter.V3));
    }
    
    private static Supplier<Pack> adaptV4(final Supplier<Pack> supplier) {
        return (Supplier<Pack>)(() -> new PackAdapterV4(supplier.get()));
    }
    
    public void updateMaxMipLevel(final int integer) {
        this.modelManager.updateMaxMipLevel(integer);
    }
    
    static {
        LOGGER = LogManager.getLogger();
        ON_OSX = (Util.getPlatform() == Util.OS.OSX);
        DEFAULT_FONT = new ResourceLocation("default");
        ALT_FONT = new ResourceLocation("alt");
        RESOURCE_RELOAD_INITIAL_TASK = CompletableFuture.<Unit>completedFuture(Unit.INSTANCE);
        Minecraft.reserve = new byte[10485760];
    }
}
