package net.minecraft.client;

import net.minecraft.util.Mth;
import net.minecraft.client.gui.components.ChatComponent;
import com.mojang.blaze3d.platform.Window;
import net.minecraft.world.entity.player.ChatVisiblity;
import net.minecraft.client.gui.chat.NarratorChatListener;
import java.util.concurrent.Executor;
import net.minecraft.Util;
import net.minecraft.client.resources.language.I18n;
import net.minecraft.client.gui.components.AbstractWidget;

public abstract class Option {
    public static final ProgressOption BIOME_BLEND_RADIUS;
    public static final ProgressOption CHAT_HEIGHT_FOCUSED;
    public static final ProgressOption CHAT_HEIGHT_UNFOCUSED;
    public static final ProgressOption CHAT_OPACITY;
    public static final ProgressOption CHAT_SCALE;
    public static final ProgressOption CHAT_WIDTH;
    public static final ProgressOption FOV;
    public static final ProgressOption FRAMERATE_LIMIT;
    public static final ProgressOption GAMMA;
    public static final ProgressOption MIPMAP_LEVELS;
    public static final ProgressOption MOUSE_WHEEL_SENSITIVITY;
    public static final BooleanOption RAW_MOUSE_INPUT;
    public static final ProgressOption RENDER_DISTANCE;
    public static final ProgressOption SENSITIVITY;
    public static final ProgressOption TEXT_BACKGROUND_OPACITY;
    public static final CycleOption AMBIENT_OCCLUSION;
    public static final CycleOption ATTACK_INDICATOR;
    public static final CycleOption CHAT_VISIBILITY;
    public static final CycleOption GRAPHICS;
    public static final CycleOption GUI_SCALE;
    public static final CycleOption MAIN_HAND;
    public static final CycleOption NARRATOR;
    public static final CycleOption PARTICLES;
    public static final CycleOption RENDER_CLOUDS;
    public static final CycleOption TEXT_BACKGROUND;
    public static final BooleanOption AUTO_JUMP;
    public static final BooleanOption AUTO_SUGGESTIONS;
    public static final BooleanOption CHAT_COLOR;
    public static final BooleanOption CHAT_LINKS;
    public static final BooleanOption CHAT_LINKS_PROMPT;
    public static final BooleanOption DISCRETE_MOUSE_SCROLL;
    public static final BooleanOption ENABLE_VSYNC;
    public static final BooleanOption ENTITY_SHADOWS;
    public static final BooleanOption FORCE_UNICODE_FONT;
    public static final BooleanOption INVERT_MOUSE;
    public static final BooleanOption REALMS_NOTIFICATIONS;
    public static final BooleanOption REDUCED_DEBUG_INFO;
    public static final BooleanOption SHOW_SUBTITLES;
    public static final BooleanOption SNOOPER_ENABLED;
    public static final CycleOption TOGGLE_CROUCH;
    public static final CycleOption TOGGLE_SPRINT;
    public static final BooleanOption TOUCHSCREEN;
    public static final BooleanOption USE_FULLSCREEN;
    public static final BooleanOption VIEW_BOBBING;
    private final String captionId;
    
    public Option(final String string) {
        this.captionId = string;
    }
    
    public abstract AbstractWidget createButton(final Options dbp, final int integer2, final int integer3, final int integer4);
    
    public String getCaption() {
        return I18n.get(this.captionId) + ": ";
    }
    
    static {
        final double double3;
        final String string5;
        final int integer2;
        BIOME_BLEND_RADIUS = new ProgressOption("options.biomeBlendRadius", 0.0, 7.0, 1.0f, dbp -> Double.valueOf(dbp.biomeBlendRadius), (dbp, double2) -> {
            dbp.biomeBlendRadius = Mth.clamp((int)(double)double2, 0, 7);
            Minecraft.getInstance().levelRenderer.allChanged();
            return;
        }, (dbp, dbr) -> {
            double3 = dbr.get(dbp);
            string5 = dbr.getCaption();
            integer2 = (int)double3 * 2 + 1;
            return string5 + I18n.get("options.biomeBlendRadius." + integer2);
        });
        final double double4;
        CHAT_HEIGHT_FOCUSED = new ProgressOption("options.chat.height.focused", 0.0, 1.0, 0.0f, dbp -> dbp.chatHeightFocused, (dbp, double2) -> {
            dbp.chatHeightFocused = double2;
            Minecraft.getInstance().gui.getChat().rescaleChat();
            return;
        }, (dbp, dbr) -> {
            double4 = dbr.toPct(dbr.get(dbp));
            return dbr.getCaption() + ChatComponent.getHeight(double4) + "px";
        });
        final double double5;
        CHAT_HEIGHT_UNFOCUSED = new ProgressOption("options.chat.height.unfocused", 0.0, 1.0, 0.0f, dbp -> dbp.chatHeightUnfocused, (dbp, double2) -> {
            dbp.chatHeightUnfocused = double2;
            Minecraft.getInstance().gui.getChat().rescaleChat();
            return;
        }, (dbp, dbr) -> {
            double5 = dbr.toPct(dbr.get(dbp));
            return dbr.getCaption() + ChatComponent.getHeight(double5) + "px";
        });
        final double double6;
        CHAT_OPACITY = new ProgressOption("options.chat.opacity", 0.0, 1.0, 0.0f, dbp -> dbp.chatOpacity, (dbp, double2) -> {
            dbp.chatOpacity = double2;
            Minecraft.getInstance().gui.getChat().rescaleChat();
            return;
        }, (dbp, dbr) -> {
            double6 = dbr.toPct(dbr.get(dbp));
            return dbr.getCaption() + (int)(double6 * 90.0 + 10.0) + "%";
        });
        final double double7;
        final String string6;
        CHAT_SCALE = new ProgressOption("options.chat.scale", 0.0, 1.0, 0.0f, dbp -> dbp.chatScale, (dbp, double2) -> {
            dbp.chatScale = double2;
            Minecraft.getInstance().gui.getChat().rescaleChat();
            return;
        }, (dbp, dbr) -> {
            double7 = dbr.toPct(dbr.get(dbp));
            string6 = dbr.getCaption();
            if (double7 == 0.0) {
                return string6 + I18n.get("options.off");
            }
            else {
                return string6 + (int)(double7 * 100.0) + "%";
            }
        });
        final double double8;
        CHAT_WIDTH = new ProgressOption("options.chat.width", 0.0, 1.0, 0.0f, dbp -> dbp.chatWidth, (dbp, double2) -> {
            dbp.chatWidth = double2;
            Minecraft.getInstance().gui.getChat().rescaleChat();
            return;
        }, (dbp, dbr) -> {
            double8 = dbr.toPct(dbr.get(dbp));
            return dbr.getCaption() + ChatComponent.getWidth(double8) + "px";
        });
        final double double9;
        final String string7;
        FOV = new ProgressOption("options.fov", 30.0, 110.0, 1.0f, dbp -> dbp.fov, (dbp, double2) -> dbp.fov = double2, (dbp, dbr) -> {
            double9 = dbr.get(dbp);
            string7 = dbr.getCaption();
            if (double9 == 70.0) {
                return string7 + I18n.get("options.fov.min");
            }
            else if (double9 == dbr.getMaxValue()) {
                return string7 + I18n.get("options.fov.max");
            }
            else {
                return string7 + (int)double9;
            }
        });
        final double double10;
        final String string8;
        FRAMERATE_LIMIT = new ProgressOption("options.framerateLimit", 10.0, 260.0, 10.0f, dbp -> Double.valueOf(dbp.framerateLimit), (dbp, double2) -> {
            dbp.framerateLimit = (int)(double)double2;
            Minecraft.getInstance().getWindow().setFramerateLimit(dbp.framerateLimit);
            return;
        }, (dbp, dbr) -> {
            double10 = dbr.get(dbp);
            string8 = dbr.getCaption();
            if (double10 == dbr.getMaxValue()) {
                return string8 + I18n.get("options.framerateLimit.max");
            }
            else {
                return string8 + I18n.get("options.framerate", (int)double10);
            }
        });
        final double double11;
        final String string9;
        GAMMA = new ProgressOption("options.gamma", 0.0, 1.0, 0.0f, dbp -> dbp.gamma, (dbp, double2) -> dbp.gamma = double2, (dbp, dbr) -> {
            double11 = dbr.toPct(dbr.get(dbp));
            string9 = dbr.getCaption();
            if (double11 == 0.0) {
                return string9 + I18n.get("options.gamma.min");
            }
            else if (double11 == 1.0) {
                return string9 + I18n.get("options.gamma.max");
            }
            else {
                return string9 + "+" + (int)(double11 * 100.0) + "%";
            }
        });
        final double double12;
        final String string10;
        MIPMAP_LEVELS = new ProgressOption("options.mipmapLevels", 0.0, 4.0, 1.0f, dbp -> Double.valueOf(dbp.mipmapLevels), (dbp, double2) -> dbp.mipmapLevels = (int)(double)double2, (dbp, dbr) -> {
            double12 = dbr.get(dbp);
            string10 = dbr.getCaption();
            if (double12 == 0.0) {
                return string10 + I18n.get("options.off");
            }
            else {
                return string10 + (int)double12;
            }
        });
        final double double13;
        MOUSE_WHEEL_SENSITIVITY = new LogaritmicProgressOption("options.mouseWheelSensitivity", 0.01, 10.0, 0.01f, dbp -> dbp.mouseWheelSensitivity, (dbp, double2) -> dbp.mouseWheelSensitivity = double2, (dbp, dbr) -> {
            double13 = dbr.toPct(dbr.get(dbp));
            return dbr.getCaption() + String.format("%.2f", dbr.toValue(double13));
        });
        final Window cxu3;
        RAW_MOUSE_INPUT = new BooleanOption("options.rawMouseInput", dbp -> dbp.rawMouseInput, (dbp, boolean2) -> {
            dbp.rawMouseInput = boolean2;
            cxu3 = Minecraft.getInstance().getWindow();
            if (cxu3 != null) {
                cxu3.updateRawMouseInput(boolean2);
            }
            return;
        });
        final double double14;
        RENDER_DISTANCE = new ProgressOption("options.renderDistance", 2.0, 16.0, 1.0f, dbp -> Double.valueOf(dbp.renderDistance), (dbp, double2) -> {
            dbp.renderDistance = (int)(double)double2;
            Minecraft.getInstance().levelRenderer.needsUpdate();
            return;
        }, (dbp, dbr) -> {
            double14 = dbr.get(dbp);
            return dbr.getCaption() + I18n.get("options.chunks", (int)double14);
        });
        final double double15;
        final String string11;
        SENSITIVITY = new ProgressOption("options.sensitivity", 0.0, 1.0, 0.0f, dbp -> dbp.sensitivity, (dbp, double2) -> dbp.sensitivity = double2, (dbp, dbr) -> {
            double15 = dbr.toPct(dbr.get(dbp));
            string11 = dbr.getCaption();
            if (double15 == 0.0) {
                return string11 + I18n.get("options.sensitivity.min");
            }
            else if (double15 == 1.0) {
                return string11 + I18n.get("options.sensitivity.max");
            }
            else {
                return string11 + (int)(double15 * 200.0) + "%";
            }
        });
        TEXT_BACKGROUND_OPACITY = new ProgressOption("options.accessibility.text_background_opacity", 0.0, 1.0, 0.0f, dbp -> dbp.textBackgroundOpacity, (dbp, double2) -> {
            dbp.textBackgroundOpacity = double2;
            Minecraft.getInstance().gui.getChat().rescaleChat();
            return;
        }, (dbp, dbr) -> dbr.getCaption() + (int)(dbr.toPct(dbr.get(dbp)) * 100.0) + "%");
        AMBIENT_OCCLUSION = new CycleOption("options.ao", (dbp, integer) -> {
            dbp.ambientOcclusion = AmbientOcclusionStatus.byId(dbp.ambientOcclusion.getId() + integer);
            Minecraft.getInstance().levelRenderer.allChanged();
            return;
        }, (dbp, dbc) -> dbc.getCaption() + I18n.get(dbp.ambientOcclusion.getKey()));
        ATTACK_INDICATOR = new CycleOption("options.attackIndicator", (dbp, integer) -> dbp.attackIndicator = AttackIndicatorStatus.byId(dbp.attackIndicator.getId() + integer), (dbp, dbc) -> dbc.getCaption() + I18n.get(dbp.attackIndicator.getKey()));
        CHAT_VISIBILITY = new CycleOption("options.chat.visibility", (dbp, integer) -> dbp.chatVisibility = ChatVisiblity.byId((dbp.chatVisibility.getId() + integer) % 3), (dbp, dbc) -> dbc.getCaption() + I18n.get(dbp.chatVisibility.getKey()));
        GRAPHICS = new CycleOption("options.graphics", (dbp, integer) -> {
            dbp.fancyGraphics = !dbp.fancyGraphics;
            Minecraft.getInstance().levelRenderer.allChanged();
            return;
        }, (dbp, dbc) -> {
            if (dbp.fancyGraphics) {
                return dbc.getCaption() + I18n.get("options.graphics.fancy");
            }
            else {
                return dbc.getCaption() + I18n.get("options.graphics.fast");
            }
        });
        GUI_SCALE = new CycleOption("options.guiScale", (dbp, integer) -> dbp.guiScale = Integer.remainderUnsigned(dbp.guiScale + integer, Minecraft.getInstance().getWindow().calculateScale(0, Minecraft.getInstance().isEnforceUnicode()) + 1), (dbp, dbc) -> dbc.getCaption() + ((dbp.guiScale == 0) ? I18n.get("options.guiScale.auto") : Integer.valueOf(dbp.guiScale)));
        MAIN_HAND = new CycleOption("options.mainHand", (dbp, integer) -> dbp.mainHand = dbp.mainHand.getOpposite(), (dbp, dbc) -> dbc.getCaption() + dbp.mainHand);
        NARRATOR = new CycleOption("options.narrator", (dbp, integer) -> {
            if (NarratorChatListener.INSTANCE.isActive()) {
                dbp.narratorStatus = NarratorStatus.byId(dbp.narratorStatus.getId() + integer);
            }
            else {
                dbp.narratorStatus = NarratorStatus.OFF;
            }
            NarratorChatListener.INSTANCE.updateNarratorStatus(dbp.narratorStatus);
            return;
        }, (dbp, dbc) -> {
            if (NarratorChatListener.INSTANCE.isActive()) {
                return dbc.getCaption() + I18n.get(dbp.narratorStatus.getKey());
            }
            else {
                return dbc.getCaption() + I18n.get("options.narrator.notavailable");
            }
        });
        PARTICLES = new CycleOption("options.particles", (dbp, integer) -> dbp.particles = ParticleStatus.byId(dbp.particles.getId() + integer), (dbp, dbc) -> dbc.getCaption() + I18n.get(dbp.particles.getKey()));
        RENDER_CLOUDS = new CycleOption("options.renderClouds", (dbp, integer) -> dbp.renderClouds = CloudStatus.byId(dbp.renderClouds.getId() + integer), (dbp, dbc) -> dbc.getCaption() + I18n.get(dbp.renderClouds.getKey()));
        TEXT_BACKGROUND = new CycleOption("options.accessibility.text_background", (dbp, integer) -> dbp.backgroundForChatOnly = !dbp.backgroundForChatOnly, (dbp, dbc) -> dbc.getCaption() + I18n.get(dbp.backgroundForChatOnly ? "options.accessibility.text_background.chat" : "options.accessibility.text_background.everywhere"));
        AUTO_JUMP = new BooleanOption("options.autoJump", dbp -> dbp.autoJump, (dbp, boolean2) -> dbp.autoJump = boolean2);
        AUTO_SUGGESTIONS = new BooleanOption("options.autoSuggestCommands", dbp -> dbp.autoSuggestions, (dbp, boolean2) -> dbp.autoSuggestions = boolean2);
        CHAT_COLOR = new BooleanOption("options.chat.color", dbp -> dbp.chatColors, (dbp, boolean2) -> dbp.chatColors = boolean2);
        CHAT_LINKS = new BooleanOption("options.chat.links", dbp -> dbp.chatLinks, (dbp, boolean2) -> dbp.chatLinks = boolean2);
        CHAT_LINKS_PROMPT = new BooleanOption("options.chat.links.prompt", dbp -> dbp.chatLinksPrompt, (dbp, boolean2) -> dbp.chatLinksPrompt = boolean2);
        DISCRETE_MOUSE_SCROLL = new BooleanOption("options.discrete_mouse_scroll", dbp -> dbp.discreteMouseScroll, (dbp, boolean2) -> dbp.discreteMouseScroll = boolean2);
        ENABLE_VSYNC = new BooleanOption("options.vsync", dbp -> dbp.enableVsync, (dbp, boolean2) -> {
            dbp.enableVsync = boolean2;
            if (Minecraft.getInstance().getWindow() != null) {
                Minecraft.getInstance().getWindow().updateVsync(dbp.enableVsync);
            }
            return;
        });
        ENTITY_SHADOWS = new BooleanOption("options.entityShadows", dbp -> dbp.entityShadows, (dbp, boolean2) -> dbp.entityShadows = boolean2);
        final Minecraft dbl3;
        FORCE_UNICODE_FONT = new BooleanOption("options.forceUnicodeFont", dbp -> dbp.forceUnicodeFont, (dbp, boolean2) -> {
            dbp.forceUnicodeFont = boolean2;
            dbl3 = Minecraft.getInstance();
            if (dbl3.getFontManager() != null) {
                dbl3.getFontManager().setForceUnicode(dbp.forceUnicodeFont, Util.backgroundExecutor(), dbl3);
            }
            return;
        });
        INVERT_MOUSE = new BooleanOption("options.invertMouse", dbp -> dbp.invertYMouse, (dbp, boolean2) -> dbp.invertYMouse = boolean2);
        REALMS_NOTIFICATIONS = new BooleanOption("options.realmsNotifications", dbp -> dbp.realmsNotifications, (dbp, boolean2) -> dbp.realmsNotifications = boolean2);
        REDUCED_DEBUG_INFO = new BooleanOption("options.reducedDebugInfo", dbp -> dbp.reducedDebugInfo, (dbp, boolean2) -> dbp.reducedDebugInfo = boolean2);
        SHOW_SUBTITLES = new BooleanOption("options.showSubtitles", dbp -> dbp.showSubtitles, (dbp, boolean2) -> dbp.showSubtitles = boolean2);
        SNOOPER_ENABLED = new BooleanOption("options.snooper", dbp -> {
            if (dbp.snooperEnabled) {}
            return false;
        }, (dbp, boolean2) -> dbp.snooperEnabled = boolean2);
        TOGGLE_CROUCH = new CycleOption("key.sneak", (dbp, integer) -> dbp.toggleCrouch = !dbp.toggleCrouch, (dbp, dbc) -> dbc.getCaption() + I18n.get(dbp.toggleCrouch ? "options.key.toggle" : "options.key.hold"));
        TOGGLE_SPRINT = new CycleOption("key.sprint", (dbp, integer) -> dbp.toggleSprint = !dbp.toggleSprint, (dbp, dbc) -> dbc.getCaption() + I18n.get(dbp.toggleSprint ? "options.key.toggle" : "options.key.hold"));
        TOUCHSCREEN = new BooleanOption("options.touchscreen", dbp -> dbp.touchscreen, (dbp, boolean2) -> dbp.touchscreen = boolean2);
        final Minecraft dbl4;
        USE_FULLSCREEN = new BooleanOption("options.fullscreen", dbp -> dbp.fullscreen, (dbp, boolean2) -> {
            dbp.fullscreen = boolean2;
            dbl4 = Minecraft.getInstance();
            if (dbl4.getWindow() != null && dbl4.getWindow().isFullscreen() != dbp.fullscreen) {
                dbl4.getWindow().toggleFullScreen();
                dbp.fullscreen = dbl4.getWindow().isFullscreen();
            }
            return;
        });
        VIEW_BOBBING = new BooleanOption("options.viewBobbing", dbp -> dbp.bobView, (dbp, boolean2) -> dbp.bobView = boolean2);
    }
}
