package net.minecraft.client.color.block;

import com.google.common.collect.ImmutableSet;
import javax.annotation.Nullable;
import net.minecraft.world.level.material.MaterialColor;
import net.minecraft.world.level.BlockGetter;
import net.minecraft.world.level.BlockAndTintGetter;
import net.minecraft.core.Registry;
import net.minecraft.core.BlockPos;
import net.minecraft.world.level.Level;
import net.minecraft.world.level.block.state.BlockState;
import net.minecraft.world.level.block.StemBlock;
import net.minecraft.world.level.block.RedStoneWireBlock;
import net.minecraft.world.level.FoliageColor;
import net.minecraft.world.level.GrassColor;
import net.minecraft.world.level.block.Blocks;
import net.minecraft.client.renderer.BiomeColors;
import net.minecraft.world.level.block.state.properties.DoubleBlockHalf;
import net.minecraft.world.level.block.ShearableDoublePlantBlock;
import com.google.common.collect.Maps;
import net.minecraft.world.level.block.state.properties.Property;
import java.util.Set;
import net.minecraft.world.level.block.Block;
import java.util.Map;
import net.minecraft.core.IdMapper;

public class BlockColors {
    private final IdMapper<BlockColor> blockColors;
    private final Map<Block, Set<Property<?>>> coloringStates;
    
    public BlockColors() {
        this.blockColors = new IdMapper<BlockColor>(32);
        this.coloringStates = Maps.newHashMap();
    }
    
    public static BlockColors createDefault() {
        final BlockColors dbz1 = new BlockColors();
        dbz1.register((byg, bjb, fk, integer) -> {
            if (bjb == null || fk == null) {
                return -1;
            }
            else {
                return BiomeColors.getAverageGrassColor(bjb, (byg.<DoubleBlockHalf>getValue(ShearableDoublePlantBlock.HALF) == DoubleBlockHalf.UPPER) ? fk.below() : fk);
            }
        }, Blocks.LARGE_FERN, Blocks.TALL_GRASS);
        dbz1.addColoringState(ShearableDoublePlantBlock.HALF, Blocks.LARGE_FERN, Blocks.TALL_GRASS);
        dbz1.register((byg, bjb, fk, integer) -> {
            if (bjb == null || fk == null) {
                return GrassColor.get(0.5, 1.0);
            }
            else {
                return BiomeColors.getAverageGrassColor(bjb, fk);
            }
        }, Blocks.GRASS_BLOCK, Blocks.FERN, Blocks.GRASS, Blocks.POTTED_FERN);
        dbz1.register((byg, bjb, fk, integer) -> FoliageColor.getEvergreenColor(), Blocks.SPRUCE_LEAVES);
        dbz1.register((byg, bjb, fk, integer) -> FoliageColor.getBirchColor(), Blocks.BIRCH_LEAVES);
        dbz1.register((byg, bjb, fk, integer) -> {
            if (bjb == null || fk == null) {
                return FoliageColor.getDefaultColor();
            }
            else {
                return BiomeColors.getAverageFoliageColor(bjb, fk);
            }
        }, Blocks.OAK_LEAVES, Blocks.JUNGLE_LEAVES, Blocks.ACACIA_LEAVES, Blocks.DARK_OAK_LEAVES, Blocks.VINE);
        dbz1.register((byg, bjb, fk, integer) -> {
            if (bjb == null || fk == null) {
                return -1;
            }
            else {
                return BiomeColors.getAverageWaterColor(bjb, fk);
            }
        }, Blocks.WATER, Blocks.BUBBLE_COLUMN, Blocks.CAULDRON);
        dbz1.register((byg, bjb, fk, integer) -> RedStoneWireBlock.getColorForData(byg.<Integer>getValue((Property<Integer>)RedStoneWireBlock.POWER)), Blocks.REDSTONE_WIRE);
        dbz1.addColoringState(RedStoneWireBlock.POWER, Blocks.REDSTONE_WIRE);
        dbz1.register((byg, bjb, fk, integer) -> {
            if (bjb == null || fk == null) {
                return -1;
            }
            else {
                return BiomeColors.getAverageGrassColor(bjb, fk);
            }
        }, Blocks.SUGAR_CANE);
        dbz1.register((byg, bjb, fk, integer) -> 14731036, Blocks.ATTACHED_MELON_STEM, Blocks.ATTACHED_PUMPKIN_STEM);
        final int integer5;
        final int integer6;
        final int integer7;
        final int integer8;
        dbz1.register((byg, bjb, fk, integer) -> {
            integer5 = byg.<Integer>getValue((Property<Integer>)StemBlock.AGE);
            integer6 = integer5 * 32;
            integer7 = 255 - integer5 * 8;
            integer8 = integer5 * 4;
            return integer6 << 16 | integer7 << 8 | integer8;
        }, Blocks.MELON_STEM, Blocks.PUMPKIN_STEM);
        dbz1.addColoringState(StemBlock.AGE, Blocks.MELON_STEM, Blocks.PUMPKIN_STEM);
        dbz1.register((byg, bjb, fk, integer) -> {
            if (bjb == null || fk == null) {
                return 7455580;
            }
            else {
                return 2129968;
            }
        }, Blocks.LILY_PAD);
        return dbz1;
    }
    
    public int getColor(final BlockState byg, final Level bjt, final BlockPos fk) {
        final BlockColor dby5 = this.blockColors.byId(Registry.BLOCK.getId(byg.getBlock()));
        if (dby5 != null) {
            return dby5.getColor(byg, null, null, 0);
        }
        final MaterialColor col6 = byg.getMapColor(bjt, fk);
        return (col6 != null) ? col6.col : -1;
    }
    
    public int getColor(final BlockState byg, @Nullable final BlockAndTintGetter bjb, @Nullable final BlockPos fk, final int integer) {
        final BlockColor dby6 = this.blockColors.byId(Registry.BLOCK.getId(byg.getBlock()));
        return (dby6 == null) ? -1 : dby6.getColor(byg, bjb, fk, integer);
    }
    
    public void register(final BlockColor dby, final Block... arr) {
        for (final Block bpe7 : arr) {
            this.blockColors.addMapping(dby, Registry.BLOCK.getId(bpe7));
        }
    }
    
    private void addColoringStates(final Set<Property<?>> set, final Block... arr) {
        for (final Block bpe7 : arr) {
            this.coloringStates.put(bpe7, set);
        }
    }
    
    private void addColoringState(final Property<?> bzj, final Block... arr) {
        this.addColoringStates(ImmutableSet.<Property<?>>of(bzj), arr);
    }
    
    public Set<Property<?>> getColoringProperties(final Block bpe) {
        return this.coloringStates.getOrDefault(bpe, ImmutableSet.of());
    }
}
