package net.minecraft.client.gui.screens.inventory;

import com.mojang.datafixers.util.Pair;
import net.minecraft.world.inventory.MenuType;
import net.minecraft.core.NonNullList;
import net.minecraft.client.player.LocalPlayer;
import net.minecraft.world.entity.LivingEntity;
import net.minecraft.ChatFormatting;
import net.minecraft.world.item.enchantment.Enchantment;
import net.minecraft.world.item.enchantment.EnchantmentHelper;
import com.google.common.collect.Lists;
import net.minecraft.world.item.TooltipFlag;
import net.minecraft.util.Mth;
import net.minecraft.client.player.inventory.Hotbar;
import net.minecraft.client.HotbarManager;
import net.minecraft.world.Container;
import com.google.common.collect.ImmutableList;
import net.minecraft.network.chat.TranslatableComponent;
import net.minecraft.world.level.ItemLike;
import net.minecraft.world.item.Items;
import com.mojang.blaze3d.systems.RenderSystem;
import net.minecraft.tags.TagCollection;
import java.util.function.Predicate;
import net.minecraft.tags.ItemTags;
import net.minecraft.client.searchtree.SearchTree;
import java.util.Iterator;
import java.util.Collection;
import java.util.Locale;
import net.minecraft.client.searchtree.SearchRegistry;
import net.minecraft.core.Registry;
import java.util.Objects;
import net.minecraft.client.Minecraft;
import net.minecraft.client.gui.Font;
import net.minecraft.world.inventory.ContainerListener;
import net.minecraft.client.resources.language.I18n;
import net.minecraft.world.entity.player.Inventory;
import net.minecraft.world.inventory.AbstractContainerMenu;
import net.minecraft.world.item.ItemStack;
import net.minecraft.world.item.CreativeModeTab;
import net.minecraft.world.inventory.ClickType;
import net.minecraft.client.gui.screens.Screen;
import com.google.common.collect.Maps;
import net.minecraft.network.chat.Component;
import net.minecraft.network.chat.TextComponent;
import net.minecraft.world.entity.player.Player;
import net.minecraft.world.item.Item;
import net.minecraft.tags.Tag;
import java.util.Map;
import javax.annotation.Nullable;
import net.minecraft.world.inventory.Slot;
import java.util.List;
import net.minecraft.client.gui.components.EditBox;
import net.minecraft.world.SimpleContainer;
import net.minecraft.resources.ResourceLocation;

public class CreativeModeInventoryScreen extends EffectRenderingInventoryScreen<ItemPickerMenu> {
    private static final ResourceLocation CREATIVE_TABS_LOCATION;
    private static final SimpleContainer CONTAINER;
    private static int selectedTab;
    private float scrollOffs;
    private boolean scrolling;
    private EditBox searchBox;
    @Nullable
    private List<Slot> originalSlots;
    @Nullable
    private Slot destroyItemSlot;
    private CreativeInventoryListener listener;
    private boolean ignoreTextInput;
    private boolean hasClickedOutside;
    private final Map<ResourceLocation, Tag<Item>> visibleTags;
    
    public CreativeModeInventoryScreen(final Player ayg) {
        super(new ItemPickerMenu(ayg), ayg.inventory, new TextComponent(""));
        this.visibleTags = Maps.newTreeMap();
        ayg.containerMenu = this.menu;
        this.passEvents = true;
        this.imageHeight = 136;
        this.imageWidth = 195;
    }
    
    @Override
    public void tick() {
        if (!this.minecraft.gameMode.hasInfiniteItems()) {
            this.minecraft.setScreen(new InventoryScreen(this.minecraft.player));
        }
        else if (this.searchBox != null) {
            this.searchBox.tick();
        }
    }
    
    @Override
    protected void slotClicked(@Nullable final Slot bbx, final int integer2, final int integer3, ClickType bas) {
        if (this.isCreativeSlot(bbx)) {
            this.searchBox.moveCursorToEnd();
            this.searchBox.setHighlightPos(0);
        }
        final boolean boolean6 = bas == ClickType.QUICK_MOVE;
        bas = ((integer2 == -999 && bas == ClickType.PICKUP) ? ClickType.THROW : bas);
        if (bbx != null || CreativeModeInventoryScreen.selectedTab == CreativeModeTab.TAB_INVENTORY.getId() || bas == ClickType.QUICK_CRAFT) {
            if (bbx != null && !bbx.mayPickup(this.minecraft.player)) {
                return;
            }
            if (bbx == this.destroyItemSlot && boolean6) {
                for (int integer4 = 0; integer4 < this.minecraft.player.inventoryMenu.getItems().size(); ++integer4) {
                    this.minecraft.gameMode.handleCreativeModeItemAdd(ItemStack.EMPTY, integer4);
                }
            }
            else if (CreativeModeInventoryScreen.selectedTab == CreativeModeTab.TAB_INVENTORY.getId()) {
                if (bbx == this.destroyItemSlot) {
                    this.minecraft.player.inventory.setCarried(ItemStack.EMPTY);
                }
                else if (bas == ClickType.THROW && bbx != null && bbx.hasItem()) {
                    final ItemStack bek7 = bbx.remove((integer3 == 0) ? 1 : bbx.getItem().getMaxStackSize());
                    final ItemStack bek8 = bbx.getItem();
                    this.minecraft.player.drop(bek7, true);
                    this.minecraft.gameMode.handleCreativeModeItemDrop(bek7);
                    this.minecraft.gameMode.handleCreativeModeItemAdd(bek8, ((SlotWrapper)bbx).target.index);
                }
                else if (bas == ClickType.THROW && !this.minecraft.player.inventory.getCarried().isEmpty()) {
                    this.minecraft.player.drop(this.minecraft.player.inventory.getCarried(), true);
                    this.minecraft.gameMode.handleCreativeModeItemDrop(this.minecraft.player.inventory.getCarried());
                    this.minecraft.player.inventory.setCarried(ItemStack.EMPTY);
                }
                else {
                    this.minecraft.player.inventoryMenu.clicked((bbx == null) ? integer2 : ((SlotWrapper)bbx).target.index, integer3, bas, this.minecraft.player);
                    this.minecraft.player.inventoryMenu.broadcastChanges();
                }
            }
            else if (bas != ClickType.QUICK_CRAFT && bbx.container == CreativeModeInventoryScreen.CONTAINER) {
                final Inventory ayf7 = this.minecraft.player.inventory;
                ItemStack bek8 = ayf7.getCarried();
                final ItemStack bek9 = bbx.getItem();
                if (bas == ClickType.SWAP) {
                    if (!bek9.isEmpty() && integer3 >= 0 && integer3 < 9) {
                        final ItemStack bek10 = bek9.copy();
                        bek10.setCount(bek10.getMaxStackSize());
                        this.minecraft.player.inventory.setItem(integer3, bek10);
                        this.minecraft.player.inventoryMenu.broadcastChanges();
                    }
                    return;
                }
                if (bas == ClickType.CLONE) {
                    if (ayf7.getCarried().isEmpty() && bbx.hasItem()) {
                        final ItemStack bek10 = bbx.getItem().copy();
                        bek10.setCount(bek10.getMaxStackSize());
                        ayf7.setCarried(bek10);
                    }
                    return;
                }
                if (bas == ClickType.THROW) {
                    if (!bek9.isEmpty()) {
                        final ItemStack bek10 = bek9.copy();
                        bek10.setCount((integer3 == 0) ? 1 : bek10.getMaxStackSize());
                        this.minecraft.player.drop(bek10, true);
                        this.minecraft.gameMode.handleCreativeModeItemDrop(bek10);
                    }
                    return;
                }
                if (!bek8.isEmpty() && !bek9.isEmpty() && bek8.sameItem(bek9) && ItemStack.tagMatches(bek8, bek9)) {
                    if (integer3 == 0) {
                        if (boolean6) {
                            bek8.setCount(bek8.getMaxStackSize());
                        }
                        else if (bek8.getCount() < bek8.getMaxStackSize()) {
                            bek8.grow(1);
                        }
                    }
                    else {
                        bek8.shrink(1);
                    }
                }
                else if (bek9.isEmpty() || !bek8.isEmpty()) {
                    if (integer3 == 0) {
                        ayf7.setCarried(ItemStack.EMPTY);
                    }
                    else {
                        ayf7.getCarried().shrink(1);
                    }
                }
                else {
                    ayf7.setCarried(bek9.copy());
                    bek8 = ayf7.getCarried();
                    if (boolean6) {
                        bek8.setCount(bek8.getMaxStackSize());
                    }
                }
            }
            else if (this.menu != null) {
                final ItemStack bek7 = (bbx == null) ? ItemStack.EMPTY : ((ItemPickerMenu)this.menu).getSlot(bbx.index).getItem();
                ((ItemPickerMenu)this.menu).clicked((bbx == null) ? integer2 : bbx.index, integer3, bas, this.minecraft.player);
                if (AbstractContainerMenu.getQuickcraftHeader(integer3) == 2) {
                    for (int integer5 = 0; integer5 < 9; ++integer5) {
                        this.minecraft.gameMode.handleCreativeModeItemAdd(((ItemPickerMenu)this.menu).getSlot(45 + integer5).getItem(), 36 + integer5);
                    }
                }
                else if (bbx != null) {
                    final ItemStack bek8 = ((ItemPickerMenu)this.menu).getSlot(bbx.index).getItem();
                    this.minecraft.gameMode.handleCreativeModeItemAdd(bek8, bbx.index - ((ItemPickerMenu)this.menu).slots.size() + 9 + 36);
                    final int integer6 = 45 + integer3;
                    if (bas == ClickType.SWAP) {
                        this.minecraft.gameMode.handleCreativeModeItemAdd(bek7, integer6 - ((ItemPickerMenu)this.menu).slots.size() + 9 + 36);
                    }
                    else if (bas == ClickType.THROW && !bek7.isEmpty()) {
                        final ItemStack bek10 = bek7.copy();
                        bek10.setCount((integer3 == 0) ? 1 : bek10.getMaxStackSize());
                        this.minecraft.player.drop(bek10, true);
                        this.minecraft.gameMode.handleCreativeModeItemDrop(bek10);
                    }
                    this.minecraft.player.inventoryMenu.broadcastChanges();
                }
            }
        }
        else {
            final Inventory ayf7 = this.minecraft.player.inventory;
            if (!ayf7.getCarried().isEmpty() && this.hasClickedOutside) {
                if (integer3 == 0) {
                    this.minecraft.player.drop(ayf7.getCarried(), true);
                    this.minecraft.gameMode.handleCreativeModeItemDrop(ayf7.getCarried());
                    ayf7.setCarried(ItemStack.EMPTY);
                }
                if (integer3 == 1) {
                    final ItemStack bek8 = ayf7.getCarried().split(1);
                    this.minecraft.player.drop(bek8, true);
                    this.minecraft.gameMode.handleCreativeModeItemDrop(bek8);
                }
            }
        }
    }
    
    private boolean isCreativeSlot(@Nullable final Slot bbx) {
        return bbx != null && bbx.container == CreativeModeInventoryScreen.CONTAINER;
    }
    
    @Override
    protected void checkEffectRendering() {
        final int integer2 = this.leftPos;
        super.checkEffectRendering();
        if (this.searchBox != null && this.leftPos != integer2) {
            this.searchBox.setX(this.leftPos + 82);
        }
    }
    
    @Override
    protected void init() {
        if (this.minecraft.gameMode.hasInfiniteItems()) {
            super.init();
            this.minecraft.keyboardHandler.setSendRepeatsToGui(true);
            final Font font = this.font;
            final int integer3 = this.leftPos + 82;
            final int integer4 = this.topPos + 6;
            final int integer5 = 80;
            this.font.getClass();
            (this.searchBox = new EditBox(font, integer3, integer4, integer5, 9, I18n.get("itemGroup.search"))).setMaxLength(50);
            this.searchBox.setBordered(false);
            this.searchBox.setVisible(false);
            this.searchBox.setTextColor(16777215);
            this.children.add(this.searchBox);
            final int integer2 = CreativeModeInventoryScreen.selectedTab;
            CreativeModeInventoryScreen.selectedTab = -1;
            this.selectTab(CreativeModeTab.TABS[integer2]);
            this.minecraft.player.inventoryMenu.removeSlotListener(this.listener);
            this.listener = new CreativeInventoryListener(this.minecraft);
            this.minecraft.player.inventoryMenu.addSlotListener(this.listener);
        }
        else {
            this.minecraft.setScreen(new InventoryScreen(this.minecraft.player));
        }
    }
    
    @Override
    public void resize(final Minecraft dbl, final int integer2, final int integer3) {
        final String string5 = this.searchBox.getValue();
        this.init(dbl, integer2, integer3);
        this.searchBox.setValue(string5);
        if (!this.searchBox.getValue().isEmpty()) {
            this.refreshSearchResults();
        }
    }
    
    @Override
    public void removed() {
        super.removed();
        if (this.minecraft.player != null && this.minecraft.player.inventory != null) {
            this.minecraft.player.inventoryMenu.removeSlotListener(this.listener);
        }
        this.minecraft.keyboardHandler.setSendRepeatsToGui(false);
    }
    
    @Override
    public boolean charTyped(final char character, final int integer) {
        if (this.ignoreTextInput) {
            return false;
        }
        if (CreativeModeInventoryScreen.selectedTab != CreativeModeTab.TAB_SEARCH.getId()) {
            return false;
        }
        final String string4 = this.searchBox.getValue();
        if (this.searchBox.charTyped(character, integer)) {
            if (!Objects.equals(string4, this.searchBox.getValue())) {
                this.refreshSearchResults();
            }
            return true;
        }
        return false;
    }
    
    @Override
    public boolean keyPressed(final int integer1, final int integer2, final int integer3) {
        this.ignoreTextInput = false;
        if (CreativeModeInventoryScreen.selectedTab != CreativeModeTab.TAB_SEARCH.getId()) {
            if (this.minecraft.options.keyChat.matches(integer1, integer2)) {
                this.ignoreTextInput = true;
                this.selectTab(CreativeModeTab.TAB_SEARCH);
                return true;
            }
            return super.keyPressed(integer1, integer2, integer3);
        }
        else {
            final boolean boolean5 = !this.isCreativeSlot(this.hoveredSlot) || (this.hoveredSlot != null && this.hoveredSlot.hasItem());
            if (boolean5 && this.checkNumkeyPressed(integer1, integer2)) {
                return this.ignoreTextInput = true;
            }
            final String string6 = this.searchBox.getValue();
            if (this.searchBox.keyPressed(integer1, integer2, integer3)) {
                if (!Objects.equals(string6, this.searchBox.getValue())) {
                    this.refreshSearchResults();
                }
                return true;
            }
            return (this.searchBox.isFocused() && this.searchBox.isVisible() && integer1 != 256) || super.keyPressed(integer1, integer2, integer3);
        }
    }
    
    @Override
    public boolean keyReleased(final int integer1, final int integer2, final int integer3) {
        this.ignoreTextInput = false;
        return super.keyReleased(integer1, integer2, integer3);
    }
    
    private void refreshSearchResults() {
        ((ItemPickerMenu)this.menu).items.clear();
        this.visibleTags.clear();
        String string2 = this.searchBox.getValue();
        if (string2.isEmpty()) {
            for (final Item bef4 : Registry.ITEM) {
                bef4.fillItemCategory(CreativeModeTab.TAB_SEARCH, ((ItemPickerMenu)this.menu).items);
            }
        }
        else {
            SearchTree<ItemStack> edf3;
            if (string2.startsWith("#")) {
                string2 = string2.substring(1);
                edf3 = this.minecraft.<ItemStack>getSearchTree(SearchRegistry.CREATIVE_TAGS);
                this.updateVisibleTags(string2);
            }
            else {
                edf3 = this.minecraft.<ItemStack>getSearchTree(SearchRegistry.CREATIVE_NAMES);
            }
            ((ItemPickerMenu)this.menu).items.addAll(edf3.search(string2.toLowerCase(Locale.ROOT)));
        }
        this.scrollOffs = 0.0f;
        ((ItemPickerMenu)this.menu).scrollTo(0.0f);
    }
    
    private void updateVisibleTags(final String string) {
        final int integer3 = string.indexOf(58);
        Predicate<ResourceLocation> predicate4;
        if (integer3 == -1) {
            predicate4 = (sm -> sm.getPath().contains(string));
        }
        else {
            final String string2 = string.substring(0, integer3).trim();
            final String string3 = string.substring(integer3 + 1).trim();
            final CharSequence s;
            final CharSequence s2;
            predicate4 = (sm -> sm.getNamespace().contains(s) && sm.getPath().contains(s2));
        }
        final TagCollection<Item> aba5 = ItemTags.getAllTags();
        final Tag<Item> tag;
        aba5.getAvailableTags().stream().filter(predicate4).forEach(sm -> tag = this.visibleTags.put(sm, aba5.getTag(sm)));
    }
    
    @Override
    protected void renderLabels(final int integer1, final int integer2) {
        final CreativeModeTab bda4 = CreativeModeTab.TABS[CreativeModeInventoryScreen.selectedTab];
        if (bda4.showTitle()) {
            RenderSystem.disableBlend();
            this.font.draw(I18n.get(bda4.getName()), 8.0f, 6.0f, 4210752);
        }
    }
    
    @Override
    public boolean mouseClicked(final double double1, final double double2, final int integer) {
        if (integer == 0) {
            final double double3 = double1 - this.leftPos;
            final double double4 = double2 - this.topPos;
            for (final CreativeModeTab bda14 : CreativeModeTab.TABS) {
                if (this.checkTabClicked(bda14, double3, double4)) {
                    return true;
                }
            }
            if (CreativeModeInventoryScreen.selectedTab != CreativeModeTab.TAB_INVENTORY.getId() && this.insideScrollbar(double1, double2)) {
                this.scrolling = this.canScroll();
                return true;
            }
        }
        return super.mouseClicked(double1, double2, integer);
    }
    
    @Override
    public boolean mouseReleased(final double double1, final double double2, final int integer) {
        if (integer == 0) {
            final double double3 = double1 - this.leftPos;
            final double double4 = double2 - this.topPos;
            this.scrolling = false;
            for (final CreativeModeTab bda14 : CreativeModeTab.TABS) {
                if (this.checkTabClicked(bda14, double3, double4)) {
                    this.selectTab(bda14);
                    return true;
                }
            }
        }
        return super.mouseReleased(double1, double2, integer);
    }
    
    private boolean canScroll() {
        return CreativeModeInventoryScreen.selectedTab != CreativeModeTab.TAB_INVENTORY.getId() && CreativeModeTab.TABS[CreativeModeInventoryScreen.selectedTab].canScroll() && ((ItemPickerMenu)this.menu).canScroll();
    }
    
    private void selectTab(final CreativeModeTab bda) {
        final int integer3 = CreativeModeInventoryScreen.selectedTab;
        CreativeModeInventoryScreen.selectedTab = bda.getId();
        this.quickCraftSlots.clear();
        ((ItemPickerMenu)this.menu).items.clear();
        if (bda == CreativeModeTab.TAB_HOTBAR) {
            final HotbarManager dbh4 = this.minecraft.getHotbarManager();
            for (int integer4 = 0; integer4 < 9; ++integer4) {
                final Hotbar dqa6 = dbh4.get(integer4);
                if (dqa6.isEmpty()) {
                    for (int integer5 = 0; integer5 < 9; ++integer5) {
                        if (integer5 == integer4) {
                            final ItemStack bek8 = new ItemStack(Items.PAPER);
                            bek8.getOrCreateTagElement("CustomCreativeLock");
                            final String string9 = this.minecraft.options.keyHotbarSlots[integer4].getTranslatedKeyMessage();
                            final String string10 = this.minecraft.options.keySaveHotbarActivator.getTranslatedKeyMessage();
                            bek8.setHoverName(new TranslatableComponent("inventory.hotbarInfo", new Object[] { string10, string9 }));
                            ((ItemPickerMenu)this.menu).items.add(bek8);
                        }
                        else {
                            ((ItemPickerMenu)this.menu).items.add(ItemStack.EMPTY);
                        }
                    }
                }
                else {
                    ((ItemPickerMenu)this.menu).items.addAll(dqa6);
                }
            }
        }
        else if (bda != CreativeModeTab.TAB_SEARCH) {
            bda.fillItemList(((ItemPickerMenu)this.menu).items);
        }
        if (bda == CreativeModeTab.TAB_INVENTORY) {
            final AbstractContainerMenu bak4 = this.minecraft.player.inventoryMenu;
            if (this.originalSlots == null) {
                this.originalSlots = ImmutableList.copyOf(((ItemPickerMenu)this.menu).slots);
            }
            ((ItemPickerMenu)this.menu).slots.clear();
            for (int integer4 = 0; integer4 < bak4.slots.size(); ++integer4) {
                int integer5;
                int integer9;
                if (integer4 >= 5 && integer4 < 9) {
                    final int integer6 = integer4 - 5;
                    final int integer7 = integer6 / 2;
                    final int integer8 = integer6 % 2;
                    integer9 = 54 + integer7 * 54;
                    integer5 = 6 + integer8 * 27;
                }
                else if (integer4 >= 0 && integer4 < 5) {
                    integer9 = -2000;
                    integer5 = -2000;
                }
                else if (integer4 == 45) {
                    integer9 = 35;
                    integer5 = 20;
                }
                else {
                    final int integer6 = integer4 - 9;
                    final int integer7 = integer6 % 9;
                    final int integer8 = integer6 / 9;
                    integer9 = 9 + integer7 * 18;
                    if (integer4 >= 36) {
                        integer5 = 112;
                    }
                    else {
                        integer5 = 54 + integer8 * 18;
                    }
                }
                final Slot bbx8 = new SlotWrapper(bak4.slots.get(integer4), integer4, integer9, integer5);
                ((ItemPickerMenu)this.menu).slots.add(bbx8);
            }
            this.destroyItemSlot = new Slot(CreativeModeInventoryScreen.CONTAINER, 0, 173, 112);
            ((ItemPickerMenu)this.menu).slots.add(this.destroyItemSlot);
        }
        else if (integer3 == CreativeModeTab.TAB_INVENTORY.getId()) {
            ((ItemPickerMenu)this.menu).slots.clear();
            ((ItemPickerMenu)this.menu).slots.addAll(this.originalSlots);
            this.originalSlots = null;
        }
        if (this.searchBox != null) {
            if (bda == CreativeModeTab.TAB_SEARCH) {
                this.searchBox.setVisible(true);
                this.searchBox.setCanLoseFocus(false);
                this.searchBox.setFocus(true);
                if (integer3 != bda.getId()) {
                    this.searchBox.setValue("");
                }
                this.refreshSearchResults();
            }
            else {
                this.searchBox.setVisible(false);
                this.searchBox.setCanLoseFocus(true);
                this.searchBox.setFocus(false);
                this.searchBox.setValue("");
            }
        }
        this.scrollOffs = 0.0f;
        ((ItemPickerMenu)this.menu).scrollTo(0.0f);
    }
    
    @Override
    public boolean mouseScrolled(final double double1, final double double2, final double double3) {
        if (!this.canScroll()) {
            return false;
        }
        final int integer8 = (((ItemPickerMenu)this.menu).items.size() + 9 - 1) / 9 - 5;
        this.scrollOffs -= (float)(double3 / integer8);
        this.scrollOffs = Mth.clamp(this.scrollOffs, 0.0f, 1.0f);
        ((ItemPickerMenu)this.menu).scrollTo(this.scrollOffs);
        return true;
    }
    
    @Override
    protected boolean hasClickedOutside(final double double1, final double double2, final int integer3, final int integer4, final int integer5) {
        final boolean boolean9 = double1 < integer3 || double2 < integer4 || double1 >= integer3 + this.imageWidth || double2 >= integer4 + this.imageHeight;
        return this.hasClickedOutside = (boolean9 && !this.checkTabClicked(CreativeModeTab.TABS[CreativeModeInventoryScreen.selectedTab], double1, double2));
    }
    
    protected boolean insideScrollbar(final double double1, final double double2) {
        final int integer6 = this.leftPos;
        final int integer7 = this.topPos;
        final int integer8 = integer6 + 175;
        final int integer9 = integer7 + 18;
        final int integer10 = integer8 + 14;
        final int integer11 = integer9 + 112;
        return double1 >= integer8 && double2 >= integer9 && double1 < integer10 && double2 < integer11;
    }
    
    @Override
    public boolean mouseDragged(final double double1, final double double2, final int integer, final double double4, final double double5) {
        if (this.scrolling) {
            final int integer2 = this.topPos + 18;
            final int integer3 = integer2 + 112;
            this.scrollOffs = ((float)double2 - integer2 - 7.5f) / (integer3 - integer2 - 15.0f);
            this.scrollOffs = Mth.clamp(this.scrollOffs, 0.0f, 1.0f);
            ((ItemPickerMenu)this.menu).scrollTo(this.scrollOffs);
            return true;
        }
        return super.mouseDragged(double1, double2, integer, double4, double5);
    }
    
    @Override
    public void render(final int integer1, final int integer2, final float float3) {
        this.renderBackground();
        super.render(integer1, integer2, float3);
        for (final CreativeModeTab bda8 : CreativeModeTab.TABS) {
            if (this.checkTabHovering(bda8, integer1, integer2)) {
                break;
            }
        }
        if (this.destroyItemSlot != null && CreativeModeInventoryScreen.selectedTab == CreativeModeTab.TAB_INVENTORY.getId() && this.isHovering(this.destroyItemSlot.x, this.destroyItemSlot.y, 16, 16, integer1, integer2)) {
            this.renderTooltip(I18n.get("inventory.binSlot"), integer1, integer2);
        }
        RenderSystem.color4f(1.0f, 1.0f, 1.0f, 1.0f);
        this.renderTooltip(integer1, integer2);
    }
    
    @Override
    protected void renderTooltip(final ItemStack bek, final int integer2, final int integer3) {
        if (CreativeModeInventoryScreen.selectedTab == CreativeModeTab.TAB_SEARCH.getId()) {
            final List<Component> list5 = bek.getTooltipLines(this.minecraft.player, this.minecraft.options.advancedItemTooltips ? TooltipFlag.Default.ADVANCED : TooltipFlag.Default.NORMAL);
            final List<String> list6 = Lists.newArrayListWithCapacity(list5.size());
            for (final Component lf8 : list5) {
                list6.add(lf8.getColoredString());
            }
            final Item bef7 = bek.getItem();
            CreativeModeTab bda8 = bef7.getItemCategory();
            if (bda8 == null && bef7 == Items.ENCHANTED_BOOK) {
                final Map<Enchantment, Integer> map9 = EnchantmentHelper.getEnchantments(bek);
                if (map9.size() == 1) {
                    final Enchantment bhu10 = map9.keySet().iterator().next();
                    for (final CreativeModeTab bda9 : CreativeModeTab.TABS) {
                        if (bda9.hasEnchantmentCategory(bhu10.category)) {
                            bda8 = bda9;
                            break;
                        }
                    }
                }
            }
            final Item object;
            final List<String> list7;
            this.visibleTags.forEach((sm, aaz) -> {
                if (aaz.contains(object)) {
                    list7.add(1, "" + ChatFormatting.BOLD + ChatFormatting.DARK_PURPLE + "#" + sm);
                }
                return;
            });
            if (bda8 != null) {
                list6.add(1, "" + ChatFormatting.BOLD + ChatFormatting.BLUE + I18n.get(bda8.getName()));
            }
            for (int integer4 = 0; integer4 < list6.size(); ++integer4) {
                if (integer4 == 0) {
                    list6.set(integer4, bek.getRarity().color + list6.get(integer4));
                }
                else {
                    list6.set(integer4, ChatFormatting.GRAY + list6.get(integer4));
                }
            }
            this.renderTooltip(list6, integer2, integer3);
        }
        else {
            super.renderTooltip(bek, integer2, integer3);
        }
    }
    
    @Override
    protected void renderBg(final float float1, final int integer2, final int integer3) {
        RenderSystem.color4f(1.0f, 1.0f, 1.0f, 1.0f);
        final CreativeModeTab bda5 = CreativeModeTab.TABS[CreativeModeInventoryScreen.selectedTab];
        for (final CreativeModeTab bda6 : CreativeModeTab.TABS) {
            this.minecraft.getTextureManager().bind(CreativeModeInventoryScreen.CREATIVE_TABS_LOCATION);
            if (bda6.getId() != CreativeModeInventoryScreen.selectedTab) {
                this.renderTabButton(bda6);
            }
        }
        this.minecraft.getTextureManager().bind(new ResourceLocation("textures/gui/container/creative_inventory/tab_" + bda5.getBackgroundSuffix()));
        this.blit(this.leftPos, this.topPos, 0, 0, this.imageWidth, this.imageHeight);
        this.searchBox.render(integer2, integer3, float1);
        RenderSystem.color4f(1.0f, 1.0f, 1.0f, 1.0f);
        final int integer4 = this.leftPos + 175;
        final int integer5 = this.topPos + 18;
        final int integer6 = integer5 + 112;
        this.minecraft.getTextureManager().bind(CreativeModeInventoryScreen.CREATIVE_TABS_LOCATION);
        if (bda5.canScroll()) {
            this.blit(integer4, integer5 + (int)((integer6 - integer5 - 17) * this.scrollOffs), 232 + (this.canScroll() ? 0 : 12), 0, 12, 15);
        }
        this.renderTabButton(bda5);
        if (bda5 == CreativeModeTab.TAB_INVENTORY) {
            InventoryScreen.renderEntityInInventory(this.leftPos + 88, this.topPos + 45, 20, (float)(this.leftPos + 88 - integer2), (float)(this.topPos + 45 - 30 - integer3), this.minecraft.player);
        }
    }
    
    protected boolean checkTabClicked(final CreativeModeTab bda, final double double2, final double double3) {
        final int integer7 = bda.getColumn();
        int integer8 = 28 * integer7;
        int integer9 = 0;
        if (bda.isAlignedRight()) {
            integer8 = this.imageWidth - 28 * (6 - integer7) + 2;
        }
        else if (integer7 > 0) {
            integer8 += integer7;
        }
        if (bda.isTopRow()) {
            integer9 -= 32;
        }
        else {
            integer9 += this.imageHeight;
        }
        return double2 >= integer8 && double2 <= integer8 + 28 && double3 >= integer9 && double3 <= integer9 + 32;
    }
    
    protected boolean checkTabHovering(final CreativeModeTab bda, final int integer2, final int integer3) {
        final int integer4 = bda.getColumn();
        int integer5 = 28 * integer4;
        int integer6 = 0;
        if (bda.isAlignedRight()) {
            integer5 = this.imageWidth - 28 * (6 - integer4) + 2;
        }
        else if (integer4 > 0) {
            integer5 += integer4;
        }
        if (bda.isTopRow()) {
            integer6 -= 32;
        }
        else {
            integer6 += this.imageHeight;
        }
        if (this.isHovering(integer5 + 3, integer6 + 3, 23, 27, integer2, integer3)) {
            this.renderTooltip(I18n.get(bda.getName()), integer2, integer3);
            return true;
        }
        return false;
    }
    
    protected void renderTabButton(final CreativeModeTab bda) {
        final boolean boolean3 = bda.getId() == CreativeModeInventoryScreen.selectedTab;
        final boolean boolean4 = bda.isTopRow();
        final int integer5 = bda.getColumn();
        final int integer6 = integer5 * 28;
        int integer7 = 0;
        int integer8 = this.leftPos + 28 * integer5;
        int integer9 = this.topPos;
        final int integer10 = 32;
        if (boolean3) {
            integer7 += 32;
        }
        if (bda.isAlignedRight()) {
            integer8 = this.leftPos + this.imageWidth - 28 * (6 - integer5);
        }
        else if (integer5 > 0) {
            integer8 += integer5;
        }
        if (boolean4) {
            integer9 -= 28;
        }
        else {
            integer7 += 64;
            integer9 += this.imageHeight - 4;
        }
        this.blit(integer8, integer9, integer6, integer7, 28, 32);
        this.setBlitOffset(100);
        this.itemRenderer.blitOffset = 100.0f;
        integer8 += 6;
        integer9 += 8 + (boolean4 ? 1 : -1);
        RenderSystem.enableRescaleNormal();
        final ItemStack bek11 = bda.getIconItem();
        this.itemRenderer.renderAndDecorateItem(bek11, integer8, integer9);
        this.itemRenderer.renderGuiItemDecorations(this.font, bek11, integer8, integer9);
        this.itemRenderer.blitOffset = 0.0f;
        this.setBlitOffset(0);
    }
    
    public int getSelectedTab() {
        return CreativeModeInventoryScreen.selectedTab;
    }
    
    public static void handleHotbarLoadOrSave(final Minecraft dbl, final int integer, final boolean boolean3, final boolean boolean4) {
        final LocalPlayer dpy5 = dbl.player;
        final HotbarManager dbh6 = dbl.getHotbarManager();
        final Hotbar dqa7 = dbh6.get(integer);
        if (boolean3) {
            for (int integer2 = 0; integer2 < Inventory.getSelectionSize(); ++integer2) {
                final ItemStack bek9 = dqa7.get(integer2).copy();
                dpy5.inventory.setItem(integer2, bek9);
                dbl.gameMode.handleCreativeModeItemAdd(bek9, 36 + integer2);
            }
            dpy5.inventoryMenu.broadcastChanges();
        }
        else if (boolean4) {
            for (int integer2 = 0; integer2 < Inventory.getSelectionSize(); ++integer2) {
                dqa7.set(integer2, dpy5.inventory.getItem(integer2).copy());
            }
            final String string8 = dbl.options.keyHotbarSlots[integer].getTranslatedKeyMessage();
            final String string9 = dbl.options.keyLoadHotbarActivator.getTranslatedKeyMessage();
            dbl.gui.setOverlayMessage(new TranslatableComponent("inventory.hotbarSaved", new Object[] { string9, string8 }), false);
            dbh6.save();
        }
    }
    
    static {
        CREATIVE_TABS_LOCATION = new ResourceLocation("textures/gui/container/creative_inventory/tabs.png");
        CONTAINER = new SimpleContainer(45);
        CreativeModeInventoryScreen.selectedTab = CreativeModeTab.TAB_BUILDING_BLOCKS.getId();
    }
    
    public static class ItemPickerMenu extends AbstractContainerMenu {
        public final NonNullList<ItemStack> items;
        
        public ItemPickerMenu(final Player ayg) {
            super(null, 0);
            this.items = NonNullList.<ItemStack>create();
            final Inventory ayf3 = ayg.inventory;
            for (int integer4 = 0; integer4 < 5; ++integer4) {
                for (int integer5 = 0; integer5 < 9; ++integer5) {
                    this.addSlot(new CustomCreativeSlot(CreativeModeInventoryScreen.CONTAINER, integer4 * 9 + integer5, 9 + integer5 * 18, 18 + integer4 * 18));
                }
            }
            for (int integer4 = 0; integer4 < 9; ++integer4) {
                this.addSlot(new Slot(ayf3, integer4, 9 + integer4 * 18, 112));
            }
            this.scrollTo(0.0f);
        }
        
        @Override
        public boolean stillValid(final Player ayg) {
            return true;
        }
        
        public void scrollTo(final float float1) {
            final int integer3 = (this.items.size() + 9 - 1) / 9 - 5;
            int integer4 = (int)(float1 * integer3 + 0.5);
            if (integer4 < 0) {
                integer4 = 0;
            }
            for (int integer5 = 0; integer5 < 5; ++integer5) {
                for (int integer6 = 0; integer6 < 9; ++integer6) {
                    final int integer7 = integer6 + (integer5 + integer4) * 9;
                    if (integer7 >= 0 && integer7 < this.items.size()) {
                        CreativeModeInventoryScreen.CONTAINER.setItem(integer6 + integer5 * 9, this.items.get(integer7));
                    }
                    else {
                        CreativeModeInventoryScreen.CONTAINER.setItem(integer6 + integer5 * 9, ItemStack.EMPTY);
                    }
                }
            }
        }
        
        public boolean canScroll() {
            return this.items.size() > 45;
        }
        
        @Override
        public ItemStack quickMoveStack(final Player ayg, final int integer) {
            if (integer >= this.slots.size() - 9 && integer < this.slots.size()) {
                final Slot bbx4 = this.slots.get(integer);
                if (bbx4 != null && bbx4.hasItem()) {
                    bbx4.set(ItemStack.EMPTY);
                }
            }
            return ItemStack.EMPTY;
        }
        
        @Override
        public boolean canTakeItemForPickAll(final ItemStack bek, final Slot bbx) {
            return bbx.container != CreativeModeInventoryScreen.CONTAINER;
        }
        
        @Override
        public boolean canDragTo(final Slot bbx) {
            return bbx.container != CreativeModeInventoryScreen.CONTAINER;
        }
    }
    
    static class SlotWrapper extends Slot {
        private final Slot target;
        
        public SlotWrapper(final Slot bbx, final int integer2, final int integer3, final int integer4) {
            super(bbx.container, integer2, integer3, integer4);
            this.target = bbx;
        }
        
        @Override
        public ItemStack onTake(final Player ayg, final ItemStack bek) {
            return this.target.onTake(ayg, bek);
        }
        
        @Override
        public boolean mayPlace(final ItemStack bek) {
            return this.target.mayPlace(bek);
        }
        
        @Override
        public ItemStack getItem() {
            return this.target.getItem();
        }
        
        @Override
        public boolean hasItem() {
            return this.target.hasItem();
        }
        
        @Override
        public void set(final ItemStack bek) {
            this.target.set(bek);
        }
        
        @Override
        public void setChanged() {
            this.target.setChanged();
        }
        
        @Override
        public int getMaxStackSize() {
            return this.target.getMaxStackSize();
        }
        
        @Override
        public int getMaxStackSize(final ItemStack bek) {
            return this.target.getMaxStackSize(bek);
        }
        
        @Nullable
        @Override
        public Pair<ResourceLocation, ResourceLocation> getNoItemIcon() {
            return this.target.getNoItemIcon();
        }
        
        @Override
        public ItemStack remove(final int integer) {
            return this.target.remove(integer);
        }
        
        @Override
        public boolean isActive() {
            return this.target.isActive();
        }
        
        @Override
        public boolean mayPickup(final Player ayg) {
            return this.target.mayPickup(ayg);
        }
    }
    
    static class CustomCreativeSlot extends Slot {
        public CustomCreativeSlot(final Container ajb, final int integer2, final int integer3, final int integer4) {
            super(ajb, integer2, integer3, integer4);
        }
        
        @Override
        public boolean mayPickup(final Player ayg) {
            if (super.mayPickup(ayg) && this.hasItem()) {
                return this.getItem().getTagElement("CustomCreativeLock") == null;
            }
            return !this.hasItem();
        }
    }
}
