package net.minecraft.client.gui.screens.recipebook;

import net.minecraft.world.item.ItemStack;
import it.unimi.dsi.fastutil.ints.IntList;
import net.minecraft.world.entity.player.StackedContents;
import java.util.Iterator;
import net.minecraft.stats.RecipeBook;
import com.google.common.collect.Sets;
import com.google.common.collect.Lists;
import java.util.Set;
import net.minecraft.world.item.crafting.Recipe;
import java.util.List;

public class RecipeCollection {
    private final List<Recipe<?>> recipes;
    private final Set<Recipe<?>> craftable;
    private final Set<Recipe<?>> fitsDimensions;
    private final Set<Recipe<?>> known;
    private boolean singleResultItem;
    
    public RecipeCollection() {
        this.recipes = Lists.newArrayList();
        this.craftable = Sets.newHashSet();
        this.fitsDimensions = Sets.newHashSet();
        this.known = Sets.newHashSet();
        this.singleResultItem = true;
    }
    
    public boolean hasKnownRecipes() {
        return !this.known.isEmpty();
    }
    
    public void updateKnownRecipes(final RecipeBook aal) {
        for (final Recipe<?> bgt4 : this.recipes) {
            if (aal.contains(bgt4)) {
                this.known.add(bgt4);
            }
        }
    }
    
    public void canCraft(final StackedContents ayi, final int integer2, final int integer3, final RecipeBook aal) {
        for (int integer4 = 0; integer4 < this.recipes.size(); ++integer4) {
            final Recipe<?> bgt7 = this.recipes.get(integer4);
            final boolean boolean8 = bgt7.canCraftInDimensions(integer2, integer3) && aal.contains(bgt7);
            if (boolean8) {
                this.fitsDimensions.add(bgt7);
            }
            else {
                this.fitsDimensions.remove(bgt7);
            }
            if (boolean8 && ayi.canCraft(bgt7, null)) {
                this.craftable.add(bgt7);
            }
            else {
                this.craftable.remove(bgt7);
            }
        }
    }
    
    public boolean isCraftable(final Recipe<?> bgt) {
        return this.craftable.contains(bgt);
    }
    
    public boolean hasCraftable() {
        return !this.craftable.isEmpty();
    }
    
    public boolean hasFitting() {
        return !this.fitsDimensions.isEmpty();
    }
    
    public List<Recipe<?>> getRecipes() {
        return this.recipes;
    }
    
    public List<Recipe<?>> getRecipes(final boolean boolean1) {
        final List<Recipe<?>> list3 = Lists.newArrayList();
        final Set<Recipe<?>> set4 = boolean1 ? this.craftable : this.fitsDimensions;
        for (final Recipe<?> bgt6 : this.recipes) {
            if (set4.contains(bgt6)) {
                list3.add(bgt6);
            }
        }
        return list3;
    }
    
    public List<Recipe<?>> getDisplayRecipes(final boolean boolean1) {
        final List<Recipe<?>> list3 = Lists.newArrayList();
        for (final Recipe<?> bgt5 : this.recipes) {
            if (this.fitsDimensions.contains(bgt5) && this.craftable.contains(bgt5) == boolean1) {
                list3.add(bgt5);
            }
        }
        return list3;
    }
    
    public void add(final Recipe<?> bgt) {
        this.recipes.add(bgt);
        if (this.singleResultItem) {
            final ItemStack bek3 = this.recipes.get(0).getResultItem();
            final ItemStack bek4 = bgt.getResultItem();
            this.singleResultItem = (ItemStack.isSame(bek3, bek4) && ItemStack.tagMatches(bek3, bek4));
        }
    }
    
    public boolean hasSingleResultItem() {
        return this.singleResultItem;
    }
}
