package net.minecraft.client.model;

import net.minecraft.util.Mth;
import com.google.common.collect.ImmutableList;
import net.minecraft.client.model.geom.ModelPart;
import net.minecraft.world.entity.Entity;

public class DolphinModel<T extends Entity> extends ListModel<T> {
    private final ModelPart body;
    private final ModelPart tail;
    private final ModelPart tailFin;
    
    public DolphinModel() {
        this.texWidth = 64;
        this.texHeight = 64;
        final float float2 = 18.0f;
        final float float3 = -8.0f;
        (this.body = new ModelPart(this, 22, 0)).addBox(-4.0f, -7.0f, 0.0f, 8.0f, 7.0f, 13.0f);
        this.body.setPos(0.0f, 22.0f, -5.0f);
        final ModelPart dnf4 = new ModelPart(this, 51, 0);
        dnf4.addBox(-0.5f, 0.0f, 8.0f, 1.0f, 4.0f, 5.0f);
        dnf4.xRot = 1.0471976f;
        this.body.addChild(dnf4);
        final ModelPart dnf5 = new ModelPart(this, 48, 20);
        dnf5.mirror = true;
        dnf5.addBox(-0.5f, -4.0f, 0.0f, 1.0f, 4.0f, 7.0f);
        dnf5.setPos(2.0f, -2.0f, 4.0f);
        dnf5.xRot = 1.0471976f;
        dnf5.zRot = 2.0943952f;
        this.body.addChild(dnf5);
        final ModelPart dnf6 = new ModelPart(this, 48, 20);
        dnf6.addBox(-0.5f, -4.0f, 0.0f, 1.0f, 4.0f, 7.0f);
        dnf6.setPos(-2.0f, -2.0f, 4.0f);
        dnf6.xRot = 1.0471976f;
        dnf6.zRot = -2.0943952f;
        this.body.addChild(dnf6);
        (this.tail = new ModelPart(this, 0, 19)).addBox(-2.0f, -2.5f, 0.0f, 4.0f, 5.0f, 11.0f);
        this.tail.setPos(0.0f, -2.5f, 11.0f);
        this.tail.xRot = -0.10471976f;
        this.body.addChild(this.tail);
        (this.tailFin = new ModelPart(this, 19, 20)).addBox(-5.0f, -0.5f, 0.0f, 10.0f, 1.0f, 6.0f);
        this.tailFin.setPos(0.0f, 0.0f, 9.0f);
        this.tailFin.xRot = 0.0f;
        this.tail.addChild(this.tailFin);
        final ModelPart dnf7 = new ModelPart(this, 0, 0);
        dnf7.addBox(-4.0f, -3.0f, -3.0f, 8.0f, 7.0f, 6.0f);
        dnf7.setPos(0.0f, -4.0f, -3.0f);
        final ModelPart dnf8 = new ModelPart(this, 0, 13);
        dnf8.addBox(-1.0f, 2.0f, -7.0f, 2.0f, 2.0f, 4.0f);
        dnf7.addChild(dnf8);
        this.body.addChild(dnf7);
    }
    
    @Override
    public Iterable<ModelPart> parts() {
        return ImmutableList.<ModelPart>of(this.body);
    }
    
    @Override
    public void setupAnim(final T akn, final float float2, final float float3, final float float4, final float float5, final float float6) {
        this.body.xRot = float6 * 0.017453292f;
        this.body.yRot = float5 * 0.017453292f;
        if (Entity.getHorizontalDistanceSqr(akn.getDeltaMovement()) > 1.0E-7) {
            final ModelPart body = this.body;
            body.xRot += -0.05f + -0.05f * Mth.cos(float4 * 0.3f);
            this.tail.xRot = -0.1f * Mth.cos(float4 * 0.3f);
            this.tailFin.xRot = -0.2f * Mth.cos(float4 * 0.3f);
        }
    }
}
