package net.minecraft.client.multiplayer;

import org.apache.logging.log4j.LogManager;
import net.minecraft.network.protocol.game.ServerboundPickItemPacket;
import net.minecraft.world.entity.animal.horse.AbstractHorse;
import net.minecraft.network.protocol.game.ServerboundSetCreativeModeSlotPacket;
import net.minecraft.network.protocol.game.ServerboundContainerButtonClickPacket;
import net.minecraft.network.protocol.game.ServerboundPlaceRecipePacket;
import net.minecraft.world.item.crafting.Recipe;
import net.minecraft.network.protocol.game.ServerboundContainerClickPacket;
import net.minecraft.world.inventory.ClickType;
import net.minecraft.world.phys.Vec3;
import net.minecraft.world.phys.EntityHitResult;
import net.minecraft.network.protocol.game.ServerboundInteractPacket;
import net.minecraft.world.entity.Entity;
import net.minecraft.client.ClientRecipeBook;
import net.minecraft.stats.StatsCounter;
import net.minecraft.world.InteractionResultHolder;
import net.minecraft.network.protocol.game.ServerboundUseItemPacket;
import net.minecraft.world.item.UseOnContext;
import net.minecraft.network.protocol.game.ServerboundUseItemOnPacket;
import net.minecraft.world.InteractionResult;
import net.minecraft.world.phys.BlockHitResult;
import net.minecraft.world.InteractionHand;
import net.minecraft.client.player.LocalPlayer;
import net.minecraft.network.protocol.Packet;
import net.minecraft.network.protocol.game.ServerboundSetCarriedItemPacket;
import net.minecraft.world.level.block.SoundType;
import net.minecraft.util.Mth;
import net.minecraft.client.resources.sounds.SoundInstance;
import net.minecraft.client.resources.sounds.SimpleSoundInstance;
import net.minecraft.sounds.SoundSource;
import net.minecraft.world.level.BlockGetter;
import net.minecraft.world.level.material.FluidState;
import net.minecraft.world.level.block.Block;
import net.minecraft.world.level.block.state.BlockState;
import net.minecraft.world.level.LevelAccessor;
import net.minecraft.world.level.block.JigsawBlock;
import net.minecraft.world.level.block.StructureBlock;
import net.minecraft.world.level.block.CommandBlock;
import net.minecraft.world.level.Level;
import net.minecraft.world.entity.player.Player;
import net.minecraft.core.Direction;
import net.minecraft.world.phys.PosAndRot;
import net.minecraft.network.protocol.game.ServerboundPlayerActionPacket;
import com.mojang.datafixers.util.Pair;
import it.unimi.dsi.fastutil.objects.Object2ObjectLinkedOpenHashMap;
import net.minecraft.world.level.GameType;
import net.minecraft.world.item.ItemStack;
import net.minecraft.core.BlockPos;
import net.minecraft.client.Minecraft;
import org.apache.logging.log4j.Logger;

public class MultiPlayerGameMode {
    private static final Logger LOGGER;
    private final Minecraft minecraft;
    private final ClientPacketListener connection;
    private BlockPos destroyBlockPos;
    private ItemStack destroyingItem;
    private float destroyProgress;
    private float destroyTicks;
    private int destroyDelay;
    private boolean isDestroying;
    private GameType localPlayerMode;
    private final Object2ObjectLinkedOpenHashMap<Pair<BlockPos, ServerboundPlayerActionPacket.Action>, PosAndRot> unAckedActions;
    private int carriedIndex;
    
    public MultiPlayerGameMode(final Minecraft dbl, final ClientPacketListener dnm) {
        this.destroyBlockPos = new BlockPos(-1, -1, -1);
        this.destroyingItem = ItemStack.EMPTY;
        this.localPlayerMode = GameType.SURVIVAL;
        this.unAckedActions = (Object2ObjectLinkedOpenHashMap<Pair<BlockPos, ServerboundPlayerActionPacket.Action>, PosAndRot>)new Object2ObjectLinkedOpenHashMap();
        this.minecraft = dbl;
        this.connection = dnm;
    }
    
    public static void creativeDestroyBlock(final Minecraft dbl, final MultiPlayerGameMode dno, final BlockPos fk, final Direction fp) {
        if (!dbl.level.extinguishFire(dbl.player, fk, fp)) {
            dno.destroyBlock(fk);
        }
    }
    
    public void adjustPlayer(final Player ayg) {
        this.localPlayerMode.updatePlayerAbilities(ayg.abilities);
    }
    
    public void setLocalMode(final GameType bjq) {
        (this.localPlayerMode = bjq).updatePlayerAbilities(this.minecraft.player.abilities);
    }
    
    public boolean canHurtPlayer() {
        return this.localPlayerMode.isSurvival();
    }
    
    public boolean destroyBlock(final BlockPos fk) {
        if (this.minecraft.player.blockActionRestricted(this.minecraft.level, fk, this.localPlayerMode)) {
            return false;
        }
        final Level bjt3 = this.minecraft.level;
        final BlockState byg4 = bjt3.getBlockState(fk);
        if (!this.minecraft.player.getMainHandItem().getItem().canAttackBlock(byg4, bjt3, fk, this.minecraft.player)) {
            return false;
        }
        final Block bpe5 = byg4.getBlock();
        if ((bpe5 instanceof CommandBlock || bpe5 instanceof StructureBlock || bpe5 instanceof JigsawBlock) && !this.minecraft.player.canUseGameMasterBlocks()) {
            return false;
        }
        if (byg4.isAir()) {
            return false;
        }
        bpe5.playerWillDestroy(bjt3, fk, byg4, this.minecraft.player);
        final FluidState cog6 = bjt3.getFluidState(fk);
        final boolean boolean7 = bjt3.setBlock(fk, cog6.createLegacyBlock(), 11);
        if (boolean7) {
            bpe5.destroy(bjt3, fk, byg4);
        }
        return boolean7;
    }
    
    public boolean startDestroyBlock(final BlockPos fk, final Direction fp) {
        if (this.minecraft.player.blockActionRestricted(this.minecraft.level, fk, this.localPlayerMode)) {
            return false;
        }
        if (!this.minecraft.level.getWorldBorder().isWithinBounds(fk)) {
            return false;
        }
        if (this.localPlayerMode.isCreative()) {
            final BlockState byg4 = this.minecraft.level.getBlockState(fk);
            this.minecraft.getTutorial().onDestroyBlock(this.minecraft.level, fk, byg4, 1.0f);
            this.sendBlockAction(ServerboundPlayerActionPacket.Action.START_DESTROY_BLOCK, fk, fp);
            creativeDestroyBlock(this.minecraft, this, fk, fp);
            this.destroyDelay = 5;
        }
        else if (!this.isDestroying || !this.sameDestroyTarget(fk)) {
            if (this.isDestroying) {
                this.sendBlockAction(ServerboundPlayerActionPacket.Action.ABORT_DESTROY_BLOCK, this.destroyBlockPos, fp);
            }
            final BlockState byg4 = this.minecraft.level.getBlockState(fk);
            this.minecraft.getTutorial().onDestroyBlock(this.minecraft.level, fk, byg4, 0.0f);
            this.sendBlockAction(ServerboundPlayerActionPacket.Action.START_DESTROY_BLOCK, fk, fp);
            final boolean boolean5 = !byg4.isAir();
            if (boolean5 && this.destroyProgress == 0.0f) {
                byg4.attack(this.minecraft.level, fk, this.minecraft.player);
            }
            if (boolean5 && byg4.getDestroyProgress(this.minecraft.player, this.minecraft.player.level, fk) >= 1.0f) {
                this.destroyBlock(fk);
            }
            else {
                this.isDestroying = true;
                this.destroyBlockPos = fk;
                this.destroyingItem = this.minecraft.player.getMainHandItem();
                this.destroyProgress = 0.0f;
                this.destroyTicks = 0.0f;
                this.minecraft.level.destroyBlockProgress(this.minecraft.player.getId(), this.destroyBlockPos, (int)(this.destroyProgress * 10.0f) - 1);
            }
        }
        return true;
    }
    
    public void stopDestroyBlock() {
        if (this.isDestroying) {
            final BlockState byg2 = this.minecraft.level.getBlockState(this.destroyBlockPos);
            this.minecraft.getTutorial().onDestroyBlock(this.minecraft.level, this.destroyBlockPos, byg2, -1.0f);
            this.sendBlockAction(ServerboundPlayerActionPacket.Action.ABORT_DESTROY_BLOCK, this.destroyBlockPos, Direction.DOWN);
            this.isDestroying = false;
            this.destroyProgress = 0.0f;
            this.minecraft.level.destroyBlockProgress(this.minecraft.player.getId(), this.destroyBlockPos, -1);
            this.minecraft.player.resetAttackStrengthTicker();
        }
    }
    
    public boolean continueDestroyBlock(final BlockPos fk, final Direction fp) {
        this.ensureHasSentCarriedItem();
        if (this.destroyDelay > 0) {
            --this.destroyDelay;
            return true;
        }
        if (this.localPlayerMode.isCreative() && this.minecraft.level.getWorldBorder().isWithinBounds(fk)) {
            this.destroyDelay = 5;
            final BlockState byg4 = this.minecraft.level.getBlockState(fk);
            this.minecraft.getTutorial().onDestroyBlock(this.minecraft.level, fk, byg4, 1.0f);
            this.sendBlockAction(ServerboundPlayerActionPacket.Action.START_DESTROY_BLOCK, fk, fp);
            creativeDestroyBlock(this.minecraft, this, fk, fp);
            return true;
        }
        if (!this.sameDestroyTarget(fk)) {
            return this.startDestroyBlock(fk, fp);
        }
        final BlockState byg4 = this.minecraft.level.getBlockState(fk);
        if (byg4.isAir()) {
            return this.isDestroying = false;
        }
        this.destroyProgress += byg4.getDestroyProgress(this.minecraft.player, this.minecraft.player.level, fk);
        if (this.destroyTicks % 4.0f == 0.0f) {
            final SoundType buj5 = byg4.getSoundType();
            this.minecraft.getSoundManager().play(new SimpleSoundInstance(buj5.getHitSound(), SoundSource.BLOCKS, (buj5.getVolume() + 1.0f) / 8.0f, buj5.getPitch() * 0.5f, fk));
        }
        ++this.destroyTicks;
        this.minecraft.getTutorial().onDestroyBlock(this.minecraft.level, fk, byg4, Mth.clamp(this.destroyProgress, 0.0f, 1.0f));
        if (this.destroyProgress >= 1.0f) {
            this.isDestroying = false;
            this.sendBlockAction(ServerboundPlayerActionPacket.Action.STOP_DESTROY_BLOCK, fk, fp);
            this.destroyBlock(fk);
            this.destroyProgress = 0.0f;
            this.destroyTicks = 0.0f;
            this.destroyDelay = 5;
        }
        this.minecraft.level.destroyBlockProgress(this.minecraft.player.getId(), this.destroyBlockPos, (int)(this.destroyProgress * 10.0f) - 1);
        return true;
    }
    
    public float getPickRange() {
        if (this.localPlayerMode.isCreative()) {
            return 5.0f;
        }
        return 4.5f;
    }
    
    public void tick() {
        this.ensureHasSentCarriedItem();
        if (this.connection.getConnection().isConnected()) {
            this.connection.getConnection().tick();
        }
        else {
            this.connection.getConnection().handleDisconnection();
        }
    }
    
    private boolean sameDestroyTarget(final BlockPos fk) {
        final ItemStack bek3 = this.minecraft.player.getMainHandItem();
        boolean boolean4 = this.destroyingItem.isEmpty() && bek3.isEmpty();
        if (!this.destroyingItem.isEmpty() && !bek3.isEmpty()) {
            boolean4 = (bek3.getItem() == this.destroyingItem.getItem() && ItemStack.tagMatches(bek3, this.destroyingItem) && (bek3.isDamageableItem() || bek3.getDamageValue() == this.destroyingItem.getDamageValue()));
        }
        return fk.equals(this.destroyBlockPos) && boolean4;
    }
    
    private void ensureHasSentCarriedItem() {
        final int integer2 = this.minecraft.player.inventory.selected;
        if (integer2 != this.carriedIndex) {
            this.carriedIndex = integer2;
            this.connection.send(new ServerboundSetCarriedItemPacket(this.carriedIndex));
        }
    }
    
    public InteractionResult useItemOn(final LocalPlayer dpy, final ClientLevel dnl, final InteractionHand ajh, final BlockHitResult cvd) {
        this.ensureHasSentCarriedItem();
        final BlockPos fk6 = cvd.getBlockPos();
        if (!this.minecraft.level.getWorldBorder().isWithinBounds(fk6)) {
            return InteractionResult.FAIL;
        }
        final ItemStack bek7 = dpy.getItemInHand(ajh);
        if (this.localPlayerMode == GameType.SPECTATOR) {
            this.connection.send(new ServerboundUseItemOnPacket(ajh, cvd));
            return InteractionResult.SUCCESS;
        }
        final boolean boolean8 = !dpy.getMainHandItem().isEmpty() || !dpy.getOffhandItem().isEmpty();
        final boolean boolean9 = dpy.isSecondaryUseActive() && boolean8;
        if (!boolean9) {
            final InteractionResult aji10 = dnl.getBlockState(fk6).use(dnl, dpy, ajh, cvd);
            if (aji10.consumesAction()) {
                this.connection.send(new ServerboundUseItemOnPacket(ajh, cvd));
                return aji10;
            }
        }
        this.connection.send(new ServerboundUseItemOnPacket(ajh, cvd));
        if (bek7.isEmpty() || dpy.getCooldowns().isOnCooldown(bek7.getItem())) {
            return InteractionResult.PASS;
        }
        final UseOnContext bfw11 = new UseOnContext(dpy, ajh, cvd);
        InteractionResult aji10;
        if (this.localPlayerMode.isCreative()) {
            final int integer12 = bek7.getCount();
            aji10 = bek7.useOn(bfw11);
            bek7.setCount(integer12);
        }
        else {
            aji10 = bek7.useOn(bfw11);
        }
        return aji10;
    }
    
    public InteractionResult useItem(final Player ayg, final Level bjt, final InteractionHand ajh) {
        if (this.localPlayerMode == GameType.SPECTATOR) {
            return InteractionResult.PASS;
        }
        this.ensureHasSentCarriedItem();
        this.connection.send(new ServerboundUseItemPacket(ajh));
        final ItemStack bek5 = ayg.getItemInHand(ajh);
        if (ayg.getCooldowns().isOnCooldown(bek5.getItem())) {
            return InteractionResult.PASS;
        }
        final int integer6 = bek5.getCount();
        final InteractionResultHolder<ItemStack> ajj7 = bek5.use(bjt, ayg, ajh);
        final ItemStack bek6 = ajj7.getObject();
        if (bek6 != bek5 || bek6.getCount() != integer6) {
            ayg.setItemInHand(ajh, bek6);
        }
        return ajj7.getResult();
    }
    
    public LocalPlayer createPlayer(final ClientLevel dnl, final StatsCounter aas, final ClientRecipeBook dba) {
        return new LocalPlayer(this.minecraft, dnl, this.connection, aas, dba);
    }
    
    public void attack(final Player ayg, final Entity akn) {
        this.ensureHasSentCarriedItem();
        this.connection.send(new ServerboundInteractPacket(akn));
        if (this.localPlayerMode != GameType.SPECTATOR) {
            ayg.attack(akn);
            ayg.resetAttackStrengthTicker();
        }
    }
    
    public InteractionResult interact(final Player ayg, final Entity akn, final InteractionHand ajh) {
        this.ensureHasSentCarriedItem();
        this.connection.send(new ServerboundInteractPacket(akn, ajh));
        if (this.localPlayerMode == GameType.SPECTATOR) {
            return InteractionResult.PASS;
        }
        return ayg.interactOn(akn, ajh);
    }
    
    public InteractionResult interactAt(final Player ayg, final Entity akn, final EntityHitResult cve, final InteractionHand ajh) {
        this.ensureHasSentCarriedItem();
        final Vec3 cvi6 = cve.getLocation().subtract(akn.getX(), akn.getY(), akn.getZ());
        this.connection.send(new ServerboundInteractPacket(akn, ajh, cvi6));
        if (this.localPlayerMode == GameType.SPECTATOR) {
            return InteractionResult.PASS;
        }
        return akn.interactAt(ayg, cvi6, ajh);
    }
    
    public ItemStack handleInventoryMouseClick(final int integer1, final int integer2, final int integer3, final ClickType bas, final Player ayg) {
        final short short7 = ayg.containerMenu.backup(ayg.inventory);
        final ItemStack bek8 = ayg.containerMenu.clicked(integer2, integer3, bas, ayg);
        this.connection.send(new ServerboundContainerClickPacket(integer1, integer2, integer3, bas, bek8, short7));
        return bek8;
    }
    
    public void handlePlaceRecipe(final int integer, final Recipe<?> bgt, final boolean boolean3) {
        this.connection.send(new ServerboundPlaceRecipePacket(integer, bgt, boolean3));
    }
    
    public void handleInventoryButtonClick(final int integer1, final int integer2) {
        this.connection.send(new ServerboundContainerButtonClickPacket(integer1, integer2));
    }
    
    public void handleCreativeModeItemAdd(final ItemStack bek, final int integer) {
        if (this.localPlayerMode.isCreative()) {
            this.connection.send(new ServerboundSetCreativeModeSlotPacket(integer, bek));
        }
    }
    
    public void handleCreativeModeItemDrop(final ItemStack bek) {
        if (this.localPlayerMode.isCreative() && !bek.isEmpty()) {
            this.connection.send(new ServerboundSetCreativeModeSlotPacket(-1, bek));
        }
    }
    
    public void releaseUsingItem(final Player ayg) {
        this.ensureHasSentCarriedItem();
        this.connection.send(new ServerboundPlayerActionPacket(ServerboundPlayerActionPacket.Action.RELEASE_USE_ITEM, BlockPos.ZERO, Direction.DOWN));
        ayg.releaseUsingItem();
    }
    
    public boolean hasExperience() {
        return this.localPlayerMode.isSurvival();
    }
    
    public boolean hasMissTime() {
        return !this.localPlayerMode.isCreative();
    }
    
    public boolean hasInfiniteItems() {
        return this.localPlayerMode.isCreative();
    }
    
    public boolean hasFarPickRange() {
        return this.localPlayerMode.isCreative();
    }
    
    public boolean isServerControlledInventory() {
        return this.minecraft.player.isPassenger() && this.minecraft.player.getVehicle() instanceof AbstractHorse;
    }
    
    public boolean isAlwaysFlying() {
        return this.localPlayerMode == GameType.SPECTATOR;
    }
    
    public GameType getPlayerMode() {
        return this.localPlayerMode;
    }
    
    public boolean isDestroying() {
        return this.isDestroying;
    }
    
    public void handlePickItem(final int integer) {
        this.connection.send(new ServerboundPickItemPacket(integer));
    }
    
    private void sendBlockAction(final ServerboundPlayerActionPacket.Action a, final BlockPos fk, final Direction fp) {
        final LocalPlayer dpy5 = this.minecraft.player;
        this.unAckedActions.put(Pair.of((Object)fk, (Object)a), new PosAndRot(dpy5.position(), dpy5.xRot, dpy5.yRot));
        this.connection.send(new ServerboundPlayerActionPacket(a, fk, fp));
    }
    
    public void handleBlockBreakAck(final ClientLevel dnl, final BlockPos fk, final BlockState byg, final ServerboundPlayerActionPacket.Action a, final boolean boolean5) {
        final PosAndRot cvg7 = (PosAndRot)this.unAckedActions.remove(Pair.of((Object)fk, (Object)a));
        if (cvg7 == null || !boolean5 || (a != ServerboundPlayerActionPacket.Action.START_DESTROY_BLOCK && dnl.getBlockState(fk) != byg)) {
            dnl.setKnownState(fk, byg);
            if (cvg7 != null) {
                final Vec3 cvi8 = cvg7.pos();
                this.minecraft.player.absMoveTo(cvi8.x, cvi8.y, cvi8.z, cvg7.yRot(), cvg7.xRot());
            }
        }
        while (this.unAckedActions.size() >= 50) {
            final Pair<BlockPos, ServerboundPlayerActionPacket.Action> pair8 = (Pair<BlockPos, ServerboundPlayerActionPacket.Action>)this.unAckedActions.firstKey();
            this.unAckedActions.removeFirst();
            MultiPlayerGameMode.LOGGER.error("Too many unacked block actions, dropping " + pair8);
        }
    }
    
    static {
        LOGGER = LogManager.getLogger();
    }
}
