package net.minecraft.client.particle;

import net.minecraft.core.particles.ParticleOptions;
import net.minecraft.core.particles.SimpleParticleType;
import net.minecraft.util.Mth;
import net.minecraft.world.level.Level;

public class DragonBreathParticle extends TextureSheetParticle {
    private boolean hasHitGround;
    private final SpriteSet sprites;
    
    private DragonBreathParticle(final Level bjt, final double double2, final double double3, final double double4, final double double5, final double double6, final double double7, final SpriteSet dpj) {
        super(bjt, double2, double3, double4);
        this.xd = double5;
        this.yd = double6;
        this.zd = double7;
        this.rCol = Mth.nextFloat(this.random, 0.7176471f, 0.8745098f);
        this.gCol = Mth.nextFloat(this.random, 0.0f, 0.0f);
        this.bCol = Mth.nextFloat(this.random, 0.8235294f, 0.9764706f);
        this.quadSize *= 0.75f;
        this.lifetime = (int)(20.0 / (this.random.nextFloat() * 0.8 + 0.2));
        this.hasHitGround = false;
        this.hasPhysics = false;
        this.setSpriteFromAge(this.sprites = dpj);
    }
    
    @Override
    public void tick() {
        this.xo = this.x;
        this.yo = this.y;
        this.zo = this.z;
        if (this.age++ >= this.lifetime) {
            this.remove();
            return;
        }
        this.setSpriteFromAge(this.sprites);
        if (this.onGround) {
            this.yd = 0.0;
            this.hasHitGround = true;
        }
        if (this.hasHitGround) {
            this.yd += 0.002;
        }
        this.move(this.xd, this.yd, this.zd);
        if (this.y == this.yo) {
            this.xd *= 1.1;
            this.zd *= 1.1;
        }
        this.xd *= 0.9599999785423279;
        this.zd *= 0.9599999785423279;
        if (this.hasHitGround) {
            this.yd *= 0.9599999785423279;
        }
    }
    
    @Override
    public ParticleRenderType getRenderType() {
        return ParticleRenderType.PARTICLE_SHEET_OPAQUE;
    }
    
    @Override
    public float getQuadSize(final float float1) {
        return this.quadSize * Mth.clamp((this.age + float1) / this.lifetime * 32.0f, 0.0f, 1.0f);
    }
    
    public static class Provider implements ParticleProvider<SimpleParticleType> {
        private final SpriteSet sprites;
        
        public Provider(final SpriteSet dpj) {
            this.sprites = dpj;
        }
        
        @Override
        public Particle createParticle(final SimpleParticleType gw, final Level bjt, final double double3, final double double4, final double double5, final double double6, final double double7, final double double8) {
            return new DragonBreathParticle(bjt, double3, double4, double5, double6, double7, double8, this.sprites, null);
        }
    }
}
