package net.minecraft.client.player;

import net.minecraft.world.entity.ai.attributes.AttributeInstance;
import net.minecraft.world.item.Items;
import net.minecraft.world.entity.monster.SharedMonsterAttributes;
import com.google.common.hash.Hashing;
import net.minecraft.client.renderer.texture.AbstractTexture;
import net.minecraft.client.renderer.texture.TextureManager;
import java.io.File;
import net.minecraft.util.StringUtil;
import net.minecraft.client.renderer.texture.HttpTexture;
import net.minecraft.client.resources.DefaultPlayerSkin;
import net.minecraft.resources.ResourceLocation;
import javax.annotation.Nullable;
import net.minecraft.world.level.GameType;
import net.minecraft.client.Minecraft;
import net.minecraft.world.level.Level;
import com.mojang.authlib.GameProfile;
import net.minecraft.client.multiplayer.ClientLevel;
import net.minecraft.client.multiplayer.PlayerInfo;
import net.minecraft.world.entity.player.Player;

public abstract class AbstractClientPlayer extends Player {
    private PlayerInfo playerInfo;
    public float elytraRotX;
    public float elytraRotY;
    public float elytraRotZ;
    public final ClientLevel clientLevel;
    
    public AbstractClientPlayer(final ClientLevel dnl, final GameProfile gameProfile) {
        super(dnl, gameProfile);
        this.clientLevel = dnl;
    }
    
    @Override
    public boolean isSpectator() {
        final PlayerInfo dnp2 = Minecraft.getInstance().getConnection().getPlayerInfo(this.getGameProfile().getId());
        return dnp2 != null && dnp2.getGameMode() == GameType.SPECTATOR;
    }
    
    @Override
    public boolean isCreative() {
        final PlayerInfo dnp2 = Minecraft.getInstance().getConnection().getPlayerInfo(this.getGameProfile().getId());
        return dnp2 != null && dnp2.getGameMode() == GameType.CREATIVE;
    }
    
    public boolean isCapeLoaded() {
        return this.getPlayerInfo() != null;
    }
    
    @Nullable
    protected PlayerInfo getPlayerInfo() {
        if (this.playerInfo == null) {
            this.playerInfo = Minecraft.getInstance().getConnection().getPlayerInfo(this.getUUID());
        }
        return this.playerInfo;
    }
    
    public boolean isSkinLoaded() {
        final PlayerInfo dnp2 = this.getPlayerInfo();
        return dnp2 != null && dnp2.isSkinLoaded();
    }
    
    public ResourceLocation getSkinTextureLocation() {
        final PlayerInfo dnp2 = this.getPlayerInfo();
        return (dnp2 == null) ? DefaultPlayerSkin.getDefaultSkin(this.getUUID()) : dnp2.getSkinLocation();
    }
    
    @Nullable
    public ResourceLocation getCloakTextureLocation() {
        final PlayerInfo dnp2 = this.getPlayerInfo();
        return (dnp2 == null) ? null : dnp2.getCapeLocation();
    }
    
    public boolean isElytraLoaded() {
        return this.getPlayerInfo() != null;
    }
    
    @Nullable
    public ResourceLocation getElytraTextureLocation() {
        final PlayerInfo dnp2 = this.getPlayerInfo();
        return (dnp2 == null) ? null : dnp2.getElytraLocation();
    }
    
    public static HttpTexture registerSkinTexture(final ResourceLocation sm, final String string) {
        final TextureManager eah3 = Minecraft.getInstance().getTextureManager();
        AbstractTexture dzt4 = eah3.getTexture(sm);
        if (dzt4 == null) {
            dzt4 = new HttpTexture(null, String.format("http://skins.minecraft.net/MinecraftSkins/%s.png", StringUtil.stripColor(string)), DefaultPlayerSkin.getDefaultSkin(Player.createPlayerUUID(string)), true, null);
            eah3.register(sm, dzt4);
        }
        return (HttpTexture)dzt4;
    }
    
    public static ResourceLocation getSkinLocation(final String string) {
        return new ResourceLocation("skins/" + Hashing.sha1().hashUnencodedChars(StringUtil.stripColor(string)));
    }
    
    public String getModelName() {
        final PlayerInfo dnp2 = this.getPlayerInfo();
        return (dnp2 == null) ? DefaultPlayerSkin.getSkinModelName(this.getUUID()) : dnp2.getModelName();
    }
    
    public float getFieldOfViewModifier() {
        float float2 = 1.0f;
        if (this.abilities.flying) {
            float2 *= 1.1f;
        }
        final AttributeInstance alo3 = this.getAttribute(SharedMonsterAttributes.MOVEMENT_SPEED);
        float2 *= (float)((alo3.getValue() / this.abilities.getWalkingSpeed() + 1.0) / 2.0);
        if (this.abilities.getWalkingSpeed() == 0.0f || Float.isNaN(float2) || Float.isInfinite(float2)) {
            float2 = 1.0f;
        }
        if (this.isUsingItem() && this.getUseItem().getItem() == Items.BOW) {
            final int integer4 = this.getTicksUsingItem();
            float float3 = integer4 / 20.0f;
            if (float3 > 1.0f) {
                float3 = 1.0f;
            }
            else {
                float3 *= float3;
            }
            float2 *= 1.0f - float3 * 0.15f;
        }
        return float2;
    }
}
