package net.minecraft.client.renderer;

import java.util.function.Function;
import java.util.Comparator;
import java.util.Arrays;
import net.minecraft.world.item.DyeColor;
import net.minecraft.world.level.block.Block;
import com.mojang.blaze3d.vertex.VertexConsumer;
import net.minecraft.client.resources.model.Material;
import net.minecraft.world.item.Item;
import net.minecraft.world.level.block.entity.BlockEntity;
import net.minecraft.client.renderer.blockentity.BlockEntityRenderDispatcher;
import net.minecraft.world.level.block.ShulkerBoxBlock;
import net.minecraft.world.level.block.Blocks;
import net.minecraft.world.level.block.BedBlock;
import net.minecraft.world.level.block.AbstractBannerBlock;
import net.minecraft.core.Direction;
import net.minecraft.client.renderer.blockentity.SkullBlockRenderer;
import net.minecraft.nbt.Tag;
import net.minecraft.nbt.CompoundTag;
import net.minecraft.world.level.block.entity.SkullBlockEntity;
import java.util.UUID;
import com.mojang.authlib.GameProfile;
import org.apache.commons.lang3.StringUtils;
import net.minecraft.nbt.NbtUtils;
import net.minecraft.world.level.block.AbstractSkullBlock;
import net.minecraft.client.renderer.blockentity.BannerRenderer;
import net.minecraft.world.item.ShieldItem;
import net.minecraft.client.renderer.entity.ItemRenderer;
import net.minecraft.client.resources.model.ModelBakery;
import net.minecraft.world.item.Items;
import net.minecraft.world.item.BlockItem;
import com.mojang.blaze3d.vertex.PoseStack;
import net.minecraft.world.item.ItemStack;
import net.minecraft.world.level.block.entity.TrappedChestBlockEntity;
import net.minecraft.client.model.TridentModel;
import net.minecraft.client.model.ShieldModel;
import net.minecraft.world.level.block.entity.ConduitBlockEntity;
import net.minecraft.world.level.block.entity.BedBlockEntity;
import net.minecraft.world.level.block.entity.BannerBlockEntity;
import net.minecraft.world.level.block.entity.EnderChestBlockEntity;
import net.minecraft.world.level.block.entity.ChestBlockEntity;
import net.minecraft.world.level.block.entity.ShulkerBoxBlockEntity;

public class BlockEntityWithoutLevelRenderer {
    private static final ShulkerBoxBlockEntity[] SHULKER_BOXES;
    private static final ShulkerBoxBlockEntity DEFAULT_SHULKER_BOX;
    public static final BlockEntityWithoutLevelRenderer instance;
    private final ChestBlockEntity chest;
    private final ChestBlockEntity trappedChest;
    private final EnderChestBlockEntity enderChest;
    private final BannerBlockEntity banner;
    private final BedBlockEntity bed;
    private final ConduitBlockEntity conduit;
    private final ShieldModel shieldModel;
    private final TridentModel tridentModel;
    
    public BlockEntityWithoutLevelRenderer() {
        this.chest = new ChestBlockEntity();
        this.trappedChest = new TrappedChestBlockEntity();
        this.enderChest = new EnderChestBlockEntity();
        this.banner = new BannerBlockEntity();
        this.bed = new BedBlockEntity();
        this.conduit = new ConduitBlockEntity();
        this.shieldModel = new ShieldModel();
        this.tridentModel = new TridentModel();
    }
    
    public void renderByItem(final ItemStack bek, final PoseStack cyi, final MultiBufferSource dqq, final int integer4, final int integer5) {
        final Item bef7 = bek.getItem();
        if (!(bef7 instanceof BlockItem)) {
            if (bef7 == Items.SHIELD) {
                final boolean boolean8 = bek.getTagElement("BlockEntityTag") != null;
                cyi.pushPose();
                cyi.scale(1.0f, -1.0f, -1.0f);
                final Material ebv9 = boolean8 ? ModelBakery.SHIELD_BASE : ModelBakery.NO_PATTERN_SHIELD;
                final VertexConsumer cyl10 = ebv9.sprite().wrap(ItemRenderer.getFoilBuffer(dqq, this.shieldModel.renderType(ebv9.atlasLocation()), false, bek.hasFoil()));
                this.shieldModel.handle().render(cyi, cyl10, integer4, integer5, 1.0f, 1.0f, 1.0f, 1.0f);
                if (boolean8) {
                    this.banner.fromItem(bek, ShieldItem.getColor(bek));
                    BannerRenderer.renderPatterns(this.banner, cyi, dqq, integer4, integer5, this.shieldModel.plate(), ebv9, false);
                }
                else {
                    this.shieldModel.plate().render(cyi, cyl10, integer4, integer5, 1.0f, 1.0f, 1.0f, 1.0f);
                }
                cyi.popPose();
            }
            else if (bef7 == Items.TRIDENT) {
                cyi.pushPose();
                cyi.scale(1.0f, -1.0f, -1.0f);
                final VertexConsumer cyl11 = ItemRenderer.getFoilBuffer(dqq, this.tridentModel.renderType(TridentModel.TEXTURE), false, bek.hasFoil());
                this.tridentModel.renderToBuffer(cyi, cyl11, integer4, integer5, 1.0f, 1.0f, 1.0f, 1.0f);
                cyi.popPose();
            }
            return;
        }
        final Block bpe8 = ((BlockItem)bef7).getBlock();
        if (bpe8 instanceof AbstractSkullBlock) {
            GameProfile gameProfile9 = null;
            if (bek.hasTag()) {
                final CompoundTag jt10 = bek.getTag();
                if (jt10.contains("SkullOwner", 10)) {
                    gameProfile9 = NbtUtils.readGameProfile(jt10.getCompound("SkullOwner"));
                }
                else if (jt10.contains("SkullOwner", 8) && !StringUtils.isBlank((CharSequence)jt10.getString("SkullOwner"))) {
                    gameProfile9 = new GameProfile((UUID)null, jt10.getString("SkullOwner"));
                    gameProfile9 = SkullBlockEntity.updateGameprofile(gameProfile9);
                    jt10.remove("SkullOwner");
                    jt10.put("SkullOwner", NbtUtils.writeGameProfile(new CompoundTag(), gameProfile9));
                }
            }
            SkullBlockRenderer.renderSkull(null, 180.0f, ((AbstractSkullBlock)bpe8).getType(), gameProfile9, 0.0f, cyi, dqq, integer4);
            return;
        }
        BlockEntity bwi9;
        if (bpe8 instanceof AbstractBannerBlock) {
            this.banner.fromItem(bek, ((AbstractBannerBlock)bpe8).getColor());
            bwi9 = this.banner;
        }
        else if (bpe8 instanceof BedBlock) {
            this.bed.setColor(((BedBlock)bpe8).getColor());
            bwi9 = this.bed;
        }
        else if (bpe8 == Blocks.CONDUIT) {
            bwi9 = this.conduit;
        }
        else if (bpe8 == Blocks.CHEST) {
            bwi9 = this.chest;
        }
        else if (bpe8 == Blocks.ENDER_CHEST) {
            bwi9 = this.enderChest;
        }
        else if (bpe8 == Blocks.TRAPPED_CHEST) {
            bwi9 = this.trappedChest;
        }
        else {
            if (!(bpe8 instanceof ShulkerBoxBlock)) {
                return;
            }
            final DyeColor bdg10 = ShulkerBoxBlock.getColorFromItem(bef7);
            if (bdg10 == null) {
                bwi9 = BlockEntityWithoutLevelRenderer.DEFAULT_SHULKER_BOX;
            }
            else {
                bwi9 = BlockEntityWithoutLevelRenderer.SHULKER_BOXES[bdg10.getId()];
            }
        }
        BlockEntityRenderDispatcher.instance.<BlockEntity>renderItem(bwi9, cyi, dqq, integer4, integer5);
    }
    
    static {
        SHULKER_BOXES = Arrays.<DyeColor>stream(DyeColor.values()).sorted(Comparator.comparingInt(DyeColor::getId)).map(ShulkerBoxBlockEntity::new).<ShulkerBoxBlockEntity>toArray(ShulkerBoxBlockEntity[]::new);
        DEFAULT_SHULKER_BOX = new ShulkerBoxBlockEntity((DyeColor)null);
        instance = new BlockEntityWithoutLevelRenderer();
    }
}
