package net.minecraft.client.renderer;

import net.minecraft.Util;
import com.google.common.collect.Maps;
import net.minecraft.world.level.block.Blocks;
import net.minecraft.world.level.material.Fluids;
import java.util.HashMap;
import net.minecraft.world.level.material.FluidState;
import net.minecraft.world.item.Item;
import net.minecraft.world.item.BlockItem;
import net.minecraft.world.item.ItemStack;
import net.minecraft.world.level.block.LeavesBlock;
import net.minecraft.world.level.block.state.BlockState;
import net.minecraft.world.level.material.Fluid;
import net.minecraft.world.level.block.Block;
import java.util.Map;

public class ItemBlockRenderTypes {
    private static final Map<Block, RenderType> TYPE_BY_BLOCK;
    private static final Map<Fluid, RenderType> TYPE_BY_FLUID;
    private static boolean renderCutout;
    
    public static RenderType getChunkRenderType(final BlockState byg) {
        final Block bpe2 = byg.getBlock();
        if (bpe2 instanceof LeavesBlock) {
            return ItemBlockRenderTypes.renderCutout ? RenderType.cutoutMipped() : RenderType.solid();
        }
        final RenderType dqy3 = ItemBlockRenderTypes.TYPE_BY_BLOCK.get(bpe2);
        if (dqy3 != null) {
            return dqy3;
        }
        return RenderType.solid();
    }
    
    public static RenderType getRenderType(final BlockState byg) {
        final RenderType dqy2 = getChunkRenderType(byg);
        if (dqy2 == RenderType.translucent()) {
            return Sheets.translucentBlockSheet();
        }
        return Sheets.cutoutBlockSheet();
    }
    
    public static RenderType getRenderType(final ItemStack bek) {
        final Item bef2 = bek.getItem();
        if (bef2 instanceof BlockItem) {
            final Block bpe3 = ((BlockItem)bef2).getBlock();
            return getRenderType(bpe3.defaultBlockState());
        }
        return Sheets.translucentBlockSheet();
    }
    
    public static RenderType getRenderLayer(final FluidState cog) {
        final RenderType dqy2 = ItemBlockRenderTypes.TYPE_BY_FLUID.get(cog.getType());
        if (dqy2 != null) {
            return dqy2;
        }
        return RenderType.solid();
    }
    
    public static void setFancy(final boolean boolean1) {
        ItemBlockRenderTypes.renderCutout = boolean1;
    }
    
    static {
        final RenderType dqy2;
        final RenderType dqy3;
        final RenderType dqy4;
        TYPE_BY_BLOCK = Util.<Map<Block, RenderType>>make(Maps.newHashMap(), hashMap -> {
            dqy2 = RenderType.cutoutMipped();
            hashMap.put(Blocks.GRASS_BLOCK, dqy2);
            hashMap.put(Blocks.IRON_BARS, dqy2);
            hashMap.put(Blocks.GLASS_PANE, dqy2);
            hashMap.put(Blocks.TRIPWIRE_HOOK, dqy2);
            hashMap.put(Blocks.HOPPER, dqy2);
            hashMap.put(Blocks.JUNGLE_LEAVES, dqy2);
            hashMap.put(Blocks.OAK_LEAVES, dqy2);
            hashMap.put(Blocks.SPRUCE_LEAVES, dqy2);
            hashMap.put(Blocks.ACACIA_LEAVES, dqy2);
            hashMap.put(Blocks.BIRCH_LEAVES, dqy2);
            hashMap.put(Blocks.DARK_OAK_LEAVES, dqy2);
            dqy3 = RenderType.cutout();
            hashMap.put(Blocks.OAK_SAPLING, dqy3);
            hashMap.put(Blocks.SPRUCE_SAPLING, dqy3);
            hashMap.put(Blocks.BIRCH_SAPLING, dqy3);
            hashMap.put(Blocks.JUNGLE_SAPLING, dqy3);
            hashMap.put(Blocks.ACACIA_SAPLING, dqy3);
            hashMap.put(Blocks.DARK_OAK_SAPLING, dqy3);
            hashMap.put(Blocks.GLASS, dqy3);
            hashMap.put(Blocks.WHITE_BED, dqy3);
            hashMap.put(Blocks.ORANGE_BED, dqy3);
            hashMap.put(Blocks.MAGENTA_BED, dqy3);
            hashMap.put(Blocks.LIGHT_BLUE_BED, dqy3);
            hashMap.put(Blocks.YELLOW_BED, dqy3);
            hashMap.put(Blocks.LIME_BED, dqy3);
            hashMap.put(Blocks.PINK_BED, dqy3);
            hashMap.put(Blocks.GRAY_BED, dqy3);
            hashMap.put(Blocks.LIGHT_GRAY_BED, dqy3);
            hashMap.put(Blocks.CYAN_BED, dqy3);
            hashMap.put(Blocks.PURPLE_BED, dqy3);
            hashMap.put(Blocks.BLUE_BED, dqy3);
            hashMap.put(Blocks.BROWN_BED, dqy3);
            hashMap.put(Blocks.GREEN_BED, dqy3);
            hashMap.put(Blocks.RED_BED, dqy3);
            hashMap.put(Blocks.BLACK_BED, dqy3);
            hashMap.put(Blocks.POWERED_RAIL, dqy3);
            hashMap.put(Blocks.DETECTOR_RAIL, dqy3);
            hashMap.put(Blocks.COBWEB, dqy3);
            hashMap.put(Blocks.GRASS, dqy3);
            hashMap.put(Blocks.FERN, dqy3);
            hashMap.put(Blocks.DEAD_BUSH, dqy3);
            hashMap.put(Blocks.SEAGRASS, dqy3);
            hashMap.put(Blocks.TALL_SEAGRASS, dqy3);
            hashMap.put(Blocks.DANDELION, dqy3);
            hashMap.put(Blocks.POPPY, dqy3);
            hashMap.put(Blocks.BLUE_ORCHID, dqy3);
            hashMap.put(Blocks.ALLIUM, dqy3);
            hashMap.put(Blocks.AZURE_BLUET, dqy3);
            hashMap.put(Blocks.RED_TULIP, dqy3);
            hashMap.put(Blocks.ORANGE_TULIP, dqy3);
            hashMap.put(Blocks.WHITE_TULIP, dqy3);
            hashMap.put(Blocks.PINK_TULIP, dqy3);
            hashMap.put(Blocks.OXEYE_DAISY, dqy3);
            hashMap.put(Blocks.CORNFLOWER, dqy3);
            hashMap.put(Blocks.WITHER_ROSE, dqy3);
            hashMap.put(Blocks.LILY_OF_THE_VALLEY, dqy3);
            hashMap.put(Blocks.BROWN_MUSHROOM, dqy3);
            hashMap.put(Blocks.RED_MUSHROOM, dqy3);
            hashMap.put(Blocks.TORCH, dqy3);
            hashMap.put(Blocks.WALL_TORCH, dqy3);
            hashMap.put(Blocks.FIRE, dqy3);
            hashMap.put(Blocks.SPAWNER, dqy3);
            hashMap.put(Blocks.REDSTONE_WIRE, dqy3);
            hashMap.put(Blocks.WHEAT, dqy3);
            hashMap.put(Blocks.OAK_DOOR, dqy3);
            hashMap.put(Blocks.LADDER, dqy3);
            hashMap.put(Blocks.RAIL, dqy3);
            hashMap.put(Blocks.IRON_DOOR, dqy3);
            hashMap.put(Blocks.REDSTONE_TORCH, dqy3);
            hashMap.put(Blocks.REDSTONE_WALL_TORCH, dqy3);
            hashMap.put(Blocks.CACTUS, dqy3);
            hashMap.put(Blocks.SUGAR_CANE, dqy3);
            hashMap.put(Blocks.REPEATER, dqy3);
            hashMap.put(Blocks.OAK_TRAPDOOR, dqy3);
            hashMap.put(Blocks.SPRUCE_TRAPDOOR, dqy3);
            hashMap.put(Blocks.BIRCH_TRAPDOOR, dqy3);
            hashMap.put(Blocks.JUNGLE_TRAPDOOR, dqy3);
            hashMap.put(Blocks.ACACIA_TRAPDOOR, dqy3);
            hashMap.put(Blocks.DARK_OAK_TRAPDOOR, dqy3);
            hashMap.put(Blocks.ATTACHED_PUMPKIN_STEM, dqy3);
            hashMap.put(Blocks.ATTACHED_MELON_STEM, dqy3);
            hashMap.put(Blocks.PUMPKIN_STEM, dqy3);
            hashMap.put(Blocks.MELON_STEM, dqy3);
            hashMap.put(Blocks.VINE, dqy3);
            hashMap.put(Blocks.LILY_PAD, dqy3);
            hashMap.put(Blocks.NETHER_WART, dqy3);
            hashMap.put(Blocks.BREWING_STAND, dqy3);
            hashMap.put(Blocks.COCOA, dqy3);
            hashMap.put(Blocks.BEACON, dqy3);
            hashMap.put(Blocks.FLOWER_POT, dqy3);
            hashMap.put(Blocks.POTTED_OAK_SAPLING, dqy3);
            hashMap.put(Blocks.POTTED_SPRUCE_SAPLING, dqy3);
            hashMap.put(Blocks.POTTED_BIRCH_SAPLING, dqy3);
            hashMap.put(Blocks.POTTED_JUNGLE_SAPLING, dqy3);
            hashMap.put(Blocks.POTTED_ACACIA_SAPLING, dqy3);
            hashMap.put(Blocks.POTTED_DARK_OAK_SAPLING, dqy3);
            hashMap.put(Blocks.POTTED_FERN, dqy3);
            hashMap.put(Blocks.POTTED_DANDELION, dqy3);
            hashMap.put(Blocks.POTTED_POPPY, dqy3);
            hashMap.put(Blocks.POTTED_BLUE_ORCHID, dqy3);
            hashMap.put(Blocks.POTTED_ALLIUM, dqy3);
            hashMap.put(Blocks.POTTED_AZURE_BLUET, dqy3);
            hashMap.put(Blocks.POTTED_RED_TULIP, dqy3);
            hashMap.put(Blocks.POTTED_ORANGE_TULIP, dqy3);
            hashMap.put(Blocks.POTTED_WHITE_TULIP, dqy3);
            hashMap.put(Blocks.POTTED_PINK_TULIP, dqy3);
            hashMap.put(Blocks.POTTED_OXEYE_DAISY, dqy3);
            hashMap.put(Blocks.POTTED_CORNFLOWER, dqy3);
            hashMap.put(Blocks.POTTED_LILY_OF_THE_VALLEY, dqy3);
            hashMap.put(Blocks.POTTED_WITHER_ROSE, dqy3);
            hashMap.put(Blocks.POTTED_RED_MUSHROOM, dqy3);
            hashMap.put(Blocks.POTTED_BROWN_MUSHROOM, dqy3);
            hashMap.put(Blocks.POTTED_DEAD_BUSH, dqy3);
            hashMap.put(Blocks.POTTED_CACTUS, dqy3);
            hashMap.put(Blocks.CARROTS, dqy3);
            hashMap.put(Blocks.POTATOES, dqy3);
            hashMap.put(Blocks.COMPARATOR, dqy3);
            hashMap.put(Blocks.ACTIVATOR_RAIL, dqy3);
            hashMap.put(Blocks.IRON_TRAPDOOR, dqy3);
            hashMap.put(Blocks.SUNFLOWER, dqy3);
            hashMap.put(Blocks.LILAC, dqy3);
            hashMap.put(Blocks.ROSE_BUSH, dqy3);
            hashMap.put(Blocks.PEONY, dqy3);
            hashMap.put(Blocks.TALL_GRASS, dqy3);
            hashMap.put(Blocks.LARGE_FERN, dqy3);
            hashMap.put(Blocks.SPRUCE_DOOR, dqy3);
            hashMap.put(Blocks.BIRCH_DOOR, dqy3);
            hashMap.put(Blocks.JUNGLE_DOOR, dqy3);
            hashMap.put(Blocks.ACACIA_DOOR, dqy3);
            hashMap.put(Blocks.DARK_OAK_DOOR, dqy3);
            hashMap.put(Blocks.END_ROD, dqy3);
            hashMap.put(Blocks.CHORUS_PLANT, dqy3);
            hashMap.put(Blocks.CHORUS_FLOWER, dqy3);
            hashMap.put(Blocks.BEETROOTS, dqy3);
            hashMap.put(Blocks.KELP, dqy3);
            hashMap.put(Blocks.KELP_PLANT, dqy3);
            hashMap.put(Blocks.TURTLE_EGG, dqy3);
            hashMap.put(Blocks.DEAD_TUBE_CORAL, dqy3);
            hashMap.put(Blocks.DEAD_BRAIN_CORAL, dqy3);
            hashMap.put(Blocks.DEAD_BUBBLE_CORAL, dqy3);
            hashMap.put(Blocks.DEAD_FIRE_CORAL, dqy3);
            hashMap.put(Blocks.DEAD_HORN_CORAL, dqy3);
            hashMap.put(Blocks.TUBE_CORAL, dqy3);
            hashMap.put(Blocks.BRAIN_CORAL, dqy3);
            hashMap.put(Blocks.BUBBLE_CORAL, dqy3);
            hashMap.put(Blocks.FIRE_CORAL, dqy3);
            hashMap.put(Blocks.HORN_CORAL, dqy3);
            hashMap.put(Blocks.DEAD_TUBE_CORAL_FAN, dqy3);
            hashMap.put(Blocks.DEAD_BRAIN_CORAL_FAN, dqy3);
            hashMap.put(Blocks.DEAD_BUBBLE_CORAL_FAN, dqy3);
            hashMap.put(Blocks.DEAD_FIRE_CORAL_FAN, dqy3);
            hashMap.put(Blocks.DEAD_HORN_CORAL_FAN, dqy3);
            hashMap.put(Blocks.TUBE_CORAL_FAN, dqy3);
            hashMap.put(Blocks.BRAIN_CORAL_FAN, dqy3);
            hashMap.put(Blocks.BUBBLE_CORAL_FAN, dqy3);
            hashMap.put(Blocks.FIRE_CORAL_FAN, dqy3);
            hashMap.put(Blocks.HORN_CORAL_FAN, dqy3);
            hashMap.put(Blocks.DEAD_TUBE_CORAL_WALL_FAN, dqy3);
            hashMap.put(Blocks.DEAD_BRAIN_CORAL_WALL_FAN, dqy3);
            hashMap.put(Blocks.DEAD_BUBBLE_CORAL_WALL_FAN, dqy3);
            hashMap.put(Blocks.DEAD_FIRE_CORAL_WALL_FAN, dqy3);
            hashMap.put(Blocks.DEAD_HORN_CORAL_WALL_FAN, dqy3);
            hashMap.put(Blocks.TUBE_CORAL_WALL_FAN, dqy3);
            hashMap.put(Blocks.BRAIN_CORAL_WALL_FAN, dqy3);
            hashMap.put(Blocks.BUBBLE_CORAL_WALL_FAN, dqy3);
            hashMap.put(Blocks.FIRE_CORAL_WALL_FAN, dqy3);
            hashMap.put(Blocks.HORN_CORAL_WALL_FAN, dqy3);
            hashMap.put(Blocks.SEA_PICKLE, dqy3);
            hashMap.put(Blocks.CONDUIT, dqy3);
            hashMap.put(Blocks.BAMBOO_SAPLING, dqy3);
            hashMap.put(Blocks.BAMBOO, dqy3);
            hashMap.put(Blocks.POTTED_BAMBOO, dqy3);
            hashMap.put(Blocks.SCAFFOLDING, dqy3);
            hashMap.put(Blocks.STONECUTTER, dqy3);
            hashMap.put(Blocks.LANTERN, dqy3);
            hashMap.put(Blocks.CAMPFIRE, dqy3);
            hashMap.put(Blocks.SWEET_BERRY_BUSH, dqy3);
            dqy4 = RenderType.translucent();
            hashMap.put(Blocks.ICE, dqy4);
            hashMap.put(Blocks.NETHER_PORTAL, dqy4);
            hashMap.put(Blocks.WHITE_STAINED_GLASS, dqy4);
            hashMap.put(Blocks.ORANGE_STAINED_GLASS, dqy4);
            hashMap.put(Blocks.MAGENTA_STAINED_GLASS, dqy4);
            hashMap.put(Blocks.LIGHT_BLUE_STAINED_GLASS, dqy4);
            hashMap.put(Blocks.YELLOW_STAINED_GLASS, dqy4);
            hashMap.put(Blocks.LIME_STAINED_GLASS, dqy4);
            hashMap.put(Blocks.PINK_STAINED_GLASS, dqy4);
            hashMap.put(Blocks.GRAY_STAINED_GLASS, dqy4);
            hashMap.put(Blocks.LIGHT_GRAY_STAINED_GLASS, dqy4);
            hashMap.put(Blocks.CYAN_STAINED_GLASS, dqy4);
            hashMap.put(Blocks.PURPLE_STAINED_GLASS, dqy4);
            hashMap.put(Blocks.BLUE_STAINED_GLASS, dqy4);
            hashMap.put(Blocks.BROWN_STAINED_GLASS, dqy4);
            hashMap.put(Blocks.GREEN_STAINED_GLASS, dqy4);
            hashMap.put(Blocks.RED_STAINED_GLASS, dqy4);
            hashMap.put(Blocks.BLACK_STAINED_GLASS, dqy4);
            hashMap.put(Blocks.TRIPWIRE, dqy4);
            hashMap.put(Blocks.WHITE_STAINED_GLASS_PANE, dqy4);
            hashMap.put(Blocks.ORANGE_STAINED_GLASS_PANE, dqy4);
            hashMap.put(Blocks.MAGENTA_STAINED_GLASS_PANE, dqy4);
            hashMap.put(Blocks.LIGHT_BLUE_STAINED_GLASS_PANE, dqy4);
            hashMap.put(Blocks.YELLOW_STAINED_GLASS_PANE, dqy4);
            hashMap.put(Blocks.LIME_STAINED_GLASS_PANE, dqy4);
            hashMap.put(Blocks.PINK_STAINED_GLASS_PANE, dqy4);
            hashMap.put(Blocks.GRAY_STAINED_GLASS_PANE, dqy4);
            hashMap.put(Blocks.LIGHT_GRAY_STAINED_GLASS_PANE, dqy4);
            hashMap.put(Blocks.CYAN_STAINED_GLASS_PANE, dqy4);
            hashMap.put(Blocks.PURPLE_STAINED_GLASS_PANE, dqy4);
            hashMap.put(Blocks.BLUE_STAINED_GLASS_PANE, dqy4);
            hashMap.put(Blocks.BROWN_STAINED_GLASS_PANE, dqy4);
            hashMap.put(Blocks.GREEN_STAINED_GLASS_PANE, dqy4);
            hashMap.put(Blocks.RED_STAINED_GLASS_PANE, dqy4);
            hashMap.put(Blocks.BLACK_STAINED_GLASS_PANE, dqy4);
            hashMap.put(Blocks.SLIME_BLOCK, dqy4);
            hashMap.put(Blocks.HONEY_BLOCK, dqy4);
            hashMap.put(Blocks.FROSTED_ICE, dqy4);
            hashMap.put(Blocks.BUBBLE_COLUMN, dqy4);
            return;
        });
        final RenderType dqy5;
        TYPE_BY_FLUID = Util.<Map<Fluid, RenderType>>make(Maps.newHashMap(), hashMap -> {
            dqy5 = RenderType.translucent();
            hashMap.put(Fluids.FLOWING_WATER, dqy5);
            hashMap.put(Fluids.WATER, dqy5);
        });
    }
}
