package net.minecraft.client.renderer;

import net.minecraft.resources.ResourceLocation;
import java.util.Objects;
import net.minecraft.world.item.CrossbowItem;
import net.minecraft.world.item.Items;
import com.google.common.base.MoreObjects;
import net.minecraft.world.InteractionHand;
import com.mojang.math.Matrix4f;
import com.mojang.blaze3d.vertex.VertexConsumer;
import net.minecraft.world.level.saveddata.maps.MapItemSavedData;
import net.minecraft.world.level.Level;
import net.minecraft.world.item.MapItem;
import net.minecraft.client.player.AbstractClientPlayer;
import com.mojang.math.Vector3f;
import net.minecraft.client.player.LocalPlayer;
import net.minecraft.client.renderer.entity.player.PlayerRenderer;
import net.minecraft.world.entity.HumanoidArm;
import net.minecraft.util.Mth;
import net.minecraft.client.renderer.texture.OverlayTexture;
import com.mojang.blaze3d.vertex.PoseStack;
import net.minecraft.client.renderer.block.model.ItemTransforms;
import net.minecraft.world.entity.LivingEntity;
import net.minecraft.client.renderer.entity.ItemRenderer;
import net.minecraft.client.renderer.entity.EntityRenderDispatcher;
import net.minecraft.world.item.ItemStack;
import net.minecraft.client.Minecraft;

public class ItemInHandRenderer {
    private static final RenderType MAP_BACKGROUND;
    private static final RenderType MAP_BACKGROUND_CHECKERBOARD;
    private final Minecraft minecraft;
    private ItemStack mainHandItem;
    private ItemStack offHandItem;
    private float mainHandHeight;
    private float oMainHandHeight;
    private float offHandHeight;
    private float oOffHandHeight;
    private final EntityRenderDispatcher entityRenderDispatcher;
    private final ItemRenderer itemRenderer;
    
    public ItemInHandRenderer(final Minecraft dbl) {
        this.mainHandItem = ItemStack.EMPTY;
        this.offHandItem = ItemStack.EMPTY;
        this.minecraft = dbl;
        this.entityRenderDispatcher = dbl.getEntityRenderDispatcher();
        this.itemRenderer = dbl.getItemRenderer();
    }
    
    public void renderItem(final LivingEntity akw, final ItemStack bek, final ItemTransforms.TransformType b, final boolean boolean4, final PoseStack cyi, final MultiBufferSource dqq, final int integer) {
        if (bek.isEmpty()) {
            return;
        }
        this.itemRenderer.renderStatic(akw, bek, b, boolean4, cyi, dqq, akw.level, integer, OverlayTexture.NO_OVERLAY);
    }
    
    private float calculateMapTilt(final float float1) {
        float float2 = 1.0f - float1 / 45.0f + 0.1f;
        float2 = Mth.clamp(float2, 0.0f, 1.0f);
        float2 = -Mth.cos(float2 * 3.1415927f) * 0.5f + 0.5f;
        return float2;
    }
    
    private void renderMapHand(final PoseStack cyi, final MultiBufferSource dqq, final int integer, final HumanoidArm akv) {
        this.minecraft.getTextureManager().bind(this.minecraft.player.getSkinTextureLocation());
        final PlayerRenderer dzq6 = (PlayerRenderer)this.entityRenderDispatcher.<LocalPlayer>getRenderer(this.minecraft.player);
        cyi.pushPose();
        final float float7 = (akv == HumanoidArm.RIGHT) ? 1.0f : -1.0f;
        cyi.mulPose(Vector3f.YP.rotationDegrees(92.0f));
        cyi.mulPose(Vector3f.XP.rotationDegrees(45.0f));
        cyi.mulPose(Vector3f.ZP.rotationDegrees(float7 * -41.0f));
        cyi.translate(float7 * 0.3f, -1.100000023841858, 0.44999998807907104);
        if (akv == HumanoidArm.RIGHT) {
            dzq6.renderRightHand(cyi, dqq, integer, this.minecraft.player);
        }
        else {
            dzq6.renderLeftHand(cyi, dqq, integer, this.minecraft.player);
        }
        cyi.popPose();
    }
    
    private void renderOneHandedMap(final PoseStack cyi, final MultiBufferSource dqq, final int integer, final float float4, final HumanoidArm akv, final float float6, final ItemStack bek) {
        final float float7 = (akv == HumanoidArm.RIGHT) ? 1.0f : -1.0f;
        cyi.translate(float7 * 0.125f, -0.125, 0.0);
        if (!this.minecraft.player.isInvisible()) {
            cyi.pushPose();
            cyi.mulPose(Vector3f.ZP.rotationDegrees(float7 * 10.0f));
            this.renderPlayerArm(cyi, dqq, integer, float4, float6, akv);
            cyi.popPose();
        }
        cyi.pushPose();
        cyi.translate(float7 * 0.51f, -0.08f + float4 * -1.2f, -0.75);
        final float float8 = Mth.sqrt(float6);
        final float float9 = Mth.sin(float8 * 3.1415927f);
        final float float10 = -0.5f * float9;
        final float float11 = 0.4f * Mth.sin(float8 * 6.2831855f);
        final float float12 = -0.3f * Mth.sin(float6 * 3.1415927f);
        cyi.translate(float7 * float10, float11 - 0.3f * float9, float12);
        cyi.mulPose(Vector3f.XP.rotationDegrees(float9 * -45.0f));
        cyi.mulPose(Vector3f.YP.rotationDegrees(float7 * float9 * -30.0f));
        this.renderMap(cyi, dqq, integer, bek);
        cyi.popPose();
    }
    
    private void renderTwoHandedMap(final PoseStack cyi, final MultiBufferSource dqq, final int integer, final float float4, final float float5, final float float6) {
        final float float7 = Mth.sqrt(float6);
        final float float8 = -0.2f * Mth.sin(float6 * 3.1415927f);
        final float float9 = -0.4f * Mth.sin(float7 * 3.1415927f);
        cyi.translate(0.0, -float8 / 2.0f, float9);
        final float float10 = this.calculateMapTilt(float4);
        cyi.translate(0.0, 0.04f + float5 * -1.2f + float10 * -0.5f, -0.7200000286102295);
        cyi.mulPose(Vector3f.XP.rotationDegrees(float10 * -85.0f));
        if (!this.minecraft.player.isInvisible()) {
            cyi.pushPose();
            cyi.mulPose(Vector3f.YP.rotationDegrees(90.0f));
            this.renderMapHand(cyi, dqq, integer, HumanoidArm.RIGHT);
            this.renderMapHand(cyi, dqq, integer, HumanoidArm.LEFT);
            cyi.popPose();
        }
        final float float11 = Mth.sin(float7 * 3.1415927f);
        cyi.mulPose(Vector3f.XP.rotationDegrees(float11 * 20.0f));
        cyi.scale(2.0f, 2.0f, 2.0f);
        this.renderMap(cyi, dqq, integer, this.mainHandItem);
    }
    
    private void renderMap(final PoseStack cyi, final MultiBufferSource dqq, final int integer, final ItemStack bek) {
        cyi.mulPose(Vector3f.YP.rotationDegrees(180.0f));
        cyi.mulPose(Vector3f.ZP.rotationDegrees(180.0f));
        cyi.scale(0.38f, 0.38f, 0.38f);
        cyi.translate(-0.5, -0.5, 0.0);
        cyi.scale(0.0078125f, 0.0078125f, 0.0078125f);
        final MapItemSavedData crd6 = MapItem.getOrCreateSavedData(bek, this.minecraft.level);
        final VertexConsumer cyl7 = dqq.getBuffer((crd6 == null) ? ItemInHandRenderer.MAP_BACKGROUND : ItemInHandRenderer.MAP_BACKGROUND_CHECKERBOARD);
        final Matrix4f b8 = cyi.last().pose();
        cyl7.vertex(b8, -7.0f, 135.0f, 0.0f).color(255, 255, 255, 255).uv(0.0f, 1.0f).uv2(integer).endVertex();
        cyl7.vertex(b8, 135.0f, 135.0f, 0.0f).color(255, 255, 255, 255).uv(1.0f, 1.0f).uv2(integer).endVertex();
        cyl7.vertex(b8, 135.0f, -7.0f, 0.0f).color(255, 255, 255, 255).uv(1.0f, 0.0f).uv2(integer).endVertex();
        cyl7.vertex(b8, -7.0f, -7.0f, 0.0f).color(255, 255, 255, 255).uv(0.0f, 0.0f).uv2(integer).endVertex();
        if (crd6 != null) {
            this.minecraft.gameRenderer.getMapRenderer().render(cyi, dqq, crd6, false, integer);
        }
    }
    
    private void renderPlayerArm(final PoseStack cyi, final MultiBufferSource dqq, final int integer, final float float4, final float float5, final HumanoidArm akv) {
        final boolean boolean8 = akv != HumanoidArm.LEFT;
        final float float6 = boolean8 ? 1.0f : -1.0f;
        final float float7 = Mth.sqrt(float5);
        final float float8 = -0.3f * Mth.sin(float7 * 3.1415927f);
        final float float9 = 0.4f * Mth.sin(float7 * 6.2831855f);
        final float float10 = -0.4f * Mth.sin(float5 * 3.1415927f);
        cyi.translate(float6 * (float8 + 0.64000005f), float9 - 0.6f + float4 * -0.6f, float10 - 0.71999997f);
        cyi.mulPose(Vector3f.YP.rotationDegrees(float6 * 45.0f));
        final float float11 = Mth.sin(float5 * float5 * 3.1415927f);
        final float float12 = Mth.sin(float7 * 3.1415927f);
        cyi.mulPose(Vector3f.YP.rotationDegrees(float6 * float12 * 70.0f));
        cyi.mulPose(Vector3f.ZP.rotationDegrees(float6 * float11 * -20.0f));
        final AbstractClientPlayer dpv16 = this.minecraft.player;
        this.minecraft.getTextureManager().bind(dpv16.getSkinTextureLocation());
        cyi.translate(float6 * -1.0f, 3.5999999046325684, 3.5);
        cyi.mulPose(Vector3f.ZP.rotationDegrees(float6 * 120.0f));
        cyi.mulPose(Vector3f.XP.rotationDegrees(200.0f));
        cyi.mulPose(Vector3f.YP.rotationDegrees(float6 * -135.0f));
        cyi.translate(float6 * 5.6f, 0.0, 0.0);
        final PlayerRenderer dzq17 = (PlayerRenderer)this.entityRenderDispatcher.<AbstractClientPlayer>getRenderer(dpv16);
        if (boolean8) {
            dzq17.renderRightHand(cyi, dqq, integer, dpv16);
        }
        else {
            dzq17.renderLeftHand(cyi, dqq, integer, dpv16);
        }
    }
    
    private void applyEatTransform(final PoseStack cyi, final float float2, final HumanoidArm akv, final ItemStack bek) {
        final float float3 = this.minecraft.player.getUseItemRemainingTicks() - float2 + 1.0f;
        final float float4 = float3 / bek.getUseDuration();
        if (float4 < 0.8f) {
            final float float5 = Mth.abs(Mth.cos(float3 / 4.0f * 3.1415927f) * 0.1f);
            cyi.translate(0.0, float5, 0.0);
        }
        final float float5 = 1.0f - (float)Math.pow(float4, 27.0);
        final int integer9 = (akv == HumanoidArm.RIGHT) ? 1 : -1;
        cyi.translate(float5 * 0.6f * integer9, float5 * -0.5f, float5 * 0.0f);
        cyi.mulPose(Vector3f.YP.rotationDegrees(integer9 * float5 * 90.0f));
        cyi.mulPose(Vector3f.XP.rotationDegrees(float5 * 10.0f));
        cyi.mulPose(Vector3f.ZP.rotationDegrees(integer9 * float5 * 30.0f));
    }
    
    private void applyItemArmAttackTransform(final PoseStack cyi, final HumanoidArm akv, final float float3) {
        final int integer5 = (akv == HumanoidArm.RIGHT) ? 1 : -1;
        final float float4 = Mth.sin(float3 * float3 * 3.1415927f);
        cyi.mulPose(Vector3f.YP.rotationDegrees(integer5 * (45.0f + float4 * -20.0f)));
        final float float5 = Mth.sin(Mth.sqrt(float3) * 3.1415927f);
        cyi.mulPose(Vector3f.ZP.rotationDegrees(integer5 * float5 * -20.0f));
        cyi.mulPose(Vector3f.XP.rotationDegrees(float5 * -80.0f));
        cyi.mulPose(Vector3f.YP.rotationDegrees(integer5 * -45.0f));
    }
    
    private void applyItemArmTransform(final PoseStack cyi, final HumanoidArm akv, final float float3) {
        final int integer5 = (akv == HumanoidArm.RIGHT) ? 1 : -1;
        cyi.translate(integer5 * 0.56f, -0.52f + float3 * -0.6f, -0.7200000286102295);
    }
    
    public void renderHandsWithItems(final float float1, final PoseStack cyi, final MultiBufferSource.BufferSource a, final LocalPlayer dpy, final int integer) {
        final float float2 = dpy.getAttackAnim(float1);
        final InteractionHand ajh8 = MoreObjects.<InteractionHand>firstNonNull(dpy.swingingArm, InteractionHand.MAIN_HAND);
        final float float3 = Mth.lerp(float1, dpy.xRotO, dpy.xRot);
        boolean boolean10 = true;
        boolean boolean11 = true;
        if (dpy.isUsingItem()) {
            final ItemStack bek12 = dpy.getUseItem();
            if (bek12.getItem() == Items.BOW || bek12.getItem() == Items.CROSSBOW) {
                boolean10 = (dpy.getUsedItemHand() == InteractionHand.MAIN_HAND);
                boolean11 = !boolean10;
            }
            final InteractionHand ajh9 = dpy.getUsedItemHand();
            if (ajh9 == InteractionHand.MAIN_HAND) {
                final ItemStack bek13 = dpy.getOffhandItem();
                if (bek13.getItem() == Items.CROSSBOW && CrossbowItem.isCharged(bek13)) {
                    boolean11 = false;
                }
            }
        }
        else {
            final ItemStack bek12 = dpy.getMainHandItem();
            final ItemStack bek14 = dpy.getOffhandItem();
            if (bek12.getItem() == Items.CROSSBOW && CrossbowItem.isCharged(bek12)) {
                boolean11 = !boolean10;
            }
            if (bek14.getItem() == Items.CROSSBOW && CrossbowItem.isCharged(bek14)) {
                boolean10 = !bek12.isEmpty();
                boolean11 = !boolean10;
            }
        }
        final float float4 = Mth.lerp(float1, dpy.xBobO, dpy.xBob);
        final float float5 = Mth.lerp(float1, dpy.yBobO, dpy.yBob);
        cyi.mulPose(Vector3f.XP.rotationDegrees((dpy.getViewXRot(float1) - float4) * 0.1f));
        cyi.mulPose(Vector3f.YP.rotationDegrees((dpy.getViewYRot(float1) - float5) * 0.1f));
        if (boolean10) {
            final float float6 = (ajh8 == InteractionHand.MAIN_HAND) ? float2 : 0.0f;
            final float float7 = 1.0f - Mth.lerp(float1, this.oMainHandHeight, this.mainHandHeight);
            this.renderArmWithItem(dpy, float1, float3, InteractionHand.MAIN_HAND, float6, this.mainHandItem, float7, cyi, a, integer);
        }
        if (boolean11) {
            final float float6 = (ajh8 == InteractionHand.OFF_HAND) ? float2 : 0.0f;
            final float float7 = 1.0f - Mth.lerp(float1, this.oOffHandHeight, this.offHandHeight);
            this.renderArmWithItem(dpy, float1, float3, InteractionHand.OFF_HAND, float6, this.offHandItem, float7, cyi, a, integer);
        }
        a.endBatch();
    }
    
    private void renderArmWithItem(final AbstractClientPlayer dpv, final float float2, final float float3, final InteractionHand ajh, final float float5, final ItemStack bek, final float float7, final PoseStack cyi, final MultiBufferSource dqq, final int integer) {
        final boolean boolean12 = ajh == InteractionHand.MAIN_HAND;
        final HumanoidArm akv13 = boolean12 ? dpv.getMainArm() : dpv.getMainArm().getOpposite();
        cyi.pushPose();
        if (bek.isEmpty()) {
            if (boolean12 && !dpv.isInvisible()) {
                this.renderPlayerArm(cyi, dqq, integer, float7, float5, akv13);
            }
        }
        else if (bek.getItem() == Items.FILLED_MAP) {
            if (boolean12 && this.offHandItem.isEmpty()) {
                this.renderTwoHandedMap(cyi, dqq, integer, float3, float7, float5);
            }
            else {
                this.renderOneHandedMap(cyi, dqq, integer, float7, akv13, float5, bek);
            }
        }
        else if (bek.getItem() == Items.CROSSBOW) {
            final boolean boolean13 = CrossbowItem.isCharged(bek);
            final boolean boolean14 = akv13 == HumanoidArm.RIGHT;
            final int integer2 = boolean14 ? 1 : -1;
            if (dpv.isUsingItem() && dpv.getUseItemRemainingTicks() > 0 && dpv.getUsedItemHand() == ajh) {
                this.applyItemArmTransform(cyi, akv13, float7);
                cyi.translate(integer2 * -0.4785682f, -0.0943870022892952, 0.05731530860066414);
                cyi.mulPose(Vector3f.XP.rotationDegrees(-11.935f));
                cyi.mulPose(Vector3f.YP.rotationDegrees(integer2 * 65.3f));
                cyi.mulPose(Vector3f.ZP.rotationDegrees(integer2 * -9.785f));
                final float float8 = bek.getUseDuration() - (this.minecraft.player.getUseItemRemainingTicks() - float2 + 1.0f);
                float float9 = float8 / CrossbowItem.getChargeDuration(bek);
                if (float9 > 1.0f) {
                    float9 = 1.0f;
                }
                if (float9 > 0.1f) {
                    final float float10 = Mth.sin((float8 - 0.1f) * 1.3f);
                    final float float11 = float9 - 0.1f;
                    final float float12 = float10 * float11;
                    cyi.translate(float12 * 0.0f, float12 * 0.004f, float12 * 0.0f);
                }
                cyi.translate(float9 * 0.0f, float9 * 0.0f, float9 * 0.04f);
                cyi.scale(1.0f, 1.0f, 1.0f + float9 * 0.2f);
                cyi.mulPose(Vector3f.YN.rotationDegrees(integer2 * 45.0f));
            }
            else {
                final float float8 = -0.4f * Mth.sin(Mth.sqrt(float5) * 3.1415927f);
                final float float9 = 0.2f * Mth.sin(Mth.sqrt(float5) * 6.2831855f);
                final float float10 = -0.2f * Mth.sin(float5 * 3.1415927f);
                cyi.translate(integer2 * float8, float9, float10);
                this.applyItemArmTransform(cyi, akv13, float7);
                this.applyItemArmAttackTransform(cyi, akv13, float5);
                if (boolean13 && float5 < 0.001f) {
                    cyi.translate(integer2 * -0.641864f, 0.0, 0.0);
                    cyi.mulPose(Vector3f.YP.rotationDegrees(integer2 * 10.0f));
                }
            }
            this.renderItem(dpv, bek, boolean14 ? ItemTransforms.TransformType.FIRST_PERSON_RIGHT_HAND : ItemTransforms.TransformType.FIRST_PERSON_LEFT_HAND, !boolean14, cyi, dqq, integer);
        }
        else {
            final boolean boolean13 = akv13 == HumanoidArm.RIGHT;
            if (dpv.isUsingItem() && dpv.getUseItemRemainingTicks() > 0 && dpv.getUsedItemHand() == ajh) {
                final int integer3 = boolean13 ? 1 : -1;
                switch (bek.getUseAnimation()) {
                    case NONE: {
                        this.applyItemArmTransform(cyi, akv13, float7);
                        break;
                    }
                    case EAT:
                    case DRINK: {
                        this.applyEatTransform(cyi, float2, akv13, bek);
                        this.applyItemArmTransform(cyi, akv13, float7);
                        break;
                    }
                    case BLOCK: {
                        this.applyItemArmTransform(cyi, akv13, float7);
                        break;
                    }
                    case BOW: {
                        this.applyItemArmTransform(cyi, akv13, float7);
                        cyi.translate(integer3 * -0.2785682f, 0.18344387412071228, 0.15731531381607056);
                        cyi.mulPose(Vector3f.XP.rotationDegrees(-13.935f));
                        cyi.mulPose(Vector3f.YP.rotationDegrees(integer3 * 35.3f));
                        cyi.mulPose(Vector3f.ZP.rotationDegrees(integer3 * -9.785f));
                        final float float13 = bek.getUseDuration() - (this.minecraft.player.getUseItemRemainingTicks() - float2 + 1.0f);
                        float float8 = float13 / 20.0f;
                        float8 = (float8 * float8 + float8 * 2.0f) / 3.0f;
                        if (float8 > 1.0f) {
                            float8 = 1.0f;
                        }
                        if (float8 > 0.1f) {
                            final float float9 = Mth.sin((float13 - 0.1f) * 1.3f);
                            final float float10 = float8 - 0.1f;
                            final float float11 = float9 * float10;
                            cyi.translate(float11 * 0.0f, float11 * 0.004f, float11 * 0.0f);
                        }
                        cyi.translate(float8 * 0.0f, float8 * 0.0f, float8 * 0.04f);
                        cyi.scale(1.0f, 1.0f, 1.0f + float8 * 0.2f);
                        cyi.mulPose(Vector3f.YN.rotationDegrees(integer3 * 45.0f));
                        break;
                    }
                    case SPEAR: {
                        this.applyItemArmTransform(cyi, akv13, float7);
                        cyi.translate(integer3 * -0.5f, 0.699999988079071, 0.10000000149011612);
                        cyi.mulPose(Vector3f.XP.rotationDegrees(-55.0f));
                        cyi.mulPose(Vector3f.YP.rotationDegrees(integer3 * 35.3f));
                        cyi.mulPose(Vector3f.ZP.rotationDegrees(integer3 * -9.785f));
                        final float float13 = bek.getUseDuration() - (this.minecraft.player.getUseItemRemainingTicks() - float2 + 1.0f);
                        float float8 = float13 / 10.0f;
                        if (float8 > 1.0f) {
                            float8 = 1.0f;
                        }
                        if (float8 > 0.1f) {
                            final float float9 = Mth.sin((float13 - 0.1f) * 1.3f);
                            final float float10 = float8 - 0.1f;
                            final float float11 = float9 * float10;
                            cyi.translate(float11 * 0.0f, float11 * 0.004f, float11 * 0.0f);
                        }
                        cyi.translate(0.0, 0.0, float8 * 0.2f);
                        cyi.scale(1.0f, 1.0f, 1.0f + float8 * 0.2f);
                        cyi.mulPose(Vector3f.YN.rotationDegrees(integer3 * 45.0f));
                        break;
                    }
                }
            }
            else if (dpv.isAutoSpinAttack()) {
                this.applyItemArmTransform(cyi, akv13, float7);
                final int integer3 = boolean13 ? 1 : -1;
                cyi.translate(integer3 * -0.4f, 0.800000011920929, 0.30000001192092896);
                cyi.mulPose(Vector3f.YP.rotationDegrees(integer3 * 65.0f));
                cyi.mulPose(Vector3f.ZP.rotationDegrees(integer3 * -85.0f));
            }
            else {
                final float float14 = -0.4f * Mth.sin(Mth.sqrt(float5) * 3.1415927f);
                final float float13 = 0.2f * Mth.sin(Mth.sqrt(float5) * 6.2831855f);
                final float float8 = -0.2f * Mth.sin(float5 * 3.1415927f);
                final int integer4 = boolean13 ? 1 : -1;
                cyi.translate(integer4 * float14, float13, float8);
                this.applyItemArmTransform(cyi, akv13, float7);
                this.applyItemArmAttackTransform(cyi, akv13, float5);
            }
            this.renderItem(dpv, bek, boolean13 ? ItemTransforms.TransformType.FIRST_PERSON_RIGHT_HAND : ItemTransforms.TransformType.FIRST_PERSON_LEFT_HAND, !boolean13, cyi, dqq, integer);
        }
        cyi.popPose();
    }
    
    public void tick() {
        this.oMainHandHeight = this.mainHandHeight;
        this.oOffHandHeight = this.offHandHeight;
        final LocalPlayer dpy2 = this.minecraft.player;
        final ItemStack bek3 = dpy2.getMainHandItem();
        final ItemStack bek4 = dpy2.getOffhandItem();
        if (dpy2.isHandsBusy()) {
            this.mainHandHeight = Mth.clamp(this.mainHandHeight - 0.4f, 0.0f, 1.0f);
            this.offHandHeight = Mth.clamp(this.offHandHeight - 0.4f, 0.0f, 1.0f);
        }
        else {
            final float float5 = dpy2.getAttackStrengthScale(1.0f);
            this.mainHandHeight += Mth.clamp((Objects.equals(this.mainHandItem, bek3) ? (float5 * float5 * float5) : 0.0f) - this.mainHandHeight, -0.4f, 0.4f);
            this.offHandHeight += Mth.clamp((float)(Objects.equals(this.offHandItem, bek4) ? 1 : 0) - this.offHandHeight, -0.4f, 0.4f);
        }
        if (this.mainHandHeight < 0.1f) {
            this.mainHandItem = bek3;
        }
        if (this.offHandHeight < 0.1f) {
            this.offHandItem = bek4;
        }
    }
    
    public void itemUsed(final InteractionHand ajh) {
        if (ajh == InteractionHand.MAIN_HAND) {
            this.mainHandHeight = 0.0f;
        }
        else {
            this.offHandHeight = 0.0f;
        }
    }
    
    static {
        MAP_BACKGROUND = RenderType.text(new ResourceLocation("textures/map/map_background.png"));
        MAP_BACKGROUND_CHECKERBOARD = RenderType.text(new ResourceLocation("textures/map/map_background_checkerboard.png"));
    }
}
