package net.minecraft.client.renderer;

import it.unimi.dsi.fastutil.objects.ObjectIterator;
import java.util.Map;
import javax.annotation.Nullable;
import net.minecraft.world.item.Item;
import net.minecraft.world.item.BlockItem;
import net.minecraft.world.item.ItemStack;
import net.minecraft.client.renderer.texture.TextureAtlasSprite;
import net.minecraft.world.level.ItemLike;
import it.unimi.dsi.fastutil.ints.Int2ObjectOpenHashMap;
import net.minecraft.client.resources.model.ModelManager;
import net.minecraft.client.resources.model.BakedModel;
import net.minecraft.client.resources.model.ModelResourceLocation;
import it.unimi.dsi.fastutil.ints.Int2ObjectMap;

public class ItemModelShaper {
    public final Int2ObjectMap<ModelResourceLocation> shapes;
    private final Int2ObjectMap<BakedModel> shapesCache;
    private final ModelManager modelManager;
    
    public ItemModelShaper(final ModelManager ebx) {
        this.shapes = (Int2ObjectMap<ModelResourceLocation>)new Int2ObjectOpenHashMap(256);
        this.shapesCache = (Int2ObjectMap<BakedModel>)new Int2ObjectOpenHashMap(256);
        this.modelManager = ebx;
    }
    
    public TextureAtlasSprite getParticleIcon(final ItemLike bjs) {
        return this.getParticleIcon(new ItemStack(bjs));
    }
    
    public TextureAtlasSprite getParticleIcon(final ItemStack bek) {
        final BakedModel ebs3 = this.getItemModel(bek);
        if (ebs3 == this.modelManager.getMissingModel() && bek.getItem() instanceof BlockItem) {
            return this.modelManager.getBlockModelShaper().getParticleIcon(((BlockItem)bek.getItem()).getBlock().defaultBlockState());
        }
        return ebs3.getParticleIcon();
    }
    
    public BakedModel getItemModel(final ItemStack bek) {
        final BakedModel ebs3 = this.getItemModel(bek.getItem());
        return (ebs3 == null) ? this.modelManager.getMissingModel() : ebs3;
    }
    
    @Nullable
    public BakedModel getItemModel(final Item bef) {
        return (BakedModel)this.shapesCache.get(getIndex(bef));
    }
    
    private static int getIndex(final Item bef) {
        return Item.getId(bef);
    }
    
    public void register(final Item bef, final ModelResourceLocation eby) {
        this.shapes.put(getIndex(bef), eby);
    }
    
    public ModelManager getModelManager() {
        return this.modelManager;
    }
    
    public void rebuildCache() {
        this.shapesCache.clear();
        for (final Map.Entry<Integer, ModelResourceLocation> entry3 : this.shapes.entrySet()) {
            this.shapesCache.put(Integer.valueOf(entry3.getKey()), this.modelManager.getModel(entry3.getValue()));
        }
    }
}
