package net.minecraft.client.renderer;

import java.util.Iterator;
import java.util.Objects;
import com.google.common.collect.Sets;
import java.util.Set;
import java.util.Optional;
import com.mojang.blaze3d.vertex.VertexConsumer;
import java.util.Map;
import com.google.common.collect.ImmutableMap;
import com.mojang.blaze3d.vertex.BufferBuilder;

public interface MultiBufferSource {
    default BufferSource immediate(final BufferBuilder cyd) {
        return immediateWithBuffers(ImmutableMap.of(), cyd);
    }
    
    default BufferSource immediateWithBuffers(final Map<RenderType, BufferBuilder> map, final BufferBuilder cyd) {
        return new BufferSource(cyd, map);
    }
    
    VertexConsumer getBuffer(final RenderType dqy);
    
    public static class BufferSource implements MultiBufferSource {
        protected final BufferBuilder builder;
        protected final Map<RenderType, BufferBuilder> fixedBuffers;
        protected Optional<RenderType> lastState;
        protected final Set<BufferBuilder> startedBuffers;
        
        protected BufferSource(final BufferBuilder cyd, final Map<RenderType, BufferBuilder> map) {
            this.lastState = Optional.<RenderType>empty();
            this.startedBuffers = Sets.newHashSet();
            this.builder = cyd;
            this.fixedBuffers = map;
        }
        
        @Override
        public VertexConsumer getBuffer(final RenderType dqy) {
            final Optional<RenderType> optional3 = Optional.<RenderType>of(dqy);
            final BufferBuilder cyd4 = this.getBuilderRaw(dqy);
            if (!Objects.equals(this.lastState, optional3)) {
                if (this.lastState.isPresent()) {
                    final RenderType dqy2 = this.lastState.get();
                    if (!this.fixedBuffers.containsKey(dqy2)) {
                        this.endBatch(dqy2);
                    }
                }
                if (this.startedBuffers.add(cyd4)) {
                    cyd4.begin(dqy.mode(), dqy.format());
                }
                this.lastState = optional3;
            }
            return cyd4;
        }
        
        private BufferBuilder getBuilderRaw(final RenderType dqy) {
            return this.fixedBuffers.getOrDefault(dqy, this.builder);
        }
        
        public void endBatch() {
            final VertexConsumer cyl3;
            this.lastState.ifPresent(dqy -> {
                cyl3 = this.getBuffer(dqy);
                if (cyl3 == this.builder) {
                    this.endBatch(dqy);
                }
                return;
            });
            for (final RenderType dqy2 : this.fixedBuffers.keySet()) {
                this.endBatch(dqy2);
            }
        }
        
        public void endBatch(final RenderType dqy) {
            final BufferBuilder cyd3 = this.getBuilderRaw(dqy);
            final boolean boolean4 = Objects.equals(this.lastState, Optional.<RenderType>of(dqy));
            if (!boolean4 && cyd3 == this.builder) {
                return;
            }
            if (!this.startedBuffers.remove(cyd3)) {
                return;
            }
            dqy.end(cyd3, 0, 0, 0);
            if (boolean4) {
                this.lastState = Optional.<RenderType>empty();
            }
        }
    }
}
