package net.minecraft.client.renderer;

import com.mojang.math.Vector3f;
import net.minecraft.util.Mth;
import net.minecraft.client.resources.model.ModelBakery;
import com.mojang.math.Matrix4f;
import com.mojang.blaze3d.vertex.BufferBuilder;
import com.mojang.blaze3d.vertex.BufferUploader;
import com.mojang.blaze3d.vertex.DefaultVertexFormat;
import com.mojang.blaze3d.vertex.Tesselator;
import net.minecraft.client.renderer.texture.TextureAtlasSprite;
import javax.annotation.Nullable;
import net.minecraft.world.level.BlockGetter;
import net.minecraft.world.level.block.RenderShape;
import net.minecraft.core.BlockPos;
import net.minecraft.world.level.block.state.BlockState;
import net.minecraft.world.entity.player.Player;
import net.minecraft.tags.FluidTags;
import com.mojang.blaze3d.systems.RenderSystem;
import com.mojang.blaze3d.vertex.PoseStack;
import net.minecraft.client.Minecraft;
import net.minecraft.resources.ResourceLocation;

public class ScreenEffectRenderer {
    private static final ResourceLocation UNDERWATER_LOCATION;
    
    public static void renderScreenEffect(final Minecraft dbl, final PoseStack cyi) {
        RenderSystem.disableAlphaTest();
        final Player ayg3 = dbl.player;
        if (!ayg3.noPhysics) {
            final BlockState byg4 = getViewBlockingState(ayg3);
            if (byg4 != null) {
                renderTex(dbl, dbl.getBlockRenderer().getBlockModelShaper().getParticleIcon(byg4), cyi);
            }
        }
        if (!dbl.player.isSpectator()) {
            if (dbl.player.isUnderLiquid(FluidTags.WATER)) {
                renderWater(dbl, cyi);
            }
            if (dbl.player.isOnFire()) {
                renderFire(dbl, cyi);
            }
        }
        RenderSystem.enableAlphaTest();
    }
    
    @Nullable
    private static BlockState getViewBlockingState(final Player ayg) {
        final BlockPos.MutableBlockPos a2 = new BlockPos.MutableBlockPos();
        for (int integer3 = 0; integer3 < 8; ++integer3) {
            final double double4 = ayg.getX() + ((integer3 >> 0) % 2 - 0.5f) * ayg.getBbWidth() * 0.8f;
            final double double5 = ayg.getEyeY() + ((integer3 >> 1) % 2 - 0.5f) * 0.1f;
            final double double6 = ayg.getZ() + ((integer3 >> 2) % 2 - 0.5f) * ayg.getBbWidth() * 0.8f;
            a2.set(double4, double5, double6);
            final BlockState byg10 = ayg.level.getBlockState(a2);
            if (byg10.getRenderShape() != RenderShape.INVISIBLE && byg10.isViewBlocking(ayg.level, a2)) {
                return byg10;
            }
        }
        return null;
    }
    
    private static void renderTex(final Minecraft dbl, final TextureAtlasSprite eag, final PoseStack cyi) {
        dbl.getTextureManager().bind(eag.atlas().location());
        final BufferBuilder cyd4 = Tesselator.getInstance().getBuilder();
        final float float5 = 0.1f;
        final float float6 = -1.0f;
        final float float7 = 1.0f;
        final float float8 = -1.0f;
        final float float9 = 1.0f;
        final float float10 = -0.5f;
        final float float11 = eag.getU0();
        final float float12 = eag.getU1();
        final float float13 = eag.getV0();
        final float float14 = eag.getV1();
        final Matrix4f b15 = cyi.last().pose();
        cyd4.begin(7, DefaultVertexFormat.POSITION_COLOR_TEX);
        cyd4.vertex(b15, -1.0f, -1.0f, -0.5f).color(0.1f, 0.1f, 0.1f, 1.0f).uv(float12, float14).endVertex();
        cyd4.vertex(b15, 1.0f, -1.0f, -0.5f).color(0.1f, 0.1f, 0.1f, 1.0f).uv(float11, float14).endVertex();
        cyd4.vertex(b15, 1.0f, 1.0f, -0.5f).color(0.1f, 0.1f, 0.1f, 1.0f).uv(float11, float13).endVertex();
        cyd4.vertex(b15, -1.0f, 1.0f, -0.5f).color(0.1f, 0.1f, 0.1f, 1.0f).uv(float12, float13).endVertex();
        cyd4.end();
        BufferUploader.end(cyd4);
    }
    
    private static void renderWater(final Minecraft dbl, final PoseStack cyi) {
        dbl.getTextureManager().bind(ScreenEffectRenderer.UNDERWATER_LOCATION);
        final BufferBuilder cyd3 = Tesselator.getInstance().getBuilder();
        final float float4 = dbl.player.getBrightness();
        RenderSystem.enableBlend();
        RenderSystem.defaultBlendFunc();
        final float float5 = 4.0f;
        final float float6 = -1.0f;
        final float float7 = 1.0f;
        final float float8 = -1.0f;
        final float float9 = 1.0f;
        final float float10 = -0.5f;
        final float float11 = -dbl.player.yRot / 64.0f;
        final float float12 = dbl.player.xRot / 64.0f;
        final Matrix4f b13 = cyi.last().pose();
        cyd3.begin(7, DefaultVertexFormat.POSITION_COLOR_TEX);
        cyd3.vertex(b13, -1.0f, -1.0f, -0.5f).color(float4, float4, float4, 0.1f).uv(4.0f + float11, 4.0f + float12).endVertex();
        cyd3.vertex(b13, 1.0f, -1.0f, -0.5f).color(float4, float4, float4, 0.1f).uv(0.0f + float11, 4.0f + float12).endVertex();
        cyd3.vertex(b13, 1.0f, 1.0f, -0.5f).color(float4, float4, float4, 0.1f).uv(0.0f + float11, 0.0f + float12).endVertex();
        cyd3.vertex(b13, -1.0f, 1.0f, -0.5f).color(float4, float4, float4, 0.1f).uv(4.0f + float11, 0.0f + float12).endVertex();
        cyd3.end();
        BufferUploader.end(cyd3);
        RenderSystem.disableBlend();
    }
    
    private static void renderFire(final Minecraft dbl, final PoseStack cyi) {
        final BufferBuilder cyd3 = Tesselator.getInstance().getBuilder();
        RenderSystem.depthFunc(519);
        RenderSystem.depthMask(false);
        RenderSystem.enableBlend();
        RenderSystem.defaultBlendFunc();
        final TextureAtlasSprite eag4 = ModelBakery.FIRE_1.sprite();
        dbl.getTextureManager().bind(eag4.atlas().location());
        final float float5 = eag4.getU0();
        final float float6 = eag4.getU1();
        final float float7 = (float5 + float6) / 2.0f;
        final float float8 = eag4.getV0();
        final float float9 = eag4.getV1();
        final float float10 = (float8 + float9) / 2.0f;
        final float float11 = eag4.uvShrinkRatio();
        final float float12 = Mth.lerp(float11, float5, float7);
        final float float13 = Mth.lerp(float11, float6, float7);
        final float float14 = Mth.lerp(float11, float8, float10);
        final float float15 = Mth.lerp(float11, float9, float10);
        final float float16 = 1.0f;
        for (int integer17 = 0; integer17 < 2; ++integer17) {
            cyi.pushPose();
            final float float17 = -0.5f;
            final float float18 = 0.5f;
            final float float19 = -0.5f;
            final float float20 = 0.5f;
            final float float21 = -0.5f;
            cyi.translate(-(integer17 * 2 - 1) * 0.24f, -0.30000001192092896, 0.0);
            cyi.mulPose(Vector3f.YP.rotationDegrees((integer17 * 2 - 1) * 10.0f));
            final Matrix4f b23 = cyi.last().pose();
            cyd3.begin(7, DefaultVertexFormat.POSITION_COLOR_TEX);
            cyd3.vertex(b23, -0.5f, -0.5f, -0.5f).color(1.0f, 1.0f, 1.0f, 0.9f).uv(float13, float15).endVertex();
            cyd3.vertex(b23, 0.5f, -0.5f, -0.5f).color(1.0f, 1.0f, 1.0f, 0.9f).uv(float12, float15).endVertex();
            cyd3.vertex(b23, 0.5f, 0.5f, -0.5f).color(1.0f, 1.0f, 1.0f, 0.9f).uv(float12, float14).endVertex();
            cyd3.vertex(b23, -0.5f, 0.5f, -0.5f).color(1.0f, 1.0f, 1.0f, 0.9f).uv(float13, float14).endVertex();
            cyd3.end();
            BufferUploader.end(cyd3);
            cyi.popPose();
        }
        RenderSystem.disableBlend();
        RenderSystem.depthMask(true);
        RenderSystem.depthFunc(515);
    }
    
    static {
        UNDERWATER_LOCATION = new ResourceLocation("textures/misc/underwater.png");
    }
}
