package net.minecraft.client.renderer.block;

import net.minecraft.world.level.block.state.properties.Property;
import net.minecraft.resources.ResourceLocation;
import net.minecraft.client.resources.model.ModelResourceLocation;
import java.util.Iterator;
import net.minecraft.world.level.block.Block;
import net.minecraft.core.Registry;
import net.minecraft.client.renderer.texture.TextureAtlasSprite;
import com.google.common.collect.Maps;
import net.minecraft.client.resources.model.ModelManager;
import net.minecraft.client.resources.model.BakedModel;
import net.minecraft.world.level.block.state.BlockState;
import java.util.Map;

public class BlockModelShaper {
    private final Map<BlockState, BakedModel> modelByStateCache;
    private final ModelManager modelManager;
    
    public BlockModelShaper(final ModelManager ebx) {
        this.modelByStateCache = Maps.newIdentityHashMap();
        this.modelManager = ebx;
    }
    
    public TextureAtlasSprite getParticleIcon(final BlockState byg) {
        return this.getBlockModel(byg).getParticleIcon();
    }
    
    public BakedModel getBlockModel(final BlockState byg) {
        BakedModel ebs3 = this.modelByStateCache.get(byg);
        if (ebs3 == null) {
            ebs3 = this.modelManager.getMissingModel();
        }
        return ebs3;
    }
    
    public ModelManager getModelManager() {
        return this.modelManager;
    }
    
    public void rebuildCache() {
        this.modelByStateCache.clear();
        for (final Block bpe3 : Registry.BLOCK) {
            final BakedModel bakedModel;
            bpe3.getStateDefinition().getPossibleStates().forEach(byg -> bakedModel = this.modelByStateCache.put(byg, this.modelManager.getModel(stateToModelLocation(byg))));
        }
    }
    
    public static ModelResourceLocation stateToModelLocation(final BlockState byg) {
        return stateToModelLocation(Registry.BLOCK.getKey(byg.getBlock()), byg);
    }
    
    public static ModelResourceLocation stateToModelLocation(final ResourceLocation sm, final BlockState byg) {
        return new ModelResourceLocation(sm, statePropertiesToString(byg.getValues()));
    }
    
    public static String statePropertiesToString(final Map<Property<?>, Comparable<?>> map) {
        final StringBuilder stringBuilder2 = new StringBuilder();
        for (final Map.Entry<Property<?>, Comparable<?>> entry4 : map.entrySet()) {
            if (stringBuilder2.length() != 0) {
                stringBuilder2.append(',');
            }
            final Property<?> bzj5 = entry4.getKey();
            stringBuilder2.append(bzj5.getName());
            stringBuilder2.append('=');
            stringBuilder2.append(BlockModelShaper.getValue(bzj5, entry4.getValue()));
        }
        return stringBuilder2.toString();
    }
    
    private static <T extends Comparable<T>> String getValue(final Property<T> bzj, final Comparable<?> comparable) {
        return bzj.getName((T)comparable);
    }
}
