package net.minecraft.client.renderer.block;

import net.minecraft.server.packs.resources.ResourceManager;
import net.minecraft.world.level.ItemLike;
import net.minecraft.world.item.ItemStack;
import net.minecraft.client.renderer.BlockEntityWithoutLevelRenderer;
import net.minecraft.client.renderer.ItemBlockRenderTypes;
import net.minecraft.client.renderer.MultiBufferSource;
import net.minecraft.world.level.material.FluidState;
import net.minecraft.ReportedException;
import net.minecraft.CrashReportCategory;
import net.minecraft.CrashReport;
import net.minecraft.client.resources.model.BakedModel;
import net.minecraft.client.renderer.texture.OverlayTexture;
import net.minecraft.world.level.block.RenderShape;
import com.mojang.blaze3d.vertex.VertexConsumer;
import com.mojang.blaze3d.vertex.PoseStack;
import net.minecraft.world.level.BlockAndTintGetter;
import net.minecraft.core.BlockPos;
import net.minecraft.world.level.block.state.BlockState;
import net.minecraft.client.color.block.BlockColors;
import java.util.Random;
import net.minecraft.server.packs.resources.ResourceManagerReloadListener;

public class BlockRenderDispatcher implements ResourceManagerReloadListener {
    private final BlockModelShaper blockModelShaper;
    private final ModelBlockRenderer modelRenderer;
    private final LiquidBlockRenderer liquidBlockRenderer;
    private final Random random;
    private final BlockColors blockColors;
    
    public BlockRenderDispatcher(final BlockModelShaper drg, final BlockColors dbz) {
        this.random = new Random();
        this.blockModelShaper = drg;
        this.blockColors = dbz;
        this.modelRenderer = new ModelBlockRenderer(this.blockColors);
        this.liquidBlockRenderer = new LiquidBlockRenderer();
    }
    
    public BlockModelShaper getBlockModelShaper() {
        return this.blockModelShaper;
    }
    
    public void renderBreakingTexture(final BlockState byg, final BlockPos fk, final BlockAndTintGetter bjb, final PoseStack cyi, final VertexConsumer cyl) {
        if (byg.getRenderShape() != RenderShape.MODEL) {
            return;
        }
        final BakedModel ebs7 = this.blockModelShaper.getBlockModel(byg);
        final long long8 = byg.getSeed(fk);
        this.modelRenderer.tesselateBlock(bjb, ebs7, byg, fk, cyi, cyl, true, this.random, long8, OverlayTexture.NO_OVERLAY);
    }
    
    public boolean renderBatched(final BlockState byg, final BlockPos fk, final BlockAndTintGetter bjb, final PoseStack cyi, final VertexConsumer cyl, final boolean boolean6, final Random random) {
        try {
            final RenderShape bto9 = byg.getRenderShape();
            return bto9 == RenderShape.MODEL && this.modelRenderer.tesselateBlock(bjb, this.getBlockModel(byg), byg, fk, cyi, cyl, boolean6, random, byg.getSeed(fk), OverlayTexture.NO_OVERLAY);
        }
        catch (Throwable throwable9) {
            final CrashReport h10 = CrashReport.forThrowable(throwable9, "Tesselating block in world");
            final CrashReportCategory i11 = h10.addCategory("Block being tesselated");
            CrashReportCategory.populateBlockDetails(i11, fk, byg);
            throw new ReportedException(h10);
        }
    }
    
    public boolean renderLiquid(final BlockPos fk, final BlockAndTintGetter bjb, final VertexConsumer cyl, final FluidState cog) {
        try {
            return this.liquidBlockRenderer.tesselate(bjb, fk, cyl, cog);
        }
        catch (Throwable throwable6) {
            final CrashReport h7 = CrashReport.forThrowable(throwable6, "Tesselating liquid in world");
            final CrashReportCategory i8 = h7.addCategory("Block being tesselated");
            CrashReportCategory.populateBlockDetails(i8, fk, null);
            throw new ReportedException(h7);
        }
    }
    
    public ModelBlockRenderer getModelRenderer() {
        return this.modelRenderer;
    }
    
    public BakedModel getBlockModel(final BlockState byg) {
        return this.blockModelShaper.getBlockModel(byg);
    }
    
    public void renderSingleBlock(final BlockState byg, final PoseStack cyi, final MultiBufferSource dqq, final int integer4, final int integer5) {
        final RenderShape bto7 = byg.getRenderShape();
        if (bto7 == RenderShape.INVISIBLE) {
            return;
        }
        switch (bto7) {
            case MODEL: {
                final BakedModel ebs8 = this.getBlockModel(byg);
                final int integer6 = this.blockColors.getColor(byg, null, null, 0);
                final float float10 = (integer6 >> 16 & 0xFF) / 255.0f;
                final float float11 = (integer6 >> 8 & 0xFF) / 255.0f;
                final float float12 = (integer6 & 0xFF) / 255.0f;
                this.modelRenderer.renderModel(cyi.last(), dqq.getBuffer(ItemBlockRenderTypes.getRenderType(byg)), byg, ebs8, float10, float11, float12, integer4, integer5);
                break;
            }
            case ENTITYBLOCK_ANIMATED: {
                BlockEntityWithoutLevelRenderer.instance.renderByItem(new ItemStack(byg.getBlock()), cyi, dqq, integer4, integer5);
                break;
            }
        }
    }
    
    @Override
    public void onResourceManagerReload(final ResourceManager zb) {
        this.liquidBlockRenderer.setupSprites();
    }
}
