package net.minecraft.client.renderer.block.model;

import com.google.gson.JsonParseException;
import com.google.gson.JsonObject;
import com.google.gson.JsonDeserializationContext;
import java.lang.reflect.Type;
import com.google.gson.JsonElement;
import com.google.gson.JsonDeserializer;

public class ItemTransforms {
    public static final ItemTransforms NO_TRANSFORMS;
    public final ItemTransform thirdPersonLeftHand;
    public final ItemTransform thirdPersonRightHand;
    public final ItemTransform firstPersonLeftHand;
    public final ItemTransform firstPersonRightHand;
    public final ItemTransform head;
    public final ItemTransform gui;
    public final ItemTransform ground;
    public final ItemTransform fixed;
    
    private ItemTransforms() {
        this(ItemTransform.NO_TRANSFORM, ItemTransform.NO_TRANSFORM, ItemTransform.NO_TRANSFORM, ItemTransform.NO_TRANSFORM, ItemTransform.NO_TRANSFORM, ItemTransform.NO_TRANSFORM, ItemTransform.NO_TRANSFORM, ItemTransform.NO_TRANSFORM);
    }
    
    public ItemTransforms(final ItemTransforms drw) {
        this.thirdPersonLeftHand = drw.thirdPersonLeftHand;
        this.thirdPersonRightHand = drw.thirdPersonRightHand;
        this.firstPersonLeftHand = drw.firstPersonLeftHand;
        this.firstPersonRightHand = drw.firstPersonRightHand;
        this.head = drw.head;
        this.gui = drw.gui;
        this.ground = drw.ground;
        this.fixed = drw.fixed;
    }
    
    public ItemTransforms(final ItemTransform drv1, final ItemTransform drv2, final ItemTransform drv3, final ItemTransform drv4, final ItemTransform drv5, final ItemTransform drv6, final ItemTransform drv7, final ItemTransform drv8) {
        this.thirdPersonLeftHand = drv1;
        this.thirdPersonRightHand = drv2;
        this.firstPersonLeftHand = drv3;
        this.firstPersonRightHand = drv4;
        this.head = drv5;
        this.gui = drv6;
        this.ground = drv7;
        this.fixed = drv8;
    }
    
    public ItemTransform getTransform(final TransformType b) {
        switch (b) {
            case THIRD_PERSON_LEFT_HAND: {
                return this.thirdPersonLeftHand;
            }
            case THIRD_PERSON_RIGHT_HAND: {
                return this.thirdPersonRightHand;
            }
            case FIRST_PERSON_LEFT_HAND: {
                return this.firstPersonLeftHand;
            }
            case FIRST_PERSON_RIGHT_HAND: {
                return this.firstPersonRightHand;
            }
            case HEAD: {
                return this.head;
            }
            case GUI: {
                return this.gui;
            }
            case GROUND: {
                return this.ground;
            }
            case FIXED: {
                return this.fixed;
            }
            default: {
                return ItemTransform.NO_TRANSFORM;
            }
        }
    }
    
    public boolean hasTransform(final TransformType b) {
        return this.getTransform(b) != ItemTransform.NO_TRANSFORM;
    }
    
    static {
        NO_TRANSFORMS = new ItemTransforms();
    }
    
    public enum TransformType {
        NONE, 
        THIRD_PERSON_LEFT_HAND, 
        THIRD_PERSON_RIGHT_HAND, 
        FIRST_PERSON_LEFT_HAND, 
        FIRST_PERSON_RIGHT_HAND, 
        HEAD, 
        GUI, 
        GROUND, 
        FIXED;
    }
    
    public static class Deserializer implements JsonDeserializer<ItemTransforms> {
        protected Deserializer() {
        }
        
        @Override
        public ItemTransforms deserialize(final JsonElement jsonElement, final Type type, final JsonDeserializationContext jsonDeserializationContext) throws JsonParseException {
            final JsonObject jsonObject5 = jsonElement.getAsJsonObject();
            final ItemTransform drv6 = this.getTransform(jsonDeserializationContext, jsonObject5, "thirdperson_righthand");
            ItemTransform drv7 = this.getTransform(jsonDeserializationContext, jsonObject5, "thirdperson_lefthand");
            if (drv7 == ItemTransform.NO_TRANSFORM) {
                drv7 = drv6;
            }
            final ItemTransform drv8 = this.getTransform(jsonDeserializationContext, jsonObject5, "firstperson_righthand");
            ItemTransform drv9 = this.getTransform(jsonDeserializationContext, jsonObject5, "firstperson_lefthand");
            if (drv9 == ItemTransform.NO_TRANSFORM) {
                drv9 = drv8;
            }
            final ItemTransform drv10 = this.getTransform(jsonDeserializationContext, jsonObject5, "head");
            final ItemTransform drv11 = this.getTransform(jsonDeserializationContext, jsonObject5, "gui");
            final ItemTransform drv12 = this.getTransform(jsonDeserializationContext, jsonObject5, "ground");
            final ItemTransform drv13 = this.getTransform(jsonDeserializationContext, jsonObject5, "fixed");
            return new ItemTransforms(drv7, drv6, drv9, drv8, drv10, drv11, drv12, drv13);
        }
        
        private ItemTransform getTransform(final JsonDeserializationContext jsonDeserializationContext, final JsonObject jsonObject, final String string) {
            if (jsonObject.has(string)) {
                return jsonDeserializationContext.<ItemTransform>deserialize(jsonObject.get(string), ItemTransform.class);
            }
            return ItemTransform.NO_TRANSFORM;
        }
    }
}
