package net.minecraft.client.renderer.blockentity;

import net.minecraft.world.level.block.entity.BlockEntity;
import java.util.List;
import net.minecraft.client.renderer.Sheets;
import net.minecraft.world.item.DyeColor;
import net.minecraft.world.level.block.entity.BannerPattern;
import net.minecraft.client.resources.model.Material;
import net.minecraft.core.BlockPos;
import com.mojang.blaze3d.vertex.VertexConsumer;
import net.minecraft.world.level.block.state.BlockState;
import net.minecraft.util.Mth;
import net.minecraft.client.renderer.RenderType;
import net.minecraft.client.resources.model.ModelBakery;
import net.minecraft.world.level.block.WallBannerBlock;
import net.minecraft.core.Direction;
import com.mojang.math.Vector3f;
import net.minecraft.world.level.block.state.properties.Property;
import net.minecraft.world.level.block.BannerBlock;
import net.minecraft.client.renderer.MultiBufferSource;
import com.mojang.blaze3d.vertex.PoseStack;
import net.minecraft.client.model.geom.ModelPart;
import net.minecraft.world.level.block.entity.BannerBlockEntity;

public class BannerRenderer extends BlockEntityRenderer<BannerBlockEntity> {
    private final ModelPart flag;
    private final ModelPart pole;
    private final ModelPart bar;
    
    public BannerRenderer(final BlockEntityRenderDispatcher dsn) {
        super(dsn);
        this.flag = makeFlag();
        (this.pole = new ModelPart(64, 64, 44, 0)).addBox(-1.0f, -30.0f, -1.0f, 2.0f, 42.0f, 2.0f, 0.0f);
        (this.bar = new ModelPart(64, 64, 0, 42)).addBox(-10.0f, -32.0f, -1.0f, 20.0f, 2.0f, 2.0f, 0.0f);
    }
    
    public static ModelPart makeFlag() {
        final ModelPart dnf1 = new ModelPart(64, 64, 0, 0);
        dnf1.addBox(-10.0f, 0.0f, -2.0f, 20.0f, 40.0f, 1.0f, 0.0f);
        return dnf1;
    }
    
    @Override
    public void render(final BannerBlockEntity bvz, final float float2, final PoseStack cyi, final MultiBufferSource dqq, final int integer5, final int integer6) {
        if (bvz.getPatterns() == null) {
            return;
        }
        final float float3 = 0.6666667f;
        final boolean boolean9 = bvz.getLevel() == null;
        cyi.pushPose();
        long long10;
        if (boolean9) {
            long10 = 0L;
            cyi.translate(0.5, 0.5, 0.5);
            this.pole.visible = !bvz.onlyRenderPattern();
        }
        else {
            long10 = bvz.getLevel().getGameTime();
            final BlockState byg12 = bvz.getBlockState();
            if (byg12.getBlock() instanceof BannerBlock) {
                cyi.translate(0.5, 0.5, 0.5);
                final float float4 = -byg12.<Integer>getValue((Property<Integer>)BannerBlock.ROTATION) * 360 / 16.0f;
                cyi.mulPose(Vector3f.YP.rotationDegrees(float4));
                this.pole.visible = true;
            }
            else {
                cyi.translate(0.5, -0.1666666716337204, 0.5);
                final float float4 = -byg12.<Direction>getValue((Property<Direction>)WallBannerBlock.FACING).toYRot();
                cyi.mulPose(Vector3f.YP.rotationDegrees(float4));
                cyi.translate(0.0, -0.3125, -0.4375);
                this.pole.visible = false;
            }
        }
        cyi.pushPose();
        cyi.scale(0.6666667f, -0.6666667f, -0.6666667f);
        final VertexConsumer cyl12 = ModelBakery.BANNER_BASE.buffer(dqq, RenderType::entitySolid);
        this.pole.render(cyi, cyl12, integer5, integer6);
        this.bar.render(cyi, cyl12, integer5, integer6);
        if (bvz.onlyRenderPattern()) {
            this.flag.xRot = 0.0f;
        }
        else {
            final BlockPos fk13 = bvz.getBlockPos();
            final float float5 = (Math.floorMod(fk13.getX() * 7 + fk13.getY() * 9 + fk13.getZ() * 13 + long10, 100L) + float2) / 100.0f;
            this.flag.xRot = (-0.0125f + 0.01f * Mth.cos(6.2831855f * float5)) * 3.1415927f;
        }
        this.flag.y = -32.0f;
        renderPatterns(bvz, cyi, dqq, integer5, integer6, this.flag, ModelBakery.BANNER_BASE, true);
        cyi.popPose();
        cyi.popPose();
    }
    
    public static void renderPatterns(final BannerBlockEntity bvz, final PoseStack cyi, final MultiBufferSource dqq, final int integer4, final int integer5, final ModelPart dnf, final Material ebv, final boolean boolean8) {
        dnf.render(cyi, ebv.buffer(dqq, RenderType::entitySolid), integer4, integer5);
        final List<BannerPattern> list9 = bvz.getPatterns();
        final List<DyeColor> list10 = bvz.getColors();
        for (int integer6 = 0; integer6 < 17 && integer6 < list9.size() && integer6 < list10.size(); ++integer6) {
            final BannerPattern bwa12 = list9.get(integer6);
            final DyeColor bdg13 = list10.get(integer6);
            final float[] arr14 = bdg13.getTextureDiffuseColors();
            final Material ebv2 = new Material(boolean8 ? Sheets.BANNER_SHEET : Sheets.SHIELD_SHEET, bwa12.location(boolean8));
            dnf.render(cyi, ebv2.buffer(dqq, RenderType::entityNoOutline), integer4, integer5, arr14[0], arr14[1], arr14[2], 1.0f);
        }
    }
}
