package net.minecraft.client.renderer.blockentity;

import net.minecraft.resources.ResourceLocation;
import net.minecraft.client.renderer.texture.TextureAtlas;
import net.minecraft.world.level.block.entity.BlockEntity;
import net.minecraft.client.Camera;
import com.mojang.blaze3d.vertex.VertexConsumer;
import com.mojang.math.Quaternion;
import net.minecraft.util.Mth;
import com.mojang.math.Vector3f;
import net.minecraft.client.renderer.RenderType;
import net.minecraft.client.renderer.MultiBufferSource;
import com.mojang.blaze3d.vertex.PoseStack;
import net.minecraft.client.model.geom.ModelPart;
import net.minecraft.client.resources.model.Material;
import net.minecraft.world.level.block.entity.ConduitBlockEntity;

public class ConduitRenderer extends BlockEntityRenderer<ConduitBlockEntity> {
    public static final Material SHELL_TEXTURE;
    public static final Material ACTIVE_SHELL_TEXTURE;
    public static final Material WIND_TEXTURE;
    public static final Material VERTICAL_WIND_TEXTURE;
    public static final Material OPEN_EYE_TEXTURE;
    public static final Material CLOSED_EYE_TEXTURE;
    private final ModelPart eye;
    private final ModelPart wind;
    private final ModelPart shell;
    private final ModelPart cage;
    
    public ConduitRenderer(final BlockEntityRenderDispatcher dsn) {
        super(dsn);
        (this.eye = new ModelPart(16, 16, 0, 0)).addBox(-4.0f, -4.0f, 0.0f, 8.0f, 8.0f, 0.0f, 0.01f);
        (this.wind = new ModelPart(64, 32, 0, 0)).addBox(-8.0f, -8.0f, -8.0f, 16.0f, 16.0f, 16.0f);
        (this.shell = new ModelPart(32, 16, 0, 0)).addBox(-3.0f, -3.0f, -3.0f, 6.0f, 6.0f, 6.0f);
        (this.cage = new ModelPart(32, 16, 0, 0)).addBox(-4.0f, -4.0f, -4.0f, 8.0f, 8.0f, 8.0f);
    }
    
    @Override
    public void render(final ConduitBlockEntity bwp, final float float2, final PoseStack cyi, final MultiBufferSource dqq, final int integer5, final int integer6) {
        final float float3 = bwp.tickCount + float2;
        if (!bwp.isActive()) {
            final float float4 = bwp.getActiveRotation(0.0f);
            final VertexConsumer cyl10 = ConduitRenderer.SHELL_TEXTURE.buffer(dqq, RenderType::entitySolid);
            cyi.pushPose();
            cyi.translate(0.5, 0.5, 0.5);
            cyi.mulPose(Vector3f.YP.rotationDegrees(float4));
            this.shell.render(cyi, cyl10, integer5, integer6);
            cyi.popPose();
            return;
        }
        final float float4 = bwp.getActiveRotation(float2) * 57.295776f;
        float float5 = Mth.sin(float3 * 0.1f) / 2.0f + 0.5f;
        float5 += float5 * float5;
        cyi.pushPose();
        cyi.translate(0.5, 0.3f + float5 * 0.2f, 0.5);
        final Vector3f e11 = new Vector3f(0.5f, 1.0f, 0.5f);
        e11.normalize();
        cyi.mulPose(new Quaternion(e11, float4, true));
        this.cage.render(cyi, ConduitRenderer.ACTIVE_SHELL_TEXTURE.buffer(dqq, RenderType::entityCutoutNoCull), integer5, integer6);
        cyi.popPose();
        final int integer7 = bwp.tickCount / 66 % 3;
        cyi.pushPose();
        cyi.translate(0.5, 0.5, 0.5);
        if (integer7 == 1) {
            cyi.mulPose(Vector3f.XP.rotationDegrees(90.0f));
        }
        else if (integer7 == 2) {
            cyi.mulPose(Vector3f.ZP.rotationDegrees(90.0f));
        }
        final VertexConsumer cyl11 = ((integer7 == 1) ? ConduitRenderer.VERTICAL_WIND_TEXTURE : ConduitRenderer.WIND_TEXTURE).buffer(dqq, RenderType::entityCutoutNoCull);
        this.wind.render(cyi, cyl11, integer5, integer6);
        cyi.popPose();
        cyi.pushPose();
        cyi.translate(0.5, 0.5, 0.5);
        cyi.scale(0.875f, 0.875f, 0.875f);
        cyi.mulPose(Vector3f.XP.rotationDegrees(180.0f));
        cyi.mulPose(Vector3f.ZP.rotationDegrees(180.0f));
        this.wind.render(cyi, cyl11, integer5, integer6);
        cyi.popPose();
        final Camera daz14 = this.renderer.camera;
        cyi.pushPose();
        cyi.translate(0.5, 0.3f + float5 * 0.2f, 0.5);
        cyi.scale(0.5f, 0.5f, 0.5f);
        final float float6 = -daz14.getYRot();
        cyi.mulPose(Vector3f.YP.rotationDegrees(float6));
        cyi.mulPose(Vector3f.XP.rotationDegrees(daz14.getXRot()));
        cyi.mulPose(Vector3f.ZP.rotationDegrees(180.0f));
        final float float7 = 1.3333334f;
        cyi.scale(1.3333334f, 1.3333334f, 1.3333334f);
        this.eye.render(cyi, (bwp.isHunting() ? ConduitRenderer.OPEN_EYE_TEXTURE : ConduitRenderer.CLOSED_EYE_TEXTURE).buffer(dqq, RenderType::entityCutoutNoCull), integer5, integer6);
        cyi.popPose();
    }
    
    static {
        SHELL_TEXTURE = new Material(TextureAtlas.LOCATION_BLOCKS, new ResourceLocation("entity/conduit/base"));
        ACTIVE_SHELL_TEXTURE = new Material(TextureAtlas.LOCATION_BLOCKS, new ResourceLocation("entity/conduit/cage"));
        WIND_TEXTURE = new Material(TextureAtlas.LOCATION_BLOCKS, new ResourceLocation("entity/conduit/wind"));
        VERTICAL_WIND_TEXTURE = new Material(TextureAtlas.LOCATION_BLOCKS, new ResourceLocation("entity/conduit/wind_vertical"));
        OPEN_EYE_TEXTURE = new Material(TextureAtlas.LOCATION_BLOCKS, new ResourceLocation("entity/conduit/open_eye"));
        CLOSED_EYE_TEXTURE = new Material(TextureAtlas.LOCATION_BLOCKS, new ResourceLocation("entity/conduit/closed_eye"));
    }
}
