package net.minecraft.client.renderer.blockentity;

import net.minecraft.Util;
import com.google.common.collect.Maps;
import net.minecraft.client.model.dragon.DragonHeadModel;
import net.minecraft.client.model.HumanoidHeadModel;
import java.util.HashMap;
import net.minecraft.world.level.block.entity.BlockEntity;
import net.minecraft.client.resources.DefaultPlayerSkin;
import net.minecraft.world.entity.player.Player;
import com.mojang.authlib.minecraft.MinecraftProfileTexture;
import net.minecraft.client.Minecraft;
import net.minecraft.client.renderer.RenderType;
import com.mojang.blaze3d.vertex.VertexConsumer;
import net.minecraft.client.renderer.texture.OverlayTexture;
import com.mojang.authlib.GameProfile;
import javax.annotation.Nullable;
import net.minecraft.world.level.block.state.BlockState;
import net.minecraft.world.level.block.AbstractSkullBlock;
import net.minecraft.world.level.block.state.properties.Property;
import net.minecraft.core.Direction;
import net.minecraft.world.level.block.WallSkullBlock;
import net.minecraft.client.renderer.MultiBufferSource;
import com.mojang.blaze3d.vertex.PoseStack;
import net.minecraft.resources.ResourceLocation;
import net.minecraft.client.model.SkullModel;
import net.minecraft.world.level.block.SkullBlock;
import java.util.Map;
import net.minecraft.world.level.block.entity.SkullBlockEntity;

public class SkullBlockRenderer extends BlockEntityRenderer<SkullBlockEntity> {
    private static final Map<SkullBlock.Type, SkullModel> MODEL_BY_TYPE;
    private static final Map<SkullBlock.Type, ResourceLocation> SKIN_BY_TYPE;
    
    public SkullBlockRenderer(final BlockEntityRenderDispatcher dsn) {
        super(dsn);
    }
    
    @Override
    public void render(final SkullBlockEntity bxf, final float float2, final PoseStack cyi, final MultiBufferSource dqq, final int integer5, final int integer6) {
        final float float3 = bxf.getMouthAnimation(float2);
        final BlockState byg9 = bxf.getBlockState();
        final boolean boolean10 = byg9.getBlock() instanceof WallSkullBlock;
        final Direction fp11 = boolean10 ? byg9.<Direction>getValue((Property<Direction>)WallSkullBlock.FACING) : null;
        final float float4 = 22.5f * (boolean10 ? ((2 + fp11.get2DDataValue()) * 4) : byg9.<Integer>getValue((Property<Integer>)SkullBlock.ROTATION));
        renderSkull(fp11, float4, ((AbstractSkullBlock)byg9.getBlock()).getType(), bxf.getOwnerProfile(), float3, cyi, dqq, integer5);
    }
    
    public static void renderSkull(@Nullable final Direction fp, final float float2, final SkullBlock.Type a, @Nullable final GameProfile gameProfile, final float float5, final PoseStack cyi, final MultiBufferSource dqq, final int integer) {
        final SkullModel dmm9 = SkullBlockRenderer.MODEL_BY_TYPE.get(a);
        cyi.pushPose();
        if (fp == null) {
            cyi.translate(0.5, 0.0, 0.5);
        }
        else {
            switch (fp) {
                case NORTH: {
                    cyi.translate(0.5, 0.25, 0.7400000095367432);
                    break;
                }
                case SOUTH: {
                    cyi.translate(0.5, 0.25, 0.25999999046325684);
                    break;
                }
                case WEST: {
                    cyi.translate(0.7400000095367432, 0.25, 0.5);
                    break;
                }
                default: {
                    cyi.translate(0.25999999046325684, 0.25, 0.5);
                    break;
                }
            }
        }
        cyi.scale(-1.0f, -1.0f, 1.0f);
        final VertexConsumer cyl10 = dqq.getBuffer(getRenderType(a, gameProfile));
        dmm9.setupAnim(float5, float2, 0.0f);
        dmm9.renderToBuffer(cyi, cyl10, integer, OverlayTexture.NO_OVERLAY, 1.0f, 1.0f, 1.0f, 1.0f);
        cyi.popPose();
    }
    
    private static RenderType getRenderType(final SkullBlock.Type a, @Nullable final GameProfile gameProfile) {
        final ResourceLocation sm3 = SkullBlockRenderer.SKIN_BY_TYPE.get(a);
        if (a != SkullBlock.Types.PLAYER || gameProfile == null) {
            return RenderType.entityCutoutNoCull(sm3);
        }
        final Minecraft dbl4 = Minecraft.getInstance();
        final Map<MinecraftProfileTexture.Type, MinecraftProfileTexture> map5 = dbl4.getSkinManager().getInsecureSkinInformation(gameProfile);
        if (map5.containsKey(MinecraftProfileTexture.Type.SKIN)) {
            return RenderType.entityTranslucent(dbl4.getSkinManager().registerTexture(map5.get(MinecraftProfileTexture.Type.SKIN), MinecraftProfileTexture.Type.SKIN));
        }
        return RenderType.entityCutoutNoCull(DefaultPlayerSkin.getDefaultSkin(Player.createPlayerUUID(gameProfile)));
    }
    
    static {
        final SkullModel dmm2;
        final HumanoidHeadModel dmm3;
        final DragonHeadModel dnd4;
        MODEL_BY_TYPE = Util.<Map<SkullBlock.Type, SkullModel>>make(Maps.newHashMap(), hashMap -> {
            dmm2 = new SkullModel(0, 0, 64, 32);
            dmm3 = new HumanoidHeadModel();
            dnd4 = new DragonHeadModel(0.0f);
            hashMap.put(SkullBlock.Types.SKELETON, (DragonHeadModel)dmm2);
            hashMap.put(SkullBlock.Types.WITHER_SKELETON, (DragonHeadModel)dmm2);
            hashMap.put(SkullBlock.Types.PLAYER, (DragonHeadModel)dmm3);
            hashMap.put(SkullBlock.Types.ZOMBIE, (DragonHeadModel)dmm3);
            hashMap.put(SkullBlock.Types.CREEPER, (DragonHeadModel)dmm2);
            hashMap.put(SkullBlock.Types.DRAGON, dnd4);
            return;
        });
        SKIN_BY_TYPE = Util.<Map<SkullBlock.Type, ResourceLocation>>make(Maps.newHashMap(), hashMap -> {
            hashMap.put(SkullBlock.Types.SKELETON, new ResourceLocation("textures/entity/skeleton/skeleton.png"));
            hashMap.put(SkullBlock.Types.WITHER_SKELETON, new ResourceLocation("textures/entity/skeleton/wither_skeleton.png"));
            hashMap.put(SkullBlock.Types.ZOMBIE, new ResourceLocation("textures/entity/zombie/zombie.png"));
            hashMap.put(SkullBlock.Types.CREEPER, new ResourceLocation("textures/entity/creeper/creeper.png"));
            hashMap.put(SkullBlock.Types.DRAGON, new ResourceLocation("textures/entity/enderdragon/dragon.png"));
            hashMap.put(SkullBlock.Types.PLAYER, DefaultPlayerSkin.getDefaultSkin());
        });
    }
}
