package net.minecraft.client.renderer.debug;

import net.minecraft.client.gui.Font;
import com.mojang.math.Transformation;
import com.mojang.math.Matrix4f;
import com.mojang.blaze3d.systems.RenderSystem;
import com.mojang.blaze3d.vertex.BufferBuilder;
import net.minecraft.client.renderer.LevelRenderer;
import com.mojang.blaze3d.vertex.DefaultVertexFormat;
import com.mojang.blaze3d.vertex.Tesselator;
import net.minecraft.client.Camera;
import net.minecraft.core.BlockPos;
import net.minecraft.world.phys.EntityHitResult;
import java.util.function.Predicate;
import net.minecraft.world.phys.AABB;
import net.minecraft.world.phys.Vec3;
import net.minecraft.world.entity.projectile.ProjectileUtil;
import java.util.Optional;
import javax.annotation.Nullable;
import net.minecraft.world.entity.Entity;
import net.minecraft.client.renderer.MultiBufferSource;
import com.mojang.blaze3d.vertex.PoseStack;
import net.minecraft.client.Minecraft;

public class DebugRenderer {
    public final PathfindingRenderer pathfindingRenderer;
    public final SimpleDebugRenderer waterDebugRenderer;
    public final SimpleDebugRenderer chunkBorderRenderer;
    public final SimpleDebugRenderer heightMapRenderer;
    public final SimpleDebugRenderer collisionBoxRenderer;
    public final SimpleDebugRenderer neighborsUpdateRenderer;
    public final CaveDebugRenderer caveRenderer;
    public final StructureRenderer structureRenderer;
    public final SimpleDebugRenderer lightDebugRenderer;
    public final SimpleDebugRenderer worldGenAttemptRenderer;
    public final SimpleDebugRenderer solidFaceRenderer;
    public final SimpleDebugRenderer chunkRenderer;
    public final VillageDebugRenderer villageDebugRenderer;
    public final BeeDebugRenderer beeDebugRenderer;
    public final RaidDebugRenderer raidDebugRenderer;
    public final GoalSelectorDebugRenderer goalSelectorRenderer;
    public final GameTestDebugRenderer gameTestDebugRenderer;
    private boolean renderChunkborder;
    
    public DebugRenderer(final Minecraft dbl) {
        this.pathfindingRenderer = new PathfindingRenderer();
        this.waterDebugRenderer = new WaterDebugRenderer(dbl);
        this.chunkBorderRenderer = new ChunkBorderRenderer(dbl);
        this.heightMapRenderer = new HeightMapRenderer(dbl);
        this.collisionBoxRenderer = new CollisionBoxRenderer(dbl);
        this.neighborsUpdateRenderer = new NeighborsUpdateRenderer(dbl);
        this.caveRenderer = new CaveDebugRenderer();
        this.structureRenderer = new StructureRenderer(dbl);
        this.lightDebugRenderer = new LightDebugRenderer(dbl);
        this.worldGenAttemptRenderer = new WorldGenAttemptRenderer();
        this.solidFaceRenderer = new SolidFaceRenderer(dbl);
        this.chunkRenderer = new ChunkDebugRenderer(dbl);
        this.villageDebugRenderer = new VillageDebugRenderer(dbl);
        this.beeDebugRenderer = new BeeDebugRenderer(dbl);
        this.raidDebugRenderer = new RaidDebugRenderer(dbl);
        this.goalSelectorRenderer = new GoalSelectorDebugRenderer(dbl);
        this.gameTestDebugRenderer = new GameTestDebugRenderer();
    }
    
    public void clear() {
        this.pathfindingRenderer.clear();
        this.waterDebugRenderer.clear();
        this.chunkBorderRenderer.clear();
        this.heightMapRenderer.clear();
        this.collisionBoxRenderer.clear();
        this.neighborsUpdateRenderer.clear();
        this.caveRenderer.clear();
        this.structureRenderer.clear();
        this.lightDebugRenderer.clear();
        this.worldGenAttemptRenderer.clear();
        this.solidFaceRenderer.clear();
        this.chunkRenderer.clear();
        this.villageDebugRenderer.clear();
        this.beeDebugRenderer.clear();
        this.raidDebugRenderer.clear();
        this.goalSelectorRenderer.clear();
        this.gameTestDebugRenderer.clear();
    }
    
    public boolean switchRenderChunkborder() {
        return this.renderChunkborder = !this.renderChunkborder;
    }
    
    public void render(final PoseStack cyi, final MultiBufferSource.BufferSource a, final double double3, final double double4, final double double5) {
        if (this.renderChunkborder && !Minecraft.getInstance().showOnlyReducedInfo()) {
            this.chunkBorderRenderer.render(cyi, a, double3, double4, double5);
        }
        this.gameTestDebugRenderer.render(cyi, a, double3, double4, double5);
    }
    
    public static Optional<Entity> getTargetedEntity(@Nullable final Entity akn, final int integer) {
        if (akn == null) {
            return Optional.<Entity>empty();
        }
        final Vec3 cvi3 = akn.getEyePosition(1.0f);
        final Vec3 cvi4 = akn.getViewVector(1.0f).scale(integer);
        final Vec3 cvi5 = cvi3.add(cvi4);
        final AABB cvc6 = akn.getBoundingBox().expandTowards(cvi4).inflate(1.0);
        final int integer2 = integer * integer;
        final Predicate<Entity> predicate8 = akn -> !akn.isSpectator() && akn.isPickable();
        final EntityHitResult cve9 = ProjectileUtil.getEntityHitResult(akn, cvi3, cvi5, cvc6, predicate8, integer2);
        if (cve9 == null) {
            return Optional.<Entity>empty();
        }
        if (cvi3.distanceToSqr(cve9.getLocation()) > integer2) {
            return Optional.<Entity>empty();
        }
        return Optional.<Entity>of(cve9.getEntity());
    }
    
    public static void renderFilledBox(final BlockPos fk1, final BlockPos fk2, final float float3, final float float4, final float float5, final float float6) {
        final Camera daz7 = Minecraft.getInstance().gameRenderer.getMainCamera();
        if (!daz7.isInitialized()) {
            return;
        }
        final Vec3 cvi8 = daz7.getPosition().reverse();
        final AABB cvc9 = new AABB(fk1, fk2).move(cvi8);
        renderFilledBox(cvc9, float3, float4, float5, float6);
    }
    
    public static void renderFilledBox(final BlockPos fk, final float float2, final float float3, final float float4, final float float5, final float float6) {
        final Camera daz7 = Minecraft.getInstance().gameRenderer.getMainCamera();
        if (!daz7.isInitialized()) {
            return;
        }
        final Vec3 cvi8 = daz7.getPosition().reverse();
        final AABB cvc9 = new AABB(fk).move(cvi8).inflate(float2);
        renderFilledBox(cvc9, float3, float4, float5, float6);
    }
    
    public static void renderFilledBox(final AABB cvc, final float float2, final float float3, final float float4, final float float5) {
        renderFilledBox(cvc.minX, cvc.minY, cvc.minZ, cvc.maxX, cvc.maxY, cvc.maxZ, float2, float3, float4, float5);
    }
    
    public static void renderFilledBox(final double double1, final double double2, final double double3, final double double4, final double double5, final double double6, final float float7, final float float8, final float float9, final float float10) {
        final Tesselator cyj17 = Tesselator.getInstance();
        final BufferBuilder cyd18 = cyj17.getBuilder();
        cyd18.begin(5, DefaultVertexFormat.POSITION_COLOR);
        LevelRenderer.addChainedFilledBoxVertices(cyd18, double1, double2, double3, double4, double5, double6, float7, float8, float9, float10);
        cyj17.end();
    }
    
    public static void renderFloatingText(final String string, final int integer2, final int integer3, final int integer4, final int integer5) {
        renderFloatingText(string, integer2 + 0.5, integer3 + 0.5, integer4 + 0.5, integer5);
    }
    
    public static void renderFloatingText(final String string, final double double2, final double double3, final double double4, final int integer) {
        renderFloatingText(string, double2, double3, double4, integer, 0.02f);
    }
    
    public static void renderFloatingText(final String string, final double double2, final double double3, final double double4, final int integer, final float float6) {
        renderFloatingText(string, double2, double3, double4, integer, float6, true, 0.0f, false);
    }
    
    public static void renderFloatingText(final String string, final double double2, final double double3, final double double4, final int integer, final float float6, final boolean boolean7, final float float8, final boolean boolean9) {
        final Minecraft dbl13 = Minecraft.getInstance();
        final Camera daz14 = dbl13.gameRenderer.getMainCamera();
        if (!daz14.isInitialized() || dbl13.getEntityRenderDispatcher().options == null) {
            return;
        }
        final Font dcf15 = dbl13.font;
        final double double5 = daz14.getPosition().x;
        final double double6 = daz14.getPosition().y;
        final double double7 = daz14.getPosition().z;
        RenderSystem.pushMatrix();
        RenderSystem.translatef((float)(double2 - double5), (float)(double3 - double6) + 0.07f, (float)(double4 - double7));
        RenderSystem.normal3f(0.0f, 1.0f, 0.0f);
        RenderSystem.multMatrix(new Matrix4f(daz14.rotation()));
        RenderSystem.scalef(float6, -float6, float6);
        RenderSystem.enableTexture();
        if (boolean9) {
            RenderSystem.disableDepthTest();
        }
        else {
            RenderSystem.enableDepthTest();
        }
        RenderSystem.depthMask(true);
        RenderSystem.scalef(-1.0f, 1.0f, 1.0f);
        float float9 = boolean7 ? (-dcf15.width(string) / 2.0f) : 0.0f;
        float9 -= float8 / float6;
        RenderSystem.enableAlphaTest();
        final MultiBufferSource.BufferSource a23 = MultiBufferSource.immediate(Tesselator.getInstance().getBuilder());
        dcf15.drawInBatch(string, float9, 0.0f, integer, false, Transformation.identity().getMatrix(), a23, boolean9, 0, 15728880);
        a23.endBatch();
        RenderSystem.color4f(1.0f, 1.0f, 1.0f, 1.0f);
        RenderSystem.enableDepthTest();
        RenderSystem.popMatrix();
    }
    
    public interface SimpleDebugRenderer {
        void render(final PoseStack cyi, final MultiBufferSource dqq, final double double3, final double double4, final double double5);
        
        default void clear() {
        }
    }
}
