package net.minecraft.client.renderer.debug;

import java.util.List;
import java.util.Iterator;
import com.mojang.blaze3d.vertex.VertexConsumer;
import net.minecraft.world.level.LevelAccessor;
import net.minecraft.client.Camera;
import net.minecraft.client.renderer.LevelRenderer;
import net.minecraft.client.renderer.RenderType;
import net.minecraft.core.BlockPos;
import net.minecraft.client.renderer.MultiBufferSource;
import com.mojang.blaze3d.vertex.PoseStack;
import com.google.common.collect.Maps;
import net.minecraft.world.level.levelgen.structure.BoundingBox;
import net.minecraft.world.level.dimension.DimensionType;
import java.util.Map;
import net.minecraft.client.Minecraft;

public class StructureRenderer implements DebugRenderer.SimpleDebugRenderer {
    private final Minecraft minecraft;
    private final Map<DimensionType, Map<String, BoundingBox>> postMainBoxes;
    private final Map<DimensionType, Map<String, BoundingBox>> postPiecesBoxes;
    private final Map<DimensionType, Map<String, Boolean>> startPiecesMap;
    
    public StructureRenderer(final Minecraft dbl) {
        this.postMainBoxes = Maps.newIdentityHashMap();
        this.postPiecesBoxes = Maps.newIdentityHashMap();
        this.startPiecesMap = Maps.newIdentityHashMap();
        this.minecraft = dbl;
    }
    
    @Override
    public void render(final PoseStack cyi, final MultiBufferSource dqq, final double double3, final double double4, final double double5) {
        final Camera daz10 = this.minecraft.gameRenderer.getMainCamera();
        final LevelAccessor bju11 = this.minecraft.level;
        final DimensionType cbf12 = bju11.getDimension().getType();
        final BlockPos fk13 = new BlockPos(daz10.getPosition().x, 0.0, daz10.getPosition().z);
        final VertexConsumer cyl14 = dqq.getBuffer(RenderType.lines());
        if (this.postMainBoxes.containsKey(cbf12)) {
            for (final BoundingBox cky16 : this.postMainBoxes.get(cbf12).values()) {
                if (fk13.closerThan(cky16.getCenter(), 500.0)) {
                    LevelRenderer.renderLineBox(cyl14, cky16.x0 - double3, cky16.y0 - double4, cky16.z0 - double5, cky16.x1 + 1 - double3, cky16.y1 + 1 - double4, cky16.z1 + 1 - double5, 1.0f, 1.0f, 1.0f, 1.0f);
                }
            }
        }
        if (this.postPiecesBoxes.containsKey(cbf12)) {
            for (final Map.Entry<String, BoundingBox> entry16 : this.postPiecesBoxes.get(cbf12).entrySet()) {
                final String string17 = entry16.getKey();
                final BoundingBox cky17 = entry16.getValue();
                final Boolean boolean19 = this.startPiecesMap.get(cbf12).get(string17);
                if (fk13.closerThan(cky17.getCenter(), 500.0)) {
                    if (boolean19) {
                        LevelRenderer.renderLineBox(cyl14, cky17.x0 - double3, cky17.y0 - double4, cky17.z0 - double5, cky17.x1 + 1 - double3, cky17.y1 + 1 - double4, cky17.z1 + 1 - double5, 0.0f, 1.0f, 0.0f, 1.0f);
                    }
                    else {
                        LevelRenderer.renderLineBox(cyl14, cky17.x0 - double3, cky17.y0 - double4, cky17.z0 - double5, cky17.x1 + 1 - double3, cky17.y1 + 1 - double4, cky17.z1 + 1 - double5, 0.0f, 0.0f, 1.0f, 1.0f);
                    }
                }
            }
        }
    }
    
    public void addBoundingBox(final BoundingBox cky, final List<BoundingBox> list2, final List<Boolean> list3, final DimensionType cbf) {
        if (!this.postMainBoxes.containsKey(cbf)) {
            this.postMainBoxes.put(cbf, Maps.newHashMap());
        }
        if (!this.postPiecesBoxes.containsKey(cbf)) {
            this.postPiecesBoxes.put(cbf, Maps.newHashMap());
            this.startPiecesMap.put(cbf, Maps.newHashMap());
        }
        this.postMainBoxes.get(cbf).put(cky.toString(), cky);
        for (int integer6 = 0; integer6 < list2.size(); ++integer6) {
            final BoundingBox cky2 = list2.get(integer6);
            final Boolean boolean8 = list3.get(integer6);
            this.postPiecesBoxes.get(cbf).put(cky2.toString(), cky2);
            this.startPiecesMap.get(cbf).put(cky2.toString(), boolean8);
        }
    }
    
    @Override
    public void clear() {
        this.postMainBoxes.clear();
        this.postPiecesBoxes.clear();
        this.startPiecesMap.clear();
    }
}
