package net.minecraft.client.renderer.entity;

import net.minecraft.world.entity.Entity;
import com.mojang.blaze3d.vertex.VertexConsumer;
import net.minecraft.client.renderer.RenderType;
import net.minecraft.client.renderer.texture.OverlayTexture;
import com.mojang.math.Quaternion;
import net.minecraft.util.Mth;
import com.mojang.math.Vector3f;
import net.minecraft.client.renderer.MultiBufferSource;
import com.mojang.blaze3d.vertex.PoseStack;
import net.minecraft.client.model.BoatModel;
import net.minecraft.resources.ResourceLocation;
import net.minecraft.world.entity.vehicle.Boat;

public class BoatRenderer extends EntityRenderer<Boat> {
    private static final ResourceLocation[] BOAT_TEXTURE_LOCATIONS;
    protected final BoatModel model;
    
    public BoatRenderer(final EntityRenderDispatcher dvc) {
        super(dvc);
        this.model = new BoatModel();
        this.shadowRadius = 0.8f;
    }
    
    @Override
    public void render(final Boat azw, final float float2, final float float3, final PoseStack cyi, final MultiBufferSource dqq, final int integer) {
        cyi.pushPose();
        cyi.translate(0.0, 0.375, 0.0);
        cyi.mulPose(Vector3f.YP.rotationDegrees(180.0f - float2));
        final float float4 = azw.getHurtTime() - float3;
        float float5 = azw.getDamage() - float3;
        if (float5 < 0.0f) {
            float5 = 0.0f;
        }
        if (float4 > 0.0f) {
            cyi.mulPose(Vector3f.XP.rotationDegrees(Mth.sin(float4) * float4 * float5 / 10.0f * azw.getHurtDir()));
        }
        final float float6 = azw.getBubbleAngle(float3);
        if (!Mth.equal(float6, 0.0f)) {
            cyi.mulPose(new Quaternion(new Vector3f(1.0f, 0.0f, 1.0f), azw.getBubbleAngle(float3), true));
        }
        cyi.scale(-1.0f, -1.0f, 1.0f);
        cyi.mulPose(Vector3f.YP.rotationDegrees(90.0f));
        this.model.setupAnim(azw, float3, 0.0f, -0.1f, 0.0f, 0.0f);
        final VertexConsumer cyl11 = dqq.getBuffer(this.model.renderType(this.getTextureLocation(azw)));
        this.model.renderToBuffer(cyi, cyl11, integer, OverlayTexture.NO_OVERLAY, 1.0f, 1.0f, 1.0f, 1.0f);
        final VertexConsumer cyl12 = dqq.getBuffer(RenderType.waterMask());
        this.model.waterPatch().render(cyi, cyl12, integer, OverlayTexture.NO_OVERLAY);
        cyi.popPose();
        super.render(azw, float2, float3, cyi, dqq, integer);
    }
    
    @Override
    public ResourceLocation getTextureLocation(final Boat azw) {
        return BoatRenderer.BOAT_TEXTURE_LOCATIONS[azw.getBoatType().ordinal()];
    }
    
    static {
        BOAT_TEXTURE_LOCATIONS = new ResourceLocation[] { new ResourceLocation("textures/entity/boat/oak.png"), new ResourceLocation("textures/entity/boat/spruce.png"), new ResourceLocation("textures/entity/boat/birch.png"), new ResourceLocation("textures/entity/boat/jungle.png"), new ResourceLocation("textures/entity/boat/acacia.png"), new ResourceLocation("textures/entity/boat/dark_oak.png") };
    }
}
