package net.minecraft.client.renderer.entity;

import com.mojang.math.Matrix4f;
import net.minecraft.client.Minecraft;
import net.minecraft.client.gui.Font;
import net.minecraft.resources.ResourceLocation;
import net.minecraft.client.renderer.MultiBufferSource;
import com.mojang.blaze3d.vertex.PoseStack;
import net.minecraft.world.phys.Vec3;
import net.minecraft.world.phys.AABB;
import net.minecraft.client.renderer.culling.Frustum;
import net.minecraft.client.renderer.LightTexture;
import net.minecraft.core.BlockPos;
import net.minecraft.world.level.LightLayer;
import net.minecraft.world.entity.Entity;

public abstract class EntityRenderer<T extends Entity> {
    protected final EntityRenderDispatcher entityRenderDispatcher;
    protected float shadowRadius;
    protected float shadowStrength;
    
    protected EntityRenderer(final EntityRenderDispatcher dvc) {
        this.shadowStrength = 1.0f;
        this.entityRenderDispatcher = dvc;
    }
    
    public final int getPackedLightCoords(final T akn, final float float2) {
        return LightTexture.pack(this.getBlockLightLevel(akn, float2), akn.level.getBrightness(LightLayer.SKY, new BlockPos(akn.getEyePosition(float2))));
    }
    
    protected int getBlockLightLevel(final T akn, final float float2) {
        if (akn.isOnFire()) {
            return 15;
        }
        return akn.level.getBrightness(LightLayer.BLOCK, new BlockPos(akn.getEyePosition(float2)));
    }
    
    public boolean shouldRender(final T akn, final Frustum dtj, final double double3, final double double4, final double double5) {
        if (!akn.shouldRender(double3, double4, double5)) {
            return false;
        }
        if (akn.noCulling) {
            return true;
        }
        AABB cvc10 = akn.getBoundingBoxForCulling().inflate(0.5);
        if (cvc10.hasNaN() || cvc10.getSize() == 0.0) {
            cvc10 = new AABB(akn.getX() - 2.0, akn.getY() - 2.0, akn.getZ() - 2.0, akn.getX() + 2.0, akn.getY() + 2.0, akn.getZ() + 2.0);
        }
        return dtj.isVisible(cvc10);
    }
    
    public Vec3 getRenderOffset(final T akn, final float float2) {
        return Vec3.ZERO;
    }
    
    public void render(final T akn, final float float2, final float float3, final PoseStack cyi, final MultiBufferSource dqq, final int integer) {
        if (!this.shouldShowName(akn)) {
            return;
        }
        this.renderNameTag(akn, akn.getDisplayName().getColoredString(), cyi, dqq, integer);
    }
    
    protected boolean shouldShowName(final T akn) {
        return akn.shouldShowName() && akn.hasCustomName();
    }
    
    public abstract ResourceLocation getTextureLocation(final T akn);
    
    public Font getFont() {
        return this.entityRenderDispatcher.getFont();
    }
    
    protected void renderNameTag(final T akn, final String string, final PoseStack cyi, final MultiBufferSource dqq, final int integer) {
        final double double7 = this.entityRenderDispatcher.distanceToSqr(akn);
        if (double7 > 4096.0) {
            return;
        }
        final boolean boolean9 = !akn.isDiscrete();
        final float float10 = akn.getBbHeight() + 0.5f;
        final int integer2 = "deadmau5".equals(string) ? -10 : 0;
        cyi.pushPose();
        cyi.translate(0.0, float10, 0.0);
        cyi.mulPose(this.entityRenderDispatcher.cameraOrientation());
        cyi.scale(-0.025f, -0.025f, 0.025f);
        final Matrix4f b12 = cyi.last().pose();
        final float float11 = Minecraft.getInstance().options.getBackgroundOpacity(0.25f);
        final int integer3 = (int)(float11 * 255.0f) << 24;
        final Font dcf15 = this.getFont();
        final float float12 = (float)(-dcf15.width(string) / 2);
        dcf15.drawInBatch(string, float12, (float)integer2, 553648127, false, b12, dqq, boolean9, integer3, integer);
        if (boolean9) {
            dcf15.drawInBatch(string, float12, (float)integer2, -1, false, b12, dqq, false, 0, integer);
        }
        cyi.popPose();
    }
    
    public EntityRenderDispatcher getDispatcher() {
        return this.entityRenderDispatcher;
    }
}
