package net.minecraft.client.renderer.entity;

import net.minecraft.world.entity.Entity;
import net.minecraft.client.renderer.texture.OverlayTexture;
import net.minecraft.world.item.ItemStack;
import com.mojang.blaze3d.vertex.VertexConsumer;
import com.mojang.math.Matrix3f;
import com.mojang.math.Matrix4f;
import net.minecraft.world.entity.player.Player;
import net.minecraft.world.phys.Vec3;
import net.minecraft.client.Minecraft;
import net.minecraft.util.Mth;
import net.minecraft.world.item.Items;
import net.minecraft.world.entity.HumanoidArm;
import com.mojang.math.Vector3f;
import net.minecraft.client.renderer.MultiBufferSource;
import com.mojang.blaze3d.vertex.PoseStack;
import net.minecraft.client.renderer.RenderType;
import net.minecraft.resources.ResourceLocation;
import net.minecraft.world.entity.fishing.FishingHook;

public class FishingHookRenderer extends EntityRenderer<FishingHook> {
    private static final ResourceLocation TEXTURE_LOCATION;
    private static final RenderType RENDER_TYPE;
    
    public FishingHookRenderer(final EntityRenderDispatcher dvc) {
        super(dvc);
    }
    
    @Override
    public void render(final FishingHook avt, final float float2, final float float3, final PoseStack cyi, final MultiBufferSource dqq, final int integer) {
        final Player ayg8 = avt.getOwner();
        if (ayg8 == null) {
            return;
        }
        cyi.pushPose();
        cyi.pushPose();
        cyi.scale(0.5f, 0.5f, 0.5f);
        cyi.mulPose(this.entityRenderDispatcher.cameraOrientation());
        cyi.mulPose(Vector3f.YP.rotationDegrees(180.0f));
        final PoseStack.Pose a9 = cyi.last();
        final Matrix4f b10 = a9.pose();
        final Matrix3f a10 = a9.normal();
        final VertexConsumer cyl12 = dqq.getBuffer(FishingHookRenderer.RENDER_TYPE);
        vertex(cyl12, b10, a10, integer, 0.0f, 0, 0, 1);
        vertex(cyl12, b10, a10, integer, 1.0f, 0, 1, 1);
        vertex(cyl12, b10, a10, integer, 1.0f, 1, 1, 0);
        vertex(cyl12, b10, a10, integer, 0.0f, 1, 0, 0);
        cyi.popPose();
        int integer2 = (ayg8.getMainArm() == HumanoidArm.RIGHT) ? 1 : -1;
        final ItemStack bek14 = ayg8.getMainHandItem();
        if (bek14.getItem() != Items.FISHING_ROD) {
            integer2 = -integer2;
        }
        final float float4 = ayg8.getAttackAnim(float3);
        final float float5 = Mth.sin(Mth.sqrt(float4) * 3.1415927f);
        final float float6 = Mth.lerp(float3, ayg8.yBodyRotO, ayg8.yBodyRot) * 0.017453292f;
        final double double18 = Mth.sin(float6);
        final double double19 = Mth.cos(float6);
        final double double20 = integer2 * 0.35;
        final double double21 = 0.8;
        double double22;
        double double23;
        double double24;
        float float7;
        if ((this.entityRenderDispatcher.options != null && this.entityRenderDispatcher.options.thirdPersonView > 0) || ayg8 != Minecraft.getInstance().player) {
            double22 = Mth.lerp(float3, ayg8.xo, ayg8.getX()) - double19 * double20 - double18 * 0.8;
            double23 = ayg8.yo + ayg8.getEyeHeight() + (ayg8.getY() - ayg8.yo) * float3 - 0.45;
            double24 = Mth.lerp(float3, ayg8.zo, ayg8.getZ()) - double18 * double20 + double19 * 0.8;
            float7 = (ayg8.isCrouching() ? -0.1875f : 0.0f);
        }
        else {
            double double25 = this.entityRenderDispatcher.options.fov;
            double25 /= 100.0;
            Vec3 cvi35 = new Vec3(integer2 * -0.36 * double25, -0.045 * double25, 0.4);
            cvi35 = cvi35.xRot(-Mth.lerp(float3, ayg8.xRotO, ayg8.xRot) * 0.017453292f);
            cvi35 = cvi35.yRot(-Mth.lerp(float3, ayg8.yRotO, ayg8.yRot) * 0.017453292f);
            cvi35 = cvi35.yRot(float5 * 0.5f);
            cvi35 = cvi35.xRot(-float5 * 0.7f);
            double22 = Mth.lerp(float3, ayg8.xo, ayg8.getX()) + cvi35.x;
            double23 = Mth.lerp(float3, ayg8.yo, ayg8.getY()) + cvi35.y;
            double24 = Mth.lerp(float3, ayg8.zo, ayg8.getZ()) + cvi35.z;
            float7 = ayg8.getEyeHeight();
        }
        double double25 = Mth.lerp(float3, avt.xo, avt.getX());
        final double double26 = Mth.lerp(float3, avt.yo, avt.getY()) + 0.25;
        final double double27 = Mth.lerp(float3, avt.zo, avt.getZ());
        final float float8 = (float)(double22 - double25);
        final float float9 = (float)(double23 - double26) + float7;
        final float float10 = (float)(double24 - double27);
        final VertexConsumer cyl13 = dqq.getBuffer(RenderType.lines());
        final Matrix4f b11 = cyi.last().pose();
        final int integer3 = 16;
        for (int integer4 = 0; integer4 < 16; ++integer4) {
            stringVertex(float8, float9, float10, cyl13, b11, fraction(integer4, 16));
            stringVertex(float8, float9, float10, cyl13, b11, fraction(integer4 + 1, 16));
        }
        cyi.popPose();
        super.render(avt, float2, float3, cyi, dqq, integer);
    }
    
    private static float fraction(final int integer1, final int integer2) {
        return integer1 / (float)integer2;
    }
    
    private static void vertex(final VertexConsumer cyl, final Matrix4f b, final Matrix3f a, final int integer4, final float float5, final int integer6, final int integer7, final int integer8) {
        cyl.vertex(b, float5 - 0.5f, integer6 - 0.5f, 0.0f).color(255, 255, 255, 255).uv((float)integer7, (float)integer8).overlayCoords(OverlayTexture.NO_OVERLAY).uv2(integer4).normal(a, 0.0f, 1.0f, 0.0f).endVertex();
    }
    
    private static void stringVertex(final float float1, final float float2, final float float3, final VertexConsumer cyl, final Matrix4f b, final float float6) {
        cyl.vertex(b, float1 * float6, float2 * (float6 * float6 + float6) * 0.5f + 0.25f, float3 * float6).color(0, 0, 0, 255).endVertex();
    }
    
    @Override
    public ResourceLocation getTextureLocation(final FishingHook avt) {
        return FishingHookRenderer.TEXTURE_LOCATION;
    }
    
    static {
        TEXTURE_LOCATION = new ResourceLocation("textures/entity/fishing_hook.png");
        RENDER_TYPE = RenderType.entityCutout(FishingHookRenderer.TEXTURE_LOCATION);
    }
}
