package net.minecraft.client.renderer.entity;

import net.minecraft.world.entity.Entity;
import net.minecraft.client.renderer.texture.TextureAtlas;
import net.minecraft.resources.ResourceLocation;
import net.minecraft.client.resources.model.BakedModel;
import net.minecraft.client.renderer.texture.OverlayTexture;
import com.mojang.math.Vector3f;
import net.minecraft.client.renderer.block.model.ItemTransforms;
import net.minecraft.util.Mth;
import net.minecraft.world.entity.LivingEntity;
import net.minecraft.world.item.Item;
import net.minecraft.client.renderer.MultiBufferSource;
import com.mojang.blaze3d.vertex.PoseStack;
import net.minecraft.world.item.ItemStack;
import java.util.Random;
import net.minecraft.world.entity.item.ItemEntity;

public class ItemEntityRenderer extends EntityRenderer<ItemEntity> {
    private final ItemRenderer itemRenderer;
    private final Random random;
    
    public ItemEntityRenderer(final EntityRenderDispatcher dvc, final ItemRenderer dvw) {
        super(dvc);
        this.random = new Random();
        this.itemRenderer = dvw;
        this.shadowRadius = 0.15f;
        this.shadowStrength = 0.75f;
    }
    
    private int getRenderAmount(final ItemStack bek) {
        int integer3 = 1;
        if (bek.getCount() > 48) {
            integer3 = 5;
        }
        else if (bek.getCount() > 32) {
            integer3 = 4;
        }
        else if (bek.getCount() > 16) {
            integer3 = 3;
        }
        else if (bek.getCount() > 1) {
            integer3 = 2;
        }
        return integer3;
    }
    
    @Override
    public void render(final ItemEntity avy, final float float2, final float float3, final PoseStack cyi, final MultiBufferSource dqq, final int integer) {
        cyi.pushPose();
        final ItemStack bek8 = avy.getItem();
        final int integer2 = bek8.isEmpty() ? 187 : (Item.getId(bek8.getItem()) + bek8.getDamageValue());
        this.random.setSeed(integer2);
        final BakedModel ebs10 = this.itemRenderer.getModel(bek8, avy.level, null);
        final boolean boolean11 = ebs10.isGui3d();
        final int integer3 = this.getRenderAmount(bek8);
        final float float4 = 0.25f;
        final float float5 = Mth.sin((avy.getAge() + float3) / 10.0f + avy.bobOffs) * 0.1f + 0.1f;
        final float float6 = ebs10.getTransforms().getTransform(ItemTransforms.TransformType.GROUND).scale.y();
        cyi.translate(0.0, float5 + 0.25f * float6, 0.0);
        final float float7 = (avy.getAge() + float3) / 20.0f + avy.bobOffs;
        cyi.mulPose(Vector3f.YP.rotation(float7));
        final float float8 = ebs10.getTransforms().ground.scale.x();
        final float float9 = ebs10.getTransforms().ground.scale.y();
        final float float10 = ebs10.getTransforms().ground.scale.z();
        if (!boolean11) {
            final float float11 = -0.0f * (integer3 - 1) * 0.5f * float8;
            final float float12 = -0.0f * (integer3 - 1) * 0.5f * float9;
            final float float13 = -0.09375f * (integer3 - 1) * 0.5f * float10;
            cyi.translate(float11, float12, float13);
        }
        for (int integer4 = 0; integer4 < integer3; ++integer4) {
            cyi.pushPose();
            if (integer4 > 0) {
                if (boolean11) {
                    final float float12 = (this.random.nextFloat() * 2.0f - 1.0f) * 0.15f;
                    final float float13 = (this.random.nextFloat() * 2.0f - 1.0f) * 0.15f;
                    final float float14 = (this.random.nextFloat() * 2.0f - 1.0f) * 0.15f;
                    cyi.translate(float12, float13, float14);
                }
                else {
                    final float float12 = (this.random.nextFloat() * 2.0f - 1.0f) * 0.15f * 0.5f;
                    final float float13 = (this.random.nextFloat() * 2.0f - 1.0f) * 0.15f * 0.5f;
                    cyi.translate(float12, float13, 0.0);
                }
            }
            this.itemRenderer.render(bek8, ItemTransforms.TransformType.GROUND, false, cyi, dqq, integer, OverlayTexture.NO_OVERLAY, ebs10);
            cyi.popPose();
            if (!boolean11) {
                cyi.translate(0.0f * float8, 0.0f * float9, 0.09375f * float10);
            }
        }
        cyi.popPose();
        super.render(avy, float2, float3, cyi, dqq, integer);
    }
    
    @Override
    public ResourceLocation getTextureLocation(final ItemEntity avy) {
        return TextureAtlas.LOCATION_BLOCKS;
    }
}
