package net.minecraft.client.renderer.entity;

import net.minecraft.world.entity.Entity;
import net.minecraft.client.renderer.texture.TextureAtlas;
import net.minecraft.resources.ResourceLocation;
import net.minecraft.world.level.saveddata.maps.MapItemSavedData;
import net.minecraft.world.item.ItemStack;
import net.minecraft.client.resources.model.ModelManager;
import net.minecraft.client.renderer.block.BlockRenderDispatcher;
import net.minecraft.world.phys.Vec3;
import net.minecraft.core.Direction;
import net.minecraft.client.renderer.block.model.ItemTransforms;
import net.minecraft.world.item.MapItem;
import net.minecraft.world.level.block.state.BlockState;
import net.minecraft.client.renderer.texture.OverlayTexture;
import net.minecraft.client.renderer.Sheets;
import net.minecraft.world.item.Items;
import com.mojang.math.Vector3f;
import net.minecraft.client.renderer.MultiBufferSource;
import com.mojang.blaze3d.vertex.PoseStack;
import net.minecraft.client.Minecraft;
import net.minecraft.client.resources.model.ModelResourceLocation;
import net.minecraft.world.entity.decoration.ItemFrame;

public class ItemFrameRenderer extends EntityRenderer<ItemFrame> {
    private static final ModelResourceLocation FRAME_LOCATION;
    private static final ModelResourceLocation MAP_FRAME_LOCATION;
    private final Minecraft minecraft;
    private final ItemRenderer itemRenderer;
    
    public ItemFrameRenderer(final EntityRenderDispatcher dvc, final ItemRenderer dvw) {
        super(dvc);
        this.minecraft = Minecraft.getInstance();
        this.itemRenderer = dvw;
    }
    
    @Override
    public void render(final ItemFrame avo, final float float2, final float float3, final PoseStack cyi, final MultiBufferSource dqq, final int integer) {
        super.render(avo, float2, float3, cyi, dqq, integer);
        cyi.pushPose();
        final Direction fp8 = avo.getDirection();
        final Vec3 cvi9 = this.getRenderOffset(avo, float3);
        cyi.translate(-cvi9.x(), -cvi9.y(), -cvi9.z());
        final double double10 = 0.46875;
        cyi.translate(fp8.getStepX() * 0.46875, fp8.getStepY() * 0.46875, fp8.getStepZ() * 0.46875);
        cyi.mulPose(Vector3f.XP.rotationDegrees(avo.xRot));
        cyi.mulPose(Vector3f.YP.rotationDegrees(180.0f - avo.yRot));
        final BlockRenderDispatcher drh12 = this.minecraft.getBlockRenderer();
        final ModelManager ebx13 = drh12.getBlockModelShaper().getModelManager();
        final ModelResourceLocation eby14 = (avo.getItem().getItem() == Items.FILLED_MAP) ? ItemFrameRenderer.MAP_FRAME_LOCATION : ItemFrameRenderer.FRAME_LOCATION;
        cyi.pushPose();
        cyi.translate(-0.5, -0.5, -0.5);
        drh12.getModelRenderer().renderModel(cyi.last(), dqq.getBuffer(Sheets.solidBlockSheet()), null, ebx13.getModel(eby14), 1.0f, 1.0f, 1.0f, integer, OverlayTexture.NO_OVERLAY);
        cyi.popPose();
        final ItemStack bek15 = avo.getItem();
        if (!bek15.isEmpty()) {
            final boolean boolean16 = bek15.getItem() == Items.FILLED_MAP;
            cyi.translate(0.0, 0.0, 0.4375);
            final int integer2 = boolean16 ? (avo.getRotation() % 4 * 2) : avo.getRotation();
            cyi.mulPose(Vector3f.ZP.rotationDegrees(integer2 * 360.0f / 8.0f));
            if (boolean16) {
                cyi.mulPose(Vector3f.ZP.rotationDegrees(180.0f));
                final float float4 = 0.0078125f;
                cyi.scale(0.0078125f, 0.0078125f, 0.0078125f);
                cyi.translate(-64.0, -64.0, 0.0);
                final MapItemSavedData crd19 = MapItem.getOrCreateSavedData(bek15, avo.level);
                cyi.translate(0.0, 0.0, -1.0);
                if (crd19 != null) {
                    this.minecraft.gameRenderer.getMapRenderer().render(cyi, dqq, crd19, true, integer);
                }
            }
            else {
                cyi.scale(0.5f, 0.5f, 0.5f);
                this.itemRenderer.renderStatic(bek15, ItemTransforms.TransformType.FIXED, integer, OverlayTexture.NO_OVERLAY, cyi, dqq);
            }
        }
        cyi.popPose();
    }
    
    @Override
    public Vec3 getRenderOffset(final ItemFrame avo, final float float2) {
        return new Vec3(avo.getDirection().getStepX() * 0.3f, -0.25, avo.getDirection().getStepZ() * 0.3f);
    }
    
    @Override
    public ResourceLocation getTextureLocation(final ItemFrame avo) {
        return TextureAtlas.LOCATION_BLOCKS;
    }
    
    @Override
    protected boolean shouldShowName(final ItemFrame avo) {
        if (!Minecraft.renderNames() || avo.getItem().isEmpty() || !avo.getItem().hasCustomHoverName() || this.entityRenderDispatcher.crosshairPickEntity != avo) {
            return false;
        }
        final double double3 = this.entityRenderDispatcher.distanceToSqr(avo);
        final float float5 = avo.isDiscrete() ? 32.0f : 64.0f;
        return double3 < float5 * float5;
    }
    
    @Override
    protected void renderNameTag(final ItemFrame avo, final String string, final PoseStack cyi, final MultiBufferSource dqq, final int integer) {
        super.renderNameTag(avo, avo.getItem().getHoverName().getColoredString(), cyi, dqq, integer);
    }
    
    static {
        FRAME_LOCATION = new ModelResourceLocation("item_frame", "map=false");
        MAP_FRAME_LOCATION = new ModelResourceLocation("item_frame", "map=true");
    }
}
