package net.minecraft.client.renderer.entity;

import com.google.common.collect.Sets;
import net.minecraft.server.packs.resources.ResourceManager;
import com.mojang.blaze3d.vertex.DefaultVertexFormat;
import net.minecraft.client.player.LocalPlayer;
import com.mojang.blaze3d.vertex.BufferBuilder;
import net.minecraft.util.Mth;
import com.mojang.blaze3d.vertex.Tesselator;
import net.minecraft.client.gui.Font;
import net.minecraft.CrashReportCategory;
import net.minecraft.ReportedException;
import net.minecraft.CrashReport;
import net.minecraft.client.renderer.texture.OverlayTexture;
import com.mojang.blaze3d.platform.Lighting;
import net.minecraft.client.Minecraft;
import com.mojang.blaze3d.platform.GlStateManager;
import net.minecraft.client.renderer.texture.TextureAtlas;
import com.mojang.blaze3d.systems.RenderSystem;
import net.minecraft.world.entity.LivingEntity;
import javax.annotation.Nullable;
import net.minecraft.world.level.Level;
import net.minecraft.client.renderer.block.model.BakedQuad;
import java.util.List;
import com.mojang.blaze3d.vertex.VertexMultiConsumer;
import net.minecraft.client.renderer.RenderType;
import java.util.Objects;
import net.minecraft.client.renderer.Sheets;
import net.minecraft.client.renderer.ItemBlockRenderTypes;
import net.minecraft.client.renderer.BlockEntityWithoutLevelRenderer;
import net.minecraft.world.item.Items;
import net.minecraft.client.renderer.MultiBufferSource;
import net.minecraft.client.renderer.block.model.ItemTransforms;
import net.minecraft.world.level.block.state.BlockState;
import net.minecraft.core.Direction;
import java.util.Random;
import com.mojang.blaze3d.vertex.VertexConsumer;
import com.mojang.blaze3d.vertex.PoseStack;
import net.minecraft.world.item.ItemStack;
import net.minecraft.client.resources.model.BakedModel;
import java.util.Iterator;
import net.minecraft.client.resources.model.ModelResourceLocation;
import net.minecraft.core.Registry;
import net.minecraft.client.resources.model.ModelManager;
import net.minecraft.client.color.item.ItemColors;
import net.minecraft.client.renderer.texture.TextureManager;
import net.minecraft.client.renderer.ItemModelShaper;
import net.minecraft.world.item.Item;
import java.util.Set;
import net.minecraft.resources.ResourceLocation;
import net.minecraft.server.packs.resources.ResourceManagerReloadListener;

public class ItemRenderer implements ResourceManagerReloadListener {
    public static final ResourceLocation ENCHANT_GLINT_LOCATION;
    private static final Set<Item> IGNORED;
    public float blitOffset;
    private final ItemModelShaper itemModelShaper;
    private final TextureManager textureManager;
    private final ItemColors itemColors;
    
    public ItemRenderer(final TextureManager eah, final ModelManager ebx, final ItemColors dcd) {
        this.textureManager = eah;
        this.itemModelShaper = new ItemModelShaper(ebx);
        for (final Item bef6 : Registry.ITEM) {
            if (!ItemRenderer.IGNORED.contains(bef6)) {
                this.itemModelShaper.register(bef6, new ModelResourceLocation(Registry.ITEM.getKey(bef6), "inventory"));
            }
        }
        this.itemColors = dcd;
    }
    
    public ItemModelShaper getItemModelShaper() {
        return this.itemModelShaper;
    }
    
    private void renderModelLists(final BakedModel ebs, final ItemStack bek, final int integer3, final int integer4, final PoseStack cyi, final VertexConsumer cyl) {
        final Random random8 = new Random();
        final long long9 = 42L;
        for (final Direction fp14 : Direction.values()) {
            random8.setSeed(42L);
            this.renderQuadList(cyi, cyl, ebs.getQuads(null, fp14, random8), bek, integer3, integer4);
        }
        random8.setSeed(42L);
        this.renderQuadList(cyi, cyl, ebs.getQuads(null, null, random8), bek, integer3, integer4);
    }
    
    public void render(final ItemStack bek, final ItemTransforms.TransformType b, final boolean boolean3, final PoseStack cyi, final MultiBufferSource dqq, final int integer6, final int integer7, BakedModel ebs) {
        if (bek.isEmpty()) {
            return;
        }
        cyi.pushPose();
        final boolean boolean4 = b == ItemTransforms.TransformType.GUI;
        final boolean boolean5 = boolean4 || b == ItemTransforms.TransformType.GROUND || b == ItemTransforms.TransformType.FIXED;
        if (bek.getItem() == Items.TRIDENT && boolean5) {
            ebs = this.itemModelShaper.getModelManager().getModel(new ModelResourceLocation("minecraft:trident#inventory"));
        }
        ebs.getTransforms().getTransform(b).apply(boolean3, cyi);
        cyi.translate(-0.5, -0.5, -0.5);
        if (ebs.isCustomRenderer() || (bek.getItem() == Items.TRIDENT && !boolean5)) {
            BlockEntityWithoutLevelRenderer.instance.renderByItem(bek, cyi, dqq, integer6, integer7);
        }
        else {
            final RenderType dqy12 = ItemBlockRenderTypes.getRenderType(bek);
            RenderType dqy13;
            if (boolean4 && Objects.equals(dqy12, Sheets.translucentBlockSheet())) {
                dqy13 = Sheets.translucentCullBlockSheet();
            }
            else {
                dqy13 = dqy12;
            }
            final VertexConsumer cyl14 = getFoilBuffer(dqq, dqy13, true, bek.hasFoil());
            this.renderModelLists(ebs, bek, integer6, integer7, cyi, cyl14);
        }
        cyi.popPose();
    }
    
    public static VertexConsumer getFoilBuffer(final MultiBufferSource dqq, final RenderType dqy, final boolean boolean3, final boolean boolean4) {
        if (boolean4) {
            return VertexMultiConsumer.create(dqq.getBuffer(boolean3 ? RenderType.glint() : RenderType.entityGlint()), dqq.getBuffer(dqy));
        }
        return dqq.getBuffer(dqy);
    }
    
    private void renderQuadList(final PoseStack cyi, final VertexConsumer cyl, final List<BakedQuad> list, final ItemStack bek, final int integer5, final int integer6) {
        final boolean boolean8 = !bek.isEmpty();
        final PoseStack.Pose a9 = cyi.last();
        for (final BakedQuad drk11 : list) {
            int integer7 = -1;
            if (boolean8 && drk11.isTinted()) {
                integer7 = this.itemColors.getColor(bek, drk11.getTintIndex());
            }
            final float float13 = (integer7 >> 16 & 0xFF) / 255.0f;
            final float float14 = (integer7 >> 8 & 0xFF) / 255.0f;
            final float float15 = (integer7 & 0xFF) / 255.0f;
            cyl.putBulkData(a9, drk11, float13, float14, float15, integer5, integer6);
        }
    }
    
    public BakedModel getModel(final ItemStack bek, @Nullable final Level bjt, @Nullable final LivingEntity akw) {
        final Item bef6 = bek.getItem();
        BakedModel ebs5;
        if (bef6 == Items.TRIDENT) {
            ebs5 = this.itemModelShaper.getModelManager().getModel(new ModelResourceLocation("minecraft:trident_in_hand#inventory"));
        }
        else {
            ebs5 = this.itemModelShaper.getItemModel(bek);
        }
        if (!bef6.hasProperties()) {
            return ebs5;
        }
        return this.resolveOverrides(ebs5, bek, bjt, akw);
    }
    
    private BakedModel resolveOverrides(final BakedModel ebs, final ItemStack bek, @Nullable final Level bjt, @Nullable final LivingEntity akw) {
        final BakedModel ebs2 = ebs.getOverrides().resolve(ebs, bek, bjt, akw);
        return (ebs2 == null) ? this.itemModelShaper.getModelManager().getMissingModel() : ebs2;
    }
    
    public void renderStatic(final ItemStack bek, final ItemTransforms.TransformType b, final int integer3, final int integer4, final PoseStack cyi, final MultiBufferSource dqq) {
        this.renderStatic(null, bek, b, false, cyi, dqq, null, integer3, integer4);
    }
    
    public void renderStatic(@Nullable final LivingEntity akw, final ItemStack bek, final ItemTransforms.TransformType b, final boolean boolean4, final PoseStack cyi, final MultiBufferSource dqq, @Nullable final Level bjt, final int integer8, final int integer9) {
        if (bek.isEmpty()) {
            return;
        }
        final BakedModel ebs11 = this.getModel(bek, bjt, akw);
        this.render(bek, b, boolean4, cyi, dqq, integer8, integer9, ebs11);
    }
    
    public void renderGuiItem(final ItemStack bek, final int integer2, final int integer3) {
        this.renderGuiItem(bek, integer2, integer3, this.getModel(bek, null, null));
    }
    
    protected void renderGuiItem(final ItemStack bek, final int integer2, final int integer3, final BakedModel ebs) {
        RenderSystem.pushMatrix();
        this.textureManager.bind(TextureAtlas.LOCATION_BLOCKS);
        this.textureManager.getTexture(TextureAtlas.LOCATION_BLOCKS).setFilter(false, false);
        RenderSystem.enableRescaleNormal();
        RenderSystem.enableAlphaTest();
        RenderSystem.defaultAlphaFunc();
        RenderSystem.enableBlend();
        RenderSystem.blendFunc(GlStateManager.SourceFactor.SRC_ALPHA, GlStateManager.DestFactor.ONE_MINUS_SRC_ALPHA);
        RenderSystem.color4f(1.0f, 1.0f, 1.0f, 1.0f);
        RenderSystem.translatef((float)integer2, (float)integer3, 100.0f + this.blitOffset);
        RenderSystem.translatef(8.0f, 8.0f, 0.0f);
        RenderSystem.scalef(1.0f, -1.0f, 1.0f);
        RenderSystem.scalef(16.0f, 16.0f, 16.0f);
        final PoseStack cyi6 = new PoseStack();
        final MultiBufferSource.BufferSource a7 = Minecraft.getInstance().renderBuffers().bufferSource();
        final Item bef8 = bek.getItem();
        final boolean boolean9 = !ebs.isGui3d() || bef8 == Items.SHIELD || bef8 == Items.TRIDENT;
        if (boolean9) {
            Lighting.setupForFlatItems();
        }
        this.render(bek, ItemTransforms.TransformType.GUI, false, cyi6, a7, 15728880, OverlayTexture.NO_OVERLAY, ebs);
        a7.endBatch();
        RenderSystem.enableDepthTest();
        if (boolean9) {
            Lighting.setupFor3DItems();
        }
        RenderSystem.disableAlphaTest();
        RenderSystem.disableRescaleNormal();
        RenderSystem.popMatrix();
    }
    
    public void renderAndDecorateItem(final ItemStack bek, final int integer2, final int integer3) {
        this.renderAndDecorateItem(Minecraft.getInstance().player, bek, integer2, integer3);
    }
    
    public void renderAndDecorateItem(@Nullable final LivingEntity akw, final ItemStack bek, final int integer3, final int integer4) {
        if (bek.isEmpty()) {
            return;
        }
        this.blitOffset += 50.0f;
        try {
            this.renderGuiItem(bek, integer3, integer4, this.getModel(bek, null, akw));
        }
        catch (Throwable throwable6) {
            final CrashReport h7 = CrashReport.forThrowable(throwable6, "Rendering item");
            final CrashReportCategory i8 = h7.addCategory("Item being rendered");
            i8.setDetail("Item Type", () -> String.valueOf(bek.getItem()));
            i8.setDetail("Item Damage", () -> String.valueOf(bek.getDamageValue()));
            i8.setDetail("Item NBT", () -> String.valueOf(bek.getTag()));
            i8.setDetail("Item Foil", () -> String.valueOf(bek.hasFoil()));
            throw new ReportedException(h7);
        }
        this.blitOffset -= 50.0f;
    }
    
    public void renderGuiItemDecorations(final Font dcf, final ItemStack bek, final int integer3, final int integer4) {
        this.renderGuiItemDecorations(dcf, bek, integer3, integer4, null);
    }
    
    public void renderGuiItemDecorations(final Font dcf, final ItemStack bek, final int integer3, final int integer4, @Nullable final String string) {
        if (bek.isEmpty()) {
            return;
        }
        final PoseStack cyi7 = new PoseStack();
        if (bek.getCount() != 1 || string != null) {
            final String string2 = (string == null) ? String.valueOf(bek.getCount()) : string;
            cyi7.translate(0.0, 0.0, this.blitOffset + 200.0f);
            final MultiBufferSource.BufferSource a9 = MultiBufferSource.immediate(Tesselator.getInstance().getBuilder());
            dcf.drawInBatch(string2, (float)(integer3 + 19 - 2 - dcf.width(string2)), (float)(integer4 + 6 + 3), 16777215, true, cyi7.last().pose(), a9, false, 0, 15728880);
            a9.endBatch();
        }
        if (bek.isDamaged()) {
            RenderSystem.disableDepthTest();
            RenderSystem.disableTexture();
            RenderSystem.disableAlphaTest();
            RenderSystem.disableBlend();
            final Tesselator cyj8 = Tesselator.getInstance();
            final BufferBuilder cyd9 = cyj8.getBuilder();
            final float float10 = (float)bek.getDamageValue();
            final float float11 = (float)bek.getMaxDamage();
            final float float12 = Math.max(0.0f, (float11 - float10) / float11);
            final int integer5 = Math.round(13.0f - float10 * 13.0f / float11);
            final int integer6 = Mth.hsvToRgb(float12 / 3.0f, 1.0f, 1.0f);
            this.fillRect(cyd9, integer3 + 2, integer4 + 13, 13, 2, 0, 0, 0, 255);
            this.fillRect(cyd9, integer3 + 2, integer4 + 13, integer5, 1, integer6 >> 16 & 0xFF, integer6 >> 8 & 0xFF, integer6 & 0xFF, 255);
            RenderSystem.enableBlend();
            RenderSystem.enableAlphaTest();
            RenderSystem.enableTexture();
            RenderSystem.enableDepthTest();
        }
        final LocalPlayer dpy8 = Minecraft.getInstance().player;
        final float float13 = (dpy8 == null) ? 0.0f : dpy8.getCooldowns().getCooldownPercent(bek.getItem(), Minecraft.getInstance().getFrameTime());
        if (float13 > 0.0f) {
            RenderSystem.disableDepthTest();
            RenderSystem.disableTexture();
            RenderSystem.enableBlend();
            RenderSystem.defaultBlendFunc();
            final Tesselator cyj9 = Tesselator.getInstance();
            final BufferBuilder cyd10 = cyj9.getBuilder();
            this.fillRect(cyd10, integer3, integer4 + Mth.floor(16.0f * (1.0f - float13)), 16, Mth.ceil(16.0f * float13), 255, 255, 255, 127);
            RenderSystem.enableTexture();
            RenderSystem.enableDepthTest();
        }
    }
    
    private void fillRect(final BufferBuilder cyd, final int integer2, final int integer3, final int integer4, final int integer5, final int integer6, final int integer7, final int integer8, final int integer9) {
        cyd.begin(7, DefaultVertexFormat.POSITION_COLOR);
        cyd.vertex(integer2 + 0, integer3 + 0, 0.0).color(integer6, integer7, integer8, integer9).endVertex();
        cyd.vertex(integer2 + 0, integer3 + integer5, 0.0).color(integer6, integer7, integer8, integer9).endVertex();
        cyd.vertex(integer2 + integer4, integer3 + integer5, 0.0).color(integer6, integer7, integer8, integer9).endVertex();
        cyd.vertex(integer2 + integer4, integer3 + 0, 0.0).color(integer6, integer7, integer8, integer9).endVertex();
        Tesselator.getInstance().end();
    }
    
    @Override
    public void onResourceManagerReload(final ResourceManager zb) {
        this.itemModelShaper.rebuildCache();
    }
    
    static {
        ENCHANT_GLINT_LOCATION = new ResourceLocation("textures/misc/enchanted_item_glint.png");
        IGNORED = Sets.<Item>newHashSet(Items.AIR);
    }
}
