package net.minecraft.client.renderer.entity;

import org.apache.logging.log4j.LogManager;
import net.minecraft.world.scores.Team;
import net.minecraft.client.player.LocalPlayer;
import net.minecraft.world.entity.Entity;
import net.minecraft.world.entity.player.PlayerModelPart;
import net.minecraft.ChatFormatting;
import com.mojang.math.Vector3f;
import net.minecraft.client.renderer.texture.OverlayTexture;
import java.util.Iterator;
import com.mojang.blaze3d.vertex.VertexConsumer;
import net.minecraft.resources.ResourceLocation;
import net.minecraft.core.Direction;
import net.minecraft.client.renderer.RenderType;
import net.minecraft.world.entity.player.Player;
import net.minecraft.client.Minecraft;
import net.minecraft.world.entity.Pose;
import net.minecraft.util.Mth;
import net.minecraft.client.renderer.MultiBufferSource;
import com.mojang.blaze3d.vertex.PoseStack;
import com.google.common.collect.Lists;
import net.minecraft.client.renderer.entity.layers.RenderLayer;
import java.util.List;
import org.apache.logging.log4j.Logger;
import net.minecraft.client.model.EntityModel;
import net.minecraft.world.entity.LivingEntity;

public abstract class LivingEntityRenderer<T extends LivingEntity, M extends EntityModel<T>> extends EntityRenderer<T> implements RenderLayerParent<T, M> {
    private static final Logger LOGGER;
    protected M model;
    protected final List<RenderLayer<T, M>> layers;
    
    public LivingEntityRenderer(final EntityRenderDispatcher dvc, final M dkx, final float float3) {
        super(dvc);
        this.layers = Lists.newArrayList();
        this.model = dkx;
        this.shadowRadius = float3;
    }
    
    protected final boolean addLayer(final RenderLayer<T, M> dza) {
        return this.layers.add(dza);
    }
    
    @Override
    public M getModel() {
        return this.model;
    }
    
    @Override
    public void render(final T akw, final float float2, final float float3, final PoseStack cyi, final MultiBufferSource dqq, final int integer) {
        cyi.pushPose();
        this.model.attackTime = this.getAttackAnim(akw, float3);
        this.model.riding = akw.isPassenger();
        this.model.young = akw.isBaby();
        float float4 = Mth.rotLerp(float3, akw.yBodyRotO, akw.yBodyRot);
        final float float5 = Mth.rotLerp(float3, akw.yHeadRotO, akw.yHeadRot);
        float float6 = float5 - float4;
        if (akw.isPassenger() && akw.getVehicle() instanceof LivingEntity) {
            final LivingEntity akw2 = (LivingEntity)akw.getVehicle();
            float4 = Mth.rotLerp(float3, akw2.yBodyRotO, akw2.yBodyRot);
            float6 = float5 - float4;
            float float7 = Mth.wrapDegrees(float6);
            if (float7 < -85.0f) {
                float7 = -85.0f;
            }
            if (float7 >= 85.0f) {
                float7 = 85.0f;
            }
            float4 = float5 - float7;
            if (float7 * float7 > 2500.0f) {
                float4 += float7 * 0.2f;
            }
            float6 = float5 - float4;
        }
        final float float8 = Mth.lerp(float3, akw.xRotO, akw.xRot);
        if (akw.getPose() == Pose.SLEEPING) {
            final Direction fp12 = akw.getBedOrientation();
            if (fp12 != null) {
                final float float9 = akw.getEyeHeight(Pose.STANDING) - 0.1f;
                cyi.translate(-fp12.getStepX() * float9, 0.0, -fp12.getStepZ() * float9);
            }
        }
        float float7 = this.getBob(akw, float3);
        this.setupRotations(akw, cyi, float7, float4, float3);
        cyi.scale(-1.0f, -1.0f, 1.0f);
        this.scale(akw, cyi, float3);
        cyi.translate(0.0, -1.5010000467300415, 0.0);
        float float9 = 0.0f;
        float float10 = 0.0f;
        if (!akw.isPassenger() && akw.isAlive()) {
            float9 = Mth.lerp(float3, akw.animationSpeedOld, akw.animationSpeed);
            float10 = akw.animationPosition - akw.animationSpeed * (1.0f - float3);
            if (akw.isBaby()) {
                float10 *= 3.0f;
            }
            if (float9 > 1.0f) {
                float9 = 1.0f;
            }
        }
        this.model.prepareMobModel(akw, float10, float9, float3);
        final boolean boolean15 = akw.isGlowing();
        final boolean boolean16 = this.isVisible(akw, false);
        final boolean boolean17 = !boolean16 && !akw.isInvisibleTo(Minecraft.getInstance().player);
        this.model.setupAnim(akw, float10, float9, float7, float6, float8);
        final ResourceLocation sm18 = this.getTextureLocation(akw);
        RenderType dqy19;
        if (boolean17) {
            dqy19 = RenderType.entityTranslucent(sm18);
        }
        else if (boolean16) {
            dqy19 = this.model.renderType(sm18);
        }
        else {
            dqy19 = RenderType.outline(sm18);
        }
        if (boolean16 || boolean17 || boolean15) {
            final VertexConsumer cyl20 = dqq.getBuffer(dqy19);
            final int integer2 = getOverlayCoords(akw, this.getWhiteOverlayProgress(akw, float3));
            this.model.renderToBuffer(cyi, cyl20, integer, integer2, 1.0f, 1.0f, 1.0f, boolean17 ? 0.15f : 1.0f);
        }
        if (!akw.isSpectator()) {
            for (final RenderLayer<T, M> dza21 : this.layers) {
                dza21.render(cyi, dqq, integer, akw, float10, float9, float3, float7, float6, float8);
            }
        }
        cyi.popPose();
        super.render(akw, float2, float3, cyi, dqq, integer);
    }
    
    public static int getOverlayCoords(final LivingEntity akw, final float float2) {
        return OverlayTexture.pack(OverlayTexture.u(float2), OverlayTexture.v(akw.hurtTime > 0 || akw.deathTime > 0));
    }
    
    protected boolean isVisible(final T akw, final boolean boolean2) {
        return !akw.isInvisible() || boolean2;
    }
    
    private static float sleepDirectionToRotation(final Direction fp) {
        switch (fp) {
            case SOUTH: {
                return 90.0f;
            }
            case WEST: {
                return 0.0f;
            }
            case NORTH: {
                return 270.0f;
            }
            case EAST: {
                return 180.0f;
            }
            default: {
                return 0.0f;
            }
        }
    }
    
    protected void setupRotations(final T akw, final PoseStack cyi, final float float3, final float float4, final float float5) {
        final Pose alg7 = akw.getPose();
        if (alg7 != Pose.SLEEPING) {
            cyi.mulPose(Vector3f.YP.rotationDegrees(180.0f - float4));
        }
        if (akw.deathTime > 0) {
            float float6 = (akw.deathTime + float5 - 1.0f) / 20.0f * 1.6f;
            float6 = Mth.sqrt(float6);
            if (float6 > 1.0f) {
                float6 = 1.0f;
            }
            cyi.mulPose(Vector3f.ZP.rotationDegrees(float6 * this.getFlipDegrees(akw)));
        }
        else if (akw.isAutoSpinAttack()) {
            cyi.mulPose(Vector3f.XP.rotationDegrees(-90.0f - akw.xRot));
            cyi.mulPose(Vector3f.YP.rotationDegrees((akw.tickCount + float5) * -75.0f));
        }
        else if (alg7 == Pose.SLEEPING) {
            final Direction fp8 = akw.getBedOrientation();
            final float float7 = (fp8 != null) ? sleepDirectionToRotation(fp8) : float4;
            cyi.mulPose(Vector3f.YP.rotationDegrees(float7));
            cyi.mulPose(Vector3f.ZP.rotationDegrees(this.getFlipDegrees(akw)));
            cyi.mulPose(Vector3f.YP.rotationDegrees(270.0f));
        }
        else if (akw.hasCustomName() || akw instanceof Player) {
            final String string8 = ChatFormatting.stripFormatting(akw.getName().getString());
            if (("Dinnerbone".equals(string8) || "Grumm".equals(string8)) && (!(akw instanceof Player) || ((Player)akw).isModelPartShown(PlayerModelPart.CAPE))) {
                cyi.translate(0.0, akw.getBbHeight() + 0.1f, 0.0);
                cyi.mulPose(Vector3f.ZP.rotationDegrees(180.0f));
            }
        }
    }
    
    protected float getAttackAnim(final T akw, final float float2) {
        return akw.getAttackAnim(float2);
    }
    
    protected float getBob(final T akw, final float float2) {
        return akw.tickCount + float2;
    }
    
    protected float getFlipDegrees(final T akw) {
        return 90.0f;
    }
    
    protected float getWhiteOverlayProgress(final T akw, final float float2) {
        return 0.0f;
    }
    
    protected void scale(final T akw, final PoseStack cyi, final float float3) {
    }
    
    @Override
    protected boolean shouldShowName(final T akw) {
        final double double3 = this.entityRenderDispatcher.distanceToSqr(akw);
        final float float5 = akw.isDiscrete() ? 32.0f : 64.0f;
        if (double3 >= float5 * float5) {
            return false;
        }
        final Minecraft dbl6 = Minecraft.getInstance();
        final LocalPlayer dpy7 = dbl6.player;
        final boolean boolean8 = !akw.isInvisibleTo(dpy7);
        if (akw != dpy7) {
            final Team cwk9 = akw.getTeam();
            final Team cwk10 = dpy7.getTeam();
            if (cwk9 != null) {
                final Team.Visibility b11 = cwk9.getNameTagVisibility();
                switch (b11) {
                    case ALWAYS: {
                        return boolean8;
                    }
                    case NEVER: {
                        return false;
                    }
                    case HIDE_FOR_OTHER_TEAMS: {
                        return (cwk10 == null) ? boolean8 : (cwk9.isAlliedTo(cwk10) && (cwk9.canSeeFriendlyInvisibles() || boolean8));
                    }
                    case HIDE_FOR_OWN_TEAM: {
                        return (cwk10 == null) ? boolean8 : (!cwk9.isAlliedTo(cwk10) && boolean8);
                    }
                    default: {
                        return true;
                    }
                }
            }
        }
        return Minecraft.renderNames() && akw != dbl6.getCameraEntity() && boolean8 && !akw.isVehicle();
    }
    
    static {
        LOGGER = LogManager.getLogger();
    }
}
