package net.minecraft.client.renderer.entity;

import net.minecraft.world.entity.Entity;
import net.minecraft.client.Minecraft;
import com.mojang.blaze3d.vertex.VertexConsumer;
import net.minecraft.world.level.block.state.BlockState;
import net.minecraft.world.phys.Vec3;
import net.minecraft.client.renderer.texture.OverlayTexture;
import net.minecraft.world.level.block.RenderShape;
import com.mojang.math.Vector3f;
import net.minecraft.util.Mth;
import net.minecraft.client.renderer.MultiBufferSource;
import com.mojang.blaze3d.vertex.PoseStack;
import net.minecraft.client.model.MinecartModel;
import net.minecraft.client.model.EntityModel;
import net.minecraft.resources.ResourceLocation;
import net.minecraft.world.entity.vehicle.AbstractMinecart;

public class MinecartRenderer<T extends AbstractMinecart> extends EntityRenderer<T> {
    private static final ResourceLocation MINECART_LOCATION;
    protected final EntityModel<T> model;
    
    public MinecartRenderer(final EntityRenderDispatcher dvc) {
        super(dvc);
        this.model = new MinecartModel<T>();
        this.shadowRadius = 0.7f;
    }
    
    @Override
    public void render(final T azu, float float2, final float float3, final PoseStack cyi, final MultiBufferSource dqq, final int integer) {
        super.render(azu, float2, float3, cyi, dqq, integer);
        cyi.pushPose();
        long long8 = azu.getId() * 493286711L;
        long8 = long8 * long8 * 4392167121L + long8 * 98761L;
        final float float4 = (((long8 >> 16 & 0x7L) + 0.5f) / 8.0f - 0.5f) * 0.004f;
        final float float5 = (((long8 >> 20 & 0x7L) + 0.5f) / 8.0f - 0.5f) * 0.004f;
        final float float6 = (((long8 >> 24 & 0x7L) + 0.5f) / 8.0f - 0.5f) * 0.004f;
        cyi.translate(float4, float5, float6);
        final double double13 = Mth.lerp(float3, azu.xOld, azu.getX());
        final double double14 = Mth.lerp(float3, azu.yOld, azu.getY());
        final double double15 = Mth.lerp(float3, azu.zOld, azu.getZ());
        final double double16 = 0.30000001192092896;
        final Vec3 cvi21 = azu.getPos(double13, double14, double15);
        float float7 = Mth.lerp(float3, azu.xRotO, azu.xRot);
        if (cvi21 != null) {
            Vec3 cvi22 = azu.getPosOffs(double13, double14, double15, 0.30000001192092896);
            Vec3 cvi23 = azu.getPosOffs(double13, double14, double15, -0.30000001192092896);
            if (cvi22 == null) {
                cvi22 = cvi21;
            }
            if (cvi23 == null) {
                cvi23 = cvi21;
            }
            cyi.translate(cvi21.x - double13, (cvi22.y + cvi23.y) / 2.0 - double14, cvi21.z - double15);
            Vec3 cvi24 = cvi23.add(-cvi22.x, -cvi22.y, -cvi22.z);
            if (cvi24.length() != 0.0) {
                cvi24 = cvi24.normalize();
                float2 = (float)(Math.atan2(cvi24.z, cvi24.x) * 180.0 / 3.141592653589793);
                float7 = (float)(Math.atan(cvi24.y) * 73.0);
            }
        }
        cyi.translate(0.0, 0.375, 0.0);
        cyi.mulPose(Vector3f.YP.rotationDegrees(180.0f - float2));
        cyi.mulPose(Vector3f.ZP.rotationDegrees(-float7));
        final float float8 = azu.getHurtTime() - float3;
        float float9 = azu.getDamage() - float3;
        if (float9 < 0.0f) {
            float9 = 0.0f;
        }
        if (float8 > 0.0f) {
            cyi.mulPose(Vector3f.XP.rotationDegrees(Mth.sin(float8) * float8 * float9 / 10.0f * azu.getHurtDir()));
        }
        final int integer2 = azu.getDisplayOffset();
        final BlockState byg26 = azu.getDisplayBlockState();
        if (byg26.getRenderShape() != RenderShape.INVISIBLE) {
            cyi.pushPose();
            final float float10 = 0.75f;
            cyi.scale(0.75f, 0.75f, 0.75f);
            cyi.translate(-0.5, (integer2 - 8) / 16.0f, 0.5);
            cyi.mulPose(Vector3f.YP.rotationDegrees(90.0f));
            this.renderMinecartContents(azu, float3, byg26, cyi, dqq, integer);
            cyi.popPose();
        }
        cyi.scale(-1.0f, -1.0f, 1.0f);
        this.model.setupAnim(azu, 0.0f, 0.0f, -0.1f, 0.0f, 0.0f);
        final VertexConsumer cyl27 = dqq.getBuffer(this.model.renderType(this.getTextureLocation(azu)));
        this.model.renderToBuffer(cyi, cyl27, integer, OverlayTexture.NO_OVERLAY, 1.0f, 1.0f, 1.0f, 1.0f);
        cyi.popPose();
    }
    
    @Override
    public ResourceLocation getTextureLocation(final T azu) {
        return MinecartRenderer.MINECART_LOCATION;
    }
    
    protected void renderMinecartContents(final T azu, final float float2, final BlockState byg, final PoseStack cyi, final MultiBufferSource dqq, final int integer) {
        Minecraft.getInstance().getBlockRenderer().renderSingleBlock(byg, cyi, dqq, integer, OverlayTexture.NO_OVERLAY);
    }
    
    static {
        MINECART_LOCATION = new ResourceLocation("textures/entity/minecart.png");
    }
}
