package net.minecraft.client.renderer.entity;

import net.minecraft.world.entity.Entity;
import com.mojang.blaze3d.vertex.VertexConsumer;
import net.minecraft.client.renderer.texture.OverlayTexture;
import com.mojang.math.Vector3f;
import net.minecraft.util.Mth;
import net.minecraft.client.renderer.MultiBufferSource;
import com.mojang.blaze3d.vertex.PoseStack;
import net.minecraft.client.model.ShulkerBulletModel;
import net.minecraft.client.renderer.RenderType;
import net.minecraft.resources.ResourceLocation;
import net.minecraft.world.entity.projectile.ShulkerBullet;

public class ShulkerBulletRenderer extends EntityRenderer<ShulkerBullet> {
    private static final ResourceLocation TEXTURE_LOCATION;
    private static final RenderType RENDER_TYPE;
    private final ShulkerBulletModel<ShulkerBullet> model;
    
    public ShulkerBulletRenderer(final EntityRenderDispatcher dvc) {
        super(dvc);
        this.model = new ShulkerBulletModel<ShulkerBullet>();
    }
    
    @Override
    protected int getBlockLightLevel(final ShulkerBullet ayx, final float float2) {
        return 15;
    }
    
    @Override
    public void render(final ShulkerBullet ayx, final float float2, final float float3, final PoseStack cyi, final MultiBufferSource dqq, final int integer) {
        cyi.pushPose();
        final float float4 = Mth.rotlerp(ayx.yRotO, ayx.yRot, float3);
        final float float5 = Mth.lerp(float3, ayx.xRotO, ayx.xRot);
        final float float6 = ayx.tickCount + float3;
        cyi.translate(0.0, 0.15000000596046448, 0.0);
        cyi.mulPose(Vector3f.YP.rotationDegrees(Mth.sin(float6 * 0.1f) * 180.0f));
        cyi.mulPose(Vector3f.XP.rotationDegrees(Mth.cos(float6 * 0.1f) * 180.0f));
        cyi.mulPose(Vector3f.ZP.rotationDegrees(Mth.sin(float6 * 0.15f) * 360.0f));
        cyi.scale(-0.5f, -0.5f, 0.5f);
        this.model.setupAnim(ayx, 0.0f, 0.0f, 0.0f, float4, float5);
        final VertexConsumer cyl11 = dqq.getBuffer(this.model.renderType(ShulkerBulletRenderer.TEXTURE_LOCATION));
        this.model.renderToBuffer(cyi, cyl11, integer, OverlayTexture.NO_OVERLAY, 1.0f, 1.0f, 1.0f, 1.0f);
        cyi.scale(1.5f, 1.5f, 1.5f);
        final VertexConsumer cyl12 = dqq.getBuffer(ShulkerBulletRenderer.RENDER_TYPE);
        this.model.renderToBuffer(cyi, cyl12, integer, OverlayTexture.NO_OVERLAY, 1.0f, 1.0f, 1.0f, 0.15f);
        cyi.popPose();
        super.render(ayx, float2, float3, cyi, dqq, integer);
    }
    
    @Override
    public ResourceLocation getTextureLocation(final ShulkerBullet ayx) {
        return ShulkerBulletRenderer.TEXTURE_LOCATION;
    }
    
    static {
        TEXTURE_LOCATION = new ResourceLocation("textures/entity/shulker/spark.png");
        RENDER_TYPE = RenderType.entityTranslucent(ShulkerBulletRenderer.TEXTURE_LOCATION);
    }
}
