package net.minecraft.client.renderer.entity;

import net.minecraft.client.renderer.texture.TextureAtlas;
import net.minecraft.resources.ResourceLocation;
import net.minecraft.client.renderer.texture.OverlayTexture;
import net.minecraft.client.renderer.block.model.ItemTransforms;
import net.minecraft.world.entity.projectile.ItemSupplier;
import com.mojang.math.Vector3f;
import net.minecraft.client.renderer.MultiBufferSource;
import com.mojang.blaze3d.vertex.PoseStack;

public class ThrownItemRenderer<T extends Entity> extends EntityRenderer<T> {
    private final ItemRenderer itemRenderer;
    private final float scale;
    private final boolean fullBright;
    
    public ThrownItemRenderer(final EntityRenderDispatcher dvc, final ItemRenderer dvw, final float float3, final boolean boolean4) {
        super(dvc);
        this.itemRenderer = dvw;
        this.scale = float3;
        this.fullBright = boolean4;
    }
    
    public ThrownItemRenderer(final EntityRenderDispatcher dvc, final ItemRenderer dvw) {
        this(dvc, dvw, 1.0f, false);
    }
    
    @Override
    protected int getBlockLightLevel(final T akn, final float float2) {
        return this.fullBright ? 15 : super.getBlockLightLevel(akn, float2);
    }
    
    @Override
    public void render(final T akn, final float float2, final float float3, final PoseStack cyi, final MultiBufferSource dqq, final int integer) {
        cyi.pushPose();
        cyi.scale(this.scale, this.scale, this.scale);
        cyi.mulPose(this.entityRenderDispatcher.cameraOrientation());
        cyi.mulPose(Vector3f.YP.rotationDegrees(180.0f));
        this.itemRenderer.renderStatic(((ItemSupplier)akn).getItem(), ItemTransforms.TransformType.GROUND, integer, OverlayTexture.NO_OVERLAY, cyi, dqq);
        cyi.popPose();
        super.render(akn, float2, float3, cyi, dqq, integer);
    }
    
    @Override
    public ResourceLocation getTextureLocation(final Entity akn) {
        return TextureAtlas.LOCATION_BLOCKS;
    }
}
