package net.minecraft.client.renderer.entity.layers;

import com.google.common.collect.Maps;
import net.minecraft.world.entity.Entity;
import java.util.function.Function;
import com.mojang.blaze3d.vertex.VertexConsumer;
import net.minecraft.client.renderer.texture.OverlayTexture;
import net.minecraft.client.renderer.entity.ItemRenderer;
import net.minecraft.client.renderer.RenderType;
import javax.annotation.Nullable;
import net.minecraft.world.item.ItemStack;
import net.minecraft.world.item.DyeableArmorItem;
import net.minecraft.world.item.ArmorItem;
import net.minecraft.world.entity.EquipmentSlot;
import net.minecraft.client.renderer.MultiBufferSource;
import com.mojang.blaze3d.vertex.PoseStack;
import net.minecraft.client.renderer.entity.RenderLayerParent;
import net.minecraft.resources.ResourceLocation;
import java.util.Map;
import net.minecraft.client.model.HumanoidModel;
import net.minecraft.world.entity.LivingEntity;

public abstract class AbstractArmorLayer<T extends LivingEntity, M extends HumanoidModel<T>, A extends HumanoidModel<T>> extends RenderLayer<T, M> {
    protected final A innerModel;
    protected final A outerModel;
    private static final Map<String, ResourceLocation> ARMOR_LOCATION_CACHE;
    
    protected AbstractArmorLayer(final RenderLayerParent<T, M> dws, final A dlg2, final A dlg3) {
        super(dws);
        this.innerModel = dlg2;
        this.outerModel = dlg3;
    }
    
    @Override
    public void render(final PoseStack cyi, final MultiBufferSource dqq, final int integer, final T akw, final float float5, final float float6, final float float7, final float float8, final float float9, final float float10) {
        this.renderArmorPiece(cyi, dqq, akw, float5, float6, float7, float8, float9, float10, EquipmentSlot.CHEST, integer);
        this.renderArmorPiece(cyi, dqq, akw, float5, float6, float7, float8, float9, float10, EquipmentSlot.LEGS, integer);
        this.renderArmorPiece(cyi, dqq, akw, float5, float6, float7, float8, float9, float10, EquipmentSlot.FEET, integer);
        this.renderArmorPiece(cyi, dqq, akw, float5, float6, float7, float8, float9, float10, EquipmentSlot.HEAD, integer);
    }
    
    private void renderArmorPiece(final PoseStack cyi, final MultiBufferSource dqq, final T akw, final float float4, final float float5, final float float6, final float float7, final float float8, final float float9, final EquipmentSlot aks, final int integer) {
        final ItemStack bek13 = akw.getItemBySlot(aks);
        if (!(bek13.getItem() instanceof ArmorItem)) {
            return;
        }
        final ArmorItem bcd14 = (ArmorItem)bek13.getItem();
        if (bcd14.getSlot() != aks) {
            return;
        }
        final A dlg15 = this.getArmorModel(aks);
        this.getParentModel().copyPropertiesTo(dlg15);
        dlg15.prepareMobModel(akw, float4, float5, float6);
        this.setPartVisibility(dlg15, aks);
        dlg15.setupAnim(akw, float4, float5, float7, float8, float9);
        final boolean boolean16 = this.usesInnerModel(aks);
        final boolean boolean17 = bek13.hasFoil();
        if (bcd14 instanceof DyeableArmorItem) {
            final int integer2 = ((DyeableArmorItem)bcd14).getColor(bek13);
            final float float10 = (integer2 >> 16 & 0xFF) / 255.0f;
            final float float11 = (integer2 >> 8 & 0xFF) / 255.0f;
            final float float12 = (integer2 & 0xFF) / 255.0f;
            this.renderModel(cyi, dqq, integer, bcd14, boolean17, dlg15, boolean16, float10, float11, float12, null);
            this.renderModel(cyi, dqq, integer, bcd14, boolean17, dlg15, boolean16, 1.0f, 1.0f, 1.0f, "overlay");
        }
        else {
            this.renderModel(cyi, dqq, integer, bcd14, boolean17, dlg15, boolean16, 1.0f, 1.0f, 1.0f, null);
        }
    }
    
    private void renderModel(final PoseStack cyi, final MultiBufferSource dqq, final int integer, final ArmorItem bcd, final boolean boolean5, final A dlg, final boolean boolean7, final float float8, final float float9, final float float10, @Nullable final String string) {
        final VertexConsumer cyl13 = ItemRenderer.getFoilBuffer(dqq, RenderType.entityCutoutNoCull(this.getArmorLocation(bcd, boolean7, string)), false, boolean5);
        dlg.renderToBuffer(cyi, cyl13, integer, OverlayTexture.NO_OVERLAY, float8, float9, float10, 1.0f);
    }
    
    public A getArmorModel(final EquipmentSlot aks) {
        return this.usesInnerModel(aks) ? this.innerModel : this.outerModel;
    }
    
    private boolean usesInnerModel(final EquipmentSlot aks) {
        return aks == EquipmentSlot.LEGS;
    }
    
    private ResourceLocation getArmorLocation(final ArmorItem bcd, final boolean boolean2, @Nullable final String string) {
        final String string2 = "textures/models/armor/" + bcd.getMaterial().getName() + "_layer_" + (boolean2 ? 2 : 1) + ((string == null) ? "" : ("_" + string)) + ".png";
        return AbstractArmorLayer.ARMOR_LOCATION_CACHE.computeIfAbsent(string2, ResourceLocation::new);
    }
    
    protected abstract void setPartVisibility(final A dlg, final EquipmentSlot aks);
    
    protected abstract void hideAllArmor(final A dlg);
    
    static {
        ARMOR_LOCATION_CACHE = Maps.newHashMap();
    }
}
