package net.minecraft.client.renderer.entity.layers;

import net.minecraft.world.entity.Entity;
import com.mojang.blaze3d.vertex.VertexConsumer;
import net.minecraft.world.item.ItemStack;
import net.minecraft.client.renderer.texture.OverlayTexture;
import net.minecraft.client.renderer.entity.ItemRenderer;
import net.minecraft.world.entity.player.PlayerModelPart;
import net.minecraft.client.player.AbstractClientPlayer;
import net.minecraft.world.item.Items;
import net.minecraft.world.entity.EquipmentSlot;
import net.minecraft.client.renderer.MultiBufferSource;
import com.mojang.blaze3d.vertex.PoseStack;
import net.minecraft.client.renderer.entity.RenderLayerParent;
import net.minecraft.client.model.ElytraModel;
import net.minecraft.resources.ResourceLocation;
import net.minecraft.client.model.EntityModel;
import net.minecraft.world.entity.LivingEntity;

public class ElytraLayer<T extends LivingEntity, M extends EntityModel<T>> extends RenderLayer<T, M> {
    private static final ResourceLocation WINGS_LOCATION;
    private final ElytraModel<T> elytraModel;
    
    public ElytraLayer(final RenderLayerParent<T, M> dws) {
        super(dws);
        this.elytraModel = new ElytraModel<T>();
    }
    
    @Override
    public void render(final PoseStack cyi, final MultiBufferSource dqq, final int integer, final T akw, final float float5, final float float6, final float float7, final float float8, final float float9, final float float10) {
        final ItemStack bek12 = akw.getItemBySlot(EquipmentSlot.CHEST);
        if (bek12.getItem() != Items.ELYTRA) {
            return;
        }
        ResourceLocation sm13;
        if (akw instanceof AbstractClientPlayer) {
            final AbstractClientPlayer dpv14 = (AbstractClientPlayer)akw;
            if (dpv14.isElytraLoaded() && dpv14.getElytraTextureLocation() != null) {
                sm13 = dpv14.getElytraTextureLocation();
            }
            else if (dpv14.isCapeLoaded() && dpv14.getCloakTextureLocation() != null && dpv14.isModelPartShown(PlayerModelPart.CAPE)) {
                sm13 = dpv14.getCloakTextureLocation();
            }
            else {
                sm13 = ElytraLayer.WINGS_LOCATION;
            }
        }
        else {
            sm13 = ElytraLayer.WINGS_LOCATION;
        }
        cyi.pushPose();
        cyi.translate(0.0, 0.0, 0.125);
        this.getParentModel().copyPropertiesTo(this.elytraModel);
        this.elytraModel.setupAnim(akw, float5, float6, float8, float9, float10);
        final VertexConsumer cyl14 = ItemRenderer.getFoilBuffer(dqq, this.elytraModel.renderType(sm13), false, bek12.hasFoil());
        this.elytraModel.renderToBuffer(cyi, cyl14, integer, OverlayTexture.NO_OVERLAY, 1.0f, 1.0f, 1.0f, 1.0f);
        cyi.popPose();
    }
    
    static {
        WINGS_LOCATION = new ResourceLocation("textures/entity/elytra.png");
    }
}
