package net.minecraft.client.renderer.entity.player;

import com.mojang.math.Vector3f;
import net.minecraft.util.Mth;
import net.minecraft.client.renderer.texture.OverlayTexture;
import net.minecraft.client.renderer.RenderType;
import net.minecraft.client.model.geom.ModelPart;
import net.minecraft.world.scores.Score;
import net.minecraft.world.scores.Objective;
import net.minecraft.world.scores.Scoreboard;
import net.minecraft.world.entity.Entity;
import net.minecraft.resources.ResourceLocation;
import net.minecraft.world.item.CrossbowItem;
import net.minecraft.world.item.Items;
import net.minecraft.world.item.UseAnim;
import net.minecraft.world.item.ItemStack;
import net.minecraft.world.entity.HumanoidArm;
import net.minecraft.world.InteractionHand;
import net.minecraft.world.entity.player.PlayerModelPart;
import net.minecraft.world.phys.Vec3;
import net.minecraft.client.renderer.MultiBufferSource;
import com.mojang.blaze3d.vertex.PoseStack;
import net.minecraft.client.renderer.entity.layers.BeeStingerLayer;
import net.minecraft.client.renderer.entity.layers.SpinAttackEffectLayer;
import net.minecraft.world.entity.player.Player;
import net.minecraft.client.renderer.entity.layers.ParrotOnShoulderLayer;
import net.minecraft.client.renderer.entity.layers.ElytraLayer;
import net.minecraft.client.renderer.entity.layers.CustomHeadLayer;
import net.minecraft.client.renderer.entity.layers.CapeLayer;
import net.minecraft.client.renderer.entity.layers.Deadmau5EarsLayer;
import net.minecraft.client.renderer.entity.layers.ArrowLayer;
import net.minecraft.client.renderer.entity.layers.ItemInHandLayer;
import net.minecraft.client.renderer.entity.layers.RenderLayer;
import net.minecraft.world.entity.LivingEntity;
import net.minecraft.client.renderer.entity.RenderLayerParent;
import net.minecraft.client.renderer.entity.layers.HumanoidArmorLayer;
import net.minecraft.client.model.HumanoidModel;
import net.minecraft.client.renderer.entity.EntityRenderDispatcher;
import net.minecraft.client.model.PlayerModel;
import net.minecraft.client.player.AbstractClientPlayer;
import net.minecraft.client.renderer.entity.LivingEntityRenderer;

public class PlayerRenderer extends LivingEntityRenderer<AbstractClientPlayer, PlayerModel<AbstractClientPlayer>> {
    public PlayerRenderer(final EntityRenderDispatcher dvc) {
        this(dvc, false);
    }
    
    public PlayerRenderer(final EntityRenderDispatcher dvc, final boolean boolean2) {
        super(dvc, new PlayerModel(0.0f, boolean2), 0.5f);
        this.addLayer((RenderLayer<AbstractClientPlayer, PlayerModel<AbstractClientPlayer>>)new HumanoidArmorLayer((RenderLayerParent<LivingEntity, HumanoidModel>)this, new HumanoidModel(0.5f), new HumanoidModel(1.0f)));
        this.addLayer(new ItemInHandLayer<AbstractClientPlayer, PlayerModel<AbstractClientPlayer>>(this));
        this.addLayer(new ArrowLayer<AbstractClientPlayer, PlayerModel<AbstractClientPlayer>>(this));
        this.addLayer(new Deadmau5EarsLayer(this));
        this.addLayer(new CapeLayer(this));
        this.addLayer(new CustomHeadLayer<AbstractClientPlayer, PlayerModel<AbstractClientPlayer>>(this));
        this.addLayer(new ElytraLayer<AbstractClientPlayer, PlayerModel<AbstractClientPlayer>>(this));
        this.addLayer((RenderLayer<AbstractClientPlayer, PlayerModel<AbstractClientPlayer>>)new ParrotOnShoulderLayer((RenderLayerParent<Player, PlayerModel<Player>>)this));
        this.addLayer((RenderLayer<AbstractClientPlayer, PlayerModel<AbstractClientPlayer>>)new SpinAttackEffectLayer((RenderLayerParent<LivingEntity, PlayerModel<LivingEntity>>)this));
        this.addLayer(new BeeStingerLayer<AbstractClientPlayer, PlayerModel<AbstractClientPlayer>>(this));
    }
    
    @Override
    public void render(final AbstractClientPlayer dpv, final float float2, final float float3, final PoseStack cyi, final MultiBufferSource dqq, final int integer) {
        this.setModelProperties(dpv);
        super.render(dpv, float2, float3, cyi, dqq, integer);
    }
    
    public Vec3 getRenderOffset(final AbstractClientPlayer dpv, final float float2) {
        if (dpv.isCrouching()) {
            return new Vec3(0.0, -0.125, 0.0);
        }
        return super.getRenderOffset((T)dpv, float2);
    }
    
    private void setModelProperties(final AbstractClientPlayer dpv) {
        final PlayerModel<AbstractClientPlayer> dlw3 = ((LivingEntityRenderer<T, PlayerModel<AbstractClientPlayer>>)this).getModel();
        if (dpv.isSpectator()) {
            dlw3.setAllVisible(false);
            dlw3.head.visible = true;
            dlw3.hat.visible = true;
        }
        else {
            final ItemStack bek4 = dpv.getMainHandItem();
            final ItemStack bek5 = dpv.getOffhandItem();
            dlw3.setAllVisible(true);
            dlw3.hat.visible = dpv.isModelPartShown(PlayerModelPart.HAT);
            dlw3.jacket.visible = dpv.isModelPartShown(PlayerModelPart.JACKET);
            dlw3.leftPants.visible = dpv.isModelPartShown(PlayerModelPart.LEFT_PANTS_LEG);
            dlw3.rightPants.visible = dpv.isModelPartShown(PlayerModelPart.RIGHT_PANTS_LEG);
            dlw3.leftSleeve.visible = dpv.isModelPartShown(PlayerModelPart.LEFT_SLEEVE);
            dlw3.rightSleeve.visible = dpv.isModelPartShown(PlayerModelPart.RIGHT_SLEEVE);
            dlw3.crouching = dpv.isCrouching();
            final HumanoidModel.ArmPose a6 = this.getArmPose(dpv, bek4, bek5, InteractionHand.MAIN_HAND);
            final HumanoidModel.ArmPose a7 = this.getArmPose(dpv, bek4, bek5, InteractionHand.OFF_HAND);
            if (dpv.getMainArm() == HumanoidArm.RIGHT) {
                dlw3.rightArmPose = a6;
                dlw3.leftArmPose = a7;
            }
            else {
                dlw3.rightArmPose = a7;
                dlw3.leftArmPose = a6;
            }
        }
    }
    
    private HumanoidModel.ArmPose getArmPose(final AbstractClientPlayer dpv, final ItemStack bek2, final ItemStack bek3, final InteractionHand ajh) {
        HumanoidModel.ArmPose a6 = HumanoidModel.ArmPose.EMPTY;
        final ItemStack bek4 = (ajh == InteractionHand.MAIN_HAND) ? bek2 : bek3;
        if (!bek4.isEmpty()) {
            a6 = HumanoidModel.ArmPose.ITEM;
            if (dpv.getUseItemRemainingTicks() > 0) {
                final UseAnim bfv8 = bek4.getUseAnimation();
                if (bfv8 == UseAnim.BLOCK) {
                    a6 = HumanoidModel.ArmPose.BLOCK;
                }
                else if (bfv8 == UseAnim.BOW) {
                    a6 = HumanoidModel.ArmPose.BOW_AND_ARROW;
                }
                else if (bfv8 == UseAnim.SPEAR) {
                    a6 = HumanoidModel.ArmPose.THROW_SPEAR;
                }
                else if (bfv8 == UseAnim.CROSSBOW && ajh == dpv.getUsedItemHand()) {
                    a6 = HumanoidModel.ArmPose.CROSSBOW_CHARGE;
                }
            }
            else {
                final boolean boolean8 = bek2.getItem() == Items.CROSSBOW;
                final boolean boolean9 = CrossbowItem.isCharged(bek2);
                final boolean boolean10 = bek3.getItem() == Items.CROSSBOW;
                final boolean boolean11 = CrossbowItem.isCharged(bek3);
                if (boolean8 && boolean9) {
                    a6 = HumanoidModel.ArmPose.CROSSBOW_HOLD;
                }
                if (boolean10 && boolean11 && bek2.getItem().getUseAnimation(bek2) == UseAnim.NONE) {
                    a6 = HumanoidModel.ArmPose.CROSSBOW_HOLD;
                }
            }
        }
        return a6;
    }
    
    @Override
    public ResourceLocation getTextureLocation(final AbstractClientPlayer dpv) {
        return dpv.getSkinTextureLocation();
    }
    
    @Override
    protected void scale(final AbstractClientPlayer dpv, final PoseStack cyi, final float float3) {
        final float float4 = 0.9375f;
        cyi.scale(0.9375f, 0.9375f, 0.9375f);
    }
    
    protected void renderNameTag(final AbstractClientPlayer dpv, final String string, final PoseStack cyi, final MultiBufferSource dqq, final int integer) {
        final double double7 = this.entityRenderDispatcher.distanceToSqr(dpv);
        cyi.pushPose();
        if (double7 < 100.0) {
            final Scoreboard cwi9 = dpv.getScoreboard();
            final Objective cwf10 = cwi9.getDisplayObjective(2);
            if (cwf10 != null) {
                final Score cwh11 = cwi9.getOrCreatePlayerScore(dpv.getScoreboardName(), cwf10);
                super.renderNameTag((T)dpv, cwh11.getScore() + " " + cwf10.getDisplayName().getColoredString(), cyi, dqq, integer);
                final double double8 = 0.0;
                this.getFont().getClass();
                cyi.translate(double8, 9.0f * 1.15f * 0.025f, 0.0);
            }
        }
        super.renderNameTag((T)dpv, string, cyi, dqq, integer);
        cyi.popPose();
    }
    
    public void renderRightHand(final PoseStack cyi, final MultiBufferSource dqq, final int integer, final AbstractClientPlayer dpv) {
        this.renderHand(cyi, dqq, integer, dpv, ((PlayerModel)this.model).rightArm, ((PlayerModel)this.model).rightSleeve);
    }
    
    public void renderLeftHand(final PoseStack cyi, final MultiBufferSource dqq, final int integer, final AbstractClientPlayer dpv) {
        this.renderHand(cyi, dqq, integer, dpv, ((PlayerModel)this.model).leftArm, ((PlayerModel)this.model).leftSleeve);
    }
    
    private void renderHand(final PoseStack cyi, final MultiBufferSource dqq, final int integer, final AbstractClientPlayer dpv, final ModelPart dnf5, final ModelPart dnf6) {
        final PlayerModel<AbstractClientPlayer> dlw8 = ((LivingEntityRenderer<T, PlayerModel<AbstractClientPlayer>>)this).getModel();
        this.setModelProperties(dpv);
        dlw8.attackTime = 0.0f;
        dlw8.crouching = false;
        dlw8.setupAnim(dpv, dlw8.swimAmount = 0.0f, 0.0f, 0.0f, 0.0f, 0.0f);
        dnf5.xRot = 0.0f;
        dnf5.render(cyi, dqq.getBuffer(RenderType.entitySolid(dpv.getSkinTextureLocation())), integer, OverlayTexture.NO_OVERLAY);
        dnf6.xRot = 0.0f;
        dnf6.render(cyi, dqq.getBuffer(RenderType.entityTranslucent(dpv.getSkinTextureLocation())), integer, OverlayTexture.NO_OVERLAY);
    }
    
    @Override
    protected void setupRotations(final AbstractClientPlayer dpv, final PoseStack cyi, final float float3, final float float4, final float float5) {
        final float float6 = dpv.getSwimAmount(float5);
        if (dpv.isFallFlying()) {
            super.setupRotations(dpv, cyi, float3, float4, float5);
            final float float7 = dpv.getFallFlyingTicks() + float5;
            final float float8 = Mth.clamp(float7 * float7 / 100.0f, 0.0f, 1.0f);
            if (!dpv.isAutoSpinAttack()) {
                cyi.mulPose(Vector3f.XP.rotationDegrees(float8 * (-90.0f - dpv.xRot)));
            }
            final Vec3 cvi10 = dpv.getViewVector(float5);
            final Vec3 cvi11 = dpv.getDeltaMovement();
            final double double12 = Entity.getHorizontalDistanceSqr(cvi11);
            final double double13 = Entity.getHorizontalDistanceSqr(cvi10);
            if (double12 > 0.0 && double13 > 0.0) {
                final double double14 = (cvi11.x * cvi10.x + cvi11.z * cvi10.z) / (Math.sqrt(double12) * Math.sqrt(double13));
                final double double15 = cvi11.x * cvi10.z - cvi11.z * cvi10.x;
                cyi.mulPose(Vector3f.YP.rotation((float)(Math.signum(double15) * Math.acos(double14))));
            }
        }
        else if (float6 > 0.0f) {
            super.setupRotations(dpv, cyi, float3, float4, float5);
            final float float7 = dpv.isInWater() ? (-90.0f - dpv.xRot) : -90.0f;
            final float float8 = Mth.lerp(float6, 0.0f, float7);
            cyi.mulPose(Vector3f.XP.rotationDegrees(float8));
            if (dpv.isVisuallySwimming()) {
                cyi.translate(0.0, -1.0, 0.30000001192092896);
            }
        }
        else {
            super.setupRotations(dpv, cyi, float3, float4, float5);
        }
    }
}
