package net.minecraft.client.resources.model;

import net.minecraft.client.renderer.texture.TextureAtlas;
import net.minecraft.world.level.material.FluidState;
import net.minecraft.util.profiling.ProfilerFiller;
import net.minecraft.server.packs.resources.ResourceManager;
import net.minecraft.world.level.block.state.BlockState;
import it.unimi.dsi.fastutil.objects.Object2IntMap;
import net.minecraft.client.color.block.BlockColors;
import net.minecraft.client.renderer.texture.TextureManager;
import net.minecraft.client.renderer.block.BlockModelShaper;
import net.minecraft.client.renderer.texture.AtlasSet;
import net.minecraft.resources.ResourceLocation;
import java.util.Map;
import net.minecraft.server.packs.resources.SimplePreparableReloadListener;

public class ModelManager extends SimplePreparableReloadListener<ModelBakery> implements AutoCloseable {
    private Map<ResourceLocation, BakedModel> bakedRegistry;
    private AtlasSet atlases;
    private final BlockModelShaper blockModelShaper;
    private final TextureManager textureManager;
    private final BlockColors blockColors;
    private int maxMipmapLevels;
    private BakedModel missingModel;
    private Object2IntMap<BlockState> modelGroups;
    
    public ModelManager(final TextureManager eah, final BlockColors dbz, final int integer) {
        this.textureManager = eah;
        this.blockColors = dbz;
        this.maxMipmapLevels = integer;
        this.blockModelShaper = new BlockModelShaper(this);
    }
    
    public BakedModel getModel(final ModelResourceLocation eby) {
        return this.bakedRegistry.getOrDefault(eby, this.missingModel);
    }
    
    public BakedModel getMissingModel() {
        return this.missingModel;
    }
    
    public BlockModelShaper getBlockModelShaper() {
        return this.blockModelShaper;
    }
    
    @Override
    protected ModelBakery prepare(final ResourceManager zb, final ProfilerFiller aim) {
        aim.startTick();
        final ModelBakery ebw4 = new ModelBakery(zb, this.blockColors, aim, this.maxMipmapLevels);
        aim.endTick();
        return ebw4;
    }
    
    @Override
    protected void apply(final ModelBakery ebw, final ResourceManager zb, final ProfilerFiller aim) {
        aim.startTick();
        aim.push("upload");
        this.atlases = ebw.uploadTextures(this.textureManager, aim);
        this.bakedRegistry = ebw.getBakedTopLevelModels();
        this.modelGroups = ebw.getModelGroups();
        this.missingModel = this.bakedRegistry.get(ModelBakery.MISSING_MODEL_LOCATION);
        aim.popPush("cache");
        this.blockModelShaper.rebuildCache();
        aim.pop();
        aim.endTick();
    }
    
    public boolean requiresRender(final BlockState byg1, final BlockState byg2) {
        if (byg1 == byg2) {
            return false;
        }
        final int integer4 = this.modelGroups.getInt(byg1);
        if (integer4 != -1) {
            final int integer5 = this.modelGroups.getInt(byg2);
            if (integer4 == integer5) {
                final FluidState cog6 = byg1.getFluidState();
                final FluidState cog7 = byg2.getFluidState();
                return cog6 != cog7;
            }
        }
        return true;
    }
    
    public TextureAtlas getAtlas(final ResourceLocation sm) {
        return this.atlases.getAtlas(sm);
    }
    
    @Override
    public void close() {
        this.atlases.close();
    }
    
    public void updateMaxMipLevel(final int integer) {
        this.maxMipmapLevels = integer;
    }
}
