package net.minecraft.client.resources.model;

import com.google.common.collect.Maps;
import com.google.common.collect.Lists;
import net.minecraft.client.renderer.block.model.BlockModel;
import java.util.Random;
import javax.annotation.Nullable;
import net.minecraft.world.level.block.state.BlockState;
import net.minecraft.client.renderer.block.model.ItemOverrides;
import net.minecraft.client.renderer.block.model.ItemTransforms;
import net.minecraft.client.renderer.texture.TextureAtlasSprite;
import net.minecraft.core.Direction;
import java.util.Map;
import net.minecraft.client.renderer.block.model.BakedQuad;
import java.util.List;

public class SimpleBakedModel implements BakedModel {
    protected final List<BakedQuad> unculledFaces;
    protected final Map<Direction, List<BakedQuad>> culledFaces;
    protected final boolean hasAmbientOcclusion;
    protected final boolean isGui3d;
    protected final TextureAtlasSprite particleIcon;
    protected final ItemTransforms transforms;
    protected final ItemOverrides overrides;
    
    public SimpleBakedModel(final List<BakedQuad> list, final Map<Direction, List<BakedQuad>> map, final boolean boolean3, final boolean boolean4, final TextureAtlasSprite eag, final ItemTransforms drw, final ItemOverrides dru) {
        this.unculledFaces = list;
        this.culledFaces = map;
        this.hasAmbientOcclusion = boolean3;
        this.isGui3d = boolean4;
        this.particleIcon = eag;
        this.transforms = drw;
        this.overrides = dru;
    }
    
    @Override
    public List<BakedQuad> getQuads(@Nullable final BlockState byg, @Nullable final Direction fp, final Random random) {
        return (fp == null) ? this.unculledFaces : this.culledFaces.get(fp);
    }
    
    @Override
    public boolean useAmbientOcclusion() {
        return this.hasAmbientOcclusion;
    }
    
    @Override
    public boolean isGui3d() {
        return this.isGui3d;
    }
    
    @Override
    public boolean isCustomRenderer() {
        return false;
    }
    
    @Override
    public TextureAtlasSprite getParticleIcon() {
        return this.particleIcon;
    }
    
    @Override
    public ItemTransforms getTransforms() {
        return this.transforms;
    }
    
    @Override
    public ItemOverrides getOverrides() {
        return this.overrides;
    }
    
    public static class Builder {
        private final List<BakedQuad> unculledFaces;
        private final Map<Direction, List<BakedQuad>> culledFaces;
        private final ItemOverrides overrides;
        private final boolean hasAmbientOcclusion;
        private TextureAtlasSprite particleIcon;
        private final boolean isGui3d;
        private final ItemTransforms transforms;
        
        public Builder(final BlockModel drp, final ItemOverrides dru) {
            this(drp.hasAmbientOcclusion(), drp.isGui3d(), drp.getTransforms(), dru);
        }
        
        private Builder(final boolean boolean1, final boolean boolean2, final ItemTransforms drw, final ItemOverrides dru) {
            this.unculledFaces = Lists.newArrayList();
            this.culledFaces = Maps.newEnumMap(Direction.class);
            for (final Direction fp9 : Direction.values()) {
                this.culledFaces.put(fp9, Lists.newArrayList());
            }
            this.overrides = dru;
            this.hasAmbientOcclusion = boolean1;
            this.isGui3d = boolean2;
            this.transforms = drw;
        }
        
        public Builder addCulledFace(final Direction fp, final BakedQuad drk) {
            this.culledFaces.get(fp).add(drk);
            return this;
        }
        
        public Builder addUnculledFace(final BakedQuad drk) {
            this.unculledFaces.add(drk);
            return this;
        }
        
        public Builder particle(final TextureAtlasSprite eag) {
            this.particleIcon = eag;
            return this;
        }
        
        public BakedModel build() {
            if (this.particleIcon == null) {
                throw new RuntimeException("Missing particle!");
            }
            return new SimpleBakedModel(this.unculledFaces, this.culledFaces, this.hasAmbientOcclusion, this.isGui3d, this.particleIcon, this.transforms, this.overrides);
        }
    }
}
