package net.minecraft.core.dispenser;

import net.minecraft.world.level.block.state.AbstractStateHolder;
import java.util.Iterator;
import java.util.List;
import net.minecraft.world.phys.AABB;
import net.minecraft.world.entity.animal.Sheep;
import net.minecraft.world.item.alchemy.PotionUtils;
import net.minecraft.world.item.alchemy.Potions;
import net.minecraft.tags.FluidTags;
import net.minecraft.world.level.block.entity.BeehiveBlockEntity;
import net.minecraft.world.level.block.BeehiveBlock;
import net.minecraft.tags.BlockTags;
import net.minecraft.world.level.block.ShulkerBoxBlock;
import net.minecraft.world.item.DyeColor;
import net.minecraft.world.level.LevelReader;
import net.minecraft.world.level.block.CarvedPumpkinBlock;
import net.minecraft.world.level.block.entity.BlockEntity;
import net.minecraft.world.level.block.entity.SkullBlockEntity;
import net.minecraft.world.level.block.SkullBlock;
import net.minecraft.world.level.block.WitherSkullBlock;
import net.minecraft.world.item.ArmorItem;
import net.minecraft.sounds.SoundSource;
import net.minecraft.sounds.SoundEvents;
import net.minecraft.world.entity.LivingEntity;
import net.minecraft.world.entity.item.PrimedTnt;
import net.minecraft.world.item.BoneMealItem;
import net.minecraft.server.level.ServerPlayer;
import net.minecraft.world.level.block.TntBlock;
import net.minecraft.world.level.block.state.properties.BlockStateProperties;
import net.minecraft.world.level.block.Blocks;
import net.minecraft.world.item.FlintAndSteelItem;
import net.minecraft.world.item.Item;
import net.minecraft.world.level.material.Fluid;
import net.minecraft.world.level.block.Block;
import net.minecraft.world.level.block.state.BlockState;
import net.minecraft.world.level.LevelAccessor;
import net.minecraft.world.level.block.entity.DispenserBlockEntity;
import net.minecraft.world.level.material.FlowingFluid;
import net.minecraft.world.level.block.BucketPickup;
import net.minecraft.world.phys.BlockHitResult;
import net.minecraft.world.item.BucketItem;
import net.minecraft.world.entity.vehicle.Boat;
import java.util.Random;
import net.minecraft.world.entity.projectile.SmallFireball;
import net.minecraft.world.entity.projectile.FireworkRocketEntity;
import net.minecraft.core.BlockPos;
import net.minecraft.world.entity.Entity;
import net.minecraft.world.entity.decoration.ArmorStand;
import net.minecraft.world.entity.EntityType;
import net.minecraft.world.entity.player.Player;
import net.minecraft.world.entity.MobSpawnType;
import net.minecraft.world.item.SpawnEggItem;
import net.minecraft.world.level.block.state.properties.Property;
import net.minecraft.core.Direction;
import net.minecraft.world.entity.projectile.ThrownPotion;
import net.minecraft.world.entity.projectile.ThrownExperienceBottle;
import net.minecraft.world.entity.projectile.Snowball;
import net.minecraft.Util;
import net.minecraft.world.entity.projectile.ThrownEgg;
import net.minecraft.world.entity.projectile.SpectralArrow;
import net.minecraft.world.level.ItemLike;
import net.minecraft.world.level.block.DispenserBlock;
import net.minecraft.world.entity.projectile.AbstractArrow;
import net.minecraft.world.entity.projectile.Arrow;
import net.minecraft.world.entity.projectile.Projectile;
import net.minecraft.core.Position;
import net.minecraft.world.level.Level;
import net.minecraft.world.item.Items;
import net.minecraft.world.item.ItemStack;
import net.minecraft.core.BlockSource;

public interface DispenseItemBehavior {
    public static final DispenseItemBehavior NOOP = (fl, bek) -> bek;
    
    ItemStack dispense(final BlockSource fl, final ItemStack bek);
    
    default void bootStrap() {
        DispenserBlock.registerBehavior(Items.ARROW, new AbstractProjectileDispenseBehavior() {
            @Override
            protected Projectile getProjectile(final Level bjt, final Position fz, final ItemStack bek) {
                final Arrow aym5 = new Arrow(bjt, fz.x(), fz.y(), fz.z());
                aym5.pickup = AbstractArrow.Pickup.ALLOWED;
                return aym5;
            }
        });
        DispenserBlock.registerBehavior(Items.TIPPED_ARROW, new AbstractProjectileDispenseBehavior() {
            @Override
            protected Projectile getProjectile(final Level bjt, final Position fz, final ItemStack bek) {
                final Arrow aym5 = new Arrow(bjt, fz.x(), fz.y(), fz.z());
                aym5.setEffectsFromItem(bek);
                aym5.pickup = AbstractArrow.Pickup.ALLOWED;
                return aym5;
            }
        });
        DispenserBlock.registerBehavior(Items.SPECTRAL_ARROW, new AbstractProjectileDispenseBehavior() {
            @Override
            protected Projectile getProjectile(final Level bjt, final Position fz, final ItemStack bek) {
                final AbstractArrow ayk5 = new SpectralArrow(bjt, fz.x(), fz.y(), fz.z());
                ayk5.pickup = AbstractArrow.Pickup.ALLOWED;
                return ayk5;
            }
        });
        DispenserBlock.registerBehavior(Items.EGG, new AbstractProjectileDispenseBehavior() {
            @Override
            protected Projectile getProjectile(final Level bjt, final Position fz, final ItemStack bek) {
                return Util.<ThrownEgg>make(new ThrownEgg(bjt, fz.x(), fz.y(), fz.z()), azd -> azd.setItem(bek));
            }
        });
        DispenserBlock.registerBehavior(Items.SNOWBALL, new AbstractProjectileDispenseBehavior() {
            @Override
            protected Projectile getProjectile(final Level bjt, final Position fz, final ItemStack bek) {
                return Util.<Snowball>make(new Snowball(bjt, fz.x(), fz.y(), fz.z()), ayz -> ayz.setItem(bek));
            }
        });
        DispenserBlock.registerBehavior(Items.EXPERIENCE_BOTTLE, new AbstractProjectileDispenseBehavior() {
            @Override
            protected Projectile getProjectile(final Level bjt, final Position fz, final ItemStack bek) {
                return Util.<ThrownExperienceBottle>make(new ThrownExperienceBottle(bjt, fz.x(), fz.y(), fz.z()), azf -> azf.setItem(bek));
            }
            
            @Override
            protected float getUncertainty() {
                return super.getUncertainty() * 0.5f;
            }
            
            @Override
            protected float getPower() {
                return super.getPower() * 1.25f;
            }
        });
        DispenserBlock.registerBehavior(Items.SPLASH_POTION, new DispenseItemBehavior() {
            @Override
            public ItemStack dispense(final BlockSource fl, final ItemStack bek) {
                return new AbstractProjectileDispenseBehavior() {
                    @Override
                    protected Projectile getProjectile(final Level bjt, final Position fz, final ItemStack bek) {
                        return Util.<ThrownPotion>make(new ThrownPotion(bjt, fz.x(), fz.y(), fz.z()), azg -> azg.setItem(bek));
                    }
                    
                    @Override
                    protected float getUncertainty() {
                        return super.getUncertainty() * 0.5f;
                    }
                    
                    @Override
                    protected float getPower() {
                        return super.getPower() * 1.25f;
                    }
                }.dispense(fl, bek);
            }
        });
        DispenserBlock.registerBehavior(Items.LINGERING_POTION, new DispenseItemBehavior() {
            @Override
            public ItemStack dispense(final BlockSource fl, final ItemStack bek) {
                return new AbstractProjectileDispenseBehavior() {
                    @Override
                    protected Projectile getProjectile(final Level bjt, final Position fz, final ItemStack bek) {
                        return Util.<ThrownPotion>make(new ThrownPotion(bjt, fz.x(), fz.y(), fz.z()), azg -> azg.setItem(bek));
                    }
                    
                    @Override
                    protected float getUncertainty() {
                        return super.getUncertainty() * 0.5f;
                    }
                    
                    @Override
                    protected float getPower() {
                        return super.getPower() * 1.25f;
                    }
                }.dispense(fl, bek);
            }
        });
        final DefaultDispenseItemBehavior gk1 = new DefaultDispenseItemBehavior() {
            public ItemStack execute(final BlockSource fl, final ItemStack bek) {
                final Direction fp4 = fl.getBlockState().<Direction>getValue((Property<Direction>)DispenserBlock.FACING);
                final EntityType<?> akr5 = ((SpawnEggItem)bek.getItem()).getType(bek.getTag());
                akr5.spawn(fl.getLevel(), bek, null, fl.getPos().relative(fp4), MobSpawnType.DISPENSER, fp4 != Direction.UP, false);
                bek.shrink(1);
                return bek;
            }
        };
        for (final SpawnEggItem bfi3 : SpawnEggItem.eggs()) {
            DispenserBlock.registerBehavior(bfi3, gk1);
        }
        DispenserBlock.registerBehavior(Items.ARMOR_STAND, new DefaultDispenseItemBehavior() {
            public ItemStack execute(final BlockSource fl, final ItemStack bek) {
                final Direction fp4 = fl.getBlockState().<Direction>getValue((Property<Direction>)DispenserBlock.FACING);
                final BlockPos fk5 = fl.getPos().relative(fp4);
                final Level bjt6 = fl.getLevel();
                final ArmorStand avm7 = new ArmorStand(bjt6, fk5.getX() + 0.5, fk5.getY(), fk5.getZ() + 0.5);
                EntityType.updateCustomEntityTag(bjt6, null, avm7, bek.getTag());
                avm7.yRot = fp4.toYRot();
                bjt6.addFreshEntity(avm7);
                bek.shrink(1);
                return bek;
            }
        });
        DispenserBlock.registerBehavior(Items.FIREWORK_ROCKET, new DefaultDispenseItemBehavior() {
            public ItemStack execute(final BlockSource fl, final ItemStack bek) {
                final Direction fp4 = fl.getBlockState().<Direction>getValue((Property<Direction>)DispenserBlock.FACING);
                final double double5 = fp4.getStepX();
                final double double6 = fp4.getStepY();
                final double double7 = fp4.getStepZ();
                final double double8 = fl.x() + double5;
                final double double9 = fl.getPos().getY() + 0.2f;
                final double double10 = fl.z() + double7;
                final FireworkRocketEntity ayr17 = new FireworkRocketEntity(fl.getLevel(), bek, double8, double9, double10, true);
                ayr17.shoot(double5, double6, double7, 0.5f, 1.0f);
                fl.getLevel().addFreshEntity(ayr17);
                bek.shrink(1);
                return bek;
            }
            
            @Override
            protected void playSound(final BlockSource fl) {
                fl.getLevel().levelEvent(1004, fl.getPos(), 0);
            }
        });
        DispenserBlock.registerBehavior(Items.FIRE_CHARGE, new DefaultDispenseItemBehavior() {
            public ItemStack execute(final BlockSource fl, final ItemStack bek) {
                final Direction fp4 = fl.getBlockState().<Direction>getValue((Property<Direction>)DispenserBlock.FACING);
                final Position fz5 = DispenserBlock.getDispensePosition(fl);
                final double double6 = fz5.x() + fp4.getStepX() * 0.3f;
                final double double7 = fz5.y() + fp4.getStepY() * 0.3f;
                final double double8 = fz5.z() + fp4.getStepZ() * 0.3f;
                final Level bjt12 = fl.getLevel();
                final Random random13 = bjt12.random;
                final double double9 = random13.nextGaussian() * 0.05 + fp4.getStepX();
                final double double10 = random13.nextGaussian() * 0.05 + fp4.getStepY();
                final double double11 = random13.nextGaussian() * 0.05 + fp4.getStepZ();
                bjt12.addFreshEntity(Util.<SmallFireball>make(new SmallFireball(bjt12, double6, double7, double8, double9, double10, double11), ayy -> ayy.setItem(bek)));
                bek.shrink(1);
                return bek;
            }
            
            @Override
            protected void playSound(final BlockSource fl) {
                fl.getLevel().levelEvent(1018, fl.getPos(), 0);
            }
        });
        DispenserBlock.registerBehavior(Items.OAK_BOAT, new BoatDispenseItemBehavior(Boat.Type.OAK));
        DispenserBlock.registerBehavior(Items.SPRUCE_BOAT, new BoatDispenseItemBehavior(Boat.Type.SPRUCE));
        DispenserBlock.registerBehavior(Items.BIRCH_BOAT, new BoatDispenseItemBehavior(Boat.Type.BIRCH));
        DispenserBlock.registerBehavior(Items.JUNGLE_BOAT, new BoatDispenseItemBehavior(Boat.Type.JUNGLE));
        DispenserBlock.registerBehavior(Items.DARK_OAK_BOAT, new BoatDispenseItemBehavior(Boat.Type.DARK_OAK));
        DispenserBlock.registerBehavior(Items.ACACIA_BOAT, new BoatDispenseItemBehavior(Boat.Type.ACACIA));
        final DispenseItemBehavior gl2 = new DefaultDispenseItemBehavior() {
            private final DefaultDispenseItemBehavior defaultDispenseItemBehavior = new DefaultDispenseItemBehavior();
            
            public ItemStack execute(final BlockSource fl, final ItemStack bek) {
                final BucketItem bcu4 = (BucketItem)bek.getItem();
                final BlockPos fk5 = fl.getPos().relative(fl.getBlockState().<Direction>getValue((Property<Direction>)DispenserBlock.FACING));
                final Level bjt6 = fl.getLevel();
                if (bcu4.emptyBucket(null, bjt6, fk5, null)) {
                    bcu4.checkExtraContent(bjt6, bek, fk5);
                    return new ItemStack(Items.BUCKET);
                }
                return this.defaultDispenseItemBehavior.dispense(fl, bek);
            }
        };
        DispenserBlock.registerBehavior(Items.LAVA_BUCKET, gl2);
        DispenserBlock.registerBehavior(Items.WATER_BUCKET, gl2);
        DispenserBlock.registerBehavior(Items.SALMON_BUCKET, gl2);
        DispenserBlock.registerBehavior(Items.COD_BUCKET, gl2);
        DispenserBlock.registerBehavior(Items.PUFFERFISH_BUCKET, gl2);
        DispenserBlock.registerBehavior(Items.TROPICAL_FISH_BUCKET, gl2);
        DispenserBlock.registerBehavior(Items.BUCKET, new DefaultDispenseItemBehavior() {
            private final DefaultDispenseItemBehavior defaultDispenseItemBehavior = new DefaultDispenseItemBehavior();
            
            public ItemStack execute(final BlockSource fl, final ItemStack bek) {
                final LevelAccessor bju4 = fl.getLevel();
                final BlockPos fk5 = fl.getPos().relative(fl.getBlockState().<Direction>getValue((Property<Direction>)DispenserBlock.FACING));
                final BlockState byg6 = bju4.getBlockState(fk5);
                final Block bpe7 = byg6.getBlock();
                if (!(bpe7 instanceof BucketPickup)) {
                    return super.execute(fl, bek);
                }
                final Fluid cof9 = ((BucketPickup)bpe7).takeLiquid(bju4, fk5, byg6);
                if (!(cof9 instanceof FlowingFluid)) {
                    return super.execute(fl, bek);
                }
                final Item bef8 = cof9.getBucket();
                bek.shrink(1);
                if (bek.isEmpty()) {
                    return new ItemStack(bef8);
                }
                if (fl.<DispenserBlockEntity>getEntity().addItem(new ItemStack(bef8)) < 0) {
                    this.defaultDispenseItemBehavior.dispense(fl, new ItemStack(bef8));
                }
                return bek;
            }
        });
        DispenserBlock.registerBehavior(Items.FLINT_AND_STEEL, new OptionalDispenseItemBehavior() {
            @Override
            protected ItemStack execute(final BlockSource fl, final ItemStack bek) {
                final Level bjt4 = fl.getLevel();
                this.success = true;
                final BlockPos fk5 = fl.getPos().relative(fl.getBlockState().<Direction>getValue((Property<Direction>)DispenserBlock.FACING));
                final BlockState byg6 = bjt4.getBlockState(fk5);
                if (FlintAndSteelItem.canUse(byg6, bjt4, fk5)) {
                    bjt4.setBlockAndUpdate(fk5, Blocks.FIRE.defaultBlockState());
                }
                else if (FlintAndSteelItem.canLightCampFire(byg6)) {
                    bjt4.setBlockAndUpdate(fk5, ((AbstractStateHolder<O, BlockState>)byg6).<Comparable, Boolean>setValue((Property<Comparable>)BlockStateProperties.LIT, true));
                }
                else if (byg6.getBlock() instanceof TntBlock) {
                    TntBlock.explode(bjt4, fk5);
                    bjt4.removeBlock(fk5, false);
                }
                else {
                    this.success = false;
                }
                if (this.success && bek.hurt(1, bjt4.random, null)) {
                    bek.setCount(0);
                }
                return bek;
            }
        });
        DispenserBlock.registerBehavior(Items.BONE_MEAL, new OptionalDispenseItemBehavior() {
            @Override
            protected ItemStack execute(final BlockSource fl, final ItemStack bek) {
                this.success = true;
                final Level bjt4 = fl.getLevel();
                final BlockPos fk5 = fl.getPos().relative(fl.getBlockState().<Direction>getValue((Property<Direction>)DispenserBlock.FACING));
                if (BoneMealItem.growCrop(bek, bjt4, fk5) || BoneMealItem.growWaterPlant(bek, bjt4, fk5, null)) {
                    if (!bjt4.isClientSide) {
                        bjt4.levelEvent(2005, fk5, 0);
                    }
                }
                else {
                    this.success = false;
                }
                return bek;
            }
        });
        DispenserBlock.registerBehavior(Blocks.TNT, new DefaultDispenseItemBehavior() {
            @Override
            protected ItemStack execute(final BlockSource fl, final ItemStack bek) {
                final Level bjt4 = fl.getLevel();
                final BlockPos fk5 = fl.getPos().relative(fl.getBlockState().<Direction>getValue((Property<Direction>)DispenserBlock.FACING));
                final PrimedTnt avz6 = new PrimedTnt(bjt4, fk5.getX() + 0.5, fk5.getY(), fk5.getZ() + 0.5, null);
                bjt4.addFreshEntity(avz6);
                bjt4.playSound(null, avz6.getX(), avz6.getY(), avz6.getZ(), SoundEvents.TNT_PRIMED, SoundSource.BLOCKS, 1.0f, 1.0f);
                bek.shrink(1);
                return bek;
            }
        });
        final DispenseItemBehavior gl3 = new OptionalDispenseItemBehavior() {
            @Override
            protected ItemStack execute(final BlockSource fl, final ItemStack bek) {
                this.success = ArmorItem.dispenseArmor(fl, bek);
                return bek;
            }
        };
        DispenserBlock.registerBehavior(Items.CREEPER_HEAD, gl3);
        DispenserBlock.registerBehavior(Items.ZOMBIE_HEAD, gl3);
        DispenserBlock.registerBehavior(Items.DRAGON_HEAD, gl3);
        DispenserBlock.registerBehavior(Items.SKELETON_SKULL, gl3);
        DispenserBlock.registerBehavior(Items.PLAYER_HEAD, gl3);
        DispenserBlock.registerBehavior(Items.WITHER_SKELETON_SKULL, new OptionalDispenseItemBehavior() {
            @Override
            protected ItemStack execute(final BlockSource fl, final ItemStack bek) {
                final Level bjt4 = fl.getLevel();
                final Direction fp5 = fl.getBlockState().<Direction>getValue((Property<Direction>)DispenserBlock.FACING);
                final BlockPos fk6 = fl.getPos().relative(fp5);
                if (bjt4.isEmptyBlock(fk6) && WitherSkullBlock.canSpawnMob(bjt4, fk6, bek)) {
                    bjt4.setBlock(fk6, ((AbstractStateHolder<O, BlockState>)Blocks.WITHER_SKELETON_SKULL.defaultBlockState()).<Comparable, Integer>setValue((Property<Comparable>)SkullBlock.ROTATION, (fp5.getAxis() == Direction.Axis.Y) ? 0 : (fp5.getOpposite().get2DDataValue() * 4)), 3);
                    final BlockEntity bwi7 = bjt4.getBlockEntity(fk6);
                    if (bwi7 instanceof SkullBlockEntity) {
                        WitherSkullBlock.checkSpawn(bjt4, fk6, (SkullBlockEntity)bwi7);
                    }
                    bek.shrink(1);
                    this.success = true;
                }
                else {
                    this.success = ArmorItem.dispenseArmor(fl, bek);
                }
                return bek;
            }
        });
        DispenserBlock.registerBehavior(Blocks.CARVED_PUMPKIN, new OptionalDispenseItemBehavior() {
            @Override
            protected ItemStack execute(final BlockSource fl, final ItemStack bek) {
                final Level bjt4 = fl.getLevel();
                final BlockPos fk5 = fl.getPos().relative(fl.getBlockState().<Direction>getValue((Property<Direction>)DispenserBlock.FACING));
                final CarvedPumpkinBlock bpr6 = (CarvedPumpkinBlock)Blocks.CARVED_PUMPKIN;
                if (bjt4.isEmptyBlock(fk5) && bpr6.canSpawnGolem(bjt4, fk5)) {
                    if (!bjt4.isClientSide) {
                        bjt4.setBlock(fk5, bpr6.defaultBlockState(), 3);
                    }
                    bek.shrink(1);
                    this.success = true;
                }
                else {
                    this.success = ArmorItem.dispenseArmor(fl, bek);
                }
                return bek;
            }
        });
        DispenserBlock.registerBehavior(Blocks.SHULKER_BOX.asItem(), new ShulkerBoxDispenseBehavior());
        for (final DyeColor bdg7 : DyeColor.values()) {
            DispenserBlock.registerBehavior(ShulkerBoxBlock.getBlockByColor(bdg7).asItem(), new ShulkerBoxDispenseBehavior());
        }
        DispenserBlock.registerBehavior(Items.GLASS_BOTTLE.asItem(), new OptionalDispenseItemBehavior() {
            private final DefaultDispenseItemBehavior defaultDispenseItemBehavior = new DefaultDispenseItemBehavior();
            
            private ItemStack takeLiquid(final BlockSource fl, final ItemStack bek2, final ItemStack bek3) {
                bek2.shrink(1);
                if (bek2.isEmpty()) {
                    return bek3.copy();
                }
                if (fl.<DispenserBlockEntity>getEntity().addItem(bek3.copy()) < 0) {
                    this.defaultDispenseItemBehavior.dispense(fl, bek3.copy());
                }
                return bek2;
            }
            
            public ItemStack execute(final BlockSource fl, final ItemStack bek) {
                this.success = false;
                final LevelAccessor bju4 = fl.getLevel();
                final BlockPos fk5 = fl.getPos().relative(fl.getBlockState().<Direction>getValue((Property<Direction>)DispenserBlock.FACING));
                final BlockState byg6 = bju4.getBlockState(fk5);
                final Block bpe7 = byg6.getBlock();
                if (bpe7.is(BlockTags.BEEHIVES) && byg6.<Integer>getValue((Property<Integer>)BeehiveBlock.HONEY_LEVEL) >= 5) {
                    ((BeehiveBlock)byg6.getBlock()).releaseBeesAndResetHoneyLevel(bju4.getLevel(), byg6, fk5, null, BeehiveBlockEntity.BeeReleaseStatus.BEE_RELEASED);
                    this.success = true;
                    return this.takeLiquid(fl, bek, new ItemStack(Items.HONEY_BOTTLE));
                }
                if (bju4.getFluidState(fk5).is(FluidTags.WATER)) {
                    this.success = true;
                    return this.takeLiquid(fl, bek, PotionUtils.setPotion(new ItemStack(Items.POTION), Potions.WATER));
                }
                return super.execute(fl, bek);
            }
        });
        DispenserBlock.registerBehavior(Items.SHEARS.asItem(), new OptionalDispenseItemBehavior() {
            @Override
            protected ItemStack execute(final BlockSource fl, final ItemStack bek) {
                final Level bjt4 = fl.getLevel();
                if (!bjt4.isClientSide()) {
                    this.success = false;
                    final BlockPos fk5 = fl.getPos().relative(fl.getBlockState().<Direction>getValue((Property<Direction>)DispenserBlock.FACING));
                    final List<Sheep> list6 = bjt4.<Sheep>getEntitiesOfClass(Sheep.class, new AABB(fk5));
                    for (final Sheep ats8 : list6) {
                        if (ats8.isAlive() && !ats8.isSheared() && !ats8.isBaby()) {
                            ats8.shear();
                            if (bek.hurt(1, bjt4.random, null)) {
                                bek.setCount(0);
                            }
                            this.success = true;
                            break;
                        }
                    }
                    if (!this.success) {
                        final BlockState byg7 = bjt4.getBlockState(fk5);
                        if (byg7.is(BlockTags.BEEHIVES)) {
                            final int integer8 = byg7.<Integer>getValue((Property<Integer>)BeehiveBlock.HONEY_LEVEL);
                            if (integer8 >= 5) {
                                if (bek.hurt(1, bjt4.random, null)) {
                                    bek.setCount(0);
                                }
                                BeehiveBlock.dropHoneycomb(bjt4, fk5);
                                ((BeehiveBlock)byg7.getBlock()).releaseBeesAndResetHoneyLevel(bjt4, byg7, fk5, null, BeehiveBlockEntity.BeeReleaseStatus.BEE_RELEASED);
                                this.success = true;
                            }
                        }
                    }
                }
                return bek;
            }
        });
    }
}
