package net.minecraft.server.commands;

import com.mojang.brigadier.Message;
import com.mojang.brigadier.context.CommandContext;
import com.mojang.brigadier.exceptions.CommandSyntaxException;
import net.minecraft.server.level.ServerPlayer;
import net.minecraft.network.chat.Component;
import net.minecraft.network.chat.TranslatableComponent;
import net.minecraft.world.scores.Score;
import java.util.Iterator;
import net.minecraft.world.scores.Scoreboard;
import java.util.List;
import net.minecraft.world.entity.Entity;
import net.minecraft.commands.SharedSuggestionProvider;
import net.minecraft.world.scores.criteria.ObjectiveCriteria;
import net.minecraft.world.scores.Objective;
import com.google.common.collect.Lists;
import com.mojang.brigadier.suggestion.Suggestions;
import java.util.concurrent.CompletableFuture;
import com.mojang.brigadier.suggestion.SuggestionsBuilder;
import com.mojang.brigadier.arguments.IntegerArgumentType;
import com.mojang.brigadier.arguments.ArgumentType;
import net.minecraft.commands.arguments.ObjectiveArgument;
import com.mojang.brigadier.builder.RequiredArgumentBuilder;
import net.minecraft.commands.Commands;
import com.mojang.brigadier.builder.LiteralArgumentBuilder;
import net.minecraft.commands.CommandSourceStack;
import com.mojang.brigadier.CommandDispatcher;
import com.mojang.brigadier.exceptions.SimpleCommandExceptionType;

public class TriggerCommand {
    private static final SimpleCommandExceptionType ERROR_NOT_PRIMED;
    private static final SimpleCommandExceptionType ERROR_INVALID_OBJECTIVE;
    
    public static void register(final CommandDispatcher<CommandSourceStack> commandDispatcher) {
        commandDispatcher.register((LiteralArgumentBuilder)Commands.literal("trigger").then(((RequiredArgumentBuilder)((RequiredArgumentBuilder)Commands.argument("objective", (com.mojang.brigadier.arguments.ArgumentType<Object>)ObjectiveArgument.objective()).suggests((commandContext, suggestionsBuilder) -> suggestObjectives((CommandSourceStack)commandContext.getSource(), suggestionsBuilder)).executes(commandContext -> simpleTrigger((CommandSourceStack)commandContext.getSource(), getScore(((CommandSourceStack)commandContext.getSource()).getPlayerOrException(), ObjectiveArgument.getObjective((CommandContext<CommandSourceStack>)commandContext, "objective"))))).then(Commands.literal("add").then(Commands.argument("value", (com.mojang.brigadier.arguments.ArgumentType<Object>)IntegerArgumentType.integer()).executes(commandContext -> addValue((CommandSourceStack)commandContext.getSource(), getScore(((CommandSourceStack)commandContext.getSource()).getPlayerOrException(), ObjectiveArgument.getObjective((CommandContext<CommandSourceStack>)commandContext, "objective")), IntegerArgumentType.getInteger(commandContext, "value")))))).then(Commands.literal("set").then(Commands.argument("value", (com.mojang.brigadier.arguments.ArgumentType<Object>)IntegerArgumentType.integer()).executes(commandContext -> setValue((CommandSourceStack)commandContext.getSource(), getScore(((CommandSourceStack)commandContext.getSource()).getPlayerOrException(), ObjectiveArgument.getObjective((CommandContext<CommandSourceStack>)commandContext, "objective")), IntegerArgumentType.getInteger(commandContext, "value")))))));
    }
    
    public static CompletableFuture<Suggestions> suggestObjectives(final CommandSourceStack cq, final SuggestionsBuilder suggestionsBuilder) {
        final Entity akn3 = cq.getEntity();
        final List<String> list4 = Lists.newArrayList();
        if (akn3 != null) {
            final Scoreboard cwi5 = cq.getServer().getScoreboard();
            final String string6 = akn3.getScoreboardName();
            for (final Objective cwf8 : cwi5.getObjectives()) {
                if (cwf8.getCriteria() == ObjectiveCriteria.TRIGGER && cwi5.hasPlayerScore(string6, cwf8)) {
                    final Score cwh9 = cwi5.getOrCreatePlayerScore(string6, cwf8);
                    if (cwh9.isLocked()) {
                        continue;
                    }
                    list4.add(cwf8.getName());
                }
            }
        }
        return SharedSuggestionProvider.suggest(list4, suggestionsBuilder);
    }
    
    private static int addValue(final CommandSourceStack cq, final Score cwh, final int integer) {
        cwh.add(integer);
        cq.sendSuccess(new TranslatableComponent("commands.trigger.add.success", new Object[] { cwh.getObjective().getFormattedDisplayName(), integer }), true);
        return cwh.getScore();
    }
    
    private static int setValue(final CommandSourceStack cq, final Score cwh, final int integer) {
        cwh.setScore(integer);
        cq.sendSuccess(new TranslatableComponent("commands.trigger.set.success", new Object[] { cwh.getObjective().getFormattedDisplayName(), integer }), true);
        return integer;
    }
    
    private static int simpleTrigger(final CommandSourceStack cq, final Score cwh) {
        cwh.add(1);
        cq.sendSuccess(new TranslatableComponent("commands.trigger.simple.success", new Object[] { cwh.getObjective().getFormattedDisplayName() }), true);
        return cwh.getScore();
    }
    
    private static Score getScore(final ServerPlayer xe, final Objective cwf) throws CommandSyntaxException {
        if (cwf.getCriteria() != ObjectiveCriteria.TRIGGER) {
            throw TriggerCommand.ERROR_INVALID_OBJECTIVE.create();
        }
        final Scoreboard cwi3 = xe.getScoreboard();
        final String string4 = xe.getScoreboardName();
        if (!cwi3.hasPlayerScore(string4, cwf)) {
            throw TriggerCommand.ERROR_NOT_PRIMED.create();
        }
        final Score cwh5 = cwi3.getOrCreatePlayerScore(string4, cwf);
        if (cwh5.isLocked()) {
            throw TriggerCommand.ERROR_NOT_PRIMED.create();
        }
        cwh5.setLocked(true);
        return cwh5;
    }
    
    static {
        ERROR_NOT_PRIMED = new SimpleCommandExceptionType((Message)new TranslatableComponent("commands.trigger.failed.unprimed", new Object[0]));
        ERROR_INVALID_OBJECTIVE = new SimpleCommandExceptionType((Message)new TranslatableComponent("commands.trigger.failed.invalid", new Object[0]));
    }
}
