package net.minecraft.world.damagesource;

import javax.annotation.Nullable;
import java.util.Iterator;
import net.minecraft.world.entity.player.Player;
import net.minecraft.world.item.ItemStack;
import net.minecraft.network.chat.TranslatableComponent;
import net.minecraft.network.chat.Component;
import net.minecraft.world.level.block.Block;
import net.minecraft.world.level.block.Blocks;
import net.minecraft.world.entity.Entity;
import net.minecraft.core.BlockPos;
import com.google.common.collect.Lists;
import net.minecraft.world.entity.LivingEntity;
import java.util.List;

public class CombatTracker {
    private final List<CombatEntry> entries;
    private final LivingEntity mob;
    private int lastDamageTime;
    private int combatStartTime;
    private int combatEndTime;
    private boolean inCombat;
    private boolean takingDamage;
    private String nextLocation;
    
    public CombatTracker(final LivingEntity akw) {
        this.entries = Lists.newArrayList();
        this.mob = akw;
    }
    
    public void prepareForDamage() {
        this.resetPreparedStatus();
        if (this.mob.onLadder()) {
            final Block bpe2 = this.mob.level.getBlockState(new BlockPos(this.mob)).getBlock();
            if (bpe2 == Blocks.LADDER) {
                this.nextLocation = "ladder";
            }
            else if (bpe2 == Blocks.VINE) {
                this.nextLocation = "vines";
            }
        }
        else if (this.mob.isInWater()) {
            this.nextLocation = "water";
        }
    }
    
    public void recordDamage(final DamageSource ajw, final float float2, final float float3) {
        this.recheckStatus();
        this.prepareForDamage();
        final CombatEntry ajt5 = new CombatEntry(ajw, this.mob.tickCount, float2, float3, this.nextLocation, this.mob.fallDistance);
        this.entries.add(ajt5);
        this.lastDamageTime = this.mob.tickCount;
        this.takingDamage = true;
        if (ajt5.isCombatRelated() && !this.inCombat && this.mob.isAlive()) {
            this.inCombat = true;
            this.combatStartTime = this.mob.tickCount;
            this.combatEndTime = this.combatStartTime;
            this.mob.onEnterCombat();
        }
    }
    
    public Component getDeathMessage() {
        if (this.entries.isEmpty()) {
            return new TranslatableComponent("death.attack.generic", new Object[] { this.mob.getDisplayName() });
        }
        final CombatEntry ajt2 = this.getMostSignificantFall();
        final CombatEntry ajt3 = this.entries.get(this.entries.size() - 1);
        final Component lf5 = ajt3.getAttackerName();
        final Entity akn6 = ajt3.getSource().getEntity();
        Component lf7;
        if (ajt2 != null && ajt3.getSource() == DamageSource.FALL) {
            final Component lf6 = ajt2.getAttackerName();
            if (ajt2.getSource() == DamageSource.FALL || ajt2.getSource() == DamageSource.OUT_OF_WORLD) {
                lf7 = new TranslatableComponent("death.fell.accident." + this.getFallLocation(ajt2), new Object[] { this.mob.getDisplayName() });
            }
            else if (lf6 != null && (lf5 == null || !lf6.equals(lf5))) {
                final Entity akn7 = ajt2.getSource().getEntity();
                final ItemStack bek9 = (akn7 instanceof LivingEntity) ? ((LivingEntity)akn7).getMainHandItem() : ItemStack.EMPTY;
                if (!bek9.isEmpty() && bek9.hasCustomHoverName()) {
                    lf7 = new TranslatableComponent("death.fell.assist.item", new Object[] { this.mob.getDisplayName(), lf6, bek9.getDisplayName() });
                }
                else {
                    lf7 = new TranslatableComponent("death.fell.assist", new Object[] { this.mob.getDisplayName(), lf6 });
                }
            }
            else if (lf5 != null) {
                final ItemStack bek10 = (akn6 instanceof LivingEntity) ? ((LivingEntity)akn6).getMainHandItem() : ItemStack.EMPTY;
                if (!bek10.isEmpty() && bek10.hasCustomHoverName()) {
                    lf7 = new TranslatableComponent("death.fell.finish.item", new Object[] { this.mob.getDisplayName(), lf5, bek10.getDisplayName() });
                }
                else {
                    lf7 = new TranslatableComponent("death.fell.finish", new Object[] { this.mob.getDisplayName(), lf5 });
                }
            }
            else {
                lf7 = new TranslatableComponent("death.fell.killer", new Object[] { this.mob.getDisplayName() });
            }
        }
        else {
            lf7 = ajt3.getSource().getLocalizedDeathMessage(this.mob);
        }
        return lf7;
    }
    
    @Nullable
    public LivingEntity getKiller() {
        LivingEntity akw2 = null;
        Player ayg3 = null;
        float float4 = 0.0f;
        float float5 = 0.0f;
        for (final CombatEntry ajt7 : this.entries) {
            if (ajt7.getSource().getEntity() instanceof Player && (ayg3 == null || ajt7.getDamage() > float5)) {
                float5 = ajt7.getDamage();
                ayg3 = (Player)ajt7.getSource().getEntity();
            }
            if (ajt7.getSource().getEntity() instanceof LivingEntity && (akw2 == null || ajt7.getDamage() > float4)) {
                float4 = ajt7.getDamage();
                akw2 = (LivingEntity)ajt7.getSource().getEntity();
            }
        }
        if (ayg3 != null && float5 >= float4 / 3.0f) {
            return ayg3;
        }
        return akw2;
    }
    
    @Nullable
    private CombatEntry getMostSignificantFall() {
        CombatEntry ajt2 = null;
        CombatEntry ajt3 = null;
        float float4 = 0.0f;
        float float5 = 0.0f;
        for (int integer6 = 0; integer6 < this.entries.size(); ++integer6) {
            final CombatEntry ajt4 = this.entries.get(integer6);
            final CombatEntry ajt5 = (integer6 > 0) ? this.entries.get(integer6 - 1) : null;
            if ((ajt4.getSource() == DamageSource.FALL || ajt4.getSource() == DamageSource.OUT_OF_WORLD) && ajt4.getFallDistance() > 0.0f && (ajt2 == null || ajt4.getFallDistance() > float5)) {
                if (integer6 > 0) {
                    ajt2 = ajt5;
                }
                else {
                    ajt2 = ajt4;
                }
                float5 = ajt4.getFallDistance();
            }
            if (ajt4.getLocation() != null && (ajt3 == null || ajt4.getDamage() > float4)) {
                ajt3 = ajt4;
                float4 = ajt4.getDamage();
            }
        }
        if (float5 > 5.0f && ajt2 != null) {
            return ajt2;
        }
        if (float4 > 5.0f && ajt3 != null) {
            return ajt3;
        }
        return null;
    }
    
    private String getFallLocation(final CombatEntry ajt) {
        return (ajt.getLocation() == null) ? "generic" : ajt.getLocation();
    }
    
    public int getCombatDuration() {
        if (this.inCombat) {
            return this.mob.tickCount - this.combatStartTime;
        }
        return this.combatEndTime - this.combatStartTime;
    }
    
    private void resetPreparedStatus() {
        this.nextLocation = null;
    }
    
    public void recheckStatus() {
        final int integer2 = this.inCombat ? 300 : 100;
        if (this.takingDamage && (!this.mob.isAlive() || this.mob.tickCount - this.lastDamageTime > integer2)) {
            final boolean boolean3 = this.inCombat;
            this.takingDamage = false;
            this.inCombat = false;
            this.combatEndTime = this.mob.tickCount;
            if (boolean3) {
                this.mob.onLeaveCombat();
            }
            this.entries.clear();
        }
    }
    
    public LivingEntity getMob() {
        return this.mob;
    }
}
