package net.minecraft.world.effect;

import net.minecraft.world.entity.ai.attributes.AttributeInstance;
import java.util.Iterator;
import net.minecraft.world.entity.ai.attributes.BaseAttributeMap;
import java.util.UUID;
import net.minecraft.network.chat.TranslatableComponent;
import net.minecraft.network.chat.Component;
import net.minecraft.Util;
import net.minecraft.world.entity.Entity;
import net.minecraft.world.entity.player.Player;
import net.minecraft.world.damagesource.DamageSource;
import net.minecraft.world.entity.LivingEntity;
import com.google.common.collect.Maps;
import net.minecraft.core.Registry;
import javax.annotation.Nullable;
import net.minecraft.world.entity.ai.attributes.AttributeModifier;
import net.minecraft.world.entity.ai.attributes.Attribute;
import java.util.Map;

public class MobEffect {
    private final Map<Attribute, AttributeModifier> attributeModifiers;
    private final MobEffectCategory category;
    private final int color;
    @Nullable
    private String descriptionId;
    
    @Nullable
    public static MobEffect byId(final int integer) {
        return Registry.MOB_EFFECT.byId(integer);
    }
    
    public static int getId(final MobEffect akf) {
        return Registry.MOB_EFFECT.getId(akf);
    }
    
    protected MobEffect(final MobEffectCategory akg, final int integer) {
        this.attributeModifiers = Maps.newHashMap();
        this.category = akg;
        this.color = integer;
    }
    
    public void applyEffectTick(final LivingEntity akw, final int integer) {
        if (this == MobEffects.REGENERATION) {
            if (akw.getHealth() < akw.getMaxHealth()) {
                akw.heal(1.0f);
            }
        }
        else if (this == MobEffects.POISON) {
            if (akw.getHealth() > 1.0f) {
                akw.hurt(DamageSource.MAGIC, 1.0f);
            }
        }
        else if (this == MobEffects.WITHER) {
            akw.hurt(DamageSource.WITHER, 1.0f);
        }
        else if (this == MobEffects.HUNGER && akw instanceof Player) {
            ((Player)akw).causeFoodExhaustion(0.005f * (integer + 1));
        }
        else if (this == MobEffects.SATURATION && akw instanceof Player) {
            if (!akw.level.isClientSide) {
                ((Player)akw).getFoodData().eat(integer + 1, 1.0f);
            }
        }
        else if ((this == MobEffects.HEAL && !akw.isInvertedHealAndHarm()) || (this == MobEffects.HARM && akw.isInvertedHealAndHarm())) {
            akw.heal((float)Math.max(4 << integer, 0));
        }
        else if ((this == MobEffects.HARM && !akw.isInvertedHealAndHarm()) || (this == MobEffects.HEAL && akw.isInvertedHealAndHarm())) {
            akw.hurt(DamageSource.MAGIC, (float)(6 << integer));
        }
    }
    
    public void applyInstantenousEffect(@Nullable final Entity akn1, @Nullable final Entity akn2, final LivingEntity akw, final int integer, final double double5) {
        if ((this == MobEffects.HEAL && !akw.isInvertedHealAndHarm()) || (this == MobEffects.HARM && akw.isInvertedHealAndHarm())) {
            final int integer2 = (int)(double5 * (4 << integer) + 0.5);
            akw.heal((float)integer2);
        }
        else if ((this == MobEffects.HARM && !akw.isInvertedHealAndHarm()) || (this == MobEffects.HEAL && akw.isInvertedHealAndHarm())) {
            final int integer2 = (int)(double5 * (6 << integer) + 0.5);
            if (akn1 == null) {
                akw.hurt(DamageSource.MAGIC, (float)integer2);
            }
            else {
                akw.hurt(DamageSource.indirectMagic(akn1, akn2), (float)integer2);
            }
        }
        else {
            this.applyEffectTick(akw, integer);
        }
    }
    
    public boolean isDurationEffectTick(final int integer1, final int integer2) {
        if (this == MobEffects.REGENERATION) {
            final int integer3 = 50 >> integer2;
            return integer3 <= 0 || integer1 % integer3 == 0;
        }
        if (this == MobEffects.POISON) {
            final int integer3 = 25 >> integer2;
            return integer3 <= 0 || integer1 % integer3 == 0;
        }
        if (this == MobEffects.WITHER) {
            final int integer3 = 40 >> integer2;
            return integer3 <= 0 || integer1 % integer3 == 0;
        }
        return this == MobEffects.HUNGER;
    }
    
    public boolean isInstantenous() {
        return false;
    }
    
    protected String getOrCreateDescriptionId() {
        if (this.descriptionId == null) {
            this.descriptionId = Util.makeDescriptionId("effect", Registry.MOB_EFFECT.getKey(this));
        }
        return this.descriptionId;
    }
    
    public String getDescriptionId() {
        return this.getOrCreateDescriptionId();
    }
    
    public Component getDisplayName() {
        return new TranslatableComponent(this.getDescriptionId(), new Object[0]);
    }
    
    public MobEffectCategory getCategory() {
        return this.category;
    }
    
    public int getColor() {
        return this.color;
    }
    
    public MobEffect addAttributeModifier(final Attribute aln, final String string, final double double3, final AttributeModifier.Operation a) {
        final AttributeModifier alp7 = new AttributeModifier(UUID.fromString(string), this::getDescriptionId, double3, a);
        this.attributeModifiers.put(aln, alp7);
        return this;
    }
    
    public Map<Attribute, AttributeModifier> getAttributeModifiers() {
        return this.attributeModifiers;
    }
    
    public void removeAttributeModifiers(final LivingEntity akw, final BaseAttributeMap alr, final int integer) {
        for (final Map.Entry<Attribute, AttributeModifier> entry6 : this.attributeModifiers.entrySet()) {
            final AttributeInstance alo7 = alr.getInstance(entry6.getKey());
            if (alo7 != null) {
                alo7.removeModifier(entry6.getValue());
            }
        }
    }
    
    public void addAttributeModifiers(final LivingEntity akw, final BaseAttributeMap alr, final int integer) {
        for (final Map.Entry<Attribute, AttributeModifier> entry6 : this.attributeModifiers.entrySet()) {
            final AttributeInstance alo7 = alr.getInstance(entry6.getKey());
            if (alo7 != null) {
                final AttributeModifier alp8 = entry6.getValue();
                alo7.removeModifier(alp8);
                alo7.addModifier(new AttributeModifier(alp8.getId(), this.getDescriptionId() + " " + integer, this.getAttributeModifierValue(integer, alp8), alp8.getOperation()));
            }
        }
    }
    
    public double getAttributeModifierValue(final int integer, final AttributeModifier alp) {
        return alp.getAmount() * (integer + 1);
    }
    
    public boolean isBeneficial() {
        return this.category == MobEffectCategory.BENEFICIAL;
    }
}
