package net.minecraft.world.effect;

import org.apache.logging.log4j.LogManager;
import com.google.common.collect.ComparisonChain;
import net.minecraft.nbt.CompoundTag;
import net.minecraft.world.entity.LivingEntity;
import org.apache.logging.log4j.Logger;

public class MobEffectInstance implements Comparable<MobEffectInstance> {
    private static final Logger LOGGER;
    private final MobEffect effect;
    private int duration;
    private int amplifier;
    private boolean splash;
    private boolean ambient;
    private boolean noCounter;
    private boolean visible;
    private boolean showIcon;
    
    public MobEffectInstance(final MobEffect akf) {
        this(akf, 0, 0);
    }
    
    public MobEffectInstance(final MobEffect akf, final int integer) {
        this(akf, integer, 0);
    }
    
    public MobEffectInstance(final MobEffect akf, final int integer2, final int integer3) {
        this(akf, integer2, integer3, false, true);
    }
    
    public MobEffectInstance(final MobEffect akf, final int integer2, final int integer3, final boolean boolean4, final boolean boolean5) {
        this(akf, integer2, integer3, boolean4, boolean5, boolean5);
    }
    
    public MobEffectInstance(final MobEffect akf, final int integer2, final int integer3, final boolean boolean4, final boolean boolean5, final boolean boolean6) {
        this.effect = akf;
        this.duration = integer2;
        this.amplifier = integer3;
        this.ambient = boolean4;
        this.visible = boolean5;
        this.showIcon = boolean6;
    }
    
    public MobEffectInstance(final MobEffectInstance akh) {
        this.effect = akh.effect;
        this.duration = akh.duration;
        this.amplifier = akh.amplifier;
        this.ambient = akh.ambient;
        this.visible = akh.visible;
        this.showIcon = akh.showIcon;
    }
    
    public boolean update(final MobEffectInstance akh) {
        if (this.effect != akh.effect) {
            MobEffectInstance.LOGGER.warn("This method should only be called for matching effects!");
        }
        boolean boolean3 = false;
        if (akh.amplifier > this.amplifier) {
            this.amplifier = akh.amplifier;
            this.duration = akh.duration;
            boolean3 = true;
        }
        else if (akh.amplifier == this.amplifier && this.duration < akh.duration) {
            this.duration = akh.duration;
            boolean3 = true;
        }
        if ((!akh.ambient && this.ambient) || boolean3) {
            this.ambient = akh.ambient;
            boolean3 = true;
        }
        if (akh.visible != this.visible) {
            this.visible = akh.visible;
            boolean3 = true;
        }
        if (akh.showIcon != this.showIcon) {
            this.showIcon = akh.showIcon;
            boolean3 = true;
        }
        return boolean3;
    }
    
    public MobEffect getEffect() {
        return this.effect;
    }
    
    public int getDuration() {
        return this.duration;
    }
    
    public int getAmplifier() {
        return this.amplifier;
    }
    
    public boolean isAmbient() {
        return this.ambient;
    }
    
    public boolean isVisible() {
        return this.visible;
    }
    
    public boolean showIcon() {
        return this.showIcon;
    }
    
    public boolean tick(final LivingEntity akw) {
        if (this.duration > 0) {
            if (this.effect.isDurationEffectTick(this.duration, this.amplifier)) {
                this.applyEffect(akw);
            }
            this.tickDownDuration();
        }
        return this.duration > 0;
    }
    
    private int tickDownDuration() {
        return --this.duration;
    }
    
    public void applyEffect(final LivingEntity akw) {
        if (this.duration > 0) {
            this.effect.applyEffectTick(akw, this.amplifier);
        }
    }
    
    public String getDescriptionId() {
        return this.effect.getDescriptionId();
    }
    
    @Override
    public String toString() {
        String string2;
        if (this.amplifier > 0) {
            string2 = this.getDescriptionId() + " x " + (this.amplifier + 1) + ", Duration: " + this.duration;
        }
        else {
            string2 = this.getDescriptionId() + ", Duration: " + this.duration;
        }
        if (this.splash) {
            string2 += ", Splash: true";
        }
        if (!this.visible) {
            string2 += ", Particles: false";
        }
        if (!this.showIcon) {
            string2 += ", Show Icon: false";
        }
        return string2;
    }
    
    @Override
    public boolean equals(final Object object) {
        if (this == object) {
            return true;
        }
        if (object instanceof MobEffectInstance) {
            final MobEffectInstance akh3 = (MobEffectInstance)object;
            return this.duration == akh3.duration && this.amplifier == akh3.amplifier && this.splash == akh3.splash && this.ambient == akh3.ambient && this.effect.equals(akh3.effect);
        }
        return false;
    }
    
    @Override
    public int hashCode() {
        int integer2 = this.effect.hashCode();
        integer2 = 31 * integer2 + this.duration;
        integer2 = 31 * integer2 + this.amplifier;
        integer2 = 31 * integer2 + (this.splash ? 1 : 0);
        integer2 = 31 * integer2 + (this.ambient ? 1 : 0);
        return integer2;
    }
    
    public CompoundTag save(final CompoundTag jt) {
        jt.putByte("Id", (byte)MobEffect.getId(this.getEffect()));
        jt.putByte("Amplifier", (byte)this.getAmplifier());
        jt.putInt("Duration", this.getDuration());
        jt.putBoolean("Ambient", this.isAmbient());
        jt.putBoolean("ShowParticles", this.isVisible());
        jt.putBoolean("ShowIcon", this.showIcon());
        return jt;
    }
    
    public static MobEffectInstance load(final CompoundTag jt) {
        final int integer2 = jt.getByte("Id");
        final MobEffect akf3 = MobEffect.byId(integer2);
        if (akf3 == null) {
            return null;
        }
        final int integer3 = jt.getByte("Amplifier");
        final int integer4 = jt.getInt("Duration");
        final boolean boolean6 = jt.getBoolean("Ambient");
        boolean boolean7 = true;
        if (jt.contains("ShowParticles", 1)) {
            boolean7 = jt.getBoolean("ShowParticles");
        }
        boolean boolean8 = boolean7;
        if (jt.contains("ShowIcon", 1)) {
            boolean8 = jt.getBoolean("ShowIcon");
        }
        return new MobEffectInstance(akf3, integer4, (integer3 < 0) ? 0 : integer3, boolean6, boolean7, boolean8);
    }
    
    public void setNoCounter(final boolean boolean1) {
        this.noCounter = boolean1;
    }
    
    public boolean isNoCounter() {
        return this.noCounter;
    }
    
    @Override
    public int compareTo(final MobEffectInstance akh) {
        final int integer3 = 32147;
        if ((this.getDuration() > 32147 && akh.getDuration() > 32147) || (this.isAmbient() && akh.isAmbient())) {
            return ComparisonChain.start().compare(this.isAmbient(), akh.isAmbient()).compare(this.getEffect().getColor(), akh.getEffect().getColor()).result();
        }
        return ComparisonChain.start().compare(this.isAmbient(), akh.isAmbient()).compare(this.getDuration(), akh.getDuration()).compare(this.getEffect().getColor(), akh.getEffect().getColor()).result();
    }
    
    static {
        LOGGER = LogManager.getLogger();
    }
}
