package net.minecraft.world.entity.ai.attributes;

import java.util.Iterator;
import java.util.Collection;
import javax.annotation.Nullable;
import com.google.common.collect.HashMultimap;
import net.minecraft.util.InsensitiveStringMap;
import com.google.common.collect.Maps;
import com.google.common.collect.Multimap;
import java.util.Map;

public abstract class BaseAttributeMap {
    protected final Map<Attribute, AttributeInstance> attributesByObject;
    protected final Map<String, AttributeInstance> attributesByName;
    protected final Multimap<Attribute, Attribute> descendantsByParent;
    
    public BaseAttributeMap() {
        this.attributesByObject = Maps.newHashMap();
        this.attributesByName = new InsensitiveStringMap<AttributeInstance>();
        this.descendantsByParent = HashMultimap.create();
    }
    
    @Nullable
    public AttributeInstance getInstance(final Attribute aln) {
        return this.attributesByObject.get(aln);
    }
    
    @Nullable
    public AttributeInstance getInstance(final String string) {
        return this.attributesByName.get(string);
    }
    
    public AttributeInstance registerAttribute(final Attribute aln) {
        if (this.attributesByName.containsKey(aln.getName())) {
            throw new IllegalArgumentException("Attribute is already registered!");
        }
        final AttributeInstance alo3 = this.createAttributeInstance(aln);
        this.attributesByName.put(aln.getName(), alo3);
        this.attributesByObject.put(aln, alo3);
        for (Attribute aln2 = aln.getParentAttribute(); aln2 != null; aln2 = aln2.getParentAttribute()) {
            this.descendantsByParent.put(aln2, aln);
        }
        return alo3;
    }
    
    protected abstract AttributeInstance createAttributeInstance(final Attribute aln);
    
    public Collection<AttributeInstance> getAttributes() {
        return this.attributesByName.values();
    }
    
    public void onAttributeModified(final AttributeInstance alo) {
    }
    
    public void removeAttributeModifiers(final Multimap<String, AttributeModifier> multimap) {
        for (final Map.Entry<String, AttributeModifier> entry4 : multimap.entries()) {
            final AttributeInstance alo5 = this.getInstance(entry4.getKey());
            if (alo5 != null) {
                alo5.removeModifier(entry4.getValue());
            }
        }
    }
    
    public void addAttributeModifiers(final Multimap<String, AttributeModifier> multimap) {
        for (final Map.Entry<String, AttributeModifier> entry4 : multimap.entries()) {
            final AttributeInstance alo5 = this.getInstance(entry4.getKey());
            if (alo5 != null) {
                alo5.removeModifier(entry4.getValue());
                alo5.addModifier(entry4.getValue());
            }
        }
    }
    
    public void assignValues(final BaseAttributeMap alr) {
        final AttributeInstance alo2;
        this.getAttributes().forEach(alo -> {
            alo2 = alr.getInstance(alo.getAttribute());
            if (alo2 != null) {
                alo.copyFrom(alo2);
            }
        });
    }
}
