package net.minecraft.world.entity.ai.attributes;

import java.util.Collection;
import java.util.Iterator;
import net.minecraft.util.InsensitiveStringMap;
import com.google.common.collect.Sets;
import java.util.Map;
import java.util.Set;

public class ModifiableAttributeMap extends BaseAttributeMap {
    private final Set<AttributeInstance> dirtyAttributes;
    protected final Map<String, AttributeInstance> attributesByLegacy;
    
    public ModifiableAttributeMap() {
        this.dirtyAttributes = Sets.newHashSet();
        this.attributesByLegacy = new InsensitiveStringMap<AttributeInstance>();
    }
    
    @Override
    public ModifiableAttributeInstance getInstance(final Attribute aln) {
        return (ModifiableAttributeInstance)super.getInstance(aln);
    }
    
    @Override
    public ModifiableAttributeInstance getInstance(final String string) {
        AttributeInstance alo3 = super.getInstance(string);
        if (alo3 == null) {
            alo3 = this.attributesByLegacy.get(string);
        }
        return (ModifiableAttributeInstance)alo3;
    }
    
    @Override
    public AttributeInstance registerAttribute(final Attribute aln) {
        final AttributeInstance alo3 = super.registerAttribute(aln);
        if (aln instanceof RangedAttribute && ((RangedAttribute)aln).getImportLegacyName() != null) {
            this.attributesByLegacy.put(((RangedAttribute)aln).getImportLegacyName(), alo3);
        }
        return alo3;
    }
    
    @Override
    protected AttributeInstance createAttributeInstance(final Attribute aln) {
        return new ModifiableAttributeInstance(this, aln);
    }
    
    @Override
    public void onAttributeModified(final AttributeInstance alo) {
        if (alo.getAttribute().isClientSyncable()) {
            this.dirtyAttributes.add(alo);
        }
        for (final Attribute aln4 : this.descendantsByParent.get(alo.getAttribute())) {
            final ModifiableAttributeInstance als5 = this.getInstance(aln4);
            if (als5 != null) {
                als5.setDirty();
            }
        }
    }
    
    public Set<AttributeInstance> getDirtyAttributes() {
        return this.dirtyAttributes;
    }
    
    public Collection<AttributeInstance> getSyncableAttributes() {
        final Set<AttributeInstance> set2 = Sets.newHashSet();
        for (final AttributeInstance alo4 : this.getAttributes()) {
            if (alo4.getAttribute().isClientSyncable()) {
                set2.add(alo4);
            }
        }
        return set2;
    }
}
