package net.minecraft.world.entity.ai.behavior;

import java.util.function.ToIntFunction;
import java.util.Comparator;
import net.minecraft.core.SectionPos;
import net.minecraft.server.level.ServerLevel;
import net.minecraft.world.phys.Vec3;
import net.minecraft.core.Vec3i;
import net.minecraft.core.BlockPos;
import net.minecraft.world.entity.item.ItemEntity;
import net.minecraft.world.item.ItemStack;
import net.minecraft.world.entity.ai.memory.WalkTarget;
import net.minecraft.world.entity.monster.SharedMonsterAttributes;
import net.minecraft.world.entity.Entity;
import net.minecraft.world.entity.EntityType;
import java.util.List;
import net.minecraft.world.entity.ai.memory.MemoryModuleType;
import net.minecraft.world.entity.ai.Brain;
import net.minecraft.world.entity.LivingEntity;

public class BehaviorUtils {
    public static void lockGazeAndWalkToEachOther(final LivingEntity akw1, final LivingEntity akw2) {
        lookAtEachOther(akw1, akw2);
        walkToEachOther(akw1, akw2);
    }
    
    public static boolean entityIsVisible(final Brain<?> alm, final LivingEntity akw) {
        return alm.<List<LivingEntity>>getMemory(MemoryModuleType.VISIBLE_LIVING_ENTITIES).filter(list -> list.contains(akw)).isPresent();
    }
    
    public static boolean targetIsValid(final Brain<?> alm, final MemoryModuleType<? extends LivingEntity> ari, final EntityType<?> akr) {
        return alm.getMemory(ari).filter(akw -> akw.getType() == akr).filter(LivingEntity::isAlive).filter(akw -> entityIsVisible(alm, akw)).isPresent();
    }
    
    public static void lookAtEachOther(final LivingEntity akw1, final LivingEntity akw2) {
        lookAtEntity(akw1, akw2);
        lookAtEntity(akw2, akw1);
    }
    
    public static void lookAtEntity(final LivingEntity akw1, final LivingEntity akw2) {
        akw1.getBrain().<EntityPosWrapper>setMemory((MemoryModuleType<EntityPosWrapper>)MemoryModuleType.LOOK_TARGET, new EntityPosWrapper(akw2));
    }
    
    public static void walkToEachOther(final LivingEntity akw1, final LivingEntity akw2) {
        final int integer3 = 2;
        walkToEntity(akw1, akw2, 2);
        walkToEntity(akw2, akw1, 2);
    }
    
    public static void walkToEntity(final LivingEntity akw1, final LivingEntity akw2, final int integer) {
        final float float4 = (float)akw1.getAttribute(SharedMonsterAttributes.MOVEMENT_SPEED).getValue();
        final EntityPosWrapper amd5 = new EntityPosWrapper(akw2);
        final WalkTarget ark6 = new WalkTarget(amd5, float4, integer);
        akw1.getBrain().<PositionWrapper>setMemory(MemoryModuleType.LOOK_TARGET, amd5);
        akw1.getBrain().<WalkTarget>setMemory(MemoryModuleType.WALK_TARGET, ark6);
    }
    
    public static void throwItem(final LivingEntity akw1, final ItemStack bek, final LivingEntity akw3) {
        final double double4 = akw1.getEyeY() - 0.30000001192092896;
        final ItemEntity avy6 = new ItemEntity(akw1.level, akw1.getX(), double4, akw1.getZ(), bek);
        final BlockPos fk7 = new BlockPos(akw3);
        final BlockPos fk8 = new BlockPos(akw1);
        final float float9 = 0.3f;
        Vec3 cvi10 = new Vec3(fk7.subtract(fk8));
        cvi10 = cvi10.normalize().scale(0.30000001192092896);
        avy6.setDeltaMovement(cvi10);
        avy6.setDefaultPickUpDelay();
        akw1.level.addFreshEntity(avy6);
    }
    
    public static SectionPos findSectionClosestToVillage(final ServerLevel xd, final SectionPos gd, final int integer) {
        final int integer2 = xd.sectionsToVillage(gd);
        return SectionPos.cube(gd, integer).filter(gd -> xd.sectionsToVillage(gd) < integer2).min(Comparator.comparingInt(xd::sectionsToVillage)).orElse(gd);
    }
}
