package net.minecraft.world.entity.ai.behavior;

import net.minecraft.world.entity.LivingEntity;
import net.minecraft.world.entity.player.Player;
import net.minecraft.sounds.SoundSource;
import net.minecraft.sounds.SoundEvents;
import net.minecraft.world.level.block.Blocks;
import net.minecraft.world.entity.ai.memory.WalkTarget;
import net.minecraft.world.level.block.Block;
import net.minecraft.world.level.block.state.BlockState;
import net.minecraft.world.level.block.FarmBlock;
import net.minecraft.world.level.block.CropBlock;
import net.minecraft.world.item.ItemStack;
import net.minecraft.world.SimpleContainer;
import net.minecraft.core.Vec3i;
import net.minecraft.world.entity.Entity;
import net.minecraft.world.item.Items;
import net.minecraft.world.entity.npc.VillagerProfession;
import net.minecraft.world.level.GameRules;
import net.minecraft.server.level.ServerLevel;
import com.google.common.collect.Lists;
import java.util.Map;
import com.google.common.collect.ImmutableMap;
import net.minecraft.world.entity.ai.memory.MemoryStatus;
import net.minecraft.world.entity.ai.memory.MemoryModuleType;
import java.util.List;
import javax.annotation.Nullable;
import net.minecraft.core.BlockPos;
import net.minecraft.world.entity.npc.Villager;

public class HarvestFarmland extends Behavior<Villager> {
    @Nullable
    private BlockPos aboveFarmlandPos;
    private boolean canPlantStuff;
    private boolean wantsToReapStuff;
    private long nextOkStartTime;
    private int timeWorkedSoFar;
    private final List<BlockPos> validFarmlandAroundVillager;
    
    public HarvestFarmland() {
        super(ImmutableMap.<MemoryModuleType<PositionWrapper>, MemoryStatus>of(MemoryModuleType.LOOK_TARGET, MemoryStatus.VALUE_ABSENT, (MemoryModuleType<PositionWrapper>)MemoryModuleType.WALK_TARGET, MemoryStatus.VALUE_ABSENT, (MemoryModuleType<PositionWrapper>)MemoryModuleType.SECONDARY_JOB_SITE, MemoryStatus.VALUE_PRESENT));
        this.validFarmlandAroundVillager = Lists.newArrayList();
    }
    
    @Override
    protected boolean checkExtraStartConditions(final ServerLevel xd, final Villager axt) {
        if (!xd.getGameRules().getBoolean(GameRules.RULE_MOBGRIEFING)) {
            return false;
        }
        if (axt.getVillagerData().getProfession() != VillagerProfession.FARMER) {
            return false;
        }
        this.canPlantStuff = axt.hasFarmSeeds();
        this.wantsToReapStuff = false;
        final SimpleContainer ajn4 = axt.getInventory();
        for (int integer5 = ajn4.getContainerSize(), integer6 = 0; integer6 < integer5; ++integer6) {
            final ItemStack bek7 = ajn4.getItem(integer6);
            if (bek7.isEmpty()) {
                this.wantsToReapStuff = true;
                break;
            }
            if (bek7.getItem() == Items.WHEAT_SEEDS || bek7.getItem() == Items.BEETROOT_SEEDS) {
                this.wantsToReapStuff = true;
                break;
            }
        }
        final BlockPos.MutableBlockPos a6 = new BlockPos.MutableBlockPos(axt);
        this.validFarmlandAroundVillager.clear();
        for (int integer7 = -1; integer7 <= 1; ++integer7) {
            for (int integer8 = -1; integer8 <= 1; ++integer8) {
                for (int integer9 = -1; integer9 <= 1; ++integer9) {
                    a6.set(axt.getX() + integer7, axt.getY() + integer8, axt.getZ() + integer9);
                    if (this.validPos(a6, xd)) {
                        this.validFarmlandAroundVillager.add(new BlockPos(a6));
                    }
                }
            }
        }
        this.aboveFarmlandPos = this.getValidFarmland(xd);
        return (this.canPlantStuff || this.wantsToReapStuff) && this.aboveFarmlandPos != null;
    }
    
    @Nullable
    private BlockPos getValidFarmland(final ServerLevel xd) {
        return this.validFarmlandAroundVillager.isEmpty() ? null : this.validFarmlandAroundVillager.get(xd.getRandom().nextInt(this.validFarmlandAroundVillager.size()));
    }
    
    private boolean validPos(final BlockPos fk, final ServerLevel xd) {
        final BlockState byg4 = xd.getBlockState(fk);
        final Block bpe5 = byg4.getBlock();
        final Block bpe6 = xd.getBlockState(fk.below()).getBlock();
        return (bpe5 instanceof CropBlock && ((CropBlock)bpe5).isMaxAge(byg4) && this.wantsToReapStuff) || (byg4.isAir() && bpe6 instanceof FarmBlock && this.canPlantStuff);
    }
    
    @Override
    protected void start(final ServerLevel xd, final Villager axt, final long long3) {
        if (long3 > this.nextOkStartTime && this.aboveFarmlandPos != null) {
            axt.getBrain().<BlockPosWrapper>setMemory((MemoryModuleType<BlockPosWrapper>)MemoryModuleType.LOOK_TARGET, new BlockPosWrapper(this.aboveFarmlandPos));
            axt.getBrain().<WalkTarget>setMemory(MemoryModuleType.WALK_TARGET, new WalkTarget(new BlockPosWrapper(this.aboveFarmlandPos), 0.5f, 1));
        }
    }
    
    @Override
    protected void stop(final ServerLevel xd, final Villager axt, final long long3) {
        axt.getBrain().<PositionWrapper>eraseMemory(MemoryModuleType.LOOK_TARGET);
        axt.getBrain().<WalkTarget>eraseMemory(MemoryModuleType.WALK_TARGET);
        this.timeWorkedSoFar = 0;
        this.nextOkStartTime = long3 + 40L;
    }
    
    @Override
    protected void tick(final ServerLevel xd, final Villager axt, final long long3) {
        if (this.aboveFarmlandPos != null && long3 > this.nextOkStartTime) {
            final BlockState byg6 = xd.getBlockState(this.aboveFarmlandPos);
            final Block bpe7 = byg6.getBlock();
            final Block bpe8 = xd.getBlockState(this.aboveFarmlandPos.below()).getBlock();
            if (bpe7 instanceof CropBlock && ((CropBlock)bpe7).isMaxAge(byg6) && this.wantsToReapStuff) {
                xd.destroyBlock(this.aboveFarmlandPos, true, axt);
            }
            if (byg6.isAir() && bpe8 instanceof FarmBlock && this.canPlantStuff) {
                final SimpleContainer ajn9 = axt.getInventory();
                int integer10 = 0;
                while (integer10 < ajn9.getContainerSize()) {
                    final ItemStack bek11 = ajn9.getItem(integer10);
                    boolean boolean12 = false;
                    if (!bek11.isEmpty()) {
                        if (bek11.getItem() == Items.WHEAT_SEEDS) {
                            xd.setBlock(this.aboveFarmlandPos, Blocks.WHEAT.defaultBlockState(), 3);
                            boolean12 = true;
                        }
                        else if (bek11.getItem() == Items.POTATO) {
                            xd.setBlock(this.aboveFarmlandPos, Blocks.POTATOES.defaultBlockState(), 3);
                            boolean12 = true;
                        }
                        else if (bek11.getItem() == Items.CARROT) {
                            xd.setBlock(this.aboveFarmlandPos, Blocks.CARROTS.defaultBlockState(), 3);
                            boolean12 = true;
                        }
                        else if (bek11.getItem() == Items.BEETROOT_SEEDS) {
                            xd.setBlock(this.aboveFarmlandPos, Blocks.BEETROOTS.defaultBlockState(), 3);
                            boolean12 = true;
                        }
                    }
                    if (boolean12) {
                        xd.playSound(null, this.aboveFarmlandPos.getX(), this.aboveFarmlandPos.getY(), this.aboveFarmlandPos.getZ(), SoundEvents.CROP_PLANTED, SoundSource.BLOCKS, 1.0f, 1.0f);
                        bek11.shrink(1);
                        if (bek11.isEmpty()) {
                            ajn9.setItem(integer10, ItemStack.EMPTY);
                            break;
                        }
                        break;
                    }
                    else {
                        ++integer10;
                    }
                }
            }
            if (bpe7 instanceof CropBlock && !((CropBlock)bpe7).isMaxAge(byg6)) {
                this.validFarmlandAroundVillager.remove(this.aboveFarmlandPos);
                this.aboveFarmlandPos = this.getValidFarmland(xd);
                if (this.aboveFarmlandPos != null) {
                    this.nextOkStartTime = long3 + 20L;
                    axt.getBrain().<WalkTarget>setMemory(MemoryModuleType.WALK_TARGET, new WalkTarget(new BlockPosWrapper(this.aboveFarmlandPos), 0.5f, 1));
                    axt.getBrain().<BlockPosWrapper>setMemory((MemoryModuleType<BlockPosWrapper>)MemoryModuleType.LOOK_TARGET, new BlockPosWrapper(this.aboveFarmlandPos));
                }
            }
        }
        ++this.timeWorkedSoFar;
    }
    
    @Override
    protected boolean canStillUse(final ServerLevel xd, final Villager axt, final long long3) {
        return this.timeWorkedSoFar < 200;
    }
}
