package net.minecraft.world.entity.ai.behavior;

import net.minecraft.world.entity.ai.Brain;
import net.minecraft.world.entity.ai.memory.WalkTarget;
import net.minecraft.world.entity.Entity;
import java.util.List;
import net.minecraft.server.level.ServerLevel;
import java.util.Map;
import com.google.common.collect.ImmutableMap;
import net.minecraft.world.entity.ai.memory.MemoryStatus;
import net.minecraft.world.entity.ai.memory.MemoryModuleType;
import java.util.function.Predicate;
import net.minecraft.world.entity.EntityType;
import net.minecraft.world.entity.LivingEntity;

public class InteractWith<E extends LivingEntity, T extends LivingEntity> extends Behavior<E> {
    private final int maxDist;
    private final float speed;
    private final EntityType<? extends T> type;
    private final int interactionRangeSqr;
    private final Predicate<T> targetFilter;
    private final Predicate<E> selfFilter;
    private final MemoryModuleType<T> memory;
    
    public InteractWith(final EntityType<? extends T> akr, final int integer2, final Predicate<E> predicate3, final Predicate<T> predicate4, final MemoryModuleType<T> ari, final float float6, final int integer7) {
        super(ImmutableMap.<MemoryModuleType<PositionWrapper>, MemoryStatus>of(MemoryModuleType.LOOK_TARGET, MemoryStatus.REGISTERED, (MemoryModuleType<PositionWrapper>)MemoryModuleType.WALK_TARGET, MemoryStatus.VALUE_ABSENT, (MemoryModuleType<PositionWrapper>)ari, MemoryStatus.VALUE_ABSENT, (MemoryModuleType<PositionWrapper>)MemoryModuleType.VISIBLE_LIVING_ENTITIES, MemoryStatus.VALUE_PRESENT));
        this.type = akr;
        this.speed = float6;
        this.interactionRangeSqr = integer2 * integer2;
        this.maxDist = integer7;
        this.targetFilter = predicate4;
        this.selfFilter = predicate3;
        this.memory = ari;
    }
    
    public static <T extends LivingEntity> InteractWith<LivingEntity, T> of(final EntityType<? extends T> akr, final int integer2, final MemoryModuleType<T> ari, final float float4, final int integer5) {
        return new InteractWith<LivingEntity, T>(akr, integer2, akw -> true, akw -> true, ari, float4, integer5);
    }
    
    @Override
    protected boolean checkExtraStartConditions(final ServerLevel xd, final E akw) {
        return this.selfFilter.test(akw) && akw.getBrain().<List<LivingEntity>>getMemory(MemoryModuleType.VISIBLE_LIVING_ENTITIES).get().stream().anyMatch(akw -> this.type.equals(akw.getType()) && this.targetFilter.test((T)akw));
    }
    
    @Override
    protected void start(final ServerLevel xd, final E akw, final long long3) {
        final Brain<?> alm6 = akw.getBrain();
        final Brain brain;
        final MemoryModuleType walk_TARGET;
        final Object object;
        alm6.<List<LivingEntity>>getMemory(MemoryModuleType.VISIBLE_LIVING_ENTITIES).ifPresent(list -> list.stream().filter(akw -> this.type.equals(akw.getType())).map(akw -> akw).filter(akw2 -> akw2.distanceToSqr(akw) <= this.interactionRangeSqr).filter(this.targetFilter).findFirst().ifPresent(akw -> {
            brain.setMemory(this.memory, akw);
            brain.setMemory(MemoryModuleType.LOOK_TARGET, new EntityPosWrapper(akw));
            walk_TARGET = MemoryModuleType.WALK_TARGET;
            new WalkTarget(new EntityPosWrapper(akw), this.speed, this.maxDist);
            brain.setMemory(walk_TARGET, object);
        }));
    }
}
