package net.minecraft.world.entity.ai.behavior;

import net.minecraft.world.entity.LivingEntity;
import net.minecraft.world.entity.ai.Brain;
import net.minecraft.world.entity.player.Player;
import net.minecraft.world.entity.Entity;
import net.minecraft.server.level.ServerLevel;
import java.util.Map;
import net.minecraft.world.entity.ai.memory.WalkTarget;
import com.google.common.collect.ImmutableMap;
import net.minecraft.world.entity.ai.memory.MemoryStatus;
import net.minecraft.world.entity.ai.memory.MemoryModuleType;
import net.minecraft.world.entity.npc.Villager;

public class LookAndFollowTradingPlayerSink extends Behavior<Villager> {
    private final float speed;
    
    public LookAndFollowTradingPlayerSink(final float float1) {
        super(ImmutableMap.<MemoryModuleType<WalkTarget>, MemoryStatus>of(MemoryModuleType.WALK_TARGET, MemoryStatus.REGISTERED, (MemoryModuleType<WalkTarget>)MemoryModuleType.LOOK_TARGET, MemoryStatus.REGISTERED), Integer.MAX_VALUE);
        this.speed = float1;
    }
    
    @Override
    protected boolean checkExtraStartConditions(final ServerLevel xd, final Villager axt) {
        final Player ayg4 = axt.getTradingPlayer();
        return axt.isAlive() && ayg4 != null && !axt.isInWater() && !axt.hurtMarked && axt.distanceToSqr(ayg4) <= 16.0 && ayg4.containerMenu != null;
    }
    
    @Override
    protected boolean canStillUse(final ServerLevel xd, final Villager axt, final long long3) {
        return this.checkExtraStartConditions(xd, axt);
    }
    
    @Override
    protected void start(final ServerLevel xd, final Villager axt, final long long3) {
        this.followPlayer(axt);
    }
    
    @Override
    protected void stop(final ServerLevel xd, final Villager axt, final long long3) {
        final Brain<?> alm6 = axt.getBrain();
        alm6.<WalkTarget>eraseMemory(MemoryModuleType.WALK_TARGET);
        alm6.<PositionWrapper>eraseMemory(MemoryModuleType.LOOK_TARGET);
    }
    
    @Override
    protected void tick(final ServerLevel xd, final Villager axt, final long long3) {
        this.followPlayer(axt);
    }
    
    @Override
    protected boolean timedOut(final long long1) {
        return false;
    }
    
    private void followPlayer(final Villager axt) {
        final EntityPosWrapper amd3 = new EntityPosWrapper(axt.getTradingPlayer());
        final Brain<?> alm4 = axt.getBrain();
        alm4.<WalkTarget>setMemory(MemoryModuleType.WALK_TARGET, new WalkTarget(amd3, this.speed, 2));
        alm4.<PositionWrapper>setMemory(MemoryModuleType.LOOK_TARGET, amd3);
    }
}
