package net.minecraft.world.entity.ai.behavior;

import net.minecraft.core.GlobalPos;
import net.minecraft.world.entity.AgableMob;
import net.minecraft.world.level.pathfinder.Path;
import net.minecraft.world.entity.ai.village.poi.PoiType;
import net.minecraft.world.entity.ai.Brain;
import net.minecraft.world.entity.EntityType;
import java.util.Optional;
import net.minecraft.network.protocol.game.DebugPackets;
import net.minecraft.core.BlockPos;
import net.minecraft.world.entity.Entity;
import net.minecraft.world.entity.LivingEntity;
import net.minecraft.server.level.ServerLevel;
import java.util.Map;
import com.google.common.collect.ImmutableMap;
import net.minecraft.world.entity.ai.memory.MemoryStatus;
import net.minecraft.world.entity.ai.memory.MemoryModuleType;
import net.minecraft.world.entity.npc.Villager;

public class MakeLove extends Behavior<Villager> {
    private long birthTimestamp;
    
    public MakeLove() {
        super(ImmutableMap.<MemoryModuleType<Villager>, MemoryStatus>of(MemoryModuleType.BREED_TARGET, MemoryStatus.VALUE_PRESENT, (MemoryModuleType<Villager>)MemoryModuleType.VISIBLE_LIVING_ENTITIES, MemoryStatus.VALUE_PRESENT), 350, 350);
    }
    
    @Override
    protected boolean checkExtraStartConditions(final ServerLevel xd, final Villager axt) {
        return this.isBreedingPossible(axt);
    }
    
    @Override
    protected boolean canStillUse(final ServerLevel xd, final Villager axt, final long long3) {
        return long3 <= this.birthTimestamp && this.isBreedingPossible(axt);
    }
    
    @Override
    protected void start(final ServerLevel xd, final Villager axt, final long long3) {
        final Villager axt2 = this.getBreedingTarget(axt);
        BehaviorUtils.lockGazeAndWalkToEachOther(axt, axt2);
        xd.broadcastEntityEvent(axt2, (byte)18);
        xd.broadcastEntityEvent(axt, (byte)18);
        final int integer7 = 275 + axt.getRandom().nextInt(50);
        this.birthTimestamp = long3 + integer7;
    }
    
    @Override
    protected void tick(final ServerLevel xd, final Villager axt, final long long3) {
        final Villager axt2 = this.getBreedingTarget(axt);
        if (axt.distanceToSqr(axt2) > 5.0) {
            return;
        }
        BehaviorUtils.lockGazeAndWalkToEachOther(axt, axt2);
        if (long3 >= this.birthTimestamp) {
            axt.eatAndDigestFood();
            axt2.eatAndDigestFood();
            this.tryToGiveBirth(xd, axt, axt2);
        }
        else if (axt.getRandom().nextInt(35) == 0) {
            xd.broadcastEntityEvent(axt2, (byte)12);
            xd.broadcastEntityEvent(axt, (byte)12);
        }
    }
    
    private void tryToGiveBirth(final ServerLevel xd, final Villager axt2, final Villager axt3) {
        final Optional<BlockPos> optional5 = this.takeVacantBed(xd, axt2);
        if (!optional5.isPresent()) {
            xd.broadcastEntityEvent(axt3, (byte)13);
            xd.broadcastEntityEvent(axt2, (byte)13);
        }
        else {
            final Optional<Villager> optional6 = this.breed(axt2, axt3);
            if (optional6.isPresent()) {
                this.giveBedToChild(xd, optional6.get(), optional5.get());
            }
            else {
                xd.getPoiManager().release(optional5.get());
                DebugPackets.sendPoiTicketCountPacket(xd, optional5.get());
            }
        }
    }
    
    @Override
    protected void tick(final ServerLevel xd, final Villager axt, final long long3) {
        axt.getBrain().<Villager>eraseMemory(MemoryModuleType.BREED_TARGET);
    }
    
    private Villager getBreedingTarget(final Villager axt) {
        return axt.getBrain().<Villager>getMemory(MemoryModuleType.BREED_TARGET).get();
    }
    
    private boolean isBreedingPossible(final Villager axt) {
        final Brain<Villager> alm3 = axt.getBrain();
        if (!alm3.<Villager>getMemory(MemoryModuleType.BREED_TARGET).isPresent()) {
            return false;
        }
        final Villager axt2 = this.getBreedingTarget(axt);
        return BehaviorUtils.targetIsValid(alm3, MemoryModuleType.BREED_TARGET, EntityType.VILLAGER) && axt.canBreed() && axt2.canBreed();
    }
    
    private Optional<BlockPos> takeVacantBed(final ServerLevel xd, final Villager axt) {
        return xd.getPoiManager().take(PoiType.HOME.getPredicate(), fk -> this.canReach(axt, fk), new BlockPos(axt), 48);
    }
    
    private boolean canReach(final Villager axt, final BlockPos fk) {
        final Path cqn4 = axt.getNavigation().createPath(fk, PoiType.HOME.getValidRange());
        return cqn4 != null && cqn4.canReach();
    }
    
    private Optional<Villager> breed(final Villager axt1, final Villager axt2) {
        final Villager axt3 = axt1.getBreedOffspring((AgableMob)axt2);
        if (axt3 == null) {
            return Optional.<Villager>empty();
        }
        axt1.setAge(6000);
        axt2.setAge(6000);
        axt3.setAge(-24000);
        axt3.moveTo(axt1.getX(), axt1.getY(), axt1.getZ(), 0.0f, 0.0f);
        axt1.level.addFreshEntity(axt3);
        axt1.level.broadcastEntityEvent(axt3, (byte)12);
        return Optional.<Villager>of(axt3);
    }
    
    private void giveBedToChild(final ServerLevel xd, final Villager axt, final BlockPos fk) {
        final GlobalPos fr5 = GlobalPos.of(xd.getDimension().getType(), fk);
        axt.getBrain().<GlobalPos>setMemory(MemoryModuleType.HOME, fr5);
    }
}
