package net.minecraft.world.entity.ai.behavior;

import net.minecraft.world.entity.LivingEntity;
import net.minecraft.world.entity.Entity;
import net.minecraft.world.entity.ai.util.RandomPos;
import net.minecraft.world.phys.Vec3;
import net.minecraft.world.entity.PathfinderMob;
import net.minecraft.core.Vec3i;
import net.minecraft.world.entity.ai.navigation.PathNavigation;
import java.util.Optional;
import net.minecraft.world.entity.ai.Brain;
import net.minecraft.world.entity.ai.memory.WalkTarget;
import net.minecraft.server.level.ServerLevel;
import java.util.Map;
import com.google.common.collect.ImmutableMap;
import net.minecraft.world.entity.ai.memory.MemoryStatus;
import net.minecraft.world.entity.ai.memory.MemoryModuleType;
import net.minecraft.core.BlockPos;
import javax.annotation.Nullable;
import net.minecraft.world.level.pathfinder.Path;
import net.minecraft.world.entity.Mob;

public class MoveToTargetSink extends Behavior<Mob> {
    @Nullable
    private Path path;
    @Nullable
    private BlockPos lastTargetPos;
    private float speed;
    private int remainingDelay;
    
    public MoveToTargetSink(final int integer) {
        super(ImmutableMap.<MemoryModuleType<Path>, MemoryStatus>of(MemoryModuleType.PATH, MemoryStatus.VALUE_ABSENT, (MemoryModuleType<Path>)MemoryModuleType.WALK_TARGET, MemoryStatus.VALUE_PRESENT), integer);
    }
    
    @Override
    protected boolean checkExtraStartConditions(final ServerLevel xd, final Mob akx) {
        final Brain<?> alm4 = akx.getBrain();
        final WalkTarget ark5 = alm4.<WalkTarget>getMemory(MemoryModuleType.WALK_TARGET).get();
        if (!this.reachedTarget(akx, ark5) && this.tryComputePath(akx, ark5, xd.getGameTime())) {
            this.lastTargetPos = ark5.getTarget().getPos();
            return true;
        }
        alm4.<WalkTarget>eraseMemory(MemoryModuleType.WALK_TARGET);
        return false;
    }
    
    @Override
    protected boolean canStillUse(final ServerLevel xd, final Mob akx, final long long3) {
        if (this.path == null || this.lastTargetPos == null) {
            return false;
        }
        final Optional<WalkTarget> optional6 = akx.getBrain().<WalkTarget>getMemory(MemoryModuleType.WALK_TARGET);
        final PathNavigation aro7 = akx.getNavigation();
        return !aro7.isDone() && optional6.isPresent() && !this.reachedTarget(akx, optional6.get());
    }
    
    @Override
    protected void stop(final ServerLevel xd, final Mob akx, final long long3) {
        akx.getNavigation().stop();
        akx.getBrain().<WalkTarget>eraseMemory(MemoryModuleType.WALK_TARGET);
        akx.getBrain().<Path>eraseMemory(MemoryModuleType.PATH);
        this.path = null;
    }
    
    @Override
    protected void start(final ServerLevel xd, final Mob akx, final long long3) {
        akx.getBrain().<Path>setMemory(MemoryModuleType.PATH, this.path);
        akx.getNavigation().moveTo(this.path, this.speed);
        this.remainingDelay = xd.getRandom().nextInt(10);
    }
    
    @Override
    protected void tick(final ServerLevel xd, final Mob akx, final long long3) {
        --this.remainingDelay;
        if (this.remainingDelay > 0) {
            return;
        }
        final Path cqn6 = akx.getNavigation().getPath();
        final Brain<?> alm7 = akx.getBrain();
        if (this.path != cqn6) {
            this.path = cqn6;
            alm7.<Path>setMemory(MemoryModuleType.PATH, cqn6);
        }
        if (cqn6 == null || this.lastTargetPos == null) {
            return;
        }
        final WalkTarget ark8 = alm7.<WalkTarget>getMemory(MemoryModuleType.WALK_TARGET).get();
        if (ark8.getTarget().getPos().distSqr(this.lastTargetPos) > 4.0 && this.tryComputePath(akx, ark8, xd.getGameTime())) {
            this.lastTargetPos = ark8.getTarget().getPos();
            this.start(xd, akx, long3);
        }
    }
    
    private boolean tryComputePath(final Mob akx, final WalkTarget ark, final long long3) {
        final BlockPos fk6 = ark.getTarget().getPos();
        this.path = akx.getNavigation().createPath(fk6, 0);
        this.speed = ark.getSpeed();
        if (!this.reachedTarget(akx, ark)) {
            final Brain<?> alm7 = akx.getBrain();
            final boolean boolean8 = this.path != null && this.path.canReach();
            if (boolean8) {
                alm7.<Long>setMemory(MemoryModuleType.CANT_REACH_WALK_TARGET_SINCE, Optional.<Long>empty());
            }
            else if (!alm7.hasMemoryValue(MemoryModuleType.CANT_REACH_WALK_TARGET_SINCE)) {
                alm7.<Long>setMemory(MemoryModuleType.CANT_REACH_WALK_TARGET_SINCE, long3);
            }
            if (this.path != null) {
                return true;
            }
            final Vec3 cvi9 = RandomPos.getPosTowards((PathfinderMob)akx, 10, 7, new Vec3(fk6));
            if (cvi9 != null) {
                this.path = akx.getNavigation().createPath(cvi9.x, cvi9.y, cvi9.z, 0);
                return this.path != null;
            }
        }
        return false;
    }
    
    private boolean reachedTarget(final Mob akx, final WalkTarget ark) {
        return ark.getTarget().getPos().distManhattan(new BlockPos(akx)) <= ark.getCloseEnoughDist();
    }
}
