package net.minecraft.world.entity.ai.behavior;

import javax.annotation.Nullable;
import java.util.function.Predicate;
import com.google.common.collect.Maps;
import java.util.function.Function;
import java.util.Comparator;
import net.minecraft.world.entity.ai.Brain;
import net.minecraft.world.entity.Entity;
import net.minecraft.world.phys.Vec3;
import net.minecraft.world.entity.ai.memory.WalkTarget;
import net.minecraft.core.BlockPos;
import net.minecraft.world.entity.ai.util.RandomPos;
import java.util.Optional;
import net.minecraft.server.level.ServerLevel;
import java.util.Map;
import net.minecraft.world.entity.LivingEntity;
import java.util.List;
import com.google.common.collect.ImmutableMap;
import net.minecraft.world.entity.ai.memory.MemoryStatus;
import net.minecraft.world.entity.ai.memory.MemoryModuleType;
import net.minecraft.world.entity.PathfinderMob;

public class PlayTagWithOtherKids extends Behavior<PathfinderMob> {
    public PlayTagWithOtherKids() {
        super(ImmutableMap.<MemoryModuleType<List<LivingEntity>>, MemoryStatus>of(MemoryModuleType.VISIBLE_VILLAGER_BABIES, MemoryStatus.VALUE_PRESENT, (MemoryModuleType<List<LivingEntity>>)MemoryModuleType.WALK_TARGET, MemoryStatus.VALUE_ABSENT, (MemoryModuleType<List<LivingEntity>>)MemoryModuleType.LOOK_TARGET, MemoryStatus.REGISTERED, (MemoryModuleType<List<LivingEntity>>)MemoryModuleType.INTERACTION_TARGET, MemoryStatus.REGISTERED));
    }
    
    @Override
    protected boolean checkExtraStartConditions(final ServerLevel xd, final PathfinderMob ald) {
        return xd.getRandom().nextInt(10) == 0 && this.hasFriendsNearby(ald);
    }
    
    @Override
    protected void start(final ServerLevel xd, final PathfinderMob ald, final long long3) {
        final LivingEntity akw2 = this.seeIfSomeoneIsChasingMe(ald);
        if (akw2 != null) {
            this.fleeFromChaser(xd, ald, akw2);
            return;
        }
        final Optional<LivingEntity> optional7 = this.findSomeoneBeingChased(ald);
        if (optional7.isPresent()) {
            chaseKid(ald, optional7.get());
            return;
        }
        this.findSomeoneToChase(ald).ifPresent(akw -> chaseKid(ald, akw));
    }
    
    private void fleeFromChaser(final ServerLevel xd, final PathfinderMob ald, final LivingEntity akw) {
        for (int integer5 = 0; integer5 < 10; ++integer5) {
            final Vec3 cvi6 = RandomPos.getLandPos(ald, 20, 8);
            if (cvi6 != null && xd.isVillage(new BlockPos(cvi6))) {
                ald.getBrain().<WalkTarget>setMemory(MemoryModuleType.WALK_TARGET, new WalkTarget(cvi6, 0.6f, 0));
                return;
            }
        }
    }
    
    private static void chaseKid(final PathfinderMob ald, final LivingEntity akw) {
        final Brain<?> alm3 = ald.getBrain();
        alm3.<LivingEntity>setMemory(MemoryModuleType.INTERACTION_TARGET, akw);
        alm3.<EntityPosWrapper>setMemory((MemoryModuleType<EntityPosWrapper>)MemoryModuleType.LOOK_TARGET, new EntityPosWrapper(akw));
        alm3.<WalkTarget>setMemory(MemoryModuleType.WALK_TARGET, new WalkTarget(new EntityPosWrapper(akw), 0.6f, 1));
    }
    
    private Optional<LivingEntity> findSomeoneToChase(final PathfinderMob ald) {
        return this.getFriendsNearby(ald).stream().findAny();
    }
    
    private Optional<LivingEntity> findSomeoneBeingChased(final PathfinderMob ald) {
        final Map<LivingEntity, Integer> map3 = this.checkHowManyChasersEachFriendHas(ald);
        return map3.entrySet().stream().sorted(Comparator.comparingInt(Map.Entry::getValue)).filter(entry -> entry.getValue() > 0 && entry.getValue() <= 5).<LivingEntity>map(Map.Entry::getKey).findFirst();
    }
    
    private Map<LivingEntity, Integer> checkHowManyChasersEachFriendHas(final PathfinderMob ald) {
        final Map<LivingEntity, Integer> map3 = Maps.newHashMap();
        final Integer n;
        this.getFriendsNearby(ald).stream().filter(this::isChasingSomeone).forEach(akw -> n = map3.compute(this.whoAreYouChasing(akw), (akw, integer) -> (integer == null) ? 1 : (integer + 1)));
        return map3;
    }
    
    private List<LivingEntity> getFriendsNearby(final PathfinderMob ald) {
        return ald.getBrain().<List<LivingEntity>>getMemory(MemoryModuleType.VISIBLE_VILLAGER_BABIES).get();
    }
    
    private LivingEntity whoAreYouChasing(final LivingEntity akw) {
        return akw.getBrain().<LivingEntity>getMemory(MemoryModuleType.INTERACTION_TARGET).get();
    }
    
    @Nullable
    private LivingEntity seeIfSomeoneIsChasingMe(final LivingEntity akw) {
        return akw.getBrain().<List<LivingEntity>>getMemory(MemoryModuleType.VISIBLE_VILLAGER_BABIES).get().stream().filter(akw2 -> this.isFriendChasingMe(akw, akw2)).findAny().orElse(null);
    }
    
    private boolean isChasingSomeone(final LivingEntity akw) {
        return akw.getBrain().<LivingEntity>getMemory(MemoryModuleType.INTERACTION_TARGET).isPresent();
    }
    
    private boolean isFriendChasingMe(final LivingEntity akw1, final LivingEntity akw2) {
        return akw2.getBrain().<LivingEntity>getMemory(MemoryModuleType.INTERACTION_TARGET).filter(akw2 -> akw2 == akw1).isPresent();
    }
    
    private boolean hasFriendsNearby(final PathfinderMob ald) {
        return ald.getBrain().hasMemoryValue(MemoryModuleType.VISIBLE_VILLAGER_BABIES);
    }
}
