package net.minecraft.world.entity.ai.behavior;

import net.minecraft.world.entity.ai.Brain;
import net.minecraft.world.entity.Entity;
import java.util.List;
import net.minecraft.server.level.ServerLevel;
import java.util.Map;
import com.google.common.collect.ImmutableMap;
import net.minecraft.world.entity.ai.memory.MemoryStatus;
import net.minecraft.world.entity.ai.memory.MemoryModuleType;
import java.util.function.Predicate;
import net.minecraft.world.entity.EntityType;
import net.minecraft.world.entity.LivingEntity;

public class SetLookAndInteract extends Behavior<LivingEntity> {
    private final EntityType<?> type;
    private final int interactionRangeSqr;
    private final Predicate<LivingEntity> targetFilter;
    private final Predicate<LivingEntity> selfFilter;
    
    public SetLookAndInteract(final EntityType<?> akr, final int integer, final Predicate<LivingEntity> predicate3, final Predicate<LivingEntity> predicate4) {
        super(ImmutableMap.<MemoryModuleType<PositionWrapper>, MemoryStatus>of(MemoryModuleType.LOOK_TARGET, MemoryStatus.REGISTERED, (MemoryModuleType<PositionWrapper>)MemoryModuleType.INTERACTION_TARGET, MemoryStatus.VALUE_ABSENT, (MemoryModuleType<PositionWrapper>)MemoryModuleType.VISIBLE_LIVING_ENTITIES, MemoryStatus.VALUE_PRESENT));
        this.type = akr;
        this.interactionRangeSqr = integer * integer;
        this.targetFilter = predicate4;
        this.selfFilter = predicate3;
    }
    
    public SetLookAndInteract(final EntityType<?> akr, final int integer) {
        this(akr, integer, akw -> true, akw -> true);
    }
    
    public boolean checkExtraStartConditions(final ServerLevel xd, final LivingEntity akw) {
        return this.selfFilter.test(akw) && this.getVisibleEntities(akw).stream().anyMatch(this::isMatchingTarget);
    }
    
    public void start(final ServerLevel xd, final LivingEntity akw, final long long3) {
        super.start(xd, akw, long3);
        final Brain<?> alm6 = akw.getBrain();
        final Brain brain;
        alm6.<List<LivingEntity>>getMemory(MemoryModuleType.VISIBLE_LIVING_ENTITIES).ifPresent(list -> list.stream().filter(akw2 -> akw2.distanceToSqr(akw) <= this.interactionRangeSqr).filter(this::isMatchingTarget).findFirst().ifPresent(akw -> {
            brain.setMemory(MemoryModuleType.INTERACTION_TARGET, akw);
            brain.setMemory(MemoryModuleType.LOOK_TARGET, new EntityPosWrapper(akw));
        }));
    }
    
    private boolean isMatchingTarget(final LivingEntity akw) {
        return this.type.equals(akw.getType()) && this.targetFilter.test(akw);
    }
    
    private List<LivingEntity> getVisibleEntities(final LivingEntity akw) {
        return akw.getBrain().<List<LivingEntity>>getMemory(MemoryModuleType.VISIBLE_LIVING_ENTITIES).get();
    }
}
