package net.minecraft.world.entity.ai.behavior;

import java.util.Iterator;
import net.minecraft.world.item.trading.MerchantOffer;
import net.minecraft.world.entity.EquipmentSlot;
import net.minecraft.world.entity.ai.Brain;
import net.minecraft.world.entity.Entity;
import net.minecraft.world.entity.EntityType;
import net.minecraft.server.level.ServerLevel;
import com.google.common.collect.Lists;
import java.util.Map;
import net.minecraft.world.entity.LivingEntity;
import com.google.common.collect.ImmutableMap;
import net.minecraft.world.entity.ai.memory.MemoryStatus;
import net.minecraft.world.entity.ai.memory.MemoryModuleType;
import java.util.List;
import javax.annotation.Nullable;
import net.minecraft.world.item.ItemStack;
import net.minecraft.world.entity.npc.Villager;

public class ShowTradesToPlayer extends Behavior<Villager> {
    @Nullable
    private ItemStack playerItemStack;
    private final List<ItemStack> displayItems;
    private int cycleCounter;
    private int displayIndex;
    private int lookTime;
    
    public ShowTradesToPlayer(final int integer1, final int integer2) {
        super(ImmutableMap.<MemoryModuleType<LivingEntity>, MemoryStatus>of(MemoryModuleType.INTERACTION_TARGET, MemoryStatus.VALUE_PRESENT), integer1, integer2);
        this.displayItems = Lists.newArrayList();
    }
    
    public boolean checkExtraStartConditions(final ServerLevel xd, final Villager axt) {
        final Brain<?> alm4 = axt.getBrain();
        if (!alm4.<LivingEntity>getMemory(MemoryModuleType.INTERACTION_TARGET).isPresent()) {
            return false;
        }
        final LivingEntity akw5 = alm4.<LivingEntity>getMemory(MemoryModuleType.INTERACTION_TARGET).get();
        return akw5.getType() == EntityType.PLAYER && axt.isAlive() && akw5.isAlive() && !axt.isBaby() && axt.distanceToSqr(akw5) <= 17.0;
    }
    
    public boolean canStillUse(final ServerLevel xd, final Villager axt, final long long3) {
        return this.checkExtraStartConditions(xd, axt) && this.lookTime > 0 && axt.getBrain().<LivingEntity>getMemory(MemoryModuleType.INTERACTION_TARGET).isPresent();
    }
    
    public void start(final ServerLevel xd, final Villager axt, final long long3) {
        super.start(xd, axt, long3);
        this.lookAtTarget(axt);
        this.cycleCounter = 0;
        this.displayIndex = 0;
        this.lookTime = 40;
    }
    
    public void tick(final ServerLevel xd, final Villager axt, final long long3) {
        final LivingEntity akw6 = this.lookAtTarget(axt);
        this.findItemsToDisplay(akw6, axt);
        if (!this.displayItems.isEmpty()) {
            this.displayCyclingItems(axt);
        }
        else {
            axt.setItemSlot(EquipmentSlot.MAINHAND, ItemStack.EMPTY);
            this.lookTime = Math.min(this.lookTime, 40);
        }
        --this.lookTime;
    }
    
    public void tick(final ServerLevel xd, final Villager axt, final long long3) {
        super.stop(xd, axt, long3);
        axt.getBrain().<LivingEntity>eraseMemory(MemoryModuleType.INTERACTION_TARGET);
        axt.setItemSlot(EquipmentSlot.MAINHAND, ItemStack.EMPTY);
        this.playerItemStack = null;
    }
    
    private void findItemsToDisplay(final LivingEntity akw, final Villager axt) {
        boolean boolean4 = false;
        final ItemStack bek5 = akw.getMainHandItem();
        if (this.playerItemStack == null || !ItemStack.isSame(this.playerItemStack, bek5)) {
            this.playerItemStack = bek5;
            boolean4 = true;
            this.displayItems.clear();
        }
        if (boolean4 && !this.playerItemStack.isEmpty()) {
            this.updateDisplayItems(axt);
            if (!this.displayItems.isEmpty()) {
                this.lookTime = 900;
                this.displayFirstItem(axt);
            }
        }
    }
    
    private void displayFirstItem(final Villager axt) {
        axt.setItemSlot(EquipmentSlot.MAINHAND, this.displayItems.get(0));
    }
    
    private void updateDisplayItems(final Villager axt) {
        for (final MerchantOffer biw4 : axt.getOffers()) {
            if (!biw4.isOutOfStock() && this.playerItemStackMatchesCostOfOffer(biw4)) {
                this.displayItems.add(biw4.getResult());
            }
        }
    }
    
    private boolean playerItemStackMatchesCostOfOffer(final MerchantOffer biw) {
        return ItemStack.isSame(this.playerItemStack, biw.getCostA()) || ItemStack.isSame(this.playerItemStack, biw.getCostB());
    }
    
    private LivingEntity lookAtTarget(final Villager axt) {
        final Brain<?> alm3 = axt.getBrain();
        final LivingEntity akw4 = alm3.<LivingEntity>getMemory(MemoryModuleType.INTERACTION_TARGET).get();
        alm3.<EntityPosWrapper>setMemory((MemoryModuleType<EntityPosWrapper>)MemoryModuleType.LOOK_TARGET, new EntityPosWrapper(akw4));
        return akw4;
    }
    
    private void displayCyclingItems(final Villager axt) {
        if (this.displayItems.size() >= 2 && ++this.cycleCounter >= 40) {
            ++this.displayIndex;
            this.cycleCounter = 0;
            if (this.displayIndex > this.displayItems.size() - 1) {
                this.displayIndex = 0;
            }
            axt.setItemSlot(EquipmentSlot.MAINHAND, this.displayItems.get(this.displayIndex));
        }
    }
}
