package net.minecraft.world.entity.ai.behavior;

import net.minecraft.world.entity.LivingEntity;
import net.minecraft.world.entity.ai.Brain;
import java.util.Optional;
import net.minecraft.core.Position;
import java.util.Objects;
import net.minecraft.server.level.ServerLevel;
import java.util.Map;
import net.minecraft.world.entity.ai.memory.WalkTarget;
import com.google.common.collect.ImmutableMap;
import net.minecraft.world.entity.ai.memory.MemoryStatus;
import net.minecraft.core.GlobalPos;
import net.minecraft.world.entity.ai.memory.MemoryModuleType;
import net.minecraft.world.entity.PathfinderMob;

public class StrollToPoi extends Behavior<PathfinderMob> {
    private final MemoryModuleType<GlobalPos> memoryType;
    private final int closeEnoughDist;
    private final int maxDistanceFromPoi;
    private long nextOkStartTime;
    
    public StrollToPoi(final MemoryModuleType<GlobalPos> ari, final int integer2, final int integer3) {
        super(ImmutableMap.<MemoryModuleType<WalkTarget>, MemoryStatus>of(MemoryModuleType.WALK_TARGET, MemoryStatus.REGISTERED, (MemoryModuleType<WalkTarget>)ari, MemoryStatus.VALUE_PRESENT));
        this.memoryType = ari;
        this.closeEnoughDist = integer2;
        this.maxDistanceFromPoi = integer3;
    }
    
    @Override
    protected boolean checkExtraStartConditions(final ServerLevel xd, final PathfinderMob ald) {
        final Optional<GlobalPos> optional4 = ald.getBrain().<GlobalPos>getMemory(this.memoryType);
        return optional4.isPresent() && Objects.equals(xd.getDimension().getType(), optional4.get().dimension()) && optional4.get().pos().closerThan(ald.position(), this.maxDistanceFromPoi);
    }
    
    @Override
    protected void start(final ServerLevel xd, final PathfinderMob ald, final long long3) {
        if (long3 > this.nextOkStartTime) {
            final Brain<?> alm6 = ald.getBrain();
            final Optional<GlobalPos> optional7 = alm6.<GlobalPos>getMemory(this.memoryType);
            optional7.ifPresent(fr -> alm6.<WalkTarget>setMemory(MemoryModuleType.WALK_TARGET, new WalkTarget(fr.pos(), 0.4f, this.closeEnoughDist)));
            this.nextOkStartTime = long3 + 80L;
        }
    }
}
