package net.minecraft.world.entity.ai.behavior;

import net.minecraft.world.SimpleContainer;
import net.minecraft.world.level.ItemLike;
import net.minecraft.world.item.ItemStack;
import java.util.stream.Collector;
import java.util.stream.Collectors;
import net.minecraft.world.entity.npc.VillagerProfession;
import net.minecraft.world.entity.Entity;
import net.minecraft.world.entity.ai.Brain;
import net.minecraft.world.entity.EntityType;
import net.minecraft.server.level.ServerLevel;
import com.google.common.collect.ImmutableSet;
import java.util.Map;
import net.minecraft.world.entity.LivingEntity;
import com.google.common.collect.ImmutableMap;
import net.minecraft.world.entity.ai.memory.MemoryStatus;
import net.minecraft.world.entity.ai.memory.MemoryModuleType;
import net.minecraft.world.item.Item;
import java.util.Set;
import net.minecraft.world.entity.npc.Villager;

public class TradeWithVillager extends Behavior<Villager> {
    private Set<Item> trades;
    
    public TradeWithVillager() {
        super(ImmutableMap.<MemoryModuleType<LivingEntity>, MemoryStatus>of(MemoryModuleType.INTERACTION_TARGET, MemoryStatus.VALUE_PRESENT, (MemoryModuleType<LivingEntity>)MemoryModuleType.VISIBLE_LIVING_ENTITIES, MemoryStatus.VALUE_PRESENT));
        this.trades = ImmutableSet.of();
    }
    
    @Override
    protected boolean checkExtraStartConditions(final ServerLevel xd, final Villager axt) {
        return BehaviorUtils.targetIsValid(axt.getBrain(), MemoryModuleType.INTERACTION_TARGET, EntityType.VILLAGER);
    }
    
    @Override
    protected boolean canStillUse(final ServerLevel xd, final Villager axt, final long long3) {
        return this.checkExtraStartConditions(xd, axt);
    }
    
    @Override
    protected void start(final ServerLevel xd, final Villager axt, final long long3) {
        final Villager axt2 = axt.getBrain().<Villager>getMemory((MemoryModuleType<Villager>)MemoryModuleType.INTERACTION_TARGET).get();
        BehaviorUtils.lockGazeAndWalkToEachOther(axt, axt2);
        this.trades = figureOutWhatIAmWillingToTrade(axt, axt2);
    }
    
    @Override
    protected void tick(final ServerLevel xd, final Villager axt, final long long3) {
        final Villager axt2 = axt.getBrain().<Villager>getMemory((MemoryModuleType<Villager>)MemoryModuleType.INTERACTION_TARGET).get();
        if (axt.distanceToSqr(axt2) > 5.0) {
            return;
        }
        BehaviorUtils.lockGazeAndWalkToEachOther(axt, axt2);
        axt.gossip(axt2, long3);
        if (axt.hasExcessFood() && (axt.getVillagerData().getProfession() == VillagerProfession.FARMER || axt2.wantsMoreFood())) {
            throwHalfStack(axt, Villager.FOOD_POINTS.keySet(), axt2);
        }
        if (!this.trades.isEmpty() && axt.getInventory().hasAnyOf(this.trades)) {
            throwHalfStack(axt, this.trades, axt2);
        }
    }
    
    @Override
    protected void tick(final ServerLevel xd, final Villager axt, final long long3) {
        axt.getBrain().<LivingEntity>eraseMemory(MemoryModuleType.INTERACTION_TARGET);
    }
    
    private static Set<Item> figureOutWhatIAmWillingToTrade(final Villager axt1, final Villager axt2) {
        final ImmutableSet<Item> immutableSet3 = axt2.getVillagerData().getProfession().getRequestedItems();
        final ImmutableSet<Item> immutableSet4 = axt1.getVillagerData().getProfession().getRequestedItems();
        return immutableSet3.stream().filter(bef -> !immutableSet4.contains(bef)).collect(Collectors.toSet());
    }
    
    private static void throwHalfStack(final Villager axt, final Set<Item> set, final LivingEntity akw) {
        final SimpleContainer ajn4 = axt.getInventory();
        ItemStack bek5 = ItemStack.EMPTY;
        for (int integer6 = 0; integer6 < ajn4.getContainerSize(); ++integer6) {
            final ItemStack bek6 = ajn4.getItem(integer6);
            if (!bek6.isEmpty()) {
                final Item bef8 = bek6.getItem();
                if (set.contains(bef8)) {
                    int integer7;
                    if (bek6.getCount() > bek6.getMaxStackSize() / 2) {
                        integer7 = bek6.getCount() / 2;
                    }
                    else {
                        if (bek6.getCount() <= 24) {
                            continue;
                        }
                        integer7 = bek6.getCount() - 24;
                    }
                    bek6.shrink(integer7);
                    bek5 = new ItemStack(bef8, integer7);
                    break;
                }
            }
        }
        if (!bek5.isEmpty()) {
            BehaviorUtils.throwItem(axt, bek5, akw);
        }
    }
}
