package net.minecraft.world.entity.ai.behavior;

import net.minecraft.world.level.block.state.BlockState;
import net.minecraft.world.level.block.state.properties.Property;
import net.minecraft.world.level.block.BedBlock;
import net.minecraft.tags.BlockTags;
import net.minecraft.core.BlockPos;
import net.minecraft.world.entity.ai.Brain;
import net.minecraft.network.protocol.game.DebugPackets;
import net.minecraft.core.Position;
import java.util.Objects;
import net.minecraft.server.level.ServerLevel;
import java.util.Map;
import com.google.common.collect.ImmutableMap;
import net.minecraft.world.entity.ai.memory.MemoryStatus;
import net.minecraft.world.entity.ai.village.poi.PoiType;
import java.util.function.Predicate;
import net.minecraft.core.GlobalPos;
import net.minecraft.world.entity.ai.memory.MemoryModuleType;
import net.minecraft.world.entity.LivingEntity;

public class ValidateNearbyPoi extends Behavior<LivingEntity> {
    private final MemoryModuleType<GlobalPos> memoryType;
    private final Predicate<PoiType> poiPredicate;
    
    public ValidateNearbyPoi(final PoiType asr, final MemoryModuleType<GlobalPos> ari) {
        super(ImmutableMap.<MemoryModuleType<GlobalPos>, MemoryStatus>of(ari, MemoryStatus.VALUE_PRESENT));
        this.poiPredicate = asr.getPredicate();
        this.memoryType = ari;
    }
    
    @Override
    protected boolean checkExtraStartConditions(final ServerLevel xd, final LivingEntity akw) {
        final GlobalPos fr4 = akw.getBrain().<GlobalPos>getMemory(this.memoryType).get();
        return Objects.equals(xd.getDimension().getType(), fr4.dimension()) && fr4.pos().closerThan(akw.position(), 5.0);
    }
    
    @Override
    protected void start(final ServerLevel xd, final LivingEntity akw, final long long3) {
        final Brain<?> alm6 = akw.getBrain();
        final GlobalPos fr7 = alm6.<GlobalPos>getMemory(this.memoryType).get();
        final BlockPos fk8 = fr7.pos();
        final ServerLevel xd2 = xd.getServer().getLevel(fr7.dimension());
        if (this.poiDoesntExist(xd2, fk8)) {
            alm6.<GlobalPos>eraseMemory(this.memoryType);
        }
        else if (this.bedIsOccupied(xd2, fk8, akw)) {
            alm6.<GlobalPos>eraseMemory(this.memoryType);
            xd.getPoiManager().release(fk8);
            DebugPackets.sendPoiTicketCountPacket(xd, fk8);
        }
    }
    
    private boolean bedIsOccupied(final ServerLevel xd, final BlockPos fk, final LivingEntity akw) {
        final BlockState byg5 = xd.getBlockState(fk);
        return byg5.getBlock().is(BlockTags.BEDS) && byg5.<Boolean>getValue((Property<Boolean>)BedBlock.OCCUPIED) && !akw.isSleeping();
    }
    
    private boolean poiDoesntExist(final ServerLevel xd, final BlockPos fk) {
        return !xd.getPoiManager().exists(fk, this.poiPredicate);
    }
}
