package net.minecraft.world.entity.ai.behavior;

import net.minecraft.world.entity.ai.Brain;
import net.minecraft.world.entity.ai.memory.WalkTarget;
import net.minecraft.world.level.pathfinder.Path;
import net.minecraft.world.entity.ai.memory.MemoryModuleType;
import net.minecraft.world.entity.schedule.Activity;
import net.minecraft.world.entity.LivingEntity;
import net.minecraft.server.level.ServerLevel;
import java.util.Map;
import com.google.common.collect.ImmutableMap;
import net.minecraft.world.entity.npc.Villager;

public class VillagerPanicTrigger extends Behavior<Villager> {
    public VillagerPanicTrigger() {
        super(ImmutableMap.of());
    }
    
    @Override
    protected boolean canStillUse(final ServerLevel xd, final Villager axt, final long long3) {
        return isHurt(axt) || hasHostile(axt);
    }
    
    @Override
    protected void start(final ServerLevel xd, final Villager axt, final long long3) {
        if (isHurt(axt) || hasHostile(axt)) {
            final Brain<?> alm6 = axt.getBrain();
            if (!alm6.isActive(Activity.PANIC)) {
                alm6.<Path>eraseMemory(MemoryModuleType.PATH);
                alm6.<WalkTarget>eraseMemory(MemoryModuleType.WALK_TARGET);
                alm6.<PositionWrapper>eraseMemory(MemoryModuleType.LOOK_TARGET);
                alm6.<Villager>eraseMemory(MemoryModuleType.BREED_TARGET);
                alm6.<LivingEntity>eraseMemory(MemoryModuleType.INTERACTION_TARGET);
            }
            alm6.setActivity(Activity.PANIC);
        }
    }
    
    @Override
    protected void tick(final ServerLevel xd, final Villager axt, final long long3) {
        if (long3 % 100L == 0L) {
            axt.spawnGolemIfNeeded(long3, 3);
        }
    }
    
    public static boolean hasHostile(final LivingEntity akw) {
        return akw.getBrain().hasMemoryValue(MemoryModuleType.NEAREST_HOSTILE);
    }
    
    public static boolean isHurt(final LivingEntity akw) {
        return akw.getBrain().hasMemoryValue(MemoryModuleType.HURT_BY);
    }
}
