package net.minecraft.world.entity.ai.goal;

import net.minecraft.world.level.block.Block;
import net.minecraft.world.level.block.state.BlockState;
import net.minecraft.world.level.block.state.properties.BedPart;
import net.minecraft.world.level.block.BedBlock;
import net.minecraft.tags.BlockTags;
import net.minecraft.world.level.block.state.properties.Property;
import net.minecraft.world.level.block.FurnaceBlock;
import net.minecraft.world.level.BlockGetter;
import net.minecraft.world.level.block.entity.ChestBlockEntity;
import net.minecraft.world.level.block.Blocks;
import net.minecraft.core.BlockPos;
import net.minecraft.world.level.LevelReader;
import net.minecraft.world.entity.PathfinderMob;
import net.minecraft.world.entity.animal.Cat;

public class CatSitOnBlockGoal extends MoveToBlockGoal {
    private final Cat cat;
    
    public CatSitOnBlockGoal(final Cat atb, final double double2) {
        super(atb, double2, 8);
        this.cat = atb;
    }
    
    @Override
    public boolean canUse() {
        return this.cat.isTame() && !this.cat.isSitting() && super.canUse();
    }
    
    @Override
    public void start() {
        super.start();
        this.cat.getSitGoal().wantToSit(false);
    }
    
    @Override
    public void stop() {
        super.stop();
        this.cat.setSitting(false);
    }
    
    @Override
    public void tick() {
        super.tick();
        this.cat.getSitGoal().wantToSit(false);
        if (!this.isReachedTarget()) {
            this.cat.setSitting(false);
        }
        else if (!this.cat.isSitting()) {
            this.cat.setSitting(true);
        }
    }
    
    @Override
    protected boolean isValidTarget(final LevelReader bjw, final BlockPos fk) {
        if (!bjw.isEmptyBlock(fk.above())) {
            return false;
        }
        final BlockState byg4 = bjw.getBlockState(fk);
        final Block bpe5 = byg4.getBlock();
        if (bpe5 == Blocks.CHEST) {
            return ChestBlockEntity.getOpenCount(bjw, fk) < 1;
        }
        return (bpe5 == Blocks.FURNACE && byg4.<Boolean>getValue((Property<Boolean>)FurnaceBlock.LIT)) || (bpe5.is(BlockTags.BEDS) && byg4.<BedPart>getValue(BedBlock.PART) != BedPart.HEAD);
    }
}
