package net.minecraft.world.entity.ai.goal;

import net.minecraft.world.InteractionHand;
import net.minecraft.world.entity.EntitySelector;
import net.minecraft.world.entity.player.Player;
import net.minecraft.core.BlockPos;
import net.minecraft.world.entity.LivingEntity;
import net.minecraft.world.entity.Entity;
import java.util.EnumSet;
import net.minecraft.world.level.pathfinder.Path;
import net.minecraft.world.entity.PathfinderMob;

public class MeleeAttackGoal extends Goal {
    protected final PathfinderMob mob;
    protected int attackTime;
    private final double speedModifier;
    private final boolean trackTarget;
    private Path path;
    private int timeToRecalcPath;
    private double pathedTargetX;
    private double pathedTargetY;
    private double pathedTargetZ;
    protected final int attackInterval = 20;
    private long lastUpdate;
    
    public MeleeAttackGoal(final PathfinderMob ald, final double double2, final boolean boolean3) {
        this.mob = ald;
        this.speedModifier = double2;
        this.trackTarget = boolean3;
        this.setFlags(EnumSet.<Flag>of(Flag.MOVE, Flag.LOOK));
    }
    
    @Override
    public boolean canUse() {
        final long long2 = this.mob.level.getGameTime();
        if (long2 - this.lastUpdate < 20L) {
            return false;
        }
        this.lastUpdate = long2;
        final LivingEntity akw4 = this.mob.getTarget();
        if (akw4 == null) {
            return false;
        }
        if (!akw4.isAlive()) {
            return false;
        }
        this.path = this.mob.getNavigation().createPath(akw4, 0);
        return this.path != null || this.getAttackReachSqr(akw4) >= this.mob.distanceToSqr(akw4.getX(), akw4.getY(), akw4.getZ());
    }
    
    @Override
    public boolean canContinueToUse() {
        final LivingEntity akw2 = this.mob.getTarget();
        if (akw2 == null) {
            return false;
        }
        if (!akw2.isAlive()) {
            return false;
        }
        if (!this.trackTarget) {
            return !this.mob.getNavigation().isDone();
        }
        return this.mob.isWithinRestriction(new BlockPos(akw2)) && (!(akw2 instanceof Player) || (!akw2.isSpectator() && !((Player)akw2).isCreative()));
    }
    
    @Override
    public void start() {
        this.mob.getNavigation().moveTo(this.path, this.speedModifier);
        this.mob.setAggressive(true);
        this.timeToRecalcPath = 0;
    }
    
    @Override
    public void stop() {
        final LivingEntity akw2 = this.mob.getTarget();
        if (!EntitySelector.NO_CREATIVE_OR_SPECTATOR.test(akw2)) {
            this.mob.setTarget(null);
        }
        this.mob.setAggressive(false);
        this.mob.getNavigation().stop();
    }
    
    @Override
    public void tick() {
        final LivingEntity akw2 = this.mob.getTarget();
        this.mob.getLookControl().setLookAt(akw2, 30.0f, 30.0f);
        final double double3 = this.mob.distanceToSqr(akw2.getX(), akw2.getY(), akw2.getZ());
        --this.timeToRecalcPath;
        if ((this.trackTarget || this.mob.getSensing().canSee(akw2)) && this.timeToRecalcPath <= 0 && ((this.pathedTargetX == 0.0 && this.pathedTargetY == 0.0 && this.pathedTargetZ == 0.0) || akw2.distanceToSqr(this.pathedTargetX, this.pathedTargetY, this.pathedTargetZ) >= 1.0 || this.mob.getRandom().nextFloat() < 0.05f)) {
            this.pathedTargetX = akw2.getX();
            this.pathedTargetY = akw2.getY();
            this.pathedTargetZ = akw2.getZ();
            this.timeToRecalcPath = 4 + this.mob.getRandom().nextInt(7);
            if (double3 > 1024.0) {
                this.timeToRecalcPath += 10;
            }
            else if (double3 > 256.0) {
                this.timeToRecalcPath += 5;
            }
            if (!this.mob.getNavigation().moveTo(akw2, this.speedModifier)) {
                this.timeToRecalcPath += 15;
            }
        }
        this.attackTime = Math.max(this.attackTime - 1, 0);
        this.checkAndPerformAttack(akw2, double3);
    }
    
    protected void checkAndPerformAttack(final LivingEntity akw, final double double2) {
        final double double3 = this.getAttackReachSqr(akw);
        if (double2 <= double3 && this.attackTime <= 0) {
            this.attackTime = 20;
            this.mob.swing(InteractionHand.MAIN_HAND);
            this.mob.doHurtTarget(akw);
        }
    }
    
    protected double getAttackReachSqr(final LivingEntity akw) {
        return this.mob.getBbWidth() * 2.0f * (this.mob.getBbWidth() * 2.0f) + akw.getBbWidth();
    }
}
