package net.minecraft.world.entity.ai.goal.target;

import net.minecraft.world.entity.ai.targeting.TargetingConditions;
import java.util.EnumSet;
import net.minecraft.world.entity.ai.goal.Goal;
import net.minecraft.world.entity.Mob;
import net.minecraft.world.entity.LivingEntity;
import net.minecraft.world.entity.TamableAnimal;

public class OwnerHurtByTargetGoal extends TargetGoal {
    private final TamableAnimal tameAnimal;
    private LivingEntity ownerLastHurtBy;
    private int timestamp;
    
    public OwnerHurtByTargetGoal(final TamableAnimal all) {
        super(all, false);
        this.tameAnimal = all;
        this.setFlags(EnumSet.<Flag>of(Flag.TARGET));
    }
    
    @Override
    public boolean canUse() {
        if (!this.tameAnimal.isTame() || this.tameAnimal.isSitting()) {
            return false;
        }
        final LivingEntity akw2 = this.tameAnimal.getOwner();
        if (akw2 == null) {
            return false;
        }
        this.ownerLastHurtBy = akw2.getLastHurtByMob();
        final int integer3 = akw2.getLastHurtByMobTimestamp();
        return integer3 != this.timestamp && this.canAttack(this.ownerLastHurtBy, TargetingConditions.DEFAULT) && this.tameAnimal.wantsToAttack(this.ownerLastHurtBy, akw2);
    }
    
    @Override
    public void start() {
        this.mob.setTarget(this.ownerLastHurtBy);
        final LivingEntity akw2 = this.tameAnimal.getOwner();
        if (akw2 != null) {
            this.timestamp = akw2.getLastHurtByMobTimestamp();
        }
        super.start();
    }
}
