package net.minecraft.world.entity.ai.goal.target;

import net.minecraft.world.level.pathfinder.Node;
import net.minecraft.world.level.pathfinder.Path;
import net.minecraft.util.Mth;
import net.minecraft.core.BlockPos;
import net.minecraft.world.entity.ai.targeting.TargetingConditions;
import javax.annotation.Nullable;
import net.minecraft.world.entity.ai.attributes.AttributeInstance;
import net.minecraft.world.entity.monster.SharedMonsterAttributes;
import net.minecraft.world.scores.Team;
import net.minecraft.world.entity.player.Player;
import net.minecraft.world.entity.Entity;
import net.minecraft.world.entity.LivingEntity;
import net.minecraft.world.entity.Mob;
import net.minecraft.world.entity.ai.goal.Goal;

public abstract class TargetGoal extends Goal {
    protected final Mob mob;
    protected final boolean mustSee;
    private final boolean mustReach;
    private int reachCache;
    private int reachCacheTime;
    private int unseenTicks;
    protected LivingEntity targetMob;
    protected int unseenMemoryTicks;
    
    public TargetGoal(final Mob akx, final boolean boolean2) {
        this(akx, boolean2, false);
    }
    
    public TargetGoal(final Mob akx, final boolean boolean2, final boolean boolean3) {
        this.unseenMemoryTicks = 60;
        this.mob = akx;
        this.mustSee = boolean2;
        this.mustReach = boolean3;
    }
    
    @Override
    public boolean canContinueToUse() {
        LivingEntity akw2 = this.mob.getTarget();
        if (akw2 == null) {
            akw2 = this.targetMob;
        }
        if (akw2 == null) {
            return false;
        }
        if (!akw2.isAlive()) {
            return false;
        }
        final Team cwk3 = this.mob.getTeam();
        final Team cwk4 = akw2.getTeam();
        if (cwk3 != null && cwk4 == cwk3) {
            return false;
        }
        final double double5 = this.getFollowDistance();
        if (this.mob.distanceToSqr(akw2) > double5 * double5) {
            return false;
        }
        if (this.mustSee) {
            if (this.mob.getSensing().canSee(akw2)) {
                this.unseenTicks = 0;
            }
            else if (++this.unseenTicks > this.unseenMemoryTicks) {
                return false;
            }
        }
        if (akw2 instanceof Player && ((Player)akw2).abilities.invulnerable) {
            return false;
        }
        this.mob.setTarget(akw2);
        return true;
    }
    
    protected double getFollowDistance() {
        final AttributeInstance alo2 = this.mob.getAttribute(SharedMonsterAttributes.FOLLOW_RANGE);
        return (alo2 == null) ? 16.0 : alo2.getValue();
    }
    
    @Override
    public void start() {
        this.reachCache = 0;
        this.reachCacheTime = 0;
        this.unseenTicks = 0;
    }
    
    @Override
    public void stop() {
        this.mob.setTarget(null);
        this.targetMob = null;
    }
    
    protected boolean canAttack(@Nullable final LivingEntity akw, final TargetingConditions ash) {
        if (akw == null) {
            return false;
        }
        if (!ash.test(this.mob, akw)) {
            return false;
        }
        if (!this.mob.isWithinRestriction(new BlockPos(akw))) {
            return false;
        }
        if (this.mustReach) {
            if (--this.reachCacheTime <= 0) {
                this.reachCache = 0;
            }
            if (this.reachCache == 0) {
                this.reachCache = (this.canReach(akw) ? 1 : 2);
            }
            if (this.reachCache == 2) {
                return false;
            }
        }
        return true;
    }
    
    private boolean canReach(final LivingEntity akw) {
        this.reachCacheTime = 10 + this.mob.getRandom().nextInt(5);
        final Path cqn3 = this.mob.getNavigation().createPath(akw, 0);
        if (cqn3 == null) {
            return false;
        }
        final Node cql4 = cqn3.last();
        if (cql4 == null) {
            return false;
        }
        final int integer5 = cql4.x - Mth.floor(akw.getX());
        final int integer6 = cql4.z - Mth.floor(akw.getZ());
        return integer5 * integer5 + integer6 * integer6 <= 2.25;
    }
    
    public TargetGoal setUnseenMemoryTicks(final int integer) {
        this.unseenMemoryTicks = integer;
        return this;
    }
}
