package net.minecraft.world.entity.ai.util;

import java.util.function.Predicate;
import net.minecraft.util.Mth;
import net.minecraft.world.level.pathfinder.BlockPathTypes;
import java.util.Random;
import net.minecraft.world.entity.ai.navigation.PathNavigation;
import net.minecraft.core.Vec3i;
import net.minecraft.world.level.BlockGetter;
import net.minecraft.world.level.pathfinder.WalkNodeEvaluator;
import net.minecraft.tags.FluidTags;
import net.minecraft.world.entity.Entity;
import net.minecraft.core.Position;
import net.minecraft.core.BlockPos;
import java.util.function.ToDoubleFunction;
import javax.annotation.Nullable;
import net.minecraft.world.phys.Vec3;
import net.minecraft.world.entity.PathfinderMob;

public class RandomPos {
    @Nullable
    public static Vec3 getPos(final PathfinderMob ald, final int integer2, final int integer3) {
        return generateRandomPos(ald, integer2, integer3, 0, null, true, 1.5707963705062866, ald::getWalkTargetValue, false, 0, 0, true);
    }
    
    @Nullable
    public static Vec3 getAirPos(final PathfinderMob ald, final int integer2, final int integer3, final int integer4, @Nullable final Vec3 cvi, final double double6) {
        return generateRandomPos(ald, integer2, integer3, integer4, cvi, true, double6, ald::getWalkTargetValue, true, 0, 0, false);
    }
    
    @Nullable
    public static Vec3 getLandPos(final PathfinderMob ald, final int integer2, final int integer3) {
        return getLandPos(ald, integer2, integer3, ald::getWalkTargetValue);
    }
    
    @Nullable
    public static Vec3 getLandPos(final PathfinderMob ald, final int integer2, final int integer3, final ToDoubleFunction<BlockPos> toDoubleFunction) {
        return generateRandomPos(ald, integer2, integer3, 0, null, false, 0.0, toDoubleFunction, true, 0, 0, true);
    }
    
    @Nullable
    public static Vec3 getAboveLandPos(final PathfinderMob ald, final int integer2, final int integer3, final Vec3 cvi, final float float5, final int integer6, final int integer7) {
        return generateRandomPos(ald, integer2, integer3, 0, cvi, false, float5, ald::getWalkTargetValue, true, integer6, integer7, true);
    }
    
    @Nullable
    public static Vec3 getPosTowards(final PathfinderMob ald, final int integer2, final int integer3, final Vec3 cvi) {
        final Vec3 cvi2 = cvi.subtract(ald.getX(), ald.getY(), ald.getZ());
        return generateRandomPos(ald, integer2, integer3, 0, cvi2, true, 1.5707963705062866, ald::getWalkTargetValue, false, 0, 0, true);
    }
    
    @Nullable
    public static Vec3 getPosTowards(final PathfinderMob ald, final int integer2, final int integer3, final Vec3 cvi, final double double5) {
        final Vec3 cvi2 = cvi.subtract(ald.getX(), ald.getY(), ald.getZ());
        return generateRandomPos(ald, integer2, integer3, 0, cvi2, true, double5, ald::getWalkTargetValue, false, 0, 0, true);
    }
    
    @Nullable
    public static Vec3 getAirPosTowards(final PathfinderMob ald, final int integer2, final int integer3, final int integer4, final Vec3 cvi, final double double6) {
        final Vec3 cvi2 = cvi.subtract(ald.getX(), ald.getY(), ald.getZ());
        return generateRandomPos(ald, integer2, integer3, integer4, cvi2, false, double6, ald::getWalkTargetValue, true, 0, 0, false);
    }
    
    @Nullable
    public static Vec3 getPosAvoid(final PathfinderMob ald, final int integer2, final int integer3, final Vec3 cvi) {
        final Vec3 cvi2 = ald.position().subtract(cvi);
        return generateRandomPos(ald, integer2, integer3, 0, cvi2, true, 1.5707963705062866, ald::getWalkTargetValue, false, 0, 0, true);
    }
    
    @Nullable
    public static Vec3 getLandPosAvoid(final PathfinderMob ald, final int integer2, final int integer3, final Vec3 cvi) {
        final Vec3 cvi2 = ald.position().subtract(cvi);
        return generateRandomPos(ald, integer2, integer3, 0, cvi2, false, 1.5707963705062866, ald::getWalkTargetValue, true, 0, 0, true);
    }
    
    @Nullable
    private static Vec3 generateRandomPos(final PathfinderMob ald, final int integer2, final int integer3, final int integer4, @Nullable final Vec3 cvi, final boolean boolean6, final double double7, final ToDoubleFunction<BlockPos> toDoubleFunction, final boolean boolean9, final int integer10, final int integer11, final boolean boolean12) {
        final PathNavigation aro14 = ald.getNavigation();
        final Random random15 = ald.getRandom();
        final boolean boolean13 = ald.hasRestriction() && ald.getRestrictCenter().closerThan(ald.position(), ald.getRestrictRadius() + integer2 + 1.0);
        boolean boolean14 = false;
        double double8 = Double.NEGATIVE_INFINITY;
        BlockPos fk2 = new BlockPos(ald);
        for (int integer12 = 0; integer12 < 10; ++integer12) {
            final BlockPos fk3 = getRandomDelta(random15, integer2, integer3, integer4, cvi, double7);
            if (fk3 != null) {
                int integer13 = fk3.getX();
                final int integer14 = fk3.getY();
                int integer15 = fk3.getZ();
                if (ald.hasRestriction() && integer2 > 1) {
                    final BlockPos fk4 = ald.getRestrictCenter();
                    if (ald.getX() > fk4.getX()) {
                        integer13 -= random15.nextInt(integer2 / 2);
                    }
                    else {
                        integer13 += random15.nextInt(integer2 / 2);
                    }
                    if (ald.getZ() > fk4.getZ()) {
                        integer15 -= random15.nextInt(integer2 / 2);
                    }
                    else {
                        integer15 += random15.nextInt(integer2 / 2);
                    }
                }
                BlockPos fk4 = new BlockPos(integer13 + ald.getX(), integer14 + ald.getY(), integer15 + ald.getZ());
                if (fk4.getY() >= 0) {
                    if (fk4.getY() <= ald.level.getMaxBuildHeight()) {
                        if (!boolean13 || ald.isWithinRestriction(fk4)) {
                            if (!boolean12 || aro14.isStableDestination(fk4)) {
                                if (boolean9) {
                                    fk4 = moveUpToAboveSolid(fk4, random15.nextInt(integer10 + 1) + integer11, ald.level.getMaxBuildHeight(), fk -> ald.level.getBlockState(fk).getMaterial().isSolid());
                                }
                                if (boolean6 || !ald.level.getFluidState(fk4).is(FluidTags.WATER)) {
                                    final BlockPathTypes cqj27 = WalkNodeEvaluator.getBlockPathTypeStatic(ald.level, fk4.getX(), fk4.getY(), fk4.getZ());
                                    if (ald.getPathfindingMalus(cqj27) == 0.0f) {
                                        final double double9 = toDoubleFunction.applyAsDouble(fk4);
                                        if (double9 > double8) {
                                            double8 = double9;
                                            fk2 = fk4;
                                            boolean14 = true;
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
        if (boolean14) {
            return new Vec3(fk2);
        }
        return null;
    }
    
    @Nullable
    private static BlockPos getRandomDelta(final Random random, final int integer2, final int integer3, final int integer4, @Nullable final Vec3 cvi, final double double6) {
        if (cvi == null || double6 >= 3.141592653589793) {
            final int integer5 = random.nextInt(2 * integer2 + 1) - integer2;
            final int integer6 = random.nextInt(2 * integer3 + 1) - integer3 + integer4;
            final int integer7 = random.nextInt(2 * integer2 + 1) - integer2;
            return new BlockPos(integer5, integer6, integer7);
        }
        final double double7 = Mth.atan2(cvi.z, cvi.x) - 1.5707963705062866;
        final double double8 = double7 + (2.0f * random.nextFloat() - 1.0f) * double6;
        final double double9 = Math.sqrt(random.nextDouble()) * Mth.SQRT_OF_TWO * integer2;
        final double double10 = -double9 * Math.sin(double8);
        final double double11 = double9 * Math.cos(double8);
        if (Math.abs(double10) > integer2 || Math.abs(double11) > integer2) {
            return null;
        }
        final int integer8 = random.nextInt(2 * integer3 + 1) - integer3 + integer4;
        return new BlockPos(double10, integer8, double11);
    }
    
    static BlockPos moveUpToAboveSolid(final BlockPos fk, final int integer2, final int integer3, final Predicate<BlockPos> predicate) {
        if (integer2 < 0) {
            throw new IllegalArgumentException("aboveSolidAmount was " + integer2 + ", expected >= 0");
        }
        if (predicate.test(fk)) {
            BlockPos fk2;
            for (fk2 = fk.above(); fk2.getY() < integer3 && predicate.test(fk2); fk2 = fk2.above()) {}
            BlockPos fk3;
            BlockPos fk4;
            for (fk3 = fk2; fk3.getY() < integer3 && fk3.getY() - fk2.getY() < integer2; fk3 = fk4) {
                fk4 = fk3.above();
                if (predicate.test(fk4)) {
                    break;
                }
            }
            return fk3;
        }
        return fk;
    }
}
