package net.minecraft.world.entity.animal.horse;

import net.minecraft.world.item.enchantment.EnchantmentHelper;
import net.minecraft.world.level.ItemLike;
import net.minecraft.world.item.ItemStack;
import net.minecraft.world.item.Items;
import net.minecraft.world.entity.EquipmentSlot;
import net.minecraft.nbt.CompoundTag;
import net.minecraft.world.entity.SpawnGroupData;
import net.minecraft.world.level.LevelAccessor;
import net.minecraft.world.entity.MobSpawnType;
import net.minecraft.world.entity.EntityType;
import net.minecraft.world.entity.monster.Skeleton;
import net.minecraft.world.DifficultyInstance;
import net.minecraft.world.entity.global.LightningBolt;
import net.minecraft.server.level.ServerLevel;
import net.minecraft.world.entity.Entity;
import net.minecraft.core.BlockPos;
import net.minecraft.world.entity.ai.goal.Goal;

public class SkeletonTrapGoal extends Goal {
    private final SkeletonHorse horse;
    
    public SkeletonTrapGoal(final SkeletonHorse aug) {
        this.horse = aug;
    }
    
    @Override
    public boolean canUse() {
        return this.horse.level.hasNearbyAlivePlayer(this.horse.getX(), this.horse.getY(), this.horse.getZ(), 10.0);
    }
    
    @Override
    public void tick() {
        final DifficultyInstance ajg2 = this.horse.level.getCurrentDifficultyAt(new BlockPos(this.horse));
        this.horse.setTrap(false);
        this.horse.setTamed(true);
        this.horse.setAge(0);
        ((ServerLevel)this.horse.level).addGlobalEntity(new LightningBolt(this.horse.level, this.horse.getX(), this.horse.getY(), this.horse.getZ(), true));
        final Skeleton axd3 = this.createSkeleton(ajg2, this.horse);
        axd3.startRiding(this.horse);
        for (int integer4 = 0; integer4 < 3; ++integer4) {
            final AbstractHorse aub5 = this.createHorse(ajg2);
            final Skeleton axd4 = this.createSkeleton(ajg2, aub5);
            axd4.startRiding(aub5);
            aub5.push(this.horse.getRandom().nextGaussian() * 0.5, 0.0, this.horse.getRandom().nextGaussian() * 0.5);
        }
    }
    
    private AbstractHorse createHorse(final DifficultyInstance ajg) {
        final SkeletonHorse aug3 = EntityType.SKELETON_HORSE.create(this.horse.level);
        aug3.finalizeSpawn(this.horse.level, ajg, MobSpawnType.TRIGGERED, null, null);
        aug3.setPos(this.horse.getX(), this.horse.getY(), this.horse.getZ());
        aug3.invulnerableTime = 60;
        aug3.setPersistenceRequired();
        aug3.setTamed(true);
        aug3.setAge(0);
        aug3.level.addFreshEntity(aug3);
        return aug3;
    }
    
    private Skeleton createSkeleton(final DifficultyInstance ajg, final AbstractHorse aub) {
        final Skeleton axd4 = EntityType.SKELETON.create(aub.level);
        axd4.finalizeSpawn(aub.level, ajg, MobSpawnType.TRIGGERED, null, null);
        axd4.setPos(aub.getX(), aub.getY(), aub.getZ());
        axd4.invulnerableTime = 60;
        axd4.setPersistenceRequired();
        if (axd4.getItemBySlot(EquipmentSlot.HEAD).isEmpty()) {
            axd4.setItemSlot(EquipmentSlot.HEAD, new ItemStack(Items.IRON_HELMET));
        }
        axd4.setItemSlot(EquipmentSlot.MAINHAND, EnchantmentHelper.enchantItem(axd4.getRandom(), axd4.getMainHandItem(), (int)(5.0f + ajg.getSpecialMultiplier() * axd4.getRandom().nextInt(18)), false));
        axd4.setItemSlot(EquipmentSlot.HEAD, EnchantmentHelper.enchantItem(axd4.getRandom(), axd4.getItemBySlot(EquipmentSlot.HEAD), (int)(5.0f + ajg.getSpecialMultiplier() * axd4.getRandom().nextInt(18)), false));
        axd4.level.addFreshEntity(axd4);
        return axd4;
    }
}
