package net.minecraft.world.entity.boss.enderdragon.phases;

import org.apache.logging.log4j.LogManager;
import net.minecraft.world.entity.boss.enderdragon.EnderDragon;
import org.apache.logging.log4j.Logger;

public class EnderDragonPhaseManager {
    private static final Logger LOGGER;
    private final EnderDragon dragon;
    private final DragonPhaseInstance[] phases;
    private DragonPhaseInstance currentPhase;
    
    public EnderDragonPhaseManager(final EnderDragon aup) {
        this.phases = new DragonPhaseInstance[EnderDragonPhase.getCount()];
        this.dragon = aup;
        this.setPhase(EnderDragonPhase.HOVERING);
    }
    
    public void setPhase(final EnderDragonPhase<?> avg) {
        if (this.currentPhase != null && avg == this.currentPhase.getPhase()) {
            return;
        }
        if (this.currentPhase != null) {
            this.currentPhase.end();
        }
        this.currentPhase = this.<DragonPhaseInstance>getPhase(avg);
        if (!this.dragon.level.isClientSide) {
            this.dragon.getEntityData().<Integer>set(EnderDragon.DATA_PHASE, avg.getId());
        }
        EnderDragonPhaseManager.LOGGER.debug("Dragon is now in phase {} on the {}", avg, (this.dragon.level.isClientSide ? "client" : "server"));
        this.currentPhase.begin();
    }
    
    public DragonPhaseInstance getCurrentPhase() {
        return this.currentPhase;
    }
    
    public <T extends DragonPhaseInstance> T getPhase(final EnderDragonPhase<T> avg) {
        final int integer3 = avg.getId();
        if (this.phases[integer3] == null) {
            this.phases[integer3] = avg.createInstance(this.dragon);
        }
        return (T)this.phases[integer3];
    }
    
    static {
        LOGGER = LogManager.getLogger();
    }
}
