package net.minecraft.world.entity.decoration;

import net.minecraft.network.syncher.SynchedEntityData;
import net.minecraft.network.syncher.EntityDataSerializers;
import org.apache.logging.log4j.LogManager;
import net.minecraft.network.protocol.game.ClientboundAddEntityPacket;
import net.minecraft.network.protocol.Packet;
import net.minecraft.world.InteractionHand;
import net.minecraft.nbt.Tag;
import net.minecraft.nbt.CompoundTag;
import net.minecraft.world.level.block.Blocks;
import net.minecraft.world.level.saveddata.maps.MapItemSavedData;
import net.minecraft.world.item.MapItem;
import net.minecraft.world.level.ItemLike;
import net.minecraft.world.item.Items;
import net.minecraft.world.entity.player.Player;
import net.minecraft.world.level.GameRules;
import javax.annotation.Nullable;
import net.minecraft.sounds.SoundEvents;
import net.minecraft.world.damagesource.DamageSource;
import net.minecraft.world.level.block.state.BlockState;
import java.util.function.Predicate;
import net.minecraft.world.level.block.DiodeBlock;
import net.minecraft.world.entity.Entity;
import net.minecraft.world.phys.AABB;
import org.apache.commons.lang3.Validate;
import net.minecraft.world.entity.EntityDimensions;
import net.minecraft.world.entity.Pose;
import net.minecraft.core.Direction;
import net.minecraft.core.BlockPos;
import net.minecraft.world.level.Level;
import net.minecraft.world.entity.EntityType;
import net.minecraft.world.item.ItemStack;
import net.minecraft.network.syncher.EntityDataAccessor;
import org.apache.logging.log4j.Logger;

public class ItemFrame extends HangingEntity {
    private static final Logger LOGGER;
    private static final EntityDataAccessor<ItemStack> DATA_ITEM;
    private static final EntityDataAccessor<Integer> DATA_ROTATION;
    private float dropChance;
    
    public ItemFrame(final EntityType<? extends ItemFrame> akr, final Level bjt) {
        super(akr, bjt);
        this.dropChance = 1.0f;
    }
    
    public ItemFrame(final Level bjt, final BlockPos fk, final Direction fp) {
        super(EntityType.ITEM_FRAME, bjt, fk);
        this.dropChance = 1.0f;
        this.setDirection(fp);
    }
    
    @Override
    protected float getEyeHeight(final Pose alg, final EntityDimensions ako) {
        return 0.0f;
    }
    
    @Override
    protected void defineSynchedData() {
        this.getEntityData().<ItemStack>define(ItemFrame.DATA_ITEM, ItemStack.EMPTY);
        this.getEntityData().<Integer>define(ItemFrame.DATA_ROTATION, 0);
    }
    
    @Override
    protected void setDirection(final Direction fp) {
        Validate.notNull(fp);
        this.direction = fp;
        if (fp.getAxis().isHorizontal()) {
            this.xRot = 0.0f;
            this.yRot = (float)(this.direction.get2DDataValue() * 90);
        }
        else {
            this.xRot = (float)(-90 * fp.getAxisDirection().getStep());
            this.yRot = 0.0f;
        }
        this.xRotO = this.xRot;
        this.yRotO = this.yRot;
        this.recalculateBoundingBox();
    }
    
    @Override
    protected void recalculateBoundingBox() {
        if (this.direction == null) {
            return;
        }
        final double double2 = 0.46875;
        final double double3 = this.pos.getX() + 0.5 - this.direction.getStepX() * 0.46875;
        final double double4 = this.pos.getY() + 0.5 - this.direction.getStepY() * 0.46875;
        final double double5 = this.pos.getZ() + 0.5 - this.direction.getStepZ() * 0.46875;
        this.setPosRaw(double3, double4, double5);
        double double6 = this.getWidth();
        double double7 = this.getHeight();
        double double8 = this.getWidth();
        final Direction.Axis a16 = this.direction.getAxis();
        switch (a16) {
            case X: {
                double6 = 1.0;
                break;
            }
            case Y: {
                double7 = 1.0;
                break;
            }
            case Z: {
                double8 = 1.0;
                break;
            }
        }
        double6 /= 32.0;
        double7 /= 32.0;
        double8 /= 32.0;
        this.setBoundingBox(new AABB(double3 - double6, double4 - double7, double5 - double8, double3 + double6, double4 + double7, double5 + double8));
    }
    
    @Override
    public boolean survives() {
        if (!this.level.noCollision(this)) {
            return false;
        }
        final BlockState byg2 = this.level.getBlockState(this.pos.relative(this.direction.getOpposite()));
        return (byg2.getMaterial().isSolid() || (this.direction.getAxis().isHorizontal() && DiodeBlock.isDiode(byg2))) && this.level.getEntities(this, this.getBoundingBox(), ItemFrame.HANGING_ENTITY).isEmpty();
    }
    
    @Override
    public float getPickRadius() {
        return 0.0f;
    }
    
    @Override
    public void kill() {
        this.removeFramedMap(this.getItem());
        super.kill();
    }
    
    @Override
    public boolean hurt(final DamageSource ajw, final float float2) {
        if (this.isInvulnerableTo(ajw)) {
            return false;
        }
        if (!ajw.isExplosion() && !this.getItem().isEmpty()) {
            if (!this.level.isClientSide) {
                this.dropItem(ajw.getEntity(), false);
                this.playSound(SoundEvents.ITEM_FRAME_REMOVE_ITEM, 1.0f, 1.0f);
            }
            return true;
        }
        return super.hurt(ajw, float2);
    }
    
    @Override
    public int getWidth() {
        return 12;
    }
    
    @Override
    public int getHeight() {
        return 12;
    }
    
    @Override
    public boolean shouldRenderAtSqrDistance(final double double1) {
        double double2 = 16.0;
        double2 *= 64.0 * getViewScale();
        return double1 < double2 * double2;
    }
    
    @Override
    public void dropItem(@Nullable final Entity akn) {
        this.playSound(SoundEvents.ITEM_FRAME_BREAK, 1.0f, 1.0f);
        this.dropItem(akn, true);
    }
    
    @Override
    public void playPlacementSound() {
        this.playSound(SoundEvents.ITEM_FRAME_PLACE, 1.0f, 1.0f);
    }
    
    private void dropItem(@Nullable final Entity akn, final boolean boolean2) {
        if (!this.level.getGameRules().getBoolean(GameRules.RULE_DOENTITYDROPS)) {
            if (akn == null) {
                this.removeFramedMap(this.getItem());
            }
            return;
        }
        ItemStack bek4 = this.getItem();
        this.setItem(ItemStack.EMPTY);
        if (akn instanceof Player) {
            final Player ayg5 = (Player)akn;
            if (ayg5.abilities.instabuild) {
                this.removeFramedMap(bek4);
                return;
            }
        }
        if (boolean2) {
            this.spawnAtLocation(Items.ITEM_FRAME);
        }
        if (!bek4.isEmpty()) {
            bek4 = bek4.copy();
            this.removeFramedMap(bek4);
            if (this.random.nextFloat() < this.dropChance) {
                this.spawnAtLocation(bek4);
            }
        }
    }
    
    private void removeFramedMap(final ItemStack bek) {
        if (bek.getItem() == Items.FILLED_MAP) {
            final MapItemSavedData crd3 = MapItem.getOrCreateSavedData(bek, this.level);
            crd3.removedFromFrame(this.pos, this.getId());
            crd3.setDirty(true);
        }
        bek.setFramed(null);
    }
    
    public ItemStack getItem() {
        return this.getEntityData().<ItemStack>get(ItemFrame.DATA_ITEM);
    }
    
    public void setItem(final ItemStack bek) {
        this.setItem(bek, true);
    }
    
    public void setItem(ItemStack bek, final boolean boolean2) {
        if (!bek.isEmpty()) {
            bek = bek.copy();
            bek.setCount(1);
            bek.setFramed(this);
        }
        this.getEntityData().<ItemStack>set(ItemFrame.DATA_ITEM, bek);
        if (!bek.isEmpty()) {
            this.playSound(SoundEvents.ITEM_FRAME_ADD_ITEM, 1.0f, 1.0f);
        }
        if (boolean2 && this.pos != null) {
            this.level.updateNeighbourForOutputSignal(this.pos, Blocks.AIR);
        }
    }
    
    @Override
    public boolean setSlot(final int integer, final ItemStack bek) {
        if (integer == 0) {
            this.setItem(bek);
            return true;
        }
        return false;
    }
    
    @Override
    public void onSyncedDataUpdated(final EntityDataAccessor<?> sb) {
        if (sb.equals(ItemFrame.DATA_ITEM)) {
            final ItemStack bek3 = this.getItem();
            if (!bek3.isEmpty() && bek3.getFrame() != this) {
                bek3.setFramed(this);
            }
        }
    }
    
    public int getRotation() {
        return this.getEntityData().<Integer>get(ItemFrame.DATA_ROTATION);
    }
    
    public void setRotation(final int integer) {
        this.setRotation(integer, true);
    }
    
    private void setRotation(final int integer, final boolean boolean2) {
        this.getEntityData().<Integer>set(ItemFrame.DATA_ROTATION, integer % 8);
        if (boolean2 && this.pos != null) {
            this.level.updateNeighbourForOutputSignal(this.pos, Blocks.AIR);
        }
    }
    
    @Override
    public void addAdditionalSaveData(final CompoundTag jt) {
        super.addAdditionalSaveData(jt);
        if (!this.getItem().isEmpty()) {
            jt.put("Item", this.getItem().save(new CompoundTag()));
            jt.putByte("ItemRotation", (byte)this.getRotation());
            jt.putFloat("ItemDropChance", this.dropChance);
        }
        jt.putByte("Facing", (byte)this.direction.get3DDataValue());
    }
    
    @Override
    public void readAdditionalSaveData(final CompoundTag jt) {
        super.readAdditionalSaveData(jt);
        final CompoundTag jt2 = jt.getCompound("Item");
        if (jt2 != null && !jt2.isEmpty()) {
            final ItemStack bek4 = ItemStack.of(jt2);
            if (bek4.isEmpty()) {
                ItemFrame.LOGGER.warn("Unable to load item from: {}", jt2);
            }
            final ItemStack bek5 = this.getItem();
            if (!bek5.isEmpty() && !ItemStack.matches(bek4, bek5)) {
                this.removeFramedMap(bek5);
            }
            this.setItem(bek4, false);
            this.setRotation(jt.getByte("ItemRotation"), false);
            if (jt.contains("ItemDropChance", 99)) {
                this.dropChance = jt.getFloat("ItemDropChance");
            }
        }
        this.setDirection(Direction.from3DDataValue(jt.getByte("Facing")));
    }
    
    @Override
    public boolean interact(final Player ayg, final InteractionHand ajh) {
        final ItemStack bek4 = ayg.getItemInHand(ajh);
        final boolean boolean5 = !this.getItem().isEmpty();
        final boolean boolean6 = !bek4.isEmpty();
        if (this.level.isClientSide) {
            return boolean5 || boolean6;
        }
        if (!boolean5) {
            if (boolean6) {
                this.setItem(bek4);
                if (!ayg.abilities.instabuild) {
                    bek4.shrink(1);
                }
            }
        }
        else {
            this.playSound(SoundEvents.ITEM_FRAME_ROTATE_ITEM, 1.0f, 1.0f);
            this.setRotation(this.getRotation() + 1);
        }
        return true;
    }
    
    public int getAnalogOutput() {
        if (this.getItem().isEmpty()) {
            return 0;
        }
        return this.getRotation() % 8 + 1;
    }
    
    @Override
    public Packet<?> getAddEntityPacket() {
        return new ClientboundAddEntityPacket(this, this.getType(), this.direction.get3DDataValue(), this.getPos());
    }
    
    static {
        LOGGER = LogManager.getLogger();
        DATA_ITEM = SynchedEntityData.<ItemStack>defineId(ItemFrame.class, EntityDataSerializers.ITEM_STACK);
        DATA_ROTATION = SynchedEntityData.<Integer>defineId(ItemFrame.class, EntityDataSerializers.INT);
    }
}
