package net.minecraft.world.entity.item;

import net.minecraft.network.syncher.SynchedEntityData;
import net.minecraft.network.syncher.EntityDataSerializers;
import net.minecraft.network.protocol.game.ClientboundAddEntityPacket;
import net.minecraft.network.protocol.Packet;
import javax.annotation.Nullable;
import net.minecraft.world.level.dimension.DimensionType;
import net.minecraft.network.chat.TranslatableComponent;
import net.minecraft.network.chat.Component;
import net.minecraft.world.item.Item;
import net.minecraft.stats.Stat;
import net.minecraft.stats.Stats;
import net.minecraft.world.entity.player.Player;
import net.minecraft.nbt.Tag;
import net.minecraft.nbt.NbtUtils;
import net.minecraft.nbt.CompoundTag;
import net.minecraft.world.item.Items;
import net.minecraft.world.damagesource.DamageSource;
import java.util.Objects;
import java.util.Iterator;
import java.util.List;
import net.minecraft.world.phys.Vec3;
import net.minecraft.sounds.SoundEvents;
import net.minecraft.util.Mth;
import net.minecraft.core.BlockPos;
import net.minecraft.world.entity.MoverType;
import net.minecraft.tags.FluidTags;
import net.minecraft.world.level.Level;
import net.minecraft.world.entity.EntityType;
import java.util.UUID;
import net.minecraft.world.item.ItemStack;
import net.minecraft.network.syncher.EntityDataAccessor;
import net.minecraft.world.entity.Entity;

public class ItemEntity extends Entity {
    private static final EntityDataAccessor<ItemStack> DATA_ITEM;
    private int age;
    private int pickupDelay;
    private int health;
    private UUID thrower;
    private UUID owner;
    public final float bobOffs;
    
    public ItemEntity(final EntityType<? extends ItemEntity> akr, final Level bjt) {
        super(akr, bjt);
        this.health = 5;
        this.bobOffs = (float)(Math.random() * 3.141592653589793 * 2.0);
    }
    
    public ItemEntity(final Level bjt, final double double2, final double double3, final double double4) {
        this(EntityType.ITEM, bjt);
        this.setPos(double2, double3, double4);
        this.yRot = this.random.nextFloat() * 360.0f;
        this.setDeltaMovement(this.random.nextDouble() * 0.2 - 0.1, 0.2, this.random.nextDouble() * 0.2 - 0.1);
    }
    
    public ItemEntity(final Level bjt, final double double2, final double double3, final double double4, final ItemStack bek) {
        this(bjt, double2, double3, double4);
        this.setItem(bek);
    }
    
    @Override
    protected boolean isMovementNoisy() {
        return false;
    }
    
    @Override
    protected void defineSynchedData() {
        this.getEntityData().<ItemStack>define(ItemEntity.DATA_ITEM, ItemStack.EMPTY);
    }
    
    @Override
    public void tick() {
        if (this.getItem().isEmpty()) {
            this.remove();
            return;
        }
        super.tick();
        if (this.pickupDelay > 0 && this.pickupDelay != 32767) {
            --this.pickupDelay;
        }
        this.xo = this.getX();
        this.yo = this.getY();
        this.zo = this.getZ();
        final Vec3 cvi2 = this.getDeltaMovement();
        if (this.isUnderLiquid(FluidTags.WATER)) {
            this.setUnderwaterMovement();
        }
        else if (!this.isNoGravity()) {
            this.setDeltaMovement(this.getDeltaMovement().add(0.0, -0.04, 0.0));
        }
        if (this.level.isClientSide) {
            this.noPhysics = false;
        }
        else {
            this.noPhysics = !this.level.noCollision(this);
            if (this.noPhysics) {
                this.checkInBlock(this.getX(), (this.getBoundingBox().minY + this.getBoundingBox().maxY) / 2.0, this.getZ());
            }
        }
        if (!this.onGround || Entity.getHorizontalDistanceSqr(this.getDeltaMovement()) > 9.999999747378752E-6 || (this.tickCount + this.getId()) % 4 == 0) {
            this.move(MoverType.SELF, this.getDeltaMovement());
            float float3 = 0.98f;
            if (this.onGround) {
                float3 = this.level.getBlockState(new BlockPos(this.getX(), this.getY() - 1.0, this.getZ())).getBlock().getFriction() * 0.98f;
            }
            this.setDeltaMovement(this.getDeltaMovement().multiply(float3, 0.98, float3));
            if (this.onGround) {
                this.setDeltaMovement(this.getDeltaMovement().multiply(1.0, -0.5, 1.0));
            }
        }
        final boolean boolean3 = Mth.floor(this.xo) != Mth.floor(this.getX()) || Mth.floor(this.yo) != Mth.floor(this.getY()) || Mth.floor(this.zo) != Mth.floor(this.getZ());
        final int integer4 = boolean3 ? 2 : 40;
        if (this.tickCount % integer4 == 0) {
            if (this.level.getFluidState(new BlockPos(this)).is(FluidTags.LAVA)) {
                this.setDeltaMovement((this.random.nextFloat() - this.random.nextFloat()) * 0.2f, 0.20000000298023224, (this.random.nextFloat() - this.random.nextFloat()) * 0.2f);
                this.playSound(SoundEvents.GENERIC_BURN, 0.4f, 2.0f + this.random.nextFloat() * 0.4f);
            }
            if (!this.level.isClientSide && this.isMergable()) {
                this.mergeWithNeighbours();
            }
        }
        if (this.age != -32768) {
            ++this.age;
        }
        this.hasImpulse |= this.updateInWaterState();
        if (!this.level.isClientSide) {
            final double double5 = this.getDeltaMovement().subtract(cvi2).lengthSqr();
            if (double5 > 0.01) {
                this.hasImpulse = true;
            }
        }
        if (!this.level.isClientSide && this.age >= 6000) {
            this.remove();
        }
    }
    
    private void setUnderwaterMovement() {
        final Vec3 cvi2 = this.getDeltaMovement();
        this.setDeltaMovement(cvi2.x * 0.9900000095367432, cvi2.y + ((cvi2.y < 0.05999999865889549) ? 5.0E-4f : 0.0f), cvi2.z * 0.9900000095367432);
    }
    
    private void mergeWithNeighbours() {
        if (!this.isMergable()) {
            return;
        }
        final List<ItemEntity> list2 = this.level.<ItemEntity>getEntitiesOfClass(ItemEntity.class, this.getBoundingBox().inflate(0.5, 0.0, 0.5), avy -> avy != this && avy.isMergable());
        for (final ItemEntity avy2 : list2) {
            if (avy2.isMergable()) {
                this.tryToMerge(avy2);
                if (this.removed) {
                    break;
                }
                continue;
            }
        }
    }
    
    private boolean isMergable() {
        final ItemStack bek2 = this.getItem();
        return this.isAlive() && this.pickupDelay != 32767 && this.age != -32768 && this.age < 6000 && bek2.getCount() < bek2.getMaxStackSize();
    }
    
    private void tryToMerge(final ItemEntity avy) {
        final ItemStack bek3 = this.getItem();
        final ItemStack bek4 = avy.getItem();
        if (!Objects.equals(this.getOwner(), avy.getOwner()) || !areMergable(bek3, bek4)) {
            return;
        }
        if (bek4.getCount() < bek3.getCount()) {
            merge(this, bek3, avy, bek4);
        }
        else {
            merge(avy, bek4, this, bek3);
        }
    }
    
    public static boolean areMergable(final ItemStack bek1, final ItemStack bek2) {
        return bek2.getItem() == bek1.getItem() && bek2.getCount() + bek1.getCount() <= bek2.getMaxStackSize() && !(bek2.hasTag() ^ bek1.hasTag()) && (!bek2.hasTag() || bek2.getTag().equals(bek1.getTag()));
    }
    
    public static ItemStack merge(final ItemStack bek1, final ItemStack bek2, final int integer) {
        final int integer2 = Math.min(Math.min(bek1.getMaxStackSize(), integer) - bek1.getCount(), bek2.getCount());
        final ItemStack bek3 = bek1.copy();
        bek3.grow(integer2);
        bek2.shrink(integer2);
        return bek3;
    }
    
    private static void merge(final ItemEntity avy, final ItemStack bek2, final ItemStack bek3) {
        final ItemStack bek4 = merge(bek2, bek3, 64);
        avy.setItem(bek4);
    }
    
    private static void merge(final ItemEntity avy1, final ItemStack bek2, final ItemEntity avy3, final ItemStack bek4) {
        merge(avy1, bek2, bek4);
        avy1.pickupDelay = Math.max(avy1.pickupDelay, avy3.pickupDelay);
        avy1.age = Math.min(avy1.age, avy3.age);
        if (bek4.isEmpty()) {
            avy3.remove();
        }
    }
    
    @Override
    protected void burn(final int integer) {
        this.hurt(DamageSource.IN_FIRE, (float)integer);
    }
    
    @Override
    public boolean hurt(final DamageSource ajw, final float float2) {
        if (this.isInvulnerableTo(ajw)) {
            return false;
        }
        if (!this.getItem().isEmpty() && this.getItem().getItem() == Items.NETHER_STAR && ajw.isExplosion()) {
            return false;
        }
        this.markHurt();
        this.health -= (int)float2;
        if (this.health <= 0) {
            this.remove();
        }
        return false;
    }
    
    public void addAdditionalSaveData(final CompoundTag jt) {
        jt.putShort("Health", (short)this.health);
        jt.putShort("Age", (short)this.age);
        jt.putShort("PickupDelay", (short)this.pickupDelay);
        if (this.getThrower() != null) {
            jt.put("Thrower", NbtUtils.createUUIDTag(this.getThrower()));
        }
        if (this.getOwner() != null) {
            jt.put("Owner", NbtUtils.createUUIDTag(this.getOwner()));
        }
        if (!this.getItem().isEmpty()) {
            jt.put("Item", this.getItem().save(new CompoundTag()));
        }
    }
    
    public void readAdditionalSaveData(final CompoundTag jt) {
        this.health = jt.getShort("Health");
        this.age = jt.getShort("Age");
        if (jt.contains("PickupDelay")) {
            this.pickupDelay = jt.getShort("PickupDelay");
        }
        if (jt.contains("Owner", 10)) {
            this.owner = NbtUtils.loadUUIDTag(jt.getCompound("Owner"));
        }
        if (jt.contains("Thrower", 10)) {
            this.thrower = NbtUtils.loadUUIDTag(jt.getCompound("Thrower"));
        }
        final CompoundTag jt2 = jt.getCompound("Item");
        this.setItem(ItemStack.of(jt2));
        if (this.getItem().isEmpty()) {
            this.remove();
        }
    }
    
    @Override
    public void playerTouch(final Player ayg) {
        if (this.level.isClientSide) {
            return;
        }
        final ItemStack bek3 = this.getItem();
        final Item bef4 = bek3.getItem();
        final int integer5 = bek3.getCount();
        if (this.pickupDelay == 0 && (this.owner == null || this.owner.equals(ayg.getUUID())) && ayg.inventory.add(bek3)) {
            ayg.take(this, integer5);
            if (bek3.isEmpty()) {
                this.remove();
                bek3.setCount(integer5);
            }
            ayg.awardStat(Stats.ITEM_PICKED_UP.get(bef4), integer5);
        }
    }
    
    @Override
    public Component getName() {
        final Component lf2 = this.getCustomName();
        if (lf2 != null) {
            return lf2;
        }
        return new TranslatableComponent(this.getItem().getDescriptionId(), new Object[0]);
    }
    
    @Override
    public boolean isAttackable() {
        return false;
    }
    
    @Nullable
    @Override
    public Entity changeDimension(final DimensionType cbf) {
        final Entity akn3 = super.changeDimension(cbf);
        if (!this.level.isClientSide && akn3 instanceof ItemEntity) {
            ((ItemEntity)akn3).mergeWithNeighbours();
        }
        return akn3;
    }
    
    public ItemStack getItem() {
        return this.getEntityData().<ItemStack>get(ItemEntity.DATA_ITEM);
    }
    
    public void setItem(final ItemStack bek) {
        this.getEntityData().<ItemStack>set(ItemEntity.DATA_ITEM, bek);
    }
    
    @Nullable
    public UUID getOwner() {
        return this.owner;
    }
    
    public void setOwner(@Nullable final UUID uUID) {
        this.owner = uUID;
    }
    
    @Nullable
    public UUID getThrower() {
        return this.thrower;
    }
    
    public void setThrower(@Nullable final UUID uUID) {
        this.thrower = uUID;
    }
    
    public int getAge() {
        return this.age;
    }
    
    public void setDefaultPickUpDelay() {
        this.pickupDelay = 10;
    }
    
    public void setNoPickUpDelay() {
        this.pickupDelay = 0;
    }
    
    public void setNeverPickUp() {
        this.pickupDelay = 32767;
    }
    
    public void setPickUpDelay(final int integer) {
        this.pickupDelay = integer;
    }
    
    public boolean hasPickUpDelay() {
        return this.pickupDelay > 0;
    }
    
    public void setExtendedLifetime() {
        this.age = -6000;
    }
    
    public void makeFakeItem() {
        this.setNeverPickUp();
        this.age = 5999;
    }
    
    @Override
    public Packet<?> getAddEntityPacket() {
        return new ClientboundAddEntityPacket(this);
    }
    
    static {
        DATA_ITEM = SynchedEntityData.<ItemStack>defineId(ItemEntity.class, EntityDataSerializers.ITEM_STACK);
    }
}
