package net.minecraft.world.entity.monster;

import net.minecraft.world.entity.ai.goal.target.NearestAttackableTargetGoal;
import net.minecraft.world.entity.Mob;
import net.minecraft.world.entity.ai.goal.target.HurtByTargetGoal;
import net.minecraft.world.level.ItemLike;
import net.minecraft.world.item.ItemStack;
import net.minecraft.world.item.Items;
import net.minecraft.world.entity.EquipmentSlot;
import net.minecraft.world.DifficultyInstance;
import net.minecraft.sounds.SoundEvent;
import net.minecraft.world.damagesource.DamageSource;
import net.minecraft.nbt.CompoundTag;
import net.minecraft.world.level.LevelReader;
import net.minecraft.world.Difficulty;
import java.util.Random;
import net.minecraft.core.BlockPos;
import net.minecraft.world.entity.MobSpawnType;
import net.minecraft.world.level.LevelAccessor;
import net.minecraft.world.entity.player.Player;
import net.minecraft.world.entity.ai.attributes.AttributeInstance;
import net.minecraft.sounds.SoundEvents;
import net.minecraft.world.entity.Entity;
import net.minecraft.world.entity.PathfinderMob;
import net.minecraft.world.entity.ai.goal.WaterAvoidingRandomStrollGoal;
import net.minecraft.world.entity.ai.goal.Goal;
import net.minecraft.world.entity.ai.goal.ZombieAttackGoal;
import javax.annotation.Nullable;
import net.minecraft.world.entity.LivingEntity;
import net.minecraft.world.level.pathfinder.BlockPathTypes;
import net.minecraft.world.level.Level;
import net.minecraft.world.entity.EntityType;
import net.minecraft.world.entity.ai.attributes.AttributeModifier;
import java.util.UUID;

public class PigZombie extends Zombie {
    private static final UUID SPEED_MODIFIER_ATTACKING_UUID;
    private static final AttributeModifier SPEED_MODIFIER_ATTACKING;
    private int angerTime;
    private int playAngrySoundIn;
    private UUID lastHurtByUUID;
    
    public PigZombie(final EntityType<? extends PigZombie> akr, final Level bjt) {
        super(akr, bjt);
        this.setPathfindingMalus(BlockPathTypes.LAVA, 8.0f);
    }
    
    @Override
    public void setLastHurtByMob(@Nullable final LivingEntity akw) {
        super.setLastHurtByMob(akw);
        if (akw != null) {
            this.lastHurtByUUID = akw.getUUID();
        }
    }
    
    @Override
    protected void addBehaviourGoals() {
        this.goalSelector.addGoal(2, new ZombieAttackGoal(this, 1.0, false));
        this.goalSelector.addGoal(7, new WaterAvoidingRandomStrollGoal(this, 1.0));
        this.targetSelector.addGoal(1, new PigZombieHurtByOtherGoal(this));
        this.targetSelector.addGoal(2, new PigZombieAngerTargetGoal(this));
    }
    
    @Override
    protected void registerAttributes() {
        super.registerAttributes();
        this.getAttribute(PigZombie.SPAWN_REINFORCEMENTS_CHANCE).setBaseValue(0.0);
        this.getAttribute(SharedMonsterAttributes.MOVEMENT_SPEED).setBaseValue(0.23000000417232513);
        this.getAttribute(SharedMonsterAttributes.ATTACK_DAMAGE).setBaseValue(5.0);
    }
    
    @Override
    protected boolean convertsInWater() {
        return false;
    }
    
    @Override
    protected void customServerAiStep() {
        final AttributeInstance alo2 = this.getAttribute(SharedMonsterAttributes.MOVEMENT_SPEED);
        final LivingEntity akw3 = this.getLastHurtByMob();
        if (this.isAngry()) {
            if (!this.isBaby() && !alo2.hasModifier(PigZombie.SPEED_MODIFIER_ATTACKING)) {
                alo2.addModifier(PigZombie.SPEED_MODIFIER_ATTACKING);
            }
            --this.angerTime;
            final LivingEntity akw4 = (akw3 != null) ? akw3 : this.getTarget();
            if (!this.isAngry() && akw4 != null) {
                if (!this.canSee(akw4)) {
                    this.setLastHurtByMob(null);
                    this.setTarget(null);
                }
                else {
                    this.angerTime = this.getAngerTime();
                }
            }
        }
        else if (alo2.hasModifier(PigZombie.SPEED_MODIFIER_ATTACKING)) {
            alo2.removeModifier(PigZombie.SPEED_MODIFIER_ATTACKING);
        }
        if (this.playAngrySoundIn > 0 && --this.playAngrySoundIn == 0) {
            this.playSound(SoundEvents.ZOMBIE_PIGMAN_ANGRY, this.getSoundVolume() * 2.0f, ((this.random.nextFloat() - this.random.nextFloat()) * 0.2f + 1.0f) * 1.8f);
        }
        if (this.isAngry() && this.lastHurtByUUID != null && akw3 == null) {
            final Player ayg4 = this.level.getPlayerByUUID(this.lastHurtByUUID);
            this.setLastHurtByMob(ayg4);
            this.lastHurtByPlayer = ayg4;
            this.lastHurtByPlayerTime = this.getLastHurtByMobTimestamp();
        }
        super.customServerAiStep();
    }
    
    public static boolean checkPigZombieSpawnRules(final EntityType<PigZombie> akr, final LevelAccessor bju, final MobSpawnType akz, final BlockPos fk, final Random random) {
        return bju.getDifficulty() != Difficulty.PEACEFUL;
    }
    
    @Override
    public boolean checkSpawnObstruction(final LevelReader bjw) {
        return bjw.isUnobstructed(this) && !bjw.containsAnyLiquid(this.getBoundingBox());
    }
    
    @Override
    public void addAdditionalSaveData(final CompoundTag jt) {
        super.addAdditionalSaveData(jt);
        jt.putShort("Anger", (short)this.angerTime);
        if (this.lastHurtByUUID != null) {
            jt.putString("HurtBy", this.lastHurtByUUID.toString());
        }
        else {
            jt.putString("HurtBy", "");
        }
    }
    
    @Override
    public void readAdditionalSaveData(final CompoundTag jt) {
        super.readAdditionalSaveData(jt);
        this.angerTime = jt.getShort("Anger");
        final String string3 = jt.getString("HurtBy");
        if (!string3.isEmpty()) {
            this.lastHurtByUUID = UUID.fromString(string3);
            final Player ayg4 = this.level.getPlayerByUUID(this.lastHurtByUUID);
            this.setLastHurtByMob(ayg4);
            if (ayg4 != null) {
                this.lastHurtByPlayer = ayg4;
                this.lastHurtByPlayerTime = this.getLastHurtByMobTimestamp();
            }
        }
    }
    
    @Override
    public boolean hurt(final DamageSource ajw, final float float2) {
        if (this.isInvulnerableTo(ajw)) {
            return false;
        }
        final Entity akn4 = ajw.getEntity();
        if (akn4 instanceof Player && !((Player)akn4).isCreative() && this.canSee(akn4)) {
            this.makeAngry((LivingEntity)akn4);
        }
        return super.hurt(ajw, float2);
    }
    
    private boolean makeAngry(final LivingEntity akw) {
        this.angerTime = this.getAngerTime();
        this.playAngrySoundIn = this.random.nextInt(40);
        this.setLastHurtByMob(akw);
        return true;
    }
    
    private int getAngerTime() {
        return 400 + this.random.nextInt(400);
    }
    
    private boolean isAngry() {
        return this.angerTime > 0;
    }
    
    @Override
    protected SoundEvent getAmbientSound() {
        return SoundEvents.ZOMBIE_PIGMAN_AMBIENT;
    }
    
    @Override
    protected SoundEvent getHurtSound(final DamageSource ajw) {
        return SoundEvents.ZOMBIE_PIGMAN_HURT;
    }
    
    @Override
    protected SoundEvent getDeathSound() {
        return SoundEvents.ZOMBIE_PIGMAN_DEATH;
    }
    
    @Override
    protected void populateDefaultEquipmentSlots(final DifficultyInstance ajg) {
        this.setItemSlot(EquipmentSlot.MAINHAND, new ItemStack(Items.GOLDEN_SWORD));
    }
    
    @Override
    protected ItemStack getSkull() {
        return ItemStack.EMPTY;
    }
    
    @Override
    public boolean isPreventingPlayerRest(final Player ayg) {
        return this.isAngry();
    }
    
    static {
        SPEED_MODIFIER_ATTACKING_UUID = UUID.fromString("49455A49-7EC5-45BA-B886-3B90B23A1718");
        SPEED_MODIFIER_ATTACKING = new AttributeModifier(PigZombie.SPEED_MODIFIER_ATTACKING_UUID, "Attacking speed boost", 0.05, AttributeModifier.Operation.ADDITION).setSerialize(false);
    }
    
    static class PigZombieHurtByOtherGoal extends HurtByTargetGoal {
        public PigZombieHurtByOtherGoal(final PigZombie aww) {
            super(aww, new Class[0]);
            this.setAlertOthers(Zombie.class);
        }
        
        @Override
        protected void alertOther(final Mob akx, final LivingEntity akw) {
            if (akx instanceof PigZombie && this.mob.canSee(akw) && ((PigZombie)akx).makeAngry(akw)) {
                akx.setTarget(akw);
            }
        }
    }
    
    static class PigZombieAngerTargetGoal extends NearestAttackableTargetGoal<Player> {
        public PigZombieAngerTargetGoal(final PigZombie aww) {
            super(aww, Player.class, true);
        }
        
        @Override
        public boolean canUse() {
            return ((PigZombie)this.mob).isAngry() && super.canUse();
        }
    }
}
