package net.minecraft.world.entity.monster;

import net.minecraft.world.entity.projectile.AbstractArrow;
import net.minecraft.world.effect.MobEffectInstance;
import net.minecraft.world.effect.MobEffects;
import net.minecraft.world.entity.LivingEntity;
import net.minecraft.world.entity.EntityDimensions;
import net.minecraft.world.entity.Pose;
import net.minecraft.nbt.CompoundTag;
import javax.annotation.Nullable;
import net.minecraft.world.entity.SpawnGroupData;
import net.minecraft.world.entity.MobSpawnType;
import net.minecraft.world.level.LevelAccessor;
import net.minecraft.world.item.ItemStack;
import net.minecraft.world.entity.EquipmentSlot;
import net.minecraft.world.DifficultyInstance;
import net.minecraft.world.entity.Entity;
import net.minecraft.world.level.ItemLike;
import net.minecraft.world.item.Items;
import net.minecraft.world.damagesource.DamageSource;
import net.minecraft.sounds.SoundEvents;
import net.minecraft.sounds.SoundEvent;
import net.minecraft.world.level.pathfinder.BlockPathTypes;
import net.minecraft.world.level.Level;
import net.minecraft.world.entity.EntityType;

public class WitherSkeleton extends AbstractSkeleton {
    public WitherSkeleton(final EntityType<? extends WitherSkeleton> akr, final Level bjt) {
        super(akr, bjt);
        this.setPathfindingMalus(BlockPathTypes.LAVA, 8.0f);
    }
    
    @Override
    protected SoundEvent getAmbientSound() {
        return SoundEvents.WITHER_SKELETON_AMBIENT;
    }
    
    @Override
    protected SoundEvent getHurtSound(final DamageSource ajw) {
        return SoundEvents.WITHER_SKELETON_HURT;
    }
    
    @Override
    protected SoundEvent getDeathSound() {
        return SoundEvents.WITHER_SKELETON_DEATH;
    }
    
    @Override
    SoundEvent getStepSound() {
        return SoundEvents.WITHER_SKELETON_STEP;
    }
    
    @Override
    protected void dropCustomDeathLoot(final DamageSource ajw, final int integer, final boolean boolean3) {
        super.dropCustomDeathLoot(ajw, integer, boolean3);
        final Entity akn5 = ajw.getEntity();
        if (akn5 instanceof Creeper) {
            final Creeper awf6 = (Creeper)akn5;
            if (awf6.canDropMobsSkull()) {
                awf6.increaseDroppedSkulls();
                this.spawnAtLocation(Items.WITHER_SKELETON_SKULL);
            }
        }
    }
    
    @Override
    protected void populateDefaultEquipmentSlots(final DifficultyInstance ajg) {
        this.setItemSlot(EquipmentSlot.MAINHAND, new ItemStack(Items.STONE_SWORD));
    }
    
    @Override
    protected void populateDefaultEquipmentEnchantments(final DifficultyInstance ajg) {
    }
    
    @Nullable
    @Override
    public SpawnGroupData finalizeSpawn(final LevelAccessor bju, final DifficultyInstance ajg, final MobSpawnType akz, @Nullable final SpawnGroupData alj, @Nullable final CompoundTag jt) {
        final SpawnGroupData alj2 = super.finalizeSpawn(bju, ajg, akz, alj, jt);
        this.getAttribute(SharedMonsterAttributes.ATTACK_DAMAGE).setBaseValue(4.0);
        this.reassessWeaponGoal();
        return alj2;
    }
    
    @Override
    protected float getStandingEyeHeight(final Pose alg, final EntityDimensions ako) {
        return 2.1f;
    }
    
    @Override
    public boolean doHurtTarget(final Entity akn) {
        if (!super.doHurtTarget(akn)) {
            return false;
        }
        if (akn instanceof LivingEntity) {
            ((LivingEntity)akn).addEffect(new MobEffectInstance(MobEffects.WITHER, 200));
        }
        return true;
    }
    
    @Override
    protected AbstractArrow getArrow(final ItemStack bek, final float float2) {
        final AbstractArrow ayk4 = super.getArrow(bek, float2);
        ayk4.setSecondsOnFire(100);
        return ayk4;
    }
    
    @Override
    public boolean canBeAffected(final MobEffectInstance akh) {
        return akh.getEffect() != MobEffects.WITHER && super.canBeAffected(akh);
    }
}
