package net.minecraft.world.entity.projectile;

import net.minecraft.world.damagesource.DamageSource;
import java.util.Iterator;
import java.util.List;
import net.minecraft.core.BlockPos;
import net.minecraft.world.effect.MobEffectInstance;
import net.minecraft.world.effect.MobEffects;
import net.minecraft.core.particles.ParticleOptions;
import net.minecraft.core.particles.ParticleTypes;
import net.minecraft.world.entity.AreaEffectCloud;
import net.minecraft.world.entity.Entity;
import net.minecraft.world.phys.EntityHitResult;
import net.minecraft.world.phys.HitResult;
import net.minecraft.world.entity.LivingEntity;
import net.minecraft.world.level.Level;
import net.minecraft.world.entity.EntityType;

public class DragonFireball extends AbstractHurtingProjectile {
    public DragonFireball(final EntityType<? extends DragonFireball> akr, final Level bjt) {
        super(akr, bjt);
    }
    
    public DragonFireball(final Level bjt, final double double2, final double double3, final double double4, final double double5, final double double6, final double double7) {
        super(EntityType.DRAGON_FIREBALL, double2, double3, double4, double5, double6, double7, bjt);
    }
    
    public DragonFireball(final Level bjt, final LivingEntity akw, final double double3, final double double4, final double double5) {
        super(EntityType.DRAGON_FIREBALL, akw, double3, double4, double5, bjt);
    }
    
    @Override
    protected void onHit(final HitResult cvf) {
        super.onHit(cvf);
        if (cvf.getType() == HitResult.Type.ENTITY && ((EntityHitResult)cvf).getEntity().is(this.owner)) {
            return;
        }
        if (!this.level.isClientSide) {
            final List<LivingEntity> list3 = this.level.<LivingEntity>getEntitiesOfClass(LivingEntity.class, this.getBoundingBox().inflate(4.0, 2.0, 4.0));
            final AreaEffectCloud akm4 = new AreaEffectCloud(this.level, this.getX(), this.getY(), this.getZ());
            akm4.setOwner(this.owner);
            akm4.setParticle(ParticleTypes.DRAGON_BREATH);
            akm4.setRadius(3.0f);
            akm4.setDuration(600);
            akm4.setRadiusPerTick((7.0f - akm4.getRadius()) / akm4.getDuration());
            akm4.addEffect(new MobEffectInstance(MobEffects.HARM, 1, 1));
            if (!list3.isEmpty()) {
                for (final LivingEntity akw6 : list3) {
                    final double double7 = this.distanceToSqr(akw6);
                    if (double7 < 16.0) {
                        akm4.setPos(akw6.getX(), akw6.getY(), akw6.getZ());
                        break;
                    }
                }
            }
            this.level.levelEvent(2006, new BlockPos(this), 0);
            this.level.addFreshEntity(akm4);
            this.remove();
        }
    }
    
    @Override
    public boolean isPickable() {
        return false;
    }
    
    @Override
    public boolean hurt(final DamageSource ajw, final float float2) {
        return false;
    }
    
    @Override
    protected ParticleOptions getTrailParticle() {
        return ParticleTypes.DRAGON_BREATH;
    }
    
    @Override
    protected boolean shouldBurn() {
        return false;
    }
}
