package net.minecraft.world.entity.projectile;

import net.minecraft.world.item.Items;
import net.minecraft.world.item.ArrowItem;
import net.minecraft.world.item.ItemStack;
import net.minecraft.world.InteractionHand;
import net.minecraft.world.item.Item;
import net.minecraft.world.entity.LivingEntity;
import net.minecraft.util.Mth;
import java.util.Optional;
import java.util.Iterator;
import net.minecraft.world.phys.BlockHitResult;
import net.minecraft.core.BlockPos;
import net.minecraft.core.Direction;
import java.util.Set;
import com.google.common.collect.ImmutableSet;
import net.minecraft.world.phys.EntityHitResult;
import net.minecraft.world.phys.Vec3;
import net.minecraft.world.level.Level;
import java.util.function.Predicate;
import net.minecraft.world.phys.AABB;
import net.minecraft.world.phys.HitResult;
import net.minecraft.world.level.ClipContext;
import javax.annotation.Nullable;
import net.minecraft.world.entity.Entity;

public final class ProjectileUtil {
    public static HitResult forwardsRaycast(final Entity akn1, final boolean boolean2, final boolean boolean3, @Nullable final Entity akn4, final ClipContext.Block a) {
        return forwardsRaycast(akn1, boolean2, boolean3, akn4, a, true, akn3 -> !akn3.isSpectator() && akn3.isPickable() && (boolean3 || !akn3.is(akn4)) && !akn3.noPhysics, akn1.getBoundingBox().expandTowards(akn1.getDeltaMovement()).inflate(1.0));
    }
    
    public static HitResult getHitResult(final Entity akn, final AABB cvc, final Predicate<Entity> predicate, final ClipContext.Block a, final boolean boolean5) {
        return forwardsRaycast(akn, boolean5, false, null, a, false, predicate, cvc);
    }
    
    @Nullable
    public static EntityHitResult getHitResult(final Level bjt, final Entity akn, final Vec3 cvi3, final Vec3 cvi4, final AABB cvc, final Predicate<Entity> predicate) {
        return getHitResult(bjt, akn, cvi3, cvi4, cvc, predicate, Double.MAX_VALUE);
    }
    
    private static HitResult forwardsRaycast(final Entity akn1, final boolean boolean2, final boolean boolean3, @Nullable final Entity akn4, final ClipContext.Block a, final boolean boolean6, final Predicate<Entity> predicate, final AABB cvc) {
        final Vec3 cvi9 = akn1.getDeltaMovement();
        final Level bjt10 = akn1.level;
        final Vec3 cvi10 = akn1.position();
        if (boolean6 && !bjt10.noCollision(akn1, akn1.getBoundingBox(), (Set<Entity>)((boolean3 || akn4 == null) ? ImmutableSet.of() : getIgnoredEntities(akn4)))) {
            return new BlockHitResult(cvi10, Direction.getNearest(cvi9.x, cvi9.y, cvi9.z), new BlockPos(akn1), false);
        }
        Vec3 cvi11 = cvi10.add(cvi9);
        HitResult cvf13 = bjt10.clip(new ClipContext(cvi10, cvi11, a, ClipContext.Fluid.NONE, akn1));
        if (boolean2) {
            if (cvf13.getType() != HitResult.Type.MISS) {
                cvi11 = cvf13.getLocation();
            }
            final HitResult cvf14 = getHitResult(bjt10, akn1, cvi10, cvi11, cvc, predicate);
            if (cvf14 != null) {
                cvf13 = cvf14;
            }
        }
        return cvf13;
    }
    
    @Nullable
    public static EntityHitResult getEntityHitResult(final Entity akn, final Vec3 cvi2, final Vec3 cvi3, final AABB cvc, final Predicate<Entity> predicate, final double double6) {
        final Level bjt8 = akn.level;
        double double7 = double6;
        Entity akn2 = null;
        Vec3 cvi4 = null;
        for (final Entity akn3 : bjt8.getEntities(akn, cvc, predicate)) {
            final AABB cvc2 = akn3.getBoundingBox().inflate(akn3.getPickRadius());
            final Optional<Vec3> optional16 = cvc2.clip(cvi2, cvi3);
            if (cvc2.contains(cvi2)) {
                if (double7 < 0.0) {
                    continue;
                }
                akn2 = akn3;
                cvi4 = optional16.orElse(cvi2);
                double7 = 0.0;
            }
            else {
                if (!optional16.isPresent()) {
                    continue;
                }
                final Vec3 cvi5 = optional16.get();
                final double double8 = cvi2.distanceToSqr(cvi5);
                if (double8 >= double7 && double7 != 0.0) {
                    continue;
                }
                if (akn3.getRootVehicle() == akn.getRootVehicle()) {
                    if (double7 != 0.0) {
                        continue;
                    }
                    akn2 = akn3;
                    cvi4 = cvi5;
                }
                else {
                    akn2 = akn3;
                    cvi4 = cvi5;
                    double7 = double8;
                }
            }
        }
        if (akn2 == null) {
            return null;
        }
        return new EntityHitResult(akn2, cvi4);
    }
    
    @Nullable
    public static EntityHitResult getHitResult(final Level bjt, final Entity akn, final Vec3 cvi3, final Vec3 cvi4, final AABB cvc, final Predicate<Entity> predicate, final double double7) {
        double double8 = double7;
        Entity akn2 = null;
        for (final Entity akn3 : bjt.getEntities(akn, cvc, predicate)) {
            final AABB cvc2 = akn3.getBoundingBox().inflate(0.30000001192092896);
            final Optional<Vec3> optional15 = cvc2.clip(cvi3, cvi4);
            if (optional15.isPresent()) {
                final double double9 = cvi3.distanceToSqr(optional15.get());
                if (double9 >= double8) {
                    continue;
                }
                akn2 = akn3;
                double8 = double9;
            }
        }
        if (akn2 == null) {
            return null;
        }
        return new EntityHitResult(akn2);
    }
    
    private static Set<Entity> getIgnoredEntities(final Entity akn) {
        final Entity akn2 = akn.getVehicle();
        return (akn2 != null) ? ImmutableSet.<Entity>of(akn, akn2) : ImmutableSet.<Entity>of(akn);
    }
    
    public static final void rotateTowardsMovement(final Entity akn, final float float2) {
        final Vec3 cvi3 = akn.getDeltaMovement();
        final float float3 = Mth.sqrt(Entity.getHorizontalDistanceSqr(cvi3));
        akn.yRot = (float)(Mth.atan2(cvi3.z, cvi3.x) * 57.2957763671875) + 90.0f;
        akn.xRot = (float)(Mth.atan2(float3, cvi3.y) * 57.2957763671875) - 90.0f;
        while (akn.xRot - akn.xRotO < -180.0f) {
            akn.xRotO -= 360.0f;
        }
        while (akn.xRot - akn.xRotO >= 180.0f) {
            akn.xRotO += 360.0f;
        }
        while (akn.yRot - akn.yRotO < -180.0f) {
            akn.yRotO -= 360.0f;
        }
        while (akn.yRot - akn.yRotO >= 180.0f) {
            akn.yRotO += 360.0f;
        }
        akn.xRot = Mth.lerp(float2, akn.xRotO, akn.xRot);
        akn.yRot = Mth.lerp(float2, akn.yRotO, akn.yRot);
    }
    
    public static InteractionHand getWeaponHoldingHand(final LivingEntity akw, final Item bef) {
        return (akw.getMainHandItem().getItem() == bef) ? InteractionHand.MAIN_HAND : InteractionHand.OFF_HAND;
    }
    
    public static AbstractArrow getMobArrow(final LivingEntity akw, final ItemStack bek, final float float3) {
        final ArrowItem bch4 = (ArrowItem)((bek.getItem() instanceof ArrowItem) ? bek.getItem() : Items.ARROW);
        final AbstractArrow ayk5 = bch4.createArrow(akw.level, bek, akw);
        ayk5.setEnchantmentEffectsFromEntity(akw, float3);
        if (bek.getItem() == Items.TIPPED_ARROW && ayk5 instanceof Arrow) {
            ((Arrow)ayk5).setEffectsFromItem(bek);
        }
        return ayk5;
    }
}
