package net.minecraft.world.entity.projectile;

import net.minecraft.world.entity.Entity;
import net.minecraft.network.syncher.SynchedEntityData;
import net.minecraft.network.syncher.EntityDataSerializers;
import net.minecraft.nbt.Tag;
import net.minecraft.nbt.CompoundTag;
import net.minecraft.world.level.ItemLike;
import net.minecraft.world.item.Item;
import net.minecraft.Util;
import net.minecraft.world.entity.LivingEntity;
import net.minecraft.world.level.Level;
import net.minecraft.world.entity.EntityType;
import net.minecraft.world.item.ItemStack;
import net.minecraft.network.syncher.EntityDataAccessor;

public abstract class ThrowableItemProjectile extends ThrowableProjectile implements ItemSupplier {
    private static final EntityDataAccessor<ItemStack> DATA_ITEM_STACK;
    
    public ThrowableItemProjectile(final EntityType<? extends ThrowableItemProjectile> akr, final Level bjt) {
        super(akr, bjt);
    }
    
    public ThrowableItemProjectile(final EntityType<? extends ThrowableItemProjectile> akr, final double double2, final double double3, final double double4, final Level bjt) {
        super(akr, double2, double3, double4, bjt);
    }
    
    public ThrowableItemProjectile(final EntityType<? extends ThrowableItemProjectile> akr, final LivingEntity akw, final Level bjt) {
        super(akr, akw, bjt);
    }
    
    public void setItem(final ItemStack bek) {
        if (bek.getItem() != this.getDefaultItem() || bek.hasTag()) {
            this.getEntityData().<ItemStack>set(ThrowableItemProjectile.DATA_ITEM_STACK, (ItemStack)Util.<T>make((T)bek.copy(), bek -> bek.setCount(1)));
        }
    }
    
    protected abstract Item getDefaultItem();
    
    protected ItemStack getItemRaw() {
        return this.getEntityData().<ItemStack>get(ThrowableItemProjectile.DATA_ITEM_STACK);
    }
    
    @Override
    public ItemStack getItem() {
        final ItemStack bek2 = this.getItemRaw();
        return bek2.isEmpty() ? new ItemStack(this.getDefaultItem()) : bek2;
    }
    
    @Override
    protected void defineSynchedData() {
        this.getEntityData().<ItemStack>define(ThrowableItemProjectile.DATA_ITEM_STACK, ItemStack.EMPTY);
    }
    
    @Override
    public void addAdditionalSaveData(final CompoundTag jt) {
        super.addAdditionalSaveData(jt);
        final ItemStack bek3 = this.getItemRaw();
        if (!bek3.isEmpty()) {
            jt.put("Item", bek3.save(new CompoundTag()));
        }
    }
    
    @Override
    public void readAdditionalSaveData(final CompoundTag jt) {
        super.readAdditionalSaveData(jt);
        final ItemStack bek3 = ItemStack.of(jt.getCompound("Item"));
        this.setItem(bek3);
    }
    
    static {
        DATA_ITEM_STACK = SynchedEntityData.<ItemStack>defineId(ThrowableItemProjectile.class, EntityDataSerializers.ITEM_STACK);
    }
}
