package net.minecraft.world.entity.projectile;

import javax.annotation.Nullable;
import net.minecraft.world.level.dimension.DimensionType;
import net.minecraft.world.entity.player.Player;
import net.minecraft.world.level.block.entity.BlockEntity;
import net.minecraft.core.BlockPos;
import net.minecraft.world.entity.monster.Endermite;
import net.minecraft.world.level.GameRules;
import net.minecraft.core.particles.ParticleOptions;
import net.minecraft.core.particles.ParticleTypes;
import net.minecraft.advancements.CriteriaTriggers;
import net.minecraft.server.level.ServerPlayer;
import net.minecraft.world.level.block.entity.TheEndGatewayBlockEntity;
import net.minecraft.world.phys.BlockHitResult;
import net.minecraft.world.entity.Entity;
import net.minecraft.world.damagesource.DamageSource;
import net.minecraft.world.phys.EntityHitResult;
import net.minecraft.world.phys.HitResult;
import net.minecraft.world.item.Items;
import net.minecraft.world.item.Item;
import net.minecraft.world.level.Level;
import net.minecraft.world.entity.EntityType;
import net.minecraft.world.entity.LivingEntity;

public class ThrownEnderpearl extends ThrowableItemProjectile {
    private LivingEntity originalOwner;
    
    public ThrownEnderpearl(final EntityType<? extends ThrownEnderpearl> akr, final Level bjt) {
        super(akr, bjt);
    }
    
    public ThrownEnderpearl(final Level bjt, final LivingEntity akw) {
        super(EntityType.ENDER_PEARL, akw, bjt);
        this.originalOwner = akw;
    }
    
    public ThrownEnderpearl(final Level bjt, final double double2, final double double3, final double double4) {
        super(EntityType.ENDER_PEARL, double2, double3, double4, bjt);
    }
    
    @Override
    protected Item getDefaultItem() {
        return Items.ENDER_PEARL;
    }
    
    @Override
    protected void onHit(final HitResult cvf) {
        final LivingEntity akw3 = this.getOwner();
        if (cvf.getType() == HitResult.Type.ENTITY) {
            final Entity akn4 = ((EntityHitResult)cvf).getEntity();
            if (akn4 == this.originalOwner) {
                return;
            }
            akn4.hurt(DamageSource.thrown(this, akw3), 0.0f);
        }
        if (cvf.getType() == HitResult.Type.BLOCK) {
            final BlockPos fk4 = ((BlockHitResult)cvf).getBlockPos();
            final BlockEntity bwi5 = this.level.getBlockEntity(fk4);
            if (bwi5 instanceof TheEndGatewayBlockEntity) {
                final TheEndGatewayBlockEntity bxj6 = (TheEndGatewayBlockEntity)bwi5;
                if (akw3 != null) {
                    if (akw3 instanceof ServerPlayer) {
                        CriteriaTriggers.ENTER_BLOCK.trigger((ServerPlayer)akw3, this.level.getBlockState(fk4));
                    }
                    bxj6.teleportEntity(akw3);
                    this.remove();
                    return;
                }
                bxj6.teleportEntity(this);
                return;
            }
        }
        for (int integer4 = 0; integer4 < 32; ++integer4) {
            this.level.addParticle(ParticleTypes.PORTAL, this.getX(), this.getY() + this.random.nextDouble() * 2.0, this.getZ(), this.random.nextGaussian(), 0.0, this.random.nextGaussian());
        }
        if (!this.level.isClientSide) {
            if (akw3 instanceof ServerPlayer) {
                final ServerPlayer xe4 = (ServerPlayer)akw3;
                if (xe4.connection.getConnection().isConnected() && xe4.level == this.level && !xe4.isSleeping()) {
                    if (this.random.nextFloat() < 0.05f && this.level.getGameRules().getBoolean(GameRules.RULE_DOMOBSPAWNING)) {
                        final Endermite awk5 = EntityType.ENDERMITE.create(this.level);
                        awk5.setPlayerSpawned(true);
                        awk5.moveTo(akw3.getX(), akw3.getY(), akw3.getZ(), akw3.yRot, akw3.xRot);
                        this.level.addFreshEntity(awk5);
                    }
                    if (akw3.isPassenger()) {
                        akw3.stopRiding();
                    }
                    akw3.teleportTo(this.getX(), this.getY(), this.getZ());
                    akw3.fallDistance = 0.0f;
                    akw3.hurt(DamageSource.FALL, 5.0f);
                }
            }
            else if (akw3 != null) {
                akw3.teleportTo(this.getX(), this.getY(), this.getZ());
                akw3.fallDistance = 0.0f;
            }
            this.remove();
        }
    }
    
    @Override
    public void tick() {
        final LivingEntity akw2 = this.getOwner();
        if (akw2 != null && akw2 instanceof Player && !akw2.isAlive()) {
            this.remove();
        }
        else {
            super.tick();
        }
    }
    
    @Nullable
    @Override
    public Entity changeDimension(final DimensionType cbf) {
        if (this.owner.dimension != cbf) {
            this.owner = null;
        }
        return super.changeDimension(cbf);
    }
}
