package net.minecraft.world.entity.projectile;

import net.minecraft.world.level.block.state.AbstractStateHolder;
import org.apache.logging.log4j.LogManager;
import net.minecraft.network.syncher.SynchedEntityData;
import net.minecraft.network.syncher.EntityDataSerializers;
import net.minecraft.world.entity.monster.Blaze;
import net.minecraft.world.entity.monster.EnderMan;
import net.minecraft.nbt.Tag;
import net.minecraft.world.level.block.Block;
import net.minecraft.world.level.block.state.BlockState;
import net.minecraft.world.level.block.state.properties.Property;
import net.minecraft.world.level.block.CampfireBlock;
import net.minecraft.world.entity.player.Player;
import net.minecraft.world.level.block.Blocks;
import net.minecraft.nbt.CompoundTag;
import net.minecraft.world.entity.AreaEffectCloud;
import net.minecraft.world.effect.MobEffect;
import javax.annotation.Nullable;
import net.minecraft.world.phys.AABB;
import net.minecraft.world.damagesource.DamageSource;
import java.util.Iterator;
import net.minecraft.world.effect.MobEffectInstance;
import java.util.List;
import net.minecraft.world.item.alchemy.Potion;
import net.minecraft.world.entity.Entity;
import net.minecraft.core.BlockPos;
import net.minecraft.world.phys.EntityHitResult;
import net.minecraft.core.Direction;
import net.minecraft.world.phys.BlockHitResult;
import net.minecraft.world.item.alchemy.Potions;
import net.minecraft.world.item.alchemy.PotionUtils;
import net.minecraft.world.phys.HitResult;
import net.minecraft.world.level.ItemLike;
import net.minecraft.world.item.Items;
import net.minecraft.world.level.Level;
import net.minecraft.world.entity.EntityType;
import net.minecraft.world.entity.LivingEntity;
import java.util.function.Predicate;
import org.apache.logging.log4j.Logger;
import net.minecraft.world.item.ItemStack;
import net.minecraft.network.syncher.EntityDataAccessor;

public class ThrownPotion extends ThrowableProjectile implements ItemSupplier {
    private static final EntityDataAccessor<ItemStack> DATA_ITEM_STACK;
    private static final Logger LOGGER;
    public static final Predicate<LivingEntity> WATER_SENSITIVE;
    
    public ThrownPotion(final EntityType<? extends ThrownPotion> akr, final Level bjt) {
        super(akr, bjt);
    }
    
    public ThrownPotion(final Level bjt, final LivingEntity akw) {
        super(EntityType.POTION, akw, bjt);
    }
    
    public ThrownPotion(final Level bjt, final double double2, final double double3, final double double4) {
        super(EntityType.POTION, double2, double3, double4, bjt);
    }
    
    @Override
    protected void defineSynchedData() {
        this.getEntityData().<ItemStack>define(ThrownPotion.DATA_ITEM_STACK, ItemStack.EMPTY);
    }
    
    @Override
    public ItemStack getItem() {
        final ItemStack bek2 = this.getEntityData().<ItemStack>get(ThrownPotion.DATA_ITEM_STACK);
        if (bek2.getItem() != Items.SPLASH_POTION && bek2.getItem() != Items.LINGERING_POTION) {
            if (this.level != null) {
                ThrownPotion.LOGGER.error("ThrownPotion entity {} has no item?!", this.getId());
            }
            return new ItemStack(Items.SPLASH_POTION);
        }
        return bek2;
    }
    
    public void setItem(final ItemStack bek) {
        this.getEntityData().<ItemStack>set(ThrownPotion.DATA_ITEM_STACK, bek.copy());
    }
    
    @Override
    protected float getGravity() {
        return 0.05f;
    }
    
    @Override
    protected void onHit(final HitResult cvf) {
        if (this.level.isClientSide) {
            return;
        }
        final ItemStack bek3 = this.getItem();
        final Potion bga4 = PotionUtils.getPotion(bek3);
        final List<MobEffectInstance> list5 = PotionUtils.getMobEffects(bek3);
        final boolean boolean6 = bga4 == Potions.WATER && list5.isEmpty();
        if (cvf.getType() == HitResult.Type.BLOCK && boolean6) {
            final BlockHitResult cvd7 = (BlockHitResult)cvf;
            final Direction fp8 = cvd7.getDirection();
            final BlockPos fk9 = cvd7.getBlockPos().relative(fp8);
            this.dowseFire(fk9, fp8);
            this.dowseFire(fk9.relative(fp8.getOpposite()), fp8);
            for (final Direction fp9 : Direction.Plane.HORIZONTAL) {
                this.dowseFire(fk9.relative(fp9), fp9);
            }
        }
        if (boolean6) {
            this.applyWater();
        }
        else if (!list5.isEmpty()) {
            if (this.isLingering()) {
                this.makeAreaOfEffectCloud(bek3, bga4);
            }
            else {
                this.applySplash(list5, (cvf.getType() == HitResult.Type.ENTITY) ? ((EntityHitResult)cvf).getEntity() : null);
            }
        }
        final int integer7 = bga4.hasInstantEffects() ? 2007 : 2002;
        this.level.levelEvent(integer7, new BlockPos(this), PotionUtils.getColor(bek3));
        this.remove();
    }
    
    private void applyWater() {
        final AABB cvc2 = this.getBoundingBox().inflate(4.0, 2.0, 4.0);
        final List<LivingEntity> list3 = this.level.<LivingEntity>getEntitiesOfClass(LivingEntity.class, cvc2, ThrownPotion.WATER_SENSITIVE);
        if (!list3.isEmpty()) {
            for (final LivingEntity akw5 : list3) {
                final double double6 = this.distanceToSqr(akw5);
                if (double6 < 16.0 && isWaterSensitiveEntity(akw5)) {
                    akw5.hurt(DamageSource.indirectMagic(akw5, this.getOwner()), 1.0f);
                }
            }
        }
    }
    
    private void applySplash(final List<MobEffectInstance> list, @Nullable final Entity akn) {
        final AABB cvc4 = this.getBoundingBox().inflate(4.0, 2.0, 4.0);
        final List<LivingEntity> list2 = this.level.<LivingEntity>getEntitiesOfClass(LivingEntity.class, cvc4);
        if (!list2.isEmpty()) {
            for (final LivingEntity akw7 : list2) {
                if (!akw7.isAffectedByPotions()) {
                    continue;
                }
                final double double8 = this.distanceToSqr(akw7);
                if (double8 >= 16.0) {
                    continue;
                }
                double double9 = 1.0 - Math.sqrt(double8) / 4.0;
                if (akw7 == akn) {
                    double9 = 1.0;
                }
                for (final MobEffectInstance akh13 : list) {
                    final MobEffect akf14 = akh13.getEffect();
                    if (akf14.isInstantenous()) {
                        akf14.applyInstantenousEffect(this, this.getOwner(), akw7, akh13.getAmplifier(), double9);
                    }
                    else {
                        final int integer15 = (int)(double9 * akh13.getDuration() + 0.5);
                        if (integer15 <= 20) {
                            continue;
                        }
                        akw7.addEffect(new MobEffectInstance(akf14, integer15, akh13.getAmplifier(), akh13.isAmbient(), akh13.isVisible()));
                    }
                }
            }
        }
    }
    
    private void makeAreaOfEffectCloud(final ItemStack bek, final Potion bga) {
        final AreaEffectCloud akm4 = new AreaEffectCloud(this.level, this.getX(), this.getY(), this.getZ());
        akm4.setOwner(this.getOwner());
        akm4.setRadius(3.0f);
        akm4.setRadiusOnUse(-0.5f);
        akm4.setWaitTime(10);
        akm4.setRadiusPerTick(-akm4.getRadius() / akm4.getDuration());
        akm4.setPotion(bga);
        for (final MobEffectInstance akh6 : PotionUtils.getCustomEffects(bek)) {
            akm4.addEffect(new MobEffectInstance(akh6));
        }
        final CompoundTag jt5 = bek.getTag();
        if (jt5 != null && jt5.contains("CustomPotionColor", 99)) {
            akm4.setFixedColor(jt5.getInt("CustomPotionColor"));
        }
        this.level.addFreshEntity(akm4);
    }
    
    private boolean isLingering() {
        return this.getItem().getItem() == Items.LINGERING_POTION;
    }
    
    private void dowseFire(final BlockPos fk, final Direction fp) {
        final BlockState byg4 = this.level.getBlockState(fk);
        final Block bpe5 = byg4.getBlock();
        if (bpe5 == Blocks.FIRE) {
            this.level.extinguishFire(null, fk.relative(fp), fp.getOpposite());
        }
        else if (bpe5 == Blocks.CAMPFIRE && byg4.<Boolean>getValue((Property<Boolean>)CampfireBlock.LIT)) {
            this.level.levelEvent(null, 1009, fk, 0);
            this.level.setBlockAndUpdate(fk, ((AbstractStateHolder<O, BlockState>)byg4).<Comparable, Boolean>setValue((Property<Comparable>)CampfireBlock.LIT, false));
        }
    }
    
    @Override
    public void readAdditionalSaveData(final CompoundTag jt) {
        super.readAdditionalSaveData(jt);
        final ItemStack bek3 = ItemStack.of(jt.getCompound("Potion"));
        if (bek3.isEmpty()) {
            this.remove();
        }
        else {
            this.setItem(bek3);
        }
    }
    
    @Override
    public void addAdditionalSaveData(final CompoundTag jt) {
        super.addAdditionalSaveData(jt);
        final ItemStack bek3 = this.getItem();
        if (!bek3.isEmpty()) {
            jt.put("Potion", bek3.save(new CompoundTag()));
        }
    }
    
    private static boolean isWaterSensitiveEntity(final LivingEntity akw) {
        return akw instanceof EnderMan || akw instanceof Blaze;
    }
    
    static {
        DATA_ITEM_STACK = SynchedEntityData.<ItemStack>defineId(ThrownPotion.class, EntityDataSerializers.ITEM_STACK);
        LOGGER = LogManager.getLogger();
        WATER_SENSITIVE = ThrownPotion::isWaterSensitiveEntity;
    }
}
