package net.minecraft.world.entity.projectile;

import net.minecraft.network.syncher.SynchedEntityData;
import net.minecraft.network.syncher.EntityDataSerializers;
import net.minecraft.nbt.Tag;
import net.minecraft.nbt.CompoundTag;
import net.minecraft.world.entity.player.Player;
import net.minecraft.core.BlockPos;
import net.minecraft.sounds.SoundEvent;
import net.minecraft.world.entity.global.LightningBolt;
import net.minecraft.server.level.ServerLevel;
import net.minecraft.world.damagesource.DamageSource;
import javax.annotation.Nullable;
import net.minecraft.world.phys.EntityHitResult;
import net.minecraft.server.level.ServerPlayer;
import net.minecraft.world.entity.Entity;
import net.minecraft.sounds.SoundEvents;
import net.minecraft.world.phys.Vec3;
import net.minecraft.world.item.enchantment.EnchantmentHelper;
import net.minecraft.world.entity.LivingEntity;
import net.minecraft.world.level.ItemLike;
import net.minecraft.world.item.Items;
import net.minecraft.world.level.Level;
import net.minecraft.world.entity.EntityType;
import net.minecraft.world.item.ItemStack;
import net.minecraft.network.syncher.EntityDataAccessor;

public class ThrownTrident extends AbstractArrow {
    private static final EntityDataAccessor<Byte> ID_LOYALTY;
    private static final EntityDataAccessor<Boolean> ID_FOIL;
    private ItemStack tridentItem;
    private boolean dealtDamage;
    public int clientSideReturnTridentTickCount;
    
    public ThrownTrident(final EntityType<? extends ThrownTrident> akr, final Level bjt) {
        super(akr, bjt);
        this.tridentItem = new ItemStack(Items.TRIDENT);
    }
    
    public ThrownTrident(final Level bjt, final LivingEntity akw, final ItemStack bek) {
        super(EntityType.TRIDENT, akw, bjt);
        this.tridentItem = new ItemStack(Items.TRIDENT);
        this.tridentItem = bek.copy();
        this.entityData.<Byte>set(ThrownTrident.ID_LOYALTY, (byte)EnchantmentHelper.getLoyalty(bek));
        this.entityData.<Boolean>set(ThrownTrident.ID_FOIL, bek.hasFoil());
    }
    
    public ThrownTrident(final Level bjt, final double double2, final double double3, final double double4) {
        super(EntityType.TRIDENT, double2, double3, double4, bjt);
        this.tridentItem = new ItemStack(Items.TRIDENT);
    }
    
    @Override
    protected void defineSynchedData() {
        super.defineSynchedData();
        this.entityData.<Byte>define(ThrownTrident.ID_LOYALTY, (Byte)0);
        this.entityData.<Boolean>define(ThrownTrident.ID_FOIL, false);
    }
    
    @Override
    public void tick() {
        if (this.inGroundTime > 4) {
            this.dealtDamage = true;
        }
        final Entity akn2 = this.getOwner();
        if ((this.dealtDamage || this.isNoPhysics()) && akn2 != null) {
            final int integer3 = this.entityData.<Byte>get(ThrownTrident.ID_LOYALTY);
            if (integer3 > 0 && !this.isAcceptibleReturnOwner()) {
                if (!this.level.isClientSide && this.pickup == Pickup.ALLOWED) {
                    this.spawnAtLocation(this.getPickupItem(), 0.1f);
                }
                this.remove();
            }
            else if (integer3 > 0) {
                this.setNoPhysics(true);
                final Vec3 cvi4 = new Vec3(akn2.getX() - this.getX(), akn2.getEyeY() - this.getY(), akn2.getZ() - this.getZ());
                this.setPosRaw(this.getX(), this.getY() + cvi4.y * 0.015 * integer3, this.getZ());
                if (this.level.isClientSide) {
                    this.yOld = this.getY();
                }
                final double double5 = 0.05 * integer3;
                this.setDeltaMovement(this.getDeltaMovement().scale(0.95).add(cvi4.normalize().scale(double5)));
                if (this.clientSideReturnTridentTickCount == 0) {
                    this.playSound(SoundEvents.TRIDENT_RETURN, 10.0f, 1.0f);
                }
                ++this.clientSideReturnTridentTickCount;
            }
        }
        super.tick();
    }
    
    private boolean isAcceptibleReturnOwner() {
        final Entity akn2 = this.getOwner();
        return akn2 != null && akn2.isAlive() && (!(akn2 instanceof ServerPlayer) || !akn2.isSpectator());
    }
    
    @Override
    protected ItemStack getPickupItem() {
        return this.tridentItem.copy();
    }
    
    public boolean isFoil() {
        return this.entityData.<Boolean>get(ThrownTrident.ID_FOIL);
    }
    
    @Nullable
    @Override
    protected EntityHitResult findHitEntity(final Vec3 cvi1, final Vec3 cvi2) {
        if (this.dealtDamage) {
            return null;
        }
        return super.findHitEntity(cvi1, cvi2);
    }
    
    @Override
    protected void onHitEntity(final EntityHitResult cve) {
        final Entity akn3 = cve.getEntity();
        float float4 = 8.0f;
        if (akn3 instanceof LivingEntity) {
            final LivingEntity akw5 = (LivingEntity)akn3;
            float4 += EnchantmentHelper.getDamageBonus(this.tridentItem, akw5.getMobType());
        }
        final Entity akn4 = this.getOwner();
        final DamageSource ajw6 = DamageSource.trident(this, (akn4 == null) ? this : akn4);
        this.dealtDamage = true;
        SoundEvent aah7 = SoundEvents.TRIDENT_HIT;
        if (akn3.hurt(ajw6, float4)) {
            if (akn3.getType() == EntityType.ENDERMAN) {
                return;
            }
            if (akn3 instanceof LivingEntity) {
                final LivingEntity akw6 = (LivingEntity)akn3;
                if (akn4 instanceof LivingEntity) {
                    EnchantmentHelper.doPostHurtEffects(akw6, akn4);
                    EnchantmentHelper.doPostDamageEffects((LivingEntity)akn4, akw6);
                }
                this.doPostHurtEffects(akw6);
            }
        }
        this.setDeltaMovement(this.getDeltaMovement().multiply(-0.01, -0.1, -0.01));
        float float5 = 1.0f;
        if (this.level instanceof ServerLevel && this.level.isThundering() && EnchantmentHelper.hasChanneling(this.tridentItem)) {
            final BlockPos fk9 = akn3.getCommandSenderBlockPosition();
            if (this.level.canSeeSky(fk9)) {
                final LightningBolt avv10 = new LightningBolt(this.level, fk9.getX() + 0.5, fk9.getY(), fk9.getZ() + 0.5, false);
                avv10.setCause((akn4 instanceof ServerPlayer) ? ((ServerPlayer)akn4) : null);
                ((ServerLevel)this.level).addGlobalEntity(avv10);
                aah7 = SoundEvents.TRIDENT_THUNDER;
                float5 = 5.0f;
            }
        }
        this.playSound(aah7, float5, 1.0f);
    }
    
    @Override
    protected SoundEvent getDefaultHitGroundSoundEvent() {
        return SoundEvents.TRIDENT_HIT_GROUND;
    }
    
    @Override
    public void playerTouch(final Player ayg) {
        final Entity akn3 = this.getOwner();
        if (akn3 != null && akn3.getUUID() != ayg.getUUID()) {
            return;
        }
        super.playerTouch(ayg);
    }
    
    @Override
    public void readAdditionalSaveData(final CompoundTag jt) {
        super.readAdditionalSaveData(jt);
        if (jt.contains("Trident", 10)) {
            this.tridentItem = ItemStack.of(jt.getCompound("Trident"));
        }
        this.dealtDamage = jt.getBoolean("DealtDamage");
        this.entityData.<Byte>set(ThrownTrident.ID_LOYALTY, (byte)EnchantmentHelper.getLoyalty(this.tridentItem));
    }
    
    @Override
    public void addAdditionalSaveData(final CompoundTag jt) {
        super.addAdditionalSaveData(jt);
        jt.put("Trident", this.tridentItem.save(new CompoundTag()));
        jt.putBoolean("DealtDamage", this.dealtDamage);
    }
    
    public void tickDespawn() {
        final int integer2 = this.entityData.<Byte>get(ThrownTrident.ID_LOYALTY);
        if (this.pickup != Pickup.ALLOWED || integer2 <= 0) {
            super.tickDespawn();
        }
    }
    
    @Override
    protected float getWaterInertia() {
        return 0.99f;
    }
    
    @Override
    public boolean shouldRender(final double double1, final double double2, final double double3) {
        return true;
    }
    
    static {
        ID_LOYALTY = SynchedEntityData.<Byte>defineId(ThrownTrident.class, EntityDataSerializers.BYTE);
        ID_FOIL = SynchedEntityData.<Boolean>defineId(ThrownTrident.class, EntityDataSerializers.BOOLEAN);
    }
}
