package net.minecraft.world.entity.schedule;

import java.util.function.Consumer;
import java.util.stream.Collector;
import java.util.stream.Collectors;
import java.util.function.Function;
import java.util.Set;
import com.google.common.collect.Lists;
import java.util.List;

public class ScheduleBuilder {
    private final Schedule schedule;
    private final List<ActivityTransition> transitions;
    
    public ScheduleBuilder(final Schedule azq) {
        this.transitions = Lists.newArrayList();
        this.schedule = azq;
    }
    
    public ScheduleBuilder changeActivityAt(final int integer, final Activity azo) {
        this.transitions.add(new ActivityTransition(integer, azo));
        return this;
    }
    
    public Schedule build() {
        this.transitions.stream().map(ActivityTransition::getActivity).collect(Collectors.toSet()).forEach(this.schedule::ensureTimelineExistsFor);
        final Activity azo3;
        this.transitions.forEach(a -> {
            azo3 = a.getActivity();
            this.schedule.getAllTimelinesExceptFor(azo3).forEach(azs -> azs.addKeyframe(a.getTime(), 0.0f));
            this.schedule.getTimelineFor(azo3).addKeyframe(a.getTime(), 1.0f);
            return;
        });
        return this.schedule;
    }
    
    static class ActivityTransition {
        private final int time;
        private final Activity activity;
        
        public ActivityTransition(final int integer, final Activity azo) {
            this.time = integer;
            this.activity = azo;
        }
        
        public int getTime() {
            return this.time;
        }
        
        public Activity getActivity() {
            return this.activity;
        }
    }
}
