package net.minecraft.world.inventory;

import net.minecraft.core.BlockPos;
import net.minecraft.world.level.block.entity.BlockEntity;
import net.minecraft.util.Mth;
import net.minecraft.server.level.ServerPlayer;
import net.minecraft.world.level.Level;
import net.minecraft.world.entity.player.Inventory;
import java.util.Iterator;
import net.minecraft.world.Container;
import net.minecraft.world.level.block.Block;
import com.google.common.collect.Sets;
import com.google.common.collect.Lists;
import net.minecraft.world.entity.player.Player;
import java.util.Set;
import javax.annotation.Nullable;
import java.util.List;
import net.minecraft.world.item.ItemStack;
import net.minecraft.core.NonNullList;

public abstract class AbstractContainerMenu {
    private final NonNullList<ItemStack> lastSlots;
    public final List<Slot> slots;
    private final List<DataSlot> dataSlots;
    @Nullable
    private final MenuType<?> menuType;
    public final int containerId;
    private short changeUid;
    private int quickcraftType;
    private int quickcraftStatus;
    private final Set<Slot> quickcraftSlots;
    private final List<ContainerListener> containerListeners;
    private final Set<Player> unSynchedPlayers;
    
    protected AbstractContainerMenu(@Nullable final MenuType<?> bbl, final int integer) {
        this.lastSlots = NonNullList.<ItemStack>create();
        this.slots = Lists.newArrayList();
        this.dataSlots = Lists.newArrayList();
        this.quickcraftType = -1;
        this.quickcraftSlots = Sets.newHashSet();
        this.containerListeners = Lists.newArrayList();
        this.unSynchedPlayers = Sets.newHashSet();
        this.menuType = bbl;
        this.containerId = integer;
    }
    
    protected static boolean stillValid(final ContainerLevelAccess bau, final Player ayg, final Block bpe) {
        return bau.<Boolean>evaluate((bjt, fk) -> {
            if (bjt.getBlockState(fk).getBlock() != bpe) {
                return false;
            }
            else {
                return ayg.distanceToSqr(fk.getX() + 0.5, fk.getY() + 0.5, fk.getZ() + 0.5) <= 64.0;
            }
        }, Boolean.valueOf(true));
    }
    
    public MenuType<?> getType() {
        if (this.menuType == null) {
            throw new UnsupportedOperationException("Unable to construct this menu by type");
        }
        return this.menuType;
    }
    
    protected static void checkContainerSize(final Container ajb, final int integer) {
        final int integer2 = ajb.getContainerSize();
        if (integer2 < integer) {
            throw new IllegalArgumentException("Container size " + integer2 + " is smaller than expected " + integer);
        }
    }
    
    protected static void checkContainerDataCount(final ContainerData bat, final int integer) {
        final int integer2 = bat.getCount();
        if (integer2 < integer) {
            throw new IllegalArgumentException("Container data count " + integer2 + " is smaller than expected " + integer);
        }
    }
    
    protected Slot addSlot(final Slot bbx) {
        bbx.index = this.slots.size();
        this.slots.add(bbx);
        this.lastSlots.add(ItemStack.EMPTY);
        return bbx;
    }
    
    protected DataSlot addDataSlot(final DataSlot bay) {
        this.dataSlots.add(bay);
        return bay;
    }
    
    protected void addDataSlots(final ContainerData bat) {
        for (int integer3 = 0; integer3 < bat.getCount(); ++integer3) {
            this.addDataSlot(DataSlot.forContainer(bat, integer3));
        }
    }
    
    public void addSlotListener(final ContainerListener bav) {
        if (this.containerListeners.contains(bav)) {
            return;
        }
        this.containerListeners.add(bav);
        bav.refreshContainer(this, this.getItems());
        this.broadcastChanges();
    }
    
    public void removeSlotListener(final ContainerListener bav) {
        this.containerListeners.remove(bav);
    }
    
    public NonNullList<ItemStack> getItems() {
        final NonNullList<ItemStack> fy2 = NonNullList.<ItemStack>create();
        for (int integer3 = 0; integer3 < this.slots.size(); ++integer3) {
            fy2.add(this.slots.get(integer3).getItem());
        }
        return fy2;
    }
    
    public void broadcastChanges() {
        for (int integer2 = 0; integer2 < this.slots.size(); ++integer2) {
            final ItemStack bek3 = this.slots.get(integer2).getItem();
            ItemStack bek4 = this.lastSlots.get(integer2);
            if (!ItemStack.matches(bek4, bek3)) {
                bek4 = bek3.copy();
                this.lastSlots.set(integer2, bek4);
                for (final ContainerListener bav6 : this.containerListeners) {
                    bav6.slotChanged(this, integer2, bek4);
                }
            }
        }
        for (int integer2 = 0; integer2 < this.dataSlots.size(); ++integer2) {
            final DataSlot bay3 = this.dataSlots.get(integer2);
            if (bay3.checkAndClearUpdateFlag()) {
                for (final ContainerListener bav7 : this.containerListeners) {
                    bav7.setContainerData(this, integer2, bay3.get());
                }
            }
        }
    }
    
    public boolean clickMenuButton(final Player ayg, final int integer) {
        return false;
    }
    
    public Slot getSlot(final int integer) {
        return this.slots.get(integer);
    }
    
    public ItemStack quickMoveStack(final Player ayg, final int integer) {
        final Slot bbx4 = this.slots.get(integer);
        if (bbx4 != null) {
            return bbx4.getItem();
        }
        return ItemStack.EMPTY;
    }
    
    public ItemStack clicked(final int integer1, final int integer2, final ClickType bas, final Player ayg) {
        ItemStack bek6 = ItemStack.EMPTY;
        final Inventory ayf7 = ayg.inventory;
        if (bas == ClickType.QUICK_CRAFT) {
            final int integer3 = this.quickcraftStatus;
            this.quickcraftStatus = getQuickcraftHeader(integer2);
            if ((integer3 != 1 || this.quickcraftStatus != 2) && integer3 != this.quickcraftStatus) {
                this.resetQuickCraft();
            }
            else if (ayf7.getCarried().isEmpty()) {
                this.resetQuickCraft();
            }
            else if (this.quickcraftStatus == 0) {
                this.quickcraftType = getQuickcraftType(integer2);
                if (isValidQuickcraftType(this.quickcraftType, ayg)) {
                    this.quickcraftStatus = 1;
                    this.quickcraftSlots.clear();
                }
                else {
                    this.resetQuickCraft();
                }
            }
            else if (this.quickcraftStatus == 1) {
                final Slot bbx9 = this.slots.get(integer1);
                final ItemStack bek7 = ayf7.getCarried();
                if (bbx9 != null && canItemQuickReplace(bbx9, bek7, true) && bbx9.mayPlace(bek7) && (this.quickcraftType == 2 || bek7.getCount() > this.quickcraftSlots.size()) && this.canDragTo(bbx9)) {
                    this.quickcraftSlots.add(bbx9);
                }
            }
            else if (this.quickcraftStatus == 2) {
                if (!this.quickcraftSlots.isEmpty()) {
                    final ItemStack bek8 = ayf7.getCarried().copy();
                    int integer4 = ayf7.getCarried().getCount();
                    for (final Slot bbx10 : this.quickcraftSlots) {
                        final ItemStack bek9 = ayf7.getCarried();
                        if (bbx10 != null && canItemQuickReplace(bbx10, bek9, true) && bbx10.mayPlace(bek9) && (this.quickcraftType == 2 || bek9.getCount() >= this.quickcraftSlots.size()) && this.canDragTo(bbx10)) {
                            final ItemStack bek10 = bek8.copy();
                            final int integer5 = bbx10.hasItem() ? bbx10.getItem().getCount() : 0;
                            getQuickCraftSlotCount(this.quickcraftSlots, this.quickcraftType, bek10, integer5);
                            final int integer6 = Math.min(bek10.getMaxStackSize(), bbx10.getMaxStackSize(bek10));
                            if (bek10.getCount() > integer6) {
                                bek10.setCount(integer6);
                            }
                            integer4 -= bek10.getCount() - integer5;
                            bbx10.set(bek10);
                        }
                    }
                    bek8.setCount(integer4);
                    ayf7.setCarried(bek8);
                }
                this.resetQuickCraft();
            }
            else {
                this.resetQuickCraft();
            }
        }
        else if (this.quickcraftStatus != 0) {
            this.resetQuickCraft();
        }
        else if ((bas == ClickType.PICKUP || bas == ClickType.QUICK_MOVE) && (integer2 == 0 || integer2 == 1)) {
            if (integer1 == -999) {
                if (!ayf7.getCarried().isEmpty()) {
                    if (integer2 == 0) {
                        ayg.drop(ayf7.getCarried(), true);
                        ayf7.setCarried(ItemStack.EMPTY);
                    }
                    if (integer2 == 1) {
                        ayg.drop(ayf7.getCarried().split(1), true);
                    }
                }
            }
            else if (bas == ClickType.QUICK_MOVE) {
                if (integer1 < 0) {
                    return ItemStack.EMPTY;
                }
                final Slot bbx11 = this.slots.get(integer1);
                if (bbx11 == null || !bbx11.mayPickup(ayg)) {
                    return ItemStack.EMPTY;
                }
                for (ItemStack bek8 = this.quickMoveStack(ayg, integer1); !bek8.isEmpty() && ItemStack.isSame(bbx11.getItem(), bek8); bek8 = this.quickMoveStack(ayg, integer1)) {
                    bek6 = bek8.copy();
                }
            }
            else {
                if (integer1 < 0) {
                    return ItemStack.EMPTY;
                }
                final Slot bbx11 = this.slots.get(integer1);
                if (bbx11 != null) {
                    ItemStack bek8 = bbx11.getItem();
                    final ItemStack bek7 = ayf7.getCarried();
                    if (!bek8.isEmpty()) {
                        bek6 = bek8.copy();
                    }
                    if (bek8.isEmpty()) {
                        if (!bek7.isEmpty() && bbx11.mayPlace(bek7)) {
                            int integer7 = (integer2 == 0) ? bek7.getCount() : 1;
                            if (integer7 > bbx11.getMaxStackSize(bek7)) {
                                integer7 = bbx11.getMaxStackSize(bek7);
                            }
                            bbx11.set(bek7.split(integer7));
                        }
                    }
                    else if (bbx11.mayPickup(ayg)) {
                        if (bek7.isEmpty()) {
                            if (bek8.isEmpty()) {
                                bbx11.set(ItemStack.EMPTY);
                                ayf7.setCarried(ItemStack.EMPTY);
                            }
                            else {
                                final int integer7 = (integer2 == 0) ? bek8.getCount() : ((bek8.getCount() + 1) / 2);
                                ayf7.setCarried(bbx11.remove(integer7));
                                if (bek8.isEmpty()) {
                                    bbx11.set(ItemStack.EMPTY);
                                }
                                bbx11.onTake(ayg, ayf7.getCarried());
                            }
                        }
                        else if (bbx11.mayPlace(bek7)) {
                            if (consideredTheSameItem(bek8, bek7)) {
                                int integer7 = (integer2 == 0) ? bek7.getCount() : 1;
                                if (integer7 > bbx11.getMaxStackSize(bek7) - bek8.getCount()) {
                                    integer7 = bbx11.getMaxStackSize(bek7) - bek8.getCount();
                                }
                                if (integer7 > bek7.getMaxStackSize() - bek8.getCount()) {
                                    integer7 = bek7.getMaxStackSize() - bek8.getCount();
                                }
                                bek7.shrink(integer7);
                                bek8.grow(integer7);
                            }
                            else if (bek7.getCount() <= bbx11.getMaxStackSize(bek7)) {
                                bbx11.set(bek7);
                                ayf7.setCarried(bek8);
                            }
                        }
                        else if (bek7.getMaxStackSize() > 1 && consideredTheSameItem(bek8, bek7) && !bek8.isEmpty()) {
                            final int integer7 = bek8.getCount();
                            if (integer7 + bek7.getCount() <= bek7.getMaxStackSize()) {
                                bek7.grow(integer7);
                                bek8 = bbx11.remove(integer7);
                                if (bek8.isEmpty()) {
                                    bbx11.set(ItemStack.EMPTY);
                                }
                                bbx11.onTake(ayg, ayf7.getCarried());
                            }
                        }
                    }
                    bbx11.setChanged();
                }
            }
        }
        else if (bas == ClickType.SWAP && integer2 >= 0 && integer2 < 9) {
            final Slot bbx11 = this.slots.get(integer1);
            final ItemStack bek8 = ayf7.getItem(integer2);
            final ItemStack bek7 = bbx11.getItem();
            if (!bek8.isEmpty() || !bek7.isEmpty()) {
                if (bek8.isEmpty()) {
                    if (bbx11.mayPickup(ayg)) {
                        ayf7.setItem(integer2, bek7);
                        bbx11.onSwapCraft(bek7.getCount());
                        bbx11.set(ItemStack.EMPTY);
                        bbx11.onTake(ayg, bek7);
                    }
                }
                else if (bek7.isEmpty()) {
                    if (bbx11.mayPlace(bek8)) {
                        final int integer7 = bbx11.getMaxStackSize(bek8);
                        if (bek8.getCount() > integer7) {
                            bbx11.set(bek8.split(integer7));
                        }
                        else {
                            bbx11.set(bek8);
                            ayf7.setItem(integer2, ItemStack.EMPTY);
                        }
                    }
                }
                else if (bbx11.mayPickup(ayg) && bbx11.mayPlace(bek8)) {
                    final int integer7 = bbx11.getMaxStackSize(bek8);
                    if (bek8.getCount() > integer7) {
                        bbx11.set(bek8.split(integer7));
                        bbx11.onTake(ayg, bek7);
                        if (!ayf7.add(bek7)) {
                            ayg.drop(bek7, true);
                        }
                    }
                    else {
                        bbx11.set(bek8);
                        ayf7.setItem(integer2, bek7);
                        bbx11.onTake(ayg, bek7);
                    }
                }
            }
        }
        else if (bas == ClickType.CLONE && ayg.abilities.instabuild && ayf7.getCarried().isEmpty() && integer1 >= 0) {
            final Slot bbx11 = this.slots.get(integer1);
            if (bbx11 != null && bbx11.hasItem()) {
                final ItemStack bek8 = bbx11.getItem().copy();
                bek8.setCount(bek8.getMaxStackSize());
                ayf7.setCarried(bek8);
            }
        }
        else if (bas == ClickType.THROW && ayf7.getCarried().isEmpty() && integer1 >= 0) {
            final Slot bbx11 = this.slots.get(integer1);
            if (bbx11 != null && bbx11.hasItem() && bbx11.mayPickup(ayg)) {
                final ItemStack bek8 = bbx11.remove((integer2 == 0) ? 1 : bbx11.getItem().getCount());
                bbx11.onTake(ayg, bek8);
                ayg.drop(bek8, true);
            }
        }
        else if (bas == ClickType.PICKUP_ALL && integer1 >= 0) {
            final Slot bbx11 = this.slots.get(integer1);
            final ItemStack bek8 = ayf7.getCarried();
            if (!bek8.isEmpty() && (bbx11 == null || !bbx11.hasItem() || !bbx11.mayPickup(ayg))) {
                final int integer4 = (integer2 == 0) ? 0 : (this.slots.size() - 1);
                final int integer7 = (integer2 == 0) ? 1 : -1;
                for (int integer8 = 0; integer8 < 2; ++integer8) {
                    for (int integer9 = integer4; integer9 >= 0 && integer9 < this.slots.size() && bek8.getCount() < bek8.getMaxStackSize(); integer9 += integer7) {
                        final Slot bbx12 = this.slots.get(integer9);
                        if (bbx12.hasItem() && canItemQuickReplace(bbx12, bek8, true) && bbx12.mayPickup(ayg) && this.canTakeItemForPickAll(bek8, bbx12)) {
                            final ItemStack bek11 = bbx12.getItem();
                            if (integer8 != 0 || bek11.getCount() != bek11.getMaxStackSize()) {
                                final int integer6 = Math.min(bek8.getMaxStackSize() - bek8.getCount(), bek11.getCount());
                                final ItemStack bek12 = bbx12.remove(integer6);
                                bek8.grow(integer6);
                                if (bek12.isEmpty()) {
                                    bbx12.set(ItemStack.EMPTY);
                                }
                                bbx12.onTake(ayg, bek12);
                            }
                        }
                    }
                }
            }
            this.broadcastChanges();
        }
        return bek6;
    }
    
    public static boolean consideredTheSameItem(final ItemStack bek1, final ItemStack bek2) {
        return bek1.getItem() == bek2.getItem() && ItemStack.tagMatches(bek1, bek2);
    }
    
    public boolean canTakeItemForPickAll(final ItemStack bek, final Slot bbx) {
        return true;
    }
    
    public void removed(final Player ayg) {
        final Inventory ayf3 = ayg.inventory;
        if (!ayf3.getCarried().isEmpty()) {
            ayg.drop(ayf3.getCarried(), false);
            ayf3.setCarried(ItemStack.EMPTY);
        }
    }
    
    protected void clearContainer(final Player ayg, final Level bjt, final Container ajb) {
        if (!ayg.isAlive() || (ayg instanceof ServerPlayer && ((ServerPlayer)ayg).hasDisconnected())) {
            for (int integer5 = 0; integer5 < ajb.getContainerSize(); ++integer5) {
                ayg.drop(ajb.removeItemNoUpdate(integer5), false);
            }
            return;
        }
        for (int integer5 = 0; integer5 < ajb.getContainerSize(); ++integer5) {
            ayg.inventory.placeItemBackInInventory(bjt, ajb.removeItemNoUpdate(integer5));
        }
    }
    
    public void slotsChanged(final Container ajb) {
        this.broadcastChanges();
    }
    
    public void setItem(final int integer, final ItemStack bek) {
        this.getSlot(integer).set(bek);
    }
    
    public void setAll(final List<ItemStack> list) {
        for (int integer3 = 0; integer3 < list.size(); ++integer3) {
            this.getSlot(integer3).set(list.get(integer3));
        }
    }
    
    public void setData(final int integer1, final int integer2) {
        this.dataSlots.get(integer1).set(integer2);
    }
    
    public short backup(final Inventory ayf) {
        return (short)(++this.changeUid);
    }
    
    public boolean isSynched(final Player ayg) {
        return !this.unSynchedPlayers.contains(ayg);
    }
    
    public void setSynched(final Player ayg, final boolean boolean2) {
        if (boolean2) {
            this.unSynchedPlayers.remove(ayg);
        }
        else {
            this.unSynchedPlayers.add(ayg);
        }
    }
    
    public abstract boolean stillValid(final Player ayg);
    
    protected boolean moveItemStackTo(final ItemStack bek, final int integer2, final int integer3, final boolean boolean4) {
        boolean boolean5 = false;
        int integer4 = integer2;
        if (boolean4) {
            integer4 = integer3 - 1;
        }
        if (bek.isStackable()) {
            while (!bek.isEmpty()) {
                if (boolean4) {
                    if (integer4 < integer2) {
                        break;
                    }
                }
                else if (integer4 >= integer3) {
                    break;
                }
                final Slot bbx8 = this.slots.get(integer4);
                final ItemStack bek2 = bbx8.getItem();
                if (!bek2.isEmpty() && consideredTheSameItem(bek, bek2)) {
                    final int integer5 = bek2.getCount() + bek.getCount();
                    if (integer5 <= bek.getMaxStackSize()) {
                        bek.setCount(0);
                        bek2.setCount(integer5);
                        bbx8.setChanged();
                        boolean5 = true;
                    }
                    else if (bek2.getCount() < bek.getMaxStackSize()) {
                        bek.shrink(bek.getMaxStackSize() - bek2.getCount());
                        bek2.setCount(bek.getMaxStackSize());
                        bbx8.setChanged();
                        boolean5 = true;
                    }
                }
                if (boolean4) {
                    --integer4;
                }
                else {
                    ++integer4;
                }
            }
        }
        if (!bek.isEmpty()) {
            if (boolean4) {
                integer4 = integer3 - 1;
            }
            else {
                integer4 = integer2;
            }
            while (true) {
                if (boolean4) {
                    if (integer4 < integer2) {
                        break;
                    }
                }
                else if (integer4 >= integer3) {
                    break;
                }
                final Slot bbx8 = this.slots.get(integer4);
                final ItemStack bek2 = bbx8.getItem();
                if (bek2.isEmpty() && bbx8.mayPlace(bek)) {
                    if (bek.getCount() > bbx8.getMaxStackSize()) {
                        bbx8.set(bek.split(bbx8.getMaxStackSize()));
                    }
                    else {
                        bbx8.set(bek.split(bek.getCount()));
                    }
                    bbx8.setChanged();
                    boolean5 = true;
                    break;
                }
                if (boolean4) {
                    --integer4;
                }
                else {
                    ++integer4;
                }
            }
        }
        return boolean5;
    }
    
    public static int getQuickcraftType(final int integer) {
        return integer >> 2 & 0x3;
    }
    
    public static int getQuickcraftHeader(final int integer) {
        return integer & 0x3;
    }
    
    public static int getQuickcraftMask(final int integer1, final int integer2) {
        return (integer1 & 0x3) | (integer2 & 0x3) << 2;
    }
    
    public static boolean isValidQuickcraftType(final int integer, final Player ayg) {
        return integer == 0 || integer == 1 || (integer == 2 && ayg.abilities.instabuild);
    }
    
    protected void resetQuickCraft() {
        this.quickcraftStatus = 0;
        this.quickcraftSlots.clear();
    }
    
    public static boolean canItemQuickReplace(@Nullable final Slot bbx, final ItemStack bek, final boolean boolean3) {
        final boolean boolean4 = bbx == null || !bbx.hasItem();
        if (!boolean4 && bek.sameItem(bbx.getItem()) && ItemStack.tagMatches(bbx.getItem(), bek)) {
            return bbx.getItem().getCount() + (boolean3 ? 0 : bek.getCount()) <= bek.getMaxStackSize();
        }
        return boolean4;
    }
    
    public static void getQuickCraftSlotCount(final Set<Slot> set, final int integer2, final ItemStack bek, final int integer4) {
        switch (integer2) {
            case 0: {
                bek.setCount(Mth.floor(bek.getCount() / (float)set.size()));
                break;
            }
            case 1: {
                bek.setCount(1);
                break;
            }
            case 2: {
                bek.setCount(bek.getItem().getMaxStackSize());
                break;
            }
        }
        bek.grow(integer4);
    }
    
    public boolean canDragTo(final Slot bbx) {
        return true;
    }
    
    public static int getRedstoneSignalFromBlockEntity(@Nullable final BlockEntity bwi) {
        if (bwi instanceof Container) {
            return getRedstoneSignalFromContainer((Container)bwi);
        }
        return 0;
    }
    
    public static int getRedstoneSignalFromContainer(@Nullable final Container ajb) {
        if (ajb == null) {
            return 0;
        }
        int integer2 = 0;
        float float3 = 0.0f;
        for (int integer3 = 0; integer3 < ajb.getContainerSize(); ++integer3) {
            final ItemStack bek5 = ajb.getItem(integer3);
            if (!bek5.isEmpty()) {
                float3 += bek5.getCount() / (float)Math.min(ajb.getMaxStackSize(), bek5.getMaxStackSize());
                ++integer2;
            }
        }
        float3 /= ajb.getContainerSize();
        return Mth.floor(float3 * 14.0f) + ((integer2 > 0) ? 1 : 0);
    }
}
