package net.minecraft.world.inventory;

import net.minecraft.world.level.block.entity.AbstractFurnaceBlockEntity;
import net.minecraft.world.item.ItemStack;
import net.minecraft.world.entity.player.Player;
import net.minecraft.recipebook.ServerPlaceSmeltingRecipe;
import net.minecraft.server.level.ServerPlayer;
import net.minecraft.world.item.crafting.Recipe;
import net.minecraft.world.entity.player.StackedContents;
import net.minecraft.world.SimpleContainer;
import net.minecraft.world.entity.player.Inventory;
import net.minecraft.world.item.crafting.AbstractCookingRecipe;
import net.minecraft.world.item.crafting.RecipeType;
import net.minecraft.world.level.Level;
import net.minecraft.world.Container;

public abstract class AbstractFurnaceMenu extends RecipeBookMenu<Container> {
    private final Container container;
    private final ContainerData data;
    protected final Level level;
    private final RecipeType<? extends AbstractCookingRecipe> recipeType;
    
    protected AbstractFurnaceMenu(final MenuType<?> bbl, final RecipeType<? extends AbstractCookingRecipe> bgw, final int integer, final Inventory ayf) {
        this(bbl, bgw, integer, ayf, new SimpleContainer(3), new SimpleContainerData(4));
    }
    
    protected AbstractFurnaceMenu(final MenuType<?> bbl, final RecipeType<? extends AbstractCookingRecipe> bgw, final int integer, final Inventory ayf, final Container ajb, final ContainerData bat) {
        super(bbl, integer);
        this.recipeType = bgw;
        AbstractContainerMenu.checkContainerSize(ajb, 3);
        AbstractContainerMenu.checkContainerDataCount(bat, 4);
        this.container = ajb;
        this.data = bat;
        this.level = ayf.player.level;
        this.addSlot(new Slot(ajb, 0, 56, 17));
        this.addSlot(new FurnaceFuelSlot(this, ajb, 1, 56, 53));
        this.addSlot(new FurnaceResultSlot(ayf.player, ajb, 2, 116, 35));
        for (int integer2 = 0; integer2 < 3; ++integer2) {
            for (int integer3 = 0; integer3 < 9; ++integer3) {
                this.addSlot(new Slot(ayf, integer3 + integer2 * 9 + 9, 8 + integer3 * 18, 84 + integer2 * 18));
            }
        }
        for (int integer2 = 0; integer2 < 9; ++integer2) {
            this.addSlot(new Slot(ayf, integer2, 8 + integer2 * 18, 142));
        }
        this.addDataSlots(bat);
    }
    
    @Override
    public void fillCraftSlotsStackedContents(final StackedContents ayi) {
        if (this.container instanceof StackedContentsCompatible) {
            ((StackedContentsCompatible)this.container).fillStackedContents(ayi);
        }
    }
    
    @Override
    public void clearCraftingContent() {
        this.container.clearContent();
    }
    
    @Override
    public void handlePlacement(final boolean boolean1, final Recipe<?> bgt, final ServerPlayer xe) {
        new ServerPlaceSmeltingRecipe<>(this).recipeClicked(xe, bgt, boolean1);
    }
    
    @Override
    public boolean recipeMatches(final Recipe<? super Container> bgt) {
        return bgt.matches(this.container, this.level);
    }
    
    @Override
    public int getResultSlotIndex() {
        return 2;
    }
    
    @Override
    public int getGridWidth() {
        return 1;
    }
    
    @Override
    public int getGridHeight() {
        return 1;
    }
    
    @Override
    public int getSize() {
        return 3;
    }
    
    @Override
    public boolean stillValid(final Player ayg) {
        return this.container.stillValid(ayg);
    }
    
    @Override
    public ItemStack quickMoveStack(final Player ayg, final int integer) {
        ItemStack bek4 = ItemStack.EMPTY;
        final Slot bbx5 = this.slots.get(integer);
        if (bbx5 != null && bbx5.hasItem()) {
            final ItemStack bek5 = bbx5.getItem();
            bek4 = bek5.copy();
            if (integer == 2) {
                if (!this.moveItemStackTo(bek5, 3, 39, true)) {
                    return ItemStack.EMPTY;
                }
                bbx5.onQuickCraft(bek5, bek4);
            }
            else if (integer == 1 || integer == 0) {
                if (!this.moveItemStackTo(bek5, 3, 39, false)) {
                    return ItemStack.EMPTY;
                }
            }
            else if (this.canSmelt(bek5)) {
                if (!this.moveItemStackTo(bek5, 0, 1, false)) {
                    return ItemStack.EMPTY;
                }
            }
            else if (this.isFuel(bek5)) {
                if (!this.moveItemStackTo(bek5, 1, 2, false)) {
                    return ItemStack.EMPTY;
                }
            }
            else if (integer >= 3 && integer < 30) {
                if (!this.moveItemStackTo(bek5, 30, 39, false)) {
                    return ItemStack.EMPTY;
                }
            }
            else if (integer >= 30 && integer < 39 && !this.moveItemStackTo(bek5, 3, 30, false)) {
                return ItemStack.EMPTY;
            }
            if (bek5.isEmpty()) {
                bbx5.set(ItemStack.EMPTY);
            }
            else {
                bbx5.setChanged();
            }
            if (bek5.getCount() == bek4.getCount()) {
                return ItemStack.EMPTY;
            }
            bbx5.onTake(ayg, bek5);
        }
        return bek4;
    }
    
    protected boolean canSmelt(final ItemStack bek) {
        return this.level.getRecipeManager().<SimpleContainer, Recipe>getRecipeFor((RecipeType<Recipe>)this.recipeType, new SimpleContainer(new ItemStack[] { bek }), this.level).isPresent();
    }
    
    protected boolean isFuel(final ItemStack bek) {
        return AbstractFurnaceBlockEntity.isFuel(bek);
    }
    
    public int getBurnProgress() {
        final int integer2 = this.data.get(2);
        final int integer3 = this.data.get(3);
        if (integer3 == 0 || integer2 == 0) {
            return 0;
        }
        return integer2 * 24 / integer3;
    }
    
    public int getLitProgress() {
        int integer2 = this.data.get(1);
        if (integer2 == 0) {
            integer2 = 200;
        }
        return this.data.get(0) * 13 / integer2;
    }
    
    public boolean isLit() {
        return this.data.get(0) > 0;
    }
}
