package net.minecraft.world.inventory;

import org.apache.logging.log4j.LogManager;
import java.util.Iterator;
import java.util.Map;
import net.minecraft.network.chat.Component;
import net.minecraft.network.chat.TextComponent;
import org.apache.commons.lang3.StringUtils;
import net.minecraft.world.item.enchantment.Enchantment;
import net.minecraft.world.item.EnchantedBookItem;
import net.minecraft.world.item.Items;
import net.minecraft.world.item.enchantment.EnchantmentHelper;
import net.minecraft.core.BlockPos;
import net.minecraft.world.level.Level;
import net.minecraft.world.level.block.state.BlockState;
import net.minecraft.world.level.block.AnvilBlock;
import net.minecraft.tags.BlockTags;
import net.minecraft.world.item.ItemStack;
import net.minecraft.world.SimpleContainer;
import net.minecraft.world.entity.player.Inventory;
import net.minecraft.world.entity.player.Player;
import net.minecraft.world.Container;
import org.apache.logging.log4j.Logger;

public class AnvilMenu extends AbstractContainerMenu {
    private static final Logger LOGGER;
    private final Container resultSlots;
    private final Container repairSlots;
    private final DataSlot cost;
    private final ContainerLevelAccess access;
    private int repairItemCountCost;
    private String itemName;
    private final Player player;
    
    public AnvilMenu(final int integer, final Inventory ayf) {
        this(integer, ayf, ContainerLevelAccess.NULL);
    }
    
    public AnvilMenu(final int integer, final Inventory ayf, final ContainerLevelAccess bau) {
        super(MenuType.ANVIL, integer);
        this.resultSlots = new ResultContainer();
        this.repairSlots = new SimpleContainer(2) {
            @Override
            public void setChanged() {
                super.setChanged();
                AnvilMenu.this.slotsChanged(this);
            }
        };
        this.cost = DataSlot.standalone();
        this.access = bau;
        this.player = ayf.player;
        this.addDataSlot(this.cost);
        this.addSlot(new Slot(this.repairSlots, 0, 27, 47));
        this.addSlot(new Slot(this.repairSlots, 1, 76, 47));
        this.addSlot(new Slot(this.resultSlots, 2, 134, 47) {
            @Override
            public boolean mayPlace(final ItemStack bek) {
                return false;
            }
            
            @Override
            public boolean mayPickup(final Player ayg) {
                return (ayg.abilities.instabuild || ayg.experienceLevel >= AnvilMenu.this.cost.get()) && AnvilMenu.this.cost.get() > 0 && this.hasItem();
            }
            
            @Override
            public ItemStack onTake(final Player ayg, final ItemStack bek) {
                if (!ayg.abilities.instabuild) {
                    ayg.giveExperienceLevels(-AnvilMenu.this.cost.get());
                }
                AnvilMenu.this.repairSlots.setItem(0, ItemStack.EMPTY);
                if (AnvilMenu.this.repairItemCountCost > 0) {
                    final ItemStack bek2 = AnvilMenu.this.repairSlots.getItem(1);
                    if (!bek2.isEmpty() && bek2.getCount() > AnvilMenu.this.repairItemCountCost) {
                        bek2.shrink(AnvilMenu.this.repairItemCountCost);
                        AnvilMenu.this.repairSlots.setItem(1, bek2);
                    }
                    else {
                        AnvilMenu.this.repairSlots.setItem(1, ItemStack.EMPTY);
                    }
                }
                else {
                    AnvilMenu.this.repairSlots.setItem(1, ItemStack.EMPTY);
                }
                AnvilMenu.this.cost.set(0);
                final BlockState byg4;
                BlockState byg5;
                bau.execute((bjt, fk) -> {
                    byg4 = bjt.getBlockState(fk);
                    if (!ayg.abilities.instabuild && byg4.is(BlockTags.ANVIL) && ayg.getRandom().nextFloat() < 0.12f) {
                        byg5 = AnvilBlock.damage(byg4);
                        if (byg5 == null) {
                            bjt.removeBlock(fk, false);
                            bjt.levelEvent(1029, fk, 0);
                        }
                        else {
                            bjt.setBlock(fk, byg5, 2);
                            bjt.levelEvent(1030, fk, 0);
                        }
                    }
                    else {
                        bjt.levelEvent(1030, fk, 0);
                    }
                    return;
                });
                return bek;
            }
        });
        for (int integer2 = 0; integer2 < 3; ++integer2) {
            for (int integer3 = 0; integer3 < 9; ++integer3) {
                this.addSlot(new Slot(ayf, integer3 + integer2 * 9 + 9, 8 + integer3 * 18, 84 + integer2 * 18));
            }
        }
        for (int integer2 = 0; integer2 < 9; ++integer2) {
            this.addSlot(new Slot(ayf, integer2, 8 + integer2 * 18, 142));
        }
    }
    
    @Override
    public void slotsChanged(final Container ajb) {
        super.slotsChanged(ajb);
        if (ajb == this.repairSlots) {
            this.createResult();
        }
    }
    
    public void createResult() {
        final ItemStack bek2 = this.repairSlots.getItem(0);
        this.cost.set(1);
        int integer3 = 0;
        int integer4 = 0;
        int integer5 = 0;
        if (bek2.isEmpty()) {
            this.resultSlots.setItem(0, ItemStack.EMPTY);
            this.cost.set(0);
            return;
        }
        ItemStack bek3 = bek2.copy();
        final ItemStack bek4 = this.repairSlots.getItem(1);
        final Map<Enchantment, Integer> map8 = EnchantmentHelper.getEnchantments(bek3);
        integer4 += bek2.getBaseRepairCost() + (bek4.isEmpty() ? 0 : bek4.getBaseRepairCost());
        this.repairItemCountCost = 0;
        if (!bek4.isEmpty()) {
            final boolean boolean9 = bek4.getItem() == Items.ENCHANTED_BOOK && !EnchantedBookItem.getEnchantments(bek4).isEmpty();
            if (bek3.isDamageableItem() && bek3.getItem().isValidRepairItem(bek2, bek4)) {
                int integer6 = Math.min(bek3.getDamageValue(), bek3.getMaxDamage() / 4);
                if (integer6 <= 0) {
                    this.resultSlots.setItem(0, ItemStack.EMPTY);
                    this.cost.set(0);
                    return;
                }
                int integer7;
                for (integer7 = 0; integer6 > 0 && integer7 < bek4.getCount(); integer6 = Math.min(bek3.getDamageValue(), bek3.getMaxDamage() / 4), ++integer7) {
                    final int integer8 = bek3.getDamageValue() - integer6;
                    bek3.setDamageValue(integer8);
                    ++integer3;
                }
                this.repairItemCountCost = integer7;
            }
            else {
                if (!boolean9 && (bek3.getItem() != bek4.getItem() || !bek3.isDamageableItem())) {
                    this.resultSlots.setItem(0, ItemStack.EMPTY);
                    this.cost.set(0);
                    return;
                }
                if (bek3.isDamageableItem() && !boolean9) {
                    final int integer6 = bek2.getMaxDamage() - bek2.getDamageValue();
                    final int integer7 = bek4.getMaxDamage() - bek4.getDamageValue();
                    final int integer8 = integer7 + bek3.getMaxDamage() * 12 / 100;
                    final int integer9 = integer6 + integer8;
                    int integer10 = bek3.getMaxDamage() - integer9;
                    if (integer10 < 0) {
                        integer10 = 0;
                    }
                    if (integer10 < bek3.getDamageValue()) {
                        bek3.setDamageValue(integer10);
                        integer3 += 2;
                    }
                }
                final Map<Enchantment, Integer> map9 = EnchantmentHelper.getEnchantments(bek4);
                boolean boolean10 = false;
                boolean boolean11 = false;
                for (final Enchantment bhu14 : map9.keySet()) {
                    if (bhu14 == null) {
                        continue;
                    }
                    final int integer11 = map8.containsKey(bhu14) ? map8.get(bhu14) : 0;
                    int integer12 = map9.get(bhu14);
                    integer12 = ((integer11 == integer12) ? (integer12 + 1) : Math.max(integer12, integer11));
                    boolean boolean12 = bhu14.canEnchant(bek2);
                    if (this.player.abilities.instabuild || bek2.getItem() == Items.ENCHANTED_BOOK) {
                        boolean12 = true;
                    }
                    for (final Enchantment bhu15 : map8.keySet()) {
                        if (bhu15 != bhu14 && !bhu14.isCompatibleWith(bhu15)) {
                            boolean12 = false;
                            ++integer3;
                        }
                    }
                    if (!boolean12) {
                        boolean11 = true;
                    }
                    else {
                        boolean10 = true;
                        if (integer12 > bhu14.getMaxLevel()) {
                            integer12 = bhu14.getMaxLevel();
                        }
                        map8.put(bhu14, integer12);
                        int integer13 = 0;
                        switch (bhu14.getRarity()) {
                            case COMMON: {
                                integer13 = 1;
                                break;
                            }
                            case UNCOMMON: {
                                integer13 = 2;
                                break;
                            }
                            case RARE: {
                                integer13 = 4;
                                break;
                            }
                            case VERY_RARE: {
                                integer13 = 8;
                                break;
                            }
                        }
                        if (boolean9) {
                            integer13 = Math.max(1, integer13 / 2);
                        }
                        integer3 += integer13 * integer12;
                        if (bek2.getCount() <= 1) {
                            continue;
                        }
                        integer3 = 40;
                    }
                }
                if (boolean11 && !boolean10) {
                    this.resultSlots.setItem(0, ItemStack.EMPTY);
                    this.cost.set(0);
                    return;
                }
            }
        }
        if (StringUtils.isBlank((CharSequence)this.itemName)) {
            if (bek2.hasCustomHoverName()) {
                integer5 = 1;
                integer3 += integer5;
                bek3.resetHoverName();
            }
        }
        else if (!this.itemName.equals(bek2.getHoverName().getString())) {
            integer5 = 1;
            integer3 += integer5;
            bek3.setHoverName(new TextComponent(this.itemName));
        }
        this.cost.set(integer4 + integer3);
        if (integer3 <= 0) {
            bek3 = ItemStack.EMPTY;
        }
        if (integer5 == integer3 && integer5 > 0 && this.cost.get() >= 40) {
            this.cost.set(39);
        }
        if (this.cost.get() >= 40 && !this.player.abilities.instabuild) {
            bek3 = ItemStack.EMPTY;
        }
        if (!bek3.isEmpty()) {
            int integer14 = bek3.getBaseRepairCost();
            if (!bek4.isEmpty() && integer14 < bek4.getBaseRepairCost()) {
                integer14 = bek4.getBaseRepairCost();
            }
            if (integer5 != integer3 || integer5 == 0) {
                integer14 = calculateIncreasedRepairCost(integer14);
            }
            bek3.setRepairCost(integer14);
            EnchantmentHelper.setEnchantments(map8, bek3);
        }
        this.resultSlots.setItem(0, bek3);
        this.broadcastChanges();
    }
    
    public static int calculateIncreasedRepairCost(final int integer) {
        return integer * 2 + 1;
    }
    
    @Override
    public void removed(final Player ayg) {
        super.removed(ayg);
        this.access.execute((bjt, fk) -> this.clearContainer(ayg, bjt, this.repairSlots));
    }
    
    @Override
    public boolean stillValid(final Player ayg) {
        return this.access.<Boolean>evaluate((bjt, fk) -> {
            if (!bjt.getBlockState(fk).is(BlockTags.ANVIL)) {
                return false;
            }
            else {
                return ayg.distanceToSqr(fk.getX() + 0.5, fk.getY() + 0.5, fk.getZ() + 0.5) <= 64.0;
            }
        }, Boolean.valueOf(true));
    }
    
    @Override
    public ItemStack quickMoveStack(final Player ayg, final int integer) {
        ItemStack bek4 = ItemStack.EMPTY;
        final Slot bbx5 = this.slots.get(integer);
        if (bbx5 != null && bbx5.hasItem()) {
            final ItemStack bek5 = bbx5.getItem();
            bek4 = bek5.copy();
            if (integer == 2) {
                if (!this.moveItemStackTo(bek5, 3, 39, true)) {
                    return ItemStack.EMPTY;
                }
                bbx5.onQuickCraft(bek5, bek4);
            }
            else if (integer == 0 || integer == 1) {
                if (!this.moveItemStackTo(bek5, 3, 39, false)) {
                    return ItemStack.EMPTY;
                }
            }
            else if (integer >= 3 && integer < 39 && !this.moveItemStackTo(bek5, 0, 2, false)) {
                return ItemStack.EMPTY;
            }
            if (bek5.isEmpty()) {
                bbx5.set(ItemStack.EMPTY);
            }
            else {
                bbx5.setChanged();
            }
            if (bek5.getCount() == bek4.getCount()) {
                return ItemStack.EMPTY;
            }
            bbx5.onTake(ayg, bek5);
        }
        return bek4;
    }
    
    public void setItemName(final String string) {
        this.itemName = string;
        if (this.getSlot(2).hasItem()) {
            final ItemStack bek3 = this.getSlot(2).getItem();
            if (StringUtils.isBlank((CharSequence)string)) {
                bek3.resetHoverName();
            }
            else {
                bek3.setHoverName(new TextComponent(this.itemName));
            }
        }
        this.createResult();
    }
    
    public int getCost() {
        return this.cost.get();
    }
    
    static {
        LOGGER = LogManager.getLogger();
    }
}
