package net.minecraft.world.inventory;

import net.minecraft.world.item.Item;
import javax.annotation.Nullable;
import net.minecraft.world.effect.MobEffect;
import net.minecraft.world.level.block.Blocks;
import net.minecraft.world.entity.player.Player;
import net.minecraft.world.item.Items;
import net.minecraft.world.item.ItemStack;
import net.minecraft.world.SimpleContainer;
import net.minecraft.world.Container;

public class BeaconMenu extends AbstractContainerMenu {
    private final Container beacon;
    private final PaymentSlot paymentSlot;
    private final ContainerLevelAccess access;
    private final ContainerData beaconData;
    
    public BeaconMenu(final int integer, final Container ajb) {
        this(integer, ajb, new SimpleContainerData(3), ContainerLevelAccess.NULL);
    }
    
    public BeaconMenu(final int integer, final Container ajb, final ContainerData bat, final ContainerLevelAccess bau) {
        super(MenuType.BEACON, integer);
        this.beacon = new SimpleContainer(1) {
            @Override
            public boolean canPlaceItem(final int integer, final ItemStack bek) {
                return bek.getItem() == Items.EMERALD || bek.getItem() == Items.DIAMOND || bek.getItem() == Items.GOLD_INGOT || bek.getItem() == Items.IRON_INGOT;
            }
            
            @Override
            public int getMaxStackSize() {
                return 1;
            }
        };
        AbstractContainerMenu.checkContainerDataCount(bat, 3);
        this.beaconData = bat;
        this.access = bau;
        this.addSlot(this.paymentSlot = new PaymentSlot(this.beacon, 0, 136, 110));
        this.addDataSlots(bat);
        final int integer2 = 36;
        final int integer3 = 137;
        for (int integer4 = 0; integer4 < 3; ++integer4) {
            for (int integer5 = 0; integer5 < 9; ++integer5) {
                this.addSlot(new Slot(ajb, integer5 + integer4 * 9 + 9, 36 + integer5 * 18, 137 + integer4 * 18));
            }
        }
        for (int integer4 = 0; integer4 < 9; ++integer4) {
            this.addSlot(new Slot(ajb, integer4, 36 + integer4 * 18, 195));
        }
    }
    
    @Override
    public void removed(final Player ayg) {
        super.removed(ayg);
        if (ayg.level.isClientSide) {
            return;
        }
        final ItemStack bek3 = this.paymentSlot.remove(this.paymentSlot.getMaxStackSize());
        if (!bek3.isEmpty()) {
            ayg.drop(bek3, false);
        }
    }
    
    @Override
    public boolean stillValid(final Player ayg) {
        return AbstractContainerMenu.stillValid(this.access, ayg, Blocks.BEACON);
    }
    
    @Override
    public void setData(final int integer1, final int integer2) {
        super.setData(integer1, integer2);
        this.broadcastChanges();
    }
    
    @Override
    public ItemStack quickMoveStack(final Player ayg, final int integer) {
        ItemStack bek4 = ItemStack.EMPTY;
        final Slot bbx5 = this.slots.get(integer);
        if (bbx5 != null && bbx5.hasItem()) {
            final ItemStack bek5 = bbx5.getItem();
            bek4 = bek5.copy();
            if (integer == 0) {
                if (!this.moveItemStackTo(bek5, 1, 37, true)) {
                    return ItemStack.EMPTY;
                }
                bbx5.onQuickCraft(bek5, bek4);
            }
            else if (!this.paymentSlot.hasItem() && this.paymentSlot.mayPlace(bek5) && bek5.getCount() == 1) {
                if (!this.moveItemStackTo(bek5, 0, 1, false)) {
                    return ItemStack.EMPTY;
                }
            }
            else if (integer >= 1 && integer < 28) {
                if (!this.moveItemStackTo(bek5, 28, 37, false)) {
                    return ItemStack.EMPTY;
                }
            }
            else if (integer >= 28 && integer < 37) {
                if (!this.moveItemStackTo(bek5, 1, 28, false)) {
                    return ItemStack.EMPTY;
                }
            }
            else if (!this.moveItemStackTo(bek5, 1, 37, false)) {
                return ItemStack.EMPTY;
            }
            if (bek5.isEmpty()) {
                bbx5.set(ItemStack.EMPTY);
            }
            else {
                bbx5.setChanged();
            }
            if (bek5.getCount() == bek4.getCount()) {
                return ItemStack.EMPTY;
            }
            bbx5.onTake(ayg, bek5);
        }
        return bek4;
    }
    
    public int getLevels() {
        return this.beaconData.get(0);
    }
    
    @Nullable
    public MobEffect getPrimaryEffect() {
        return MobEffect.byId(this.beaconData.get(1));
    }
    
    @Nullable
    public MobEffect getSecondaryEffect() {
        return MobEffect.byId(this.beaconData.get(2));
    }
    
    public void updateEffects(final int integer1, final int integer2) {
        if (this.paymentSlot.hasItem()) {
            this.beaconData.set(1, integer1);
            this.beaconData.set(2, integer2);
            this.paymentSlot.remove(1);
        }
    }
    
    public boolean hasPayment() {
        return !this.beacon.getItem(0).isEmpty();
    }
    
    class PaymentSlot extends Slot {
        public PaymentSlot(final Container ajb, final int integer3, final int integer4, final int integer5) {
            super(ajb, integer3, integer4, integer5);
        }
        
        @Override
        public boolean mayPlace(final ItemStack bek) {
            final Item bef3 = bek.getItem();
            return bef3 == Items.EMERALD || bef3 == Items.DIAMOND || bef3 == Items.GOLD_INGOT || bef3 == Items.IRON_INGOT;
        }
        
        @Override
        public int getMaxStackSize() {
            return 1;
        }
    }
}
