package net.minecraft.world.inventory;

import net.minecraft.world.item.alchemy.PotionBrewing;
import net.minecraft.world.item.Item;
import net.minecraft.world.item.Items;
import net.minecraft.world.item.alchemy.Potion;
import net.minecraft.advancements.CriteriaTriggers;
import net.minecraft.server.level.ServerPlayer;
import net.minecraft.world.item.alchemy.PotionUtils;
import net.minecraft.world.item.ItemStack;
import net.minecraft.world.entity.player.Player;
import net.minecraft.world.SimpleContainer;
import net.minecraft.world.entity.player.Inventory;
import net.minecraft.world.Container;

public class BrewingStandMenu extends AbstractContainerMenu {
    private final Container brewingStand;
    private final ContainerData brewingStandData;
    private final Slot ingredientSlot;
    
    public BrewingStandMenu(final int integer, final Inventory ayf) {
        this(integer, ayf, new SimpleContainer(5), new SimpleContainerData(2));
    }
    
    public BrewingStandMenu(final int integer, final Inventory ayf, final Container ajb, final ContainerData bat) {
        super(MenuType.BREWING_STAND, integer);
        AbstractContainerMenu.checkContainerSize(ajb, 5);
        AbstractContainerMenu.checkContainerDataCount(bat, 2);
        this.brewingStand = ajb;
        this.brewingStandData = bat;
        this.addSlot(new PotionSlot(ajb, 0, 56, 51));
        this.addSlot(new PotionSlot(ajb, 1, 79, 58));
        this.addSlot(new PotionSlot(ajb, 2, 102, 51));
        this.ingredientSlot = this.addSlot(new IngredientsSlot(ajb, 3, 79, 17));
        this.addSlot(new FuelSlot(ajb, 4, 17, 17));
        this.addDataSlots(bat);
        for (int integer2 = 0; integer2 < 3; ++integer2) {
            for (int integer3 = 0; integer3 < 9; ++integer3) {
                this.addSlot(new Slot(ayf, integer3 + integer2 * 9 + 9, 8 + integer3 * 18, 84 + integer2 * 18));
            }
        }
        for (int integer2 = 0; integer2 < 9; ++integer2) {
            this.addSlot(new Slot(ayf, integer2, 8 + integer2 * 18, 142));
        }
    }
    
    @Override
    public boolean stillValid(final Player ayg) {
        return this.brewingStand.stillValid(ayg);
    }
    
    @Override
    public ItemStack quickMoveStack(final Player ayg, final int integer) {
        ItemStack bek4 = ItemStack.EMPTY;
        final Slot bbx5 = this.slots.get(integer);
        if (bbx5 != null && bbx5.hasItem()) {
            final ItemStack bek5 = bbx5.getItem();
            bek4 = bek5.copy();
            if ((integer >= 0 && integer <= 2) || integer == 3 || integer == 4) {
                if (!this.moveItemStackTo(bek5, 5, 41, true)) {
                    return ItemStack.EMPTY;
                }
                bbx5.onQuickCraft(bek5, bek4);
            }
            else if (FuelSlot.mayPlaceItem(bek4)) {
                if (this.moveItemStackTo(bek5, 4, 5, false) || (this.ingredientSlot.mayPlace(bek5) && !this.moveItemStackTo(bek5, 3, 4, false))) {
                    return ItemStack.EMPTY;
                }
            }
            else if (this.ingredientSlot.mayPlace(bek5)) {
                if (!this.moveItemStackTo(bek5, 3, 4, false)) {
                    return ItemStack.EMPTY;
                }
            }
            else if (PotionSlot.mayPlaceItem(bek4) && bek4.getCount() == 1) {
                if (!this.moveItemStackTo(bek5, 0, 3, false)) {
                    return ItemStack.EMPTY;
                }
            }
            else if (integer >= 5 && integer < 32) {
                if (!this.moveItemStackTo(bek5, 32, 41, false)) {
                    return ItemStack.EMPTY;
                }
            }
            else if (integer >= 32 && integer < 41) {
                if (!this.moveItemStackTo(bek5, 5, 32, false)) {
                    return ItemStack.EMPTY;
                }
            }
            else if (!this.moveItemStackTo(bek5, 5, 41, false)) {
                return ItemStack.EMPTY;
            }
            if (bek5.isEmpty()) {
                bbx5.set(ItemStack.EMPTY);
            }
            else {
                bbx5.setChanged();
            }
            if (bek5.getCount() == bek4.getCount()) {
                return ItemStack.EMPTY;
            }
            bbx5.onTake(ayg, bek5);
        }
        return bek4;
    }
    
    public int getFuel() {
        return this.brewingStandData.get(1);
    }
    
    public int getBrewingTicks() {
        return this.brewingStandData.get(0);
    }
    
    static class PotionSlot extends Slot {
        public PotionSlot(final Container ajb, final int integer2, final int integer3, final int integer4) {
            super(ajb, integer2, integer3, integer4);
        }
        
        @Override
        public boolean mayPlace(final ItemStack bek) {
            return mayPlaceItem(bek);
        }
        
        @Override
        public int getMaxStackSize() {
            return 1;
        }
        
        @Override
        public ItemStack onTake(final Player ayg, final ItemStack bek) {
            final Potion bga4 = PotionUtils.getPotion(bek);
            if (ayg instanceof ServerPlayer) {
                CriteriaTriggers.BREWED_POTION.trigger((ServerPlayer)ayg, bga4);
            }
            super.onTake(ayg, bek);
            return bek;
        }
        
        public static boolean mayPlaceItem(final ItemStack bek) {
            final Item bef2 = bek.getItem();
            return bef2 == Items.POTION || bef2 == Items.SPLASH_POTION || bef2 == Items.LINGERING_POTION || bef2 == Items.GLASS_BOTTLE;
        }
    }
    
    static class IngredientsSlot extends Slot {
        public IngredientsSlot(final Container ajb, final int integer2, final int integer3, final int integer4) {
            super(ajb, integer2, integer3, integer4);
        }
        
        @Override
        public boolean mayPlace(final ItemStack bek) {
            return PotionBrewing.isIngredient(bek);
        }
        
        @Override
        public int getMaxStackSize() {
            return 64;
        }
    }
    
    static class FuelSlot extends Slot {
        public FuelSlot(final Container ajb, final int integer2, final int integer3, final int integer4) {
            super(ajb, integer2, integer3, integer4);
        }
        
        @Override
        public boolean mayPlace(final ItemStack bek) {
            return mayPlaceItem(bek);
        }
        
        public static boolean mayPlaceItem(final ItemStack bek) {
            return bek.getItem() == Items.BLAZE_POWDER;
        }
        
        @Override
        public int getMaxStackSize() {
            return 64;
        }
    }
}
