package net.minecraft.world.inventory;

import net.minecraft.world.level.saveddata.maps.MapItemSavedData;
import net.minecraft.world.level.block.Blocks;
import net.minecraft.core.BlockPos;
import net.minecraft.world.level.Level;
import net.minecraft.sounds.SoundSource;
import net.minecraft.sounds.SoundEvents;
import net.minecraft.world.entity.player.Player;
import net.minecraft.world.item.MapItem;
import net.minecraft.world.item.Item;
import net.minecraft.world.item.Items;
import net.minecraft.world.item.ItemStack;
import net.minecraft.world.SimpleContainer;
import net.minecraft.world.entity.player.Inventory;
import net.minecraft.world.Container;

public class CartographyTableMenu extends AbstractContainerMenu {
    private final ContainerLevelAccess access;
    private boolean quickMoved;
    private long lastSoundTime;
    public final Container container;
    private final ResultContainer resultContainer;
    
    public CartographyTableMenu(final int integer, final Inventory ayf) {
        this(integer, ayf, ContainerLevelAccess.NULL);
    }
    
    public CartographyTableMenu(final int integer, final Inventory ayf, final ContainerLevelAccess bau) {
        super(MenuType.CARTOGRAPHY_TABLE, integer);
        this.container = new SimpleContainer(2) {
            @Override
            public void setChanged() {
                CartographyTableMenu.this.slotsChanged(this);
                super.setChanged();
            }
        };
        this.resultContainer = new ResultContainer() {
            @Override
            public void setChanged() {
                CartographyTableMenu.this.slotsChanged(this);
                super.setChanged();
            }
        };
        this.access = bau;
        this.addSlot(new Slot(this.container, 0, 15, 15) {
            @Override
            public boolean mayPlace(final ItemStack bek) {
                return bek.getItem() == Items.FILLED_MAP;
            }
        });
        this.addSlot(new Slot(this.container, 1, 15, 52) {
            @Override
            public boolean mayPlace(final ItemStack bek) {
                final Item bef3 = bek.getItem();
                return bef3 == Items.PAPER || bef3 == Items.MAP || bef3 == Items.GLASS_PANE;
            }
        });
        this.addSlot(new Slot(this.resultContainer, 2, 145, 39) {
            @Override
            public boolean mayPlace(final ItemStack bek) {
                return false;
            }
            
            @Override
            public ItemStack remove(final int integer) {
                final ItemStack bek3 = super.remove(integer);
                ItemStack bek5;
                final ItemStack itemStack;
                final ItemStack bek4 = bau.<ItemStack>evaluate((bjt, fk) -> {
                    if (!CartographyTableMenu.this.quickMoved && CartographyTableMenu.this.container.getItem(1).getItem() == Items.GLASS_PANE) {
                        bek5 = MapItem.lockMap(bjt, CartographyTableMenu.this.container.getItem(0));
                        if (bek5 != null) {
                            bek5.setCount(1);
                            return bek5;
                        }
                    }
                    return itemStack;
                }).orElse(bek3);
                CartographyTableMenu.this.container.removeItem(0, 1);
                CartographyTableMenu.this.container.removeItem(1, 1);
                return bek4;
            }
            
            @Override
            protected void onQuickCraft(final ItemStack bek, final int integer) {
                this.remove(integer);
                super.onQuickCraft(bek, integer);
            }
            
            @Override
            public ItemStack onTake(final Player ayg, final ItemStack bek) {
                bek.getItem().onCraftedBy(bek, ayg.level, ayg);
                final long long4;
                bau.execute((bjt, fk) -> {
                    long4 = bjt.getGameTime();
                    if (CartographyTableMenu.this.lastSoundTime != long4) {
                        bjt.playSound(null, fk, SoundEvents.UI_CARTOGRAPHY_TABLE_TAKE_RESULT, SoundSource.BLOCKS, 1.0f, 1.0f);
                        CartographyTableMenu.this.lastSoundTime = long4;
                    }
                    return;
                });
                return super.onTake(ayg, bek);
            }
        });
        for (int integer2 = 0; integer2 < 3; ++integer2) {
            for (int integer3 = 0; integer3 < 9; ++integer3) {
                this.addSlot(new Slot(ayf, integer3 + integer2 * 9 + 9, 8 + integer3 * 18, 84 + integer2 * 18));
            }
        }
        for (int integer2 = 0; integer2 < 9; ++integer2) {
            this.addSlot(new Slot(ayf, integer2, 8 + integer2 * 18, 142));
        }
    }
    
    @Override
    public boolean stillValid(final Player ayg) {
        return AbstractContainerMenu.stillValid(this.access, ayg, Blocks.CARTOGRAPHY_TABLE);
    }
    
    @Override
    public void slotsChanged(final Container ajb) {
        final ItemStack bek3 = this.container.getItem(0);
        final ItemStack bek4 = this.container.getItem(1);
        final ItemStack bek5 = this.resultContainer.getItem(2);
        if (!bek5.isEmpty() && (bek3.isEmpty() || bek4.isEmpty())) {
            this.resultContainer.removeItemNoUpdate(2);
        }
        else if (!bek3.isEmpty() && !bek4.isEmpty()) {
            this.setupResultSlot(bek3, bek4, bek5);
        }
    }
    
    private void setupResultSlot(final ItemStack bek1, final ItemStack bek2, final ItemStack bek3) {
        final Item bef7;
        final MapItemSavedData crd8;
        ItemStack bek4;
        this.access.execute((bjt, fk) -> {
            bef7 = bek2.getItem();
            crd8 = MapItem.getSavedData(bek1, bjt);
            if (crd8 != null) {
                if (bef7 == Items.PAPER && !crd8.locked && crd8.scale < 4) {
                    bek4 = bek1.copy();
                    bek4.setCount(1);
                    bek4.getOrCreateTag().putInt("map_scale_direction", 1);
                    this.broadcastChanges();
                }
                else if (bef7 == Items.GLASS_PANE && !crd8.locked) {
                    bek4 = bek1.copy();
                    bek4.setCount(1);
                    this.broadcastChanges();
                }
                else if (bef7 == Items.MAP) {
                    bek4 = bek1.copy();
                    bek4.setCount(2);
                    this.broadcastChanges();
                }
                else {
                    this.resultContainer.removeItemNoUpdate(2);
                    this.broadcastChanges();
                    return;
                }
                if (!ItemStack.matches(bek4, bek3)) {
                    this.resultContainer.setItem(2, bek4);
                    this.broadcastChanges();
                }
            }
        });
    }
    
    @Override
    public boolean canTakeItemForPickAll(final ItemStack bek, final Slot bbx) {
        return bbx.container != this.resultContainer && super.canTakeItemForPickAll(bek, bbx);
    }
    
    @Override
    public ItemStack quickMoveStack(final Player ayg, final int integer) {
        ItemStack bek4 = ItemStack.EMPTY;
        final Slot bbx5 = this.slots.get(integer);
        if (bbx5 != null && bbx5.hasItem()) {
            ItemStack bek6;
            final ItemStack bek5 = bek6 = bbx5.getItem();
            final Item bef8 = bek6.getItem();
            bek4 = bek6.copy();
            if (integer == 2) {
                if (this.container.getItem(1).getItem() == Items.GLASS_PANE) {
                    final ItemStack bek7;
                    final ItemStack itemStack;
                    bek6 = this.access.<ItemStack>evaluate((bjt, fk) -> {
                        bek7 = MapItem.lockMap(bjt, this.container.getItem(0));
                        if (bek7 != null) {
                            bek7.setCount(1);
                            return bek7;
                        }
                        else {
                            return itemStack;
                        }
                    }).orElse(bek6);
                }
                bef8.onCraftedBy(bek6, ayg.level, ayg);
                if (!this.moveItemStackTo(bek6, 3, 39, true)) {
                    return ItemStack.EMPTY;
                }
                bbx5.onQuickCraft(bek6, bek4);
            }
            else if (integer == 1 || integer == 0) {
                if (!this.moveItemStackTo(bek6, 3, 39, false)) {
                    return ItemStack.EMPTY;
                }
            }
            else if (bef8 == Items.FILLED_MAP) {
                if (!this.moveItemStackTo(bek6, 0, 1, false)) {
                    return ItemStack.EMPTY;
                }
            }
            else if (bef8 == Items.PAPER || bef8 == Items.MAP || bef8 == Items.GLASS_PANE) {
                if (!this.moveItemStackTo(bek6, 1, 2, false)) {
                    return ItemStack.EMPTY;
                }
            }
            else if (integer >= 3 && integer < 30) {
                if (!this.moveItemStackTo(bek6, 30, 39, false)) {
                    return ItemStack.EMPTY;
                }
            }
            else if (integer >= 30 && integer < 39 && !this.moveItemStackTo(bek6, 3, 30, false)) {
                return ItemStack.EMPTY;
            }
            if (bek6.isEmpty()) {
                bbx5.set(ItemStack.EMPTY);
            }
            bbx5.setChanged();
            if (bek6.getCount() == bek4.getCount()) {
                return ItemStack.EMPTY;
            }
            this.quickMoved = true;
            bbx5.onTake(ayg, bek6);
            this.quickMoved = false;
            this.broadcastChanges();
        }
        return bek4;
    }
    
    @Override
    public void removed(final Player ayg) {
        super.removed(ayg);
        this.resultContainer.removeItemNoUpdate(2);
        this.access.execute((bjt, fk) -> this.clearContainer(ayg, ayg.level, this.container));
    }
}
