package net.minecraft.world.inventory;

import net.minecraft.world.item.ItemStack;
import net.minecraft.world.entity.player.Player;
import net.minecraft.world.SimpleContainer;
import net.minecraft.world.entity.player.Inventory;
import net.minecraft.world.Container;

public class ChestMenu extends AbstractContainerMenu {
    private final Container container;
    private final int containerRows;
    
    private ChestMenu(final MenuType<?> bbl, final int integer2, final Inventory ayf, final int integer4) {
        this(bbl, integer2, ayf, new SimpleContainer(9 * integer4), integer4);
    }
    
    public static ChestMenu oneRow(final int integer, final Inventory ayf) {
        return new ChestMenu(MenuType.GENERIC_9x1, integer, ayf, 1);
    }
    
    public static ChestMenu twoRows(final int integer, final Inventory ayf) {
        return new ChestMenu(MenuType.GENERIC_9x2, integer, ayf, 2);
    }
    
    public static ChestMenu threeRows(final int integer, final Inventory ayf) {
        return new ChestMenu(MenuType.GENERIC_9x3, integer, ayf, 3);
    }
    
    public static ChestMenu fourRows(final int integer, final Inventory ayf) {
        return new ChestMenu(MenuType.GENERIC_9x4, integer, ayf, 4);
    }
    
    public static ChestMenu fiveRows(final int integer, final Inventory ayf) {
        return new ChestMenu(MenuType.GENERIC_9x5, integer, ayf, 5);
    }
    
    public static ChestMenu sixRows(final int integer, final Inventory ayf) {
        return new ChestMenu(MenuType.GENERIC_9x6, integer, ayf, 6);
    }
    
    public static ChestMenu threeRows(final int integer, final Inventory ayf, final Container ajb) {
        return new ChestMenu(MenuType.GENERIC_9x3, integer, ayf, ajb, 3);
    }
    
    public static ChestMenu sixRows(final int integer, final Inventory ayf, final Container ajb) {
        return new ChestMenu(MenuType.GENERIC_9x6, integer, ayf, ajb, 6);
    }
    
    public ChestMenu(final MenuType<?> bbl, final int integer2, final Inventory ayf, final Container ajb, final int integer5) {
        super(bbl, integer2);
        AbstractContainerMenu.checkContainerSize(ajb, integer5 * 9);
        this.container = ajb;
        this.containerRows = integer5;
        ajb.startOpen(ayf.player);
        final int integer6 = (this.containerRows - 4) * 18;
        for (int integer7 = 0; integer7 < this.containerRows; ++integer7) {
            for (int integer8 = 0; integer8 < 9; ++integer8) {
                this.addSlot(new Slot(ajb, integer8 + integer7 * 9, 8 + integer8 * 18, 18 + integer7 * 18));
            }
        }
        for (int integer7 = 0; integer7 < 3; ++integer7) {
            for (int integer8 = 0; integer8 < 9; ++integer8) {
                this.addSlot(new Slot(ayf, integer8 + integer7 * 9 + 9, 8 + integer8 * 18, 103 + integer7 * 18 + integer6));
            }
        }
        for (int integer7 = 0; integer7 < 9; ++integer7) {
            this.addSlot(new Slot(ayf, integer7, 8 + integer7 * 18, 161 + integer6));
        }
    }
    
    @Override
    public boolean stillValid(final Player ayg) {
        return this.container.stillValid(ayg);
    }
    
    @Override
    public ItemStack quickMoveStack(final Player ayg, final int integer) {
        ItemStack bek4 = ItemStack.EMPTY;
        final Slot bbx5 = this.slots.get(integer);
        if (bbx5 != null && bbx5.hasItem()) {
            final ItemStack bek5 = bbx5.getItem();
            bek4 = bek5.copy();
            if (integer < this.containerRows * 9) {
                if (!this.moveItemStackTo(bek5, this.containerRows * 9, this.slots.size(), true)) {
                    return ItemStack.EMPTY;
                }
            }
            else if (!this.moveItemStackTo(bek5, 0, this.containerRows * 9, false)) {
                return ItemStack.EMPTY;
            }
            if (bek5.isEmpty()) {
                bbx5.set(ItemStack.EMPTY);
            }
            else {
                bbx5.setChanged();
            }
        }
        return bek4;
    }
    
    @Override
    public void removed(final Player ayg) {
        super.removed(ayg);
        this.container.stopOpen(ayg);
    }
    
    public Container getContainer() {
        return this.container;
    }
    
    public int getRowCount() {
        return this.containerRows;
    }
}
