package net.minecraft.world.inventory;

import net.minecraft.core.BlockPos;
import net.minecraft.world.level.block.Blocks;
import net.minecraft.world.entity.player.StackedContents;
import java.util.Optional;
import net.minecraft.network.protocol.Packet;
import net.minecraft.network.protocol.game.ClientboundContainerSetSlotPacket;
import net.minecraft.world.item.crafting.Recipe;
import net.minecraft.world.item.crafting.CraftingRecipe;
import net.minecraft.world.item.crafting.RecipeType;
import net.minecraft.world.item.ItemStack;
import net.minecraft.server.level.ServerPlayer;
import net.minecraft.world.level.Level;
import net.minecraft.world.Container;
import net.minecraft.world.entity.player.Inventory;
import net.minecraft.world.entity.player.Player;

public class CraftingMenu extends RecipeBookMenu<CraftingContainer> {
    private final CraftingContainer craftSlots;
    private final ResultContainer resultSlots;
    private final ContainerLevelAccess access;
    private final Player player;
    
    public CraftingMenu(final int integer, final Inventory ayf) {
        this(integer, ayf, ContainerLevelAccess.NULL);
    }
    
    public CraftingMenu(final int integer, final Inventory ayf, final ContainerLevelAccess bau) {
        super(MenuType.CRAFTING, integer);
        this.craftSlots = new CraftingContainer(this, 3, 3);
        this.resultSlots = new ResultContainer();
        this.access = bau;
        this.player = ayf.player;
        this.addSlot(new ResultSlot(ayf.player, this.craftSlots, this.resultSlots, 0, 124, 35));
        for (int integer2 = 0; integer2 < 3; ++integer2) {
            for (int integer3 = 0; integer3 < 3; ++integer3) {
                this.addSlot(new Slot(this.craftSlots, integer3 + integer2 * 3, 30 + integer3 * 18, 17 + integer2 * 18));
            }
        }
        for (int integer2 = 0; integer2 < 3; ++integer2) {
            for (int integer3 = 0; integer3 < 9; ++integer3) {
                this.addSlot(new Slot(ayf, integer3 + integer2 * 9 + 9, 8 + integer3 * 18, 84 + integer2 * 18));
            }
        }
        for (int integer2 = 0; integer2 < 9; ++integer2) {
            this.addSlot(new Slot(ayf, integer2, 8 + integer2 * 18, 142));
        }
    }
    
    protected static void slotChangedCraftingGrid(final int integer, final Level bjt, final Player ayg, final CraftingContainer baw, final ResultContainer bbs) {
        if (bjt.isClientSide) {
            return;
        }
        final ServerPlayer xe6 = (ServerPlayer)ayg;
        ItemStack bek7 = ItemStack.EMPTY;
        final Optional<CraftingRecipe> optional8 = bjt.getServer().getRecipeManager().<CraftingContainer, CraftingRecipe>getRecipeFor(RecipeType.CRAFTING, baw, bjt);
        if (optional8.isPresent()) {
            final CraftingRecipe bgl9 = optional8.get();
            if (bbs.setRecipeUsed(bjt, xe6, bgl9)) {
                bek7 = bgl9.assemble(baw);
            }
        }
        bbs.setItem(0, bek7);
        xe6.connection.send(new ClientboundContainerSetSlotPacket(integer, 0, bek7));
    }
    
    @Override
    public void slotsChanged(final Container ajb) {
        this.access.execute((bjt, fk) -> slotChangedCraftingGrid(this.containerId, bjt, this.player, this.craftSlots, this.resultSlots));
    }
    
    @Override
    public void fillCraftSlotsStackedContents(final StackedContents ayi) {
        this.craftSlots.fillStackedContents(ayi);
    }
    
    @Override
    public void clearCraftingContent() {
        this.craftSlots.clearContent();
        this.resultSlots.clearContent();
    }
    
    @Override
    public boolean recipeMatches(final Recipe<? super CraftingContainer> bgt) {
        return bgt.matches(this.craftSlots, this.player.level);
    }
    
    @Override
    public void removed(final Player ayg) {
        super.removed(ayg);
        this.access.execute((bjt, fk) -> this.clearContainer(ayg, bjt, this.craftSlots));
    }
    
    @Override
    public boolean stillValid(final Player ayg) {
        return AbstractContainerMenu.stillValid(this.access, ayg, Blocks.CRAFTING_TABLE);
    }
    
    @Override
    public ItemStack quickMoveStack(final Player ayg, final int integer) {
        ItemStack bek4 = ItemStack.EMPTY;
        final Slot bbx5 = this.slots.get(integer);
        if (bbx5 != null && bbx5.hasItem()) {
            final ItemStack bek5 = bbx5.getItem();
            bek4 = bek5.copy();
            if (integer == 0) {
                final ItemStack bek7;
                this.access.execute((bjt, fk) -> bek7.getItem().onCraftedBy(bek7, bjt, ayg));
                if (!this.moveItemStackTo(bek5, 10, 46, true)) {
                    return ItemStack.EMPTY;
                }
                bbx5.onQuickCraft(bek5, bek4);
            }
            else if (integer >= 10 && integer < 46) {
                if (!this.moveItemStackTo(bek5, 1, 10, false)) {
                    if (integer < 37) {
                        if (!this.moveItemStackTo(bek5, 37, 46, false)) {
                            return ItemStack.EMPTY;
                        }
                    }
                    else if (!this.moveItemStackTo(bek5, 10, 37, false)) {
                        return ItemStack.EMPTY;
                    }
                }
            }
            else if (!this.moveItemStackTo(bek5, 10, 46, false)) {
                return ItemStack.EMPTY;
            }
            if (bek5.isEmpty()) {
                bbx5.set(ItemStack.EMPTY);
            }
            else {
                bbx5.setChanged();
            }
            if (bek5.getCount() == bek4.getCount()) {
                return ItemStack.EMPTY;
            }
            final ItemStack bek6 = bbx5.onTake(ayg, bek5);
            if (integer == 0) {
                ayg.drop(bek6, false);
            }
        }
        return bek4;
    }
    
    @Override
    public boolean canTakeItemForPickAll(final ItemStack bek, final Slot bbx) {
        return bbx.container != this.resultSlots && super.canTakeItemForPickAll(bek, bbx);
    }
    
    @Override
    public int getResultSlotIndex() {
        return 0;
    }
    
    @Override
    public int getGridWidth() {
        return this.craftSlots.getWidth();
    }
    
    @Override
    public int getGridHeight() {
        return this.craftSlots.getHeight();
    }
    
    @Override
    public int getSize() {
        return 10;
    }
}
