package net.minecraft.world.inventory;

import net.minecraft.world.level.block.Blocks;
import net.minecraft.world.level.ItemLike;
import java.util.function.Function;
import java.util.stream.Collectors;
import net.minecraft.world.item.Item;
import net.minecraft.core.BlockPos;
import java.util.Iterator;
import net.minecraft.world.item.enchantment.Enchantment;
import java.util.Map;
import net.minecraft.world.item.enchantment.EnchantmentHelper;
import net.minecraft.world.level.Level;
import net.minecraft.world.entity.Entity;
import net.minecraft.world.entity.ExperienceOrb;
import net.minecraft.world.entity.player.Player;
import net.minecraft.world.item.Items;
import net.minecraft.world.item.ItemStack;
import net.minecraft.world.SimpleContainer;
import net.minecraft.world.entity.player.Inventory;
import net.minecraft.world.Container;

public class GrindstoneMenu extends AbstractContainerMenu {
    private final Container resultSlots;
    private final Container repairSlots;
    private final ContainerLevelAccess access;
    
    public GrindstoneMenu(final int integer, final Inventory ayf) {
        this(integer, ayf, ContainerLevelAccess.NULL);
    }
    
    public GrindstoneMenu(final int integer, final Inventory ayf, final ContainerLevelAccess bau) {
        super(MenuType.GRINDSTONE, integer);
        this.resultSlots = new ResultContainer();
        this.repairSlots = new SimpleContainer(2) {
            @Override
            public void setChanged() {
                super.setChanged();
                GrindstoneMenu.this.slotsChanged(this);
            }
        };
        this.access = bau;
        this.addSlot(new Slot(this.repairSlots, 0, 49, 19) {
            @Override
            public boolean mayPlace(final ItemStack bek) {
                return bek.isDamageableItem() || bek.getItem() == Items.ENCHANTED_BOOK || bek.isEnchanted();
            }
        });
        this.addSlot(new Slot(this.repairSlots, 1, 49, 40) {
            @Override
            public boolean mayPlace(final ItemStack bek) {
                return bek.isDamageableItem() || bek.getItem() == Items.ENCHANTED_BOOK || bek.isEnchanted();
            }
        });
        this.addSlot(new Slot(this.resultSlots, 2, 129, 34) {
            @Override
            public boolean mayPlace(final ItemStack bek) {
                return false;
            }
            
            @Override
            public ItemStack onTake(final Player ayg, final ItemStack bek) {
                int integer4;
                int integer5;
                bau.execute((bjt, fk) -> {
                    integer4 = this.getExperienceAmount(bjt);
                    while (integer4 > 0) {
                        integer5 = ExperienceOrb.getExperienceValue(integer4);
                        integer4 -= integer5;
                        bjt.addFreshEntity(new ExperienceOrb(bjt, fk.getX(), fk.getY() + 0.5, fk.getZ() + 0.5, integer5));
                    }
                    bjt.levelEvent(1042, fk, 0);
                    return;
                });
                GrindstoneMenu.this.repairSlots.setItem(0, ItemStack.EMPTY);
                GrindstoneMenu.this.repairSlots.setItem(1, ItemStack.EMPTY);
                return bek;
            }
            
            private int getExperienceAmount(final Level bjt) {
                int integer3 = 0;
                integer3 += this.getExperienceFromItem(GrindstoneMenu.this.repairSlots.getItem(0));
                integer3 += this.getExperienceFromItem(GrindstoneMenu.this.repairSlots.getItem(1));
                if (integer3 > 0) {
                    final int integer4 = (int)Math.ceil(integer3 / 2.0);
                    return integer4 + bjt.random.nextInt(integer4);
                }
                return 0;
            }
            
            private int getExperienceFromItem(final ItemStack bek) {
                int integer3 = 0;
                final Map<Enchantment, Integer> map4 = EnchantmentHelper.getEnchantments(bek);
                for (final Map.Entry<Enchantment, Integer> entry6 : map4.entrySet()) {
                    final Enchantment bhu7 = entry6.getKey();
                    final Integer integer4 = entry6.getValue();
                    if (!bhu7.isCurse()) {
                        integer3 += bhu7.getMinCost(integer4);
                    }
                }
                return integer3;
            }
        });
        for (int integer2 = 0; integer2 < 3; ++integer2) {
            for (int integer3 = 0; integer3 < 9; ++integer3) {
                this.addSlot(new Slot(ayf, integer3 + integer2 * 9 + 9, 8 + integer3 * 18, 84 + integer2 * 18));
            }
        }
        for (int integer2 = 0; integer2 < 9; ++integer2) {
            this.addSlot(new Slot(ayf, integer2, 8 + integer2 * 18, 142));
        }
    }
    
    @Override
    public void slotsChanged(final Container ajb) {
        super.slotsChanged(ajb);
        if (ajb == this.repairSlots) {
            this.createResult();
        }
    }
    
    private void createResult() {
        final ItemStack bek2 = this.repairSlots.getItem(0);
        final ItemStack bek3 = this.repairSlots.getItem(1);
        final boolean boolean4 = !bek2.isEmpty() || !bek3.isEmpty();
        final boolean boolean5 = !bek2.isEmpty() && !bek3.isEmpty();
        if (boolean4) {
            final boolean boolean6 = (!bek2.isEmpty() && bek2.getItem() != Items.ENCHANTED_BOOK && !bek2.isEnchanted()) || (!bek3.isEmpty() && bek3.getItem() != Items.ENCHANTED_BOOK && !bek3.isEnchanted());
            if (bek2.getCount() > 1 || bek3.getCount() > 1 || (!boolean5 && boolean6)) {
                this.resultSlots.setItem(0, ItemStack.EMPTY);
                this.broadcastChanges();
                return;
            }
            int integer8 = 1;
            int integer12;
            ItemStack bek4;
            if (boolean5) {
                if (bek2.getItem() != bek3.getItem()) {
                    this.resultSlots.setItem(0, ItemStack.EMPTY);
                    this.broadcastChanges();
                    return;
                }
                final Item bef10 = bek2.getItem();
                final int integer9 = bef10.getMaxDamage() - bek2.getDamageValue();
                final int integer10 = bef10.getMaxDamage() - bek3.getDamageValue();
                final int integer11 = integer9 + integer10 + bef10.getMaxDamage() * 5 / 100;
                integer12 = Math.max(bef10.getMaxDamage() - integer11, 0);
                bek4 = this.mergeEnchants(bek2, bek3);
                if (!bek4.isDamageableItem()) {
                    if (!ItemStack.matches(bek2, bek3)) {
                        this.resultSlots.setItem(0, ItemStack.EMPTY);
                        this.broadcastChanges();
                        return;
                    }
                    integer8 = 2;
                }
            }
            else {
                final boolean boolean7 = !bek2.isEmpty();
                integer12 = (boolean7 ? bek2.getDamageValue() : bek3.getDamageValue());
                bek4 = (boolean7 ? bek2 : bek3);
            }
            this.resultSlots.setItem(0, this.removeNonCurses(bek4, integer12, integer8));
        }
        else {
            this.resultSlots.setItem(0, ItemStack.EMPTY);
        }
        this.broadcastChanges();
    }
    
    private ItemStack mergeEnchants(final ItemStack bek1, final ItemStack bek2) {
        final ItemStack bek3 = bek1.copy();
        final Map<Enchantment, Integer> map5 = EnchantmentHelper.getEnchantments(bek2);
        for (final Map.Entry<Enchantment, Integer> entry7 : map5.entrySet()) {
            final Enchantment bhu8 = entry7.getKey();
            if (!bhu8.isCurse() || EnchantmentHelper.getItemEnchantmentLevel(bhu8, bek3) == 0) {
                bek3.enchant(bhu8, entry7.getValue());
            }
        }
        return bek3;
    }
    
    private ItemStack removeNonCurses(final ItemStack bek, final int integer2, final int integer3) {
        ItemStack bek2 = bek.copy();
        bek2.removeTagKey("Enchantments");
        bek2.removeTagKey("StoredEnchantments");
        if (integer2 > 0) {
            bek2.setDamageValue(integer2);
        }
        else {
            bek2.removeTagKey("Damage");
        }
        bek2.setCount(integer3);
        final Map<Enchantment, Integer> map6 = EnchantmentHelper.getEnchantments(bek).entrySet().stream().filter(entry -> entry.getKey().isCurse()).collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
        EnchantmentHelper.setEnchantments(map6, bek2);
        bek2.setRepairCost(0);
        if (bek2.getItem() == Items.ENCHANTED_BOOK && map6.size() == 0) {
            bek2 = new ItemStack(Items.BOOK);
            if (bek.hasCustomHoverName()) {
                bek2.setHoverName(bek.getHoverName());
            }
        }
        for (int integer4 = 0; integer4 < map6.size(); ++integer4) {
            bek2.setRepairCost(AnvilMenu.calculateIncreasedRepairCost(bek2.getBaseRepairCost()));
        }
        return bek2;
    }
    
    @Override
    public void removed(final Player ayg) {
        super.removed(ayg);
        this.access.execute((bjt, fk) -> this.clearContainer(ayg, bjt, this.repairSlots));
    }
    
    @Override
    public boolean stillValid(final Player ayg) {
        return AbstractContainerMenu.stillValid(this.access, ayg, Blocks.GRINDSTONE);
    }
    
    @Override
    public ItemStack quickMoveStack(final Player ayg, final int integer) {
        ItemStack bek4 = ItemStack.EMPTY;
        final Slot bbx5 = this.slots.get(integer);
        if (bbx5 != null && bbx5.hasItem()) {
            final ItemStack bek5 = bbx5.getItem();
            bek4 = bek5.copy();
            final ItemStack bek6 = this.repairSlots.getItem(0);
            final ItemStack bek7 = this.repairSlots.getItem(1);
            if (integer == 2) {
                if (!this.moveItemStackTo(bek5, 3, 39, true)) {
                    return ItemStack.EMPTY;
                }
                bbx5.onQuickCraft(bek5, bek4);
            }
            else if (integer == 0 || integer == 1) {
                if (!this.moveItemStackTo(bek5, 3, 39, false)) {
                    return ItemStack.EMPTY;
                }
            }
            else if (bek6.isEmpty() || bek7.isEmpty()) {
                if (!this.moveItemStackTo(bek5, 0, 2, false)) {
                    return ItemStack.EMPTY;
                }
            }
            else if (integer >= 3 && integer < 30) {
                if (!this.moveItemStackTo(bek5, 30, 39, false)) {
                    return ItemStack.EMPTY;
                }
            }
            else if (integer >= 30 && integer < 39 && !this.moveItemStackTo(bek5, 3, 30, false)) {
                return ItemStack.EMPTY;
            }
            if (bek5.isEmpty()) {
                bbx5.set(ItemStack.EMPTY);
            }
            else {
                bbx5.setChanged();
            }
            if (bek5.getCount() == bek4.getCount()) {
                return ItemStack.EMPTY;
            }
            bbx5.onTake(ayg, bek5);
        }
        return bek4;
    }
}
