package net.minecraft.world.inventory;

import net.minecraft.world.entity.Entity;
import net.minecraft.world.entity.player.Player;
import net.minecraft.world.entity.animal.horse.AbstractChestedHorse;
import net.minecraft.world.item.Items;
import net.minecraft.world.item.ItemStack;
import net.minecraft.world.entity.player.Inventory;
import net.minecraft.world.entity.animal.horse.AbstractHorse;
import net.minecraft.world.Container;

public class HorseInventoryMenu extends AbstractContainerMenu {
    private final Container horseContainer;
    private final AbstractHorse horse;
    
    public HorseInventoryMenu(final int integer, final Inventory ayf, final Container ajb, final AbstractHorse aub) {
        super(null, integer);
        this.horseContainer = ajb;
        this.horse = aub;
        final int integer2 = 3;
        ajb.startOpen(ayf.player);
        final int integer3 = -18;
        this.addSlot(new Slot(ajb, 0, 8, 18) {
            @Override
            public boolean mayPlace(final ItemStack bek) {
                return bek.getItem() == Items.SADDLE && !this.hasItem() && aub.canBeSaddled();
            }
            
            @Override
            public boolean isActive() {
                return aub.canBeSaddled();
            }
        });
        this.addSlot(new Slot(ajb, 1, 8, 36) {
            @Override
            public boolean mayPlace(final ItemStack bek) {
                return aub.isArmor(bek);
            }
            
            @Override
            public boolean isActive() {
                return aub.wearsArmor();
            }
            
            @Override
            public int getMaxStackSize() {
                return 1;
            }
        });
        if (aub instanceof AbstractChestedHorse && ((AbstractChestedHorse)aub).hasChest()) {
            for (int integer4 = 0; integer4 < 3; ++integer4) {
                for (int integer5 = 0; integer5 < ((AbstractChestedHorse)aub).getInventoryColumns(); ++integer5) {
                    this.addSlot(new Slot(ajb, 2 + integer5 + integer4 * ((AbstractChestedHorse)aub).getInventoryColumns(), 80 + integer5 * 18, 18 + integer4 * 18));
                }
            }
        }
        for (int integer4 = 0; integer4 < 3; ++integer4) {
            for (int integer5 = 0; integer5 < 9; ++integer5) {
                this.addSlot(new Slot(ayf, integer5 + integer4 * 9 + 9, 8 + integer5 * 18, 102 + integer4 * 18 - 18));
            }
        }
        for (int integer4 = 0; integer4 < 9; ++integer4) {
            this.addSlot(new Slot(ayf, integer4, 8 + integer4 * 18, 142));
        }
    }
    
    @Override
    public boolean stillValid(final Player ayg) {
        return this.horseContainer.stillValid(ayg) && this.horse.isAlive() && this.horse.distanceTo(ayg) < 8.0f;
    }
    
    @Override
    public ItemStack quickMoveStack(final Player ayg, final int integer) {
        ItemStack bek4 = ItemStack.EMPTY;
        final Slot bbx5 = this.slots.get(integer);
        if (bbx5 != null && bbx5.hasItem()) {
            final ItemStack bek5 = bbx5.getItem();
            bek4 = bek5.copy();
            final int integer2 = this.horseContainer.getContainerSize();
            if (integer < integer2) {
                if (!this.moveItemStackTo(bek5, integer2, this.slots.size(), true)) {
                    return ItemStack.EMPTY;
                }
            }
            else if (this.getSlot(1).mayPlace(bek5) && !this.getSlot(1).hasItem()) {
                if (!this.moveItemStackTo(bek5, 1, 2, false)) {
                    return ItemStack.EMPTY;
                }
            }
            else if (this.getSlot(0).mayPlace(bek5)) {
                if (!this.moveItemStackTo(bek5, 0, 1, false)) {
                    return ItemStack.EMPTY;
                }
            }
            else if (integer2 <= 2 || !this.moveItemStackTo(bek5, 2, integer2, false)) {
                final int integer3 = integer2;
                final int integer5;
                final int integer4 = integer5 = integer3 + 27;
                final int integer6 = integer5 + 9;
                if (integer >= integer5 && integer < integer6) {
                    if (!this.moveItemStackTo(bek5, integer3, integer4, false)) {
                        return ItemStack.EMPTY;
                    }
                }
                else if (integer >= integer3 && integer < integer4) {
                    if (!this.moveItemStackTo(bek5, integer5, integer6, false)) {
                        return ItemStack.EMPTY;
                    }
                }
                else if (!this.moveItemStackTo(bek5, integer5, integer4, false)) {
                    return ItemStack.EMPTY;
                }
                return ItemStack.EMPTY;
            }
            if (bek5.isEmpty()) {
                bbx5.set(ItemStack.EMPTY);
            }
            else {
                bbx5.setChanged();
            }
        }
        return bek4;
    }
    
    @Override
    public void removed(final Player ayg) {
        super.removed(ayg);
        this.horseContainer.stopOpen(ayg);
    }
}
