package net.minecraft.world.inventory;

import net.minecraft.world.item.crafting.Recipe;
import net.minecraft.world.entity.player.StackedContents;
import com.mojang.datafixers.util.Pair;
import net.minecraft.world.item.enchantment.EnchantmentHelper;
import net.minecraft.world.entity.Mob;
import net.minecraft.world.item.ItemStack;
import net.minecraft.world.Container;
import net.minecraft.world.entity.player.Inventory;
import net.minecraft.world.entity.player.Player;
import net.minecraft.world.entity.EquipmentSlot;
import net.minecraft.resources.ResourceLocation;

public class InventoryMenu extends RecipeBookMenu<CraftingContainer> {
    public static final ResourceLocation BLOCK_ATLAS;
    public static final ResourceLocation EMPTY_ARMOR_SLOT_HELMET;
    public static final ResourceLocation EMPTY_ARMOR_SLOT_CHESTPLATE;
    public static final ResourceLocation EMPTY_ARMOR_SLOT_LEGGINGS;
    public static final ResourceLocation EMPTY_ARMOR_SLOT_BOOTS;
    public static final ResourceLocation EMPTY_ARMOR_SLOT_SHIELD;
    private static final ResourceLocation[] TEXTURE_EMPTY_SLOTS;
    private static final EquipmentSlot[] SLOT_IDS;
    private final CraftingContainer craftSlots;
    private final ResultContainer resultSlots;
    public final boolean active;
    private final Player owner;
    
    public InventoryMenu(final Inventory ayf, final boolean boolean2, final Player ayg) {
        super(null, 0);
        this.craftSlots = new CraftingContainer(this, 2, 2);
        this.resultSlots = new ResultContainer();
        this.active = boolean2;
        this.owner = ayg;
        this.addSlot(new ResultSlot(ayf.player, this.craftSlots, this.resultSlots, 0, 154, 28));
        for (int integer5 = 0; integer5 < 2; ++integer5) {
            for (int integer6 = 0; integer6 < 2; ++integer6) {
                this.addSlot(new Slot(this.craftSlots, integer6 + integer5 * 2, 98 + integer6 * 18, 18 + integer5 * 18));
            }
        }
        for (int integer5 = 0; integer5 < 4; ++integer5) {
            final EquipmentSlot aks6 = InventoryMenu.SLOT_IDS[integer5];
            this.addSlot(new Slot(ayf, 39 - integer5, 8, 8 + integer5 * 18) {
                @Override
                public int getMaxStackSize() {
                    return 1;
                }
                
                @Override
                public boolean mayPlace(final ItemStack bek) {
                    return aks6 == Mob.getEquipmentSlotForItem(bek);
                }
                
                @Override
                public boolean mayPickup(final Player ayg) {
                    final ItemStack bek3 = this.getItem();
                    return (bek3.isEmpty() || ayg.isCreative() || !EnchantmentHelper.hasBindingCurse(bek3)) && super.mayPickup(ayg);
                }
                
                @Override
                public Pair<ResourceLocation, ResourceLocation> getNoItemIcon() {
                    return (Pair<ResourceLocation, ResourceLocation>)Pair.of(InventoryMenu.BLOCK_ATLAS, InventoryMenu.TEXTURE_EMPTY_SLOTS[aks6.getIndex()]);
                }
            });
        }
        for (int integer5 = 0; integer5 < 3; ++integer5) {
            for (int integer6 = 0; integer6 < 9; ++integer6) {
                this.addSlot(new Slot(ayf, integer6 + (integer5 + 1) * 9, 8 + integer6 * 18, 84 + integer5 * 18));
            }
        }
        for (int integer5 = 0; integer5 < 9; ++integer5) {
            this.addSlot(new Slot(ayf, integer5, 8 + integer5 * 18, 142));
        }
        this.addSlot(new Slot(ayf, 40, 77, 62) {
            @Override
            public Pair<ResourceLocation, ResourceLocation> getNoItemIcon() {
                return (Pair<ResourceLocation, ResourceLocation>)Pair.of(InventoryMenu.BLOCK_ATLAS, InventoryMenu.EMPTY_ARMOR_SLOT_SHIELD);
            }
        });
    }
    
    @Override
    public void fillCraftSlotsStackedContents(final StackedContents ayi) {
        this.craftSlots.fillStackedContents(ayi);
    }
    
    @Override
    public void clearCraftingContent() {
        this.resultSlots.clearContent();
        this.craftSlots.clearContent();
    }
    
    @Override
    public boolean recipeMatches(final Recipe<? super CraftingContainer> bgt) {
        return bgt.matches(this.craftSlots, this.owner.level);
    }
    
    @Override
    public void slotsChanged(final Container ajb) {
        CraftingMenu.slotChangedCraftingGrid(this.containerId, this.owner.level, this.owner, this.craftSlots, this.resultSlots);
    }
    
    @Override
    public void removed(final Player ayg) {
        super.removed(ayg);
        this.resultSlots.clearContent();
        if (ayg.level.isClientSide) {
            return;
        }
        this.clearContainer(ayg, ayg.level, this.craftSlots);
    }
    
    @Override
    public boolean stillValid(final Player ayg) {
        return true;
    }
    
    @Override
    public ItemStack quickMoveStack(final Player ayg, final int integer) {
        ItemStack bek4 = ItemStack.EMPTY;
        final Slot bbx5 = this.slots.get(integer);
        if (bbx5 != null && bbx5.hasItem()) {
            final ItemStack bek5 = bbx5.getItem();
            bek4 = bek5.copy();
            final EquipmentSlot aks7 = Mob.getEquipmentSlotForItem(bek4);
            if (integer == 0) {
                if (!this.moveItemStackTo(bek5, 9, 45, true)) {
                    return ItemStack.EMPTY;
                }
                bbx5.onQuickCraft(bek5, bek4);
            }
            else if (integer >= 1 && integer < 5) {
                if (!this.moveItemStackTo(bek5, 9, 45, false)) {
                    return ItemStack.EMPTY;
                }
            }
            else if (integer >= 5 && integer < 9) {
                if (!this.moveItemStackTo(bek5, 9, 45, false)) {
                    return ItemStack.EMPTY;
                }
            }
            else if (aks7.getType() == EquipmentSlot.Type.ARMOR && !this.slots.get(8 - aks7.getIndex()).hasItem()) {
                final int integer2 = 8 - aks7.getIndex();
                if (!this.moveItemStackTo(bek5, integer2, integer2 + 1, false)) {
                    return ItemStack.EMPTY;
                }
            }
            else if (aks7 == EquipmentSlot.OFFHAND && !this.slots.get(45).hasItem()) {
                if (!this.moveItemStackTo(bek5, 45, 46, false)) {
                    return ItemStack.EMPTY;
                }
            }
            else if (integer >= 9 && integer < 36) {
                if (!this.moveItemStackTo(bek5, 36, 45, false)) {
                    return ItemStack.EMPTY;
                }
            }
            else if (integer >= 36 && integer < 45) {
                if (!this.moveItemStackTo(bek5, 9, 36, false)) {
                    return ItemStack.EMPTY;
                }
            }
            else if (!this.moveItemStackTo(bek5, 9, 45, false)) {
                return ItemStack.EMPTY;
            }
            if (bek5.isEmpty()) {
                bbx5.set(ItemStack.EMPTY);
            }
            else {
                bbx5.setChanged();
            }
            if (bek5.getCount() == bek4.getCount()) {
                return ItemStack.EMPTY;
            }
            final ItemStack bek6 = bbx5.onTake(ayg, bek5);
            if (integer == 0) {
                ayg.drop(bek6, false);
            }
        }
        return bek4;
    }
    
    @Override
    public boolean canTakeItemForPickAll(final ItemStack bek, final Slot bbx) {
        return bbx.container != this.resultSlots && super.canTakeItemForPickAll(bek, bbx);
    }
    
    @Override
    public int getResultSlotIndex() {
        return 0;
    }
    
    @Override
    public int getGridWidth() {
        return this.craftSlots.getWidth();
    }
    
    @Override
    public int getGridHeight() {
        return this.craftSlots.getHeight();
    }
    
    @Override
    public int getSize() {
        return 5;
    }
    
    static {
        BLOCK_ATLAS = new ResourceLocation("textures/atlas/blocks.png");
        EMPTY_ARMOR_SLOT_HELMET = new ResourceLocation("item/empty_armor_slot_helmet");
        EMPTY_ARMOR_SLOT_CHESTPLATE = new ResourceLocation("item/empty_armor_slot_chestplate");
        EMPTY_ARMOR_SLOT_LEGGINGS = new ResourceLocation("item/empty_armor_slot_leggings");
        EMPTY_ARMOR_SLOT_BOOTS = new ResourceLocation("item/empty_armor_slot_boots");
        EMPTY_ARMOR_SLOT_SHIELD = new ResourceLocation("item/empty_armor_slot_shield");
        TEXTURE_EMPTY_SLOTS = new ResourceLocation[] { InventoryMenu.EMPTY_ARMOR_SLOT_BOOTS, InventoryMenu.EMPTY_ARMOR_SLOT_LEGGINGS, InventoryMenu.EMPTY_ARMOR_SLOT_CHESTPLATE, InventoryMenu.EMPTY_ARMOR_SLOT_HELMET };
        SLOT_IDS = new EquipmentSlot[] { EquipmentSlot.HEAD, EquipmentSlot.CHEST, EquipmentSlot.LEGS, EquipmentSlot.FEET };
    }
}
