package net.minecraft.world.inventory;

import java.util.AbstractList;
import net.minecraft.world.item.DyeColor;
import net.minecraft.nbt.Tag;
import net.minecraft.nbt.ListTag;
import net.minecraft.nbt.CompoundTag;
import net.minecraft.world.level.block.entity.BannerPattern;
import net.minecraft.world.level.block.Blocks;
import net.minecraft.core.BlockPos;
import net.minecraft.world.level.Level;
import net.minecraft.sounds.SoundSource;
import net.minecraft.sounds.SoundEvents;
import net.minecraft.world.entity.player.Player;
import net.minecraft.world.item.BannerPatternItem;
import net.minecraft.world.item.DyeItem;
import net.minecraft.world.item.BannerItem;
import net.minecraft.world.item.ItemStack;
import net.minecraft.world.SimpleContainer;
import net.minecraft.world.entity.player.Inventory;
import net.minecraft.world.Container;

public class LoomMenu extends AbstractContainerMenu {
    private final ContainerLevelAccess access;
    private final DataSlot selectedBannerPatternIndex;
    private Runnable slotUpdateListener;
    private final Slot bannerSlot;
    private final Slot dyeSlot;
    private final Slot patternSlot;
    private final Slot resultSlot;
    private long lastSoundTime;
    private final Container inputContainer;
    private final Container outputContainer;
    
    public LoomMenu(final int integer, final Inventory ayf) {
        this(integer, ayf, ContainerLevelAccess.NULL);
    }
    
    public LoomMenu(final int integer, final Inventory ayf, final ContainerLevelAccess bau) {
        super(MenuType.LOOM, integer);
        this.selectedBannerPatternIndex = DataSlot.standalone();
        this.slotUpdateListener = (() -> {});
        this.inputContainer = new SimpleContainer(3) {
            @Override
            public void setChanged() {
                super.setChanged();
                LoomMenu.this.slotsChanged(this);
                LoomMenu.this.slotUpdateListener.run();
            }
        };
        this.outputContainer = new SimpleContainer(1) {
            @Override
            public void setChanged() {
                super.setChanged();
                LoomMenu.this.slotUpdateListener.run();
            }
        };
        this.access = bau;
        this.bannerSlot = this.addSlot(new Slot(this.inputContainer, 0, 13, 26) {
            @Override
            public boolean mayPlace(final ItemStack bek) {
                return bek.getItem() instanceof BannerItem;
            }
        });
        this.dyeSlot = this.addSlot(new Slot(this.inputContainer, 1, 33, 26) {
            @Override
            public boolean mayPlace(final ItemStack bek) {
                return bek.getItem() instanceof DyeItem;
            }
        });
        this.patternSlot = this.addSlot(new Slot(this.inputContainer, 2, 23, 45) {
            @Override
            public boolean mayPlace(final ItemStack bek) {
                return bek.getItem() instanceof BannerPatternItem;
            }
        });
        this.resultSlot = this.addSlot(new Slot(this.outputContainer, 0, 143, 58) {
            @Override
            public boolean mayPlace(final ItemStack bek) {
                return false;
            }
            
            @Override
            public ItemStack onTake(final Player ayg, final ItemStack bek) {
                LoomMenu.this.bannerSlot.remove(1);
                LoomMenu.this.dyeSlot.remove(1);
                if (!LoomMenu.this.bannerSlot.hasItem() || !LoomMenu.this.dyeSlot.hasItem()) {
                    LoomMenu.this.selectedBannerPatternIndex.set(0);
                }
                final long long4;
                bau.execute((bjt, fk) -> {
                    long4 = bjt.getGameTime();
                    if (LoomMenu.this.lastSoundTime != long4) {
                        bjt.playSound(null, fk, SoundEvents.UI_LOOM_TAKE_RESULT, SoundSource.BLOCKS, 1.0f, 1.0f);
                        LoomMenu.this.lastSoundTime = long4;
                    }
                    return;
                });
                return super.onTake(ayg, bek);
            }
        });
        for (int integer2 = 0; integer2 < 3; ++integer2) {
            for (int integer3 = 0; integer3 < 9; ++integer3) {
                this.addSlot(new Slot(ayf, integer3 + integer2 * 9 + 9, 8 + integer3 * 18, 84 + integer2 * 18));
            }
        }
        for (int integer2 = 0; integer2 < 9; ++integer2) {
            this.addSlot(new Slot(ayf, integer2, 8 + integer2 * 18, 142));
        }
        this.addDataSlot(this.selectedBannerPatternIndex);
    }
    
    public int getSelectedBannerPatternIndex() {
        return this.selectedBannerPatternIndex.get();
    }
    
    @Override
    public boolean stillValid(final Player ayg) {
        return AbstractContainerMenu.stillValid(this.access, ayg, Blocks.LOOM);
    }
    
    @Override
    public boolean clickMenuButton(final Player ayg, final int integer) {
        if (integer > 0 && integer <= BannerPattern.AVAILABLE_PATTERNS) {
            this.selectedBannerPatternIndex.set(integer);
            this.setupResultSlot();
            return true;
        }
        return false;
    }
    
    @Override
    public void slotsChanged(final Container ajb) {
        final ItemStack bek3 = this.bannerSlot.getItem();
        final ItemStack bek4 = this.dyeSlot.getItem();
        final ItemStack bek5 = this.patternSlot.getItem();
        final ItemStack bek6 = this.resultSlot.getItem();
        if (!bek6.isEmpty() && (bek3.isEmpty() || bek4.isEmpty() || this.selectedBannerPatternIndex.get() <= 0 || (this.selectedBannerPatternIndex.get() >= BannerPattern.COUNT - 5 && bek5.isEmpty()))) {
            this.resultSlot.set(ItemStack.EMPTY);
            this.selectedBannerPatternIndex.set(0);
        }
        else if (!bek5.isEmpty() && bek5.getItem() instanceof BannerPatternItem) {
            final CompoundTag jt7 = bek3.getOrCreateTagElement("BlockEntityTag");
            final boolean boolean8 = jt7.contains("Patterns", 9) && !bek3.isEmpty() && jt7.getList("Patterns", 10).size() >= 6;
            if (boolean8) {
                this.selectedBannerPatternIndex.set(0);
            }
            else {
                this.selectedBannerPatternIndex.set(((BannerPatternItem)bek5.getItem()).getBannerPattern().ordinal());
            }
        }
        this.setupResultSlot();
        this.broadcastChanges();
    }
    
    public void registerUpdateListener(final Runnable runnable) {
        this.slotUpdateListener = runnable;
    }
    
    @Override
    public ItemStack quickMoveStack(final Player ayg, final int integer) {
        ItemStack bek4 = ItemStack.EMPTY;
        final Slot bbx5 = this.slots.get(integer);
        if (bbx5 != null && bbx5.hasItem()) {
            final ItemStack bek5 = bbx5.getItem();
            bek4 = bek5.copy();
            if (integer == this.resultSlot.index) {
                if (!this.moveItemStackTo(bek5, 4, 40, true)) {
                    return ItemStack.EMPTY;
                }
                bbx5.onQuickCraft(bek5, bek4);
            }
            else if (integer == this.dyeSlot.index || integer == this.bannerSlot.index || integer == this.patternSlot.index) {
                if (!this.moveItemStackTo(bek5, 4, 40, false)) {
                    return ItemStack.EMPTY;
                }
            }
            else if (bek5.getItem() instanceof BannerItem) {
                if (!this.moveItemStackTo(bek5, this.bannerSlot.index, this.bannerSlot.index + 1, false)) {
                    return ItemStack.EMPTY;
                }
            }
            else if (bek5.getItem() instanceof DyeItem) {
                if (!this.moveItemStackTo(bek5, this.dyeSlot.index, this.dyeSlot.index + 1, false)) {
                    return ItemStack.EMPTY;
                }
            }
            else if (bek5.getItem() instanceof BannerPatternItem) {
                if (!this.moveItemStackTo(bek5, this.patternSlot.index, this.patternSlot.index + 1, false)) {
                    return ItemStack.EMPTY;
                }
            }
            else if (integer >= 4 && integer < 31) {
                if (!this.moveItemStackTo(bek5, 31, 40, false)) {
                    return ItemStack.EMPTY;
                }
            }
            else if (integer >= 31 && integer < 40 && !this.moveItemStackTo(bek5, 4, 31, false)) {
                return ItemStack.EMPTY;
            }
            if (bek5.isEmpty()) {
                bbx5.set(ItemStack.EMPTY);
            }
            else {
                bbx5.setChanged();
            }
            if (bek5.getCount() == bek4.getCount()) {
                return ItemStack.EMPTY;
            }
            bbx5.onTake(ayg, bek5);
        }
        return bek4;
    }
    
    @Override
    public void removed(final Player ayg) {
        super.removed(ayg);
        this.access.execute((bjt, fk) -> this.clearContainer(ayg, ayg.level, this.inputContainer));
    }
    
    private void setupResultSlot() {
        if (this.selectedBannerPatternIndex.get() > 0) {
            final ItemStack bek2 = this.bannerSlot.getItem();
            final ItemStack bek3 = this.dyeSlot.getItem();
            ItemStack bek4 = ItemStack.EMPTY;
            if (!bek2.isEmpty() && !bek3.isEmpty()) {
                bek4 = bek2.copy();
                bek4.setCount(1);
                final BannerPattern bwa5 = BannerPattern.values()[this.selectedBannerPatternIndex.get()];
                final DyeColor bdg6 = ((DyeItem)bek3.getItem()).getDyeColor();
                final CompoundTag jt7 = bek4.getOrCreateTagElement("BlockEntityTag");
                ListTag jz8;
                if (jt7.contains("Patterns", 9)) {
                    jz8 = jt7.getList("Patterns", 10);
                }
                else {
                    jz8 = new ListTag();
                    jt7.put("Patterns", jz8);
                }
                final CompoundTag jt8 = new CompoundTag();
                jt8.putString("Pattern", bwa5.getHashname());
                jt8.putInt("Color", bdg6.getId());
                ((AbstractList<CompoundTag>)jz8).add(jt8);
            }
            if (!ItemStack.matches(bek4, this.resultSlot.getItem())) {
                this.resultSlot.set(bek4);
            }
        }
    }
    
    public Slot getBannerSlot() {
        return this.bannerSlot;
    }
    
    public Slot getDyeSlot() {
        return this.dyeSlot;
    }
    
    public Slot getPatternSlot() {
        return this.patternSlot;
    }
    
    public Slot getResultSlot() {
        return this.resultSlot;
    }
}
