package net.minecraft.world.inventory;

import net.minecraft.world.item.trading.MerchantOffers;
import net.minecraft.world.entity.player.Player;
import java.util.List;
import net.minecraft.world.ContainerHelper;
import java.util.Iterator;
import javax.annotation.Nullable;
import net.minecraft.world.item.trading.MerchantOffer;
import net.minecraft.world.item.ItemStack;
import net.minecraft.core.NonNullList;
import net.minecraft.world.item.trading.Merchant;
import net.minecraft.world.Container;

public class MerchantContainer implements Container {
    private final Merchant merchant;
    private final NonNullList<ItemStack> itemStacks;
    @Nullable
    private MerchantOffer activeOffer;
    private int selectionHint;
    private int futureXp;
    
    public MerchantContainer(final Merchant biv) {
        this.itemStacks = NonNullList.<ItemStack>withSize(3, ItemStack.EMPTY);
        this.merchant = biv;
    }
    
    @Override
    public int getContainerSize() {
        return this.itemStacks.size();
    }
    
    @Override
    public boolean isEmpty() {
        for (final ItemStack bek3 : this.itemStacks) {
            if (!bek3.isEmpty()) {
                return false;
            }
        }
        return true;
    }
    
    @Override
    public ItemStack getItem(final int integer) {
        return this.itemStacks.get(integer);
    }
    
    @Override
    public ItemStack removeItem(final int integer1, final int integer2) {
        final ItemStack bek4 = this.itemStacks.get(integer1);
        if (integer1 == 2 && !bek4.isEmpty()) {
            return ContainerHelper.removeItem(this.itemStacks, integer1, bek4.getCount());
        }
        final ItemStack bek5 = ContainerHelper.removeItem(this.itemStacks, integer1, integer2);
        if (!bek5.isEmpty() && this.isPaymentSlot(integer1)) {
            this.updateSellItem();
        }
        return bek5;
    }
    
    private boolean isPaymentSlot(final int integer) {
        return integer == 0 || integer == 1;
    }
    
    @Override
    public ItemStack removeItemNoUpdate(final int integer) {
        return ContainerHelper.takeItem(this.itemStacks, integer);
    }
    
    @Override
    public void setItem(final int integer, final ItemStack bek) {
        this.itemStacks.set(integer, bek);
        if (!bek.isEmpty() && bek.getCount() > this.getMaxStackSize()) {
            bek.setCount(this.getMaxStackSize());
        }
        if (this.isPaymentSlot(integer)) {
            this.updateSellItem();
        }
    }
    
    @Override
    public boolean stillValid(final Player ayg) {
        return this.merchant.getTradingPlayer() == ayg;
    }
    
    @Override
    public void setChanged() {
        this.updateSellItem();
    }
    
    public void updateSellItem() {
        this.activeOffer = null;
        ItemStack bek2;
        ItemStack bek3;
        if (this.itemStacks.get(0).isEmpty()) {
            bek2 = this.itemStacks.get(1);
            bek3 = ItemStack.EMPTY;
        }
        else {
            bek2 = this.itemStacks.get(0);
            bek3 = this.itemStacks.get(1);
        }
        if (bek2.isEmpty()) {
            this.setItem(2, ItemStack.EMPTY);
            this.futureXp = 0;
            return;
        }
        final MerchantOffers bix4 = this.merchant.getOffers();
        if (!bix4.isEmpty()) {
            MerchantOffer biw5 = bix4.getRecipeFor(bek2, bek3, this.selectionHint);
            if (biw5 == null || biw5.isOutOfStock()) {
                this.activeOffer = biw5;
                biw5 = bix4.getRecipeFor(bek3, bek2, this.selectionHint);
            }
            if (biw5 != null && !biw5.isOutOfStock()) {
                this.activeOffer = biw5;
                this.setItem(2, biw5.assemble());
                this.futureXp = biw5.getXp();
            }
            else {
                this.setItem(2, ItemStack.EMPTY);
                this.futureXp = 0;
            }
        }
        this.merchant.notifyTradeUpdated(this.getItem(2));
    }
    
    @Nullable
    public MerchantOffer getActiveOffer() {
        return this.activeOffer;
    }
    
    public void setSelectionHint(final int integer) {
        this.selectionHint = integer;
        this.updateSellItem();
    }
    
    @Override
    public void clearContent() {
        this.itemStacks.clear();
    }
    
    public int getFutureXp() {
        return this.futureXp;
    }
}
