package net.minecraft.world.inventory;

import net.minecraft.world.item.trading.MerchantOffers;
import net.minecraft.world.item.trading.MerchantOffer;
import net.minecraft.server.level.ServerPlayer;
import net.minecraft.sounds.SoundSource;
import net.minecraft.world.entity.Entity;
import net.minecraft.world.item.ItemStack;
import net.minecraft.world.entity.player.Player;
import net.minecraft.world.Container;
import net.minecraft.world.entity.npc.ClientSideMerchant;
import net.minecraft.world.entity.player.Inventory;
import net.minecraft.world.item.trading.Merchant;

public class MerchantMenu extends AbstractContainerMenu {
    private final Merchant trader;
    private final MerchantContainer tradeContainer;
    private int merchantLevel;
    private boolean showProgressBar;
    private boolean canRestock;
    
    public MerchantMenu(final int integer, final Inventory ayf) {
        this(integer, ayf, new ClientSideMerchant(ayf.player));
    }
    
    public MerchantMenu(final int integer, final Inventory ayf, final Merchant biv) {
        super(MenuType.MERCHANT, integer);
        this.trader = biv;
        this.tradeContainer = new MerchantContainer(biv);
        this.addSlot(new Slot(this.tradeContainer, 0, 136, 37));
        this.addSlot(new Slot(this.tradeContainer, 1, 162, 37));
        this.addSlot(new MerchantResultSlot(ayf.player, biv, this.tradeContainer, 2, 220, 37));
        for (int integer2 = 0; integer2 < 3; ++integer2) {
            for (int integer3 = 0; integer3 < 9; ++integer3) {
                this.addSlot(new Slot(ayf, integer3 + integer2 * 9 + 9, 108 + integer3 * 18, 84 + integer2 * 18));
            }
        }
        for (int integer2 = 0; integer2 < 9; ++integer2) {
            this.addSlot(new Slot(ayf, integer2, 108 + integer2 * 18, 142));
        }
    }
    
    public void setShowProgressBar(final boolean boolean1) {
        this.showProgressBar = boolean1;
    }
    
    @Override
    public void slotsChanged(final Container ajb) {
        this.tradeContainer.updateSellItem();
        super.slotsChanged(ajb);
    }
    
    public void setSelectionHint(final int integer) {
        this.tradeContainer.setSelectionHint(integer);
    }
    
    @Override
    public boolean stillValid(final Player ayg) {
        return this.trader.getTradingPlayer() == ayg;
    }
    
    public int getTraderXp() {
        return this.trader.getVillagerXp();
    }
    
    public int getFutureTraderXp() {
        return this.tradeContainer.getFutureXp();
    }
    
    public void setXp(final int integer) {
        this.trader.overrideXp(integer);
    }
    
    public int getTraderLevel() {
        return this.merchantLevel;
    }
    
    public void setMerchantLevel(final int integer) {
        this.merchantLevel = integer;
    }
    
    public void setCanRestock(final boolean boolean1) {
        this.canRestock = boolean1;
    }
    
    public boolean canRestock() {
        return this.canRestock;
    }
    
    @Override
    public boolean canTakeItemForPickAll(final ItemStack bek, final Slot bbx) {
        return false;
    }
    
    @Override
    public ItemStack quickMoveStack(final Player ayg, final int integer) {
        ItemStack bek4 = ItemStack.EMPTY;
        final Slot bbx5 = this.slots.get(integer);
        if (bbx5 != null && bbx5.hasItem()) {
            final ItemStack bek5 = bbx5.getItem();
            bek4 = bek5.copy();
            if (integer == 2) {
                if (!this.moveItemStackTo(bek5, 3, 39, true)) {
                    return ItemStack.EMPTY;
                }
                bbx5.onQuickCraft(bek5, bek4);
                this.playTradeSound();
            }
            else if (integer == 0 || integer == 1) {
                if (!this.moveItemStackTo(bek5, 3, 39, false)) {
                    return ItemStack.EMPTY;
                }
            }
            else if (integer >= 3 && integer < 30) {
                if (!this.moveItemStackTo(bek5, 30, 39, false)) {
                    return ItemStack.EMPTY;
                }
            }
            else if (integer >= 30 && integer < 39 && !this.moveItemStackTo(bek5, 3, 30, false)) {
                return ItemStack.EMPTY;
            }
            if (bek5.isEmpty()) {
                bbx5.set(ItemStack.EMPTY);
            }
            else {
                bbx5.setChanged();
            }
            if (bek5.getCount() == bek4.getCount()) {
                return ItemStack.EMPTY;
            }
            bbx5.onTake(ayg, bek5);
        }
        return bek4;
    }
    
    private void playTradeSound() {
        if (!this.trader.getLevel().isClientSide) {
            final Entity akn2 = (Entity)this.trader;
            this.trader.getLevel().playLocalSound(akn2.getX(), akn2.getY(), akn2.getZ(), this.trader.getNotifyTradeSound(), SoundSource.NEUTRAL, 1.0f, 1.0f, false);
        }
    }
    
    @Override
    public void removed(final Player ayg) {
        super.removed(ayg);
        this.trader.setTradingPlayer(null);
        if (this.trader.getLevel().isClientSide) {
            return;
        }
        if (!ayg.isAlive() || (ayg instanceof ServerPlayer && ((ServerPlayer)ayg).hasDisconnected())) {
            ItemStack bek3 = this.tradeContainer.removeItemNoUpdate(0);
            if (!bek3.isEmpty()) {
                ayg.drop(bek3, false);
            }
            bek3 = this.tradeContainer.removeItemNoUpdate(1);
            if (!bek3.isEmpty()) {
                ayg.drop(bek3, false);
            }
        }
        else {
            ayg.inventory.placeItemBackInInventory(ayg.level, this.tradeContainer.removeItemNoUpdate(0));
            ayg.inventory.placeItemBackInInventory(ayg.level, this.tradeContainer.removeItemNoUpdate(1));
        }
    }
    
    public void tryMoveItems(final int integer) {
        if (this.getOffers().size() <= integer) {
            return;
        }
        final ItemStack bek3 = this.tradeContainer.getItem(0);
        if (!bek3.isEmpty()) {
            if (!this.moveItemStackTo(bek3, 3, 39, true)) {
                return;
            }
            this.tradeContainer.setItem(0, bek3);
        }
        final ItemStack bek4 = this.tradeContainer.getItem(1);
        if (!bek4.isEmpty()) {
            if (!this.moveItemStackTo(bek4, 3, 39, true)) {
                return;
            }
            this.tradeContainer.setItem(1, bek4);
        }
        if (this.tradeContainer.getItem(0).isEmpty() && this.tradeContainer.getItem(1).isEmpty()) {
            final ItemStack bek5 = this.getOffers().get(integer).getCostA();
            this.moveFromInventoryToPaymentSlot(0, bek5);
            final ItemStack bek6 = this.getOffers().get(integer).getCostB();
            this.moveFromInventoryToPaymentSlot(1, bek6);
        }
    }
    
    private void moveFromInventoryToPaymentSlot(final int integer, final ItemStack bek) {
        if (!bek.isEmpty()) {
            for (int integer2 = 3; integer2 < 39; ++integer2) {
                final ItemStack bek2 = this.slots.get(integer2).getItem();
                if (!bek2.isEmpty() && this.isSameItem(bek, bek2)) {
                    final ItemStack bek3 = this.tradeContainer.getItem(integer);
                    final int integer3 = bek3.isEmpty() ? 0 : bek3.getCount();
                    final int integer4 = Math.min(bek.getMaxStackSize() - integer3, bek2.getCount());
                    final ItemStack bek4 = bek2.copy();
                    final int integer5 = integer3 + integer4;
                    bek2.shrink(integer4);
                    bek4.setCount(integer5);
                    this.tradeContainer.setItem(integer, bek4);
                    if (integer5 >= bek.getMaxStackSize()) {
                        break;
                    }
                }
            }
        }
    }
    
    private boolean isSameItem(final ItemStack bek1, final ItemStack bek2) {
        return bek1.getItem() == bek2.getItem() && ItemStack.tagMatches(bek1, bek2);
    }
    
    public void setOffers(final MerchantOffers bix) {
        this.trader.overrideOffers(bix);
    }
    
    public MerchantOffers getOffers() {
        return this.trader.getOffers();
    }
    
    public boolean showProgressBar() {
        return this.showProgressBar;
    }
}
