package net.minecraft.world.inventory;

import net.minecraft.world.item.trading.MerchantOffer;
import net.minecraft.stats.Stats;
import net.minecraft.world.item.ItemStack;
import net.minecraft.world.Container;
import net.minecraft.world.item.trading.Merchant;
import net.minecraft.world.entity.player.Player;

public class MerchantResultSlot extends Slot {
    private final MerchantContainer slots;
    private final Player player;
    private int removeCount;
    private final Merchant merchant;
    
    public MerchantResultSlot(final Player ayg, final Merchant biv, final MerchantContainer bbm, final int integer4, final int integer5, final int integer6) {
        super(bbm, integer4, integer5, integer6);
        this.player = ayg;
        this.merchant = biv;
        this.slots = bbm;
    }
    
    @Override
    public boolean mayPlace(final ItemStack bek) {
        return false;
    }
    
    @Override
    public ItemStack remove(final int integer) {
        if (this.hasItem()) {
            this.removeCount += Math.min(integer, this.getItem().getCount());
        }
        return super.remove(integer);
    }
    
    @Override
    protected void onQuickCraft(final ItemStack bek, final int integer) {
        this.removeCount += integer;
        this.checkTakeAchievements(bek);
    }
    
    @Override
    protected void checkTakeAchievements(final ItemStack bek) {
        bek.onCraftedBy(this.player.level, this.player, this.removeCount);
        this.removeCount = 0;
    }
    
    @Override
    public ItemStack onTake(final Player ayg, final ItemStack bek) {
        this.checkTakeAchievements(bek);
        final MerchantOffer biw4 = this.slots.getActiveOffer();
        if (biw4 != null) {
            final ItemStack bek2 = this.slots.getItem(0);
            final ItemStack bek3 = this.slots.getItem(1);
            if (biw4.take(bek2, bek3) || biw4.take(bek3, bek2)) {
                this.merchant.notifyTrade(biw4);
                ayg.awardStat(Stats.TRADED_WITH_VILLAGER);
                this.slots.setItem(0, bek2);
                this.slots.setItem(1, bek3);
            }
            this.merchant.overrideXp(this.merchant.getVillagerXp() + biw4.getXp());
        }
        return bek;
    }
}
