package net.minecraft.world.inventory;

import net.minecraft.core.NonNullList;
import net.minecraft.world.item.crafting.CraftingRecipe;
import net.minecraft.world.item.crafting.RecipeType;
import net.minecraft.world.item.ItemStack;
import net.minecraft.world.Container;
import net.minecraft.world.entity.player.Player;

public class ResultSlot extends Slot {
    private final CraftingContainer craftSlots;
    private final Player player;
    private int removeCount;
    
    public ResultSlot(final Player ayg, final CraftingContainer baw, final Container ajb, final int integer4, final int integer5, final int integer6) {
        super(ajb, integer4, integer5, integer6);
        this.player = ayg;
        this.craftSlots = baw;
    }
    
    @Override
    public boolean mayPlace(final ItemStack bek) {
        return false;
    }
    
    @Override
    public ItemStack remove(final int integer) {
        if (this.hasItem()) {
            this.removeCount += Math.min(integer, this.getItem().getCount());
        }
        return super.remove(integer);
    }
    
    @Override
    protected void onQuickCraft(final ItemStack bek, final int integer) {
        this.removeCount += integer;
        this.checkTakeAchievements(bek);
    }
    
    @Override
    protected void onSwapCraft(final int integer) {
        this.removeCount += integer;
    }
    
    @Override
    protected void checkTakeAchievements(final ItemStack bek) {
        if (this.removeCount > 0) {
            bek.onCraftedBy(this.player.level, this.player, this.removeCount);
        }
        if (this.container instanceof RecipeHolder) {
            ((RecipeHolder)this.container).awardAndReset(this.player);
        }
        this.removeCount = 0;
    }
    
    @Override
    public ItemStack onTake(final Player ayg, final ItemStack bek) {
        this.checkTakeAchievements(bek);
        final NonNullList<ItemStack> fy4 = ayg.level.getRecipeManager().<CraftingContainer, CraftingRecipe>getRemainingItemsFor(RecipeType.CRAFTING, this.craftSlots, ayg.level);
        for (int integer5 = 0; integer5 < fy4.size(); ++integer5) {
            ItemStack bek2 = this.craftSlots.getItem(integer5);
            final ItemStack bek3 = fy4.get(integer5);
            if (!bek2.isEmpty()) {
                this.craftSlots.removeItem(integer5, 1);
                bek2 = this.craftSlots.getItem(integer5);
            }
            if (!bek3.isEmpty()) {
                if (bek2.isEmpty()) {
                    this.craftSlots.setItem(integer5, bek3);
                }
                else if (ItemStack.isSame(bek2, bek3) && ItemStack.tagMatches(bek2, bek3)) {
                    bek3.grow(bek2.getCount());
                    this.craftSlots.setItem(integer5, bek3);
                }
                else if (!this.player.inventory.add(bek3)) {
                    this.player.drop(bek3, false);
                }
            }
        }
        return bek;
    }
}
