package net.minecraft.world.inventory;

import net.minecraft.world.item.Item;
import net.minecraft.world.item.crafting.RecipeType;
import net.minecraft.world.level.block.Blocks;
import net.minecraft.core.BlockPos;
import net.minecraft.sounds.SoundSource;
import net.minecraft.sounds.SoundEvents;
import net.minecraft.world.entity.player.Player;
import net.minecraft.world.SimpleContainer;
import com.google.common.collect.Lists;
import net.minecraft.world.entity.player.Inventory;
import net.minecraft.world.Container;
import net.minecraft.world.item.ItemStack;
import net.minecraft.world.item.crafting.StonecutterRecipe;
import java.util.List;
import net.minecraft.world.level.Level;

public class StonecutterMenu extends AbstractContainerMenu {
    private final ContainerLevelAccess access;
    private final DataSlot selectedRecipeIndex;
    private final Level level;
    private List<StonecutterRecipe> recipes;
    private ItemStack input;
    private long lastSoundTime;
    final Slot inputSlot;
    final Slot resultSlot;
    private Runnable slotUpdateListener;
    public final Container container;
    private final ResultContainer resultContainer;
    
    public StonecutterMenu(final int integer, final Inventory ayf) {
        this(integer, ayf, ContainerLevelAccess.NULL);
    }
    
    public StonecutterMenu(final int integer, final Inventory ayf, final ContainerLevelAccess bau) {
        super(MenuType.STONECUTTER, integer);
        this.selectedRecipeIndex = DataSlot.standalone();
        this.recipes = Lists.newArrayList();
        this.input = ItemStack.EMPTY;
        this.slotUpdateListener = (() -> {});
        this.container = new SimpleContainer(1) {
            @Override
            public void setChanged() {
                super.setChanged();
                StonecutterMenu.this.slotsChanged(this);
                StonecutterMenu.this.slotUpdateListener.run();
            }
        };
        this.resultContainer = new ResultContainer();
        this.access = bau;
        this.level = ayf.player.level;
        this.inputSlot = this.addSlot(new Slot(this.container, 0, 20, 33));
        this.resultSlot = this.addSlot(new Slot(this.resultContainer, 1, 143, 33) {
            @Override
            public boolean mayPlace(final ItemStack bek) {
                return false;
            }
            
            @Override
            public ItemStack onTake(final Player ayg, final ItemStack bek) {
                final ItemStack bek2 = StonecutterMenu.this.inputSlot.remove(1);
                if (!bek2.isEmpty()) {
                    StonecutterMenu.this.setupResultSlot();
                }
                bek.getItem().onCraftedBy(bek, ayg.level, ayg);
                final long long4;
                bau.execute((bjt, fk) -> {
                    long4 = bjt.getGameTime();
                    if (StonecutterMenu.this.lastSoundTime != long4) {
                        bjt.playSound(null, fk, SoundEvents.UI_STONECUTTER_TAKE_RESULT, SoundSource.BLOCKS, 1.0f, 1.0f);
                        StonecutterMenu.this.lastSoundTime = long4;
                    }
                    return;
                });
                return super.onTake(ayg, bek);
            }
        });
        for (int integer2 = 0; integer2 < 3; ++integer2) {
            for (int integer3 = 0; integer3 < 9; ++integer3) {
                this.addSlot(new Slot(ayf, integer3 + integer2 * 9 + 9, 8 + integer3 * 18, 84 + integer2 * 18));
            }
        }
        for (int integer2 = 0; integer2 < 9; ++integer2) {
            this.addSlot(new Slot(ayf, integer2, 8 + integer2 * 18, 142));
        }
        this.addDataSlot(this.selectedRecipeIndex);
    }
    
    public int getSelectedRecipeIndex() {
        return this.selectedRecipeIndex.get();
    }
    
    public List<StonecutterRecipe> getRecipes() {
        return this.recipes;
    }
    
    public int getNumRecipes() {
        return this.recipes.size();
    }
    
    public boolean hasInputItem() {
        return this.inputSlot.hasItem() && !this.recipes.isEmpty();
    }
    
    @Override
    public boolean stillValid(final Player ayg) {
        return AbstractContainerMenu.stillValid(this.access, ayg, Blocks.STONECUTTER);
    }
    
    @Override
    public boolean clickMenuButton(final Player ayg, final int integer) {
        if (integer >= 0 && integer < this.recipes.size()) {
            this.selectedRecipeIndex.set(integer);
            this.setupResultSlot();
        }
        return true;
    }
    
    @Override
    public void slotsChanged(final Container ajb) {
        final ItemStack bek3 = this.inputSlot.getItem();
        if (bek3.getItem() != this.input.getItem()) {
            this.input = bek3.copy();
            this.setupRecipeList(ajb, bek3);
        }
    }
    
    private void setupRecipeList(final Container ajb, final ItemStack bek) {
        this.recipes.clear();
        this.selectedRecipeIndex.set(-1);
        this.resultSlot.set(ItemStack.EMPTY);
        if (!bek.isEmpty()) {
            this.recipes = this.level.getRecipeManager().<Container, StonecutterRecipe>getRecipesFor(RecipeType.STONECUTTING, ajb, this.level);
        }
    }
    
    private void setupResultSlot() {
        if (!this.recipes.isEmpty()) {
            final StonecutterRecipe bhh2 = this.recipes.get(this.selectedRecipeIndex.get());
            this.resultSlot.set(bhh2.assemble(this.container));
        }
        else {
            this.resultSlot.set(ItemStack.EMPTY);
        }
        this.broadcastChanges();
    }
    
    @Override
    public MenuType<?> getType() {
        return MenuType.STONECUTTER;
    }
    
    public void registerUpdateListener(final Runnable runnable) {
        this.slotUpdateListener = runnable;
    }
    
    @Override
    public boolean canTakeItemForPickAll(final ItemStack bek, final Slot bbx) {
        return bbx.container != this.resultContainer && super.canTakeItemForPickAll(bek, bbx);
    }
    
    @Override
    public ItemStack quickMoveStack(final Player ayg, final int integer) {
        ItemStack bek4 = ItemStack.EMPTY;
        final Slot bbx5 = this.slots.get(integer);
        if (bbx5 != null && bbx5.hasItem()) {
            final ItemStack bek5 = bbx5.getItem();
            final Item bef7 = bek5.getItem();
            bek4 = bek5.copy();
            if (integer == 1) {
                bef7.onCraftedBy(bek5, ayg.level, ayg);
                if (!this.moveItemStackTo(bek5, 2, 38, true)) {
                    return ItemStack.EMPTY;
                }
                bbx5.onQuickCraft(bek5, bek4);
            }
            else if (integer == 0) {
                if (!this.moveItemStackTo(bek5, 2, 38, false)) {
                    return ItemStack.EMPTY;
                }
            }
            else if (this.level.getRecipeManager().<SimpleContainer, StonecutterRecipe>getRecipeFor(RecipeType.STONECUTTING, new SimpleContainer(new ItemStack[] { bek5 }), this.level).isPresent()) {
                if (!this.moveItemStackTo(bek5, 0, 1, false)) {
                    return ItemStack.EMPTY;
                }
            }
            else if (integer >= 2 && integer < 29) {
                if (!this.moveItemStackTo(bek5, 29, 38, false)) {
                    return ItemStack.EMPTY;
                }
            }
            else if (integer >= 29 && integer < 38 && !this.moveItemStackTo(bek5, 2, 29, false)) {
                return ItemStack.EMPTY;
            }
            if (bek5.isEmpty()) {
                bbx5.set(ItemStack.EMPTY);
            }
            bbx5.setChanged();
            if (bek5.getCount() == bek4.getCount()) {
                return ItemStack.EMPTY;
            }
            bbx5.onTake(ayg, bek5);
            this.broadcastChanges();
        }
        return bek4;
    }
    
    @Override
    public void removed(final Player ayg) {
        super.removed(ayg);
        this.resultContainer.removeItemNoUpdate(1);
        this.access.execute((bjt, fk) -> this.clearContainer(ayg, ayg.level, this.container));
    }
}
