package net.minecraft.world.item;

import net.minecraft.core.Rotations;
import java.util.Random;
import java.util.List;
import net.minecraft.core.BlockPos;
import net.minecraft.world.level.Level;
import net.minecraft.world.entity.player.Player;
import net.minecraft.sounds.SoundSource;
import net.minecraft.sounds.SoundEvents;
import net.minecraft.world.entity.EntityType;
import net.minecraft.util.Mth;
import net.minecraft.world.entity.decoration.ArmorStand;
import net.minecraft.world.entity.Entity;
import net.minecraft.world.phys.AABB;
import net.minecraft.core.Direction;
import net.minecraft.world.InteractionResult;

public class ArmorStandItem extends Item {
    public ArmorStandItem(final Properties a) {
        super(a);
    }
    
    @Override
    public InteractionResult useOn(final UseOnContext bfw) {
        final Direction fp3 = bfw.getClickedFace();
        if (fp3 == Direction.DOWN) {
            return InteractionResult.FAIL;
        }
        final Level bjt4 = bfw.getLevel();
        final BlockPlaceContext bcn5 = new BlockPlaceContext(bfw);
        final BlockPos fk6 = bcn5.getClickedPos();
        final BlockPos fk7 = fk6.above();
        if (!bcn5.canPlace() || !bjt4.getBlockState(fk7).canBeReplaced(bcn5)) {
            return InteractionResult.FAIL;
        }
        final double double8 = fk6.getX();
        final double double9 = fk6.getY();
        final double double10 = fk6.getZ();
        final List<Entity> list14 = bjt4.getEntities(null, new AABB(double8, double9, double10, double8 + 1.0, double9 + 2.0, double10 + 1.0));
        if (!list14.isEmpty()) {
            return InteractionResult.FAIL;
        }
        final ItemStack bek15 = bfw.getItemInHand();
        if (!bjt4.isClientSide) {
            bjt4.removeBlock(fk6, false);
            bjt4.removeBlock(fk7, false);
            final ArmorStand avm16 = new ArmorStand(bjt4, double8 + 0.5, double9, double10 + 0.5);
            final float float17 = Mth.floor((Mth.wrapDegrees(bfw.getRotation() - 180.0f) + 22.5f) / 45.0f) * 45.0f;
            avm16.moveTo(double8 + 0.5, double9, double10 + 0.5, float17, 0.0f);
            this.randomizePose(avm16, bjt4.random);
            EntityType.updateCustomEntityTag(bjt4, bfw.getPlayer(), avm16, bek15.getTag());
            bjt4.addFreshEntity(avm16);
            bjt4.playSound(null, avm16.getX(), avm16.getY(), avm16.getZ(), SoundEvents.ARMOR_STAND_PLACE, SoundSource.BLOCKS, 0.75f, 0.8f);
        }
        bek15.shrink(1);
        return InteractionResult.SUCCESS;
    }
    
    private void randomizePose(final ArmorStand avm, final Random random) {
        Rotations gc4 = avm.getHeadPose();
        float float6 = random.nextFloat() * 5.0f;
        final float float7 = random.nextFloat() * 20.0f - 10.0f;
        Rotations gc5 = new Rotations(gc4.getX() + float6, gc4.getY() + float7, gc4.getZ());
        avm.setHeadPose(gc5);
        gc4 = avm.getBodyPose();
        float6 = random.nextFloat() * 10.0f - 5.0f;
        gc5 = new Rotations(gc4.getX(), gc4.getY() + float6, gc4.getZ());
        avm.setBodyPose(gc5);
    }
}
