package net.minecraft.world.item;

import net.minecraft.world.level.block.state.AbstractStateHolder;
import net.minecraft.core.particles.ParticleOptions;
import net.minecraft.core.particles.ParticleTypes;
import net.minecraft.world.level.LevelAccessor;
import net.minecraft.world.level.biome.Biome;
import net.minecraft.world.level.LevelReader;
import net.minecraft.world.level.block.state.properties.Property;
import net.minecraft.world.level.block.BaseCoralWallFanBlock;
import net.minecraft.tags.BlockTags;
import net.minecraft.world.level.block.Block;
import net.minecraft.world.level.biome.Biomes;
import net.minecraft.world.level.block.Blocks;
import javax.annotation.Nullable;
import net.minecraft.core.Direction;
import net.minecraft.server.level.ServerLevel;
import net.minecraft.world.level.block.BonemealableBlock;
import net.minecraft.world.level.block.state.BlockState;
import net.minecraft.core.BlockPos;
import net.minecraft.world.level.Level;
import net.minecraft.world.level.BlockGetter;
import net.minecraft.world.InteractionResult;

public class BoneMealItem extends Item {
    public BoneMealItem(final Properties a) {
        super(a);
    }
    
    @Override
    public InteractionResult useOn(final UseOnContext bfw) {
        final Level bjt3 = bfw.getLevel();
        final BlockPos fk4 = bfw.getClickedPos();
        final BlockPos fk5 = fk4.relative(bfw.getClickedFace());
        if (growCrop(bfw.getItemInHand(), bjt3, fk4)) {
            if (!bjt3.isClientSide) {
                bjt3.levelEvent(2005, fk4, 0);
            }
            return InteractionResult.SUCCESS;
        }
        final BlockState byg6 = bjt3.getBlockState(fk4);
        final boolean boolean7 = byg6.isFaceSturdy(bjt3, fk4, bfw.getClickedFace());
        if (boolean7 && growWaterPlant(bfw.getItemInHand(), bjt3, fk5, bfw.getClickedFace())) {
            if (!bjt3.isClientSide) {
                bjt3.levelEvent(2005, fk5, 0);
            }
            return InteractionResult.SUCCESS;
        }
        return InteractionResult.PASS;
    }
    
    public static boolean growCrop(final ItemStack bek, final Level bjt, final BlockPos fk) {
        final BlockState byg4 = bjt.getBlockState(fk);
        if (byg4.getBlock() instanceof BonemealableBlock) {
            final BonemealableBlock bpg5 = (BonemealableBlock)byg4.getBlock();
            if (bpg5.isValidBonemealTarget(bjt, fk, byg4, bjt.isClientSide)) {
                if (bjt instanceof ServerLevel) {
                    if (bpg5.isBonemealSuccess(bjt, bjt.random, fk, byg4)) {
                        bpg5.performBonemeal((ServerLevel)bjt, bjt.random, fk, byg4);
                    }
                    bek.shrink(1);
                }
                return true;
            }
        }
        return false;
    }
    
    public static boolean growWaterPlant(final ItemStack bek, final Level bjt, final BlockPos fk, @Nullable final Direction fp) {
        if (bjt.getBlockState(fk).getBlock() != Blocks.WATER || bjt.getFluidState(fk).getAmount() != 8) {
            return false;
        }
        if (!(bjt instanceof ServerLevel)) {
            return true;
        }
        int integer5 = 0;
    Label_0043:
        while (integer5 < 128) {
            BlockPos fk2 = fk;
            Biome bkq7 = bjt.getBiome(fk2);
            BlockState byg8 = Blocks.SEAGRASS.defaultBlockState();
            while (true) {
                for (int integer6 = 0; integer6 < integer5 / 16; ++integer6) {
                    fk2 = fk2.offset(BoneMealItem.random.nextInt(3) - 1, (BoneMealItem.random.nextInt(3) - 1) * BoneMealItem.random.nextInt(3) / 2, BoneMealItem.random.nextInt(3) - 1);
                    bkq7 = bjt.getBiome(fk2);
                    if (bjt.getBlockState(fk2).isCollisionShapeFullBlock(bjt, fk2)) {
                        ++integer5;
                        continue Label_0043;
                    }
                }
                if (bkq7 == Biomes.WARM_OCEAN || bkq7 == Biomes.DEEP_WARM_OCEAN) {
                    if (integer5 == 0 && fp != null && fp.getAxis().isHorizontal()) {
                        byg8 = ((AbstractStateHolder<O, BlockState>)BlockTags.WALL_CORALS.getRandomElement(bjt.random).defaultBlockState()).<Comparable, Direction>setValue((Property<Comparable>)BaseCoralWallFanBlock.FACING, fp);
                    }
                    else if (BoneMealItem.random.nextInt(4) == 0) {
                        byg8 = BlockTags.UNDERWATER_BONEMEALS.getRandomElement(BoneMealItem.random).defaultBlockState();
                    }
                }
                if (byg8.getBlock().is(BlockTags.WALL_CORALS)) {
                    for (int integer6 = 0; !byg8.canSurvive(bjt, fk2) && integer6 < 4; byg8 = ((AbstractStateHolder<O, BlockState>)byg8).<Comparable, Direction>setValue((Property<Comparable>)BaseCoralWallFanBlock.FACING, Direction.Plane.HORIZONTAL.getRandomDirection(BoneMealItem.random)), ++integer6) {}
                }
                if (!byg8.canSurvive(bjt, fk2)) {
                    continue;
                }
                final BlockState byg9 = bjt.getBlockState(fk2);
                if (byg9.getBlock() == Blocks.WATER && bjt.getFluidState(fk2).getAmount() == 8) {
                    bjt.setBlock(fk2, byg8, 3);
                    continue;
                }
                if (byg9.getBlock() == Blocks.SEAGRASS && BoneMealItem.random.nextInt(10) == 0) {
                    ((BonemealableBlock)Blocks.SEAGRASS).performBonemeal((ServerLevel)bjt, BoneMealItem.random, fk2, byg9);
                }
                continue;
            }
        }
        bek.shrink(1);
        return true;
    }
    
    public static void addGrowthParticles(final LevelAccessor bju, final BlockPos fk, int integer) {
        if (integer == 0) {
            integer = 15;
        }
        final BlockState byg4 = bju.getBlockState(fk);
        if (byg4.isAir()) {
            return;
        }
        for (int integer2 = 0; integer2 < integer; ++integer2) {
            final double double6 = BoneMealItem.random.nextGaussian() * 0.02;
            final double double7 = BoneMealItem.random.nextGaussian() * 0.02;
            final double double8 = BoneMealItem.random.nextGaussian() * 0.02;
            bju.addParticle(ParticleTypes.HAPPY_VILLAGER, fk.getX() + BoneMealItem.random.nextFloat(), fk.getY() + BoneMealItem.random.nextFloat() * byg4.getShape(bju, fk).max(Direction.Axis.Y), fk.getZ() + BoneMealItem.random.nextFloat(), double6, double7, double8);
        }
    }
}
