package net.minecraft.world.item;

import java.util.function.Predicate;
import net.minecraft.world.InteractionResultHolder;
import net.minecraft.world.InteractionHand;
import net.minecraft.stats.Stat;
import net.minecraft.stats.Stats;
import net.minecraft.sounds.SoundSource;
import net.minecraft.sounds.SoundEvents;
import net.minecraft.world.entity.projectile.AbstractArrow;
import net.minecraft.world.entity.Entity;
import net.minecraft.world.level.ItemLike;
import net.minecraft.world.item.enchantment.EnchantmentHelper;
import net.minecraft.world.item.enchantment.Enchantments;
import net.minecraft.world.entity.player.Player;
import net.minecraft.world.entity.LivingEntity;
import net.minecraft.world.level.Level;
import net.minecraft.resources.ResourceLocation;

public class BowItem extends ProjectileWeaponItem {
    public BowItem(final Properties a) {
        super(a);
        this.addProperty(new ResourceLocation("pull"), (bek, bjt, akw) -> {
            if (akw == null) {
                return 0.0f;
            }
            else if (akw.getUseItem().getItem() != Items.BOW) {
                return 0.0f;
            }
            else {
                return (bek.getUseDuration() - akw.getUseItemRemainingTicks()) / 20.0f;
            }
        });
        this.addProperty(new ResourceLocation("pulling"), (bek, bjt, akw) -> (akw != null && akw.isUsingItem() && akw.getUseItem() == bek) ? 1.0f : 0.0f);
    }
    
    @Override
    public void releaseUsing(final ItemStack bek, final Level bjt, final LivingEntity akw, final int integer) {
        if (!(akw instanceof Player)) {
            return;
        }
        final Player ayg3 = (Player)akw;
        final boolean boolean7 = ayg3.abilities.instabuild || EnchantmentHelper.getItemEnchantmentLevel(Enchantments.INFINITY_ARROWS, bek) > 0;
        ItemStack bek2 = ayg3.getProjectile(bek);
        if (bek2.isEmpty() && !boolean7) {
            return;
        }
        if (bek2.isEmpty()) {
            bek2 = new ItemStack(Items.ARROW);
        }
        final int integer2 = this.getUseDuration(bek) - integer;
        final float float10 = getPowerForTime(integer2);
        if (float10 < 0.1) {
            return;
        }
        final boolean boolean8 = boolean7 && bek2.getItem() == Items.ARROW;
        if (!bjt.isClientSide) {
            final ArrowItem bch12 = (ArrowItem)((bek2.getItem() instanceof ArrowItem) ? bek2.getItem() : Items.ARROW);
            final AbstractArrow ayk13 = bch12.createArrow(bjt, bek2, ayg3);
            ayk13.shootFromRotation(ayg3, ayg3.xRot, ayg3.yRot, 0.0f, float10 * 3.0f, 1.0f);
            if (float10 == 1.0f) {
                ayk13.setCritArrow(true);
            }
            final int integer3 = EnchantmentHelper.getItemEnchantmentLevel(Enchantments.POWER_ARROWS, bek);
            if (integer3 > 0) {
                ayk13.setBaseDamage(ayk13.getBaseDamage() + integer3 * 0.5 + 0.5);
            }
            final int integer4 = EnchantmentHelper.getItemEnchantmentLevel(Enchantments.PUNCH_ARROWS, bek);
            if (integer4 > 0) {
                ayk13.setKnockback(integer4);
            }
            if (EnchantmentHelper.getItemEnchantmentLevel(Enchantments.FLAMING_ARROWS, bek) > 0) {
                ayk13.setSecondsOnFire(100);
            }
            bek.<Player>hurtAndBreak(1, ayg3, ayg2 -> ayg2.broadcastBreakEvent(ayg3.getUsedItemHand()));
            if (boolean8 || (ayg3.abilities.instabuild && (bek2.getItem() == Items.SPECTRAL_ARROW || bek2.getItem() == Items.TIPPED_ARROW))) {
                ayk13.pickup = AbstractArrow.Pickup.CREATIVE_ONLY;
            }
            bjt.addFreshEntity(ayk13);
        }
        bjt.playSound(null, ayg3.getX(), ayg3.getY(), ayg3.getZ(), SoundEvents.ARROW_SHOOT, SoundSource.PLAYERS, 1.0f, 1.0f / (BowItem.random.nextFloat() * 0.4f + 1.2f) + float10 * 0.5f);
        if (!boolean8 && !ayg3.abilities.instabuild) {
            bek2.shrink(1);
            if (bek2.isEmpty()) {
                ayg3.inventory.removeItem(bek2);
            }
        }
        ayg3.awardStat(Stats.ITEM_USED.get(this));
    }
    
    public static float getPowerForTime(final int integer) {
        float float2 = integer / 20.0f;
        float2 = (float2 * float2 + float2 * 2.0f) / 3.0f;
        if (float2 > 1.0f) {
            float2 = 1.0f;
        }
        return float2;
    }
    
    @Override
    public int getUseDuration(final ItemStack bek) {
        return 72000;
    }
    
    @Override
    public UseAnim getUseAnimation(final ItemStack bek) {
        return UseAnim.BOW;
    }
    
    @Override
    public InteractionResultHolder<ItemStack> use(final Level bjt, final Player ayg, final InteractionHand ajh) {
        final ItemStack bek5 = ayg.getItemInHand(ajh);
        final boolean boolean6 = !ayg.getProjectile(bek5).isEmpty();
        if (ayg.abilities.instabuild || boolean6) {
            ayg.startUsingItem(ajh);
            return InteractionResultHolder.<ItemStack>consume(bek5);
        }
        return InteractionResultHolder.<ItemStack>fail(bek5);
    }
    
    @Override
    public Predicate<ItemStack> getAllSupportedProjectiles() {
        return BowItem.ARROW_ONLY;
    }
}
