package net.minecraft.world.item;

import net.minecraft.world.entity.monster.SharedMonsterAttributes;
import net.minecraft.world.entity.ai.attributes.AttributeModifier;
import com.google.common.collect.Multimap;
import net.minecraft.world.level.BlockGetter;
import net.minecraft.world.entity.EquipmentSlot;
import net.minecraft.world.InteractionResultHolder;
import net.minecraft.world.InteractionHand;
import net.minecraft.sounds.SoundEvent;
import net.minecraft.world.phys.Vec3;
import net.minecraft.world.entity.MoverType;
import net.minecraft.util.Mth;
import net.minecraft.stats.Stat;
import net.minecraft.stats.Stats;
import net.minecraft.sounds.SoundSource;
import net.minecraft.sounds.SoundEvents;
import net.minecraft.world.entity.projectile.AbstractArrow;
import net.minecraft.world.entity.Entity;
import net.minecraft.world.entity.projectile.ThrownTrident;
import net.minecraft.world.item.enchantment.EnchantmentHelper;
import net.minecraft.world.entity.LivingEntity;
import net.minecraft.world.entity.player.Player;
import net.minecraft.core.BlockPos;
import net.minecraft.world.level.Level;
import net.minecraft.world.level.block.state.BlockState;
import net.minecraft.resources.ResourceLocation;

public class TridentItem extends Item {
    public TridentItem(final Properties a) {
        super(a);
        this.addProperty(new ResourceLocation("throwing"), (bek, bjt, akw) -> (akw != null && akw.isUsingItem() && akw.getUseItem() == bek) ? 1.0f : 0.0f);
    }
    
    @Override
    public boolean canAttackBlock(final BlockState byg, final Level bjt, final BlockPos fk, final Player ayg) {
        return !ayg.isCreative();
    }
    
    @Override
    public UseAnim getUseAnimation(final ItemStack bek) {
        return UseAnim.SPEAR;
    }
    
    @Override
    public int getUseDuration(final ItemStack bek) {
        return 72000;
    }
    
    @Override
    public void releaseUsing(final ItemStack bek, final Level bjt, final LivingEntity akw, final int integer) {
        if (!(akw instanceof Player)) {
            return;
        }
        final Player ayg2 = (Player)akw;
        final int integer2 = this.getUseDuration(bek) - integer;
        if (integer2 < 10) {
            return;
        }
        final int integer3 = EnchantmentHelper.getRiptide(bek);
        if (integer3 > 0 && !ayg2.isInWaterOrRain()) {
            return;
        }
        if (!bjt.isClientSide) {
            bek.<Player>hurtAndBreak(1, ayg2, ayg -> ayg.broadcastBreakEvent(akw.getUsedItemHand()));
            if (integer3 == 0) {
                final ThrownTrident azh9 = new ThrownTrident(bjt, ayg2, bek);
                azh9.shootFromRotation(ayg2, ayg2.xRot, ayg2.yRot, 0.0f, 2.5f + integer3 * 0.5f, 1.0f);
                if (ayg2.abilities.instabuild) {
                    azh9.pickup = AbstractArrow.Pickup.CREATIVE_ONLY;
                }
                bjt.addFreshEntity(azh9);
                bjt.playSound(null, azh9, SoundEvents.TRIDENT_THROW, SoundSource.PLAYERS, 1.0f, 1.0f);
                if (!ayg2.abilities.instabuild) {
                    ayg2.inventory.removeItem(bek);
                }
            }
        }
        ayg2.awardStat(Stats.ITEM_USED.get(this));
        if (integer3 > 0) {
            final float float9 = ayg2.yRot;
            final float float10 = ayg2.xRot;
            float float11 = -Mth.sin(float9 * 0.017453292f) * Mth.cos(float10 * 0.017453292f);
            float float12 = -Mth.sin(float10 * 0.017453292f);
            float float13 = Mth.cos(float9 * 0.017453292f) * Mth.cos(float10 * 0.017453292f);
            final float float14 = Mth.sqrt(float11 * float11 + float12 * float12 + float13 * float13);
            final float float15 = 3.0f * ((1.0f + integer3) / 4.0f);
            float11 *= float15 / float14;
            float12 *= float15 / float14;
            float13 *= float15 / float14;
            ayg2.push(float11, float12, float13);
            ayg2.startAutoSpinAttack(20);
            if (ayg2.onGround) {
                final float float16 = 1.1999999f;
                ayg2.move(MoverType.SELF, new Vec3(0.0, 1.1999999284744263, 0.0));
            }
            SoundEvent aah16;
            if (integer3 >= 3) {
                aah16 = SoundEvents.TRIDENT_RIPTIDE_3;
            }
            else if (integer3 == 2) {
                aah16 = SoundEvents.TRIDENT_RIPTIDE_2;
            }
            else {
                aah16 = SoundEvents.TRIDENT_RIPTIDE_1;
            }
            bjt.playSound(null, ayg2, aah16, SoundSource.PLAYERS, 1.0f, 1.0f);
        }
    }
    
    @Override
    public InteractionResultHolder<ItemStack> use(final Level bjt, final Player ayg, final InteractionHand ajh) {
        final ItemStack bek5 = ayg.getItemInHand(ajh);
        if (bek5.getDamageValue() >= bek5.getMaxDamage() - 1) {
            return InteractionResultHolder.<ItemStack>fail(bek5);
        }
        if (EnchantmentHelper.getRiptide(bek5) > 0 && !ayg.isInWaterOrRain()) {
            return InteractionResultHolder.<ItemStack>fail(bek5);
        }
        ayg.startUsingItem(ajh);
        return InteractionResultHolder.<ItemStack>consume(bek5);
    }
    
    @Override
    public boolean hurtEnemy(final ItemStack bek, final LivingEntity akw2, final LivingEntity akw3) {
        bek.<LivingEntity>hurtAndBreak(1, akw3, akw -> akw.broadcastBreakEvent(EquipmentSlot.MAINHAND));
        return true;
    }
    
    @Override
    public boolean mineBlock(final ItemStack bek, final Level bjt, final BlockState byg, final BlockPos fk, final LivingEntity akw) {
        if (byg.getDestroySpeed(bjt, fk) != 0.0) {
            bek.<LivingEntity>hurtAndBreak(2, akw, akw -> akw.broadcastBreakEvent(EquipmentSlot.MAINHAND));
        }
        return true;
    }
    
    @Override
    public Multimap<String, AttributeModifier> getDefaultAttributeModifiers(final EquipmentSlot aks) {
        final Multimap<String, AttributeModifier> multimap3 = super.getDefaultAttributeModifiers(aks);
        if (aks == EquipmentSlot.MAINHAND) {
            multimap3.put(SharedMonsterAttributes.ATTACK_DAMAGE.getName(), new AttributeModifier(TridentItem.BASE_ATTACK_DAMAGE_UUID, "Tool modifier", 8.0, AttributeModifier.Operation.ADDITION));
            multimap3.put(SharedMonsterAttributes.ATTACK_SPEED.getName(), new AttributeModifier(TridentItem.BASE_ATTACK_SPEED_UUID, "Tool modifier", -2.9000000953674316, AttributeModifier.Operation.ADDITION));
        }
        return multimap3;
    }
    
    @Override
    public int getEnchantmentValue() {
        return 1;
    }
}
