package net.minecraft.world.item.alchemy;

import java.util.AbstractList;
import net.minecraft.world.effect.MobEffect;
import net.minecraft.network.chat.TextComponent;
import net.minecraft.world.effect.MobEffectUtil;
import net.minecraft.util.Tuple;
import net.minecraft.world.entity.ai.attributes.AttributeModifier;
import net.minecraft.world.entity.ai.attributes.Attribute;
import java.util.Map;
import net.minecraft.ChatFormatting;
import net.minecraft.network.chat.TranslatableComponent;
import net.minecraft.network.chat.Component;
import net.minecraft.nbt.Tag;
import net.minecraft.resources.ResourceLocation;
import net.minecraft.core.Registry;
import java.util.Iterator;
import net.minecraft.nbt.ListTag;
import javax.annotation.Nullable;
import net.minecraft.nbt.CompoundTag;
import com.google.common.collect.Lists;
import java.util.Collection;
import net.minecraft.world.effect.MobEffectInstance;
import java.util.List;
import net.minecraft.world.item.ItemStack;

public class PotionUtils {
    public static List<MobEffectInstance> getMobEffects(final ItemStack bek) {
        return getAllEffects(bek.getTag());
    }
    
    public static List<MobEffectInstance> getAllEffects(final Potion bga, final Collection<MobEffectInstance> collection) {
        final List<MobEffectInstance> list3 = Lists.newArrayList();
        list3.addAll(bga.getEffects());
        list3.addAll(collection);
        return list3;
    }
    
    public static List<MobEffectInstance> getAllEffects(@Nullable final CompoundTag jt) {
        final List<MobEffectInstance> list2 = Lists.newArrayList();
        list2.addAll(getPotion(jt).getEffects());
        getCustomEffects(jt, list2);
        return list2;
    }
    
    public static List<MobEffectInstance> getCustomEffects(final ItemStack bek) {
        return getCustomEffects(bek.getTag());
    }
    
    public static List<MobEffectInstance> getCustomEffects(@Nullable final CompoundTag jt) {
        final List<MobEffectInstance> list2 = Lists.newArrayList();
        getCustomEffects(jt, list2);
        return list2;
    }
    
    public static void getCustomEffects(@Nullable final CompoundTag jt, final List<MobEffectInstance> list) {
        if (jt != null && jt.contains("CustomPotionEffects", 9)) {
            final ListTag jz3 = jt.getList("CustomPotionEffects", 10);
            for (int integer4 = 0; integer4 < jz3.size(); ++integer4) {
                final CompoundTag jt2 = jz3.getCompound(integer4);
                final MobEffectInstance akh6 = MobEffectInstance.load(jt2);
                if (akh6 != null) {
                    list.add(akh6);
                }
            }
        }
    }
    
    public static int getColor(final ItemStack bek) {
        final CompoundTag jt2 = bek.getTag();
        if (jt2 != null && jt2.contains("CustomPotionColor", 99)) {
            return jt2.getInt("CustomPotionColor");
        }
        return (getPotion(bek) == Potions.EMPTY) ? 16253176 : getColor(getMobEffects(bek));
    }
    
    public static int getColor(final Potion bga) {
        return (bga == Potions.EMPTY) ? 16253176 : getColor(bga.getEffects());
    }
    
    public static int getColor(final Collection<MobEffectInstance> collection) {
        final int integer2 = 3694022;
        if (collection.isEmpty()) {
            return 3694022;
        }
        float float3 = 0.0f;
        float float4 = 0.0f;
        float float5 = 0.0f;
        int integer3 = 0;
        for (final MobEffectInstance akh8 : collection) {
            if (!akh8.isVisible()) {
                continue;
            }
            final int integer4 = akh8.getEffect().getColor();
            final int integer5 = akh8.getAmplifier() + 1;
            float3 += integer5 * (integer4 >> 16 & 0xFF) / 255.0f;
            float4 += integer5 * (integer4 >> 8 & 0xFF) / 255.0f;
            float5 += integer5 * (integer4 >> 0 & 0xFF) / 255.0f;
            integer3 += integer5;
        }
        if (integer3 == 0) {
            return 0;
        }
        float3 = float3 / integer3 * 255.0f;
        float4 = float4 / integer3 * 255.0f;
        float5 = float5 / integer3 * 255.0f;
        return (int)float3 << 16 | (int)float4 << 8 | (int)float5;
    }
    
    public static Potion getPotion(final ItemStack bek) {
        return getPotion(bek.getTag());
    }
    
    public static Potion getPotion(@Nullable final CompoundTag jt) {
        if (jt == null) {
            return Potions.EMPTY;
        }
        return Potion.byName(jt.getString("Potion"));
    }
    
    public static ItemStack setPotion(final ItemStack bek, final Potion bga) {
        final ResourceLocation sm3 = Registry.POTION.getKey(bga);
        if (bga == Potions.EMPTY) {
            bek.removeTagKey("Potion");
        }
        else {
            bek.getOrCreateTag().putString("Potion", sm3.toString());
        }
        return bek;
    }
    
    public static ItemStack setCustomEffects(final ItemStack bek, final Collection<MobEffectInstance> collection) {
        if (collection.isEmpty()) {
            return bek;
        }
        final CompoundTag jt3 = bek.getOrCreateTag();
        final ListTag jz4 = jt3.getList("CustomPotionEffects", 9);
        for (final MobEffectInstance akh6 : collection) {
            ((AbstractList<CompoundTag>)jz4).add(akh6.save(new CompoundTag()));
        }
        jt3.put("CustomPotionEffects", jz4);
        return bek;
    }
    
    public static void addPotionTooltip(final ItemStack bek, final List<Component> list, final float float3) {
        final List<MobEffectInstance> list2 = getMobEffects(bek);
        final List<Tuple<String, AttributeModifier>> list3 = Lists.newArrayList();
        if (list2.isEmpty()) {
            list.add(new TranslatableComponent("effect.none", new Object[0]).withStyle(ChatFormatting.GRAY));
        }
        else {
            for (final MobEffectInstance akh7 : list2) {
                final Component lf8 = new TranslatableComponent(akh7.getDescriptionId(), new Object[0]);
                final MobEffect akf9 = akh7.getEffect();
                final Map<Attribute, AttributeModifier> map10 = akf9.getAttributeModifiers();
                if (!map10.isEmpty()) {
                    for (final Map.Entry<Attribute, AttributeModifier> entry12 : map10.entrySet()) {
                        final AttributeModifier alp13 = entry12.getValue();
                        final AttributeModifier alp14 = new AttributeModifier(alp13.getName(), akf9.getAttributeModifierValue(akh7.getAmplifier(), alp13), alp13.getOperation());
                        list3.add(new Tuple<String, AttributeModifier>(entry12.getKey().getName(), alp14));
                    }
                }
                if (akh7.getAmplifier() > 0) {
                    lf8.append(" ").append(new TranslatableComponent("potion.potency." + akh7.getAmplifier(), new Object[0]));
                }
                if (akh7.getDuration() > 20) {
                    lf8.append(" (").append(MobEffectUtil.formatDuration(akh7, float3)).append(")");
                }
                list.add(lf8.withStyle(akf9.getCategory().getTooltipFormatting()));
            }
        }
        if (!list3.isEmpty()) {
            list.add(new TextComponent(""));
            list.add(new TranslatableComponent("potion.whenDrank", new Object[0]).withStyle(ChatFormatting.DARK_PURPLE));
            for (final Tuple<String, AttributeModifier> abz7 : list3) {
                final AttributeModifier alp15 = abz7.getB();
                final double double9 = alp15.getAmount();
                double double10;
                if (alp15.getOperation() == AttributeModifier.Operation.MULTIPLY_BASE || alp15.getOperation() == AttributeModifier.Operation.MULTIPLY_TOTAL) {
                    double10 = alp15.getAmount() * 100.0;
                }
                else {
                    double10 = alp15.getAmount();
                }
                if (double9 > 0.0) {
                    list.add(new TranslatableComponent("attribute.modifier.plus." + alp15.getOperation().toValue(), new Object[] { ItemStack.ATTRIBUTE_MODIFIER_FORMAT.format(double10), new TranslatableComponent("attribute.name." + abz7.getA(), new Object[0]) }).withStyle(ChatFormatting.BLUE));
                }
                else {
                    if (double9 >= 0.0) {
                        continue;
                    }
                    double10 *= -1.0;
                    list.add(new TranslatableComponent("attribute.modifier.take." + alp15.getOperation().toValue(), new Object[] { ItemStack.ATTRIBUTE_MODIFIER_FORMAT.format(double10), new TranslatableComponent("attribute.name." + abz7.getA(), new Object[0]) }).withStyle(ChatFormatting.RED));
                }
            }
        }
    }
}
