package net.minecraft.world.item.crafting;

import net.minecraft.world.Container;
import java.util.Iterator;
import net.minecraft.world.level.saveddata.maps.MapDecoration;
import net.minecraft.world.level.saveddata.maps.MapItemSavedData;
import net.minecraft.world.item.MapItem;
import net.minecraft.world.level.Level;
import net.minecraft.world.inventory.CraftingContainer;
import net.minecraft.world.item.ItemStack;
import net.minecraft.core.NonNullList;
import net.minecraft.world.item.Items;
import net.minecraft.world.level.ItemLike;
import net.minecraft.resources.ResourceLocation;

public class MapExtendingRecipe extends ShapedRecipe {
    public MapExtendingRecipe(final ResourceLocation sm) {
        super(sm, "", 3, 3, NonNullList.<Ingredient>of(Ingredient.EMPTY, new Ingredient[] { Ingredient.of(Items.PAPER), Ingredient.of(Items.PAPER), Ingredient.of(Items.PAPER), Ingredient.of(Items.PAPER), Ingredient.of(Items.FILLED_MAP), Ingredient.of(Items.PAPER), Ingredient.of(Items.PAPER), Ingredient.of(Items.PAPER), Ingredient.of(Items.PAPER) }), new ItemStack(Items.MAP));
    }
    
    @Override
    public boolean matches(final CraftingContainer baw, final Level bjt) {
        if (!super.matches(baw, bjt)) {
            return false;
        }
        ItemStack bek4 = ItemStack.EMPTY;
        for (int integer5 = 0; integer5 < baw.getContainerSize() && bek4.isEmpty(); ++integer5) {
            final ItemStack bek5 = baw.getItem(integer5);
            if (bek5.getItem() == Items.FILLED_MAP) {
                bek4 = bek5;
            }
        }
        if (bek4.isEmpty()) {
            return false;
        }
        final MapItemSavedData crd5 = MapItem.getOrCreateSavedData(bek4, bjt);
        return crd5 != null && !this.isExplorationMap(crd5) && crd5.scale < 4;
    }
    
    private boolean isExplorationMap(final MapItemSavedData crd) {
        if (crd.decorations != null) {
            for (final MapDecoration cra4 : crd.decorations.values()) {
                if (cra4.getType() == MapDecoration.Type.MANSION || cra4.getType() == MapDecoration.Type.MONUMENT) {
                    return true;
                }
            }
        }
        return false;
    }
    
    @Override
    public ItemStack assemble(final CraftingContainer baw) {
        ItemStack bek3 = ItemStack.EMPTY;
        for (int integer4 = 0; integer4 < baw.getContainerSize() && bek3.isEmpty(); ++integer4) {
            final ItemStack bek4 = baw.getItem(integer4);
            if (bek4.getItem() == Items.FILLED_MAP) {
                bek3 = bek4;
            }
        }
        bek3 = bek3.copy();
        bek3.setCount(1);
        bek3.getOrCreateTag().putInt("map_scale_direction", 1);
        return bek3;
    }
    
    @Override
    public boolean isSpecial() {
        return true;
    }
    
    @Override
    public RecipeSerializer<?> getSerializer() {
        return RecipeSerializer.MAP_EXTENDING;
    }
}
