package net.minecraft.world.item.enchantment;

import net.minecraft.world.item.CrossbowItem;
import net.minecraft.world.level.block.CarvedPumpkinBlock;
import net.minecraft.world.level.block.AbstractSkullBlock;
import net.minecraft.world.item.ElytraItem;
import net.minecraft.world.level.block.Block;
import net.minecraft.world.item.BowItem;
import net.minecraft.world.item.TridentItem;
import net.minecraft.world.item.FishingRodItem;
import net.minecraft.world.item.DiggerItem;
import net.minecraft.world.item.SwordItem;
import net.minecraft.world.entity.EquipmentSlot;
import net.minecraft.world.item.ArmorItem;
import net.minecraft.world.item.Item;

public enum EnchantmentCategory {
    ALL {
        @Override
        public boolean canEnchant(final Item bef) {
            for (final EnchantmentCategory bhv6 : EnchantmentCategory.values()) {
                if (bhv6 != EnchantmentCategory.ALL) {
                    if (bhv6.canEnchant(bef)) {
                        return true;
                    }
                }
            }
            return false;
        }
    }, 
    ARMOR {
        @Override
        public boolean canEnchant(final Item bef) {
            return bef instanceof ArmorItem;
        }
    }, 
    ARMOR_FEET {
        @Override
        public boolean canEnchant(final Item bef) {
            return bef instanceof ArmorItem && ((ArmorItem)bef).getSlot() == EquipmentSlot.FEET;
        }
    }, 
    ARMOR_LEGS {
        @Override
        public boolean canEnchant(final Item bef) {
            return bef instanceof ArmorItem && ((ArmorItem)bef).getSlot() == EquipmentSlot.LEGS;
        }
    }, 
    ARMOR_CHEST {
        @Override
        public boolean canEnchant(final Item bef) {
            return bef instanceof ArmorItem && ((ArmorItem)bef).getSlot() == EquipmentSlot.CHEST;
        }
    }, 
    ARMOR_HEAD {
        @Override
        public boolean canEnchant(final Item bef) {
            return bef instanceof ArmorItem && ((ArmorItem)bef).getSlot() == EquipmentSlot.HEAD;
        }
    }, 
    WEAPON {
        @Override
        public boolean canEnchant(final Item bef) {
            return bef instanceof SwordItem;
        }
    }, 
    DIGGER {
        @Override
        public boolean canEnchant(final Item bef) {
            return bef instanceof DiggerItem;
        }
    }, 
    FISHING_ROD {
        @Override
        public boolean canEnchant(final Item bef) {
            return bef instanceof FishingRodItem;
        }
    }, 
    TRIDENT {
        @Override
        public boolean canEnchant(final Item bef) {
            return bef instanceof TridentItem;
        }
    }, 
    BREAKABLE {
        @Override
        public boolean canEnchant(final Item bef) {
            return bef.canBeDepleted();
        }
    }, 
    BOW {
        @Override
        public boolean canEnchant(final Item bef) {
            return bef instanceof BowItem;
        }
    }, 
    WEARABLE {
        @Override
        public boolean canEnchant(final Item bef) {
            final Block bpe3 = Block.byItem(bef);
            return bef instanceof ArmorItem || bef instanceof ElytraItem || bpe3 instanceof AbstractSkullBlock || bpe3 instanceof CarvedPumpkinBlock;
        }
    }, 
    CROSSBOW {
        @Override
        public boolean canEnchant(final Item bef) {
            return bef instanceof CrossbowItem;
        }
    };
    
    public abstract boolean canEnchant(final Item bef);
}
