package net.minecraft.world.item.enchantment;

import java.util.AbstractList;
import java.util.Collection;
import net.minecraft.Util;
import net.minecraft.util.WeighedRandom;
import net.minecraft.world.level.ItemLike;
import net.minecraft.world.item.Item;
import java.util.Random;
import javax.annotation.Nullable;
import java.util.List;
import com.google.common.collect.Lists;
import net.minecraft.world.entity.EquipmentSlot;
import net.minecraft.world.entity.player.Player;
import net.minecraft.world.entity.Entity;
import net.minecraft.world.entity.LivingEntity;
import org.apache.commons.lang3.mutable.MutableFloat;
import net.minecraft.world.entity.MobType;
import org.apache.commons.lang3.mutable.MutableInt;
import net.minecraft.world.damagesource.DamageSource;
import java.util.Iterator;
import net.minecraft.nbt.Tag;
import com.google.common.collect.Maps;
import net.minecraft.world.item.EnchantedBookItem;
import net.minecraft.world.item.Items;
import java.util.Map;
import net.minecraft.nbt.CompoundTag;
import net.minecraft.nbt.ListTag;
import net.minecraft.util.Mth;
import net.minecraft.resources.ResourceLocation;
import net.minecraft.core.Registry;
import net.minecraft.world.item.ItemStack;

public class EnchantmentHelper {
    public static int getItemEnchantmentLevel(final Enchantment bhu, final ItemStack bek) {
        if (bek.isEmpty()) {
            return 0;
        }
        final ResourceLocation sm3 = Registry.ENCHANTMENT.getKey(bhu);
        final ListTag jz4 = bek.getEnchantmentTags();
        for (int integer5 = 0; integer5 < jz4.size(); ++integer5) {
            final CompoundTag jt6 = jz4.getCompound(integer5);
            final ResourceLocation sm4 = ResourceLocation.tryParse(jt6.getString("id"));
            if (sm4 != null && sm4.equals(sm3)) {
                return Mth.clamp(jt6.getInt("lvl"), 0, 255);
            }
        }
        return 0;
    }
    
    public static Map<Enchantment, Integer> getEnchantments(final ItemStack bek) {
        final ListTag jz2 = (bek.getItem() == Items.ENCHANTED_BOOK) ? EnchantedBookItem.getEnchantments(bek) : bek.getEnchantmentTags();
        return deserializeEnchantments(jz2);
    }
    
    public static Map<Enchantment, Integer> deserializeEnchantments(final ListTag jz) {
        final Map<Enchantment, Integer> map2 = Maps.newLinkedHashMap();
        for (int integer3 = 0; integer3 < jz.size(); ++integer3) {
            final CompoundTag jt4 = jz.getCompound(integer3);
            final Integer n;
            Registry.ENCHANTMENT.getOptional(ResourceLocation.tryParse(jt4.getString("id"))).ifPresent(bhu -> n = map2.put(bhu, jt4.getInt("lvl")));
        }
        return map2;
    }
    
    public static void setEnchantments(final Map<Enchantment, Integer> map, final ItemStack bek) {
        final ListTag jz3 = new ListTag();
        for (final Map.Entry<Enchantment, Integer> entry5 : map.entrySet()) {
            final Enchantment bhu6 = entry5.getKey();
            if (bhu6 == null) {
                continue;
            }
            final int integer7 = entry5.getValue();
            final CompoundTag jt8 = new CompoundTag();
            jt8.putString("id", String.valueOf(Registry.ENCHANTMENT.getKey(bhu6)));
            jt8.putShort("lvl", (short)integer7);
            ((AbstractList<CompoundTag>)jz3).add(jt8);
            if (bek.getItem() != Items.ENCHANTED_BOOK) {
                continue;
            }
            EnchantedBookItem.addEnchantment(bek, new EnchantmentInstance(bhu6, integer7));
        }
        if (jz3.isEmpty()) {
            bek.removeTagKey("Enchantments");
        }
        else if (bek.getItem() != Items.ENCHANTED_BOOK) {
            bek.addTagElement("Enchantments", jz3);
        }
    }
    
    private static void runIterationOnItem(final EnchantmentVisitor a, final ItemStack bek) {
        if (bek.isEmpty()) {
            return;
        }
        final ListTag jz3 = bek.getEnchantmentTags();
        for (int integer4 = 0; integer4 < jz3.size(); ++integer4) {
            final String string5 = jz3.getCompound(integer4).getString("id");
            final int integer5 = jz3.getCompound(integer4).getInt("lvl");
            Registry.ENCHANTMENT.getOptional(ResourceLocation.tryParse(string5)).ifPresent(bhu -> a.accept(bhu, integer5));
        }
    }
    
    private static void runIterationOnInventory(final EnchantmentVisitor a, final Iterable<ItemStack> iterable) {
        for (final ItemStack bek4 : iterable) {
            runIterationOnItem(a, bek4);
        }
    }
    
    public static int getDamageProtection(final Iterable<ItemStack> iterable, final DamageSource ajw) {
        final MutableInt mutableInt3 = new MutableInt();
        runIterationOnInventory((bhu, integer) -> mutableInt3.add(bhu.getDamageProtection(integer, ajw)), iterable);
        return mutableInt3.intValue();
    }
    
    public static float getDamageBonus(final ItemStack bek, final MobType ala) {
        final MutableFloat mutableFloat3 = new MutableFloat();
        runIterationOnItem((bhu, integer) -> mutableFloat3.add(bhu.getDamageBonus(integer, ala)), bek);
        return mutableFloat3.floatValue();
    }
    
    public static float getSweepingDamageRatio(final LivingEntity akw) {
        final int integer2 = getEnchantmentLevel(Enchantments.SWEEPING_EDGE, akw);
        if (integer2 > 0) {
            return SweepingEdgeEnchantment.getSweepingDamageRatio(integer2);
        }
        return 0.0f;
    }
    
    public static void doPostHurtEffects(final LivingEntity akw, final Entity akn) {
        final EnchantmentVisitor a3 = (bhu, integer) -> bhu.doPostHurt(akw, akn, integer);
        if (akw != null) {
            runIterationOnInventory(a3, akw.getAllSlots());
        }
        if (akn instanceof Player) {
            runIterationOnItem(a3, akw.getMainHandItem());
        }
    }
    
    public static void doPostDamageEffects(final LivingEntity akw, final Entity akn) {
        final EnchantmentVisitor a3 = (bhu, integer) -> bhu.doPostAttack(akw, akn, integer);
        if (akw != null) {
            runIterationOnInventory(a3, akw.getAllSlots());
        }
        if (akw instanceof Player) {
            runIterationOnItem(a3, akw.getMainHandItem());
        }
    }
    
    public static int getEnchantmentLevel(final Enchantment bhu, final LivingEntity akw) {
        final Iterable<ItemStack> iterable3 = bhu.getSlotItems(akw).values();
        if (iterable3 == null) {
            return 0;
        }
        int integer4 = 0;
        for (final ItemStack bek6 : iterable3) {
            final int integer5 = getItemEnchantmentLevel(bhu, bek6);
            if (integer5 > integer4) {
                integer4 = integer5;
            }
        }
        return integer4;
    }
    
    public static int getKnockbackBonus(final LivingEntity akw) {
        return getEnchantmentLevel(Enchantments.KNOCKBACK, akw);
    }
    
    public static int getFireAspect(final LivingEntity akw) {
        return getEnchantmentLevel(Enchantments.FIRE_ASPECT, akw);
    }
    
    public static int getRespiration(final LivingEntity akw) {
        return getEnchantmentLevel(Enchantments.RESPIRATION, akw);
    }
    
    public static int getDepthStrider(final LivingEntity akw) {
        return getEnchantmentLevel(Enchantments.DEPTH_STRIDER, akw);
    }
    
    public static int getBlockEfficiency(final LivingEntity akw) {
        return getEnchantmentLevel(Enchantments.BLOCK_EFFICIENCY, akw);
    }
    
    public static int getFishingLuckBonus(final ItemStack bek) {
        return getItemEnchantmentLevel(Enchantments.FISHING_LUCK, bek);
    }
    
    public static int getFishingSpeedBonus(final ItemStack bek) {
        return getItemEnchantmentLevel(Enchantments.FISHING_SPEED, bek);
    }
    
    public static int getMobLooting(final LivingEntity akw) {
        return getEnchantmentLevel(Enchantments.MOB_LOOTING, akw);
    }
    
    public static boolean hasAquaAffinity(final LivingEntity akw) {
        return getEnchantmentLevel(Enchantments.AQUA_AFFINITY, akw) > 0;
    }
    
    public static boolean hasFrostWalker(final LivingEntity akw) {
        return getEnchantmentLevel(Enchantments.FROST_WALKER, akw) > 0;
    }
    
    public static boolean hasBindingCurse(final ItemStack bek) {
        return getItemEnchantmentLevel(Enchantments.BINDING_CURSE, bek) > 0;
    }
    
    public static boolean hasVanishingCurse(final ItemStack bek) {
        return getItemEnchantmentLevel(Enchantments.VANISHING_CURSE, bek) > 0;
    }
    
    public static int getLoyalty(final ItemStack bek) {
        return getItemEnchantmentLevel(Enchantments.LOYALTY, bek);
    }
    
    public static int getRiptide(final ItemStack bek) {
        return getItemEnchantmentLevel(Enchantments.RIPTIDE, bek);
    }
    
    public static boolean hasChanneling(final ItemStack bek) {
        return getItemEnchantmentLevel(Enchantments.CHANNELING, bek) > 0;
    }
    
    @Nullable
    public static Map.Entry<EquipmentSlot, ItemStack> getRandomItemWith(final Enchantment bhu, final LivingEntity akw) {
        final Map<EquipmentSlot, ItemStack> map3 = bhu.getSlotItems(akw);
        if (map3.isEmpty()) {
            return null;
        }
        final List<Map.Entry<EquipmentSlot, ItemStack>> list4 = Lists.newArrayList();
        for (final Map.Entry<EquipmentSlot, ItemStack> entry6 : map3.entrySet()) {
            final ItemStack bek7 = entry6.getValue();
            if (!bek7.isEmpty() && getItemEnchantmentLevel(bhu, bek7) > 0) {
                list4.add(entry6);
            }
        }
        return list4.isEmpty() ? null : list4.get(akw.getRandom().nextInt(list4.size()));
    }
    
    public static int getEnchantmentCost(final Random random, final int integer2, int integer3, final ItemStack bek) {
        final Item bef5 = bek.getItem();
        final int integer4 = bef5.getEnchantmentValue();
        if (integer4 <= 0) {
            return 0;
        }
        if (integer3 > 15) {
            integer3 = 15;
        }
        final int integer5 = random.nextInt(8) + 1 + (integer3 >> 1) + random.nextInt(integer3 + 1);
        if (integer2 == 0) {
            return Math.max(integer5 / 3, 1);
        }
        if (integer2 == 1) {
            return integer5 * 2 / 3 + 1;
        }
        return Math.max(integer5, integer3 * 2);
    }
    
    public static ItemStack enchantItem(final Random random, ItemStack bek, final int integer, final boolean boolean4) {
        final List<EnchantmentInstance> list5 = selectEnchantment(random, bek, integer, boolean4);
        final boolean boolean5 = bek.getItem() == Items.BOOK;
        if (boolean5) {
            bek = new ItemStack(Items.ENCHANTED_BOOK);
        }
        for (final EnchantmentInstance bhx8 : list5) {
            if (boolean5) {
                EnchantedBookItem.addEnchantment(bek, bhx8);
            }
            else {
                bek.enchant(bhx8.enchantment, bhx8.level);
            }
        }
        return bek;
    }
    
    public static List<EnchantmentInstance> selectEnchantment(final Random random, final ItemStack bek, int integer, final boolean boolean4) {
        final List<EnchantmentInstance> list5 = Lists.newArrayList();
        final Item bef6 = bek.getItem();
        final int integer2 = bef6.getEnchantmentValue();
        if (integer2 <= 0) {
            return list5;
        }
        integer += 1 + random.nextInt(integer2 / 4 + 1) + random.nextInt(integer2 / 4 + 1);
        final float float8 = (random.nextFloat() + random.nextFloat() - 1.0f) * 0.15f;
        integer = Mth.clamp(Math.round(integer + integer * float8), 1, Integer.MAX_VALUE);
        final List<EnchantmentInstance> list6 = getAvailableEnchantmentResults(integer, bek, boolean4);
        if (!list6.isEmpty()) {
            list5.add(WeighedRandom.<EnchantmentInstance>getRandomItem(random, list6));
            while (random.nextInt(50) <= integer) {
                filterCompatibleEnchantments(list6, Util.<EnchantmentInstance>lastOf(list5));
                if (list6.isEmpty()) {
                    break;
                }
                list5.add(WeighedRandom.<EnchantmentInstance>getRandomItem(random, list6));
                integer /= 2;
            }
        }
        return list5;
    }
    
    public static void filterCompatibleEnchantments(final List<EnchantmentInstance> list, final EnchantmentInstance bhx) {
        final Iterator<EnchantmentInstance> iterator3 = list.iterator();
        while (iterator3.hasNext()) {
            if (!bhx.enchantment.isCompatibleWith(iterator3.next().enchantment)) {
                iterator3.remove();
            }
        }
    }
    
    public static boolean isEnchantmentCompatible(final Collection<Enchantment> collection, final Enchantment bhu) {
        for (final Enchantment bhu2 : collection) {
            if (!bhu2.isCompatibleWith(bhu)) {
                return false;
            }
        }
        return true;
    }
    
    public static List<EnchantmentInstance> getAvailableEnchantmentResults(final int integer, final ItemStack bek, final boolean boolean3) {
        final List<EnchantmentInstance> list4 = Lists.newArrayList();
        final Item bef5 = bek.getItem();
        final boolean boolean4 = bek.getItem() == Items.BOOK;
        for (final Enchantment bhu8 : Registry.ENCHANTMENT) {
            if (bhu8.isTreasureOnly() && !boolean3) {
                continue;
            }
            if (!bhu8.category.canEnchant(bef5) && !boolean4) {
                continue;
            }
            for (int integer2 = bhu8.getMaxLevel(); integer2 > bhu8.getMinLevel() - 1; --integer2) {
                if (integer >= bhu8.getMinCost(integer2) && integer <= bhu8.getMaxCost(integer2)) {
                    list4.add(new EnchantmentInstance(bhu8, integer2));
                    break;
                }
            }
        }
        return list4;
    }
    
    @FunctionalInterface
    interface EnchantmentVisitor {
        void accept(final Enchantment bhu, final int integer);
    }
}
