package net.minecraft.world.item.enchantment;

import java.util.Iterator;
import net.minecraft.world.level.block.state.BlockState;
import net.minecraft.util.Mth;
import net.minecraft.world.phys.shapes.CollisionContext;
import net.minecraft.world.level.LevelReader;
import net.minecraft.world.level.block.state.properties.Property;
import net.minecraft.world.level.block.LiquidBlock;
import net.minecraft.world.level.material.Material;
import net.minecraft.core.Position;
import net.minecraft.world.level.block.Blocks;
import net.minecraft.core.BlockPos;
import net.minecraft.world.level.Level;
import net.minecraft.world.entity.LivingEntity;
import net.minecraft.world.entity.EquipmentSlot;

public class FrostWalkerEnchantment extends Enchantment {
    public FrostWalkerEnchantment(final Rarity a, final EquipmentSlot... arr) {
        super(a, EnchantmentCategory.ARMOR_FEET, arr);
    }
    
    @Override
    public int getMinCost(final int integer) {
        return integer * 10;
    }
    
    @Override
    public int getMaxCost(final int integer) {
        return this.getMinCost(integer) + 15;
    }
    
    @Override
    public boolean isTreasureOnly() {
        return true;
    }
    
    @Override
    public int getMaxLevel() {
        return 2;
    }
    
    public static void onEntityMoved(final LivingEntity akw, final Level bjt, final BlockPos fk, final int integer) {
        if (!akw.onGround) {
            return;
        }
        final BlockState byg5 = Blocks.FROSTED_ICE.defaultBlockState();
        final float float6 = (float)Math.min(16, 2 + integer);
        final BlockPos.MutableBlockPos a7 = new BlockPos.MutableBlockPos();
        for (final BlockPos fk2 : BlockPos.betweenClosed(fk.offset(-float6, -1.0, -float6), fk.offset(float6, -1.0, float6))) {
            if (fk2.closerThan(akw.position(), float6)) {
                a7.set(fk2.getX(), fk2.getY() + 1, fk2.getZ());
                final BlockState byg6 = bjt.getBlockState(a7);
                if (!byg6.isAir()) {
                    continue;
                }
                final BlockState byg7 = bjt.getBlockState(fk2);
                if (byg7.getMaterial() != Material.WATER || byg7.<Integer>getValue((Property<Integer>)LiquidBlock.LEVEL) != 0 || !byg5.canSurvive(bjt, fk2) || !bjt.isUnobstructed(byg5, fk2, CollisionContext.empty())) {
                    continue;
                }
                bjt.setBlockAndUpdate(fk2, byg5);
                bjt.getBlockTicks().scheduleTick(fk2, Blocks.FROSTED_ICE, Mth.nextInt(akw.getRandom(), 60, 120));
            }
        }
    }
    
    public boolean checkCompatibility(final Enchantment bhu) {
        return super.checkCompatibility(bhu) && bhu != Enchantments.DEPTH_STRIDER;
    }
}
