package net.minecraft.world.item.trading;

import java.util.AbstractList;
import net.minecraft.nbt.Tag;
import net.minecraft.network.FriendlyByteBuf;
import javax.annotation.Nullable;
import net.minecraft.world.item.ItemStack;
import net.minecraft.nbt.ListTag;
import net.minecraft.nbt.CompoundTag;
import java.util.ArrayList;

public class MerchantOffers extends ArrayList<MerchantOffer> {
    public MerchantOffers() {
    }
    
    public MerchantOffers(final CompoundTag jt) {
        final ListTag jz3 = jt.getList("Recipes", 10);
        for (int integer4 = 0; integer4 < jz3.size(); ++integer4) {
            this.add(new MerchantOffer(jz3.getCompound(integer4)));
        }
    }
    
    @Nullable
    public MerchantOffer getRecipeFor(final ItemStack bek1, final ItemStack bek2, final int integer) {
        if (integer <= 0 || integer >= this.size()) {
            for (int integer2 = 0; integer2 < this.size(); ++integer2) {
                final MerchantOffer biw6 = this.get(integer2);
                if (biw6.satisfiedBy(bek1, bek2)) {
                    return biw6;
                }
            }
            return null;
        }
        final MerchantOffer biw7 = this.get(integer);
        if (biw7.satisfiedBy(bek1, bek2)) {
            return biw7;
        }
        return null;
    }
    
    public void writeToStream(final FriendlyByteBuf kv) {
        kv.writeByte((byte)(this.size() & 0xFF));
        for (int integer3 = 0; integer3 < this.size(); ++integer3) {
            final MerchantOffer biw4 = this.get(integer3);
            kv.writeItem(biw4.getBaseCostA());
            kv.writeItem(biw4.getResult());
            final ItemStack bek5 = biw4.getCostB();
            kv.writeBoolean(!bek5.isEmpty());
            if (!bek5.isEmpty()) {
                kv.writeItem(bek5);
            }
            kv.writeBoolean(biw4.isOutOfStock());
            kv.writeInt(biw4.getUses());
            kv.writeInt(biw4.getMaxUses());
            kv.writeInt(biw4.getXp());
            kv.writeInt(biw4.getSpecialPriceDiff());
            kv.writeFloat(biw4.getPriceMultiplier());
            kv.writeInt(biw4.getDemand());
        }
    }
    
    public static MerchantOffers createFromStream(final FriendlyByteBuf kv) {
        final MerchantOffers bix2 = new MerchantOffers();
        for (int integer3 = kv.readByte() & 0xFF, integer4 = 0; integer4 < integer3; ++integer4) {
            final ItemStack bek5 = kv.readItem();
            final ItemStack bek6 = kv.readItem();
            ItemStack bek7 = ItemStack.EMPTY;
            if (kv.readBoolean()) {
                bek7 = kv.readItem();
            }
            final boolean boolean8 = kv.readBoolean();
            final int integer5 = kv.readInt();
            final int integer6 = kv.readInt();
            final int integer7 = kv.readInt();
            final int integer8 = kv.readInt();
            final float float13 = kv.readFloat();
            final int integer9 = kv.readInt();
            final MerchantOffer biw15 = new MerchantOffer(bek5, bek7, bek6, integer5, integer6, integer7, float13, integer9);
            if (boolean8) {
                biw15.setToOutOfStock();
            }
            biw15.setSpecialPriceDiff(integer8);
            bix2.add(biw15);
        }
        return bix2;
    }
    
    public CompoundTag createTag() {
        final CompoundTag jt2 = new CompoundTag();
        final ListTag jz3 = new ListTag();
        for (int integer4 = 0; integer4 < this.size(); ++integer4) {
            final MerchantOffer biw5 = this.get(integer4);
            ((AbstractList<CompoundTag>)jz3).add(biw5.createTag());
        }
        jt2.put("Recipes", jz3);
        return jt2;
    }
}
