package net.minecraft.world.level.block;

import net.minecraft.world.level.block.state.AbstractStateHolder;
import net.minecraft.world.phys.shapes.BooleanOp;
import net.minecraft.world.phys.shapes.Shapes;
import net.minecraft.world.level.block.state.properties.BlockStateProperties;
import net.minecraft.world.level.pathfinder.PathComputationType;
import net.minecraft.world.level.block.state.StateDefinition;
import net.minecraft.util.Mth;
import net.minecraft.world.item.Item;
import net.minecraft.world.item.BlockItem;
import net.minecraft.world.level.block.entity.BannerBlockEntity;
import net.minecraft.world.item.BannerItem;
import net.minecraft.world.item.DyeableLeatherItem;
import net.minecraft.world.inventory.AbstractContainerMenu;
import net.minecraft.server.level.ServerPlayer;
import net.minecraft.world.item.alchemy.PotionUtils;
import net.minecraft.world.item.alchemy.Potions;
import net.minecraft.sounds.SoundSource;
import net.minecraft.sounds.SoundEvents;
import net.minecraft.stats.Stats;
import net.minecraft.world.level.ItemLike;
import net.minecraft.world.item.ItemStack;
import net.minecraft.world.item.Items;
import net.minecraft.world.InteractionResult;
import net.minecraft.world.phys.BlockHitResult;
import net.minecraft.world.InteractionHand;
import net.minecraft.world.entity.player.Player;
import net.minecraft.world.entity.Entity;
import net.minecraft.world.level.Level;
import net.minecraft.world.phys.shapes.CollisionContext;
import net.minecraft.core.BlockPos;
import net.minecraft.world.level.BlockGetter;
import net.minecraft.world.level.block.state.properties.Property;
import net.minecraft.world.level.block.state.BlockState;
import net.minecraft.world.phys.shapes.VoxelShape;
import net.minecraft.world.level.block.state.properties.IntegerProperty;

public class CauldronBlock extends Block {
    public static final IntegerProperty LEVEL;
    private static final VoxelShape INSIDE;
    protected static final VoxelShape SHAPE;
    
    public CauldronBlock(final Properties c) {
        super(c);
        this.registerDefaultState(((AbstractStateHolder<O, BlockState>)this.stateDefinition.any()).<Comparable, Integer>setValue((Property<Comparable>)CauldronBlock.LEVEL, 0));
    }
    
    @Override
    public VoxelShape getShape(final BlockState byg, final BlockGetter bjd, final BlockPos fk, final CollisionContext cvn) {
        return CauldronBlock.SHAPE;
    }
    
    @Override
    public VoxelShape getInteractionShape(final BlockState byg, final BlockGetter bjd, final BlockPos fk) {
        return CauldronBlock.INSIDE;
    }
    
    @Override
    public void entityInside(final BlockState byg, final Level bjt, final BlockPos fk, final Entity akn) {
        final int integer6 = byg.<Integer>getValue((Property<Integer>)CauldronBlock.LEVEL);
        final float float7 = fk.getY() + (6.0f + 3 * integer6) / 16.0f;
        if (!bjt.isClientSide && akn.isOnFire() && integer6 > 0 && akn.getY() <= float7) {
            akn.clearFire();
            this.setWaterLevel(bjt, fk, byg, integer6 - 1);
        }
    }
    
    @Override
    public InteractionResult use(final BlockState byg, final Level bjt, final BlockPos fk, final Player ayg, final InteractionHand ajh, final BlockHitResult cvd) {
        final ItemStack bek8 = ayg.getItemInHand(ajh);
        if (bek8.isEmpty()) {
            return InteractionResult.PASS;
        }
        final int integer9 = byg.<Integer>getValue((Property<Integer>)CauldronBlock.LEVEL);
        final Item bef10 = bek8.getItem();
        if (bef10 == Items.WATER_BUCKET) {
            if (integer9 < 3 && !bjt.isClientSide) {
                if (!ayg.abilities.instabuild) {
                    ayg.setItemInHand(ajh, new ItemStack(Items.BUCKET));
                }
                ayg.awardStat(Stats.FILL_CAULDRON);
                this.setWaterLevel(bjt, fk, byg, 3);
                bjt.playSound(null, fk, SoundEvents.BUCKET_EMPTY, SoundSource.BLOCKS, 1.0f, 1.0f);
            }
            return InteractionResult.SUCCESS;
        }
        if (bef10 == Items.BUCKET) {
            if (integer9 == 3 && !bjt.isClientSide) {
                if (!ayg.abilities.instabuild) {
                    bek8.shrink(1);
                    if (bek8.isEmpty()) {
                        ayg.setItemInHand(ajh, new ItemStack(Items.WATER_BUCKET));
                    }
                    else if (!ayg.inventory.add(new ItemStack(Items.WATER_BUCKET))) {
                        ayg.drop(new ItemStack(Items.WATER_BUCKET), false);
                    }
                }
                ayg.awardStat(Stats.USE_CAULDRON);
                this.setWaterLevel(bjt, fk, byg, 0);
                bjt.playSound(null, fk, SoundEvents.BUCKET_FILL, SoundSource.BLOCKS, 1.0f, 1.0f);
            }
            return InteractionResult.SUCCESS;
        }
        if (bef10 == Items.GLASS_BOTTLE) {
            if (integer9 > 0 && !bjt.isClientSide) {
                if (!ayg.abilities.instabuild) {
                    final ItemStack bek9 = PotionUtils.setPotion(new ItemStack(Items.POTION), Potions.WATER);
                    ayg.awardStat(Stats.USE_CAULDRON);
                    bek8.shrink(1);
                    if (bek8.isEmpty()) {
                        ayg.setItemInHand(ajh, bek9);
                    }
                    else if (!ayg.inventory.add(bek9)) {
                        ayg.drop(bek9, false);
                    }
                    else if (ayg instanceof ServerPlayer) {
                        ((ServerPlayer)ayg).refreshContainer(ayg.inventoryMenu);
                    }
                }
                bjt.playSound(null, fk, SoundEvents.BOTTLE_FILL, SoundSource.BLOCKS, 1.0f, 1.0f);
                this.setWaterLevel(bjt, fk, byg, integer9 - 1);
            }
            return InteractionResult.SUCCESS;
        }
        if (bef10 == Items.POTION && PotionUtils.getPotion(bek8) == Potions.WATER) {
            if (integer9 < 3 && !bjt.isClientSide) {
                if (!ayg.abilities.instabuild) {
                    final ItemStack bek9 = new ItemStack(Items.GLASS_BOTTLE);
                    ayg.awardStat(Stats.USE_CAULDRON);
                    ayg.setItemInHand(ajh, bek9);
                    if (ayg instanceof ServerPlayer) {
                        ((ServerPlayer)ayg).refreshContainer(ayg.inventoryMenu);
                    }
                }
                bjt.playSound(null, fk, SoundEvents.BOTTLE_EMPTY, SoundSource.BLOCKS, 1.0f, 1.0f);
                this.setWaterLevel(bjt, fk, byg, integer9 + 1);
            }
            return InteractionResult.SUCCESS;
        }
        if (integer9 > 0 && bef10 instanceof DyeableLeatherItem) {
            final DyeableLeatherItem bdk11 = (DyeableLeatherItem)bef10;
            if (bdk11.hasCustomColor(bek8) && !bjt.isClientSide) {
                bdk11.clearColor(bek8);
                this.setWaterLevel(bjt, fk, byg, integer9 - 1);
                ayg.awardStat(Stats.CLEAN_ARMOR);
                return InteractionResult.SUCCESS;
            }
        }
        if (integer9 > 0 && bef10 instanceof BannerItem) {
            if (BannerBlockEntity.getPatternCount(bek8) > 0 && !bjt.isClientSide) {
                final ItemStack bek9 = bek8.copy();
                bek9.setCount(1);
                BannerBlockEntity.removeLastPattern(bek9);
                ayg.awardStat(Stats.CLEAN_BANNER);
                if (!ayg.abilities.instabuild) {
                    bek8.shrink(1);
                    this.setWaterLevel(bjt, fk, byg, integer9 - 1);
                }
                if (bek8.isEmpty()) {
                    ayg.setItemInHand(ajh, bek9);
                }
                else if (!ayg.inventory.add(bek9)) {
                    ayg.drop(bek9, false);
                }
                else if (ayg instanceof ServerPlayer) {
                    ((ServerPlayer)ayg).refreshContainer(ayg.inventoryMenu);
                }
            }
            return InteractionResult.SUCCESS;
        }
        if (integer9 <= 0 || !(bef10 instanceof BlockItem)) {
            return InteractionResult.PASS;
        }
        final Block bpe11 = ((BlockItem)bef10).getBlock();
        if (bpe11 instanceof ShulkerBoxBlock && !bjt.isClientSide()) {
            final ItemStack bek10 = new ItemStack(Blocks.SHULKER_BOX, 1);
            if (bek8.hasTag()) {
                bek10.setTag(bek8.getTag().copy());
            }
            ayg.setItemInHand(ajh, bek10);
            this.setWaterLevel(bjt, fk, byg, integer9 - 1);
            ayg.awardStat(Stats.CLEAN_SHULKER_BOX);
            return InteractionResult.SUCCESS;
        }
        return InteractionResult.CONSUME;
    }
    
    public void setWaterLevel(final Level bjt, final BlockPos fk, final BlockState byg, final int integer) {
        bjt.setBlock(fk, ((AbstractStateHolder<O, BlockState>)byg).<Comparable, Integer>setValue((Property<Comparable>)CauldronBlock.LEVEL, Mth.clamp(integer, 0, 3)), 2);
        bjt.updateNeighbourForOutputSignal(fk, this);
    }
    
    @Override
    public void handleRain(final Level bjt, final BlockPos fk) {
        if (bjt.random.nextInt(20) != 1) {
            return;
        }
        final float float4 = bjt.getBiome(fk).getTemperature(fk);
        if (float4 < 0.15f) {
            return;
        }
        final BlockState byg5 = bjt.getBlockState(fk);
        if (byg5.<Integer>getValue((Property<Integer>)CauldronBlock.LEVEL) < 3) {
            bjt.setBlock(fk, ((AbstractStateHolder<O, BlockState>)byg5).<Comparable>cycle((Property<Comparable>)CauldronBlock.LEVEL), 2);
        }
    }
    
    @Override
    public boolean hasAnalogOutputSignal(final BlockState byg) {
        return true;
    }
    
    @Override
    public int getAnalogOutputSignal(final BlockState byg, final Level bjt, final BlockPos fk) {
        return byg.<Integer>getValue((Property<Integer>)CauldronBlock.LEVEL);
    }
    
    @Override
    protected void createBlockStateDefinition(final StateDefinition.Builder<Block, BlockState> a) {
        a.add(CauldronBlock.LEVEL);
    }
    
    @Override
    public boolean isPathfindable(final BlockState byg, final BlockGetter bjd, final BlockPos fk, final PathComputationType cqo) {
        return false;
    }
    
    static {
        LEVEL = BlockStateProperties.LEVEL_CAULDRON;
        INSIDE = Block.box(2.0, 4.0, 2.0, 14.0, 16.0, 14.0);
        SHAPE = Shapes.join(Shapes.block(), Shapes.or(Block.box(0.0, 0.0, 4.0, 16.0, 3.0, 12.0), Block.box(4.0, 0.0, 0.0, 12.0, 3.0, 16.0), Block.box(2.0, 0.0, 2.0, 14.0, 3.0, 14.0), CauldronBlock.INSIDE), BooleanOp.ONLY_FIRST);
    }
}
