package net.minecraft.world.level.block;

import net.minecraft.world.level.block.state.AbstractStateHolder;
import net.minecraft.world.level.block.state.properties.BlockStateProperties;
import net.minecraft.world.level.block.state.StateDefinition;
import java.util.Random;
import net.minecraft.server.level.ServerLevel;
import net.minecraft.world.level.TickPriority;
import net.minecraft.sounds.SoundSource;
import net.minecraft.sounds.SoundEvents;
import net.minecraft.world.InteractionResult;
import net.minecraft.world.phys.BlockHitResult;
import net.minecraft.world.InteractionHand;
import net.minecraft.world.entity.player.Player;
import javax.annotation.Nullable;
import java.util.List;
import net.minecraft.world.phys.AABB;
import net.minecraft.world.entity.decoration.ItemFrame;
import net.minecraft.world.level.LevelReader;
import net.minecraft.world.level.Level;
import net.minecraft.world.level.block.entity.BlockEntity;
import net.minecraft.world.level.block.entity.ComparatorBlockEntity;
import net.minecraft.core.BlockPos;
import net.minecraft.world.level.BlockGetter;
import net.minecraft.world.level.block.state.properties.Property;
import net.minecraft.core.Direction;
import net.minecraft.world.level.block.state.BlockState;
import net.minecraft.world.level.block.state.properties.ComparatorMode;
import net.minecraft.world.level.block.state.properties.EnumProperty;

public class ComparatorBlock extends DiodeBlock implements EntityBlock {
    public static final EnumProperty<ComparatorMode> MODE;
    
    public ComparatorBlock(final Properties c) {
        super(c);
        this.registerDefaultState(((((AbstractStateHolder<O, BlockState>)this.stateDefinition.any()).setValue((Property<Comparable>)ComparatorBlock.FACING, Direction.NORTH)).setValue((Property<Comparable>)ComparatorBlock.POWERED, false)).<ComparatorMode, ComparatorMode>setValue(ComparatorBlock.MODE, ComparatorMode.COMPARE));
    }
    
    @Override
    protected int getDelay(final BlockState byg) {
        return 2;
    }
    
    @Override
    protected int getOutputSignal(final BlockGetter bjd, final BlockPos fk, final BlockState byg) {
        final BlockEntity bwi5 = bjd.getBlockEntity(fk);
        if (bwi5 instanceof ComparatorBlockEntity) {
            return ((ComparatorBlockEntity)bwi5).getOutputSignal();
        }
        return 0;
    }
    
    private int calculateOutputSignal(final Level bjt, final BlockPos fk, final BlockState byg) {
        if (byg.<ComparatorMode>getValue(ComparatorBlock.MODE) == ComparatorMode.SUBTRACT) {
            return Math.max(this.getInputSignal(bjt, fk, byg) - this.getAlternateSignal(bjt, fk, byg), 0);
        }
        return this.getInputSignal(bjt, fk, byg);
    }
    
    @Override
    protected boolean shouldTurnOn(final Level bjt, final BlockPos fk, final BlockState byg) {
        final int integer5 = this.getInputSignal(bjt, fk, byg);
        if (integer5 == 0) {
            return false;
        }
        final int integer6 = this.getAlternateSignal(bjt, fk, byg);
        return integer5 > integer6 || (integer5 == integer6 && byg.<ComparatorMode>getValue(ComparatorBlock.MODE) == ComparatorMode.COMPARE);
    }
    
    @Override
    protected int getInputSignal(final Level bjt, final BlockPos fk, final BlockState byg) {
        int integer5 = super.getInputSignal(bjt, fk, byg);
        final Direction fp6 = byg.<Direction>getValue((Property<Direction>)ComparatorBlock.FACING);
        BlockPos fk2 = fk.relative(fp6);
        BlockState byg2 = bjt.getBlockState(fk2);
        if (byg2.hasAnalogOutputSignal()) {
            integer5 = byg2.getAnalogOutputSignal(bjt, fk2);
        }
        else if (integer5 < 15 && byg2.isRedstoneConductor(bjt, fk2)) {
            fk2 = fk2.relative(fp6);
            byg2 = bjt.getBlockState(fk2);
            if (byg2.hasAnalogOutputSignal()) {
                integer5 = byg2.getAnalogOutputSignal(bjt, fk2);
            }
            else if (byg2.isAir()) {
                final ItemFrame avo9 = this.getItemFrame(bjt, fp6, fk2);
                if (avo9 != null) {
                    integer5 = avo9.getAnalogOutput();
                }
            }
        }
        return integer5;
    }
    
    @Nullable
    private ItemFrame getItemFrame(final Level bjt, final Direction fp, final BlockPos fk) {
        final List<ItemFrame> list5 = bjt.<ItemFrame>getEntitiesOfClass(ItemFrame.class, new AABB(fk.getX(), fk.getY(), fk.getZ(), fk.getX() + 1, fk.getY() + 1, fk.getZ() + 1), avo -> avo != null && avo.getDirection() == fp);
        if (list5.size() == 1) {
            return list5.get(0);
        }
        return null;
    }
    
    @Override
    public InteractionResult use(BlockState byg, final Level bjt, final BlockPos fk, final Player ayg, final InteractionHand ajh, final BlockHitResult cvd) {
        if (!ayg.abilities.mayBuild) {
            return InteractionResult.PASS;
        }
        byg = ((AbstractStateHolder<O, BlockState>)byg).<ComparatorMode>cycle(ComparatorBlock.MODE);
        final float float8 = (byg.<ComparatorMode>getValue(ComparatorBlock.MODE) == ComparatorMode.SUBTRACT) ? 0.55f : 0.5f;
        bjt.playSound(ayg, fk, SoundEvents.COMPARATOR_CLICK, SoundSource.BLOCKS, 0.3f, float8);
        bjt.setBlock(fk, byg, 2);
        this.refreshOutputState(bjt, fk, byg);
        return InteractionResult.SUCCESS;
    }
    
    @Override
    protected void checkTickOnNeighbor(final Level bjt, final BlockPos fk, final BlockState byg) {
        if (bjt.getBlockTicks().willTickThisTick(fk, this)) {
            return;
        }
        final int integer5 = this.calculateOutputSignal(bjt, fk, byg);
        final BlockEntity bwi6 = bjt.getBlockEntity(fk);
        final int integer6 = (bwi6 instanceof ComparatorBlockEntity) ? ((ComparatorBlockEntity)bwi6).getOutputSignal() : 0;
        if (integer5 != integer6 || byg.<Boolean>getValue((Property<Boolean>)ComparatorBlock.POWERED) != this.shouldTurnOn(bjt, fk, byg)) {
            final TickPriority bkk8 = this.shouldPrioritize(bjt, fk, byg) ? TickPriority.HIGH : TickPriority.NORMAL;
            bjt.getBlockTicks().scheduleTick(fk, this, 2, bkk8);
        }
    }
    
    private void refreshOutputState(final Level bjt, final BlockPos fk, final BlockState byg) {
        final int integer5 = this.calculateOutputSignal(bjt, fk, byg);
        final BlockEntity bwi6 = bjt.getBlockEntity(fk);
        int integer6 = 0;
        if (bwi6 instanceof ComparatorBlockEntity) {
            final ComparatorBlockEntity bwo8 = (ComparatorBlockEntity)bwi6;
            integer6 = bwo8.getOutputSignal();
            bwo8.setOutputSignal(integer5);
        }
        if (integer6 != integer5 || byg.<ComparatorMode>getValue(ComparatorBlock.MODE) == ComparatorMode.COMPARE) {
            final boolean boolean8 = this.shouldTurnOn(bjt, fk, byg);
            final boolean boolean9 = byg.<Boolean>getValue((Property<Boolean>)ComparatorBlock.POWERED);
            if (boolean9 && !boolean8) {
                bjt.setBlock(fk, ((AbstractStateHolder<O, BlockState>)byg).<Comparable, Boolean>setValue((Property<Comparable>)ComparatorBlock.POWERED, false), 2);
            }
            else if (!boolean9 && boolean8) {
                bjt.setBlock(fk, ((AbstractStateHolder<O, BlockState>)byg).<Comparable, Boolean>setValue((Property<Comparable>)ComparatorBlock.POWERED, true), 2);
            }
            this.updateNeighborsInFront(bjt, fk, byg);
        }
    }
    
    @Override
    public void tick(final BlockState byg, final ServerLevel xd, final BlockPos fk, final Random random) {
        this.refreshOutputState(xd, fk, byg);
    }
    
    @Override
    public boolean triggerEvent(final BlockState byg, final Level bjt, final BlockPos fk, final int integer4, final int integer5) {
        super.triggerEvent(byg, bjt, fk, integer4, integer5);
        final BlockEntity bwi7 = bjt.getBlockEntity(fk);
        return bwi7 != null && bwi7.triggerEvent(integer4, integer5);
    }
    
    @Override
    public BlockEntity newBlockEntity(final BlockGetter bjd) {
        return new ComparatorBlockEntity();
    }
    
    @Override
    protected void createBlockStateDefinition(final StateDefinition.Builder<Block, BlockState> a) {
        a.add(ComparatorBlock.FACING, ComparatorBlock.MODE, ComparatorBlock.POWERED);
    }
    
    static {
        MODE = BlockStateProperties.MODE_COMPARATOR;
    }
}
