package net.minecraft.world.level.block;

import net.minecraft.world.level.block.state.AbstractStateHolder;
import net.minecraft.world.level.block.state.properties.BlockStateProperties;
import net.minecraft.world.level.block.state.StateDefinition;
import net.minecraft.world.level.block.entity.DaylightDetectorBlockEntity;
import net.minecraft.world.level.block.entity.BlockEntity;
import net.minecraft.world.InteractionResult;
import net.minecraft.world.phys.BlockHitResult;
import net.minecraft.world.InteractionHand;
import net.minecraft.world.entity.player.Player;
import net.minecraft.util.Mth;
import net.minecraft.world.level.LightLayer;
import net.minecraft.world.level.Level;
import net.minecraft.core.Direction;
import net.minecraft.world.phys.shapes.CollisionContext;
import net.minecraft.core.BlockPos;
import net.minecraft.world.level.BlockGetter;
import net.minecraft.world.level.block.state.properties.Property;
import net.minecraft.world.level.block.state.BlockState;
import net.minecraft.world.phys.shapes.VoxelShape;
import net.minecraft.world.level.block.state.properties.BooleanProperty;
import net.minecraft.world.level.block.state.properties.IntegerProperty;

public class DaylightDetectorBlock extends BaseEntityBlock {
    public static final IntegerProperty POWER;
    public static final BooleanProperty INVERTED;
    protected static final VoxelShape SHAPE;
    
    public DaylightDetectorBlock(final Properties c) {
        super(c);
        this.registerDefaultState((((AbstractStateHolder<O, BlockState>)this.stateDefinition.any()).setValue((Property<Comparable>)DaylightDetectorBlock.POWER, 0)).<Comparable, Boolean>setValue((Property<Comparable>)DaylightDetectorBlock.INVERTED, false));
    }
    
    @Override
    public VoxelShape getShape(final BlockState byg, final BlockGetter bjd, final BlockPos fk, final CollisionContext cvn) {
        return DaylightDetectorBlock.SHAPE;
    }
    
    @Override
    public boolean useShapeForLightOcclusion(final BlockState byg) {
        return true;
    }
    
    @Override
    public int getSignal(final BlockState byg, final BlockGetter bjd, final BlockPos fk, final Direction fp) {
        return byg.<Integer>getValue((Property<Integer>)DaylightDetectorBlock.POWER);
    }
    
    public static void updateSignalStrength(final BlockState byg, final Level bjt, final BlockPos fk) {
        if (!bjt.dimension.isHasSkyLight()) {
            return;
        }
        int integer4 = bjt.getBrightness(LightLayer.SKY, fk) - bjt.getSkyDarken();
        float float5 = bjt.getSunAngle(1.0f);
        final boolean boolean6 = byg.<Boolean>getValue((Property<Boolean>)DaylightDetectorBlock.INVERTED);
        if (boolean6) {
            integer4 = 15 - integer4;
        }
        else if (integer4 > 0) {
            final float float6 = (float5 < 3.1415927f) ? 0.0f : 6.2831855f;
            float5 += (float6 - float5) * 0.2f;
            integer4 = Math.round(integer4 * Mth.cos(float5));
        }
        integer4 = Mth.clamp(integer4, 0, 15);
        if (byg.<Integer>getValue((Property<Integer>)DaylightDetectorBlock.POWER) != integer4) {
            bjt.setBlock(fk, ((AbstractStateHolder<O, BlockState>)byg).<Comparable, Integer>setValue((Property<Comparable>)DaylightDetectorBlock.POWER, integer4), 3);
        }
    }
    
    @Override
    public InteractionResult use(final BlockState byg, final Level bjt, final BlockPos fk, final Player ayg, final InteractionHand ajh, final BlockHitResult cvd) {
        if (!ayg.mayBuild()) {
            return super.use(byg, bjt, fk, ayg, ajh, cvd);
        }
        if (bjt.isClientSide) {
            return InteractionResult.SUCCESS;
        }
        final BlockState byg2 = ((AbstractStateHolder<O, BlockState>)byg).<Comparable>cycle((Property<Comparable>)DaylightDetectorBlock.INVERTED);
        bjt.setBlock(fk, byg2, 4);
        updateSignalStrength(byg2, bjt, fk);
        return InteractionResult.SUCCESS;
    }
    
    @Override
    public RenderShape getRenderShape(final BlockState byg) {
        return RenderShape.MODEL;
    }
    
    @Override
    public boolean isSignalSource(final BlockState byg) {
        return true;
    }
    
    @Override
    public BlockEntity newBlockEntity(final BlockGetter bjd) {
        return new DaylightDetectorBlockEntity();
    }
    
    @Override
    protected void createBlockStateDefinition(final StateDefinition.Builder<Block, BlockState> a) {
        a.add(DaylightDetectorBlock.POWER, DaylightDetectorBlock.INVERTED);
    }
    
    static {
        POWER = BlockStateProperties.POWER;
        INVERTED = BlockStateProperties.INVERTED;
        SHAPE = Block.box(0.0, 0.0, 0.0, 16.0, 6.0, 16.0);
    }
}
