package net.minecraft.world.level.block;

import net.minecraft.world.level.block.state.AbstractStateHolder;
import net.minecraft.world.level.block.state.properties.BlockStateProperties;
import net.minecraft.world.level.block.state.StateDefinition;
import net.minecraft.world.phys.AABB;
import javax.annotation.Nullable;
import net.minecraft.world.inventory.AbstractContainerMenu;
import net.minecraft.world.Container;
import net.minecraft.world.entity.EntitySelector;
import net.minecraft.world.entity.vehicle.MinecartCommandBlock;
import java.util.Iterator;
import java.util.List;
import java.util.function.Predicate;
import net.minecraft.world.entity.vehicle.AbstractMinecart;
import net.minecraft.core.Direction;
import net.minecraft.world.level.BlockGetter;
import java.util.Random;
import net.minecraft.server.level.ServerLevel;
import net.minecraft.world.entity.Entity;
import net.minecraft.core.BlockPos;
import net.minecraft.world.level.Level;
import net.minecraft.world.level.LevelReader;
import net.minecraft.world.level.block.state.properties.Property;
import net.minecraft.world.level.block.state.BlockState;
import net.minecraft.world.level.block.state.properties.BooleanProperty;
import net.minecraft.world.level.block.state.properties.RailShape;
import net.minecraft.world.level.block.state.properties.EnumProperty;

public class DetectorRailBlock extends BaseRailBlock {
    public static final EnumProperty<RailShape> SHAPE;
    public static final BooleanProperty POWERED;
    
    public DetectorRailBlock(final Properties c) {
        super(true, c);
        this.registerDefaultState((((AbstractStateHolder<O, BlockState>)this.stateDefinition.any()).setValue((Property<Comparable>)DetectorRailBlock.POWERED, false)).<RailShape, RailShape>setValue(DetectorRailBlock.SHAPE, RailShape.NORTH_SOUTH));
    }
    
    @Override
    public int getTickDelay(final LevelReader bjw) {
        return 20;
    }
    
    @Override
    public boolean isSignalSource(final BlockState byg) {
        return true;
    }
    
    @Override
    public void entityInside(final BlockState byg, final Level bjt, final BlockPos fk, final Entity akn) {
        if (bjt.isClientSide) {
            return;
        }
        if (byg.<Boolean>getValue((Property<Boolean>)DetectorRailBlock.POWERED)) {
            return;
        }
        this.checkPressed(bjt, fk, byg);
    }
    
    @Override
    public void tick(final BlockState byg, final ServerLevel xd, final BlockPos fk, final Random random) {
        if (!byg.<Boolean>getValue((Property<Boolean>)DetectorRailBlock.POWERED)) {
            return;
        }
        this.checkPressed(xd, fk, byg);
    }
    
    @Override
    public int getSignal(final BlockState byg, final BlockGetter bjd, final BlockPos fk, final Direction fp) {
        return byg.<Boolean>getValue((Property<Boolean>)DetectorRailBlock.POWERED) ? 15 : 0;
    }
    
    @Override
    public int getDirectSignal(final BlockState byg, final BlockGetter bjd, final BlockPos fk, final Direction fp) {
        if (!byg.<Boolean>getValue((Property<Boolean>)DetectorRailBlock.POWERED)) {
            return 0;
        }
        return (fp == Direction.UP) ? 15 : 0;
    }
    
    private void checkPressed(final Level bjt, final BlockPos fk, final BlockState byg) {
        final boolean boolean5 = byg.<Boolean>getValue((Property<Boolean>)DetectorRailBlock.POWERED);
        boolean boolean6 = false;
        final List<AbstractMinecart> list7 = this.<AbstractMinecart>getInteractingMinecartOfType(bjt, fk, AbstractMinecart.class, null);
        if (!list7.isEmpty()) {
            boolean6 = true;
        }
        if (boolean6 && !boolean5) {
            final BlockState byg2 = ((AbstractStateHolder<O, BlockState>)byg).<Comparable, Boolean>setValue((Property<Comparable>)DetectorRailBlock.POWERED, true);
            bjt.setBlock(fk, byg2, 3);
            this.updatePowerToConnected(bjt, fk, byg2, true);
            bjt.updateNeighborsAt(fk, this);
            bjt.updateNeighborsAt(fk.below(), this);
            bjt.setBlocksDirty(fk, byg, byg2);
        }
        if (!boolean6 && boolean5) {
            final BlockState byg2 = ((AbstractStateHolder<O, BlockState>)byg).<Comparable, Boolean>setValue((Property<Comparable>)DetectorRailBlock.POWERED, false);
            bjt.setBlock(fk, byg2, 3);
            this.updatePowerToConnected(bjt, fk, byg2, false);
            bjt.updateNeighborsAt(fk, this);
            bjt.updateNeighborsAt(fk.below(), this);
            bjt.setBlocksDirty(fk, byg, byg2);
        }
        if (boolean6) {
            bjt.getBlockTicks().scheduleTick(fk, this, this.getTickDelay(bjt));
        }
        bjt.updateNeighbourForOutputSignal(fk, this);
    }
    
    protected void updatePowerToConnected(final Level bjt, final BlockPos fk, final BlockState byg, final boolean boolean4) {
        final RailState bti6 = new RailState(bjt, fk, byg);
        final List<BlockPos> list7 = bti6.getConnections();
        for (final BlockPos fk2 : list7) {
            final BlockState byg2 = bjt.getBlockState(fk2);
            byg2.neighborChanged(bjt, fk2, byg2.getBlock(), fk, false);
        }
    }
    
    @Override
    public void onPlace(final BlockState byg1, final Level bjt, final BlockPos fk, final BlockState byg4, final boolean boolean5) {
        if (byg4.getBlock() == byg1.getBlock()) {
            return;
        }
        super.onPlace(byg1, bjt, fk, byg4, boolean5);
        this.checkPressed(bjt, fk, byg1);
    }
    
    @Override
    public Property<RailShape> getShapeProperty() {
        return DetectorRailBlock.SHAPE;
    }
    
    @Override
    public boolean hasAnalogOutputSignal(final BlockState byg) {
        return true;
    }
    
    @Override
    public int getAnalogOutputSignal(final BlockState byg, final Level bjt, final BlockPos fk) {
        if (byg.<Boolean>getValue((Property<Boolean>)DetectorRailBlock.POWERED)) {
            final List<MinecartCommandBlock> list5 = this.<MinecartCommandBlock>getInteractingMinecartOfType(bjt, fk, MinecartCommandBlock.class, null);
            if (!list5.isEmpty()) {
                return list5.get(0).getCommandBlock().getSuccessCount();
            }
            final List<AbstractMinecart> list6 = this.<AbstractMinecart>getInteractingMinecartOfType(bjt, fk, AbstractMinecart.class, EntitySelector.CONTAINER_ENTITY_SELECTOR);
            if (!list6.isEmpty()) {
                return AbstractContainerMenu.getRedstoneSignalFromContainer((Container)list6.get(0));
            }
        }
        return 0;
    }
    
    protected <T extends AbstractMinecart> List<T> getInteractingMinecartOfType(final Level bjt, final BlockPos fk, final Class<T> class3, @Nullable final Predicate<Entity> predicate) {
        return bjt.<T>getEntitiesOfClass(class3, this.getSearchBB(fk), predicate);
    }
    
    private AABB getSearchBB(final BlockPos fk) {
        final float float3 = 0.2f;
        return new AABB(fk.getX() + 0.2f, fk.getY(), fk.getZ() + 0.2f, fk.getX() + 1 - 0.2f, fk.getY() + 1 - 0.2f, fk.getZ() + 1 - 0.2f);
    }
    
    @Override
    public BlockState rotate(final BlockState byg, final Rotation btr) {
        Label_0424: {
            switch (btr) {
                case CLOCKWISE_180: {
                    switch (byg.<RailShape>getValue(DetectorRailBlock.SHAPE)) {
                        case ASCENDING_EAST: {
                            return ((AbstractStateHolder<O, BlockState>)byg).<RailShape, RailShape>setValue(DetectorRailBlock.SHAPE, RailShape.ASCENDING_WEST);
                        }
                        case ASCENDING_WEST: {
                            return ((AbstractStateHolder<O, BlockState>)byg).<RailShape, RailShape>setValue(DetectorRailBlock.SHAPE, RailShape.ASCENDING_EAST);
                        }
                        case ASCENDING_NORTH: {
                            return ((AbstractStateHolder<O, BlockState>)byg).<RailShape, RailShape>setValue(DetectorRailBlock.SHAPE, RailShape.ASCENDING_SOUTH);
                        }
                        case ASCENDING_SOUTH: {
                            return ((AbstractStateHolder<O, BlockState>)byg).<RailShape, RailShape>setValue(DetectorRailBlock.SHAPE, RailShape.ASCENDING_NORTH);
                        }
                        case SOUTH_EAST: {
                            return ((AbstractStateHolder<O, BlockState>)byg).<RailShape, RailShape>setValue(DetectorRailBlock.SHAPE, RailShape.NORTH_WEST);
                        }
                        case SOUTH_WEST: {
                            return ((AbstractStateHolder<O, BlockState>)byg).<RailShape, RailShape>setValue(DetectorRailBlock.SHAPE, RailShape.NORTH_EAST);
                        }
                        case NORTH_WEST: {
                            return ((AbstractStateHolder<O, BlockState>)byg).<RailShape, RailShape>setValue(DetectorRailBlock.SHAPE, RailShape.SOUTH_EAST);
                        }
                        case NORTH_EAST: {
                            return ((AbstractStateHolder<O, BlockState>)byg).<RailShape, RailShape>setValue(DetectorRailBlock.SHAPE, RailShape.SOUTH_WEST);
                        }
                        default: {
                            break Label_0424;
                        }
                    }
                    break;
                }
                case COUNTERCLOCKWISE_90: {
                    switch (byg.<RailShape>getValue(DetectorRailBlock.SHAPE)) {
                        case NORTH_SOUTH: {
                            return ((AbstractStateHolder<O, BlockState>)byg).<RailShape, RailShape>setValue(DetectorRailBlock.SHAPE, RailShape.EAST_WEST);
                        }
                        case EAST_WEST: {
                            return ((AbstractStateHolder<O, BlockState>)byg).<RailShape, RailShape>setValue(DetectorRailBlock.SHAPE, RailShape.NORTH_SOUTH);
                        }
                        case ASCENDING_EAST: {
                            return ((AbstractStateHolder<O, BlockState>)byg).<RailShape, RailShape>setValue(DetectorRailBlock.SHAPE, RailShape.ASCENDING_NORTH);
                        }
                        case ASCENDING_WEST: {
                            return ((AbstractStateHolder<O, BlockState>)byg).<RailShape, RailShape>setValue(DetectorRailBlock.SHAPE, RailShape.ASCENDING_SOUTH);
                        }
                        case ASCENDING_NORTH: {
                            return ((AbstractStateHolder<O, BlockState>)byg).<RailShape, RailShape>setValue(DetectorRailBlock.SHAPE, RailShape.ASCENDING_WEST);
                        }
                        case ASCENDING_SOUTH: {
                            return ((AbstractStateHolder<O, BlockState>)byg).<RailShape, RailShape>setValue(DetectorRailBlock.SHAPE, RailShape.ASCENDING_EAST);
                        }
                        case SOUTH_EAST: {
                            return ((AbstractStateHolder<O, BlockState>)byg).<RailShape, RailShape>setValue(DetectorRailBlock.SHAPE, RailShape.NORTH_EAST);
                        }
                        case SOUTH_WEST: {
                            return ((AbstractStateHolder<O, BlockState>)byg).<RailShape, RailShape>setValue(DetectorRailBlock.SHAPE, RailShape.SOUTH_EAST);
                        }
                        case NORTH_WEST: {
                            return ((AbstractStateHolder<O, BlockState>)byg).<RailShape, RailShape>setValue(DetectorRailBlock.SHAPE, RailShape.SOUTH_WEST);
                        }
                        case NORTH_EAST: {
                            return ((AbstractStateHolder<O, BlockState>)byg).<RailShape, RailShape>setValue(DetectorRailBlock.SHAPE, RailShape.NORTH_WEST);
                        }
                        default: {
                            break Label_0424;
                        }
                    }
                    break;
                }
                case CLOCKWISE_90: {
                    switch (byg.<RailShape>getValue(DetectorRailBlock.SHAPE)) {
                        case NORTH_SOUTH: {
                            return ((AbstractStateHolder<O, BlockState>)byg).<RailShape, RailShape>setValue(DetectorRailBlock.SHAPE, RailShape.EAST_WEST);
                        }
                        case EAST_WEST: {
                            return ((AbstractStateHolder<O, BlockState>)byg).<RailShape, RailShape>setValue(DetectorRailBlock.SHAPE, RailShape.NORTH_SOUTH);
                        }
                        case ASCENDING_EAST: {
                            return ((AbstractStateHolder<O, BlockState>)byg).<RailShape, RailShape>setValue(DetectorRailBlock.SHAPE, RailShape.ASCENDING_SOUTH);
                        }
                        case ASCENDING_WEST: {
                            return ((AbstractStateHolder<O, BlockState>)byg).<RailShape, RailShape>setValue(DetectorRailBlock.SHAPE, RailShape.ASCENDING_NORTH);
                        }
                        case ASCENDING_NORTH: {
                            return ((AbstractStateHolder<O, BlockState>)byg).<RailShape, RailShape>setValue(DetectorRailBlock.SHAPE, RailShape.ASCENDING_EAST);
                        }
                        case ASCENDING_SOUTH: {
                            return ((AbstractStateHolder<O, BlockState>)byg).<RailShape, RailShape>setValue(DetectorRailBlock.SHAPE, RailShape.ASCENDING_WEST);
                        }
                        case SOUTH_EAST: {
                            return ((AbstractStateHolder<O, BlockState>)byg).<RailShape, RailShape>setValue(DetectorRailBlock.SHAPE, RailShape.SOUTH_WEST);
                        }
                        case SOUTH_WEST: {
                            return ((AbstractStateHolder<O, BlockState>)byg).<RailShape, RailShape>setValue(DetectorRailBlock.SHAPE, RailShape.NORTH_WEST);
                        }
                        case NORTH_WEST: {
                            return ((AbstractStateHolder<O, BlockState>)byg).<RailShape, RailShape>setValue(DetectorRailBlock.SHAPE, RailShape.NORTH_EAST);
                        }
                        case NORTH_EAST: {
                            return ((AbstractStateHolder<O, BlockState>)byg).<RailShape, RailShape>setValue(DetectorRailBlock.SHAPE, RailShape.SOUTH_EAST);
                        }
                        default: {
                            break Label_0424;
                        }
                    }
                    break;
                }
            }
        }
        return byg;
    }
    
    @Override
    public BlockState mirror(final BlockState byg, final Mirror bsr) {
        final RailShape bzk4 = byg.<RailShape>getValue(DetectorRailBlock.SHAPE);
        Label_0319: {
            switch (bsr) {
                case LEFT_RIGHT: {
                    switch (bzk4) {
                        case ASCENDING_NORTH: {
                            return ((AbstractStateHolder<O, BlockState>)byg).<RailShape, RailShape>setValue(DetectorRailBlock.SHAPE, RailShape.ASCENDING_SOUTH);
                        }
                        case ASCENDING_SOUTH: {
                            return ((AbstractStateHolder<O, BlockState>)byg).<RailShape, RailShape>setValue(DetectorRailBlock.SHAPE, RailShape.ASCENDING_NORTH);
                        }
                        case SOUTH_EAST: {
                            return ((AbstractStateHolder<O, BlockState>)byg).<RailShape, RailShape>setValue(DetectorRailBlock.SHAPE, RailShape.NORTH_EAST);
                        }
                        case SOUTH_WEST: {
                            return ((AbstractStateHolder<O, BlockState>)byg).<RailShape, RailShape>setValue(DetectorRailBlock.SHAPE, RailShape.NORTH_WEST);
                        }
                        case NORTH_WEST: {
                            return ((AbstractStateHolder<O, BlockState>)byg).<RailShape, RailShape>setValue(DetectorRailBlock.SHAPE, RailShape.SOUTH_WEST);
                        }
                        case NORTH_EAST: {
                            return ((AbstractStateHolder<O, BlockState>)byg).<RailShape, RailShape>setValue(DetectorRailBlock.SHAPE, RailShape.SOUTH_EAST);
                        }
                        default: {
                            break Label_0319;
                        }
                    }
                    break;
                }
                case FRONT_BACK: {
                    switch (bzk4) {
                        case ASCENDING_EAST: {
                            return ((AbstractStateHolder<O, BlockState>)byg).<RailShape, RailShape>setValue(DetectorRailBlock.SHAPE, RailShape.ASCENDING_WEST);
                        }
                        case ASCENDING_WEST: {
                            return ((AbstractStateHolder<O, BlockState>)byg).<RailShape, RailShape>setValue(DetectorRailBlock.SHAPE, RailShape.ASCENDING_EAST);
                        }
                        case SOUTH_EAST: {
                            return ((AbstractStateHolder<O, BlockState>)byg).<RailShape, RailShape>setValue(DetectorRailBlock.SHAPE, RailShape.SOUTH_WEST);
                        }
                        case SOUTH_WEST: {
                            return ((AbstractStateHolder<O, BlockState>)byg).<RailShape, RailShape>setValue(DetectorRailBlock.SHAPE, RailShape.SOUTH_EAST);
                        }
                        case NORTH_WEST: {
                            return ((AbstractStateHolder<O, BlockState>)byg).<RailShape, RailShape>setValue(DetectorRailBlock.SHAPE, RailShape.NORTH_EAST);
                        }
                        case NORTH_EAST: {
                            return ((AbstractStateHolder<O, BlockState>)byg).<RailShape, RailShape>setValue(DetectorRailBlock.SHAPE, RailShape.NORTH_WEST);
                        }
                        default: {
                            break Label_0319;
                        }
                    }
                    break;
                }
            }
        }
        return super.mirror(byg, bsr);
    }
    
    @Override
    protected void createBlockStateDefinition(final StateDefinition.Builder<Block, BlockState> a) {
        a.add(DetectorRailBlock.SHAPE, DetectorRailBlock.POWERED);
    }
    
    static {
        SHAPE = BlockStateProperties.RAIL_SHAPE_STRAIGHT;
        POWERED = BlockStateProperties.POWERED;
    }
}
