package net.minecraft.world.level.block;

import net.minecraft.world.level.block.state.AbstractStateHolder;
import net.minecraft.Util;
import net.minecraft.world.level.block.state.properties.BlockStateProperties;
import net.minecraft.core.dispenser.DefaultDispenseItemBehavior;
import it.unimi.dsi.fastutil.objects.Object2ObjectOpenHashMap;
import net.minecraft.world.level.block.state.StateDefinition;
import net.minecraft.world.inventory.AbstractContainerMenu;
import net.minecraft.core.PositionImpl;
import net.minecraft.core.Position;
import net.minecraft.world.Container;
import net.minecraft.world.Containers;
import net.minecraft.world.entity.LivingEntity;
import net.minecraft.world.item.BlockPlaceContext;
import net.minecraft.world.level.BlockGetter;
import java.util.Random;
import net.minecraft.server.level.ServerLevel;
import net.minecraft.world.item.ItemStack;
import net.minecraft.core.BlockSource;
import net.minecraft.core.BlockSourceImpl;
import net.minecraft.world.level.block.entity.BlockEntity;
import net.minecraft.stats.Stats;
import net.minecraft.world.level.block.entity.DropperBlockEntity;
import net.minecraft.world.MenuProvider;
import net.minecraft.world.level.block.entity.DispenserBlockEntity;
import net.minecraft.world.InteractionResult;
import net.minecraft.world.phys.BlockHitResult;
import net.minecraft.world.InteractionHand;
import net.minecraft.world.entity.player.Player;
import net.minecraft.core.BlockPos;
import net.minecraft.world.level.Level;
import net.minecraft.world.level.LevelReader;
import net.minecraft.world.level.block.state.properties.Property;
import net.minecraft.core.Direction;
import net.minecraft.world.level.block.state.BlockState;
import net.minecraft.world.level.ItemLike;
import net.minecraft.core.dispenser.DispenseItemBehavior;
import net.minecraft.world.item.Item;
import java.util.Map;
import net.minecraft.world.level.block.state.properties.BooleanProperty;
import net.minecraft.world.level.block.state.properties.DirectionProperty;

public class DispenserBlock extends BaseEntityBlock {
    public static final DirectionProperty FACING;
    public static final BooleanProperty TRIGGERED;
    private static final Map<Item, DispenseItemBehavior> DISPENSER_REGISTRY;
    
    public static void registerBehavior(final ItemLike bjs, final DispenseItemBehavior gl) {
        DispenserBlock.DISPENSER_REGISTRY.put(bjs.asItem(), gl);
    }
    
    protected DispenserBlock(final Properties c) {
        super(c);
        this.registerDefaultState((((AbstractStateHolder<O, BlockState>)this.stateDefinition.any()).setValue((Property<Comparable>)DispenserBlock.FACING, Direction.NORTH)).<Comparable, Boolean>setValue((Property<Comparable>)DispenserBlock.TRIGGERED, false));
    }
    
    @Override
    public int getTickDelay(final LevelReader bjw) {
        return 4;
    }
    
    @Override
    public InteractionResult use(final BlockState byg, final Level bjt, final BlockPos fk, final Player ayg, final InteractionHand ajh, final BlockHitResult cvd) {
        if (bjt.isClientSide) {
            return InteractionResult.SUCCESS;
        }
        final BlockEntity bwi8 = bjt.getBlockEntity(fk);
        if (bwi8 instanceof DispenserBlockEntity) {
            ayg.openMenu((MenuProvider)bwi8);
            if (bwi8 instanceof DropperBlockEntity) {
                ayg.awardStat(Stats.INSPECT_DROPPER);
            }
            else {
                ayg.awardStat(Stats.INSPECT_DISPENSER);
            }
        }
        return InteractionResult.SUCCESS;
    }
    
    protected void dispenseFrom(final Level bjt, final BlockPos fk) {
        final BlockSourceImpl fm4 = new BlockSourceImpl(bjt, fk);
        final DispenserBlockEntity bwr5 = fm4.<DispenserBlockEntity>getEntity();
        final int integer6 = bwr5.getRandomSlot();
        if (integer6 < 0) {
            bjt.levelEvent(1001, fk, 0);
            return;
        }
        final ItemStack bek7 = bwr5.getItem(integer6);
        final DispenseItemBehavior gl8 = this.getDispenseMethod(bek7);
        if (gl8 != DispenseItemBehavior.NOOP) {
            bwr5.setItem(integer6, gl8.dispense(fm4, bek7));
        }
    }
    
    protected DispenseItemBehavior getDispenseMethod(final ItemStack bek) {
        return DispenserBlock.DISPENSER_REGISTRY.get(bek.getItem());
    }
    
    @Override
    public void neighborChanged(final BlockState byg, final Level bjt, final BlockPos fk3, final Block bpe, final BlockPos fk5, final boolean boolean6) {
        final boolean boolean7 = bjt.hasNeighborSignal(fk3) || bjt.hasNeighborSignal(fk3.above());
        final boolean boolean8 = byg.<Boolean>getValue((Property<Boolean>)DispenserBlock.TRIGGERED);
        if (boolean7 && !boolean8) {
            bjt.getBlockTicks().scheduleTick(fk3, this, this.getTickDelay(bjt));
            bjt.setBlock(fk3, ((AbstractStateHolder<O, BlockState>)byg).<Comparable, Boolean>setValue((Property<Comparable>)DispenserBlock.TRIGGERED, true), 4);
        }
        else if (!boolean7 && boolean8) {
            bjt.setBlock(fk3, ((AbstractStateHolder<O, BlockState>)byg).<Comparable, Boolean>setValue((Property<Comparable>)DispenserBlock.TRIGGERED, false), 4);
        }
    }
    
    @Override
    public void tick(final BlockState byg, final ServerLevel xd, final BlockPos fk, final Random random) {
        this.dispenseFrom(xd, fk);
    }
    
    @Override
    public BlockEntity newBlockEntity(final BlockGetter bjd) {
        return new DispenserBlockEntity();
    }
    
    @Override
    public BlockState getStateForPlacement(final BlockPlaceContext bcn) {
        return ((AbstractStateHolder<O, BlockState>)this.defaultBlockState()).<Comparable, Direction>setValue((Property<Comparable>)DispenserBlock.FACING, bcn.getNearestLookingDirection().getOpposite());
    }
    
    @Override
    public void setPlacedBy(final Level bjt, final BlockPos fk, final BlockState byg, final LivingEntity akw, final ItemStack bek) {
        if (bek.hasCustomHoverName()) {
            final BlockEntity bwi7 = bjt.getBlockEntity(fk);
            if (bwi7 instanceof DispenserBlockEntity) {
                ((DispenserBlockEntity)bwi7).setCustomName(bek.getHoverName());
            }
        }
    }
    
    @Override
    public void onRemove(final BlockState byg1, final Level bjt, final BlockPos fk, final BlockState byg4, final boolean boolean5) {
        if (byg1.getBlock() == byg4.getBlock()) {
            return;
        }
        final BlockEntity bwi7 = bjt.getBlockEntity(fk);
        if (bwi7 instanceof DispenserBlockEntity) {
            Containers.dropContents(bjt, fk, (Container)bwi7);
            bjt.updateNeighbourForOutputSignal(fk, this);
        }
        super.onRemove(byg1, bjt, fk, byg4, boolean5);
    }
    
    public static Position getDispensePosition(final BlockSource fl) {
        final Direction fp2 = fl.getBlockState().<Direction>getValue((Property<Direction>)DispenserBlock.FACING);
        final double double3 = fl.x() + 0.7 * fp2.getStepX();
        final double double4 = fl.y() + 0.7 * fp2.getStepY();
        final double double5 = fl.z() + 0.7 * fp2.getStepZ();
        return new PositionImpl(double3, double4, double5);
    }
    
    @Override
    public boolean hasAnalogOutputSignal(final BlockState byg) {
        return true;
    }
    
    @Override
    public int getAnalogOutputSignal(final BlockState byg, final Level bjt, final BlockPos fk) {
        return AbstractContainerMenu.getRedstoneSignalFromBlockEntity(bjt.getBlockEntity(fk));
    }
    
    @Override
    public RenderShape getRenderShape(final BlockState byg) {
        return RenderShape.MODEL;
    }
    
    @Override
    public BlockState rotate(final BlockState byg, final Rotation btr) {
        return ((AbstractStateHolder<O, BlockState>)byg).<Comparable, Direction>setValue((Property<Comparable>)DispenserBlock.FACING, btr.rotate(byg.<Direction>getValue((Property<Direction>)DispenserBlock.FACING)));
    }
    
    @Override
    public BlockState mirror(final BlockState byg, final Mirror bsr) {
        return byg.rotate(bsr.getRotation(byg.<Direction>getValue((Property<Direction>)DispenserBlock.FACING)));
    }
    
    @Override
    protected void createBlockStateDefinition(final StateDefinition.Builder<Block, BlockState> a) {
        a.add(DispenserBlock.FACING, DispenserBlock.TRIGGERED);
    }
    
    static {
        FACING = DirectionalBlock.FACING;
        TRIGGERED = BlockStateProperties.TRIGGERED;
        DISPENSER_REGISTRY = Util.<Map<Item, DispenseItemBehavior>>make((Map<Item, DispenseItemBehavior>)new Object2ObjectOpenHashMap(), object2ObjectOpenHashMap -> object2ObjectOpenHashMap.defaultReturnValue(new DefaultDispenseItemBehavior()));
    }
}
