package net.minecraft.world.level.block;

import net.minecraft.world.level.block.state.AbstractStateHolder;
import net.minecraft.world.level.block.state.properties.BlockStateProperties;
import net.minecraft.world.level.pathfinder.PathComputationType;
import net.minecraft.world.level.block.state.StateDefinition;
import java.util.Iterator;
import net.minecraft.tags.FluidTags;
import net.minecraft.world.level.GameRules;
import net.minecraft.world.entity.player.Player;
import net.minecraft.world.entity.LivingEntity;
import net.minecraft.world.entity.Entity;
import net.minecraft.world.level.Level;
import java.util.Random;
import net.minecraft.server.level.ServerLevel;
import net.minecraft.world.phys.shapes.CollisionContext;
import net.minecraft.world.level.BlockGetter;
import net.minecraft.world.item.BlockPlaceContext;
import net.minecraft.world.level.block.piston.MovingPistonBlock;
import net.minecraft.world.level.LevelReader;
import net.minecraft.core.BlockPos;
import net.minecraft.world.level.LevelAccessor;
import net.minecraft.core.Direction;
import net.minecraft.world.level.block.state.properties.Property;
import net.minecraft.world.level.block.state.BlockState;
import net.minecraft.world.phys.shapes.VoxelShape;
import net.minecraft.world.level.block.state.properties.IntegerProperty;

public class FarmBlock extends Block {
    public static final IntegerProperty MOISTURE;
    protected static final VoxelShape SHAPE;
    
    protected FarmBlock(final Properties c) {
        super(c);
        this.registerDefaultState(((AbstractStateHolder<O, BlockState>)this.stateDefinition.any()).<Comparable, Integer>setValue((Property<Comparable>)FarmBlock.MOISTURE, 0));
    }
    
    @Override
    public BlockState updateShape(final BlockState byg1, final Direction fp, final BlockState byg3, final LevelAccessor bju, final BlockPos fk5, final BlockPos fk6) {
        if (fp == Direction.UP && !byg1.canSurvive(bju, fk5)) {
            bju.getBlockTicks().scheduleTick(fk5, this, 1);
        }
        return super.updateShape(byg1, fp, byg3, bju, fk5, fk6);
    }
    
    @Override
    public boolean canSurvive(final BlockState byg, final LevelReader bjw, final BlockPos fk) {
        final BlockState byg2 = bjw.getBlockState(fk.above());
        return !byg2.getMaterial().isSolid() || byg2.getBlock() instanceof FenceGateBlock || byg2.getBlock() instanceof MovingPistonBlock;
    }
    
    @Override
    public BlockState getStateForPlacement(final BlockPlaceContext bcn) {
        if (!this.defaultBlockState().canSurvive(bcn.getLevel(), bcn.getClickedPos())) {
            return Blocks.DIRT.defaultBlockState();
        }
        return super.getStateForPlacement(bcn);
    }
    
    @Override
    public boolean useShapeForLightOcclusion(final BlockState byg) {
        return true;
    }
    
    @Override
    public VoxelShape getShape(final BlockState byg, final BlockGetter bjd, final BlockPos fk, final CollisionContext cvn) {
        return FarmBlock.SHAPE;
    }
    
    @Override
    public void tick(final BlockState byg, final ServerLevel xd, final BlockPos fk, final Random random) {
        if (!byg.canSurvive(xd, fk)) {
            turnToDirt(byg, xd, fk);
            return;
        }
        final int integer6 = byg.<Integer>getValue((Property<Integer>)FarmBlock.MOISTURE);
        if (isNearWater(xd, fk) || xd.isRainingAt(fk.above())) {
            if (integer6 < 7) {
                xd.setBlock(fk, ((AbstractStateHolder<O, BlockState>)byg).<Comparable, Integer>setValue((Property<Comparable>)FarmBlock.MOISTURE, 7), 2);
            }
        }
        else if (integer6 > 0) {
            xd.setBlock(fk, ((AbstractStateHolder<O, BlockState>)byg).<Comparable, Integer>setValue((Property<Comparable>)FarmBlock.MOISTURE, integer6 - 1), 2);
        }
        else if (!isUnderCrops(xd, fk)) {
            turnToDirt(byg, xd, fk);
        }
    }
    
    @Override
    public void fallOn(final Level bjt, final BlockPos fk, final Entity akn, final float float4) {
        if (!bjt.isClientSide && bjt.random.nextFloat() < float4 - 0.5f && akn instanceof LivingEntity && (akn instanceof Player || bjt.getGameRules().getBoolean(GameRules.RULE_MOBGRIEFING)) && akn.getBbWidth() * akn.getBbWidth() * akn.getBbHeight() > 0.512f) {
            turnToDirt(bjt.getBlockState(fk), bjt, fk);
        }
        super.fallOn(bjt, fk, akn, float4);
    }
    
    public static void turnToDirt(final BlockState byg, final Level bjt, final BlockPos fk) {
        bjt.setBlockAndUpdate(fk, Block.pushEntitiesUp(byg, Blocks.DIRT.defaultBlockState(), bjt, fk));
    }
    
    private static boolean isUnderCrops(final BlockGetter bjd, final BlockPos fk) {
        final Block bpe3 = bjd.getBlockState(fk.above()).getBlock();
        return bpe3 instanceof CropBlock || bpe3 instanceof StemBlock || bpe3 instanceof AttachedStemBlock;
    }
    
    private static boolean isNearWater(final LevelReader bjw, final BlockPos fk) {
        for (final BlockPos fk2 : BlockPos.betweenClosed(fk.offset(-4, 0, -4), fk.offset(4, 1, 4))) {
            if (bjw.getFluidState(fk2).is(FluidTags.WATER)) {
                return true;
            }
        }
        return false;
    }
    
    @Override
    protected void createBlockStateDefinition(final StateDefinition.Builder<Block, BlockState> a) {
        a.add(FarmBlock.MOISTURE);
    }
    
    @Override
    public boolean isPathfindable(final BlockState byg, final BlockGetter bjd, final BlockPos fk, final PathComputationType cqo) {
        return false;
    }
    
    @Override
    public boolean isViewBlocking(final BlockState byg, final BlockGetter bjd, final BlockPos fk) {
        return true;
    }
    
    static {
        MOISTURE = BlockStateProperties.MOISTURE;
        SHAPE = Block.box(0.0, 0.0, 0.0, 16.0, 15.0, 16.0);
    }
}
