package net.minecraft.world.level.block;

import net.minecraft.world.level.block.state.AbstractStateHolder;
import net.minecraft.world.level.block.state.StateDefinition;
import net.minecraft.world.level.block.state.properties.BooleanProperty;
import net.minecraft.world.level.LevelReader;
import net.minecraft.world.level.LevelAccessor;
import net.minecraft.world.level.material.FluidState;
import net.minecraft.world.level.material.Fluids;
import net.minecraft.world.item.BlockPlaceContext;
import net.minecraft.world.item.ItemStack;
import net.minecraft.world.item.Items;
import net.minecraft.world.item.LeadItem;
import net.minecraft.world.InteractionResult;
import net.minecraft.world.phys.BlockHitResult;
import net.minecraft.world.InteractionHand;
import net.minecraft.world.entity.player.Player;
import net.minecraft.world.level.Level;
import net.minecraft.tags.BlockTags;
import net.minecraft.core.Direction;
import net.minecraft.world.level.pathfinder.PathComputationType;
import net.minecraft.core.BlockPos;
import net.minecraft.world.level.BlockGetter;
import net.minecraft.world.level.block.state.properties.Property;
import net.minecraft.world.level.block.state.BlockState;
import net.minecraft.world.phys.shapes.VoxelShape;

public class FenceBlock extends CrossCollisionBlock {
    private final VoxelShape[] occlusionByIndex;
    
    public FenceBlock(final Properties c) {
        super(2.0f, 2.0f, 16.0f, 16.0f, 24.0f, c);
        this.registerDefaultState(((((((AbstractStateHolder<O, BlockState>)this.stateDefinition.any()).setValue((Property<Comparable>)FenceBlock.NORTH, false)).setValue((Property<Comparable>)FenceBlock.EAST, false)).setValue((Property<Comparable>)FenceBlock.SOUTH, false)).setValue((Property<Comparable>)FenceBlock.WEST, false)).<Comparable, Boolean>setValue((Property<Comparable>)FenceBlock.WATERLOGGED, false));
        this.occlusionByIndex = this.makeShapes(2.0f, 1.0f, 16.0f, 6.0f, 15.0f);
    }
    
    @Override
    public VoxelShape getOcclusionShape(final BlockState byg, final BlockGetter bjd, final BlockPos fk) {
        return this.occlusionByIndex[this.getAABBIndex(byg)];
    }
    
    @Override
    public boolean isPathfindable(final BlockState byg, final BlockGetter bjd, final BlockPos fk, final PathComputationType cqo) {
        return false;
    }
    
    public boolean connectsTo(final BlockState byg, final boolean boolean2, final Direction fp) {
        final Block bpe5 = byg.getBlock();
        final boolean boolean3 = bpe5.is(BlockTags.FENCES) && byg.getMaterial() == this.material;
        final boolean boolean4 = bpe5 instanceof FenceGateBlock && FenceGateBlock.connectsToDirection(byg, fp);
        return (!Block.isExceptionForConnection(bpe5) && boolean2) || boolean3 || boolean4;
    }
    
    @Override
    public InteractionResult use(final BlockState byg, final Level bjt, final BlockPos fk, final Player ayg, final InteractionHand ajh, final BlockHitResult cvd) {
        if (!bjt.isClientSide) {
            return LeadItem.bindPlayerMobs(ayg, bjt, fk);
        }
        final ItemStack bek8 = ayg.getItemInHand(ajh);
        if (bek8.getItem() == Items.LEAD) {
            return InteractionResult.SUCCESS;
        }
        return InteractionResult.PASS;
    }
    
    @Override
    public BlockState getStateForPlacement(final BlockPlaceContext bcn) {
        final BlockGetter bjd3 = bcn.getLevel();
        final BlockPos fk4 = bcn.getClickedPos();
        final FluidState cog5 = bcn.getLevel().getFluidState(bcn.getClickedPos());
        final BlockPos fk5 = fk4.north();
        final BlockPos fk6 = fk4.east();
        final BlockPos fk7 = fk4.south();
        final BlockPos fk8 = fk4.west();
        final BlockState byg10 = bjd3.getBlockState(fk5);
        final BlockState byg11 = bjd3.getBlockState(fk6);
        final BlockState byg12 = bjd3.getBlockState(fk7);
        final BlockState byg13 = bjd3.getBlockState(fk8);
        return ((((((AbstractStateHolder<O, BlockState>)super.getStateForPlacement(bcn)).setValue((Property<Comparable>)FenceBlock.NORTH, this.connectsTo(byg10, byg10.isFaceSturdy(bjd3, fk5, Direction.SOUTH), Direction.SOUTH))).setValue((Property<Comparable>)FenceBlock.EAST, this.connectsTo(byg11, byg11.isFaceSturdy(bjd3, fk6, Direction.WEST), Direction.WEST))).setValue((Property<Comparable>)FenceBlock.SOUTH, this.connectsTo(byg12, byg12.isFaceSturdy(bjd3, fk7, Direction.NORTH), Direction.NORTH))).setValue((Property<Comparable>)FenceBlock.WEST, this.connectsTo(byg13, byg13.isFaceSturdy(bjd3, fk8, Direction.EAST), Direction.EAST))).<Comparable, Boolean>setValue((Property<Comparable>)FenceBlock.WATERLOGGED, cog5.getType() == Fluids.WATER);
    }
    
    @Override
    public BlockState updateShape(final BlockState byg1, final Direction fp, final BlockState byg3, final LevelAccessor bju, final BlockPos fk5, final BlockPos fk6) {
        if (byg1.<Boolean>getValue((Property<Boolean>)FenceBlock.WATERLOGGED)) {
            bju.getLiquidTicks().scheduleTick(fk5, Fluids.WATER, Fluids.WATER.getTickDelay(bju));
        }
        if (fp.getAxis().getPlane() == Direction.Plane.HORIZONTAL) {
            return ((AbstractStateHolder<O, BlockState>)byg1).<Comparable, Boolean>setValue((Property<Comparable>)FenceBlock.PROPERTY_BY_DIRECTION.get(fp), this.connectsTo(byg3, byg3.isFaceSturdy(bju, fk6, fp.getOpposite()), fp.getOpposite()));
        }
        return super.updateShape(byg1, fp, byg3, bju, fk5, fk6);
    }
    
    @Override
    protected void createBlockStateDefinition(final StateDefinition.Builder<Block, BlockState> a) {
        a.add(FenceBlock.NORTH, FenceBlock.EAST, FenceBlock.WEST, FenceBlock.SOUTH, FenceBlock.WATERLOGGED);
    }
}
