package net.minecraft.world.level.block;

import net.minecraft.world.level.pathfinder.PathComputationType;
import net.minecraft.world.phys.shapes.CollisionContext;
import net.minecraft.world.level.BlockGetter;
import net.minecraft.world.level.Level;
import java.util.Random;
import net.minecraft.server.level.ServerLevel;
import net.minecraft.core.BlockPos;
import net.minecraft.world.level.LevelAccessor;
import net.minecraft.core.Direction;
import net.minecraft.world.level.LevelReader;
import net.minecraft.world.item.BlockPlaceContext;
import net.minecraft.world.level.block.state.BlockState;
import net.minecraft.world.phys.shapes.VoxelShape;

public class GrassPathBlock extends Block {
    protected static final VoxelShape SHAPE;
    
    protected GrassPathBlock(final Properties c) {
        super(c);
    }
    
    @Override
    public boolean useShapeForLightOcclusion(final BlockState byg) {
        return true;
    }
    
    @Override
    public BlockState getStateForPlacement(final BlockPlaceContext bcn) {
        if (!this.defaultBlockState().canSurvive(bcn.getLevel(), bcn.getClickedPos())) {
            return Block.pushEntitiesUp(this.defaultBlockState(), Blocks.DIRT.defaultBlockState(), bcn.getLevel(), bcn.getClickedPos());
        }
        return super.getStateForPlacement(bcn);
    }
    
    @Override
    public BlockState updateShape(final BlockState byg1, final Direction fp, final BlockState byg3, final LevelAccessor bju, final BlockPos fk5, final BlockPos fk6) {
        if (fp == Direction.UP && !byg1.canSurvive(bju, fk5)) {
            bju.getBlockTicks().scheduleTick(fk5, this, 1);
        }
        return super.updateShape(byg1, fp, byg3, bju, fk5, fk6);
    }
    
    @Override
    public void tick(final BlockState byg, final ServerLevel xd, final BlockPos fk, final Random random) {
        FarmBlock.turnToDirt(byg, xd, fk);
    }
    
    @Override
    public boolean canSurvive(final BlockState byg, final LevelReader bjw, final BlockPos fk) {
        final BlockState byg2 = bjw.getBlockState(fk.above());
        return !byg2.getMaterial().isSolid() || byg2.getBlock() instanceof FenceGateBlock;
    }
    
    @Override
    public VoxelShape getShape(final BlockState byg, final BlockGetter bjd, final BlockPos fk, final CollisionContext cvn) {
        return GrassPathBlock.SHAPE;
    }
    
    @Override
    public boolean isPathfindable(final BlockState byg, final BlockGetter bjd, final BlockPos fk, final PathComputationType cqo) {
        return false;
    }
    
    @Override
    public boolean isViewBlocking(final BlockState byg, final BlockGetter bjd, final BlockPos fk) {
        return true;
    }
    
    static {
        SHAPE = FarmBlock.SHAPE;
    }
}
