package net.minecraft.world.level.block;

import net.minecraft.world.level.block.state.AbstractStateHolder;
import net.minecraft.world.level.block.state.properties.BlockStateProperties;
import net.minecraft.world.level.block.state.StateDefinition;
import net.minecraft.core.particles.ParticleOptions;
import net.minecraft.core.particles.ParticleTypes;
import net.minecraft.sounds.SoundSource;
import net.minecraft.stats.Stats;
import net.minecraft.world.InteractionResult;
import net.minecraft.world.phys.BlockHitResult;
import net.minecraft.world.InteractionHand;
import net.minecraft.world.entity.player.Player;
import net.minecraft.world.level.Level;
import net.minecraft.core.BlockPos;
import net.minecraft.world.level.LevelAccessor;
import net.minecraft.core.Direction;
import net.minecraft.world.item.BlockPlaceContext;
import net.minecraft.world.level.block.state.properties.Property;
import net.minecraft.world.level.block.state.BlockState;
import net.minecraft.world.level.block.state.properties.IntegerProperty;
import net.minecraft.world.level.block.state.properties.BooleanProperty;
import net.minecraft.world.level.block.state.properties.NoteBlockInstrument;
import net.minecraft.world.level.block.state.properties.EnumProperty;

public class NoteBlock extends Block {
    public static final EnumProperty<NoteBlockInstrument> INSTRUMENT;
    public static final BooleanProperty POWERED;
    public static final IntegerProperty NOTE;
    
    public NoteBlock(final Properties c) {
        super(c);
        this.registerDefaultState(((((AbstractStateHolder<O, BlockState>)this.stateDefinition.any()).setValue(NoteBlock.INSTRUMENT, NoteBlockInstrument.HARP)).setValue((Property<Comparable>)NoteBlock.NOTE, 0)).<Comparable, Boolean>setValue((Property<Comparable>)NoteBlock.POWERED, false));
    }
    
    @Override
    public BlockState getStateForPlacement(final BlockPlaceContext bcn) {
        return ((AbstractStateHolder<O, BlockState>)this.defaultBlockState()).<NoteBlockInstrument, NoteBlockInstrument>setValue(NoteBlock.INSTRUMENT, NoteBlockInstrument.byState(bcn.getLevel().getBlockState(bcn.getClickedPos().below())));
    }
    
    @Override
    public BlockState updateShape(final BlockState byg1, final Direction fp, final BlockState byg3, final LevelAccessor bju, final BlockPos fk5, final BlockPos fk6) {
        if (fp == Direction.DOWN) {
            return ((AbstractStateHolder<O, BlockState>)byg1).<NoteBlockInstrument, NoteBlockInstrument>setValue(NoteBlock.INSTRUMENT, NoteBlockInstrument.byState(byg3));
        }
        return super.updateShape(byg1, fp, byg3, bju, fk5, fk6);
    }
    
    @Override
    public void neighborChanged(final BlockState byg, final Level bjt, final BlockPos fk3, final Block bpe, final BlockPos fk5, final boolean boolean6) {
        final boolean boolean7 = bjt.hasNeighborSignal(fk3);
        if (boolean7 != byg.<Boolean>getValue((Property<Boolean>)NoteBlock.POWERED)) {
            if (boolean7) {
                this.playNote(bjt, fk3);
            }
            bjt.setBlock(fk3, ((AbstractStateHolder<O, BlockState>)byg).<Comparable, Boolean>setValue((Property<Comparable>)NoteBlock.POWERED, boolean7), 3);
        }
    }
    
    private void playNote(final Level bjt, final BlockPos fk) {
        if (bjt.getBlockState(fk.above()).isAir()) {
            bjt.blockEvent(fk, this, 0, 0);
        }
    }
    
    @Override
    public InteractionResult use(BlockState byg, final Level bjt, final BlockPos fk, final Player ayg, final InteractionHand ajh, final BlockHitResult cvd) {
        if (bjt.isClientSide) {
            return InteractionResult.SUCCESS;
        }
        byg = ((AbstractStateHolder<O, BlockState>)byg).<Comparable>cycle((Property<Comparable>)NoteBlock.NOTE);
        bjt.setBlock(fk, byg, 3);
        this.playNote(bjt, fk);
        ayg.awardStat(Stats.TUNE_NOTEBLOCK);
        return InteractionResult.SUCCESS;
    }
    
    @Override
    public void attack(final BlockState byg, final Level bjt, final BlockPos fk, final Player ayg) {
        if (bjt.isClientSide) {
            return;
        }
        this.playNote(bjt, fk);
        ayg.awardStat(Stats.PLAY_NOTEBLOCK);
    }
    
    @Override
    public boolean triggerEvent(final BlockState byg, final Level bjt, final BlockPos fk, final int integer4, final int integer5) {
        final int integer6 = byg.<Integer>getValue((Property<Integer>)NoteBlock.NOTE);
        final float float8 = (float)Math.pow(2.0, (integer6 - 12) / 12.0);
        bjt.playSound(null, fk, byg.<NoteBlockInstrument>getValue(NoteBlock.INSTRUMENT).getSoundEvent(), SoundSource.RECORDS, 3.0f, float8);
        bjt.addParticle(ParticleTypes.NOTE, fk.getX() + 0.5, fk.getY() + 1.2, fk.getZ() + 0.5, integer6 / 24.0, 0.0, 0.0);
        return true;
    }
    
    @Override
    protected void createBlockStateDefinition(final StateDefinition.Builder<Block, BlockState> a) {
        a.add(NoteBlock.INSTRUMENT, NoteBlock.POWERED, NoteBlock.NOTE);
    }
    
    static {
        INSTRUMENT = BlockStateProperties.NOTEBLOCK_INSTRUMENT;
        POWERED = BlockStateProperties.POWERED;
        NOTE = BlockStateProperties.NOTE;
    }
}
