package net.minecraft.world.level.block;

import net.minecraft.world.level.block.state.AbstractStateHolder;
import net.minecraft.world.level.block.state.StateDefinition;
import net.minecraft.core.particles.ParticleOptions;
import net.minecraft.core.particles.DustParticleOptions;
import java.util.Random;
import net.minecraft.world.level.Level;
import javax.annotation.Nullable;
import net.minecraft.world.item.BlockPlaceContext;
import net.minecraft.world.level.LevelAccessor;
import net.minecraft.world.level.LevelReader;
import net.minecraft.world.phys.shapes.VoxelShape;
import net.minecraft.world.phys.shapes.CollisionContext;
import net.minecraft.core.BlockPos;
import net.minecraft.world.level.BlockGetter;
import net.minecraft.world.level.block.state.properties.Property;
import net.minecraft.core.Direction;
import net.minecraft.world.level.block.state.BlockState;
import net.minecraft.world.level.block.state.properties.BooleanProperty;
import net.minecraft.world.level.block.state.properties.DirectionProperty;

public class RedstoneWallTorchBlock extends RedstoneTorchBlock {
    public static final DirectionProperty FACING;
    public static final BooleanProperty LIT;
    
    protected RedstoneWallTorchBlock(final Properties c) {
        super(c);
        this.registerDefaultState((((AbstractStateHolder<O, BlockState>)this.stateDefinition.any()).setValue((Property<Comparable>)RedstoneWallTorchBlock.FACING, Direction.NORTH)).<Comparable, Boolean>setValue((Property<Comparable>)RedstoneWallTorchBlock.LIT, true));
    }
    
    @Override
    public String getDescriptionId() {
        return this.asItem().getDescriptionId();
    }
    
    @Override
    public VoxelShape getShape(final BlockState byg, final BlockGetter bjd, final BlockPos fk, final CollisionContext cvn) {
        return WallTorchBlock.getShape(byg);
    }
    
    @Override
    public boolean canSurvive(final BlockState byg, final LevelReader bjw, final BlockPos fk) {
        return Blocks.WALL_TORCH.canSurvive(byg, bjw, fk);
    }
    
    @Override
    public BlockState updateShape(final BlockState byg1, final Direction fp, final BlockState byg3, final LevelAccessor bju, final BlockPos fk5, final BlockPos fk6) {
        return Blocks.WALL_TORCH.updateShape(byg1, fp, byg3, bju, fk5, fk6);
    }
    
    @Nullable
    @Override
    public BlockState getStateForPlacement(final BlockPlaceContext bcn) {
        final BlockState byg3 = Blocks.WALL_TORCH.getStateForPlacement(bcn);
        return (byg3 == null) ? null : ((AbstractStateHolder<O, BlockState>)this.defaultBlockState()).<Comparable, Comparable>setValue((Property<Comparable>)RedstoneWallTorchBlock.FACING, (Comparable)byg3.<V>getValue((Property<V>)RedstoneWallTorchBlock.FACING));
    }
    
    @Override
    public void animateTick(final BlockState byg, final Level bjt, final BlockPos fk, final Random random) {
        if (!byg.<Boolean>getValue((Property<Boolean>)RedstoneWallTorchBlock.LIT)) {
            return;
        }
        final Direction fp6 = byg.<Direction>getValue((Property<Direction>)RedstoneWallTorchBlock.FACING).getOpposite();
        final double double7 = 0.27;
        final double double8 = fk.getX() + 0.5 + (random.nextDouble() - 0.5) * 0.2 + 0.27 * fp6.getStepX();
        final double double9 = fk.getY() + 0.7 + (random.nextDouble() - 0.5) * 0.2 + 0.22;
        final double double10 = fk.getZ() + 0.5 + (random.nextDouble() - 0.5) * 0.2 + 0.27 * fp6.getStepZ();
        bjt.addParticle(DustParticleOptions.REDSTONE, double8, double9, double10, 0.0, 0.0, 0.0);
    }
    
    @Override
    protected boolean hasNeighborSignal(final Level bjt, final BlockPos fk, final BlockState byg) {
        final Direction fp5 = byg.<Direction>getValue((Property<Direction>)RedstoneWallTorchBlock.FACING).getOpposite();
        return bjt.hasSignal(fk.relative(fp5), fp5);
    }
    
    @Override
    public int getSignal(final BlockState byg, final BlockGetter bjd, final BlockPos fk, final Direction fp) {
        if (byg.<Boolean>getValue((Property<Boolean>)RedstoneWallTorchBlock.LIT) && byg.<Comparable>getValue((Property<Comparable>)RedstoneWallTorchBlock.FACING) != fp) {
            return 15;
        }
        return 0;
    }
    
    @Override
    public BlockState rotate(final BlockState byg, final Rotation btr) {
        return Blocks.WALL_TORCH.rotate(byg, btr);
    }
    
    @Override
    public BlockState mirror(final BlockState byg, final Mirror bsr) {
        return Blocks.WALL_TORCH.mirror(byg, bsr);
    }
    
    @Override
    protected void createBlockStateDefinition(final StateDefinition.Builder<Block, BlockState> a) {
        a.add(RedstoneWallTorchBlock.FACING, RedstoneWallTorchBlock.LIT);
    }
    
    static {
        FACING = HorizontalDirectionalBlock.FACING;
        LIT = RedstoneTorchBlock.LIT;
    }
}
