package net.minecraft.world.level.block;

import net.minecraft.world.level.block.state.AbstractStateHolder;
import net.minecraft.world.level.block.state.properties.BlockStateProperties;
import java.util.Iterator;
import net.minecraft.core.Vec3i;
import net.minecraft.world.level.material.FluidState;
import net.minecraft.world.entity.Entity;
import net.minecraft.world.entity.item.FallingBlockEntity;
import java.util.Random;
import net.minecraft.server.level.ServerLevel;
import net.minecraft.world.level.LevelReader;
import net.minecraft.world.level.LevelAccessor;
import net.minecraft.core.Direction;
import net.minecraft.world.level.Level;
import net.minecraft.world.level.material.Fluids;
import net.minecraft.world.item.BlockPlaceContext;
import net.minecraft.world.phys.shapes.Shapes;
import net.minecraft.world.phys.shapes.CollisionContext;
import net.minecraft.core.BlockPos;
import net.minecraft.world.level.BlockGetter;
import net.minecraft.world.level.block.state.StateDefinition;
import net.minecraft.world.level.block.state.properties.Property;
import net.minecraft.world.level.block.state.BlockState;
import net.minecraft.world.level.block.state.properties.BooleanProperty;
import net.minecraft.world.level.block.state.properties.IntegerProperty;
import net.minecraft.world.phys.shapes.VoxelShape;

public class ScaffoldingBlock extends Block implements SimpleWaterloggedBlock {
    private static final VoxelShape STABLE_SHAPE;
    private static final VoxelShape UNSTABLE_SHAPE;
    private static final VoxelShape UNSTABLE_SHAPE_BOTTOM;
    private static final VoxelShape BELOW_BLOCK;
    public static final IntegerProperty DISTANCE;
    public static final BooleanProperty WATERLOGGED;
    public static final BooleanProperty BOTTOM;
    
    protected ScaffoldingBlock(final Properties c) {
        super(c);
        this.registerDefaultState(((((AbstractStateHolder<O, BlockState>)this.stateDefinition.any()).setValue((Property<Comparable>)ScaffoldingBlock.DISTANCE, 7)).setValue((Property<Comparable>)ScaffoldingBlock.WATERLOGGED, false)).<Comparable, Boolean>setValue((Property<Comparable>)ScaffoldingBlock.BOTTOM, false));
    }
    
    @Override
    protected void createBlockStateDefinition(final StateDefinition.Builder<Block, BlockState> a) {
        a.add(ScaffoldingBlock.DISTANCE, ScaffoldingBlock.WATERLOGGED, ScaffoldingBlock.BOTTOM);
    }
    
    @Override
    public VoxelShape getShape(final BlockState byg, final BlockGetter bjd, final BlockPos fk, final CollisionContext cvn) {
        if (!cvn.isHoldingItem(byg.getBlock().asItem())) {
            return byg.<Boolean>getValue((Property<Boolean>)ScaffoldingBlock.BOTTOM) ? ScaffoldingBlock.UNSTABLE_SHAPE : ScaffoldingBlock.STABLE_SHAPE;
        }
        return Shapes.block();
    }
    
    @Override
    public VoxelShape getInteractionShape(final BlockState byg, final BlockGetter bjd, final BlockPos fk) {
        return Shapes.block();
    }
    
    @Override
    public boolean canBeReplaced(final BlockState byg, final BlockPlaceContext bcn) {
        return bcn.getItemInHand().getItem() == this.asItem();
    }
    
    @Override
    public BlockState getStateForPlacement(final BlockPlaceContext bcn) {
        final BlockPos fk3 = bcn.getClickedPos();
        final Level bjt4 = bcn.getLevel();
        final int integer5 = getDistance(bjt4, fk3);
        return ((((AbstractStateHolder<O, BlockState>)this.defaultBlockState()).setValue((Property<Comparable>)ScaffoldingBlock.WATERLOGGED, bjt4.getFluidState(fk3).getType() == Fluids.WATER)).setValue((Property<Comparable>)ScaffoldingBlock.DISTANCE, integer5)).<Comparable, Boolean>setValue((Property<Comparable>)ScaffoldingBlock.BOTTOM, this.isBottom(bjt4, fk3, integer5));
    }
    
    @Override
    public void onPlace(final BlockState byg1, final Level bjt, final BlockPos fk, final BlockState byg4, final boolean boolean5) {
        if (!bjt.isClientSide) {
            bjt.getBlockTicks().scheduleTick(fk, this, 1);
        }
    }
    
    @Override
    public BlockState updateShape(final BlockState byg1, final Direction fp, final BlockState byg3, final LevelAccessor bju, final BlockPos fk5, final BlockPos fk6) {
        if (byg1.<Boolean>getValue((Property<Boolean>)ScaffoldingBlock.WATERLOGGED)) {
            bju.getLiquidTicks().scheduleTick(fk5, Fluids.WATER, Fluids.WATER.getTickDelay(bju));
        }
        if (!bju.isClientSide()) {
            bju.getBlockTicks().scheduleTick(fk5, this, 1);
        }
        return byg1;
    }
    
    @Override
    public void tick(final BlockState byg, final ServerLevel xd, final BlockPos fk, final Random random) {
        final int integer6 = getDistance(xd, fk);
        final BlockState byg2 = (((AbstractStateHolder<O, BlockState>)byg).setValue((Property<Comparable>)ScaffoldingBlock.DISTANCE, integer6)).<Comparable, Boolean>setValue((Property<Comparable>)ScaffoldingBlock.BOTTOM, this.isBottom(xd, fk, integer6));
        if (byg2.<Integer>getValue((Property<Integer>)ScaffoldingBlock.DISTANCE) == 7) {
            if (byg.<Integer>getValue((Property<Integer>)ScaffoldingBlock.DISTANCE) == 7) {
                xd.addFreshEntity(new FallingBlockEntity(xd, fk.getX() + 0.5, fk.getY(), fk.getZ() + 0.5, ((AbstractStateHolder<O, BlockState>)byg2).<Comparable, Boolean>setValue((Property<Comparable>)ScaffoldingBlock.WATERLOGGED, false)));
            }
            else {
                xd.destroyBlock(fk, true);
            }
        }
        else if (byg != byg2) {
            xd.setBlock(fk, byg2, 3);
        }
    }
    
    @Override
    public boolean canSurvive(final BlockState byg, final LevelReader bjw, final BlockPos fk) {
        return getDistance(bjw, fk) < 7;
    }
    
    @Override
    public VoxelShape getCollisionShape(final BlockState byg, final BlockGetter bjd, final BlockPos fk, final CollisionContext cvn) {
        if (cvn.isAbove(Shapes.block(), fk, true) && !cvn.isDescending()) {
            return ScaffoldingBlock.STABLE_SHAPE;
        }
        if (byg.<Integer>getValue((Property<Integer>)ScaffoldingBlock.DISTANCE) != 0 && byg.<Boolean>getValue((Property<Boolean>)ScaffoldingBlock.BOTTOM) && cvn.isAbove(ScaffoldingBlock.BELOW_BLOCK, fk, true)) {
            return ScaffoldingBlock.UNSTABLE_SHAPE_BOTTOM;
        }
        return Shapes.empty();
    }
    
    @Override
    public FluidState getFluidState(final BlockState byg) {
        if (byg.<Boolean>getValue((Property<Boolean>)ScaffoldingBlock.WATERLOGGED)) {
            return Fluids.WATER.getSource(false);
        }
        return super.getFluidState(byg);
    }
    
    private boolean isBottom(final BlockGetter bjd, final BlockPos fk, final int integer) {
        return integer > 0 && bjd.getBlockState(fk.below()).getBlock() != this;
    }
    
    public static int getDistance(final BlockGetter bjd, final BlockPos fk) {
        final BlockPos.MutableBlockPos a3 = new BlockPos.MutableBlockPos(fk).move(Direction.DOWN);
        final BlockState byg4 = bjd.getBlockState(a3);
        int integer5 = 7;
        if (byg4.getBlock() == Blocks.SCAFFOLDING) {
            integer5 = byg4.<Integer>getValue((Property<Integer>)ScaffoldingBlock.DISTANCE);
        }
        else if (byg4.isFaceSturdy(bjd, a3, Direction.UP)) {
            return 0;
        }
        for (final Direction fp7 : Direction.Plane.HORIZONTAL) {
            final BlockState byg5 = bjd.getBlockState(a3.set(fk).move(fp7));
            if (byg5.getBlock() != Blocks.SCAFFOLDING) {
                continue;
            }
            integer5 = Math.min(integer5, byg5.<Integer>getValue((Property<Integer>)ScaffoldingBlock.DISTANCE) + 1);
            if (integer5 == 1) {
                break;
            }
        }
        return integer5;
    }
    
    static {
        UNSTABLE_SHAPE_BOTTOM = Block.box(0.0, 0.0, 0.0, 16.0, 2.0, 16.0);
        BELOW_BLOCK = Shapes.block().move(0.0, -1.0, 0.0);
        DISTANCE = BlockStateProperties.STABILITY_DISTANCE;
        WATERLOGGED = BlockStateProperties.WATERLOGGED;
        BOTTOM = BlockStateProperties.BOTTOM;
        final VoxelShape cwc1 = Block.box(0.0, 14.0, 0.0, 16.0, 16.0, 16.0);
        final VoxelShape cwc2 = Block.box(0.0, 0.0, 0.0, 2.0, 16.0, 2.0);
        final VoxelShape cwc3 = Block.box(14.0, 0.0, 0.0, 16.0, 16.0, 2.0);
        final VoxelShape cwc4 = Block.box(0.0, 0.0, 14.0, 2.0, 16.0, 16.0);
        final VoxelShape cwc5 = Block.box(14.0, 0.0, 14.0, 16.0, 16.0, 16.0);
        STABLE_SHAPE = Shapes.or(cwc1, cwc2, cwc3, cwc4, cwc5);
        final VoxelShape cwc6 = Block.box(0.0, 0.0, 0.0, 2.0, 2.0, 16.0);
        final VoxelShape cwc7 = Block.box(14.0, 0.0, 0.0, 16.0, 2.0, 16.0);
        final VoxelShape cwc8 = Block.box(0.0, 0.0, 14.0, 16.0, 2.0, 16.0);
        final VoxelShape cwc9 = Block.box(0.0, 0.0, 0.0, 16.0, 2.0, 2.0);
        UNSTABLE_SHAPE = Shapes.or(ScaffoldingBlock.UNSTABLE_SHAPE_BOTTOM, ScaffoldingBlock.STABLE_SHAPE, cwc7, cwc6, cwc9, cwc8);
    }
}
