package net.minecraft.world.level.block;

import net.minecraft.world.level.block.state.AbstractStateHolder;
import net.minecraft.world.level.block.state.properties.BlockStateProperties;
import net.minecraft.tags.BlockTags;
import net.minecraft.server.level.ServerLevel;
import java.util.Random;
import net.minecraft.world.level.Level;
import net.minecraft.world.level.block.state.StateDefinition;
import net.minecraft.world.phys.shapes.CollisionContext;
import net.minecraft.world.level.material.Fluids;
import net.minecraft.world.level.LevelAccessor;
import net.minecraft.world.level.LevelReader;
import net.minecraft.core.Direction;
import net.minecraft.core.BlockPos;
import net.minecraft.world.level.BlockGetter;
import javax.annotation.Nullable;
import net.minecraft.world.level.material.FluidState;
import net.minecraft.tags.FluidTags;
import net.minecraft.world.item.BlockPlaceContext;
import net.minecraft.world.level.block.state.properties.Property;
import net.minecraft.world.level.block.state.BlockState;
import net.minecraft.world.phys.shapes.VoxelShape;
import net.minecraft.world.level.block.state.properties.BooleanProperty;
import net.minecraft.world.level.block.state.properties.IntegerProperty;

public class SeaPickleBlock extends BushBlock implements BonemealableBlock, SimpleWaterloggedBlock {
    public static final IntegerProperty PICKLES;
    public static final BooleanProperty WATERLOGGED;
    protected static final VoxelShape ONE_AABB;
    protected static final VoxelShape TWO_AABB;
    protected static final VoxelShape THREE_AABB;
    protected static final VoxelShape FOUR_AABB;
    
    protected SeaPickleBlock(final Properties c) {
        super(c);
        this.registerDefaultState((((AbstractStateHolder<O, BlockState>)this.stateDefinition.any()).setValue((Property<Comparable>)SeaPickleBlock.PICKLES, 1)).<Comparable, Boolean>setValue((Property<Comparable>)SeaPickleBlock.WATERLOGGED, true));
    }
    
    @Override
    public int getLightEmission(final BlockState byg) {
        return this.isDead(byg) ? 0 : (super.getLightEmission(byg) + 3 * byg.<Integer>getValue((Property<Integer>)SeaPickleBlock.PICKLES));
    }
    
    @Nullable
    @Override
    public BlockState getStateForPlacement(final BlockPlaceContext bcn) {
        final BlockState byg3 = bcn.getLevel().getBlockState(bcn.getClickedPos());
        if (byg3.getBlock() == this) {
            return ((AbstractStateHolder<O, BlockState>)byg3).<Comparable, Integer>setValue((Property<Comparable>)SeaPickleBlock.PICKLES, Math.min(4, byg3.<Integer>getValue((Property<Integer>)SeaPickleBlock.PICKLES) + 1));
        }
        final FluidState cog4 = bcn.getLevel().getFluidState(bcn.getClickedPos());
        final boolean boolean5 = cog4.is(FluidTags.WATER) && cog4.getAmount() == 8;
        return ((AbstractStateHolder<O, BlockState>)super.getStateForPlacement(bcn)).<Comparable, Boolean>setValue((Property<Comparable>)SeaPickleBlock.WATERLOGGED, boolean5);
    }
    
    private boolean isDead(final BlockState byg) {
        return !byg.<Boolean>getValue((Property<Boolean>)SeaPickleBlock.WATERLOGGED);
    }
    
    @Override
    protected boolean mayPlaceOn(final BlockState byg, final BlockGetter bjd, final BlockPos fk) {
        return !byg.getCollisionShape(bjd, fk).getFaceShape(Direction.UP).isEmpty();
    }
    
    @Override
    public boolean canSurvive(final BlockState byg, final LevelReader bjw, final BlockPos fk) {
        final BlockPos fk2 = fk.below();
        return this.mayPlaceOn(bjw.getBlockState(fk2), bjw, fk2);
    }
    
    @Override
    public BlockState updateShape(final BlockState byg1, final Direction fp, final BlockState byg3, final LevelAccessor bju, final BlockPos fk5, final BlockPos fk6) {
        if (!byg1.canSurvive(bju, fk5)) {
            return Blocks.AIR.defaultBlockState();
        }
        if (byg1.<Boolean>getValue((Property<Boolean>)SeaPickleBlock.WATERLOGGED)) {
            bju.getLiquidTicks().scheduleTick(fk5, Fluids.WATER, Fluids.WATER.getTickDelay(bju));
        }
        return super.updateShape(byg1, fp, byg3, bju, fk5, fk6);
    }
    
    @Override
    public boolean canBeReplaced(final BlockState byg, final BlockPlaceContext bcn) {
        return (bcn.getItemInHand().getItem() == this.asItem() && byg.<Integer>getValue((Property<Integer>)SeaPickleBlock.PICKLES) < 4) || super.canBeReplaced(byg, bcn);
    }
    
    @Override
    public VoxelShape getShape(final BlockState byg, final BlockGetter bjd, final BlockPos fk, final CollisionContext cvn) {
        switch (byg.<Integer>getValue((Property<Integer>)SeaPickleBlock.PICKLES)) {
            default: {
                return SeaPickleBlock.ONE_AABB;
            }
            case 2: {
                return SeaPickleBlock.TWO_AABB;
            }
            case 3: {
                return SeaPickleBlock.THREE_AABB;
            }
            case 4: {
                return SeaPickleBlock.FOUR_AABB;
            }
        }
    }
    
    @Override
    public FluidState getFluidState(final BlockState byg) {
        if (byg.<Boolean>getValue((Property<Boolean>)SeaPickleBlock.WATERLOGGED)) {
            return Fluids.WATER.getSource(false);
        }
        return super.getFluidState(byg);
    }
    
    @Override
    protected void createBlockStateDefinition(final StateDefinition.Builder<Block, BlockState> a) {
        a.add(SeaPickleBlock.PICKLES, SeaPickleBlock.WATERLOGGED);
    }
    
    @Override
    public boolean isValidBonemealTarget(final BlockGetter bjd, final BlockPos fk, final BlockState byg, final boolean boolean4) {
        return true;
    }
    
    @Override
    public boolean isBonemealSuccess(final Level bjt, final Random random, final BlockPos fk, final BlockState byg) {
        return true;
    }
    
    @Override
    public void performBonemeal(final ServerLevel xd, final Random random, final BlockPos fk, final BlockState byg) {
        if (!this.isDead(byg) && xd.getBlockState(fk.below()).is(BlockTags.CORAL_BLOCKS)) {
            final int integer6 = 5;
            int integer7 = 1;
            final int integer8 = 2;
            int integer9 = 0;
            final int integer10 = fk.getX() - 2;
            int integer11 = 0;
            for (int integer12 = 0; integer12 < 5; ++integer12) {
                for (int integer13 = 0; integer13 < integer7; ++integer13) {
                    for (int integer14 = 2 + fk.getY() - 1, integer15 = integer14 - 2; integer15 < integer14; ++integer15) {
                        final BlockPos fk2 = new BlockPos(integer10 + integer12, integer15, fk.getZ() - integer11 + integer13);
                        if (fk2 != fk) {
                            if (random.nextInt(6) == 0 && xd.getBlockState(fk2).getBlock() == Blocks.WATER) {
                                final BlockState byg2 = xd.getBlockState(fk2.below());
                                if (byg2.is(BlockTags.CORAL_BLOCKS)) {
                                    xd.setBlock(fk2, ((AbstractStateHolder<O, BlockState>)Blocks.SEA_PICKLE.defaultBlockState()).<Comparable, Integer>setValue((Property<Comparable>)SeaPickleBlock.PICKLES, random.nextInt(4) + 1), 3);
                                }
                            }
                        }
                    }
                }
                if (integer9 < 2) {
                    integer7 += 2;
                    ++integer11;
                }
                else {
                    integer7 -= 2;
                    --integer11;
                }
                ++integer9;
            }
            xd.setBlock(fk, ((AbstractStateHolder<O, BlockState>)byg).<Comparable, Integer>setValue((Property<Comparable>)SeaPickleBlock.PICKLES, 4), 2);
        }
    }
    
    static {
        PICKLES = BlockStateProperties.PICKLES;
        WATERLOGGED = BlockStateProperties.WATERLOGGED;
        ONE_AABB = Block.box(6.0, 0.0, 6.0, 10.0, 6.0, 10.0);
        TWO_AABB = Block.box(3.0, 0.0, 3.0, 13.0, 6.0, 13.0);
        THREE_AABB = Block.box(2.0, 0.0, 2.0, 14.0, 6.0, 14.0);
        FOUR_AABB = Block.box(2.0, 0.0, 2.0, 14.0, 7.0, 14.0);
    }
}
