package net.minecraft.world.level.block;

import net.minecraft.world.level.block.state.properties.BlockStateProperties;
import net.minecraft.world.level.material.FluidState;
import net.minecraft.world.item.ItemStack;
import net.minecraft.world.item.DyeItem;
import net.minecraft.world.InteractionResult;
import net.minecraft.world.phys.BlockHitResult;
import net.minecraft.world.InteractionHand;
import net.minecraft.world.entity.player.Player;
import net.minecraft.world.level.Level;
import net.minecraft.world.level.block.entity.SignBlockEntity;
import net.minecraft.world.level.block.entity.BlockEntity;
import net.minecraft.world.phys.shapes.CollisionContext;
import net.minecraft.world.level.BlockGetter;
import net.minecraft.world.level.LevelReader;
import net.minecraft.world.level.material.Fluids;
import net.minecraft.world.level.block.state.properties.Property;
import net.minecraft.core.BlockPos;
import net.minecraft.world.level.LevelAccessor;
import net.minecraft.core.Direction;
import net.minecraft.world.level.block.state.BlockState;
import net.minecraft.world.level.block.state.properties.WoodType;
import net.minecraft.world.phys.shapes.VoxelShape;
import net.minecraft.world.level.block.state.properties.BooleanProperty;

public abstract class SignBlock extends BaseEntityBlock implements SimpleWaterloggedBlock {
    public static final BooleanProperty WATERLOGGED;
    protected static final VoxelShape SHAPE;
    private final WoodType type;
    
    protected SignBlock(final Properties c, final WoodType bzp) {
        super(c);
        this.type = bzp;
    }
    
    @Override
    public BlockState updateShape(final BlockState byg1, final Direction fp, final BlockState byg3, final LevelAccessor bju, final BlockPos fk5, final BlockPos fk6) {
        if (byg1.<Boolean>getValue((Property<Boolean>)SignBlock.WATERLOGGED)) {
            bju.getLiquidTicks().scheduleTick(fk5, Fluids.WATER, Fluids.WATER.getTickDelay(bju));
        }
        return super.updateShape(byg1, fp, byg3, bju, fk5, fk6);
    }
    
    @Override
    public VoxelShape getShape(final BlockState byg, final BlockGetter bjd, final BlockPos fk, final CollisionContext cvn) {
        return SignBlock.SHAPE;
    }
    
    @Override
    public boolean isPossibleToRespawnInThis() {
        return true;
    }
    
    @Override
    public BlockEntity newBlockEntity(final BlockGetter bjd) {
        return new SignBlockEntity();
    }
    
    @Override
    public InteractionResult use(final BlockState byg, final Level bjt, final BlockPos fk, final Player ayg, final InteractionHand ajh, final BlockHitResult cvd) {
        final ItemStack bek8 = ayg.getItemInHand(ajh);
        final boolean boolean9 = bek8.getItem() instanceof DyeItem && ayg.abilities.mayBuild;
        if (bjt.isClientSide) {
            return boolean9 ? InteractionResult.SUCCESS : InteractionResult.CONSUME;
        }
        final BlockEntity bwi10 = bjt.getBlockEntity(fk);
        if (bwi10 instanceof SignBlockEntity) {
            final SignBlockEntity bxe11 = (SignBlockEntity)bwi10;
            if (boolean9) {
                final boolean boolean10 = bxe11.setColor(((DyeItem)bek8.getItem()).getDyeColor());
                if (boolean10 && !ayg.isCreative()) {
                    bek8.shrink(1);
                }
            }
            return bxe11.executeClickCommands(ayg) ? InteractionResult.SUCCESS : InteractionResult.PASS;
        }
        return InteractionResult.PASS;
    }
    
    @Override
    public FluidState getFluidState(final BlockState byg) {
        if (byg.<Boolean>getValue((Property<Boolean>)SignBlock.WATERLOGGED)) {
            return Fluids.WATER.getSource(false);
        }
        return super.getFluidState(byg);
    }
    
    public WoodType type() {
        return this.type;
    }
    
    static {
        WATERLOGGED = BlockStateProperties.WATERLOGGED;
        SHAPE = Block.box(4.0, 0.0, 4.0, 12.0, 16.0, 12.0);
    }
}
