package net.minecraft.world.level.block;

import net.minecraft.world.level.block.state.AbstractStateHolder;
import net.minecraft.world.level.block.state.properties.BlockStateProperties;
import net.minecraft.tags.FluidTags;
import net.minecraft.world.level.pathfinder.PathComputationType;
import net.minecraft.world.level.LevelReader;
import net.minecraft.world.level.material.Fluid;
import net.minecraft.world.level.LevelAccessor;
import net.minecraft.world.item.ItemStack;
import javax.annotation.Nullable;
import net.minecraft.world.level.material.FluidState;
import net.minecraft.core.Direction;
import net.minecraft.world.level.material.Fluids;
import net.minecraft.world.item.BlockPlaceContext;
import net.minecraft.world.phys.shapes.Shapes;
import net.minecraft.world.phys.shapes.CollisionContext;
import net.minecraft.core.BlockPos;
import net.minecraft.world.level.BlockGetter;
import net.minecraft.world.level.block.state.StateDefinition;
import net.minecraft.world.level.block.state.properties.Property;
import net.minecraft.world.level.block.state.BlockState;
import net.minecraft.world.phys.shapes.VoxelShape;
import net.minecraft.world.level.block.state.properties.BooleanProperty;
import net.minecraft.world.level.block.state.properties.SlabType;
import net.minecraft.world.level.block.state.properties.EnumProperty;

public class SlabBlock extends Block implements SimpleWaterloggedBlock {
    public static final EnumProperty<SlabType> TYPE;
    public static final BooleanProperty WATERLOGGED;
    protected static final VoxelShape BOTTOM_AABB;
    protected static final VoxelShape TOP_AABB;
    
    public SlabBlock(final Properties c) {
        super(c);
        this.registerDefaultState((((AbstractStateHolder<O, BlockState>)this.defaultBlockState()).setValue(SlabBlock.TYPE, SlabType.BOTTOM)).<Comparable, Boolean>setValue((Property<Comparable>)SlabBlock.WATERLOGGED, false));
    }
    
    @Override
    public boolean useShapeForLightOcclusion(final BlockState byg) {
        return byg.<SlabType>getValue(SlabBlock.TYPE) != SlabType.DOUBLE;
    }
    
    @Override
    protected void createBlockStateDefinition(final StateDefinition.Builder<Block, BlockState> a) {
        a.add(SlabBlock.TYPE, SlabBlock.WATERLOGGED);
    }
    
    @Override
    public VoxelShape getShape(final BlockState byg, final BlockGetter bjd, final BlockPos fk, final CollisionContext cvn) {
        final SlabType bzm6 = byg.<SlabType>getValue(SlabBlock.TYPE);
        switch (bzm6) {
            case DOUBLE: {
                return Shapes.block();
            }
            case TOP: {
                return SlabBlock.TOP_AABB;
            }
            default: {
                return SlabBlock.BOTTOM_AABB;
            }
        }
    }
    
    @Nullable
    @Override
    public BlockState getStateForPlacement(final BlockPlaceContext bcn) {
        final BlockPos fk3 = bcn.getClickedPos();
        final BlockState byg4 = bcn.getLevel().getBlockState(fk3);
        if (byg4.getBlock() == this) {
            return (((AbstractStateHolder<O, BlockState>)byg4).setValue(SlabBlock.TYPE, SlabType.DOUBLE)).<Comparable, Boolean>setValue((Property<Comparable>)SlabBlock.WATERLOGGED, false);
        }
        final FluidState cog5 = bcn.getLevel().getFluidState(fk3);
        final BlockState byg5 = (((AbstractStateHolder<O, BlockState>)this.defaultBlockState()).setValue(SlabBlock.TYPE, SlabType.BOTTOM)).<Comparable, Boolean>setValue((Property<Comparable>)SlabBlock.WATERLOGGED, cog5.getType() == Fluids.WATER);
        final Direction fp7 = bcn.getClickedFace();
        if (fp7 == Direction.DOWN || (fp7 != Direction.UP && bcn.getClickLocation().y - fk3.getY() > 0.5)) {
            return ((AbstractStateHolder<O, BlockState>)byg5).<SlabType, SlabType>setValue(SlabBlock.TYPE, SlabType.TOP);
        }
        return byg5;
    }
    
    @Override
    public boolean canBeReplaced(final BlockState byg, final BlockPlaceContext bcn) {
        final ItemStack bek4 = bcn.getItemInHand();
        final SlabType bzm5 = byg.<SlabType>getValue(SlabBlock.TYPE);
        if (bzm5 == SlabType.DOUBLE || bek4.getItem() != this.asItem()) {
            return false;
        }
        if (!bcn.replacingClickedOnBlock()) {
            return true;
        }
        final boolean boolean6 = bcn.getClickLocation().y - bcn.getClickedPos().getY() > 0.5;
        final Direction fp7 = bcn.getClickedFace();
        if (bzm5 == SlabType.BOTTOM) {
            return fp7 == Direction.UP || (boolean6 && fp7.getAxis().isHorizontal());
        }
        return fp7 == Direction.DOWN || (!boolean6 && fp7.getAxis().isHorizontal());
    }
    
    @Override
    public FluidState getFluidState(final BlockState byg) {
        if (byg.<Boolean>getValue((Property<Boolean>)SlabBlock.WATERLOGGED)) {
            return Fluids.WATER.getSource(false);
        }
        return super.getFluidState(byg);
    }
    
    @Override
    public boolean placeLiquid(final LevelAccessor bju, final BlockPos fk, final BlockState byg, final FluidState cog) {
        return byg.<SlabType>getValue(SlabBlock.TYPE) != SlabType.DOUBLE && super.placeLiquid(bju, fk, byg, cog);
    }
    
    @Override
    public boolean canPlaceLiquid(final BlockGetter bjd, final BlockPos fk, final BlockState byg, final Fluid cof) {
        return byg.<SlabType>getValue(SlabBlock.TYPE) != SlabType.DOUBLE && super.canPlaceLiquid(bjd, fk, byg, cof);
    }
    
    @Override
    public BlockState updateShape(final BlockState byg1, final Direction fp, final BlockState byg3, final LevelAccessor bju, final BlockPos fk5, final BlockPos fk6) {
        if (byg1.<Boolean>getValue((Property<Boolean>)SlabBlock.WATERLOGGED)) {
            bju.getLiquidTicks().scheduleTick(fk5, Fluids.WATER, Fluids.WATER.getTickDelay(bju));
        }
        return super.updateShape(byg1, fp, byg3, bju, fk5, fk6);
    }
    
    @Override
    public boolean isPathfindable(final BlockState byg, final BlockGetter bjd, final BlockPos fk, final PathComputationType cqo) {
        switch (cqo) {
            case LAND: {
                return false;
            }
            case WATER: {
                return bjd.getFluidState(fk).is(FluidTags.WATER);
            }
            case AIR: {
                return false;
            }
            default: {
                return false;
            }
        }
    }
    
    static {
        TYPE = BlockStateProperties.SLAB_TYPE;
        WATERLOGGED = BlockStateProperties.WATERLOGGED;
        BOTTOM_AABB = Block.box(0.0, 0.0, 0.0, 16.0, 8.0, 16.0);
        TOP_AABB = Block.box(0.0, 8.0, 0.0, 16.0, 16.0, 16.0);
    }
}
