package net.minecraft.world.level.block;

import net.minecraft.world.level.block.state.AbstractStateHolder;
import net.minecraft.world.level.block.state.properties.BlockStateProperties;
import net.minecraft.world.level.pathfinder.PathComputationType;
import net.minecraft.world.level.block.state.StateDefinition;
import net.minecraft.world.level.material.FluidState;
import net.minecraft.world.level.material.Fluids;
import net.minecraft.world.item.BlockPlaceContext;
import net.minecraft.world.level.Explosion;
import net.minecraft.world.InteractionResult;
import net.minecraft.world.phys.BlockHitResult;
import net.minecraft.world.InteractionHand;
import net.minecraft.server.level.ServerLevel;
import net.minecraft.world.entity.Entity;
import net.minecraft.world.level.LevelReader;
import net.minecraft.world.level.LevelAccessor;
import net.minecraft.world.entity.player.Player;
import java.util.Random;
import net.minecraft.world.level.Level;
import net.minecraft.world.phys.shapes.CollisionContext;
import net.minecraft.core.BlockPos;
import net.minecraft.world.level.BlockGetter;
import net.minecraft.world.level.block.state.properties.Property;
import net.minecraft.core.Direction;
import net.minecraft.world.phys.shapes.Shapes;
import java.util.stream.IntStream;
import net.minecraft.world.level.block.state.BlockState;
import net.minecraft.world.phys.shapes.VoxelShape;
import net.minecraft.world.level.block.state.properties.BooleanProperty;
import net.minecraft.world.level.block.state.properties.StairsShape;
import net.minecraft.world.level.block.state.properties.Half;
import net.minecraft.world.level.block.state.properties.EnumProperty;
import net.minecraft.world.level.block.state.properties.DirectionProperty;

public class StairBlock extends Block implements SimpleWaterloggedBlock {
    public static final DirectionProperty FACING;
    public static final EnumProperty<Half> HALF;
    public static final EnumProperty<StairsShape> SHAPE;
    public static final BooleanProperty WATERLOGGED;
    protected static final VoxelShape TOP_AABB;
    protected static final VoxelShape BOTTOM_AABB;
    protected static final VoxelShape OCTET_NNN;
    protected static final VoxelShape OCTET_NNP;
    protected static final VoxelShape OCTET_NPN;
    protected static final VoxelShape OCTET_NPP;
    protected static final VoxelShape OCTET_PNN;
    protected static final VoxelShape OCTET_PNP;
    protected static final VoxelShape OCTET_PPN;
    protected static final VoxelShape OCTET_PPP;
    protected static final VoxelShape[] TOP_SHAPES;
    protected static final VoxelShape[] BOTTOM_SHAPES;
    private static final int[] SHAPE_BY_STATE;
    private final Block base;
    private final BlockState baseState;
    
    private static VoxelShape[] makeShapes(final VoxelShape cwc1, final VoxelShape cwc2, final VoxelShape cwc3, final VoxelShape cwc4, final VoxelShape cwc5) {
        return IntStream.range(0, 16).mapToObj(integer -> makeStairShape(integer, cwc1, cwc2, cwc3, cwc4, cwc5)).<VoxelShape>toArray(VoxelShape[]::new);
    }
    
    private static VoxelShape makeStairShape(final int integer, final VoxelShape cwc2, final VoxelShape cwc3, final VoxelShape cwc4, final VoxelShape cwc5, final VoxelShape cwc6) {
        VoxelShape cwc7 = cwc2;
        if ((integer & 0x1) != 0x0) {
            cwc7 = Shapes.or(cwc7, cwc3);
        }
        if ((integer & 0x2) != 0x0) {
            cwc7 = Shapes.or(cwc7, cwc4);
        }
        if ((integer & 0x4) != 0x0) {
            cwc7 = Shapes.or(cwc7, cwc5);
        }
        if ((integer & 0x8) != 0x0) {
            cwc7 = Shapes.or(cwc7, cwc6);
        }
        return cwc7;
    }
    
    protected StairBlock(final BlockState byg, final Properties c) {
        super(c);
        this.registerDefaultState((((((AbstractStateHolder<O, BlockState>)this.stateDefinition.any()).setValue((Property<Comparable>)StairBlock.FACING, Direction.NORTH)).setValue(StairBlock.HALF, Half.BOTTOM)).setValue(StairBlock.SHAPE, StairsShape.STRAIGHT)).<Comparable, Boolean>setValue((Property<Comparable>)StairBlock.WATERLOGGED, false));
        this.base = byg.getBlock();
        this.baseState = byg;
    }
    
    @Override
    public boolean useShapeForLightOcclusion(final BlockState byg) {
        return true;
    }
    
    @Override
    public VoxelShape getShape(final BlockState byg, final BlockGetter bjd, final BlockPos fk, final CollisionContext cvn) {
        return ((byg.<Half>getValue(StairBlock.HALF) == Half.TOP) ? StairBlock.TOP_SHAPES : StairBlock.BOTTOM_SHAPES)[StairBlock.SHAPE_BY_STATE[this.getShapeIndex(byg)]];
    }
    
    private int getShapeIndex(final BlockState byg) {
        return byg.<StairsShape>getValue(StairBlock.SHAPE).ordinal() * 4 + byg.<Direction>getValue((Property<Direction>)StairBlock.FACING).get2DDataValue();
    }
    
    @Override
    public void animateTick(final BlockState byg, final Level bjt, final BlockPos fk, final Random random) {
        this.base.animateTick(byg, bjt, fk, random);
    }
    
    @Override
    public void attack(final BlockState byg, final Level bjt, final BlockPos fk, final Player ayg) {
        this.baseState.attack(bjt, fk, ayg);
    }
    
    @Override
    public void destroy(final LevelAccessor bju, final BlockPos fk, final BlockState byg) {
        this.base.destroy(bju, fk, byg);
    }
    
    @Override
    public float getExplosionResistance() {
        return this.base.getExplosionResistance();
    }
    
    @Override
    public int getTickDelay(final LevelReader bjw) {
        return this.base.getTickDelay(bjw);
    }
    
    @Override
    public void onPlace(final BlockState byg1, final Level bjt, final BlockPos fk, final BlockState byg4, final boolean boolean5) {
        if (byg1.getBlock() == byg1.getBlock()) {
            return;
        }
        this.baseState.neighborChanged(bjt, fk, Blocks.AIR, fk, false);
        this.base.onPlace(this.baseState, bjt, fk, byg4, false);
    }
    
    @Override
    public void onRemove(final BlockState byg1, final Level bjt, final BlockPos fk, final BlockState byg4, final boolean boolean5) {
        if (byg1.getBlock() == byg4.getBlock()) {
            return;
        }
        this.baseState.onRemove(bjt, fk, byg4, boolean5);
    }
    
    @Override
    public void stepOn(final Level bjt, final BlockPos fk, final Entity akn) {
        this.base.stepOn(bjt, fk, akn);
    }
    
    @Override
    public void tick(final BlockState byg, final ServerLevel xd, final BlockPos fk, final Random random) {
        this.base.tick(byg, xd, fk, random);
    }
    
    @Override
    public InteractionResult use(final BlockState byg, final Level bjt, final BlockPos fk, final Player ayg, final InteractionHand ajh, final BlockHitResult cvd) {
        return this.baseState.use(bjt, ayg, ajh, cvd);
    }
    
    @Override
    public void wasExploded(final Level bjt, final BlockPos fk, final Explosion bjm) {
        this.base.wasExploded(bjt, fk, bjm);
    }
    
    @Override
    public BlockState getStateForPlacement(final BlockPlaceContext bcn) {
        final Direction fp3 = bcn.getClickedFace();
        final BlockPos fk4 = bcn.getClickedPos();
        final FluidState cog5 = bcn.getLevel().getFluidState(fk4);
        final BlockState byg6 = ((((AbstractStateHolder<O, BlockState>)this.defaultBlockState()).setValue((Property<Comparable>)StairBlock.FACING, bcn.getHorizontalDirection())).setValue(StairBlock.HALF, (fp3 == Direction.DOWN || (fp3 != Direction.UP && bcn.getClickLocation().y - fk4.getY() > 0.5)) ? Half.TOP : Half.BOTTOM)).<Comparable, Boolean>setValue((Property<Comparable>)StairBlock.WATERLOGGED, cog5.getType() == Fluids.WATER);
        return ((AbstractStateHolder<O, BlockState>)byg6).<StairsShape, StairsShape>setValue(StairBlock.SHAPE, getStairsShape(byg6, bcn.getLevel(), fk4));
    }
    
    @Override
    public BlockState updateShape(final BlockState byg1, final Direction fp, final BlockState byg3, final LevelAccessor bju, final BlockPos fk5, final BlockPos fk6) {
        if (byg1.<Boolean>getValue((Property<Boolean>)StairBlock.WATERLOGGED)) {
            bju.getLiquidTicks().scheduleTick(fk5, Fluids.WATER, Fluids.WATER.getTickDelay(bju));
        }
        if (fp.getAxis().isHorizontal()) {
            return ((AbstractStateHolder<O, BlockState>)byg1).<StairsShape, StairsShape>setValue(StairBlock.SHAPE, getStairsShape(byg1, bju, fk5));
        }
        return super.updateShape(byg1, fp, byg3, bju, fk5, fk6);
    }
    
    private static StairsShape getStairsShape(final BlockState byg, final BlockGetter bjd, final BlockPos fk) {
        final Direction fp4 = byg.<Direction>getValue((Property<Direction>)StairBlock.FACING);
        final BlockState byg2 = bjd.getBlockState(fk.relative(fp4));
        if (isStairs(byg2) && byg.<Half>getValue(StairBlock.HALF) == byg2.<Half>getValue(StairBlock.HALF)) {
            final Direction fp5 = byg2.<Direction>getValue((Property<Direction>)StairBlock.FACING);
            if (fp5.getAxis() != byg.<Direction>getValue((Property<Direction>)StairBlock.FACING).getAxis() && canTakeShape(byg, bjd, fk, fp5.getOpposite())) {
                if (fp5 == fp4.getCounterClockWise()) {
                    return StairsShape.OUTER_LEFT;
                }
                return StairsShape.OUTER_RIGHT;
            }
        }
        final BlockState byg3 = bjd.getBlockState(fk.relative(fp4.getOpposite()));
        if (isStairs(byg3) && byg.<Half>getValue(StairBlock.HALF) == byg3.<Half>getValue(StairBlock.HALF)) {
            final Direction fp6 = byg3.<Direction>getValue((Property<Direction>)StairBlock.FACING);
            if (fp6.getAxis() != byg.<Direction>getValue((Property<Direction>)StairBlock.FACING).getAxis() && canTakeShape(byg, bjd, fk, fp6)) {
                if (fp6 == fp4.getCounterClockWise()) {
                    return StairsShape.INNER_LEFT;
                }
                return StairsShape.INNER_RIGHT;
            }
        }
        return StairsShape.STRAIGHT;
    }
    
    private static boolean canTakeShape(final BlockState byg, final BlockGetter bjd, final BlockPos fk, final Direction fp) {
        final BlockState byg2 = bjd.getBlockState(fk.relative(fp));
        return !isStairs(byg2) || byg2.<Comparable>getValue((Property<Comparable>)StairBlock.FACING) != byg.<Comparable>getValue((Property<Comparable>)StairBlock.FACING) || byg2.<Half>getValue(StairBlock.HALF) != byg.<Half>getValue(StairBlock.HALF);
    }
    
    public static boolean isStairs(final BlockState byg) {
        return byg.getBlock() instanceof StairBlock;
    }
    
    @Override
    public BlockState rotate(final BlockState byg, final Rotation btr) {
        return ((AbstractStateHolder<O, BlockState>)byg).<Comparable, Direction>setValue((Property<Comparable>)StairBlock.FACING, btr.rotate(byg.<Direction>getValue((Property<Direction>)StairBlock.FACING)));
    }
    
    @Override
    public BlockState mirror(final BlockState byg, final Mirror bsr) {
        final Direction fp4 = byg.<Direction>getValue((Property<Direction>)StairBlock.FACING);
        final StairsShape bzn5 = byg.<StairsShape>getValue(StairBlock.SHAPE);
        Label_0335: {
            switch (bsr) {
                case LEFT_RIGHT: {
                    if (fp4.getAxis() != Direction.Axis.Z) {
                        break;
                    }
                    switch (bzn5) {
                        case INNER_LEFT: {
                            return ((AbstractStateHolder<O, BlockState>)byg.rotate(Rotation.CLOCKWISE_180)).<StairsShape, StairsShape>setValue(StairBlock.SHAPE, StairsShape.INNER_RIGHT);
                        }
                        case INNER_RIGHT: {
                            return ((AbstractStateHolder<O, BlockState>)byg.rotate(Rotation.CLOCKWISE_180)).<StairsShape, StairsShape>setValue(StairBlock.SHAPE, StairsShape.INNER_LEFT);
                        }
                        case OUTER_LEFT: {
                            return ((AbstractStateHolder<O, BlockState>)byg.rotate(Rotation.CLOCKWISE_180)).<StairsShape, StairsShape>setValue(StairBlock.SHAPE, StairsShape.OUTER_RIGHT);
                        }
                        case OUTER_RIGHT: {
                            return ((AbstractStateHolder<O, BlockState>)byg.rotate(Rotation.CLOCKWISE_180)).<StairsShape, StairsShape>setValue(StairBlock.SHAPE, StairsShape.OUTER_LEFT);
                        }
                        default: {
                            return byg.rotate(Rotation.CLOCKWISE_180);
                        }
                    }
                    break;
                }
                case FRONT_BACK: {
                    if (fp4.getAxis() != Direction.Axis.X) {
                        break;
                    }
                    switch (bzn5) {
                        case INNER_LEFT: {
                            return ((AbstractStateHolder<O, BlockState>)byg.rotate(Rotation.CLOCKWISE_180)).<StairsShape, StairsShape>setValue(StairBlock.SHAPE, StairsShape.INNER_LEFT);
                        }
                        case INNER_RIGHT: {
                            return ((AbstractStateHolder<O, BlockState>)byg.rotate(Rotation.CLOCKWISE_180)).<StairsShape, StairsShape>setValue(StairBlock.SHAPE, StairsShape.INNER_RIGHT);
                        }
                        case OUTER_LEFT: {
                            return ((AbstractStateHolder<O, BlockState>)byg.rotate(Rotation.CLOCKWISE_180)).<StairsShape, StairsShape>setValue(StairBlock.SHAPE, StairsShape.OUTER_RIGHT);
                        }
                        case OUTER_RIGHT: {
                            return ((AbstractStateHolder<O, BlockState>)byg.rotate(Rotation.CLOCKWISE_180)).<StairsShape, StairsShape>setValue(StairBlock.SHAPE, StairsShape.OUTER_LEFT);
                        }
                        case STRAIGHT: {
                            return byg.rotate(Rotation.CLOCKWISE_180);
                        }
                        default: {
                            break Label_0335;
                        }
                    }
                    break;
                }
            }
        }
        return super.mirror(byg, bsr);
    }
    
    @Override
    protected void createBlockStateDefinition(final StateDefinition.Builder<Block, BlockState> a) {
        a.add(StairBlock.FACING, StairBlock.HALF, StairBlock.SHAPE, StairBlock.WATERLOGGED);
    }
    
    @Override
    public FluidState getFluidState(final BlockState byg) {
        if (byg.<Boolean>getValue((Property<Boolean>)StairBlock.WATERLOGGED)) {
            return Fluids.WATER.getSource(false);
        }
        return super.getFluidState(byg);
    }
    
    @Override
    public boolean isPathfindable(final BlockState byg, final BlockGetter bjd, final BlockPos fk, final PathComputationType cqo) {
        return false;
    }
    
    static {
        FACING = HorizontalDirectionalBlock.FACING;
        HALF = BlockStateProperties.HALF;
        SHAPE = BlockStateProperties.STAIRS_SHAPE;
        WATERLOGGED = BlockStateProperties.WATERLOGGED;
        TOP_AABB = SlabBlock.TOP_AABB;
        BOTTOM_AABB = SlabBlock.BOTTOM_AABB;
        OCTET_NNN = Block.box(0.0, 0.0, 0.0, 8.0, 8.0, 8.0);
        OCTET_NNP = Block.box(0.0, 0.0, 8.0, 8.0, 8.0, 16.0);
        OCTET_NPN = Block.box(0.0, 8.0, 0.0, 8.0, 16.0, 8.0);
        OCTET_NPP = Block.box(0.0, 8.0, 8.0, 8.0, 16.0, 16.0);
        OCTET_PNN = Block.box(8.0, 0.0, 0.0, 16.0, 8.0, 8.0);
        OCTET_PNP = Block.box(8.0, 0.0, 8.0, 16.0, 8.0, 16.0);
        OCTET_PPN = Block.box(8.0, 8.0, 0.0, 16.0, 16.0, 8.0);
        OCTET_PPP = Block.box(8.0, 8.0, 8.0, 16.0, 16.0, 16.0);
        TOP_SHAPES = makeShapes(StairBlock.TOP_AABB, StairBlock.OCTET_NNN, StairBlock.OCTET_PNN, StairBlock.OCTET_NNP, StairBlock.OCTET_PNP);
        BOTTOM_SHAPES = makeShapes(StairBlock.BOTTOM_AABB, StairBlock.OCTET_NPN, StairBlock.OCTET_PPN, StairBlock.OCTET_NPP, StairBlock.OCTET_PPP);
        SHAPE_BY_STATE = new int[] { 12, 5, 3, 10, 14, 13, 7, 11, 13, 7, 11, 14, 8, 4, 1, 2, 4, 1, 2, 8 };
    }
}
