package net.minecraft.world.level.block;

import net.minecraft.world.level.block.state.AbstractStateHolder;
import net.minecraft.world.level.block.state.properties.BlockStateProperties;
import net.minecraft.world.level.block.state.StateDefinition;
import java.util.Iterator;
import java.util.List;
import net.minecraft.world.level.LevelReader;
import net.minecraft.core.Vec3i;
import java.util.Random;
import net.minecraft.server.level.ServerLevel;
import net.minecraft.world.entity.Entity;
import net.minecraft.world.item.Items;
import net.minecraft.world.entity.player.Player;
import net.minecraft.world.level.Level;
import net.minecraft.world.level.LevelAccessor;
import net.minecraft.world.item.BlockPlaceContext;
import net.minecraft.world.phys.shapes.CollisionContext;
import net.minecraft.core.BlockPos;
import net.minecraft.world.level.BlockGetter;
import net.minecraft.world.level.block.state.properties.Property;
import net.minecraft.world.level.block.state.BlockState;
import net.minecraft.world.phys.shapes.VoxelShape;
import net.minecraft.core.Direction;
import java.util.Map;
import net.minecraft.world.level.block.state.properties.BooleanProperty;

public class TripWireBlock extends Block {
    public static final BooleanProperty POWERED;
    public static final BooleanProperty ATTACHED;
    public static final BooleanProperty DISARMED;
    public static final BooleanProperty NORTH;
    public static final BooleanProperty EAST;
    public static final BooleanProperty SOUTH;
    public static final BooleanProperty WEST;
    private static final Map<Direction, BooleanProperty> PROPERTY_BY_DIRECTION;
    protected static final VoxelShape AABB;
    protected static final VoxelShape NOT_ATTACHED_AABB;
    private final TripWireHookBlock hook;
    
    public TripWireBlock(final TripWireHookBlock bvh, final Properties c) {
        super(c);
        this.registerDefaultState(((((((((AbstractStateHolder<O, BlockState>)this.stateDefinition.any()).setValue((Property<Comparable>)TripWireBlock.POWERED, false)).setValue((Property<Comparable>)TripWireBlock.ATTACHED, false)).setValue((Property<Comparable>)TripWireBlock.DISARMED, false)).setValue((Property<Comparable>)TripWireBlock.NORTH, false)).setValue((Property<Comparable>)TripWireBlock.EAST, false)).setValue((Property<Comparable>)TripWireBlock.SOUTH, false)).<Comparable, Boolean>setValue((Property<Comparable>)TripWireBlock.WEST, false));
        this.hook = bvh;
    }
    
    @Override
    public VoxelShape getShape(final BlockState byg, final BlockGetter bjd, final BlockPos fk, final CollisionContext cvn) {
        return byg.<Boolean>getValue((Property<Boolean>)TripWireBlock.ATTACHED) ? TripWireBlock.AABB : TripWireBlock.NOT_ATTACHED_AABB;
    }
    
    @Override
    public BlockState getStateForPlacement(final BlockPlaceContext bcn) {
        final BlockGetter bjd3 = bcn.getLevel();
        final BlockPos fk4 = bcn.getClickedPos();
        return (((((AbstractStateHolder<O, BlockState>)this.defaultBlockState()).setValue((Property<Comparable>)TripWireBlock.NORTH, this.shouldConnectTo(bjd3.getBlockState(fk4.north()), Direction.NORTH))).setValue((Property<Comparable>)TripWireBlock.EAST, this.shouldConnectTo(bjd3.getBlockState(fk4.east()), Direction.EAST))).setValue((Property<Comparable>)TripWireBlock.SOUTH, this.shouldConnectTo(bjd3.getBlockState(fk4.south()), Direction.SOUTH))).<Comparable, Boolean>setValue((Property<Comparable>)TripWireBlock.WEST, this.shouldConnectTo(bjd3.getBlockState(fk4.west()), Direction.WEST));
    }
    
    @Override
    public BlockState updateShape(final BlockState byg1, final Direction fp, final BlockState byg3, final LevelAccessor bju, final BlockPos fk5, final BlockPos fk6) {
        if (fp.getAxis().isHorizontal()) {
            return ((AbstractStateHolder<O, BlockState>)byg1).<Comparable, Boolean>setValue((Property<Comparable>)TripWireBlock.PROPERTY_BY_DIRECTION.get(fp), this.shouldConnectTo(byg3, fp));
        }
        return super.updateShape(byg1, fp, byg3, bju, fk5, fk6);
    }
    
    @Override
    public void onPlace(final BlockState byg1, final Level bjt, final BlockPos fk, final BlockState byg4, final boolean boolean5) {
        if (byg4.getBlock() == byg1.getBlock()) {
            return;
        }
        this.updateSource(bjt, fk, byg1);
    }
    
    @Override
    public void onRemove(final BlockState byg1, final Level bjt, final BlockPos fk, final BlockState byg4, final boolean boolean5) {
        if (boolean5 || byg1.getBlock() == byg4.getBlock()) {
            return;
        }
        this.updateSource(bjt, fk, ((AbstractStateHolder<O, BlockState>)byg1).<Comparable, Boolean>setValue((Property<Comparable>)TripWireBlock.POWERED, true));
    }
    
    @Override
    public void playerWillDestroy(final Level bjt, final BlockPos fk, final BlockState byg, final Player ayg) {
        if (!bjt.isClientSide && !ayg.getMainHandItem().isEmpty() && ayg.getMainHandItem().getItem() == Items.SHEARS) {
            bjt.setBlock(fk, ((AbstractStateHolder<O, BlockState>)byg).<Comparable, Boolean>setValue((Property<Comparable>)TripWireBlock.DISARMED, true), 4);
        }
        super.playerWillDestroy(bjt, fk, byg, ayg);
    }
    
    private void updateSource(final Level bjt, final BlockPos fk, final BlockState byg) {
        for (final Direction fp8 : new Direction[] { Direction.SOUTH, Direction.WEST }) {
            int integer9 = 1;
            while (integer9 < 42) {
                final BlockPos fk2 = fk.relative(fp8, integer9);
                final BlockState byg2 = bjt.getBlockState(fk2);
                if (byg2.getBlock() == this.hook) {
                    if (byg2.<Comparable>getValue((Property<Comparable>)TripWireHookBlock.FACING) == fp8.getOpposite()) {
                        this.hook.calculateState(bjt, fk2, byg2, false, true, integer9, byg);
                        break;
                    }
                    break;
                }
                else {
                    if (byg2.getBlock() != this) {
                        break;
                    }
                    ++integer9;
                }
            }
        }
    }
    
    @Override
    public void entityInside(final BlockState byg, final Level bjt, final BlockPos fk, final Entity akn) {
        if (bjt.isClientSide) {
            return;
        }
        if (byg.<Boolean>getValue((Property<Boolean>)TripWireBlock.POWERED)) {
            return;
        }
        this.checkPressed(bjt, fk);
    }
    
    @Override
    public void tick(final BlockState byg, final ServerLevel xd, final BlockPos fk, final Random random) {
        if (!xd.getBlockState(fk).<Boolean>getValue((Property<Boolean>)TripWireBlock.POWERED)) {
            return;
        }
        this.checkPressed(xd, fk);
    }
    
    private void checkPressed(final Level bjt, final BlockPos fk) {
        BlockState byg4 = bjt.getBlockState(fk);
        final boolean boolean5 = byg4.<Boolean>getValue((Property<Boolean>)TripWireBlock.POWERED);
        boolean boolean6 = false;
        final List<? extends Entity> list7 = bjt.getEntities(null, byg4.getShape(bjt, fk).bounds().move(fk));
        if (!list7.isEmpty()) {
            for (final Entity akn9 : list7) {
                if (!akn9.isIgnoringBlockTriggers()) {
                    boolean6 = true;
                    break;
                }
            }
        }
        if (boolean6 != boolean5) {
            byg4 = ((AbstractStateHolder<O, BlockState>)byg4).<Comparable, Boolean>setValue((Property<Comparable>)TripWireBlock.POWERED, boolean6);
            bjt.setBlock(fk, byg4, 3);
            this.updateSource(bjt, fk, byg4);
        }
        if (boolean6) {
            bjt.getBlockTicks().scheduleTick(new BlockPos(fk), this, this.getTickDelay(bjt));
        }
    }
    
    public boolean shouldConnectTo(final BlockState byg, final Direction fp) {
        final Block bpe4 = byg.getBlock();
        if (bpe4 == this.hook) {
            return byg.<Comparable>getValue((Property<Comparable>)TripWireHookBlock.FACING) == fp.getOpposite();
        }
        return bpe4 == this;
    }
    
    @Override
    public BlockState rotate(final BlockState byg, final Rotation btr) {
        switch (btr) {
            case CLOCKWISE_180: {
                return (((((AbstractStateHolder<O, BlockState>)byg).setValue((Property<Comparable>)TripWireBlock.NORTH, (Comparable)byg.<V>getValue((Property<V>)TripWireBlock.SOUTH))).setValue((Property<Comparable>)TripWireBlock.EAST, (Comparable)byg.<V>getValue((Property<V>)TripWireBlock.WEST))).setValue((Property<Comparable>)TripWireBlock.SOUTH, (Comparable)byg.<V>getValue((Property<V>)TripWireBlock.NORTH))).<Comparable, Comparable>setValue((Property<Comparable>)TripWireBlock.WEST, (Comparable)byg.<V>getValue((Property<V>)TripWireBlock.EAST));
            }
            case COUNTERCLOCKWISE_90: {
                return (((((AbstractStateHolder<O, BlockState>)byg).setValue((Property<Comparable>)TripWireBlock.NORTH, (Comparable)byg.<V>getValue((Property<V>)TripWireBlock.EAST))).setValue((Property<Comparable>)TripWireBlock.EAST, (Comparable)byg.<V>getValue((Property<V>)TripWireBlock.SOUTH))).setValue((Property<Comparable>)TripWireBlock.SOUTH, (Comparable)byg.<V>getValue((Property<V>)TripWireBlock.WEST))).<Comparable, Comparable>setValue((Property<Comparable>)TripWireBlock.WEST, (Comparable)byg.<V>getValue((Property<V>)TripWireBlock.NORTH));
            }
            case CLOCKWISE_90: {
                return (((((AbstractStateHolder<O, BlockState>)byg).setValue((Property<Comparable>)TripWireBlock.NORTH, (Comparable)byg.<V>getValue((Property<V>)TripWireBlock.WEST))).setValue((Property<Comparable>)TripWireBlock.EAST, (Comparable)byg.<V>getValue((Property<V>)TripWireBlock.NORTH))).setValue((Property<Comparable>)TripWireBlock.SOUTH, (Comparable)byg.<V>getValue((Property<V>)TripWireBlock.EAST))).<Comparable, Comparable>setValue((Property<Comparable>)TripWireBlock.WEST, (Comparable)byg.<V>getValue((Property<V>)TripWireBlock.SOUTH));
            }
            default: {
                return byg;
            }
        }
    }
    
    @Override
    public BlockState mirror(final BlockState byg, final Mirror bsr) {
        switch (bsr) {
            case LEFT_RIGHT: {
                return (((AbstractStateHolder<O, BlockState>)byg).setValue((Property<Comparable>)TripWireBlock.NORTH, (Comparable)byg.<V>getValue((Property<V>)TripWireBlock.SOUTH))).<Comparable, Comparable>setValue((Property<Comparable>)TripWireBlock.SOUTH, (Comparable)byg.<V>getValue((Property<V>)TripWireBlock.NORTH));
            }
            case FRONT_BACK: {
                return (((AbstractStateHolder<O, BlockState>)byg).setValue((Property<Comparable>)TripWireBlock.EAST, (Comparable)byg.<V>getValue((Property<V>)TripWireBlock.WEST))).<Comparable, Comparable>setValue((Property<Comparable>)TripWireBlock.WEST, (Comparable)byg.<V>getValue((Property<V>)TripWireBlock.EAST));
            }
            default: {
                return super.mirror(byg, bsr);
            }
        }
    }
    
    @Override
    protected void createBlockStateDefinition(final StateDefinition.Builder<Block, BlockState> a) {
        a.add(TripWireBlock.POWERED, TripWireBlock.ATTACHED, TripWireBlock.DISARMED, TripWireBlock.NORTH, TripWireBlock.EAST, TripWireBlock.WEST, TripWireBlock.SOUTH);
    }
    
    static {
        POWERED = BlockStateProperties.POWERED;
        ATTACHED = BlockStateProperties.ATTACHED;
        DISARMED = BlockStateProperties.DISARMED;
        NORTH = PipeBlock.NORTH;
        EAST = PipeBlock.EAST;
        SOUTH = PipeBlock.SOUTH;
        WEST = PipeBlock.WEST;
        PROPERTY_BY_DIRECTION = CrossCollisionBlock.PROPERTY_BY_DIRECTION;
        AABB = Block.box(0.0, 1.0, 0.0, 16.0, 2.5, 16.0);
        NOT_ATTACHED_AABB = Block.box(0.0, 0.0, 0.0, 16.0, 8.0, 16.0);
    }
}
