package net.minecraft.world.level.block.piston;

import net.minecraft.world.level.block.state.AbstractStateHolder;
import net.minecraft.world.level.pathfinder.PathComputationType;
import net.minecraft.world.level.block.state.StateDefinition;
import net.minecraft.world.level.block.Mirror;
import net.minecraft.world.level.block.Rotation;
import net.minecraft.world.phys.shapes.Shapes;
import net.minecraft.world.phys.shapes.VoxelShape;
import net.minecraft.world.phys.shapes.CollisionContext;
import java.util.Collections;
import net.minecraft.world.level.storage.loot.parameters.LootContextParams;
import net.minecraft.world.item.ItemStack;
import java.util.List;
import net.minecraft.world.level.storage.loot.LootContext;
import net.minecraft.world.InteractionResult;
import net.minecraft.world.phys.BlockHitResult;
import net.minecraft.world.InteractionHand;
import net.minecraft.world.entity.player.Player;
import net.minecraft.world.level.LevelAccessor;
import net.minecraft.core.BlockPos;
import net.minecraft.world.level.Level;
import javax.annotation.Nullable;
import net.minecraft.world.level.block.entity.BlockEntity;
import net.minecraft.world.level.BlockGetter;
import net.minecraft.world.level.block.state.properties.Property;
import net.minecraft.core.Direction;
import net.minecraft.world.level.block.state.BlockState;
import net.minecraft.world.level.block.Block;
import net.minecraft.world.level.block.state.properties.PistonType;
import net.minecraft.world.level.block.state.properties.EnumProperty;
import net.minecraft.world.level.block.state.properties.DirectionProperty;
import net.minecraft.world.level.block.BaseEntityBlock;

public class MovingPistonBlock extends BaseEntityBlock {
    public static final DirectionProperty FACING;
    public static final EnumProperty<PistonType> TYPE;
    
    public MovingPistonBlock(final Properties c) {
        super(c);
        this.registerDefaultState((((AbstractStateHolder<O, BlockState>)this.stateDefinition.any()).setValue((Property<Comparable>)MovingPistonBlock.FACING, Direction.NORTH)).<PistonType, PistonType>setValue(MovingPistonBlock.TYPE, PistonType.DEFAULT));
    }
    
    @Nullable
    @Override
    public BlockEntity newBlockEntity(final BlockGetter bjd) {
        return null;
    }
    
    public static BlockEntity newMovingBlockEntity(final BlockState byg, final Direction fp, final boolean boolean3, final boolean boolean4) {
        return new PistonMovingBlockEntity(byg, fp, boolean3, boolean4);
    }
    
    @Override
    public void onRemove(final BlockState byg1, final Level bjt, final BlockPos fk, final BlockState byg4, final boolean boolean5) {
        if (byg1.getBlock() == byg4.getBlock()) {
            return;
        }
        final BlockEntity bwi7 = bjt.getBlockEntity(fk);
        if (bwi7 instanceof PistonMovingBlockEntity) {
            ((PistonMovingBlockEntity)bwi7).finalTick();
        }
    }
    
    @Override
    public void destroy(final LevelAccessor bju, final BlockPos fk, final BlockState byg) {
        final BlockPos fk2 = fk.relative(byg.<Direction>getValue((Property<Direction>)MovingPistonBlock.FACING).getOpposite());
        final BlockState byg2 = bju.getBlockState(fk2);
        if (byg2.getBlock() instanceof PistonBaseBlock && byg2.<Boolean>getValue((Property<Boolean>)PistonBaseBlock.EXTENDED)) {
            bju.removeBlock(fk2, false);
        }
    }
    
    @Override
    public boolean isRedstoneConductor(final BlockState byg, final BlockGetter bjd, final BlockPos fk) {
        return false;
    }
    
    @Override
    public boolean isSuffocating(final BlockState byg, final BlockGetter bjd, final BlockPos fk) {
        return false;
    }
    
    @Override
    public InteractionResult use(final BlockState byg, final Level bjt, final BlockPos fk, final Player ayg, final InteractionHand ajh, final BlockHitResult cvd) {
        if (!bjt.isClientSide && bjt.getBlockEntity(fk) == null) {
            bjt.removeBlock(fk, false);
            return InteractionResult.CONSUME;
        }
        return InteractionResult.PASS;
    }
    
    @Override
    public List<ItemStack> getDrops(final BlockState byg, final LootContext.Builder a) {
        final PistonMovingBlockEntity byc4 = this.getBlockEntity(a.getLevel(), a.<BlockPos>getParameter(LootContextParams.BLOCK_POS));
        if (byc4 == null) {
            return Collections.<ItemStack>emptyList();
        }
        return byc4.getMovedState().getDrops(a);
    }
    
    @Override
    public VoxelShape getShape(final BlockState byg, final BlockGetter bjd, final BlockPos fk, final CollisionContext cvn) {
        return Shapes.empty();
    }
    
    @Override
    public VoxelShape getCollisionShape(final BlockState byg, final BlockGetter bjd, final BlockPos fk, final CollisionContext cvn) {
        final PistonMovingBlockEntity byc6 = this.getBlockEntity(bjd, fk);
        if (byc6 != null) {
            return byc6.getCollisionShape(bjd, fk);
        }
        return Shapes.empty();
    }
    
    @Nullable
    private PistonMovingBlockEntity getBlockEntity(final BlockGetter bjd, final BlockPos fk) {
        final BlockEntity bwi4 = bjd.getBlockEntity(fk);
        if (bwi4 instanceof PistonMovingBlockEntity) {
            return (PistonMovingBlockEntity)bwi4;
        }
        return null;
    }
    
    @Override
    public ItemStack getCloneItemStack(final BlockGetter bjd, final BlockPos fk, final BlockState byg) {
        return ItemStack.EMPTY;
    }
    
    @Override
    public BlockState rotate(final BlockState byg, final Rotation btr) {
        return ((AbstractStateHolder<O, BlockState>)byg).<Comparable, Direction>setValue((Property<Comparable>)MovingPistonBlock.FACING, btr.rotate(byg.<Direction>getValue((Property<Direction>)MovingPistonBlock.FACING)));
    }
    
    @Override
    public BlockState mirror(final BlockState byg, final Mirror bsr) {
        return byg.rotate(bsr.getRotation(byg.<Direction>getValue((Property<Direction>)MovingPistonBlock.FACING)));
    }
    
    @Override
    protected void createBlockStateDefinition(final StateDefinition.Builder<Block, BlockState> a) {
        a.add(MovingPistonBlock.FACING, MovingPistonBlock.TYPE);
    }
    
    @Override
    public boolean isPathfindable(final BlockState byg, final BlockGetter bjd, final BlockPos fk, final PathComputationType cqo) {
        return false;
    }
    
    static {
        FACING = PistonHeadBlock.FACING;
        TYPE = PistonHeadBlock.TYPE;
    }
}
