package net.minecraft.world.level.block.piston;

import net.minecraft.world.level.block.state.AbstractStateHolder;
import net.minecraft.world.level.block.state.properties.BlockStateProperties;
import net.minecraft.world.level.pathfinder.PathComputationType;
import net.minecraft.world.level.block.state.StateDefinition;
import net.minecraft.world.level.block.Mirror;
import net.minecraft.world.level.block.Rotation;
import java.util.Iterator;
import java.util.List;
import net.minecraft.world.level.LevelAccessor;
import java.util.Map;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import net.minecraft.world.level.material.PushReaction;
import net.minecraft.world.level.block.state.properties.PistonType;
import net.minecraft.world.entity.player.Player;
import net.minecraft.sounds.SoundSource;
import net.minecraft.sounds.SoundEvents;
import net.minecraft.world.level.block.entity.BlockEntity;
import net.minecraft.server.level.ServerLevel;
import net.minecraft.world.level.block.Blocks;
import net.minecraft.world.item.BlockPlaceContext;
import net.minecraft.world.item.ItemStack;
import net.minecraft.world.entity.LivingEntity;
import net.minecraft.world.level.Level;
import net.minecraft.world.phys.shapes.Shapes;
import net.minecraft.world.phys.shapes.CollisionContext;
import net.minecraft.core.BlockPos;
import net.minecraft.world.level.BlockGetter;
import net.minecraft.world.level.block.state.properties.Property;
import net.minecraft.core.Direction;
import net.minecraft.world.level.block.state.BlockState;
import net.minecraft.world.level.block.Block;
import net.minecraft.world.phys.shapes.VoxelShape;
import net.minecraft.world.level.block.state.properties.BooleanProperty;
import net.minecraft.world.level.block.DirectionalBlock;

public class PistonBaseBlock extends DirectionalBlock {
    public static final BooleanProperty EXTENDED;
    protected static final VoxelShape EAST_AABB;
    protected static final VoxelShape WEST_AABB;
    protected static final VoxelShape SOUTH_AABB;
    protected static final VoxelShape NORTH_AABB;
    protected static final VoxelShape UP_AABB;
    protected static final VoxelShape DOWN_AABB;
    private final boolean isSticky;
    
    public PistonBaseBlock(final boolean boolean1, final Properties c) {
        super(c);
        this.registerDefaultState((((AbstractStateHolder<O, BlockState>)this.stateDefinition.any()).setValue((Property<Comparable>)PistonBaseBlock.FACING, Direction.NORTH)).<Comparable, Boolean>setValue((Property<Comparable>)PistonBaseBlock.EXTENDED, false));
        this.isSticky = boolean1;
    }
    
    @Override
    public boolean isSuffocating(final BlockState byg, final BlockGetter bjd, final BlockPos fk) {
        return !byg.<Boolean>getValue((Property<Boolean>)PistonBaseBlock.EXTENDED);
    }
    
    @Override
    public VoxelShape getShape(final BlockState byg, final BlockGetter bjd, final BlockPos fk, final CollisionContext cvn) {
        if (!byg.<Boolean>getValue((Property<Boolean>)PistonBaseBlock.EXTENDED)) {
            return Shapes.block();
        }
        switch (byg.<Direction>getValue((Property<Direction>)PistonBaseBlock.FACING)) {
            case DOWN: {
                return PistonBaseBlock.DOWN_AABB;
            }
            default: {
                return PistonBaseBlock.UP_AABB;
            }
            case NORTH: {
                return PistonBaseBlock.NORTH_AABB;
            }
            case SOUTH: {
                return PistonBaseBlock.SOUTH_AABB;
            }
            case WEST: {
                return PistonBaseBlock.WEST_AABB;
            }
            case EAST: {
                return PistonBaseBlock.EAST_AABB;
            }
        }
    }
    
    @Override
    public boolean isRedstoneConductor(final BlockState byg, final BlockGetter bjd, final BlockPos fk) {
        return false;
    }
    
    @Override
    public void setPlacedBy(final Level bjt, final BlockPos fk, final BlockState byg, final LivingEntity akw, final ItemStack bek) {
        if (!bjt.isClientSide) {
            this.checkIfExtend(bjt, fk, byg);
        }
    }
    
    @Override
    public void neighborChanged(final BlockState byg, final Level bjt, final BlockPos fk3, final Block bpe, final BlockPos fk5, final boolean boolean6) {
        if (!bjt.isClientSide) {
            this.checkIfExtend(bjt, fk3, byg);
        }
    }
    
    @Override
    public void onPlace(final BlockState byg1, final Level bjt, final BlockPos fk, final BlockState byg4, final boolean boolean5) {
        if (byg4.getBlock() == byg1.getBlock()) {
            return;
        }
        if (!bjt.isClientSide && bjt.getBlockEntity(fk) == null) {
            this.checkIfExtend(bjt, fk, byg1);
        }
    }
    
    @Override
    public BlockState getStateForPlacement(final BlockPlaceContext bcn) {
        return (((AbstractStateHolder<O, BlockState>)this.defaultBlockState()).setValue((Property<Comparable>)PistonBaseBlock.FACING, bcn.getNearestLookingDirection().getOpposite())).<Comparable, Boolean>setValue((Property<Comparable>)PistonBaseBlock.EXTENDED, false);
    }
    
    private void checkIfExtend(final Level bjt, final BlockPos fk, final BlockState byg) {
        final Direction fp5 = byg.<Direction>getValue((Property<Direction>)PistonBaseBlock.FACING);
        final boolean boolean6 = this.getNeighborSignal(bjt, fk, fp5);
        if (boolean6 && !byg.<Boolean>getValue((Property<Boolean>)PistonBaseBlock.EXTENDED)) {
            if (new PistonStructureResolver(bjt, fk, fp5, true).resolve()) {
                bjt.blockEvent(fk, this, 0, fp5.get3DDataValue());
            }
        }
        else if (!boolean6 && byg.<Boolean>getValue((Property<Boolean>)PistonBaseBlock.EXTENDED)) {
            final BlockPos fk2 = fk.relative(fp5, 2);
            final BlockState byg2 = bjt.getBlockState(fk2);
            int integer9 = 1;
            if (byg2.getBlock() == Blocks.MOVING_PISTON && byg2.<Comparable>getValue((Property<Comparable>)PistonBaseBlock.FACING) == fp5) {
                final BlockEntity bwi10 = bjt.getBlockEntity(fk2);
                if (bwi10 instanceof PistonMovingBlockEntity) {
                    final PistonMovingBlockEntity byc11 = (PistonMovingBlockEntity)bwi10;
                    if (byc11.isExtending() && (byc11.getProgress(0.0f) < 0.5f || bjt.getGameTime() == byc11.getLastTicked() || ((ServerLevel)bjt).isHandlingTick())) {
                        integer9 = 2;
                    }
                }
            }
            bjt.blockEvent(fk, this, integer9, fp5.get3DDataValue());
        }
    }
    
    private boolean getNeighborSignal(final Level bjt, final BlockPos fk, final Direction fp) {
        for (final Direction fp2 : Direction.values()) {
            if (fp2 != fp && bjt.hasSignal(fk.relative(fp2), fp2)) {
                return true;
            }
        }
        if (bjt.hasSignal(fk, Direction.DOWN)) {
            return true;
        }
        final BlockPos fk2 = fk.above();
        for (final Direction fp3 : Direction.values()) {
            if (fp3 != Direction.DOWN && bjt.hasSignal(fk2.relative(fp3), fp3)) {
                return true;
            }
        }
        return false;
    }
    
    @Override
    public boolean triggerEvent(final BlockState byg, final Level bjt, final BlockPos fk, final int integer4, final int integer5) {
        final Direction fp7 = byg.<Direction>getValue((Property<Direction>)PistonBaseBlock.FACING);
        if (!bjt.isClientSide) {
            final boolean boolean8 = this.getNeighborSignal(bjt, fk, fp7);
            if (boolean8 && (integer4 == 1 || integer4 == 2)) {
                bjt.setBlock(fk, ((AbstractStateHolder<O, BlockState>)byg).<Comparable, Boolean>setValue((Property<Comparable>)PistonBaseBlock.EXTENDED, true), 2);
                return false;
            }
            if (!boolean8 && integer4 == 0) {
                return false;
            }
        }
        if (integer4 == 0) {
            if (!this.moveBlocks(bjt, fk, fp7, true)) {
                return false;
            }
            bjt.setBlock(fk, ((AbstractStateHolder<O, BlockState>)byg).<Comparable, Boolean>setValue((Property<Comparable>)PistonBaseBlock.EXTENDED, true), 67);
            bjt.playSound(null, fk, SoundEvents.PISTON_EXTEND, SoundSource.BLOCKS, 0.5f, bjt.random.nextFloat() * 0.25f + 0.6f);
        }
        else if (integer4 == 1 || integer4 == 2) {
            final BlockEntity bwi8 = bjt.getBlockEntity(fk.relative(fp7));
            if (bwi8 instanceof PistonMovingBlockEntity) {
                ((PistonMovingBlockEntity)bwi8).finalTick();
            }
            bjt.setBlock(fk, (((AbstractStateHolder<O, BlockState>)Blocks.MOVING_PISTON.defaultBlockState()).setValue((Property<Comparable>)MovingPistonBlock.FACING, fp7)).<PistonType, PistonType>setValue(MovingPistonBlock.TYPE, this.isSticky ? PistonType.STICKY : PistonType.DEFAULT), 3);
            bjt.setBlockEntity(fk, MovingPistonBlock.newMovingBlockEntity(((AbstractStateHolder<O, BlockState>)this.defaultBlockState()).<Comparable, Direction>setValue((Property<Comparable>)PistonBaseBlock.FACING, Direction.from3DDataValue(integer5 & 0x7)), fp7, false, true));
            if (this.isSticky) {
                final BlockPos fk2 = fk.offset(fp7.getStepX() * 2, fp7.getStepY() * 2, fp7.getStepZ() * 2);
                final BlockState byg2 = bjt.getBlockState(fk2);
                final Block bpe11 = byg2.getBlock();
                boolean boolean9 = false;
                if (bpe11 == Blocks.MOVING_PISTON) {
                    final BlockEntity bwi9 = bjt.getBlockEntity(fk2);
                    if (bwi9 instanceof PistonMovingBlockEntity) {
                        final PistonMovingBlockEntity byc14 = (PistonMovingBlockEntity)bwi9;
                        if (byc14.getDirection() == fp7 && byc14.isExtending()) {
                            byc14.finalTick();
                            boolean9 = true;
                        }
                    }
                }
                if (!boolean9) {
                    if (integer4 == 1 && !byg2.isAir() && isPushable(byg2, bjt, fk2, fp7.getOpposite(), false, fp7) && (byg2.getPistonPushReaction() == PushReaction.NORMAL || bpe11 == Blocks.PISTON || bpe11 == Blocks.STICKY_PISTON)) {
                        this.moveBlocks(bjt, fk, fp7, false);
                    }
                    else {
                        bjt.removeBlock(fk.relative(fp7), false);
                    }
                }
            }
            else {
                bjt.removeBlock(fk.relative(fp7), false);
            }
            bjt.playSound(null, fk, SoundEvents.PISTON_CONTRACT, SoundSource.BLOCKS, 0.5f, bjt.random.nextFloat() * 0.15f + 0.6f);
        }
        return true;
    }
    
    public static boolean isPushable(final BlockState byg, final Level bjt, final BlockPos fk, final Direction fp4, final boolean boolean5, final Direction fp6) {
        final Block bpe7 = byg.getBlock();
        if (bpe7 == Blocks.OBSIDIAN) {
            return false;
        }
        if (!bjt.getWorldBorder().isWithinBounds(fk)) {
            return false;
        }
        if (fk.getY() < 0 || (fp4 == Direction.DOWN && fk.getY() == 0)) {
            return false;
        }
        if (fk.getY() > bjt.getMaxBuildHeight() - 1 || (fp4 == Direction.UP && fk.getY() == bjt.getMaxBuildHeight() - 1)) {
            return false;
        }
        if (bpe7 == Blocks.PISTON || bpe7 == Blocks.STICKY_PISTON) {
            if (byg.<Boolean>getValue((Property<Boolean>)PistonBaseBlock.EXTENDED)) {
                return false;
            }
        }
        else {
            if (byg.getDestroySpeed(bjt, fk) == -1.0f) {
                return false;
            }
            switch (byg.getPistonPushReaction()) {
                case BLOCK: {
                    return false;
                }
                case DESTROY: {
                    return boolean5;
                }
                case PUSH_ONLY: {
                    return fp4 == fp6;
                }
            }
        }
        return !bpe7.isEntityBlock();
    }
    
    private boolean moveBlocks(final Level bjt, final BlockPos fk, final Direction fp, final boolean boolean4) {
        final BlockPos fk2 = fk.relative(fp);
        if (!boolean4 && bjt.getBlockState(fk2).getBlock() == Blocks.PISTON_HEAD) {
            bjt.setBlock(fk2, Blocks.AIR.defaultBlockState(), 20);
        }
        final PistonStructureResolver byd7 = new PistonStructureResolver(bjt, fk, fp, boolean4);
        if (!byd7.resolve()) {
            return false;
        }
        final Map<BlockPos, BlockState> map8 = Maps.newHashMap();
        final List<BlockPos> list9 = byd7.getToPush();
        final List<BlockState> list10 = Lists.newArrayList();
        for (int integer11 = 0; integer11 < list9.size(); ++integer11) {
            final BlockPos fk3 = list9.get(integer11);
            final BlockState byg13 = bjt.getBlockState(fk3);
            list10.add(byg13);
            map8.put(fk3, byg13);
        }
        final List<BlockPos> list11 = byd7.getToDestroy();
        int integer12 = list9.size() + list11.size();
        final BlockState[] arr13 = new BlockState[integer12];
        final Direction fp2 = boolean4 ? fp : fp.getOpposite();
        for (int integer13 = list11.size() - 1; integer13 >= 0; --integer13) {
            final BlockPos fk4 = list11.get(integer13);
            final BlockState byg14 = bjt.getBlockState(fk4);
            final BlockEntity bwi18 = byg14.getBlock().isEntityBlock() ? bjt.getBlockEntity(fk4) : null;
            Block.dropResources(byg14, bjt, fk4, bwi18);
            bjt.setBlock(fk4, Blocks.AIR.defaultBlockState(), 18);
            arr13[--integer12] = byg14;
        }
        for (int integer13 = list9.size() - 1; integer13 >= 0; --integer13) {
            BlockPos fk4 = list9.get(integer13);
            final BlockState byg14 = bjt.getBlockState(fk4);
            fk4 = fk4.relative(fp2);
            map8.remove(fk4);
            bjt.setBlock(fk4, ((AbstractStateHolder<O, BlockState>)Blocks.MOVING_PISTON.defaultBlockState()).<Comparable, Direction>setValue((Property<Comparable>)PistonBaseBlock.FACING, fp), 68);
            bjt.setBlockEntity(fk4, MovingPistonBlock.newMovingBlockEntity(list10.get(integer13), fp, boolean4, false));
            arr13[--integer12] = byg14;
        }
        if (boolean4) {
            final PistonType bzi15 = this.isSticky ? PistonType.STICKY : PistonType.DEFAULT;
            final BlockState byg15 = (((AbstractStateHolder<O, BlockState>)Blocks.PISTON_HEAD.defaultBlockState()).setValue((Property<Comparable>)PistonHeadBlock.FACING, fp)).<PistonType, PistonType>setValue(PistonHeadBlock.TYPE, bzi15);
            final BlockState byg14 = (((AbstractStateHolder<O, BlockState>)Blocks.MOVING_PISTON.defaultBlockState()).setValue((Property<Comparable>)MovingPistonBlock.FACING, fp)).<PistonType, PistonType>setValue(MovingPistonBlock.TYPE, this.isSticky ? PistonType.STICKY : PistonType.DEFAULT);
            map8.remove(fk2);
            bjt.setBlock(fk2, byg14, 68);
            bjt.setBlockEntity(fk2, MovingPistonBlock.newMovingBlockEntity(byg15, fp, true, true));
        }
        final BlockState byg16 = Blocks.AIR.defaultBlockState();
        for (final BlockPos fk5 : map8.keySet()) {
            bjt.setBlock(fk5, byg16, 82);
        }
        for (final Map.Entry<BlockPos, BlockState> entry17 : map8.entrySet()) {
            final BlockPos fk6 = entry17.getKey();
            final BlockState byg17 = entry17.getValue();
            byg17.updateIndirectNeighbourShapes(bjt, fk6, 2);
            byg16.updateNeighbourShapes(bjt, fk6, 2);
            byg16.updateIndirectNeighbourShapes(bjt, fk6, 2);
        }
        for (int integer14 = list11.size() - 1; integer14 >= 0; --integer14) {
            final BlockState byg14 = arr13[integer12++];
            final BlockPos fk6 = list11.get(integer14);
            byg14.updateIndirectNeighbourShapes(bjt, fk6, 2);
            bjt.updateNeighborsAt(fk6, byg14.getBlock());
        }
        for (int integer14 = list9.size() - 1; integer14 >= 0; --integer14) {
            bjt.updateNeighborsAt(list9.get(integer14), arr13[integer12++].getBlock());
        }
        if (boolean4) {
            bjt.updateNeighborsAt(fk2, Blocks.PISTON_HEAD);
        }
        return true;
    }
    
    @Override
    public BlockState rotate(final BlockState byg, final Rotation btr) {
        return ((AbstractStateHolder<O, BlockState>)byg).<Comparable, Direction>setValue((Property<Comparable>)PistonBaseBlock.FACING, btr.rotate(byg.<Direction>getValue((Property<Direction>)PistonBaseBlock.FACING)));
    }
    
    @Override
    public BlockState mirror(final BlockState byg, final Mirror bsr) {
        return byg.rotate(bsr.getRotation(byg.<Direction>getValue((Property<Direction>)PistonBaseBlock.FACING)));
    }
    
    @Override
    protected void createBlockStateDefinition(final StateDefinition.Builder<Block, BlockState> a) {
        a.add(PistonBaseBlock.FACING, PistonBaseBlock.EXTENDED);
    }
    
    @Override
    public boolean useShapeForLightOcclusion(final BlockState byg) {
        return byg.<Boolean>getValue((Property<Boolean>)PistonBaseBlock.EXTENDED);
    }
    
    @Override
    public boolean isPathfindable(final BlockState byg, final BlockGetter bjd, final BlockPos fk, final PathComputationType cqo) {
        return false;
    }
    
    static {
        EXTENDED = BlockStateProperties.EXTENDED;
        EAST_AABB = Block.box(0.0, 0.0, 0.0, 12.0, 16.0, 16.0);
        WEST_AABB = Block.box(4.0, 0.0, 0.0, 16.0, 16.0, 16.0);
        SOUTH_AABB = Block.box(0.0, 0.0, 0.0, 16.0, 16.0, 12.0);
        NORTH_AABB = Block.box(0.0, 0.0, 4.0, 16.0, 16.0, 16.0);
        UP_AABB = Block.box(0.0, 0.0, 0.0, 16.0, 12.0, 16.0);
        DOWN_AABB = Block.box(0.0, 4.0, 0.0, 16.0, 16.0, 16.0);
    }
}
