package net.minecraft.world.level.block.piston;

import net.minecraft.world.level.block.state.AbstractStateHolder;
import net.minecraft.world.level.block.state.properties.BlockStateProperties;
import net.minecraft.world.level.pathfinder.PathComputationType;
import net.minecraft.world.level.block.state.StateDefinition;
import net.minecraft.world.level.block.Mirror;
import net.minecraft.world.level.block.Rotation;
import net.minecraft.world.level.ItemLike;
import net.minecraft.world.item.ItemStack;
import net.minecraft.world.level.LevelReader;
import net.minecraft.world.level.LevelAccessor;
import net.minecraft.world.level.block.Blocks;
import net.minecraft.world.entity.player.Player;
import net.minecraft.world.level.Level;
import net.minecraft.world.phys.shapes.Shapes;
import net.minecraft.world.phys.shapes.CollisionContext;
import net.minecraft.core.BlockPos;
import net.minecraft.world.level.BlockGetter;
import net.minecraft.world.level.block.state.properties.Property;
import net.minecraft.core.Direction;
import net.minecraft.world.level.block.state.BlockState;
import net.minecraft.world.level.block.Block;
import net.minecraft.world.phys.shapes.VoxelShape;
import net.minecraft.world.level.block.state.properties.BooleanProperty;
import net.minecraft.world.level.block.state.properties.PistonType;
import net.minecraft.world.level.block.state.properties.EnumProperty;
import net.minecraft.world.level.block.DirectionalBlock;

public class PistonHeadBlock extends DirectionalBlock {
    public static final EnumProperty<PistonType> TYPE;
    public static final BooleanProperty SHORT;
    protected static final VoxelShape EAST_AABB;
    protected static final VoxelShape WEST_AABB;
    protected static final VoxelShape SOUTH_AABB;
    protected static final VoxelShape NORTH_AABB;
    protected static final VoxelShape UP_AABB;
    protected static final VoxelShape DOWN_AABB;
    protected static final VoxelShape UP_ARM_AABB;
    protected static final VoxelShape DOWN_ARM_AABB;
    protected static final VoxelShape SOUTH_ARM_AABB;
    protected static final VoxelShape NORTH_ARM_AABB;
    protected static final VoxelShape EAST_ARM_AABB;
    protected static final VoxelShape WEST_ARM_AABB;
    protected static final VoxelShape SHORT_UP_ARM_AABB;
    protected static final VoxelShape SHORT_DOWN_ARM_AABB;
    protected static final VoxelShape SHORT_SOUTH_ARM_AABB;
    protected static final VoxelShape SHORT_NORTH_ARM_AABB;
    protected static final VoxelShape SHORT_EAST_ARM_AABB;
    protected static final VoxelShape SHORT_WEST_ARM_AABB;
    
    public PistonHeadBlock(final Properties c) {
        super(c);
        this.registerDefaultState(((((AbstractStateHolder<O, BlockState>)this.stateDefinition.any()).setValue((Property<Comparable>)PistonHeadBlock.FACING, Direction.NORTH)).setValue(PistonHeadBlock.TYPE, PistonType.DEFAULT)).<Comparable, Boolean>setValue((Property<Comparable>)PistonHeadBlock.SHORT, false));
    }
    
    private VoxelShape getBaseShape(final BlockState byg) {
        switch (byg.<Direction>getValue((Property<Direction>)PistonHeadBlock.FACING)) {
            default: {
                return PistonHeadBlock.DOWN_AABB;
            }
            case UP: {
                return PistonHeadBlock.UP_AABB;
            }
            case NORTH: {
                return PistonHeadBlock.NORTH_AABB;
            }
            case SOUTH: {
                return PistonHeadBlock.SOUTH_AABB;
            }
            case WEST: {
                return PistonHeadBlock.WEST_AABB;
            }
            case EAST: {
                return PistonHeadBlock.EAST_AABB;
            }
        }
    }
    
    @Override
    public boolean useShapeForLightOcclusion(final BlockState byg) {
        return true;
    }
    
    @Override
    public VoxelShape getShape(final BlockState byg, final BlockGetter bjd, final BlockPos fk, final CollisionContext cvn) {
        return Shapes.or(this.getBaseShape(byg), this.getArmShape(byg));
    }
    
    private VoxelShape getArmShape(final BlockState byg) {
        final boolean boolean3 = byg.<Boolean>getValue((Property<Boolean>)PistonHeadBlock.SHORT);
        switch (byg.<Direction>getValue((Property<Direction>)PistonHeadBlock.FACING)) {
            default: {
                return boolean3 ? PistonHeadBlock.SHORT_DOWN_ARM_AABB : PistonHeadBlock.DOWN_ARM_AABB;
            }
            case UP: {
                return boolean3 ? PistonHeadBlock.SHORT_UP_ARM_AABB : PistonHeadBlock.UP_ARM_AABB;
            }
            case NORTH: {
                return boolean3 ? PistonHeadBlock.SHORT_NORTH_ARM_AABB : PistonHeadBlock.NORTH_ARM_AABB;
            }
            case SOUTH: {
                return boolean3 ? PistonHeadBlock.SHORT_SOUTH_ARM_AABB : PistonHeadBlock.SOUTH_ARM_AABB;
            }
            case WEST: {
                return boolean3 ? PistonHeadBlock.SHORT_WEST_ARM_AABB : PistonHeadBlock.WEST_ARM_AABB;
            }
            case EAST: {
                return boolean3 ? PistonHeadBlock.SHORT_EAST_ARM_AABB : PistonHeadBlock.EAST_ARM_AABB;
            }
        }
    }
    
    @Override
    public void playerWillDestroy(final Level bjt, final BlockPos fk, final BlockState byg, final Player ayg) {
        if (!bjt.isClientSide && ayg.abilities.instabuild) {
            final BlockPos fk2 = fk.relative(byg.<Direction>getValue((Property<Direction>)PistonHeadBlock.FACING).getOpposite());
            final Block bpe7 = bjt.getBlockState(fk2).getBlock();
            if (bpe7 == Blocks.PISTON || bpe7 == Blocks.STICKY_PISTON) {
                bjt.removeBlock(fk2, false);
            }
        }
        super.playerWillDestroy(bjt, fk, byg, ayg);
    }
    
    @Override
    public void onRemove(final BlockState byg1, final Level bjt, BlockPos fk, final BlockState byg4, final boolean boolean5) {
        if (byg1.getBlock() == byg4.getBlock()) {
            return;
        }
        super.onRemove(byg1, bjt, fk, byg4, boolean5);
        final Direction fp7 = byg1.<Direction>getValue((Property<Direction>)PistonHeadBlock.FACING).getOpposite();
        fk = fk.relative(fp7);
        final BlockState byg5 = bjt.getBlockState(fk);
        if ((byg5.getBlock() == Blocks.PISTON || byg5.getBlock() == Blocks.STICKY_PISTON) && byg5.<Boolean>getValue((Property<Boolean>)PistonBaseBlock.EXTENDED)) {
            Block.dropResources(byg5, bjt, fk);
            bjt.removeBlock(fk, false);
        }
    }
    
    @Override
    public BlockState updateShape(final BlockState byg1, final Direction fp, final BlockState byg3, final LevelAccessor bju, final BlockPos fk5, final BlockPos fk6) {
        if (fp.getOpposite() == byg1.<Direction>getValue((Property<Direction>)PistonHeadBlock.FACING) && !byg1.canSurvive(bju, fk5)) {
            return Blocks.AIR.defaultBlockState();
        }
        return super.updateShape(byg1, fp, byg3, bju, fk5, fk6);
    }
    
    @Override
    public boolean canSurvive(final BlockState byg, final LevelReader bjw, final BlockPos fk) {
        final Block bpe5 = bjw.getBlockState(fk.relative(byg.<Direction>getValue((Property<Direction>)PistonHeadBlock.FACING).getOpposite())).getBlock();
        return bpe5 == Blocks.PISTON || bpe5 == Blocks.STICKY_PISTON || bpe5 == Blocks.MOVING_PISTON;
    }
    
    @Override
    public void neighborChanged(final BlockState byg, final Level bjt, final BlockPos fk3, final Block bpe, final BlockPos fk5, final boolean boolean6) {
        if (byg.canSurvive(bjt, fk3)) {
            final BlockPos fk6 = fk3.relative(byg.<Direction>getValue((Property<Direction>)PistonHeadBlock.FACING).getOpposite());
            bjt.getBlockState(fk6).neighborChanged(bjt, fk6, bpe, fk5, false);
        }
    }
    
    @Override
    public ItemStack getCloneItemStack(final BlockGetter bjd, final BlockPos fk, final BlockState byg) {
        return new ItemStack((byg.<PistonType>getValue(PistonHeadBlock.TYPE) == PistonType.STICKY) ? Blocks.STICKY_PISTON : Blocks.PISTON);
    }
    
    @Override
    public BlockState rotate(final BlockState byg, final Rotation btr) {
        return ((AbstractStateHolder<O, BlockState>)byg).<Comparable, Direction>setValue((Property<Comparable>)PistonHeadBlock.FACING, btr.rotate(byg.<Direction>getValue((Property<Direction>)PistonHeadBlock.FACING)));
    }
    
    @Override
    public BlockState mirror(final BlockState byg, final Mirror bsr) {
        return byg.rotate(bsr.getRotation(byg.<Direction>getValue((Property<Direction>)PistonHeadBlock.FACING)));
    }
    
    @Override
    protected void createBlockStateDefinition(final StateDefinition.Builder<Block, BlockState> a) {
        a.add(PistonHeadBlock.FACING, PistonHeadBlock.TYPE, PistonHeadBlock.SHORT);
    }
    
    @Override
    public boolean isPathfindable(final BlockState byg, final BlockGetter bjd, final BlockPos fk, final PathComputationType cqo) {
        return false;
    }
    
    static {
        TYPE = BlockStateProperties.PISTON_TYPE;
        SHORT = BlockStateProperties.SHORT;
        EAST_AABB = Block.box(12.0, 0.0, 0.0, 16.0, 16.0, 16.0);
        WEST_AABB = Block.box(0.0, 0.0, 0.0, 4.0, 16.0, 16.0);
        SOUTH_AABB = Block.box(0.0, 0.0, 12.0, 16.0, 16.0, 16.0);
        NORTH_AABB = Block.box(0.0, 0.0, 0.0, 16.0, 16.0, 4.0);
        UP_AABB = Block.box(0.0, 12.0, 0.0, 16.0, 16.0, 16.0);
        DOWN_AABB = Block.box(0.0, 0.0, 0.0, 16.0, 4.0, 16.0);
        UP_ARM_AABB = Block.box(6.0, -4.0, 6.0, 10.0, 12.0, 10.0);
        DOWN_ARM_AABB = Block.box(6.0, 4.0, 6.0, 10.0, 20.0, 10.0);
        SOUTH_ARM_AABB = Block.box(6.0, 6.0, -4.0, 10.0, 10.0, 12.0);
        NORTH_ARM_AABB = Block.box(6.0, 6.0, 4.0, 10.0, 10.0, 20.0);
        EAST_ARM_AABB = Block.box(-4.0, 6.0, 6.0, 12.0, 10.0, 10.0);
        WEST_ARM_AABB = Block.box(4.0, 6.0, 6.0, 20.0, 10.0, 10.0);
        SHORT_UP_ARM_AABB = Block.box(6.0, 0.0, 6.0, 10.0, 12.0, 10.0);
        SHORT_DOWN_ARM_AABB = Block.box(6.0, 4.0, 6.0, 10.0, 16.0, 10.0);
        SHORT_SOUTH_ARM_AABB = Block.box(6.0, 6.0, 0.0, 10.0, 10.0, 12.0);
        SHORT_NORTH_ARM_AABB = Block.box(6.0, 6.0, 4.0, 10.0, 10.0, 16.0);
        SHORT_EAST_ARM_AABB = Block.box(0.0, 6.0, 6.0, 12.0, 10.0, 10.0);
        SHORT_WEST_ARM_AABB = Block.box(4.0, 6.0, 6.0, 16.0, 10.0, 10.0);
    }
}
