package net.minecraft.world.level.block.state.predicate;

import java.util.Iterator;
import javax.annotation.Nullable;
import com.google.common.collect.Maps;
import net.minecraft.world.level.block.state.properties.Property;
import java.util.Map;
import net.minecraft.world.level.block.Block;
import net.minecraft.world.level.block.state.StateDefinition;
import net.minecraft.world.level.block.state.BlockState;
import java.util.function.Predicate;

public class BlockStatePredicate implements Predicate<BlockState> {
    public static final Predicate<BlockState> ANY;
    private final StateDefinition<Block, BlockState> definition;
    private final Map<Property<?>, Predicate<Object>> properties;
    
    private BlockStatePredicate(final StateDefinition<Block, BlockState> byh) {
        this.properties = Maps.newHashMap();
        this.definition = byh;
    }
    
    public static BlockStatePredicate forBlock(final Block bpe) {
        return new BlockStatePredicate(bpe.getStateDefinition());
    }
    
    @Override
    public boolean test(@Nullable final BlockState byg) {
        if (byg == null || !byg.getBlock().equals(this.definition.getOwner())) {
            return false;
        }
        if (this.properties.isEmpty()) {
            return true;
        }
        for (final Map.Entry<Property<?>, Predicate<Object>> entry4 : this.properties.entrySet()) {
            if (!this.applies(byg, entry4.getKey(), entry4.getValue())) {
                return false;
            }
        }
        return true;
    }
    
    protected <T extends Comparable<T>> boolean applies(final BlockState byg, final Property<T> bzj, final Predicate<Object> predicate) {
        final T comparable5 = byg.<T>getValue(bzj);
        return predicate.test(comparable5);
    }
    
    public <V extends Comparable<V>> BlockStatePredicate where(final Property<V> bzj, final Predicate<Object> predicate) {
        if (!this.definition.getProperties().contains(bzj)) {
            throw new IllegalArgumentException(this.definition + " cannot support property " + bzj);
        }
        this.properties.put(bzj, predicate);
        return this;
    }
    
    static {
        ANY = (byg -> true);
    }
}
