package net.minecraft.world.level.chunk;

import net.minecraft.world.level.levelgen.Heightmap;
import net.minecraft.network.protocol.game.DebugPackets;
import java.util.Map;
import java.util.Iterator;
import net.minecraft.world.level.levelgen.structure.BoundingBox;
import net.minecraft.world.level.levelgen.structure.StructureStart;
import net.minecraft.world.level.levelgen.structure.templatesystem.StructureManager;
import net.minecraft.world.entity.MobCategory;
import net.minecraft.world.level.levelgen.feature.configurations.FeatureConfiguration;
import net.minecraft.server.level.ServerLevel;
import net.minecraft.ReportedException;
import net.minecraft.core.Registry;
import net.minecraft.CrashReport;
import net.minecraft.server.level.WorldGenRegion;
import javax.annotation.Nullable;
import java.util.Locale;
import net.minecraft.world.level.levelgen.feature.Feature;
import net.minecraft.world.level.levelgen.feature.StructureFeature;
import net.minecraft.world.level.Level;
import java.util.ListIterator;
import java.util.List;
import java.util.BitSet;
import java.util.Random;
import net.minecraft.world.level.levelgen.carver.ConfiguredWorldCarver;
import net.minecraft.world.level.levelgen.WorldgenRandom;
import net.minecraft.world.level.levelgen.GenerationStep;
import net.minecraft.world.level.biome.Biome;
import net.minecraft.core.BlockPos;
import net.minecraft.world.level.biome.BiomeManager;
import net.minecraft.world.level.ChunkPos;
import net.minecraft.world.level.biome.BiomeSource;
import net.minecraft.world.level.LevelAccessor;
import net.minecraft.world.level.levelgen.ChunkGeneratorSettings;

public abstract class ChunkGenerator<C extends ChunkGeneratorSettings> {
    protected final LevelAccessor level;
    protected final long seed;
    protected final BiomeSource biomeSource;
    protected final C settings;
    
    public ChunkGenerator(final LevelAccessor bju, final BiomeSource bkt, final C cbn) {
        this.level = bju;
        this.seed = bju.getSeed();
        this.biomeSource = bkt;
        this.settings = cbn;
    }
    
    public void createBiomes(final ChunkAccess bzv) {
        final ChunkPos bje3 = bzv.getPos();
        ((ProtoChunk)bzv).setBiomes(new ChunkBiomeContainer(bje3, this.biomeSource));
    }
    
    protected Biome getCarvingOrDecorationBiome(final BiomeManager bks, final BlockPos fk) {
        return bks.getBiome(fk);
    }
    
    public void applyCarvers(final BiomeManager bks, final ChunkAccess bzv, final GenerationStep.Carving a) {
        final WorldgenRandom ccc5 = new WorldgenRandom();
        final int integer6 = 8;
        final ChunkPos bje7 = bzv.getPos();
        final int integer7 = bje7.x;
        final int integer8 = bje7.z;
        final Biome bkq10 = this.getCarvingOrDecorationBiome(bks, bje7.getWorldPosition());
        final BitSet bitSet11 = bzv.getCarvingMask(a);
        for (int integer9 = integer7 - 8; integer9 <= integer7 + 8; ++integer9) {
            for (int integer10 = integer8 - 8; integer10 <= integer8 + 8; ++integer10) {
                final List<ConfiguredWorldCarver<?>> list14 = bkq10.getCarvers(a);
                final ListIterator<ConfiguredWorldCarver<?>> listIterator15 = list14.listIterator();
                while (listIterator15.hasNext()) {
                    final int integer11 = listIterator15.nextIndex();
                    final ConfiguredWorldCarver<?> ccg17 = listIterator15.next();
                    ccc5.setLargeFeatureSeed(this.seed + integer11, integer9, integer10);
                    if (ccg17.isStartChunk(ccc5, integer9, integer10)) {
                        ccg17.carve(bzv, fk -> this.getCarvingOrDecorationBiome(bks, fk), (Random)ccc5, this.getSeaLevel(), integer9, integer10, integer7, integer8, bitSet11);
                    }
                }
            }
        }
    }
    
    @Nullable
    public BlockPos findNearestMapFeature(final Level bjt, final String string, final BlockPos fk, final int integer, final boolean boolean5) {
        final StructureFeature<?> cfi7 = Feature.STRUCTURES_REGISTRY.get(string.toLowerCase(Locale.ROOT));
        if (cfi7 != null) {
            return cfi7.getNearestGeneratedFeature(bjt, this, fk, integer, boolean5);
        }
        return null;
    }
    
    public void applyBiomeDecoration(final WorldGenRegion xj) {
        final int integer3 = xj.getCenterX();
        final int integer4 = xj.getCenterZ();
        final int integer5 = integer3 * 16;
        final int integer6 = integer4 * 16;
        final BlockPos fk7 = new BlockPos(integer5, 0, integer6);
        final Biome bkq8 = this.getCarvingOrDecorationBiome(xj.getBiomeManager(), fk7.offset(8, 8, 8));
        final WorldgenRandom ccc9 = new WorldgenRandom();
        final long long10 = ccc9.setDecorationSeed(xj.getSeed(), integer5, integer6);
        for (final GenerationStep.Decoration b15 : GenerationStep.Decoration.values()) {
            try {
                bkq8.generate(b15, this, xj, long10, ccc9, fk7);
            }
            catch (Exception exception16) {
                final CrashReport h17 = CrashReport.forThrowable(exception16, "Biome decoration");
                h17.addCategory("Generation").setDetail("CenterX", integer3).setDetail("CenterZ", integer4).setDetail("Step", b15).setDetail("Seed", long10).setDetail("Biome", Registry.BIOME.getKey(bkq8));
                throw new ReportedException(h17);
            }
        }
    }
    
    public abstract void buildSurfaceAndBedrock(final WorldGenRegion xj, final ChunkAccess bzv);
    
    public void spawnOriginalMobs(final WorldGenRegion xj) {
    }
    
    public C getSettings() {
        return this.settings;
    }
    
    public abstract int getSpawnHeight();
    
    public void tickCustomSpawners(final ServerLevel xd, final boolean boolean2, final boolean boolean3) {
    }
    
    public boolean isBiomeValidStartForStructure(final Biome bkq, final StructureFeature<? extends FeatureConfiguration> cfi) {
        return bkq.isValidStart(cfi);
    }
    
    @Nullable
    public <C extends FeatureConfiguration> C getStructureConfiguration(final Biome bkq, final StructureFeature<C> cfi) {
        return bkq.<C>getStructureConfiguration(cfi);
    }
    
    public BiomeSource getBiomeSource() {
        return this.biomeSource;
    }
    
    public long getSeed() {
        return this.seed;
    }
    
    public int getGenDepth() {
        return 256;
    }
    
    public List<Biome.SpawnerData> getMobsAt(final MobCategory aky, final BlockPos fk) {
        return this.level.getBiome(fk).getMobs(aky);
    }
    
    public void createStructures(final BiomeManager bks, final ChunkAccess bzv, final ChunkGenerator<?> bzx, final StructureManager cml) {
        for (final StructureFeature<?> cfi7 : Feature.STRUCTURES_REGISTRY.values()) {
            if (!bzx.getBiomeSource().canGenerateStructure(cfi7)) {
                continue;
            }
            final StructureStart cls8 = bzv.getStartForFeature(cfi7.getFeatureName());
            final int integer9 = (cls8 != null) ? cls8.getReferences() : 0;
            final WorldgenRandom ccc10 = new WorldgenRandom();
            final ChunkPos bje11 = bzv.getPos();
            StructureStart cls9 = StructureStart.INVALID_START;
            final Biome bkq13 = bks.getBiome(new BlockPos(bje11.getMinBlockX() + 9, 0, bje11.getMinBlockZ() + 9));
            if (cfi7.isFeatureChunk(bks, bzx, ccc10, bje11.x, bje11.z, bkq13)) {
                final StructureStart cls10 = cfi7.getStartFactory().create(cfi7, bje11.x, bje11.z, BoundingBox.getUnknownBox(), integer9, bzx.getSeed());
                cls10.generatePieces(this, cml, bje11.x, bje11.z, bkq13);
                cls9 = (cls10.isValid() ? cls10 : StructureStart.INVALID_START);
            }
            bzv.setStartForFeature(cfi7.getFeatureName(), cls9);
        }
    }
    
    public void createReferences(final LevelAccessor bju, final ChunkAccess bzv) {
        final int integer4 = 8;
        final int integer5 = bzv.getPos().x;
        final int integer6 = bzv.getPos().z;
        final int integer7 = integer5 << 4;
        final int integer8 = integer6 << 4;
        for (int integer9 = integer5 - 8; integer9 <= integer5 + 8; ++integer9) {
            for (int integer10 = integer6 - 8; integer10 <= integer6 + 8; ++integer10) {
                final long long11 = ChunkPos.asLong(integer9, integer10);
                for (final Map.Entry<String, StructureStart> entry14 : bju.getChunk(integer9, integer10).getAllStarts().entrySet()) {
                    final StructureStart cls15 = entry14.getValue();
                    if (cls15 != StructureStart.INVALID_START && cls15.getBoundingBox().intersects(integer7, integer8, integer7 + 15, integer8 + 15)) {
                        bzv.addReferenceForFeature(entry14.getKey(), long11);
                        DebugPackets.sendStructurePacket(bju, cls15);
                    }
                }
            }
        }
    }
    
    public abstract void fillFromNoise(final LevelAccessor bju, final ChunkAccess bzv);
    
    public int getSeaLevel() {
        return 63;
    }
    
    public abstract int getBaseHeight(final int integer1, final int integer2, final Heightmap.Types a);
    
    public int getFirstFreeHeight(final int integer1, final int integer2, final Heightmap.Types a) {
        return this.getBaseHeight(integer1, integer2, a);
    }
    
    public int getFirstOccupiedHeight(final int integer1, final int integer2, final Heightmap.Types a) {
        return this.getBaseHeight(integer1, integer2, a) - 1;
    }
}
