package net.minecraft.world.level.levelgen.feature;

import net.minecraft.world.level.levelgen.ChunkGeneratorSettings;
import net.minecraft.world.level.LevelAccessor;
import net.minecraft.world.level.levelgen.structure.OceanMonumentPieces;
import net.minecraft.core.Direction;
import net.minecraft.world.level.levelgen.structure.templatesystem.StructureManager;
import net.minecraft.world.level.levelgen.structure.BoundingBox;
import net.minecraft.world.level.levelgen.structure.StructureStart;
import com.google.common.collect.Lists;
import net.minecraft.world.entity.EntityType;
import java.util.Iterator;
import java.util.Set;
import net.minecraft.world.level.levelgen.feature.configurations.FeatureConfiguration;
import net.minecraft.world.level.biome.BiomeManager;
import net.minecraft.world.level.levelgen.WorldgenRandom;
import net.minecraft.world.level.ChunkPos;
import java.util.Random;
import net.minecraft.world.level.chunk.ChunkGenerator;
import com.mojang.datafixers.Dynamic;
import java.util.function.Function;
import net.minecraft.world.level.biome.Biome;
import java.util.List;
import net.minecraft.world.level.levelgen.feature.configurations.NoneFeatureConfiguration;

public class OceanMonumentFeature extends StructureFeature<NoneFeatureConfiguration> {
    private static final List<Biome.SpawnerData> MONUMENT_ENEMIES;
    
    public OceanMonumentFeature(final Function<Dynamic<?>, ? extends NoneFeatureConfiguration> function) {
        super(function);
    }
    
    @Override
    protected ChunkPos getPotentialFeatureChunkFromLocationWithOffset(final ChunkGenerator<?> bzx, final Random random, final int integer3, final int integer4, final int integer5, final int integer6) {
        final int integer7 = ((ChunkGeneratorSettings)bzx.getSettings()).getMonumentsSpacing();
        final int integer8 = ((ChunkGeneratorSettings)bzx.getSettings()).getMonumentsSeparation();
        final int integer9 = integer3 + integer7 * integer5;
        final int integer10 = integer4 + integer7 * integer6;
        final int integer11 = (integer9 < 0) ? (integer9 - integer7 + 1) : integer9;
        final int integer12 = (integer10 < 0) ? (integer10 - integer7 + 1) : integer10;
        int integer13 = integer11 / integer7;
        int integer14 = integer12 / integer7;
        ((WorldgenRandom)random).setLargeFeatureWithSalt(bzx.getSeed(), integer13, integer14, 10387313);
        integer13 *= integer7;
        integer14 *= integer7;
        integer13 += (random.nextInt(integer7 - integer8) + random.nextInt(integer7 - integer8)) / 2;
        integer14 += (random.nextInt(integer7 - integer8) + random.nextInt(integer7 - integer8)) / 2;
        return new ChunkPos(integer13, integer14);
    }
    
    @Override
    public boolean isFeatureChunk(final BiomeManager bks, final ChunkGenerator<?> bzx, final Random random, final int integer4, final int integer5, final Biome bkq) {
        final ChunkPos bje8 = this.getPotentialFeatureChunkFromLocationWithOffset(bzx, random, integer4, integer5, 0, 0);
        if (integer4 == bje8.x && integer5 == bje8.z) {
            final Set<Biome> set9 = bzx.getBiomeSource().getBiomesWithin(integer4 * 16 + 9, bzx.getSeaLevel(), integer5 * 16 + 9, 16);
            for (final Biome bkq2 : set9) {
                if (!bzx.isBiomeValidStartForStructure(bkq2, this)) {
                    return false;
                }
            }
            final Set<Biome> set10 = bzx.getBiomeSource().getBiomesWithin(integer4 * 16 + 9, bzx.getSeaLevel(), integer5 * 16 + 9, 29);
            for (final Biome bkq3 : set10) {
                if (bkq3.getBiomeCategory() != Biome.BiomeCategory.OCEAN && bkq3.getBiomeCategory() != Biome.BiomeCategory.RIVER) {
                    return false;
                }
            }
            return true;
        }
        return false;
    }
    
    @Override
    public StructureStartFactory getStartFactory() {
        return OceanMonumentStart::new;
    }
    
    @Override
    public String getFeatureName() {
        return "Monument";
    }
    
    @Override
    public int getLookupRange() {
        return 8;
    }
    
    @Override
    public List<Biome.SpawnerData> getSpecialEnemies() {
        return OceanMonumentFeature.MONUMENT_ENEMIES;
    }
    
    static {
        MONUMENT_ENEMIES = Lists.<Biome.SpawnerData>newArrayList(new Biome.SpawnerData(EntityType.GUARDIAN, 1, 2, 4));
    }
    
    public static class OceanMonumentStart extends StructureStart {
        private boolean isCreated;
        
        public OceanMonumentStart(final StructureFeature<?> cfi, final int integer2, final int integer3, final BoundingBox cky, final int integer5, final long long6) {
            super(cfi, integer2, integer3, cky, integer5, long6);
        }
        
        @Override
        public void generatePieces(final ChunkGenerator<?> bzx, final StructureManager cml, final int integer3, final int integer4, final Biome bkq) {
            this.generatePieces(integer3, integer4);
        }
        
        private void generatePieces(final int integer1, final int integer2) {
            final int integer3 = integer1 * 16 - 29;
            final int integer4 = integer2 * 16 - 29;
            final Direction fp6 = Direction.Plane.HORIZONTAL.getRandomDirection(this.random);
            this.pieces.add(new OceanMonumentPieces.MonumentBuilding(this.random, integer3, integer4, fp6));
            this.calculateBoundingBox();
            this.isCreated = true;
        }
        
        @Override
        public void postProcess(final LevelAccessor bju, final ChunkGenerator<?> bzx, final Random random, final BoundingBox cky, final ChunkPos bje) {
            if (!this.isCreated) {
                this.pieces.clear();
                this.generatePieces(this.getChunkX(), this.getChunkZ());
            }
            super.postProcess(bju, bzx, random, cky, bje);
        }
    }
}
