package net.minecraft.world.level.levelgen.feature.treedecorators;

import java.util.Map;
import com.google.common.collect.ImmutableMap;
import net.minecraft.core.Registry;
import com.mojang.datafixers.types.DynamicOps;
import net.minecraft.world.level.LevelWriter;
import net.minecraft.world.level.block.state.properties.BooleanProperty;
import net.minecraft.world.level.LevelSimulatedRW;
import net.minecraft.world.level.block.VineBlock;
import net.minecraft.world.level.LevelSimulatedReader;
import net.minecraft.world.level.levelgen.feature.AbstractTreeFeature;
import net.minecraft.world.level.levelgen.structure.BoundingBox;
import java.util.Set;
import net.minecraft.core.BlockPos;
import java.util.List;
import java.util.Random;
import net.minecraft.world.level.LevelAccessor;
import com.mojang.datafixers.Dynamic;

public class LeaveVineDecorator extends TreeDecorator {
    public LeaveVineDecorator() {
        super(TreeDecoratorType.LEAVE_VINE);
    }
    
    public <T> LeaveVineDecorator(final Dynamic<T> dynamic) {
        this();
    }
    
    @Override
    public void place(final LevelAccessor bju, final Random random, final List<BlockPos> list3, final List<BlockPos> list4, final Set<BlockPos> set, final BoundingBox cky) {
        BlockPos fk2;
        BlockPos fk3;
        BlockPos fk4;
        BlockPos fk5;
        list4.forEach(fk -> {
            if (random.nextInt(4) == 0) {
                fk2 = fk.west();
                if (AbstractTreeFeature.isAir(bju, fk2)) {
                    this.addHangingVine(bju, fk2, VineBlock.EAST, set, cky);
                }
            }
            if (random.nextInt(4) == 0) {
                fk3 = fk.east();
                if (AbstractTreeFeature.isAir(bju, fk3)) {
                    this.addHangingVine(bju, fk3, VineBlock.WEST, set, cky);
                }
            }
            if (random.nextInt(4) == 0) {
                fk4 = fk.north();
                if (AbstractTreeFeature.isAir(bju, fk4)) {
                    this.addHangingVine(bju, fk4, VineBlock.SOUTH, set, cky);
                }
            }
            if (random.nextInt(4) == 0) {
                fk5 = fk.south();
                if (AbstractTreeFeature.isAir(bju, fk5)) {
                    this.addHangingVine(bju, fk5, VineBlock.NORTH, set, cky);
                }
            }
        });
    }
    
    private void addHangingVine(final LevelSimulatedRW bjy, BlockPos fk, final BooleanProperty byy, final Set<BlockPos> set, final BoundingBox cky) {
        this.placeVine(bjy, fk, byy, set, cky);
        int integer7;
        for (integer7 = 4, fk = fk.below(); AbstractTreeFeature.isAir(bjy, fk) && integer7 > 0; fk = fk.below(), --integer7) {
            this.placeVine(bjy, fk, byy, set, cky);
        }
    }
    
    @Override
    public <T> T serialize(final DynamicOps<T> dynamicOps) {
        return (T)new Dynamic((DynamicOps)dynamicOps, dynamicOps.createMap((Map)ImmutableMap.of(dynamicOps.createString("type"), dynamicOps.createString(Registry.TREE_DECORATOR_TYPES.getKey(this.type).toString())))).getValue();
    }
}
