package net.minecraft.world.level.levelgen.structure;

import net.minecraft.world.level.block.state.BlockState;
import net.minecraft.world.level.storage.loot.BuiltInLootTables;
import net.minecraft.core.Direction;
import net.minecraft.world.level.block.Blocks;
import net.minecraft.world.level.levelgen.Heightmap;
import net.minecraft.world.level.ChunkPos;
import java.util.Random;
import net.minecraft.world.level.chunk.ChunkGenerator;
import net.minecraft.world.level.LevelAccessor;
import net.minecraft.nbt.CompoundTag;
import net.minecraft.world.level.levelgen.structure.templatesystem.StructureManager;
import net.minecraft.world.level.levelgen.feature.StructurePieceType;
import net.minecraft.core.BlockPos;

public class BuriedTreasurePieces {
    public static class BuriedTreasurePiece extends StructurePiece {
        public BuriedTreasurePiece(final BlockPos fk) {
            super(StructurePieceType.BURIED_TREASURE_PIECE, 0);
            this.boundingBox = new BoundingBox(fk.getX(), fk.getY(), fk.getZ(), fk.getX(), fk.getY(), fk.getZ());
        }
        
        public BuriedTreasurePiece(final StructureManager cml, final CompoundTag jt) {
            super(StructurePieceType.BURIED_TREASURE_PIECE, jt);
        }
        
        @Override
        protected void addAdditionalSaveData(final CompoundTag jt) {
        }
        
        @Override
        public boolean postProcess(final LevelAccessor bju, final ChunkGenerator<?> bzx, final Random random, final BoundingBox cky, final ChunkPos bje) {
            final int integer7 = bju.getHeight(Heightmap.Types.OCEAN_FLOOR_WG, this.boundingBox.x0, this.boundingBox.z0);
            final BlockPos.MutableBlockPos a8 = new BlockPos.MutableBlockPos(this.boundingBox.x0, integer7, this.boundingBox.z0);
            while (a8.getY() > 0) {
                final BlockState byg9 = bju.getBlockState(a8);
                final BlockState byg10 = bju.getBlockState(a8.below());
                if (byg10 == Blocks.SANDSTONE.defaultBlockState() || byg10 == Blocks.STONE.defaultBlockState() || byg10 == Blocks.ANDESITE.defaultBlockState() || byg10 == Blocks.GRANITE.defaultBlockState() || byg10 == Blocks.DIORITE.defaultBlockState()) {
                    final BlockState byg11 = (byg9.isAir() || this.isLiquid(byg9)) ? Blocks.SAND.defaultBlockState() : byg9;
                    for (final Direction fp15 : Direction.values()) {
                        final BlockPos fk16 = a8.relative(fp15);
                        final BlockState byg12 = bju.getBlockState(fk16);
                        if (byg12.isAir() || this.isLiquid(byg12)) {
                            final BlockPos fk17 = fk16.below();
                            final BlockState byg13 = bju.getBlockState(fk17);
                            if ((byg13.isAir() || this.isLiquid(byg13)) && fp15 != Direction.UP) {
                                bju.setBlock(fk16, byg10, 3);
                            }
                            else {
                                bju.setBlock(fk16, byg11, 3);
                            }
                        }
                    }
                    this.boundingBox = new BoundingBox(a8.getX(), a8.getY(), a8.getZ(), a8.getX(), a8.getY(), a8.getZ());
                    return this.createChest(bju, cky, random, a8, BuiltInLootTables.BURIED_TREASURE, null);
                }
                a8.move(0, -1, 0);
            }
            return false;
        }
        
        private boolean isLiquid(final BlockState byg) {
            return byg == Blocks.WATER.defaultBlockState() || byg == Blocks.LAVA.defaultBlockState();
        }
    }
}
