package net.minecraft.world.level.levelgen.structure;

import net.minecraft.world.level.block.state.AbstractStateHolder;
import java.util.Iterator;
import net.minecraft.world.level.storage.loot.BuiltInLootTables;
import net.minecraft.world.level.block.state.properties.Property;
import net.minecraft.core.Direction;
import net.minecraft.world.level.block.StairBlock;
import net.minecraft.world.level.block.state.BlockState;
import net.minecraft.world.level.block.Blocks;
import net.minecraft.world.level.ChunkPos;
import net.minecraft.world.level.chunk.ChunkGenerator;
import net.minecraft.world.level.LevelAccessor;
import net.minecraft.nbt.CompoundTag;
import net.minecraft.world.level.levelgen.structure.templatesystem.StructureManager;
import net.minecraft.world.level.levelgen.feature.StructurePieceType;
import java.util.Random;

public class DesertPyramidPiece extends ScatteredFeaturePiece {
    private final boolean[] hasPlacedChest;
    
    public DesertPyramidPiece(final Random random, final int integer2, final int integer3) {
        super(StructurePieceType.DESERT_PYRAMID_PIECE, random, integer2, 64, integer3, 21, 15, 21);
        this.hasPlacedChest = new boolean[4];
    }
    
    public DesertPyramidPiece(final StructureManager cml, final CompoundTag jt) {
        super(StructurePieceType.DESERT_PYRAMID_PIECE, jt);
        (this.hasPlacedChest = new boolean[4])[0] = jt.getBoolean("hasPlacedChest0");
        this.hasPlacedChest[1] = jt.getBoolean("hasPlacedChest1");
        this.hasPlacedChest[2] = jt.getBoolean("hasPlacedChest2");
        this.hasPlacedChest[3] = jt.getBoolean("hasPlacedChest3");
    }
    
    @Override
    protected void addAdditionalSaveData(final CompoundTag jt) {
        super.addAdditionalSaveData(jt);
        jt.putBoolean("hasPlacedChest0", this.hasPlacedChest[0]);
        jt.putBoolean("hasPlacedChest1", this.hasPlacedChest[1]);
        jt.putBoolean("hasPlacedChest2", this.hasPlacedChest[2]);
        jt.putBoolean("hasPlacedChest3", this.hasPlacedChest[3]);
    }
    
    @Override
    public boolean postProcess(final LevelAccessor bju, final ChunkGenerator<?> bzx, final Random random, final BoundingBox cky, final ChunkPos bje) {
        this.generateBox(bju, cky, 0, -4, 0, this.width - 1, 0, this.depth - 1, Blocks.SANDSTONE.defaultBlockState(), Blocks.SANDSTONE.defaultBlockState(), false);
        for (int integer7 = 1; integer7 <= 9; ++integer7) {
            this.generateBox(bju, cky, integer7, integer7, integer7, this.width - 1 - integer7, integer7, this.depth - 1 - integer7, Blocks.SANDSTONE.defaultBlockState(), Blocks.SANDSTONE.defaultBlockState(), false);
            this.generateBox(bju, cky, integer7 + 1, integer7, integer7 + 1, this.width - 2 - integer7, integer7, this.depth - 2 - integer7, Blocks.AIR.defaultBlockState(), Blocks.AIR.defaultBlockState(), false);
        }
        for (int integer7 = 0; integer7 < this.width; ++integer7) {
            for (int integer8 = 0; integer8 < this.depth; ++integer8) {
                final int integer9 = -5;
                this.fillColumnDown(bju, Blocks.SANDSTONE.defaultBlockState(), integer7, -5, integer8, cky);
            }
        }
        final BlockState byg7 = ((AbstractStateHolder<O, BlockState>)Blocks.SANDSTONE_STAIRS.defaultBlockState()).<Comparable, Direction>setValue((Property<Comparable>)StairBlock.FACING, Direction.NORTH);
        final BlockState byg8 = ((AbstractStateHolder<O, BlockState>)Blocks.SANDSTONE_STAIRS.defaultBlockState()).<Comparable, Direction>setValue((Property<Comparable>)StairBlock.FACING, Direction.SOUTH);
        final BlockState byg9 = ((AbstractStateHolder<O, BlockState>)Blocks.SANDSTONE_STAIRS.defaultBlockState()).<Comparable, Direction>setValue((Property<Comparable>)StairBlock.FACING, Direction.EAST);
        final BlockState byg10 = ((AbstractStateHolder<O, BlockState>)Blocks.SANDSTONE_STAIRS.defaultBlockState()).<Comparable, Direction>setValue((Property<Comparable>)StairBlock.FACING, Direction.WEST);
        this.generateBox(bju, cky, 0, 0, 0, 4, 9, 4, Blocks.SANDSTONE.defaultBlockState(), Blocks.AIR.defaultBlockState(), false);
        this.generateBox(bju, cky, 1, 10, 1, 3, 10, 3, Blocks.SANDSTONE.defaultBlockState(), Blocks.SANDSTONE.defaultBlockState(), false);
        this.placeBlock(bju, byg7, 2, 10, 0, cky);
        this.placeBlock(bju, byg8, 2, 10, 4, cky);
        this.placeBlock(bju, byg9, 0, 10, 2, cky);
        this.placeBlock(bju, byg10, 4, 10, 2, cky);
        this.generateBox(bju, cky, this.width - 5, 0, 0, this.width - 1, 9, 4, Blocks.SANDSTONE.defaultBlockState(), Blocks.AIR.defaultBlockState(), false);
        this.generateBox(bju, cky, this.width - 4, 10, 1, this.width - 2, 10, 3, Blocks.SANDSTONE.defaultBlockState(), Blocks.SANDSTONE.defaultBlockState(), false);
        this.placeBlock(bju, byg7, this.width - 3, 10, 0, cky);
        this.placeBlock(bju, byg8, this.width - 3, 10, 4, cky);
        this.placeBlock(bju, byg9, this.width - 5, 10, 2, cky);
        this.placeBlock(bju, byg10, this.width - 1, 10, 2, cky);
        this.generateBox(bju, cky, 8, 0, 0, 12, 4, 4, Blocks.SANDSTONE.defaultBlockState(), Blocks.AIR.defaultBlockState(), false);
        this.generateBox(bju, cky, 9, 1, 0, 11, 3, 4, Blocks.AIR.defaultBlockState(), Blocks.AIR.defaultBlockState(), false);
        this.placeBlock(bju, Blocks.CUT_SANDSTONE.defaultBlockState(), 9, 1, 1, cky);
        this.placeBlock(bju, Blocks.CUT_SANDSTONE.defaultBlockState(), 9, 2, 1, cky);
        this.placeBlock(bju, Blocks.CUT_SANDSTONE.defaultBlockState(), 9, 3, 1, cky);
        this.placeBlock(bju, Blocks.CUT_SANDSTONE.defaultBlockState(), 10, 3, 1, cky);
        this.placeBlock(bju, Blocks.CUT_SANDSTONE.defaultBlockState(), 11, 3, 1, cky);
        this.placeBlock(bju, Blocks.CUT_SANDSTONE.defaultBlockState(), 11, 2, 1, cky);
        this.placeBlock(bju, Blocks.CUT_SANDSTONE.defaultBlockState(), 11, 1, 1, cky);
        this.generateBox(bju, cky, 4, 1, 1, 8, 3, 3, Blocks.SANDSTONE.defaultBlockState(), Blocks.AIR.defaultBlockState(), false);
        this.generateBox(bju, cky, 4, 1, 2, 8, 2, 2, Blocks.AIR.defaultBlockState(), Blocks.AIR.defaultBlockState(), false);
        this.generateBox(bju, cky, 12, 1, 1, 16, 3, 3, Blocks.SANDSTONE.defaultBlockState(), Blocks.AIR.defaultBlockState(), false);
        this.generateBox(bju, cky, 12, 1, 2, 16, 2, 2, Blocks.AIR.defaultBlockState(), Blocks.AIR.defaultBlockState(), false);
        this.generateBox(bju, cky, 5, 4, 5, this.width - 6, 4, this.depth - 6, Blocks.SANDSTONE.defaultBlockState(), Blocks.SANDSTONE.defaultBlockState(), false);
        this.generateBox(bju, cky, 9, 4, 9, 11, 4, 11, Blocks.AIR.defaultBlockState(), Blocks.AIR.defaultBlockState(), false);
        this.generateBox(bju, cky, 8, 1, 8, 8, 3, 8, Blocks.CUT_SANDSTONE.defaultBlockState(), Blocks.CUT_SANDSTONE.defaultBlockState(), false);
        this.generateBox(bju, cky, 12, 1, 8, 12, 3, 8, Blocks.CUT_SANDSTONE.defaultBlockState(), Blocks.CUT_SANDSTONE.defaultBlockState(), false);
        this.generateBox(bju, cky, 8, 1, 12, 8, 3, 12, Blocks.CUT_SANDSTONE.defaultBlockState(), Blocks.CUT_SANDSTONE.defaultBlockState(), false);
        this.generateBox(bju, cky, 12, 1, 12, 12, 3, 12, Blocks.CUT_SANDSTONE.defaultBlockState(), Blocks.CUT_SANDSTONE.defaultBlockState(), false);
        this.generateBox(bju, cky, 1, 1, 5, 4, 4, 11, Blocks.SANDSTONE.defaultBlockState(), Blocks.SANDSTONE.defaultBlockState(), false);
        this.generateBox(bju, cky, this.width - 5, 1, 5, this.width - 2, 4, 11, Blocks.SANDSTONE.defaultBlockState(), Blocks.SANDSTONE.defaultBlockState(), false);
        this.generateBox(bju, cky, 6, 7, 9, 6, 7, 11, Blocks.SANDSTONE.defaultBlockState(), Blocks.SANDSTONE.defaultBlockState(), false);
        this.generateBox(bju, cky, this.width - 7, 7, 9, this.width - 7, 7, 11, Blocks.SANDSTONE.defaultBlockState(), Blocks.SANDSTONE.defaultBlockState(), false);
        this.generateBox(bju, cky, 5, 5, 9, 5, 7, 11, Blocks.CUT_SANDSTONE.defaultBlockState(), Blocks.CUT_SANDSTONE.defaultBlockState(), false);
        this.generateBox(bju, cky, this.width - 6, 5, 9, this.width - 6, 7, 11, Blocks.CUT_SANDSTONE.defaultBlockState(), Blocks.CUT_SANDSTONE.defaultBlockState(), false);
        this.placeBlock(bju, Blocks.AIR.defaultBlockState(), 5, 5, 10, cky);
        this.placeBlock(bju, Blocks.AIR.defaultBlockState(), 5, 6, 10, cky);
        this.placeBlock(bju, Blocks.AIR.defaultBlockState(), 6, 6, 10, cky);
        this.placeBlock(bju, Blocks.AIR.defaultBlockState(), this.width - 6, 5, 10, cky);
        this.placeBlock(bju, Blocks.AIR.defaultBlockState(), this.width - 6, 6, 10, cky);
        this.placeBlock(bju, Blocks.AIR.defaultBlockState(), this.width - 7, 6, 10, cky);
        this.generateBox(bju, cky, 2, 4, 4, 2, 6, 4, Blocks.AIR.defaultBlockState(), Blocks.AIR.defaultBlockState(), false);
        this.generateBox(bju, cky, this.width - 3, 4, 4, this.width - 3, 6, 4, Blocks.AIR.defaultBlockState(), Blocks.AIR.defaultBlockState(), false);
        this.placeBlock(bju, byg7, 2, 4, 5, cky);
        this.placeBlock(bju, byg7, 2, 3, 4, cky);
        this.placeBlock(bju, byg7, this.width - 3, 4, 5, cky);
        this.placeBlock(bju, byg7, this.width - 3, 3, 4, cky);
        this.generateBox(bju, cky, 1, 1, 3, 2, 2, 3, Blocks.SANDSTONE.defaultBlockState(), Blocks.SANDSTONE.defaultBlockState(), false);
        this.generateBox(bju, cky, this.width - 3, 1, 3, this.width - 2, 2, 3, Blocks.SANDSTONE.defaultBlockState(), Blocks.SANDSTONE.defaultBlockState(), false);
        this.placeBlock(bju, Blocks.SANDSTONE.defaultBlockState(), 1, 1, 2, cky);
        this.placeBlock(bju, Blocks.SANDSTONE.defaultBlockState(), this.width - 2, 1, 2, cky);
        this.placeBlock(bju, Blocks.SANDSTONE_SLAB.defaultBlockState(), 1, 2, 2, cky);
        this.placeBlock(bju, Blocks.SANDSTONE_SLAB.defaultBlockState(), this.width - 2, 2, 2, cky);
        this.placeBlock(bju, byg10, 2, 1, 2, cky);
        this.placeBlock(bju, byg9, this.width - 3, 1, 2, cky);
        this.generateBox(bju, cky, 4, 3, 5, 4, 3, 17, Blocks.SANDSTONE.defaultBlockState(), Blocks.SANDSTONE.defaultBlockState(), false);
        this.generateBox(bju, cky, this.width - 5, 3, 5, this.width - 5, 3, 17, Blocks.SANDSTONE.defaultBlockState(), Blocks.SANDSTONE.defaultBlockState(), false);
        this.generateBox(bju, cky, 3, 1, 5, 4, 2, 16, Blocks.AIR.defaultBlockState(), Blocks.AIR.defaultBlockState(), false);
        this.generateBox(bju, cky, this.width - 6, 1, 5, this.width - 5, 2, 16, Blocks.AIR.defaultBlockState(), Blocks.AIR.defaultBlockState(), false);
        for (int integer10 = 5; integer10 <= 17; integer10 += 2) {
            this.placeBlock(bju, Blocks.CUT_SANDSTONE.defaultBlockState(), 4, 1, integer10, cky);
            this.placeBlock(bju, Blocks.CHISELED_SANDSTONE.defaultBlockState(), 4, 2, integer10, cky);
            this.placeBlock(bju, Blocks.CUT_SANDSTONE.defaultBlockState(), this.width - 5, 1, integer10, cky);
            this.placeBlock(bju, Blocks.CHISELED_SANDSTONE.defaultBlockState(), this.width - 5, 2, integer10, cky);
        }
        this.placeBlock(bju, Blocks.ORANGE_TERRACOTTA.defaultBlockState(), 10, 0, 7, cky);
        this.placeBlock(bju, Blocks.ORANGE_TERRACOTTA.defaultBlockState(), 10, 0, 8, cky);
        this.placeBlock(bju, Blocks.ORANGE_TERRACOTTA.defaultBlockState(), 9, 0, 9, cky);
        this.placeBlock(bju, Blocks.ORANGE_TERRACOTTA.defaultBlockState(), 11, 0, 9, cky);
        this.placeBlock(bju, Blocks.ORANGE_TERRACOTTA.defaultBlockState(), 8, 0, 10, cky);
        this.placeBlock(bju, Blocks.ORANGE_TERRACOTTA.defaultBlockState(), 12, 0, 10, cky);
        this.placeBlock(bju, Blocks.ORANGE_TERRACOTTA.defaultBlockState(), 7, 0, 10, cky);
        this.placeBlock(bju, Blocks.ORANGE_TERRACOTTA.defaultBlockState(), 13, 0, 10, cky);
        this.placeBlock(bju, Blocks.ORANGE_TERRACOTTA.defaultBlockState(), 9, 0, 11, cky);
        this.placeBlock(bju, Blocks.ORANGE_TERRACOTTA.defaultBlockState(), 11, 0, 11, cky);
        this.placeBlock(bju, Blocks.ORANGE_TERRACOTTA.defaultBlockState(), 10, 0, 12, cky);
        this.placeBlock(bju, Blocks.ORANGE_TERRACOTTA.defaultBlockState(), 10, 0, 13, cky);
        this.placeBlock(bju, Blocks.BLUE_TERRACOTTA.defaultBlockState(), 10, 0, 10, cky);
        for (int integer10 = 0; integer10 <= this.width - 1; integer10 += this.width - 1) {
            this.placeBlock(bju, Blocks.CUT_SANDSTONE.defaultBlockState(), integer10, 2, 1, cky);
            this.placeBlock(bju, Blocks.ORANGE_TERRACOTTA.defaultBlockState(), integer10, 2, 2, cky);
            this.placeBlock(bju, Blocks.CUT_SANDSTONE.defaultBlockState(), integer10, 2, 3, cky);
            this.placeBlock(bju, Blocks.CUT_SANDSTONE.defaultBlockState(), integer10, 3, 1, cky);
            this.placeBlock(bju, Blocks.ORANGE_TERRACOTTA.defaultBlockState(), integer10, 3, 2, cky);
            this.placeBlock(bju, Blocks.CUT_SANDSTONE.defaultBlockState(), integer10, 3, 3, cky);
            this.placeBlock(bju, Blocks.ORANGE_TERRACOTTA.defaultBlockState(), integer10, 4, 1, cky);
            this.placeBlock(bju, Blocks.CHISELED_SANDSTONE.defaultBlockState(), integer10, 4, 2, cky);
            this.placeBlock(bju, Blocks.ORANGE_TERRACOTTA.defaultBlockState(), integer10, 4, 3, cky);
            this.placeBlock(bju, Blocks.CUT_SANDSTONE.defaultBlockState(), integer10, 5, 1, cky);
            this.placeBlock(bju, Blocks.ORANGE_TERRACOTTA.defaultBlockState(), integer10, 5, 2, cky);
            this.placeBlock(bju, Blocks.CUT_SANDSTONE.defaultBlockState(), integer10, 5, 3, cky);
            this.placeBlock(bju, Blocks.ORANGE_TERRACOTTA.defaultBlockState(), integer10, 6, 1, cky);
            this.placeBlock(bju, Blocks.CHISELED_SANDSTONE.defaultBlockState(), integer10, 6, 2, cky);
            this.placeBlock(bju, Blocks.ORANGE_TERRACOTTA.defaultBlockState(), integer10, 6, 3, cky);
            this.placeBlock(bju, Blocks.ORANGE_TERRACOTTA.defaultBlockState(), integer10, 7, 1, cky);
            this.placeBlock(bju, Blocks.ORANGE_TERRACOTTA.defaultBlockState(), integer10, 7, 2, cky);
            this.placeBlock(bju, Blocks.ORANGE_TERRACOTTA.defaultBlockState(), integer10, 7, 3, cky);
            this.placeBlock(bju, Blocks.CUT_SANDSTONE.defaultBlockState(), integer10, 8, 1, cky);
            this.placeBlock(bju, Blocks.CUT_SANDSTONE.defaultBlockState(), integer10, 8, 2, cky);
            this.placeBlock(bju, Blocks.CUT_SANDSTONE.defaultBlockState(), integer10, 8, 3, cky);
        }
        for (int integer10 = 2; integer10 <= this.width - 3; integer10 += this.width - 3 - 2) {
            this.placeBlock(bju, Blocks.CUT_SANDSTONE.defaultBlockState(), integer10 - 1, 2, 0, cky);
            this.placeBlock(bju, Blocks.ORANGE_TERRACOTTA.defaultBlockState(), integer10, 2, 0, cky);
            this.placeBlock(bju, Blocks.CUT_SANDSTONE.defaultBlockState(), integer10 + 1, 2, 0, cky);
            this.placeBlock(bju, Blocks.CUT_SANDSTONE.defaultBlockState(), integer10 - 1, 3, 0, cky);
            this.placeBlock(bju, Blocks.ORANGE_TERRACOTTA.defaultBlockState(), integer10, 3, 0, cky);
            this.placeBlock(bju, Blocks.CUT_SANDSTONE.defaultBlockState(), integer10 + 1, 3, 0, cky);
            this.placeBlock(bju, Blocks.ORANGE_TERRACOTTA.defaultBlockState(), integer10 - 1, 4, 0, cky);
            this.placeBlock(bju, Blocks.CHISELED_SANDSTONE.defaultBlockState(), integer10, 4, 0, cky);
            this.placeBlock(bju, Blocks.ORANGE_TERRACOTTA.defaultBlockState(), integer10 + 1, 4, 0, cky);
            this.placeBlock(bju, Blocks.CUT_SANDSTONE.defaultBlockState(), integer10 - 1, 5, 0, cky);
            this.placeBlock(bju, Blocks.ORANGE_TERRACOTTA.defaultBlockState(), integer10, 5, 0, cky);
            this.placeBlock(bju, Blocks.CUT_SANDSTONE.defaultBlockState(), integer10 + 1, 5, 0, cky);
            this.placeBlock(bju, Blocks.ORANGE_TERRACOTTA.defaultBlockState(), integer10 - 1, 6, 0, cky);
            this.placeBlock(bju, Blocks.CHISELED_SANDSTONE.defaultBlockState(), integer10, 6, 0, cky);
            this.placeBlock(bju, Blocks.ORANGE_TERRACOTTA.defaultBlockState(), integer10 + 1, 6, 0, cky);
            this.placeBlock(bju, Blocks.ORANGE_TERRACOTTA.defaultBlockState(), integer10 - 1, 7, 0, cky);
            this.placeBlock(bju, Blocks.ORANGE_TERRACOTTA.defaultBlockState(), integer10, 7, 0, cky);
            this.placeBlock(bju, Blocks.ORANGE_TERRACOTTA.defaultBlockState(), integer10 + 1, 7, 0, cky);
            this.placeBlock(bju, Blocks.CUT_SANDSTONE.defaultBlockState(), integer10 - 1, 8, 0, cky);
            this.placeBlock(bju, Blocks.CUT_SANDSTONE.defaultBlockState(), integer10, 8, 0, cky);
            this.placeBlock(bju, Blocks.CUT_SANDSTONE.defaultBlockState(), integer10 + 1, 8, 0, cky);
        }
        this.generateBox(bju, cky, 8, 4, 0, 12, 6, 0, Blocks.CUT_SANDSTONE.defaultBlockState(), Blocks.CUT_SANDSTONE.defaultBlockState(), false);
        this.placeBlock(bju, Blocks.AIR.defaultBlockState(), 8, 6, 0, cky);
        this.placeBlock(bju, Blocks.AIR.defaultBlockState(), 12, 6, 0, cky);
        this.placeBlock(bju, Blocks.ORANGE_TERRACOTTA.defaultBlockState(), 9, 5, 0, cky);
        this.placeBlock(bju, Blocks.CHISELED_SANDSTONE.defaultBlockState(), 10, 5, 0, cky);
        this.placeBlock(bju, Blocks.ORANGE_TERRACOTTA.defaultBlockState(), 11, 5, 0, cky);
        this.generateBox(bju, cky, 8, -14, 8, 12, -11, 12, Blocks.CUT_SANDSTONE.defaultBlockState(), Blocks.CUT_SANDSTONE.defaultBlockState(), false);
        this.generateBox(bju, cky, 8, -10, 8, 12, -10, 12, Blocks.CHISELED_SANDSTONE.defaultBlockState(), Blocks.CHISELED_SANDSTONE.defaultBlockState(), false);
        this.generateBox(bju, cky, 8, -9, 8, 12, -9, 12, Blocks.CUT_SANDSTONE.defaultBlockState(), Blocks.CUT_SANDSTONE.defaultBlockState(), false);
        this.generateBox(bju, cky, 8, -8, 8, 12, -1, 12, Blocks.SANDSTONE.defaultBlockState(), Blocks.SANDSTONE.defaultBlockState(), false);
        this.generateBox(bju, cky, 9, -11, 9, 11, -1, 11, Blocks.AIR.defaultBlockState(), Blocks.AIR.defaultBlockState(), false);
        this.placeBlock(bju, Blocks.STONE_PRESSURE_PLATE.defaultBlockState(), 10, -11, 10, cky);
        this.generateBox(bju, cky, 9, -13, 9, 11, -13, 11, Blocks.TNT.defaultBlockState(), Blocks.AIR.defaultBlockState(), false);
        this.placeBlock(bju, Blocks.AIR.defaultBlockState(), 8, -11, 10, cky);
        this.placeBlock(bju, Blocks.AIR.defaultBlockState(), 8, -10, 10, cky);
        this.placeBlock(bju, Blocks.CHISELED_SANDSTONE.defaultBlockState(), 7, -10, 10, cky);
        this.placeBlock(bju, Blocks.CUT_SANDSTONE.defaultBlockState(), 7, -11, 10, cky);
        this.placeBlock(bju, Blocks.AIR.defaultBlockState(), 12, -11, 10, cky);
        this.placeBlock(bju, Blocks.AIR.defaultBlockState(), 12, -10, 10, cky);
        this.placeBlock(bju, Blocks.CHISELED_SANDSTONE.defaultBlockState(), 13, -10, 10, cky);
        this.placeBlock(bju, Blocks.CUT_SANDSTONE.defaultBlockState(), 13, -11, 10, cky);
        this.placeBlock(bju, Blocks.AIR.defaultBlockState(), 10, -11, 8, cky);
        this.placeBlock(bju, Blocks.AIR.defaultBlockState(), 10, -10, 8, cky);
        this.placeBlock(bju, Blocks.CHISELED_SANDSTONE.defaultBlockState(), 10, -10, 7, cky);
        this.placeBlock(bju, Blocks.CUT_SANDSTONE.defaultBlockState(), 10, -11, 7, cky);
        this.placeBlock(bju, Blocks.AIR.defaultBlockState(), 10, -11, 12, cky);
        this.placeBlock(bju, Blocks.AIR.defaultBlockState(), 10, -10, 12, cky);
        this.placeBlock(bju, Blocks.CHISELED_SANDSTONE.defaultBlockState(), 10, -10, 13, cky);
        this.placeBlock(bju, Blocks.CUT_SANDSTONE.defaultBlockState(), 10, -11, 13, cky);
        for (final Direction fp12 : Direction.Plane.HORIZONTAL) {
            if (!this.hasPlacedChest[fp12.get2DDataValue()]) {
                final int integer11 = fp12.getStepX() * 2;
                final int integer12 = fp12.getStepZ() * 2;
                this.hasPlacedChest[fp12.get2DDataValue()] = this.createChest(bju, cky, random, 10 + integer11, -11, 10 + integer12, BuiltInLootTables.DESERT_PYRAMID);
            }
        }
        return true;
    }
}
