package net.minecraft.world.level.material;

import net.minecraft.world.level.block.state.properties.BlockStateProperties;
import net.minecraft.world.level.Level;
import net.minecraft.world.level.block.Blocks;
import net.minecraft.tags.BlockTags;
import net.minecraft.world.level.block.DoorBlock;
import it.unimi.dsi.fastutil.shorts.Short2BooleanOpenHashMap;
import it.unimi.dsi.fastutil.shorts.Short2ObjectOpenHashMap;
import it.unimi.dsi.fastutil.shorts.Short2BooleanMap;
import com.mojang.datafixers.util.Pair;
import it.unimi.dsi.fastutil.shorts.Short2ObjectMap;
import net.minecraft.world.level.block.LiquidBlockContainer;
import net.minecraft.world.phys.shapes.Shapes;
import net.minecraft.world.level.LevelReader;
import net.minecraft.world.level.LevelAccessor;
import net.minecraft.world.level.block.state.BlockState;
import java.util.Iterator;
import net.minecraft.core.Vec3i;
import net.minecraft.core.Direction;
import net.minecraft.world.phys.Vec3;
import net.minecraft.core.BlockPos;
import net.minecraft.world.level.BlockGetter;
import net.minecraft.world.level.block.state.properties.Property;
import net.minecraft.world.level.block.state.StateDefinition;
import com.google.common.collect.Maps;
import net.minecraft.world.phys.shapes.VoxelShape;
import java.util.Map;
import net.minecraft.world.level.block.Block;
import it.unimi.dsi.fastutil.objects.Object2ByteLinkedOpenHashMap;
import net.minecraft.world.level.block.state.properties.IntegerProperty;
import net.minecraft.world.level.block.state.properties.BooleanProperty;

public abstract class FlowingFluid extends Fluid {
    public static final BooleanProperty FALLING;
    public static final IntegerProperty LEVEL;
    private static final ThreadLocal<Object2ByteLinkedOpenHashMap<Block.BlockStatePairKey>> OCCLUSION_CACHE;
    private final Map<FluidState, VoxelShape> shapes;
    
    public FlowingFluid() {
        this.shapes = Maps.newIdentityHashMap();
    }
    
    @Override
    protected void createFluidStateDefinition(final StateDefinition.Builder<Fluid, FluidState> a) {
        a.add(FlowingFluid.FALLING);
    }
    
    public Vec3 getFlow(final BlockGetter bjd, final BlockPos fk, final FluidState cog) {
        double double5 = 0.0;
        double double6 = 0.0;
        try (final BlockPos.PooledMutableBlockPos b9 = BlockPos.PooledMutableBlockPos.acquire()) {
            for (final Direction fp12 : Direction.Plane.HORIZONTAL) {
                b9.set((Vec3i)fk).move(fp12);
                final FluidState cog2 = bjd.getFluidState(b9);
                if (!this.affectsFlow(cog2)) {
                    continue;
                }
                float float14 = cog2.getOwnHeight();
                float float15 = 0.0f;
                if (float14 == 0.0f) {
                    if (!bjd.getBlockState(b9).getMaterial().blocksMotion()) {
                        final BlockPos fk2 = b9.below();
                        final FluidState cog3 = bjd.getFluidState(fk2);
                        if (this.affectsFlow(cog3)) {
                            float14 = cog3.getOwnHeight();
                            if (float14 > 0.0f) {
                                float15 = cog.getOwnHeight() - (float14 - 0.8888889f);
                            }
                        }
                    }
                }
                else if (float14 > 0.0f) {
                    float15 = cog.getOwnHeight() - float14;
                }
                if (float15 == 0.0f) {
                    continue;
                }
                double5 += fp12.getStepX() * float15;
                double6 += fp12.getStepZ() * float15;
            }
            Vec3 cvi11 = new Vec3(double5, 0.0, double6);
            if (cog.<Boolean>getValue((Property<Boolean>)FlowingFluid.FALLING)) {
                for (final Direction fp13 : Direction.Plane.HORIZONTAL) {
                    b9.set((Vec3i)fk).move(fp13);
                    if (this.isSolidFace(bjd, b9, fp13) || this.isSolidFace(bjd, b9.above(), fp13)) {
                        cvi11 = cvi11.normalize().add(0.0, -6.0, 0.0);
                        break;
                    }
                }
            }
            return cvi11.normalize();
        }
    }
    
    private boolean affectsFlow(final FluidState cog) {
        return cog.isEmpty() || cog.getType().isSame(this);
    }
    
    protected boolean isSolidFace(final BlockGetter bjd, final BlockPos fk, final Direction fp) {
        final BlockState byg5 = bjd.getBlockState(fk);
        final FluidState cog6 = bjd.getFluidState(fk);
        return !cog6.getType().isSame(this) && (fp == Direction.UP || (byg5.getMaterial() != Material.ICE && byg5.isFaceSturdy(bjd, fk, fp)));
    }
    
    protected void spread(final LevelAccessor bju, final BlockPos fk, final FluidState cog) {
        if (cog.isEmpty()) {
            return;
        }
        final BlockState byg5 = bju.getBlockState(fk);
        final BlockPos fk2 = fk.below();
        final BlockState byg6 = bju.getBlockState(fk2);
        final FluidState cog2 = this.getNewLiquid(bju, fk2, byg6);
        if (this.canSpreadTo(bju, fk, byg5, Direction.DOWN, fk2, byg6, bju.getFluidState(fk2), cog2.getType())) {
            this.spreadTo(bju, fk2, byg6, Direction.DOWN, cog2);
            if (this.sourceNeighborCount(bju, fk) >= 3) {
                this.spreadToSides(bju, fk, cog, byg5);
            }
        }
        else if (cog.isSource() || !this.isWaterHole(bju, cog2.getType(), fk, byg5, fk2, byg6)) {
            this.spreadToSides(bju, fk, cog, byg5);
        }
    }
    
    private void spreadToSides(final LevelAccessor bju, final BlockPos fk, final FluidState cog, final BlockState byg) {
        int integer6 = cog.getAmount() - this.getDropOff(bju);
        if (cog.<Boolean>getValue((Property<Boolean>)FlowingFluid.FALLING)) {
            integer6 = 7;
        }
        if (integer6 <= 0) {
            return;
        }
        final Map<Direction, FluidState> map7 = this.getSpread(bju, fk, byg);
        for (final Map.Entry<Direction, FluidState> entry9 : map7.entrySet()) {
            final Direction fp10 = entry9.getKey();
            final FluidState cog2 = entry9.getValue();
            final BlockPos fk2 = fk.relative(fp10);
            final BlockState byg2 = bju.getBlockState(fk2);
            if (this.canSpreadTo(bju, fk, byg, fp10, fk2, byg2, bju.getFluidState(fk2), cog2.getType())) {
                this.spreadTo(bju, fk2, byg2, fp10, cog2);
            }
        }
    }
    
    protected FluidState getNewLiquid(final LevelReader bjw, final BlockPos fk, final BlockState byg) {
        int integer5 = 0;
        int integer6 = 0;
        for (final Direction fp8 : Direction.Plane.HORIZONTAL) {
            final BlockPos fk2 = fk.relative(fp8);
            final BlockState byg2 = bjw.getBlockState(fk2);
            final FluidState cog11 = byg2.getFluidState();
            if (cog11.getType().isSame(this) && this.canPassThroughWall(fp8, bjw, fk, byg, fk2, byg2)) {
                if (cog11.isSource()) {
                    ++integer6;
                }
                integer5 = Math.max(integer5, cog11.getAmount());
            }
        }
        if (this.canConvertToSource() && integer6 >= 2) {
            final BlockState byg3 = bjw.getBlockState(fk.below());
            final FluidState cog12 = byg3.getFluidState();
            if (byg3.getMaterial().isSolid() || this.isSourceBlockOfThisType(cog12)) {
                return this.getSource(false);
            }
        }
        final BlockPos fk3 = fk.above();
        final BlockState byg4 = bjw.getBlockState(fk3);
        final FluidState cog13 = byg4.getFluidState();
        if (!cog13.isEmpty() && cog13.getType().isSame(this) && this.canPassThroughWall(Direction.UP, bjw, fk, byg, fk3, byg4)) {
            return this.getFlowing(8, true);
        }
        final int integer7 = integer5 - this.getDropOff(bjw);
        if (integer7 <= 0) {
            return Fluids.EMPTY.defaultFluidState();
        }
        return this.getFlowing(integer7, false);
    }
    
    private boolean canPassThroughWall(final Direction fp, final BlockGetter bjd, final BlockPos fk3, final BlockState byg4, final BlockPos fk5, final BlockState byg6) {
        Object2ByteLinkedOpenHashMap<Block.BlockStatePairKey> object2ByteLinkedOpenHashMap8;
        if (byg4.getBlock().hasDynamicShape() || byg6.getBlock().hasDynamicShape()) {
            object2ByteLinkedOpenHashMap8 = null;
        }
        else {
            object2ByteLinkedOpenHashMap8 = FlowingFluid.OCCLUSION_CACHE.get();
        }
        Block.BlockStatePairKey a9;
        if (object2ByteLinkedOpenHashMap8 != null) {
            a9 = new Block.BlockStatePairKey(byg4, byg6, fp);
            final byte byte10 = object2ByteLinkedOpenHashMap8.getAndMoveToFirst(a9);
            if (byte10 != 127) {
                return byte10 != 0;
            }
        }
        else {
            a9 = null;
        }
        final VoxelShape cwc10 = byg4.getCollisionShape(bjd, fk3);
        final VoxelShape cwc11 = byg6.getCollisionShape(bjd, fk5);
        final boolean boolean12 = !Shapes.mergedFaceOccludes(cwc10, cwc11, fp);
        if (object2ByteLinkedOpenHashMap8 != null) {
            if (object2ByteLinkedOpenHashMap8.size() == 200) {
                object2ByteLinkedOpenHashMap8.removeLastByte();
            }
            object2ByteLinkedOpenHashMap8.putAndMoveToFirst(a9, (byte)(byte)(boolean12 ? 1 : 0));
        }
        return boolean12;
    }
    
    public abstract Fluid getFlowing();
    
    public FluidState getFlowing(final int integer, final boolean boolean2) {
        return this.getFlowing().defaultFluidState().<Comparable, Integer>setValue((Property<Comparable>)FlowingFluid.LEVEL, integer).<Comparable, Boolean>setValue((Property<Comparable>)FlowingFluid.FALLING, boolean2);
    }
    
    public abstract Fluid getSource();
    
    public FluidState getSource(final boolean boolean1) {
        return this.getSource().defaultFluidState().<Comparable, Boolean>setValue((Property<Comparable>)FlowingFluid.FALLING, boolean1);
    }
    
    protected abstract boolean canConvertToSource();
    
    protected void spreadTo(final LevelAccessor bju, final BlockPos fk, final BlockState byg, final Direction fp, final FluidState cog) {
        if (byg.getBlock() instanceof LiquidBlockContainer) {
            ((LiquidBlockContainer)byg.getBlock()).placeLiquid(bju, fk, byg, cog);
        }
        else {
            if (!byg.isAir()) {
                this.beforeDestroyingBlock(bju, fk, byg);
            }
            bju.setBlock(fk, cog.createLegacyBlock(), 3);
        }
    }
    
    protected abstract void beforeDestroyingBlock(final LevelAccessor bju, final BlockPos fk, final BlockState byg);
    
    private static short getCacheKey(final BlockPos fk1, final BlockPos fk2) {
        final int integer3 = fk2.getX() - fk1.getX();
        final int integer4 = fk2.getZ() - fk1.getZ();
        return (short)((integer3 + 128 & 0xFF) << 8 | (integer4 + 128 & 0xFF));
    }
    
    protected int getSlopeDistance(final LevelReader bjw, final BlockPos fk2, final int integer, final Direction fp, final BlockState byg, final BlockPos fk6, final Short2ObjectMap<Pair<BlockState, FluidState>> short2ObjectMap, final Short2BooleanMap short2BooleanMap) {
        int integer2 = 1000;
        for (final Direction fp2 : Direction.Plane.HORIZONTAL) {
            if (fp2 == fp) {
                continue;
            }
            final BlockPos fk7 = fk2.relative(fp2);
            final short short14 = getCacheKey(fk6, fk7);
            final BlockState byg2;
            final Pair<BlockState, FluidState> pair15 = (Pair<BlockState, FluidState>)short2ObjectMap.computeIfAbsent(short14, integer -> {
                byg2 = bjw.getBlockState(fk7);
                return Pair.of(byg2, byg2.getFluidState());
            });
            final BlockState byg3 = (BlockState)pair15.getFirst();
            final FluidState cog17 = (FluidState)pair15.getSecond();
            if (!this.canPassThrough(bjw, this.getFlowing(), fk2, byg, fp2, fk7, byg3, cog17)) {
                continue;
            }
            final BlockPos fk9;
            final BlockPos fk8;
            final BlockState byg4;
            final BlockState byg5;
            final boolean boolean18 = short2BooleanMap.computeIfAbsent(short14, integer -> {
                fk8 = fk9.below();
                byg4 = bjw.getBlockState(fk8);
                return this.isWaterHole(bjw, this.getFlowing(), fk9, byg5, fk8, byg4);
            });
            if (boolean18) {
                return integer;
            }
            if (integer >= this.getSlopeFindDistance(bjw)) {
                continue;
            }
            final int integer3 = this.getSlopeDistance(bjw, fk7, integer + 1, fp2.getOpposite(), byg3, fk6, short2ObjectMap, short2BooleanMap);
            if (integer3 >= integer2) {
                continue;
            }
            integer2 = integer3;
        }
        return integer2;
    }
    
    private boolean isWaterHole(final BlockGetter bjd, final Fluid cof, final BlockPos fk3, final BlockState byg4, final BlockPos fk5, final BlockState byg6) {
        return this.canPassThroughWall(Direction.DOWN, bjd, fk3, byg4, fk5, byg6) && (byg6.getFluidState().getType().isSame(this) || this.canHoldFluid(bjd, fk5, byg6, cof));
    }
    
    private boolean canPassThrough(final BlockGetter bjd, final Fluid cof, final BlockPos fk3, final BlockState byg4, final Direction fp, final BlockPos fk6, final BlockState byg7, final FluidState cog) {
        return !this.isSourceBlockOfThisType(cog) && this.canPassThroughWall(fp, bjd, fk3, byg4, fk6, byg7) && this.canHoldFluid(bjd, fk6, byg7, cof);
    }
    
    private boolean isSourceBlockOfThisType(final FluidState cog) {
        return cog.getType().isSame(this) && cog.isSource();
    }
    
    protected abstract int getSlopeFindDistance(final LevelReader bjw);
    
    private int sourceNeighborCount(final LevelReader bjw, final BlockPos fk) {
        int integer4 = 0;
        for (final Direction fp6 : Direction.Plane.HORIZONTAL) {
            final BlockPos fk2 = fk.relative(fp6);
            final FluidState cog8 = bjw.getFluidState(fk2);
            if (this.isSourceBlockOfThisType(cog8)) {
                ++integer4;
            }
        }
        return integer4;
    }
    
    protected Map<Direction, FluidState> getSpread(final LevelReader bjw, final BlockPos fk, final BlockState byg) {
        int integer5 = 1000;
        final Map<Direction, FluidState> map6 = Maps.newEnumMap(Direction.class);
        final Short2ObjectMap<Pair<BlockState, FluidState>> short2ObjectMap7 = (Short2ObjectMap<Pair<BlockState, FluidState>>)new Short2ObjectOpenHashMap();
        final Short2BooleanMap short2BooleanMap8 = (Short2BooleanMap)new Short2BooleanOpenHashMap();
        for (final Direction fp10 : Direction.Plane.HORIZONTAL) {
            final BlockPos fk2 = fk.relative(fp10);
            final short short12 = getCacheKey(fk, fk2);
            final BlockState byg2;
            final Pair<BlockState, FluidState> pair13 = (Pair<BlockState, FluidState>)short2ObjectMap7.computeIfAbsent(short12, integer -> {
                byg2 = bjw.getBlockState(fk2);
                return Pair.of(byg2, byg2.getFluidState());
            });
            final BlockState byg3 = (BlockState)pair13.getFirst();
            final FluidState cog15 = (FluidState)pair13.getSecond();
            final FluidState cog16 = this.getNewLiquid(bjw, fk2, byg3);
            if (this.canPassThrough(bjw, cog16.getType(), fk, byg, fp10, fk2, byg3, cog15)) {
                final BlockPos fk3 = fk2.below();
                final BlockPos blockPos;
                final BlockState byg4;
                final BlockPos fk4;
                final BlockState byg5;
                final boolean boolean19 = short2BooleanMap8.computeIfAbsent(short12, integer -> {
                    byg4 = bjw.getBlockState(blockPos);
                    return this.isWaterHole(bjw, this.getFlowing(), fk4, byg5, blockPos, byg4);
                });
                int integer6;
                if (boolean19) {
                    integer6 = 0;
                }
                else {
                    integer6 = this.getSlopeDistance(bjw, fk2, 1, fp10.getOpposite(), byg3, fk, short2ObjectMap7, short2BooleanMap8);
                }
                if (integer6 < integer5) {
                    map6.clear();
                }
                if (integer6 > integer5) {
                    continue;
                }
                map6.put(fp10, cog16);
                integer5 = integer6;
            }
        }
        return map6;
    }
    
    private boolean canHoldFluid(final BlockGetter bjd, final BlockPos fk, final BlockState byg, final Fluid cof) {
        final Block bpe6 = byg.getBlock();
        if (bpe6 instanceof LiquidBlockContainer) {
            return ((LiquidBlockContainer)bpe6).canPlaceLiquid(bjd, fk, byg, cof);
        }
        if (bpe6 instanceof DoorBlock || bpe6.is(BlockTags.SIGNS) || bpe6 == Blocks.LADDER || bpe6 == Blocks.SUGAR_CANE || bpe6 == Blocks.BUBBLE_COLUMN) {
            return false;
        }
        final Material cok7 = byg.getMaterial();
        return cok7 != Material.PORTAL && cok7 != Material.STRUCTURAL_AIR && cok7 != Material.WATER_PLANT && cok7 != Material.REPLACEABLE_WATER_PLANT && !cok7.blocksMotion();
    }
    
    protected boolean canSpreadTo(final BlockGetter bjd, final BlockPos fk2, final BlockState byg3, final Direction fp, final BlockPos fk5, final BlockState byg6, final FluidState cog, final Fluid cof) {
        return cog.canBeReplacedWith(bjd, fk5, cof, fp) && this.canPassThroughWall(fp, bjd, fk2, byg3, fk5, byg6) && this.canHoldFluid(bjd, fk5, byg6, cof);
    }
    
    protected abstract int getDropOff(final LevelReader bjw);
    
    protected int getSpreadDelay(final Level bjt, final BlockPos fk, final FluidState cog3, final FluidState cog4) {
        return this.getTickDelay(bjt);
    }
    
    public void tick(final Level bjt, final BlockPos fk, FluidState cog) {
        if (!cog.isSource()) {
            final FluidState cog2 = this.getNewLiquid(bjt, fk, bjt.getBlockState(fk));
            final int integer6 = this.getSpreadDelay(bjt, fk, cog, cog2);
            if (cog2.isEmpty()) {
                cog = cog2;
                bjt.setBlock(fk, Blocks.AIR.defaultBlockState(), 3);
            }
            else if (!cog2.equals(cog)) {
                cog = cog2;
                final BlockState byg7 = cog.createLegacyBlock();
                bjt.setBlock(fk, byg7, 2);
                bjt.getLiquidTicks().scheduleTick(fk, cog.getType(), integer6);
                bjt.updateNeighborsAt(fk, byg7.getBlock());
            }
        }
        this.spread(bjt, fk, cog);
    }
    
    protected static int getLegacyLevel(final FluidState cog) {
        if (cog.isSource()) {
            return 0;
        }
        return 8 - Math.min(cog.getAmount(), 8) + (cog.<Boolean>getValue((Property<Boolean>)FlowingFluid.FALLING) ? 8 : 0);
    }
    
    private static boolean hasSameAbove(final FluidState cog, final BlockGetter bjd, final BlockPos fk) {
        return cog.getType().isSame(bjd.getFluidState(fk.above()).getType());
    }
    
    @Override
    public float getHeight(final FluidState cog, final BlockGetter bjd, final BlockPos fk) {
        if (hasSameAbove(cog, bjd, fk)) {
            return 1.0f;
        }
        return cog.getOwnHeight();
    }
    
    @Override
    public float getOwnHeight(final FluidState cog) {
        return cog.getAmount() / 9.0f;
    }
    
    @Override
    public VoxelShape getShape(final FluidState cog, final BlockGetter bjd, final BlockPos fk) {
        if (cog.getAmount() == 9 && hasSameAbove(cog, bjd, fk)) {
            return Shapes.block();
        }
        return this.shapes.computeIfAbsent(cog, cog -> Shapes.box(0.0, 0.0, 0.0, 1.0, cog.getHeight(bjd, fk), 1.0));
    }
    
    static {
        FALLING = BlockStateProperties.FALLING;
        LEVEL = BlockStateProperties.LEVEL_FLOWING;
        final Object2ByteLinkedOpenHashMap<Block.BlockStatePairKey> object2ByteLinkedOpenHashMap1;
        OCCLUSION_CACHE = ThreadLocal.<Object2ByteLinkedOpenHashMap<Block.BlockStatePairKey>>withInitial(() -> {
            object2ByteLinkedOpenHashMap1 = new Object2ByteLinkedOpenHashMap<Block.BlockStatePairKey>(200) {
                protected void rehash(final int integer) {
                }
            };
            object2ByteLinkedOpenHashMap1.defaultReturnValue((byte)127);
            return object2ByteLinkedOpenHashMap1;
        });
    }
}
