package net.minecraft.world.level.material;

import net.minecraft.world.phys.shapes.VoxelShape;
import net.minecraft.tags.Tag;
import net.minecraft.world.level.block.state.BlockState;
import net.minecraft.world.level.LevelReader;
import net.minecraft.world.phys.Vec3;
import net.minecraft.core.Direction;
import net.minecraft.world.level.BlockGetter;
import javax.annotation.Nullable;
import net.minecraft.core.particles.ParticleOptions;
import java.util.Random;
import net.minecraft.core.BlockPos;
import net.minecraft.world.level.Level;
import net.minecraft.world.item.Item;
import net.minecraft.world.level.block.state.StateDefinition;
import net.minecraft.core.IdMapper;

public abstract class Fluid {
    public static final IdMapper<FluidState> FLUID_STATE_REGISTRY;
    protected final StateDefinition<Fluid, FluidState> stateDefinition;
    private FluidState defaultFluidState;
    
    protected Fluid() {
        final StateDefinition.Builder<Fluid, FluidState> a2 = new StateDefinition.Builder<Fluid, FluidState>(this);
        this.createFluidStateDefinition(a2);
        this.stateDefinition = a2.<FluidStateImpl>create(FluidStateImpl::new);
        this.registerDefaultState(this.stateDefinition.any());
    }
    
    protected void createFluidStateDefinition(final StateDefinition.Builder<Fluid, FluidState> a) {
    }
    
    public StateDefinition<Fluid, FluidState> getStateDefinition() {
        return this.stateDefinition;
    }
    
    protected final void registerDefaultState(final FluidState cog) {
        this.defaultFluidState = cog;
    }
    
    public final FluidState defaultFluidState() {
        return this.defaultFluidState;
    }
    
    public abstract Item getBucket();
    
    protected void animateTick(final Level bjt, final BlockPos fk, final FluidState cog, final Random random) {
    }
    
    protected void tick(final Level bjt, final BlockPos fk, final FluidState cog) {
    }
    
    protected void randomTick(final Level bjt, final BlockPos fk, final FluidState cog, final Random random) {
    }
    
    @Nullable
    protected ParticleOptions getDripParticle() {
        return null;
    }
    
    protected abstract boolean canBeReplacedWith(final FluidState cog, final BlockGetter bjd, final BlockPos fk, final Fluid cof, final Direction fp);
    
    protected abstract Vec3 getFlow(final BlockGetter bjd, final BlockPos fk, final FluidState cog);
    
    public abstract int getTickDelay(final LevelReader bjw);
    
    protected boolean isRandomlyTicking() {
        return false;
    }
    
    protected boolean isEmpty() {
        return false;
    }
    
    protected abstract float getExplosionResistance();
    
    public abstract float getHeight(final FluidState cog, final BlockGetter bjd, final BlockPos fk);
    
    public abstract float getOwnHeight(final FluidState cog);
    
    protected abstract BlockState createLegacyBlock(final FluidState cog);
    
    public abstract boolean isSource(final FluidState cog);
    
    public abstract int getAmount(final FluidState cog);
    
    public boolean isSame(final Fluid cof) {
        return cof == this;
    }
    
    public boolean is(final Tag<Fluid> aaz) {
        return aaz.contains(this);
    }
    
    public abstract VoxelShape getShape(final FluidState cog, final BlockGetter bjd, final BlockPos fk);
    
    static {
        FLUID_STATE_REGISTRY = new IdMapper<FluidState>();
    }
}
