package net.minecraft.world.level.material;

import net.minecraft.world.level.block.state.AbstractStateHolder;
import net.minecraft.world.level.block.state.StateDefinition;
import net.minecraft.tags.FluidTags;
import net.minecraft.world.level.block.state.properties.Property;
import net.minecraft.world.level.block.LiquidBlock;
import net.minecraft.world.level.LevelAccessor;
import javax.annotation.Nullable;
import net.minecraft.core.Direction;
import net.minecraft.world.level.block.state.BlockState;
import net.minecraft.world.level.block.Blocks;
import net.minecraft.world.level.LevelReader;
import net.minecraft.world.level.GameRules;
import net.minecraft.sounds.SoundSource;
import net.minecraft.sounds.SoundEvents;
import net.minecraft.core.particles.ParticleOptions;
import net.minecraft.core.particles.ParticleTypes;
import net.minecraft.world.level.BlockGetter;
import java.util.Random;
import net.minecraft.core.BlockPos;
import net.minecraft.world.level.Level;
import net.minecraft.world.item.Items;
import net.minecraft.world.item.Item;

public abstract class LavaFluid extends FlowingFluid {
    @Override
    public Fluid getFlowing() {
        return Fluids.FLOWING_LAVA;
    }
    
    @Override
    public Fluid getSource() {
        return Fluids.LAVA;
    }
    
    @Override
    public Item getBucket() {
        return Items.LAVA_BUCKET;
    }
    
    public void animateTick(final Level bjt, final BlockPos fk, final FluidState cog, final Random random) {
        final BlockPos fk2 = fk.above();
        if (bjt.getBlockState(fk2).isAir() && !bjt.getBlockState(fk2).isSolidRender(bjt, fk2)) {
            if (random.nextInt(100) == 0) {
                final double double7 = fk.getX() + random.nextFloat();
                final double double8 = fk.getY() + 1;
                final double double9 = fk.getZ() + random.nextFloat();
                bjt.addParticle(ParticleTypes.LAVA, double7, double8, double9, 0.0, 0.0, 0.0);
                bjt.playLocalSound(double7, double8, double9, SoundEvents.LAVA_POP, SoundSource.BLOCKS, 0.2f + random.nextFloat() * 0.2f, 0.9f + random.nextFloat() * 0.15f, false);
            }
            if (random.nextInt(200) == 0) {
                bjt.playLocalSound(fk.getX(), fk.getY(), fk.getZ(), SoundEvents.LAVA_AMBIENT, SoundSource.BLOCKS, 0.2f + random.nextFloat() * 0.2f, 0.9f + random.nextFloat() * 0.15f, false);
            }
        }
    }
    
    public void randomTick(final Level bjt, final BlockPos fk, final FluidState cog, final Random random) {
        if (!bjt.getGameRules().getBoolean(GameRules.RULE_DOFIRETICK)) {
            return;
        }
        final int integer6 = random.nextInt(3);
        if (integer6 > 0) {
            BlockPos fk2 = fk;
            for (int integer7 = 0; integer7 < integer6; ++integer7) {
                fk2 = fk2.offset(random.nextInt(3) - 1, 1, random.nextInt(3) - 1);
                if (!bjt.isLoaded(fk2)) {
                    return;
                }
                final BlockState byg9 = bjt.getBlockState(fk2);
                if (byg9.isAir()) {
                    if (this.hasFlammableNeighbours(bjt, fk2)) {
                        bjt.setBlockAndUpdate(fk2, Blocks.FIRE.defaultBlockState());
                        return;
                    }
                }
                else if (byg9.getMaterial().blocksMotion()) {
                    return;
                }
            }
        }
        else {
            for (int integer8 = 0; integer8 < 3; ++integer8) {
                final BlockPos fk3 = fk.offset(random.nextInt(3) - 1, 0, random.nextInt(3) - 1);
                if (!bjt.isLoaded(fk3)) {
                    return;
                }
                if (bjt.isEmptyBlock(fk3.above()) && this.isFlammable(bjt, fk3)) {
                    bjt.setBlockAndUpdate(fk3.above(), Blocks.FIRE.defaultBlockState());
                }
            }
        }
    }
    
    private boolean hasFlammableNeighbours(final LevelReader bjw, final BlockPos fk) {
        for (final Direction fp7 : Direction.values()) {
            if (this.isFlammable(bjw, fk.relative(fp7))) {
                return true;
            }
        }
        return false;
    }
    
    private boolean isFlammable(final LevelReader bjw, final BlockPos fk) {
        return (fk.getY() < 0 || fk.getY() >= 256 || bjw.hasChunkAt(fk)) && bjw.getBlockState(fk).getMaterial().isFlammable();
    }
    
    @Nullable
    public ParticleOptions getDripParticle() {
        return ParticleTypes.DRIPPING_LAVA;
    }
    
    @Override
    protected void beforeDestroyingBlock(final LevelAccessor bju, final BlockPos fk, final BlockState byg) {
        this.fizz(bju, fk);
    }
    
    public int getSlopeFindDistance(final LevelReader bjw) {
        return bjw.getDimension().isUltraWarm() ? 4 : 2;
    }
    
    public BlockState createLegacyBlock(final FluidState cog) {
        return ((AbstractStateHolder<O, BlockState>)Blocks.LAVA.defaultBlockState()).<Comparable, Integer>setValue((Property<Comparable>)LiquidBlock.LEVEL, FlowingFluid.getLegacyLevel(cog));
    }
    
    @Override
    public boolean isSame(final Fluid cof) {
        return cof == Fluids.LAVA || cof == Fluids.FLOWING_LAVA;
    }
    
    public int getDropOff(final LevelReader bjw) {
        return bjw.getDimension().isUltraWarm() ? 1 : 2;
    }
    
    public boolean canBeReplacedWith(final FluidState cog, final BlockGetter bjd, final BlockPos fk, final Fluid cof, final Direction fp) {
        return cog.getHeight(bjd, fk) >= 0.44444445f && cof.is(FluidTags.WATER);
    }
    
    @Override
    public int getTickDelay(final LevelReader bjw) {
        return bjw.getDimension().isHasCeiling() ? 10 : 30;
    }
    
    public int getSpreadDelay(final Level bjt, final BlockPos fk, final FluidState cog3, final FluidState cog4) {
        int integer6 = this.getTickDelay(bjt);
        if (!cog3.isEmpty() && !cog4.isEmpty() && !cog3.<Boolean>getValue((Property<Boolean>)LavaFluid.FALLING) && !cog4.<Boolean>getValue((Property<Boolean>)LavaFluid.FALLING) && cog4.getHeight(bjt, fk) > cog3.getHeight(bjt, fk) && bjt.getRandom().nextInt(4) != 0) {
            integer6 *= 4;
        }
        return integer6;
    }
    
    private void fizz(final LevelAccessor bju, final BlockPos fk) {
        bju.levelEvent(1501, fk, 0);
    }
    
    @Override
    protected boolean canConvertToSource() {
        return false;
    }
    
    @Override
    protected void spreadTo(final LevelAccessor bju, final BlockPos fk, final BlockState byg, final Direction fp, final FluidState cog) {
        if (fp == Direction.DOWN) {
            final FluidState cog2 = bju.getFluidState(fk);
            if (this.is(FluidTags.LAVA) && cog2.is(FluidTags.WATER)) {
                if (byg.getBlock() instanceof LiquidBlock) {
                    bju.setBlock(fk, Blocks.STONE.defaultBlockState(), 3);
                }
                this.fizz(bju, fk);
                return;
            }
        }
        super.spreadTo(bju, fk, byg, fp, cog);
    }
    
    @Override
    protected boolean isRandomlyTicking() {
        return true;
    }
    
    @Override
    protected float getExplosionResistance() {
        return 100.0f;
    }
    
    public static class Source extends LavaFluid {
        @Override
        public int getAmount(final FluidState cog) {
            return 8;
        }
        
        @Override
        public boolean isSource(final FluidState cog) {
            return true;
        }
    }
    
    public static class Flowing extends LavaFluid {
        @Override
        protected void createFluidStateDefinition(final StateDefinition.Builder<Fluid, FluidState> a) {
            super.createFluidStateDefinition(a);
            a.add(Flowing.LEVEL);
        }
        
        @Override
        public int getAmount(final FluidState cog) {
            return cog.<Integer>getValue((Property<Integer>)Flowing.LEVEL);
        }
        
        @Override
        public boolean isSource(final FluidState cog) {
            return false;
        }
    }
}
