package net.minecraft.world.level.newbiome.layer;

import net.minecraft.world.level.biome.Biomes;
import net.minecraft.core.Registry;
import net.minecraft.world.level.biome.Biome;
import net.minecraft.world.level.newbiome.area.LazyArea;
import net.minecraft.world.level.newbiome.context.LazyAreaContext;
import net.minecraft.world.level.levelgen.OverworldGeneratorSettings;
import net.minecraft.world.level.LevelType;
import net.minecraft.world.level.newbiome.context.BigContext;
import net.minecraft.world.level.newbiome.area.Area;
import java.util.function.LongFunction;
import net.minecraft.world.level.newbiome.area.AreaFactory;
import net.minecraft.world.level.newbiome.layer.traits.AreaTransformer1;

public class Layers {
    protected static final int WARM_OCEAN;
    protected static final int LUKEWARM_OCEAN;
    protected static final int OCEAN;
    protected static final int COLD_OCEAN;
    protected static final int FROZEN_OCEAN;
    protected static final int DEEP_WARM_OCEAN;
    protected static final int DEEP_LUKEWARM_OCEAN;
    protected static final int DEEP_OCEAN;
    protected static final int DEEP_COLD_OCEAN;
    protected static final int DEEP_FROZEN_OCEAN;
    
    private static <T extends Area, C extends BigContext<T>> AreaFactory<T> zoom(final long long1, final AreaTransformer1 cpw, final AreaFactory<T> cor, final int integer, final LongFunction<C> longFunction) {
        AreaFactory<T> cor2 = cor;
        for (int integer2 = 0; integer2 < integer; ++integer2) {
            cor2 = cpw.<T>run(longFunction.apply(long1 + integer2), cor2);
        }
        return cor2;
    }
    
    public static <T extends Area, C extends BigContext<T>> AreaFactory<T> getDefaultLayer(final LevelType bka, final OverworldGeneratorSettings cbw, final LongFunction<C> longFunction) {
        AreaFactory<T> cor4 = IslandLayer.INSTANCE.<T>run((BigContext<T>)longFunction.apply(1L));
        cor4 = ZoomLayer.FUZZY.<T>run(longFunction.apply(2000L), cor4);
        cor4 = AddIslandLayer.INSTANCE.<T>run(longFunction.apply(1L), cor4);
        cor4 = ZoomLayer.NORMAL.<T>run(longFunction.apply(2001L), cor4);
        cor4 = AddIslandLayer.INSTANCE.<T>run(longFunction.apply(2L), cor4);
        cor4 = AddIslandLayer.INSTANCE.<T>run(longFunction.apply(50L), cor4);
        cor4 = AddIslandLayer.INSTANCE.<T>run(longFunction.apply(70L), cor4);
        cor4 = RemoveTooMuchOceanLayer.INSTANCE.<T>run(longFunction.apply(2L), cor4);
        AreaFactory<T> cor5 = OceanLayer.INSTANCE.<T>run((BigContext<T>)longFunction.apply(2L));
        cor5 = Layers.<T, C>zoom(2001L, ZoomLayer.NORMAL, cor5, 6, longFunction);
        cor4 = AddSnowLayer.INSTANCE.<T>run(longFunction.apply(2L), cor4);
        cor4 = AddIslandLayer.INSTANCE.<T>run(longFunction.apply(3L), cor4);
        cor4 = AddEdgeLayer.CoolWarm.INSTANCE.<T>run(longFunction.apply(2L), cor4);
        cor4 = AddEdgeLayer.HeatIce.INSTANCE.<T>run(longFunction.apply(2L), cor4);
        cor4 = AddEdgeLayer.IntroduceSpecial.INSTANCE.<T>run(longFunction.apply(3L), cor4);
        cor4 = ZoomLayer.NORMAL.<T>run(longFunction.apply(2002L), cor4);
        cor4 = ZoomLayer.NORMAL.<T>run(longFunction.apply(2003L), cor4);
        cor4 = AddIslandLayer.INSTANCE.<T>run(longFunction.apply(4L), cor4);
        cor4 = AddMushroomIslandLayer.INSTANCE.<T>run(longFunction.apply(5L), cor4);
        cor4 = AddDeepOceanLayer.INSTANCE.<T>run(longFunction.apply(4L), cor4);
        cor4 = Layers.<T, C>zoom(1000L, ZoomLayer.NORMAL, cor4, 0, longFunction);
        final int integer6 = (bka == LevelType.LARGE_BIOMES) ? 6 : cbw.getBiomeSize();
        final int integer7 = cbw.getRiverSize();
        AreaFactory<T> cor6 = cor4;
        cor6 = Layers.<T, C>zoom(1000L, ZoomLayer.NORMAL, cor6, 0, longFunction);
        cor6 = RiverInitLayer.INSTANCE.<T>run(longFunction.apply(100L), cor6);
        AreaFactory<T> cor7 = cor4;
        cor7 = new BiomeInitLayer(bka, cbw.getFixedBiome()).<T>run(longFunction.apply(200L), cor7);
        cor7 = RareBiomeLargeLayer.INSTANCE.<T>run(longFunction.apply(1001L), cor7);
        cor7 = Layers.<T, C>zoom(1000L, ZoomLayer.NORMAL, cor7, 2, longFunction);
        cor7 = BiomeEdgeLayer.INSTANCE.<T>run(longFunction.apply(1000L), cor7);
        AreaFactory<T> cor8 = cor6;
        cor8 = Layers.<T, C>zoom(1000L, ZoomLayer.NORMAL, cor8, 2, longFunction);
        cor7 = RegionHillsLayer.INSTANCE.<T>run(longFunction.apply(1000L), cor7, cor8);
        cor6 = Layers.<T, C>zoom(1000L, ZoomLayer.NORMAL, cor6, 2, longFunction);
        cor6 = Layers.<T, C>zoom(1000L, ZoomLayer.NORMAL, cor6, integer7, longFunction);
        cor6 = RiverLayer.INSTANCE.<T>run(longFunction.apply(1L), cor6);
        cor6 = SmoothLayer.INSTANCE.<T>run(longFunction.apply(1000L), cor6);
        cor7 = RareBiomeSpotLayer.INSTANCE.<T>run(longFunction.apply(1001L), cor7);
        for (int integer8 = 0; integer8 < integer6; ++integer8) {
            cor7 = ZoomLayer.NORMAL.<T>run(longFunction.apply(1000 + integer8), cor7);
            if (integer8 == 0) {
                cor7 = AddIslandLayer.INSTANCE.<T>run(longFunction.apply(3L), cor7);
            }
            if (integer8 == 1 || integer6 == 1) {
                cor7 = ShoreLayer.INSTANCE.<T>run(longFunction.apply(1000L), cor7);
            }
        }
        cor7 = SmoothLayer.INSTANCE.<T>run(longFunction.apply(1000L), cor7);
        cor7 = RiverMixerLayer.INSTANCE.<T>run(longFunction.apply(100L), cor7, cor6);
        cor7 = OceanMixerLayer.INSTANCE.<T>run(longFunction.apply(100L), cor7, cor5);
        return cor7;
    }
    
    public static Layer getDefaultLayer(final long long1, final LevelType bka, final OverworldGeneratorSettings cbw) {
        final int integer5 = 25;
        final AreaFactory<LazyArea> cor6 = Layers.<LazyArea, LazyAreaContext>getDefaultLayer(bka, cbw, long2 -> new LazyAreaContext(25, long1, long2));
        return new Layer(cor6);
    }
    
    public static boolean isSame(final int integer1, final int integer2) {
        if (integer1 == integer2) {
            return true;
        }
        final Biome bkq3 = Registry.BIOME.byId(integer1);
        final Biome bkq4 = Registry.BIOME.byId(integer2);
        if (bkq3 == null || bkq4 == null) {
            return false;
        }
        if (bkq3 == Biomes.WOODED_BADLANDS_PLATEAU || bkq3 == Biomes.BADLANDS_PLATEAU) {
            return bkq4 == Biomes.WOODED_BADLANDS_PLATEAU || bkq4 == Biomes.BADLANDS_PLATEAU;
        }
        return (bkq3.getBiomeCategory() != Biome.BiomeCategory.NONE && bkq4.getBiomeCategory() != Biome.BiomeCategory.NONE && bkq3.getBiomeCategory() == bkq4.getBiomeCategory()) || bkq3 == bkq4;
    }
    
    protected static boolean isOcean(final int integer) {
        return integer == Layers.WARM_OCEAN || integer == Layers.LUKEWARM_OCEAN || integer == Layers.OCEAN || integer == Layers.COLD_OCEAN || integer == Layers.FROZEN_OCEAN || integer == Layers.DEEP_WARM_OCEAN || integer == Layers.DEEP_LUKEWARM_OCEAN || integer == Layers.DEEP_OCEAN || integer == Layers.DEEP_COLD_OCEAN || integer == Layers.DEEP_FROZEN_OCEAN;
    }
    
    protected static boolean isShallowOcean(final int integer) {
        return integer == Layers.WARM_OCEAN || integer == Layers.LUKEWARM_OCEAN || integer == Layers.OCEAN || integer == Layers.COLD_OCEAN || integer == Layers.FROZEN_OCEAN;
    }
    
    static {
        WARM_OCEAN = Registry.BIOME.getId(Biomes.WARM_OCEAN);
        LUKEWARM_OCEAN = Registry.BIOME.getId(Biomes.LUKEWARM_OCEAN);
        OCEAN = Registry.BIOME.getId(Biomes.OCEAN);
        COLD_OCEAN = Registry.BIOME.getId(Biomes.COLD_OCEAN);
        FROZEN_OCEAN = Registry.BIOME.getId(Biomes.FROZEN_OCEAN);
        DEEP_WARM_OCEAN = Registry.BIOME.getId(Biomes.DEEP_WARM_OCEAN);
        DEEP_LUKEWARM_OCEAN = Registry.BIOME.getId(Biomes.DEEP_LUKEWARM_OCEAN);
        DEEP_OCEAN = Registry.BIOME.getId(Biomes.DEEP_OCEAN);
        DEEP_COLD_OCEAN = Registry.BIOME.getId(Biomes.DEEP_COLD_OCEAN);
        DEEP_FROZEN_OCEAN = Registry.BIOME.getId(Biomes.DEEP_FROZEN_OCEAN);
    }
}
