package net.minecraft.world.level.pathfinder;

import java.util.EnumSet;
import net.minecraft.world.level.BlockGetter;
import javax.annotation.Nullable;
import java.util.Iterator;
import java.util.Set;
import net.minecraft.world.level.block.Block;
import com.google.common.collect.Sets;
import net.minecraft.world.entity.Entity;
import net.minecraft.world.level.block.Blocks;
import net.minecraft.core.BlockPos;
import net.minecraft.util.Mth;
import net.minecraft.world.entity.Mob;
import net.minecraft.world.level.PathNavigationRegion;

public class FlyNodeEvaluator extends WalkNodeEvaluator {
    @Override
    public void prepare(final PathNavigationRegion bke, final Mob akx) {
        super.prepare(bke, akx);
        this.oldWaterCost = akx.getPathfindingMalus(BlockPathTypes.WATER);
    }
    
    @Override
    public void done() {
        this.mob.setPathfindingMalus(BlockPathTypes.WATER, this.oldWaterCost);
        super.done();
    }
    
    @Override
    public Node getStart() {
        int integer2;
        if (this.canFloat() && this.mob.isInWater()) {
            integer2 = Mth.floor(this.mob.getY());
            final BlockPos.MutableBlockPos a3 = new BlockPos.MutableBlockPos(this.mob.getX(), integer2, this.mob.getZ());
            for (Block bpe4 = this.level.getBlockState(a3).getBlock(); bpe4 == Blocks.WATER; bpe4 = this.level.getBlockState(a3).getBlock()) {
                ++integer2;
                a3.set(this.mob.getX(), integer2, this.mob.getZ());
            }
        }
        else {
            integer2 = Mth.floor(this.mob.getY() + 0.5);
        }
        final BlockPos fk3 = new BlockPos(this.mob);
        final BlockPathTypes cqj4 = this.getBlockPathType(this.mob, fk3.getX(), integer2, fk3.getZ());
        if (this.mob.getPathfindingMalus(cqj4) < 0.0f) {
            final Set<BlockPos> set5 = Sets.newHashSet();
            set5.add(new BlockPos(this.mob.getBoundingBox().minX, integer2, this.mob.getBoundingBox().minZ));
            set5.add(new BlockPos(this.mob.getBoundingBox().minX, integer2, this.mob.getBoundingBox().maxZ));
            set5.add(new BlockPos(this.mob.getBoundingBox().maxX, integer2, this.mob.getBoundingBox().minZ));
            set5.add(new BlockPos(this.mob.getBoundingBox().maxX, integer2, this.mob.getBoundingBox().maxZ));
            for (final BlockPos fk4 : set5) {
                final BlockPathTypes cqj5 = this.getBlockPathType(this.mob, fk4);
                if (this.mob.getPathfindingMalus(cqj5) >= 0.0f) {
                    return super.getNode(fk4.getX(), fk4.getY(), fk4.getZ());
                }
            }
        }
        return super.getNode(fk3.getX(), integer2, fk3.getZ());
    }
    
    @Override
    public Target getGoal(final double double1, final double double2, final double double3) {
        return new Target(super.getNode(Mth.floor(double1), Mth.floor(double2), Mth.floor(double3)));
    }
    
    @Override
    public int getNeighbors(final Node[] arr, final Node cql) {
        int integer4 = 0;
        final Node cql2 = this.getNode(cql.x, cql.y, cql.z + 1);
        if (this.isOpen(cql2)) {
            arr[integer4++] = cql2;
        }
        final Node cql3 = this.getNode(cql.x - 1, cql.y, cql.z);
        if (this.isOpen(cql3)) {
            arr[integer4++] = cql3;
        }
        final Node cql4 = this.getNode(cql.x + 1, cql.y, cql.z);
        if (this.isOpen(cql4)) {
            arr[integer4++] = cql4;
        }
        final Node cql5 = this.getNode(cql.x, cql.y, cql.z - 1);
        if (this.isOpen(cql5)) {
            arr[integer4++] = cql5;
        }
        final Node cql6 = this.getNode(cql.x, cql.y + 1, cql.z);
        if (this.isOpen(cql6)) {
            arr[integer4++] = cql6;
        }
        final Node cql7 = this.getNode(cql.x, cql.y - 1, cql.z);
        if (this.isOpen(cql7)) {
            arr[integer4++] = cql7;
        }
        final Node cql8 = this.getNode(cql.x, cql.y + 1, cql.z + 1);
        if (this.isOpen(cql8) && this.hasMalus(cql2) && this.hasMalus(cql6)) {
            arr[integer4++] = cql8;
        }
        final Node cql9 = this.getNode(cql.x - 1, cql.y + 1, cql.z);
        if (this.isOpen(cql9) && this.hasMalus(cql3) && this.hasMalus(cql6)) {
            arr[integer4++] = cql9;
        }
        final Node cql10 = this.getNode(cql.x + 1, cql.y + 1, cql.z);
        if (this.isOpen(cql10) && this.hasMalus(cql4) && this.hasMalus(cql6)) {
            arr[integer4++] = cql10;
        }
        final Node cql11 = this.getNode(cql.x, cql.y + 1, cql.z - 1);
        if (this.isOpen(cql11) && this.hasMalus(cql5) && this.hasMalus(cql6)) {
            arr[integer4++] = cql11;
        }
        final Node cql12 = this.getNode(cql.x, cql.y - 1, cql.z + 1);
        if (this.isOpen(cql12) && this.hasMalus(cql2) && this.hasMalus(cql7)) {
            arr[integer4++] = cql12;
        }
        final Node cql13 = this.getNode(cql.x - 1, cql.y - 1, cql.z);
        if (this.isOpen(cql13) && this.hasMalus(cql3) && this.hasMalus(cql7)) {
            arr[integer4++] = cql13;
        }
        final Node cql14 = this.getNode(cql.x + 1, cql.y - 1, cql.z);
        if (this.isOpen(cql14) && this.hasMalus(cql4) && this.hasMalus(cql7)) {
            arr[integer4++] = cql14;
        }
        final Node cql15 = this.getNode(cql.x, cql.y - 1, cql.z - 1);
        if (this.isOpen(cql15) && this.hasMalus(cql5) && this.hasMalus(cql7)) {
            arr[integer4++] = cql15;
        }
        final Node cql16 = this.getNode(cql.x + 1, cql.y, cql.z - 1);
        if (this.isOpen(cql16) && this.hasMalus(cql5) && this.hasMalus(cql4)) {
            arr[integer4++] = cql16;
        }
        final Node cql17 = this.getNode(cql.x + 1, cql.y, cql.z + 1);
        if (this.isOpen(cql17) && this.hasMalus(cql2) && this.hasMalus(cql4)) {
            arr[integer4++] = cql17;
        }
        final Node cql18 = this.getNode(cql.x - 1, cql.y, cql.z - 1);
        if (this.isOpen(cql18) && this.hasMalus(cql5) && this.hasMalus(cql3)) {
            arr[integer4++] = cql18;
        }
        final Node cql19 = this.getNode(cql.x - 1, cql.y, cql.z + 1);
        if (this.isOpen(cql19) && this.hasMalus(cql2) && this.hasMalus(cql3)) {
            arr[integer4++] = cql19;
        }
        final Node cql20 = this.getNode(cql.x + 1, cql.y + 1, cql.z - 1);
        if (this.isOpen(cql20) && this.hasMalus(cql16) && this.hasMalus(cql11) && this.hasMalus(cql10)) {
            arr[integer4++] = cql20;
        }
        final Node cql21 = this.getNode(cql.x + 1, cql.y + 1, cql.z + 1);
        if (this.isOpen(cql21) && this.hasMalus(cql17) && this.hasMalus(cql8) && this.hasMalus(cql10)) {
            arr[integer4++] = cql21;
        }
        final Node cql22 = this.getNode(cql.x - 1, cql.y + 1, cql.z - 1);
        if (this.isOpen(cql22) && this.hasMalus(cql18) && this.hasMalus(cql11) && this.hasMalus(cql9)) {
            arr[integer4++] = cql22;
        }
        final Node cql23 = this.getNode(cql.x - 1, cql.y + 1, cql.z + 1);
        if (this.isOpen(cql23) && this.hasMalus(cql19) && this.hasMalus(cql8) && this.hasMalus(cql9)) {
            arr[integer4++] = cql23;
        }
        final Node cql24 = this.getNode(cql.x + 1, cql.y - 1, cql.z - 1);
        if (this.isOpen(cql24) && this.hasMalus(cql16) && this.hasMalus(cql15) && this.hasMalus(cql14)) {
            arr[integer4++] = cql24;
        }
        final Node cql25 = this.getNode(cql.x + 1, cql.y - 1, cql.z + 1);
        if (this.isOpen(cql25) && this.hasMalus(cql17) && this.hasMalus(cql12) && this.hasMalus(cql14)) {
            arr[integer4++] = cql25;
        }
        final Node cql26 = this.getNode(cql.x - 1, cql.y - 1, cql.z - 1);
        if (this.isOpen(cql26) && this.hasMalus(cql18) && this.hasMalus(cql15) && this.hasMalus(cql13)) {
            arr[integer4++] = cql26;
        }
        final Node cql27 = this.getNode(cql.x - 1, cql.y - 1, cql.z + 1);
        if (this.isOpen(cql27) && this.hasMalus(cql19) && this.hasMalus(cql12) && this.hasMalus(cql13)) {
            arr[integer4++] = cql27;
        }
        return integer4;
    }
    
    private boolean hasMalus(@Nullable final Node cql) {
        return cql != null && cql.costMalus >= 0.0f;
    }
    
    private boolean isOpen(@Nullable final Node cql) {
        return cql != null && !cql.closed;
    }
    
    @Nullable
    @Override
    protected Node getNode(final int integer1, final int integer2, final int integer3) {
        Node cql5 = null;
        final BlockPathTypes cqj6 = this.getBlockPathType(this.mob, integer1, integer2, integer3);
        final float float7 = this.mob.getPathfindingMalus(cqj6);
        if (float7 >= 0.0f) {
            cql5 = super.getNode(integer1, integer2, integer3);
            cql5.type = cqj6;
            cql5.costMalus = Math.max(cql5.costMalus, float7);
            if (cqj6 == BlockPathTypes.WALKABLE) {
                final Node node = cql5;
                ++node.costMalus;
            }
        }
        if (cqj6 == BlockPathTypes.OPEN || cqj6 == BlockPathTypes.WALKABLE) {
            return cql5;
        }
        return cql5;
    }
    
    @Override
    public BlockPathTypes getBlockPathType(final BlockGetter bjd, final int integer2, final int integer3, final int integer4, final Mob akx, final int integer6, final int integer7, final int integer8, final boolean boolean9, final boolean boolean10) {
        final EnumSet<BlockPathTypes> enumSet12 = EnumSet.<BlockPathTypes>noneOf(BlockPathTypes.class);
        BlockPathTypes cqj13 = BlockPathTypes.BLOCKED;
        final BlockPos fk14 = new BlockPos(akx);
        cqj13 = this.getBlockPathTypes(bjd, integer2, integer3, integer4, integer6, integer7, integer8, boolean9, boolean10, enumSet12, cqj13, fk14);
        if (enumSet12.contains(BlockPathTypes.FENCE)) {
            return BlockPathTypes.FENCE;
        }
        BlockPathTypes cqj14 = BlockPathTypes.BLOCKED;
        for (final BlockPathTypes cqj15 : enumSet12) {
            if (akx.getPathfindingMalus(cqj15) < 0.0f) {
                return cqj15;
            }
            if (akx.getPathfindingMalus(cqj15) < akx.getPathfindingMalus(cqj14)) {
                continue;
            }
            cqj14 = cqj15;
        }
        if (cqj13 == BlockPathTypes.OPEN && akx.getPathfindingMalus(cqj14) == 0.0f) {
            return BlockPathTypes.OPEN;
        }
        return cqj14;
    }
    
    @Override
    public BlockPathTypes getBlockPathType(final BlockGetter bjd, final int integer2, final int integer3, final int integer4) {
        BlockPathTypes cqj6 = WalkNodeEvaluator.getBlockPathTypeRaw(bjd, integer2, integer3, integer4);
        if (cqj6 == BlockPathTypes.OPEN && integer3 >= 1) {
            final Block bpe7 = bjd.getBlockState(new BlockPos(integer2, integer3 - 1, integer4)).getBlock();
            final BlockPathTypes cqj7 = WalkNodeEvaluator.getBlockPathTypeRaw(bjd, integer2, integer3 - 1, integer4);
            if (cqj7 == BlockPathTypes.DAMAGE_FIRE || bpe7 == Blocks.MAGMA_BLOCK || cqj7 == BlockPathTypes.LAVA || bpe7 == Blocks.CAMPFIRE) {
                cqj6 = BlockPathTypes.DAMAGE_FIRE;
            }
            else if (cqj7 == BlockPathTypes.DAMAGE_CACTUS) {
                cqj6 = BlockPathTypes.DAMAGE_CACTUS;
            }
            else if (cqj7 == BlockPathTypes.DAMAGE_OTHER) {
                cqj6 = BlockPathTypes.DAMAGE_OTHER;
            }
            else if (cqj7 == BlockPathTypes.COCOA) {
                cqj6 = BlockPathTypes.COCOA;
            }
            else if (cqj7 == BlockPathTypes.FENCE) {
                cqj6 = BlockPathTypes.FENCE;
            }
            else {
                cqj6 = ((cqj7 == BlockPathTypes.WALKABLE || cqj7 == BlockPathTypes.OPEN || cqj7 == BlockPathTypes.WATER) ? BlockPathTypes.OPEN : BlockPathTypes.WALKABLE);
            }
        }
        if (cqj6 == BlockPathTypes.WALKABLE || cqj6 == BlockPathTypes.OPEN) {
            cqj6 = WalkNodeEvaluator.checkNeighbourBlocks(bjd, integer2, integer3, integer4, cqj6);
        }
        return cqj6;
    }
    
    private BlockPathTypes getBlockPathType(final Mob akx, final BlockPos fk) {
        return this.getBlockPathType(akx, fk.getX(), fk.getY(), fk.getZ());
    }
    
    private BlockPathTypes getBlockPathType(final Mob akx, final int integer2, final int integer3, final int integer4) {
        return this.getBlockPathType(this.level, integer2, integer3, integer4, akx, this.entityWidth, this.entityHeight, this.entityDepth, this.canOpenDoors(), this.canPassDoors());
    }
}
