package net.minecraft.world.level.pathfinder;

import net.minecraft.network.FriendlyByteBuf;
import net.minecraft.core.BlockPos;
import net.minecraft.util.Mth;

public class Node {
    public final int x;
    public final int y;
    public final int z;
    private final int hash;
    public int heapIdx;
    public float g;
    public float h;
    public float f;
    public Node cameFrom;
    public boolean closed;
    public float walkedDistance;
    public float costMalus;
    public BlockPathTypes type;
    
    public Node(final int integer1, final int integer2, final int integer3) {
        this.heapIdx = -1;
        this.type = BlockPathTypes.BLOCKED;
        this.x = integer1;
        this.y = integer2;
        this.z = integer3;
        this.hash = createHash(integer1, integer2, integer3);
    }
    
    public Node cloneMove(final int integer1, final int integer2, final int integer3) {
        final Node cql5 = new Node(integer1, integer2, integer3);
        cql5.heapIdx = this.heapIdx;
        cql5.g = this.g;
        cql5.h = this.h;
        cql5.f = this.f;
        cql5.cameFrom = this.cameFrom;
        cql5.closed = this.closed;
        cql5.walkedDistance = this.walkedDistance;
        cql5.costMalus = this.costMalus;
        cql5.type = this.type;
        return cql5;
    }
    
    public static int createHash(final int integer1, final int integer2, final int integer3) {
        return (integer2 & 0xFF) | (integer1 & 0x7FFF) << 8 | (integer3 & 0x7FFF) << 24 | ((integer1 < 0) ? Integer.MIN_VALUE : 0) | ((integer3 < 0) ? 32768 : 0);
    }
    
    public float distanceTo(final Node cql) {
        final float float3 = (float)(cql.x - this.x);
        final float float4 = (float)(cql.y - this.y);
        final float float5 = (float)(cql.z - this.z);
        return Mth.sqrt(float3 * float3 + float4 * float4 + float5 * float5);
    }
    
    public float distanceToSqr(final Node cql) {
        final float float3 = (float)(cql.x - this.x);
        final float float4 = (float)(cql.y - this.y);
        final float float5 = (float)(cql.z - this.z);
        return float3 * float3 + float4 * float4 + float5 * float5;
    }
    
    public float distanceManhattan(final Node cql) {
        final float float3 = (float)Math.abs(cql.x - this.x);
        final float float4 = (float)Math.abs(cql.y - this.y);
        final float float5 = (float)Math.abs(cql.z - this.z);
        return float3 + float4 + float5;
    }
    
    public float distanceManhattan(final BlockPos fk) {
        final float float3 = (float)Math.abs(fk.getX() - this.x);
        final float float4 = (float)Math.abs(fk.getY() - this.y);
        final float float5 = (float)Math.abs(fk.getZ() - this.z);
        return float3 + float4 + float5;
    }
    
    public BlockPos asBlockPos() {
        return new BlockPos(this.x, this.y, this.z);
    }
    
    @Override
    public boolean equals(final Object object) {
        if (object instanceof Node) {
            final Node cql3 = (Node)object;
            return this.hash == cql3.hash && this.x == cql3.x && this.y == cql3.y && this.z == cql3.z;
        }
        return false;
    }
    
    @Override
    public int hashCode() {
        return this.hash;
    }
    
    public boolean inOpenSet() {
        return this.heapIdx >= 0;
    }
    
    @Override
    public String toString() {
        return "Node{x=" + this.x + ", y=" + this.y + ", z=" + this.z + '}';
    }
    
    public static Node createFromStream(final FriendlyByteBuf kv) {
        final Node cql2 = new Node(kv.readInt(), kv.readInt(), kv.readInt());
        cql2.walkedDistance = kv.readFloat();
        cql2.costMalus = kv.readFloat();
        cql2.closed = kv.readBoolean();
        cql2.type = BlockPathTypes.values()[kv.readInt()];
        cql2.f = kv.readFloat();
        return cql2;
    }
}
