package net.minecraft.world.level.pathfinder;

import com.google.common.collect.Lists;
import com.google.common.collect.Sets;
import net.minecraft.network.FriendlyByteBuf;
import net.minecraft.world.phys.Vec3;
import net.minecraft.world.entity.Entity;
import javax.annotation.Nullable;
import net.minecraft.core.BlockPos;
import java.util.Set;
import java.util.List;

public class Path {
    private final List<Node> nodes;
    private Node[] openSet;
    private Node[] closedSet;
    private Set<Target> targetNodes;
    private int index;
    private final BlockPos target;
    private final float distToTarget;
    private final boolean reached;
    
    public Path(final List<Node> list, final BlockPos fk, final boolean boolean3) {
        this.openSet = new Node[0];
        this.closedSet = new Node[0];
        this.nodes = list;
        this.target = fk;
        this.distToTarget = (list.isEmpty() ? Float.MAX_VALUE : this.nodes.get(this.nodes.size() - 1).distanceManhattan(this.target));
        this.reached = boolean3;
    }
    
    public void next() {
        ++this.index;
    }
    
    public boolean isDone() {
        return this.index >= this.nodes.size();
    }
    
    @Nullable
    public Node last() {
        if (!this.nodes.isEmpty()) {
            return this.nodes.get(this.nodes.size() - 1);
        }
        return null;
    }
    
    public Node get(final int integer) {
        return this.nodes.get(integer);
    }
    
    public List<Node> getNodes() {
        return this.nodes;
    }
    
    public void truncate(final int integer) {
        if (this.nodes.size() > integer) {
            this.nodes.subList(integer, this.nodes.size()).clear();
        }
    }
    
    public void set(final int integer, final Node cql) {
        this.nodes.set(integer, cql);
    }
    
    public int getSize() {
        return this.nodes.size();
    }
    
    public int getIndex() {
        return this.index;
    }
    
    public void setIndex(final int integer) {
        this.index = integer;
    }
    
    public Vec3 getPos(final Entity akn, final int integer) {
        final Node cql4 = this.nodes.get(integer);
        final double double5 = cql4.x + (int)(akn.getBbWidth() + 1.0f) * 0.5;
        final double double6 = cql4.y;
        final double double7 = cql4.z + (int)(akn.getBbWidth() + 1.0f) * 0.5;
        return new Vec3(double5, double6, double7);
    }
    
    public Vec3 currentPos(final Entity akn) {
        return this.getPos(akn, this.index);
    }
    
    public Vec3 currentPos() {
        final Node cql2 = this.nodes.get(this.index);
        return new Vec3(cql2.x, cql2.y, cql2.z);
    }
    
    public boolean sameAs(@Nullable final Path cqn) {
        if (cqn == null) {
            return false;
        }
        if (cqn.nodes.size() != this.nodes.size()) {
            return false;
        }
        for (int integer3 = 0; integer3 < this.nodes.size(); ++integer3) {
            final Node cql4 = this.nodes.get(integer3);
            final Node cql5 = cqn.nodes.get(integer3);
            if (cql4.x != cql5.x || cql4.y != cql5.y || cql4.z != cql5.z) {
                return false;
            }
        }
        return true;
    }
    
    public boolean canReach() {
        return this.reached;
    }
    
    public Node[] getOpenSet() {
        return this.openSet;
    }
    
    public Node[] getClosedSet() {
        return this.closedSet;
    }
    
    public static Path createFromStream(final FriendlyByteBuf kv) {
        final boolean boolean2 = kv.readBoolean();
        final int integer3 = kv.readInt();
        final int integer4 = kv.readInt();
        final Set<Target> set5 = Sets.newHashSet();
        for (int integer5 = 0; integer5 < integer4; ++integer5) {
            set5.add(Target.createFromStream(kv));
        }
        final BlockPos fk6 = new BlockPos(kv.readInt(), kv.readInt(), kv.readInt());
        final List<Node> list7 = Lists.newArrayList();
        for (int integer6 = kv.readInt(), integer7 = 0; integer7 < integer6; ++integer7) {
            list7.add(Node.createFromStream(kv));
        }
        final Node[] arr9 = new Node[kv.readInt()];
        for (int integer8 = 0; integer8 < arr9.length; ++integer8) {
            arr9[integer8] = Node.createFromStream(kv);
        }
        final Node[] arr10 = new Node[kv.readInt()];
        for (int integer9 = 0; integer9 < arr10.length; ++integer9) {
            arr10[integer9] = Node.createFromStream(kv);
        }
        final Path cqn11 = new Path(list7, fk6, boolean2);
        cqn11.openSet = arr9;
        cqn11.closedSet = arr10;
        cqn11.targetNodes = set5;
        cqn11.index = integer3;
        return cqn11;
    }
    
    @Override
    public String toString() {
        return "Path(length=" + this.nodes.size() + ")";
    }
    
    public BlockPos getTarget() {
        return this.target;
    }
    
    public float getDistToTarget() {
        return this.distToTarget;
    }
}
