package net.minecraft.world.level.pathfinder;

import java.util.List;
import com.google.common.collect.Lists;
import java.util.Iterator;
import java.util.Optional;
import java.util.stream.Stream;
import java.util.Comparator;
import javax.annotation.Nullable;
import java.util.stream.Collectors;
import java.util.function.Function;
import java.util.Map;
import net.minecraft.core.BlockPos;
import net.minecraft.world.entity.Mob;
import net.minecraft.world.level.PathNavigationRegion;
import com.google.common.collect.Sets;
import java.util.Set;

public class PathFinder {
    private final BinaryHeap openSet;
    private final Set<Node> closedSet;
    private final Node[] neighbors;
    private final int maxVisitedNodes;
    private final NodeEvaluator nodeEvaluator;
    
    public PathFinder(final NodeEvaluator cqm, final int integer) {
        this.openSet = new BinaryHeap();
        this.closedSet = Sets.newHashSet();
        this.neighbors = new Node[32];
        this.nodeEvaluator = cqm;
        this.maxVisitedNodes = integer;
    }
    
    @Nullable
    public Path findPath(final PathNavigationRegion bke, final Mob akx, final Set<BlockPos> set, final float float4, final int integer, final float float6) {
        this.openSet.clear();
        this.nodeEvaluator.prepare(bke, akx);
        final Node cql8 = this.nodeEvaluator.getStart();
        final Map<Target, BlockPos> map9 = set.stream().collect(Collectors.toMap(fk -> this.nodeEvaluator.getGoal(fk.getX(), fk.getY(), fk.getZ()), Function.identity()));
        final Path cqn10 = this.findPath(cql8, map9, float4, integer, float6);
        this.nodeEvaluator.done();
        return cqn10;
    }
    
    @Nullable
    private Path findPath(final Node cql, final Map<Target, BlockPos> map, final float float3, final int integer, final float float5) {
        final Set<Target> set7 = map.keySet();
        cql.g = 0.0f;
        cql.h = this.getBestH(cql, set7);
        cql.f = cql.h;
        this.openSet.clear();
        this.closedSet.clear();
        this.openSet.insert(cql);
        int integer2 = 0;
        final int integer3 = (int)(this.maxVisitedNodes * float5);
        while (!this.openSet.isEmpty() && ++integer2 < integer3) {
            final Node cql2 = this.openSet.pop();
            cql2.closed = true;
            set7.stream().filter(cqr -> cql2.distanceManhattan(cqr) <= integer).forEach(Target::setReached);
            if (set7.stream().anyMatch(Target::isReached)) {
                break;
            }
            if (cql2.distanceTo(cql) >= float3) {
                continue;
            }
            for (int integer4 = this.nodeEvaluator.getNeighbors(this.neighbors, cql2), integer5 = 0; integer5 < integer4; ++integer5) {
                final Node cql3 = this.neighbors[integer5];
                final float float6 = cql2.distanceTo(cql3);
                cql3.walkedDistance = cql2.walkedDistance + float6;
                final float float7 = cql2.g + float6 + cql3.costMalus;
                if (cql3.walkedDistance < float3 && (!cql3.inOpenSet() || float7 < cql3.g)) {
                    cql3.cameFrom = cql2;
                    cql3.g = float7;
                    cql3.h = this.getBestH(cql3, set7) * 1.5f;
                    if (cql3.inOpenSet()) {
                        this.openSet.changeCost(cql3, cql3.g + cql3.h);
                    }
                    else {
                        cql3.f = cql3.g + cql3.h;
                        this.openSet.insert(cql3);
                    }
                }
            }
        }
        Stream<Path> stream10;
        if (set7.stream().anyMatch(Target::isReached)) {
            stream10 = set7.stream().filter(Target::isReached).<Path>map(cqr -> this.reconstructPath(cqr.getBestNode(), map.get(cqr), true)).sorted(Comparator.comparingInt(Path::getSize));
        }
        else {
            stream10 = set7.stream().<Path>map(cqr -> this.reconstructPath(cqr.getBestNode(), map.get(cqr), false)).sorted(Comparator.comparingDouble(Path::getDistToTarget).thenComparingInt(Path::getSize));
        }
        final Optional<Path> optional11 = stream10.findFirst();
        if (!optional11.isPresent()) {
            return null;
        }
        final Path cqn12 = optional11.get();
        return cqn12;
    }
    
    private float getBestH(final Node cql, final Set<Target> set) {
        float float4 = Float.MAX_VALUE;
        for (final Target cqr6 : set) {
            final float float5 = cql.distanceTo(cqr6);
            cqr6.updateBest(float5, cql);
            float4 = Math.min(float5, float4);
        }
        return float4;
    }
    
    private Path reconstructPath(final Node cql, final BlockPos fk, final boolean boolean3) {
        final List<Node> list5 = Lists.newArrayList();
        Node cql2 = cql;
        list5.add(0, cql2);
        while (cql2.cameFrom != null) {
            cql2 = cql2.cameFrom;
            list5.add(0, cql2);
        }
        return new Path(list5, fk, boolean3);
    }
}
