package net.minecraft.world.level.pathfinder;

import net.minecraft.world.level.block.Block;
import net.minecraft.world.level.block.Blocks;
import net.minecraft.world.level.block.BaseRailBlock;
import javax.annotation.Nullable;
import net.minecraft.world.entity.Entity;
import net.minecraft.world.phys.AABB;
import net.minecraft.world.phys.shapes.VoxelShape;
import net.minecraft.core.Direction;
import net.minecraft.world.level.BlockGetter;
import net.minecraft.core.BlockPos;
import net.minecraft.util.Mth;
import net.minecraft.world.entity.Mob;
import net.minecraft.world.level.PathNavigationRegion;

public class TurtleNodeEvaluator extends WalkNodeEvaluator {
    private float oldWalkableCost;
    private float oldWaterBorderCost;
    
    @Override
    public void prepare(final PathNavigationRegion bke, final Mob akx) {
        super.prepare(bke, akx);
        akx.setPathfindingMalus(BlockPathTypes.WATER, 0.0f);
        this.oldWalkableCost = akx.getPathfindingMalus(BlockPathTypes.WALKABLE);
        akx.setPathfindingMalus(BlockPathTypes.WALKABLE, 6.0f);
        this.oldWaterBorderCost = akx.getPathfindingMalus(BlockPathTypes.WATER_BORDER);
        akx.setPathfindingMalus(BlockPathTypes.WATER_BORDER, 4.0f);
    }
    
    @Override
    public void done() {
        this.mob.setPathfindingMalus(BlockPathTypes.WALKABLE, this.oldWalkableCost);
        this.mob.setPathfindingMalus(BlockPathTypes.WATER_BORDER, this.oldWaterBorderCost);
        super.done();
    }
    
    @Override
    public Node getStart() {
        return this.getNode(Mth.floor(this.mob.getBoundingBox().minX), Mth.floor(this.mob.getBoundingBox().minY + 0.5), Mth.floor(this.mob.getBoundingBox().minZ));
    }
    
    @Override
    public Target getGoal(final double double1, final double double2, final double double3) {
        return new Target(this.getNode(Mth.floor(double1), Mth.floor(double2 + 0.5), Mth.floor(double3)));
    }
    
    @Override
    public int getNeighbors(final Node[] arr, final Node cql) {
        int integer4 = 0;
        final int integer5 = 1;
        final BlockPos fk6 = new BlockPos(cql.x, cql.y, cql.z);
        final double double7 = this.inWaterDependentPosHeight(fk6);
        final Node cql2 = this.getAcceptedNode(cql.x, cql.y, cql.z + 1, 1, double7);
        final Node cql3 = this.getAcceptedNode(cql.x - 1, cql.y, cql.z, 1, double7);
        final Node cql4 = this.getAcceptedNode(cql.x + 1, cql.y, cql.z, 1, double7);
        final Node cql5 = this.getAcceptedNode(cql.x, cql.y, cql.z - 1, 1, double7);
        final Node cql6 = this.getAcceptedNode(cql.x, cql.y + 1, cql.z, 0, double7);
        final Node cql7 = this.getAcceptedNode(cql.x, cql.y - 1, cql.z, 1, double7);
        if (cql2 != null && !cql2.closed) {
            arr[integer4++] = cql2;
        }
        if (cql3 != null && !cql3.closed) {
            arr[integer4++] = cql3;
        }
        if (cql4 != null && !cql4.closed) {
            arr[integer4++] = cql4;
        }
        if (cql5 != null && !cql5.closed) {
            arr[integer4++] = cql5;
        }
        if (cql6 != null && !cql6.closed) {
            arr[integer4++] = cql6;
        }
        if (cql7 != null && !cql7.closed) {
            arr[integer4++] = cql7;
        }
        final boolean boolean15 = cql5 == null || cql5.type == BlockPathTypes.OPEN || cql5.costMalus != 0.0f;
        final boolean boolean16 = cql2 == null || cql2.type == BlockPathTypes.OPEN || cql2.costMalus != 0.0f;
        final boolean boolean17 = cql4 == null || cql4.type == BlockPathTypes.OPEN || cql4.costMalus != 0.0f;
        final boolean boolean18 = cql3 == null || cql3.type == BlockPathTypes.OPEN || cql3.costMalus != 0.0f;
        if (boolean15 && boolean18) {
            final Node cql8 = this.getAcceptedNode(cql.x - 1, cql.y, cql.z - 1, 1, double7);
            if (cql8 != null && !cql8.closed) {
                arr[integer4++] = cql8;
            }
        }
        if (boolean15 && boolean17) {
            final Node cql8 = this.getAcceptedNode(cql.x + 1, cql.y, cql.z - 1, 1, double7);
            if (cql8 != null && !cql8.closed) {
                arr[integer4++] = cql8;
            }
        }
        if (boolean16 && boolean18) {
            final Node cql8 = this.getAcceptedNode(cql.x - 1, cql.y, cql.z + 1, 1, double7);
            if (cql8 != null && !cql8.closed) {
                arr[integer4++] = cql8;
            }
        }
        if (boolean16 && boolean17) {
            final Node cql8 = this.getAcceptedNode(cql.x + 1, cql.y, cql.z + 1, 1, double7);
            if (cql8 != null && !cql8.closed) {
                arr[integer4++] = cql8;
            }
        }
        return integer4;
    }
    
    private double inWaterDependentPosHeight(final BlockPos fk) {
        if (!this.mob.isInWater()) {
            final BlockPos fk2 = fk.below();
            final VoxelShape cwc4 = this.level.getBlockState(fk2).getCollisionShape(this.level, fk2);
            return fk2.getY() + (cwc4.isEmpty() ? 0.0 : cwc4.max(Direction.Axis.Y));
        }
        return fk.getY() + 0.5;
    }
    
    @Nullable
    private Node getAcceptedNode(final int integer1, int integer2, final int integer3, final int integer4, final double double5) {
        Node cql8 = null;
        final BlockPos fk9 = new BlockPos(integer1, integer2, integer3);
        final double double6 = this.inWaterDependentPosHeight(fk9);
        if (double6 - double5 > 1.125) {
            return null;
        }
        BlockPathTypes cqj12 = this.getBlockPathType(this.level, integer1, integer2, integer3, this.mob, this.entityWidth, this.entityHeight, this.entityDepth, false, false);
        float float13 = this.mob.getPathfindingMalus(cqj12);
        final double double7 = this.mob.getBbWidth() / 2.0;
        if (float13 >= 0.0f) {
            cql8 = this.getNode(integer1, integer2, integer3);
            cql8.type = cqj12;
            cql8.costMalus = Math.max(cql8.costMalus, float13);
        }
        if (cqj12 == BlockPathTypes.WATER || cqj12 == BlockPathTypes.WALKABLE) {
            if (integer2 < this.mob.level.getSeaLevel() - 10 && cql8 != null) {
                final Node node = cql8;
                ++node.costMalus;
            }
            return cql8;
        }
        if (cql8 == null && integer4 > 0 && cqj12 != BlockPathTypes.FENCE && cqj12 != BlockPathTypes.TRAPDOOR) {
            cql8 = this.getAcceptedNode(integer1, integer2 + 1, integer3, integer4 - 1, double5);
        }
        if (cqj12 == BlockPathTypes.OPEN) {
            final AABB cvc16 = new AABB(integer1 - double7 + 0.5, integer2 + 0.001, integer3 - double7 + 0.5, integer1 + double7 + 0.5, integer2 + this.mob.getBbHeight(), integer3 + double7 + 0.5);
            if (!this.mob.level.noCollision(this.mob, cvc16)) {
                return null;
            }
            final BlockPathTypes cqj13 = this.getBlockPathType(this.level, integer1, integer2 - 1, integer3, this.mob, this.entityWidth, this.entityHeight, this.entityDepth, false, false);
            if (cqj13 == BlockPathTypes.BLOCKED) {
                cql8 = this.getNode(integer1, integer2, integer3);
                cql8.type = BlockPathTypes.WALKABLE;
                cql8.costMalus = Math.max(cql8.costMalus, float13);
                return cql8;
            }
            if (cqj13 == BlockPathTypes.WATER) {
                cql8 = this.getNode(integer1, integer2, integer3);
                cql8.type = BlockPathTypes.WATER;
                cql8.costMalus = Math.max(cql8.costMalus, float13);
                return cql8;
            }
            int integer5 = 0;
            while (integer2 > 0 && cqj12 == BlockPathTypes.OPEN) {
                --integer2;
                if (integer5++ >= this.mob.getMaxFallDistance()) {
                    return null;
                }
                cqj12 = this.getBlockPathType(this.level, integer1, integer2, integer3, this.mob, this.entityWidth, this.entityHeight, this.entityDepth, false, false);
                float13 = this.mob.getPathfindingMalus(cqj12);
                if (cqj12 != BlockPathTypes.OPEN && float13 >= 0.0f) {
                    cql8 = this.getNode(integer1, integer2, integer3);
                    cql8.type = cqj12;
                    cql8.costMalus = Math.max(cql8.costMalus, float13);
                    break;
                }
                if (float13 < 0.0f) {
                    return null;
                }
            }
        }
        return cql8;
    }
    
    @Override
    protected BlockPathTypes evaluateBlockPathType(final BlockGetter bjd, final boolean boolean2, final boolean boolean3, final BlockPos fk, BlockPathTypes cqj) {
        if (cqj == BlockPathTypes.RAIL && !(bjd.getBlockState(fk).getBlock() instanceof BaseRailBlock) && !(bjd.getBlockState(fk.below()).getBlock() instanceof BaseRailBlock)) {
            cqj = BlockPathTypes.FENCE;
        }
        if (cqj == BlockPathTypes.DOOR_OPEN || cqj == BlockPathTypes.DOOR_WOOD_CLOSED || cqj == BlockPathTypes.DOOR_IRON_CLOSED) {
            cqj = BlockPathTypes.BLOCKED;
        }
        if (cqj == BlockPathTypes.LEAVES) {
            cqj = BlockPathTypes.BLOCKED;
        }
        return cqj;
    }
    
    @Override
    public BlockPathTypes getBlockPathType(final BlockGetter bjd, final int integer2, final int integer3, final int integer4) {
        BlockPathTypes cqj6 = WalkNodeEvaluator.getBlockPathTypeRaw(bjd, integer2, integer3, integer4);
        if (cqj6 == BlockPathTypes.WATER) {
            for (final Direction fp10 : Direction.values()) {
                final BlockPathTypes cqj7 = WalkNodeEvaluator.getBlockPathTypeRaw(bjd, integer2 + fp10.getStepX(), integer3 + fp10.getStepY(), integer4 + fp10.getStepZ());
                if (cqj7 == BlockPathTypes.BLOCKED) {
                    return BlockPathTypes.WATER_BORDER;
                }
            }
            return BlockPathTypes.WATER;
        }
        if (cqj6 == BlockPathTypes.OPEN && integer3 >= 1) {
            final Block bpe7 = bjd.getBlockState(new BlockPos(integer2, integer3 - 1, integer4)).getBlock();
            final BlockPathTypes cqj8 = WalkNodeEvaluator.getBlockPathTypeRaw(bjd, integer2, integer3 - 1, integer4);
            if (cqj8 == BlockPathTypes.WALKABLE || cqj8 == BlockPathTypes.OPEN || cqj8 == BlockPathTypes.LAVA) {
                cqj6 = BlockPathTypes.OPEN;
            }
            else {
                cqj6 = BlockPathTypes.WALKABLE;
            }
            if (cqj8 == BlockPathTypes.DAMAGE_FIRE || bpe7 == Blocks.MAGMA_BLOCK || bpe7 == Blocks.CAMPFIRE) {
                cqj6 = BlockPathTypes.DAMAGE_FIRE;
            }
            if (cqj8 == BlockPathTypes.DAMAGE_CACTUS) {
                cqj6 = BlockPathTypes.DAMAGE_CACTUS;
            }
            if (cqj8 == BlockPathTypes.DAMAGE_OTHER) {
                cqj6 = BlockPathTypes.DAMAGE_OTHER;
            }
        }
        if (cqj6 == BlockPathTypes.WALKABLE) {
            cqj6 = WalkNodeEvaluator.checkNeighbourBlocks(bjd, integer2, integer3, integer4, cqj6);
        }
        return cqj6;
    }
}
