package net.minecraft.world.level.saveddata.maps;

import net.minecraft.world.level.block.state.BlockState;
import net.minecraft.nbt.Tag;
import java.util.Objects;
import net.minecraft.world.level.block.entity.BlockEntity;
import net.minecraft.world.level.block.entity.BannerBlockEntity;
import net.minecraft.world.level.BlockGetter;
import net.minecraft.nbt.NbtUtils;
import net.minecraft.nbt.CompoundTag;
import javax.annotation.Nullable;
import net.minecraft.network.chat.Component;
import net.minecraft.world.item.DyeColor;
import net.minecraft.core.BlockPos;

public class MapBanner {
    private final BlockPos pos;
    private final DyeColor color;
    @Nullable
    private final Component name;
    
    public MapBanner(final BlockPos fk, final DyeColor bdg, @Nullable final Component lf) {
        this.pos = fk;
        this.color = bdg;
        this.name = lf;
    }
    
    public static MapBanner load(final CompoundTag jt) {
        final BlockPos fk2 = NbtUtils.readBlockPos(jt.getCompound("Pos"));
        final DyeColor bdg3 = DyeColor.byName(jt.getString("Color"), DyeColor.WHITE);
        final Component lf4 = jt.contains("Name") ? Component.Serializer.fromJson(jt.getString("Name")) : null;
        return new MapBanner(fk2, bdg3, lf4);
    }
    
    @Nullable
    public static MapBanner fromWorld(final BlockGetter bjd, final BlockPos fk) {
        final BlockEntity bwi3 = bjd.getBlockEntity(fk);
        if (bwi3 instanceof BannerBlockEntity) {
            final BannerBlockEntity bvz4 = (BannerBlockEntity)bwi3;
            final DyeColor bdg5 = bvz4.getBaseColor(() -> bjd.getBlockState(fk));
            final Component lf6 = bvz4.hasCustomName() ? bvz4.getCustomName() : null;
            return new MapBanner(fk, bdg5, lf6);
        }
        return null;
    }
    
    public BlockPos getPos() {
        return this.pos;
    }
    
    public MapDecoration.Type getDecoration() {
        switch (this.color) {
            case WHITE: {
                return MapDecoration.Type.BANNER_WHITE;
            }
            case ORANGE: {
                return MapDecoration.Type.BANNER_ORANGE;
            }
            case MAGENTA: {
                return MapDecoration.Type.BANNER_MAGENTA;
            }
            case LIGHT_BLUE: {
                return MapDecoration.Type.BANNER_LIGHT_BLUE;
            }
            case YELLOW: {
                return MapDecoration.Type.BANNER_YELLOW;
            }
            case LIME: {
                return MapDecoration.Type.BANNER_LIME;
            }
            case PINK: {
                return MapDecoration.Type.BANNER_PINK;
            }
            case GRAY: {
                return MapDecoration.Type.BANNER_GRAY;
            }
            case LIGHT_GRAY: {
                return MapDecoration.Type.BANNER_LIGHT_GRAY;
            }
            case CYAN: {
                return MapDecoration.Type.BANNER_CYAN;
            }
            case PURPLE: {
                return MapDecoration.Type.BANNER_PURPLE;
            }
            case BLUE: {
                return MapDecoration.Type.BANNER_BLUE;
            }
            case BROWN: {
                return MapDecoration.Type.BANNER_BROWN;
            }
            case GREEN: {
                return MapDecoration.Type.BANNER_GREEN;
            }
            case RED: {
                return MapDecoration.Type.BANNER_RED;
            }
            default: {
                return MapDecoration.Type.BANNER_BLACK;
            }
        }
    }
    
    @Nullable
    public Component getName() {
        return this.name;
    }
    
    @Override
    public boolean equals(final Object object) {
        if (this == object) {
            return true;
        }
        if (object == null || this.getClass() != object.getClass()) {
            return false;
        }
        final MapBanner cqz3 = (MapBanner)object;
        return Objects.equals(this.pos, cqz3.pos) && this.color == cqz3.color && Objects.equals(this.name, cqz3.name);
    }
    
    @Override
    public int hashCode() {
        return Objects.hash(this.pos, this.color, this.name);
    }
    
    public CompoundTag save() {
        final CompoundTag jt2 = new CompoundTag();
        jt2.put("Pos", NbtUtils.writeBlockPos(this.pos));
        jt2.putString("Color", this.color.getName());
        if (this.name != null) {
            jt2.putString("Name", Component.Serializer.toJson(this.name));
        }
        return jt2;
    }
    
    public String getId() {
        return "banner-" + this.pos.getX() + "," + this.pos.getY() + "," + this.pos.getZ();
    }
}
