package net.minecraft.world.level.storage.loot.functions;

import net.minecraft.util.GsonHelper;
import com.google.gson.JsonDeserializationContext;
import com.google.gson.JsonSerializationContext;
import com.google.gson.JsonObject;
import net.minecraft.world.level.storage.loot.LootTable;
import net.minecraft.world.level.storage.loot.ValidationContext;
import net.minecraft.nbt.Tag;
import net.minecraft.nbt.CompoundTag;
import net.minecraft.world.level.storage.loot.LootContext;
import net.minecraft.world.item.ItemStack;
import net.minecraft.world.level.storage.loot.predicates.LootItemCondition;
import net.minecraft.resources.ResourceLocation;

public class SetContainerLootTable extends LootItemConditionalFunction {
    private final ResourceLocation name;
    private final long seed;
    
    private SetContainerLootTable(final LootItemCondition[] arr, final ResourceLocation sm, final long long3) {
        super(arr);
        this.name = sm;
        this.seed = long3;
    }
    
    public ItemStack run(final ItemStack bek, final LootContext cru) {
        if (bek.isEmpty()) {
            return bek;
        }
        final CompoundTag jt4 = new CompoundTag();
        jt4.putString("LootTable", this.name.toString());
        if (this.seed != 0L) {
            jt4.putLong("LootTableSeed", this.seed);
        }
        bek.getOrCreateTag().put("BlockEntityTag", jt4);
        return bek;
    }
    
    @Override
    public void validate(final ValidationContext csd) {
        if (csd.hasVisitedTable(this.name)) {
            csd.reportProblem("Table " + this.name + " is recursively called");
            return;
        }
        super.validate(csd);
        final LootTable crx3 = csd.resolveLootTable(this.name);
        if (crx3 == null) {
            csd.reportProblem("Unknown loot table called " + this.name);
        }
        else {
            crx3.validate(csd.enterTable("->{" + this.name + "}", this.name));
        }
    }
    
    public static class Serializer extends LootItemConditionalFunction.Serializer<SetContainerLootTable> {
        protected Serializer() {
            super(new ResourceLocation("set_loot_table"), SetContainerLootTable.class);
        }
        
        @Override
        public void serialize(final JsonObject jsonObject, final SetContainerLootTable ctk, final JsonSerializationContext jsonSerializationContext) {
            super.serialize(jsonObject, ctk, jsonSerializationContext);
            jsonObject.addProperty("name", ctk.name.toString());
            if (ctk.seed != 0L) {
                jsonObject.addProperty("seed", ctk.seed);
            }
        }
        
        @Override
        public SetContainerLootTable deserialize(final JsonObject jsonObject, final JsonDeserializationContext jsonDeserializationContext, final LootItemCondition[] arr) {
            final ResourceLocation sm5 = new ResourceLocation(GsonHelper.getAsString(jsonObject, "name"));
            final long long6 = GsonHelper.getAsLong(jsonObject, "seed", 0L);
            return new SetContainerLootTable(arr, sm5, long6, null);
        }
    }
}
