package net.minecraft.world.phys.shapes;

import it.unimi.dsi.fastutil.doubles.DoubleArrayList;
import com.google.common.annotations.VisibleForTesting;
import java.util.Objects;
import it.unimi.dsi.fastutil.doubles.DoubleList;
import com.google.common.math.DoubleMath;
import net.minecraft.world.level.block.state.BlockState;
import net.minecraft.world.level.BlockGetter;
import net.minecraft.world.level.block.Blocks;
import net.minecraft.util.Mth;
import net.minecraft.core.BlockPos;
import net.minecraft.world.level.LevelReader;
import java.util.Iterator;
import java.util.stream.Stream;
import net.minecraft.core.AxisCycle;
import net.minecraft.core.Direction;
import net.minecraft.Util;
import java.util.Arrays;
import com.google.common.math.IntMath;
import net.minecraft.world.phys.AABB;

public final class Shapes {
    private static final VoxelShape BLOCK;
    public static final VoxelShape INFINITY;
    private static final VoxelShape EMPTY;
    
    public static VoxelShape empty() {
        return Shapes.EMPTY;
    }
    
    public static VoxelShape block() {
        return Shapes.BLOCK;
    }
    
    public static VoxelShape box(final double double1, final double double2, final double double3, final double double4, final double double5, final double double6) {
        return create(new AABB(double1, double2, double3, double4, double5, double6));
    }
    
    public static VoxelShape create(final AABB cvc) {
        final int integer2 = findBits(cvc.minX, cvc.maxX);
        final int integer3 = findBits(cvc.minY, cvc.maxY);
        final int integer4 = findBits(cvc.minZ, cvc.maxZ);
        if (integer2 < 0 || integer3 < 0 || integer4 < 0) {
            return new ArrayVoxelShape(Shapes.BLOCK.shape, new double[] { cvc.minX, cvc.maxX }, new double[] { cvc.minY, cvc.maxY }, new double[] { cvc.minZ, cvc.maxZ });
        }
        if (integer2 == 0 && integer3 == 0 && integer4 == 0) {
            return cvc.contains(0.5, 0.5, 0.5) ? block() : empty();
        }
        final int integer5 = 1 << integer2;
        final int integer6 = 1 << integer3;
        final int integer7 = 1 << integer4;
        final int integer8 = (int)Math.round(cvc.minX * integer5);
        final int integer9 = (int)Math.round(cvc.maxX * integer5);
        final int integer10 = (int)Math.round(cvc.minY * integer6);
        final int integer11 = (int)Math.round(cvc.maxY * integer6);
        final int integer12 = (int)Math.round(cvc.minZ * integer7);
        final int integer13 = (int)Math.round(cvc.maxZ * integer7);
        final BitSetDiscreteVoxelShape cvl14 = new BitSetDiscreteVoxelShape(integer5, integer6, integer7, integer8, integer10, integer12, integer9, integer11, integer13);
        for (long long15 = integer8; long15 < integer9; ++long15) {
            for (long long16 = integer10; long16 < integer11; ++long16) {
                for (long long17 = integer12; long17 < integer13; ++long17) {
                    cvl14.setFull((int)long15, (int)long16, (int)long17, false, true);
                }
            }
        }
        return new CubeVoxelShape(cvl14);
    }
    
    private static int findBits(final double double1, final double double2) {
        if (double1 < -1.0E-7 || double2 > 1.0000001) {
            return -1;
        }
        for (int integer5 = 0; integer5 <= 3; ++integer5) {
            final double double3 = double1 * (1 << integer5);
            final double double4 = double2 * (1 << integer5);
            final boolean boolean10 = Math.abs(double3 - Math.floor(double3)) < 1.0E-7;
            final boolean boolean11 = Math.abs(double4 - Math.floor(double4)) < 1.0E-7;
            if (boolean10 && boolean11) {
                return integer5;
            }
        }
        return -1;
    }
    
    protected static long lcm(final int integer1, final int integer2) {
        return integer1 * (long)(integer2 / IntMath.gcd(integer1, integer2));
    }
    
    public static VoxelShape or(final VoxelShape cwc1, final VoxelShape cwc2) {
        return join(cwc1, cwc2, BooleanOp.OR);
    }
    
    public static VoxelShape or(final VoxelShape cwc, final VoxelShape... arr) {
        return Arrays.<VoxelShape>stream(arr).reduce(cwc, Shapes::or);
    }
    
    public static VoxelShape join(final VoxelShape cwc1, final VoxelShape cwc2, final BooleanOp cvm) {
        return joinUnoptimized(cwc1, cwc2, cvm).optimize();
    }
    
    public static VoxelShape joinUnoptimized(final VoxelShape cwc1, final VoxelShape cwc2, final BooleanOp cvm) {
        if (cvm.apply(false, false)) {
            throw Util.<IllegalArgumentException>pauseInIde(new IllegalArgumentException());
        }
        if (cwc1 == cwc2) {
            return cvm.apply(true, true) ? cwc1 : empty();
        }
        final boolean boolean4 = cvm.apply(true, false);
        final boolean boolean5 = cvm.apply(false, true);
        if (cwc1.isEmpty()) {
            return boolean5 ? cwc2 : empty();
        }
        if (cwc2.isEmpty()) {
            return boolean4 ? cwc1 : empty();
        }
        final IndexMerger cvu6 = createIndexMerger(1, cwc1.getCoords(Direction.Axis.X), cwc2.getCoords(Direction.Axis.X), boolean4, boolean5);
        final IndexMerger cvu7 = createIndexMerger(cvu6.getList().size() - 1, cwc1.getCoords(Direction.Axis.Y), cwc2.getCoords(Direction.Axis.Y), boolean4, boolean5);
        final IndexMerger cvu8 = createIndexMerger((cvu6.getList().size() - 1) * (cvu7.getList().size() - 1), cwc1.getCoords(Direction.Axis.Z), cwc2.getCoords(Direction.Axis.Z), boolean4, boolean5);
        final BitSetDiscreteVoxelShape cvl9 = BitSetDiscreteVoxelShape.join(cwc1.shape, cwc2.shape, cvu6, cvu7, cvu8, cvm);
        if (cvu6 instanceof DiscreteCubeMerger && cvu7 instanceof DiscreteCubeMerger && cvu8 instanceof DiscreteCubeMerger) {
            return new CubeVoxelShape(cvl9);
        }
        return new ArrayVoxelShape(cvl9, cvu6.getList(), cvu7.getList(), cvu8.getList());
    }
    
    public static boolean joinIsNotEmpty(final VoxelShape cwc1, final VoxelShape cwc2, final BooleanOp cvm) {
        if (cvm.apply(false, false)) {
            throw Util.<IllegalArgumentException>pauseInIde(new IllegalArgumentException());
        }
        if (cwc1 == cwc2) {
            return cvm.apply(true, true);
        }
        if (cwc1.isEmpty()) {
            return cvm.apply(false, !cwc2.isEmpty());
        }
        if (cwc2.isEmpty()) {
            return cvm.apply(!cwc1.isEmpty(), false);
        }
        final boolean boolean4 = cvm.apply(true, false);
        final boolean boolean5 = cvm.apply(false, true);
        for (final Direction.Axis a9 : AxisCycle.AXIS_VALUES) {
            if (cwc1.max(a9) < cwc2.min(a9) - 1.0E-7) {
                return boolean4 || boolean5;
            }
            if (cwc2.max(a9) < cwc1.min(a9) - 1.0E-7) {
                return boolean4 || boolean5;
            }
        }
        final IndexMerger cvu6 = createIndexMerger(1, cwc1.getCoords(Direction.Axis.X), cwc2.getCoords(Direction.Axis.X), boolean4, boolean5);
        final IndexMerger cvu7 = createIndexMerger(cvu6.getList().size() - 1, cwc1.getCoords(Direction.Axis.Y), cwc2.getCoords(Direction.Axis.Y), boolean4, boolean5);
        final IndexMerger cvu8 = createIndexMerger((cvu6.getList().size() - 1) * (cvu7.getList().size() - 1), cwc1.getCoords(Direction.Axis.Z), cwc2.getCoords(Direction.Axis.Z), boolean4, boolean5);
        return joinIsNotEmpty(cvu6, cvu7, cvu8, cwc1.shape, cwc2.shape, cvm);
    }
    
    private static boolean joinIsNotEmpty(final IndexMerger cvu1, final IndexMerger cvu2, final IndexMerger cvu3, final DiscreteVoxelShape cvr4, final DiscreteVoxelShape cvr5, final BooleanOp cvm) {
        return !cvu1.forMergedIndexes((integer6, integer7, integer8) -> cvu2.forMergedIndexes((integer7, integer8, integer9) -> cvu3.forMergedIndexes((integer8, integer9, integer10) -> !cvm.apply(cvr4.isFullWide(integer6, integer7, integer8), cvr5.isFullWide(integer7, integer8, integer9)))));
    }
    
    public static double collide(final Direction.Axis a, final AABB cvc, final Stream<VoxelShape> stream, double double4) {
        final Iterator<VoxelShape> iterator6 = stream.iterator();
        while (iterator6.hasNext()) {
            if (Math.abs(double4) < 1.0E-7) {
                return 0.0;
            }
            double4 = iterator6.next().collide(a, cvc, double4);
        }
        return double4;
    }
    
    public static double collide(final Direction.Axis a, final AABB cvc, final LevelReader bjw, final double double4, final CollisionContext cvn, final Stream<VoxelShape> stream) {
        return collide(cvc, bjw, double4, cvn, AxisCycle.between(a, Direction.Axis.Z), stream);
    }
    
    private static double collide(final AABB cvc, final LevelReader bjw, double double3, final CollisionContext cvn, final AxisCycle fi, final Stream<VoxelShape> stream) {
        if (cvc.getXsize() < 1.0E-6 || cvc.getYsize() < 1.0E-6 || cvc.getZsize() < 1.0E-6) {
            return double3;
        }
        if (Math.abs(double3) < 1.0E-7) {
            return 0.0;
        }
        final AxisCycle fi2 = fi.inverse();
        final Direction.Axis a9 = fi2.cycle(Direction.Axis.X);
        final Direction.Axis a10 = fi2.cycle(Direction.Axis.Y);
        final Direction.Axis a11 = fi2.cycle(Direction.Axis.Z);
        final BlockPos.MutableBlockPos a12 = new BlockPos.MutableBlockPos();
        final int integer13 = Mth.floor(cvc.min(a9) - 1.0E-7) - 1;
        final int integer14 = Mth.floor(cvc.max(a9) + 1.0E-7) + 1;
        final int integer15 = Mth.floor(cvc.min(a10) - 1.0E-7) - 1;
        final int integer16 = Mth.floor(cvc.max(a10) + 1.0E-7) + 1;
        final double double4 = cvc.min(a11) - 1.0E-7;
        final double double5 = cvc.max(a11) + 1.0E-7;
        final boolean boolean21 = double3 > 0.0;
        final int integer17 = boolean21 ? (Mth.floor(cvc.max(a11) - 1.0E-7) - 1) : (Mth.floor(cvc.min(a11) + 1.0E-7) + 1);
        int integer18 = lastC(double3, double4, double5);
        final int integer19 = boolean21 ? 1 : -1;
        int integer20 = integer17;
        while (true) {
            if (boolean21) {
                if (integer20 > integer18) {
                    break;
                }
            }
            else if (integer20 < integer18) {
                break;
            }
            for (int integer21 = integer13; integer21 <= integer14; ++integer21) {
                for (int integer22 = integer15; integer22 <= integer16; ++integer22) {
                    int integer23 = 0;
                    if (integer21 == integer13 || integer21 == integer14) {
                        ++integer23;
                    }
                    if (integer22 == integer15 || integer22 == integer16) {
                        ++integer23;
                    }
                    if (integer20 == integer17 || integer20 == integer18) {
                        ++integer23;
                    }
                    if (integer23 < 3) {
                        a12.set(fi2, integer21, integer22, integer20);
                        final BlockState byg29 = bjw.getBlockState(a12);
                        if (integer23 != 1 || byg29.hasLargeCollisionShape()) {
                            if (integer23 != 2 || byg29.getBlock() == Blocks.MOVING_PISTON) {
                                double3 = byg29.getCollisionShape(bjw, a12, cvn).collide(a11, cvc.move(-a12.getX(), -a12.getY(), -a12.getZ()), double3);
                                if (Math.abs(double3) < 1.0E-7) {
                                    return 0.0;
                                }
                                integer18 = lastC(double3, double4, double5);
                            }
                        }
                    }
                }
            }
            integer20 += integer19;
        }
        final double[] arr25 = { double3 };
        final Object o;
        stream.forEach(cwc -> o[0] = cwc.collide(a11, cvc, o[0]));
        return arr25[0];
    }
    
    private static int lastC(final double double1, final double double2, final double double3) {
        return (double1 > 0.0) ? (Mth.floor(double3 + double1) + 1) : (Mth.floor(double2 + double1) - 1);
    }
    
    public static boolean blockOccudes(final VoxelShape cwc1, final VoxelShape cwc2, final Direction fp) {
        if (cwc1 == block() && cwc2 == block()) {
            return true;
        }
        if (cwc2.isEmpty()) {
            return false;
        }
        final Direction.Axis a4 = fp.getAxis();
        final Direction.AxisDirection b5 = fp.getAxisDirection();
        final VoxelShape cwc3 = (b5 == Direction.AxisDirection.POSITIVE) ? cwc1 : cwc2;
        final VoxelShape cwc4 = (b5 == Direction.AxisDirection.POSITIVE) ? cwc2 : cwc1;
        final BooleanOp cvm8 = (b5 == Direction.AxisDirection.POSITIVE) ? BooleanOp.ONLY_FIRST : BooleanOp.ONLY_SECOND;
        return DoubleMath.fuzzyEquals(cwc3.max(a4), 1.0, 1.0E-7) && DoubleMath.fuzzyEquals(cwc4.min(a4), 0.0, 1.0E-7) && !joinIsNotEmpty(new SliceShape(cwc3, a4, cwc3.shape.getSize(a4) - 1), new SliceShape(cwc4, a4, 0), cvm8);
    }
    
    public static VoxelShape getFaceShape(final VoxelShape cwc, final Direction fp) {
        if (cwc == block()) {
            return block();
        }
        final Direction.Axis a5 = fp.getAxis();
        boolean boolean3;
        int integer4;
        if (fp.getAxisDirection() == Direction.AxisDirection.POSITIVE) {
            boolean3 = DoubleMath.fuzzyEquals(cwc.max(a5), 1.0, 1.0E-7);
            integer4 = cwc.shape.getSize(a5) - 1;
        }
        else {
            boolean3 = DoubleMath.fuzzyEquals(cwc.min(a5), 0.0, 1.0E-7);
            integer4 = 0;
        }
        if (!boolean3) {
            return empty();
        }
        return new SliceShape(cwc, a5, integer4);
    }
    
    public static boolean mergedFaceOccludes(final VoxelShape cwc1, final VoxelShape cwc2, final Direction fp) {
        if (cwc1 == block() || cwc2 == block()) {
            return true;
        }
        final Direction.Axis a4 = fp.getAxis();
        final Direction.AxisDirection b5 = fp.getAxisDirection();
        VoxelShape cwc3 = (b5 == Direction.AxisDirection.POSITIVE) ? cwc1 : cwc2;
        VoxelShape cwc4 = (b5 == Direction.AxisDirection.POSITIVE) ? cwc2 : cwc1;
        if (!DoubleMath.fuzzyEquals(cwc3.max(a4), 1.0, 1.0E-7)) {
            cwc3 = empty();
        }
        if (!DoubleMath.fuzzyEquals(cwc4.min(a4), 0.0, 1.0E-7)) {
            cwc4 = empty();
        }
        return !joinIsNotEmpty(block(), joinUnoptimized(new SliceShape(cwc3, a4, cwc3.shape.getSize(a4) - 1), new SliceShape(cwc4, a4, 0), BooleanOp.OR), BooleanOp.ONLY_FIRST);
    }
    
    public static boolean faceShapeOccludes(final VoxelShape cwc1, final VoxelShape cwc2) {
        return cwc1 == block() || cwc2 == block() || ((!cwc1.isEmpty() || !cwc2.isEmpty()) && !joinIsNotEmpty(block(), joinUnoptimized(cwc1, cwc2, BooleanOp.OR), BooleanOp.ONLY_FIRST));
    }
    
    @VisibleForTesting
    protected static IndexMerger createIndexMerger(final int integer, final DoubleList doubleList2, final DoubleList doubleList3, final boolean boolean4, final boolean boolean5) {
        final int integer2 = doubleList2.size() - 1;
        final int integer3 = doubleList3.size() - 1;
        if (doubleList2 instanceof CubePointRange && doubleList3 instanceof CubePointRange) {
            final long long8 = lcm(integer2, integer3);
            if (integer * long8 <= 256L) {
                return new DiscreteCubeMerger(integer2, integer3);
            }
        }
        if (doubleList2.getDouble(integer2) < doubleList3.getDouble(0) - 1.0E-7) {
            return new NonOverlappingMerger(doubleList2, doubleList3, false);
        }
        if (doubleList3.getDouble(integer3) < doubleList2.getDouble(0) - 1.0E-7) {
            return new NonOverlappingMerger(doubleList3, doubleList2, true);
        }
        if (integer2 != integer3 || !Objects.equals(doubleList2, doubleList3)) {
            return new IndirectMerger(doubleList2, doubleList3, boolean4, boolean5);
        }
        if (doubleList2 instanceof IdenticalMerger) {
            return (IndexMerger)doubleList2;
        }
        if (doubleList3 instanceof IdenticalMerger) {
            return (IndexMerger)doubleList3;
        }
        return new IdenticalMerger(doubleList2);
    }
    
    static {
        final BitSetDiscreteVoxelShape cvr1;
        BLOCK = Util.<CubeVoxelShape>make(() -> {
            cvr1 = new BitSetDiscreteVoxelShape(1, 1, 1);
            cvr1.setFull(0, 0, 0, true, true);
            return new CubeVoxelShape(cvr1);
        });
        INFINITY = box(Double.NEGATIVE_INFINITY, Double.NEGATIVE_INFINITY, Double.NEGATIVE_INFINITY, Double.POSITIVE_INFINITY, Double.POSITIVE_INFINITY, Double.POSITIVE_INFINITY);
        EMPTY = new ArrayVoxelShape(new BitSetDiscreteVoxelShape(0, 0, 0), (DoubleList)new DoubleArrayList(new double[] { 0.0 }), (DoubleList)new DoubleArrayList(new double[] { 0.0 }), (DoubleList)new DoubleArrayList(new double[] { 0.0 }));
    }
    
    public interface DoubleLineConsumer {
        void consume(final double double1, final double double2, final double double3, final double double4, final double double5, final double double6);
    }
}
