package net.minecraft.world.scores;

import java.util.AbstractList;
import org.apache.logging.log4j.LogManager;
import java.util.Iterator;
import java.util.Collection;
import net.minecraft.nbt.StringTag;
import net.minecraft.nbt.Tag;
import net.minecraft.world.scores.criteria.ObjectiveCriteria;
import net.minecraft.ChatFormatting;
import net.minecraft.network.chat.Component;
import net.minecraft.nbt.ListTag;
import net.minecraft.nbt.CompoundTag;
import org.apache.logging.log4j.Logger;
import net.minecraft.world.level.saveddata.SavedData;

public class ScoreboardSaveData extends SavedData {
    private static final Logger LOGGER;
    private Scoreboard scoreboard;
    private CompoundTag delayLoad;
    
    public ScoreboardSaveData() {
        super("scoreboard");
    }
    
    public void setScoreboard(final Scoreboard cwi) {
        this.scoreboard = cwi;
        if (this.delayLoad != null) {
            this.load(this.delayLoad);
        }
    }
    
    @Override
    public void load(final CompoundTag jt) {
        if (this.scoreboard == null) {
            this.delayLoad = jt;
            return;
        }
        this.loadObjectives(jt.getList("Objectives", 10));
        this.scoreboard.loadPlayerScores(jt.getList("PlayerScores", 10));
        if (jt.contains("DisplaySlots", 10)) {
            this.loadDisplaySlots(jt.getCompound("DisplaySlots"));
        }
        if (jt.contains("Teams", 9)) {
            this.loadTeams(jt.getList("Teams", 10));
        }
    }
    
    protected void loadTeams(final ListTag jz) {
        for (int integer3 = 0; integer3 < jz.size(); ++integer3) {
            final CompoundTag jt4 = jz.getCompound(integer3);
            String string5 = jt4.getString("Name");
            if (string5.length() > 16) {
                string5 = string5.substring(0, 16);
            }
            final PlayerTeam cwg6 = this.scoreboard.addPlayerTeam(string5);
            final Component lf7 = Component.Serializer.fromJson(jt4.getString("DisplayName"));
            if (lf7 != null) {
                cwg6.setDisplayName(lf7);
            }
            if (jt4.contains("TeamColor", 8)) {
                cwg6.setColor(ChatFormatting.getByName(jt4.getString("TeamColor")));
            }
            if (jt4.contains("AllowFriendlyFire", 99)) {
                cwg6.setAllowFriendlyFire(jt4.getBoolean("AllowFriendlyFire"));
            }
            if (jt4.contains("SeeFriendlyInvisibles", 99)) {
                cwg6.setSeeFriendlyInvisibles(jt4.getBoolean("SeeFriendlyInvisibles"));
            }
            if (jt4.contains("MemberNamePrefix", 8)) {
                final Component lf8 = Component.Serializer.fromJson(jt4.getString("MemberNamePrefix"));
                if (lf8 != null) {
                    cwg6.setPlayerPrefix(lf8);
                }
            }
            if (jt4.contains("MemberNameSuffix", 8)) {
                final Component lf8 = Component.Serializer.fromJson(jt4.getString("MemberNameSuffix"));
                if (lf8 != null) {
                    cwg6.setPlayerSuffix(lf8);
                }
            }
            if (jt4.contains("NameTagVisibility", 8)) {
                final Team.Visibility b8 = Team.Visibility.byName(jt4.getString("NameTagVisibility"));
                if (b8 != null) {
                    cwg6.setNameTagVisibility(b8);
                }
            }
            if (jt4.contains("DeathMessageVisibility", 8)) {
                final Team.Visibility b8 = Team.Visibility.byName(jt4.getString("DeathMessageVisibility"));
                if (b8 != null) {
                    cwg6.setDeathMessageVisibility(b8);
                }
            }
            if (jt4.contains("CollisionRule", 8)) {
                final Team.CollisionRule a8 = Team.CollisionRule.byName(jt4.getString("CollisionRule"));
                if (a8 != null) {
                    cwg6.setCollisionRule(a8);
                }
            }
            this.loadTeamPlayers(cwg6, jt4.getList("Players", 8));
        }
    }
    
    protected void loadTeamPlayers(final PlayerTeam cwg, final ListTag jz) {
        for (int integer4 = 0; integer4 < jz.size(); ++integer4) {
            this.scoreboard.addPlayerToTeam(jz.getString(integer4), cwg);
        }
    }
    
    protected void loadDisplaySlots(final CompoundTag jt) {
        for (int integer3 = 0; integer3 < 19; ++integer3) {
            if (jt.contains("slot_" + integer3, 8)) {
                final String string4 = jt.getString("slot_" + integer3);
                final Objective cwf5 = this.scoreboard.getObjective(string4);
                this.scoreboard.setDisplayObjective(integer3, cwf5);
            }
        }
    }
    
    protected void loadObjectives(final ListTag jz) {
        for (int integer3 = 0; integer3 < jz.size(); ++integer3) {
            final CompoundTag jt4 = jz.getCompound(integer3);
            final CompoundTag compoundTag;
            String string4;
            final Component lf5;
            final ObjectiveCriteria.RenderType a6;
            ObjectiveCriteria.byName(jt4.getString("CriteriaName")).ifPresent(cwl -> {
                string4 = compoundTag.getString("Name");
                if (string4.length() > 16) {
                    string4 = string4.substring(0, 16);
                }
                lf5 = Component.Serializer.fromJson(compoundTag.getString("DisplayName"));
                a6 = ObjectiveCriteria.RenderType.byId(compoundTag.getString("RenderType"));
                this.scoreboard.addObjective(string4, cwl, lf5, a6);
                return;
            });
        }
    }
    
    @Override
    public CompoundTag save(final CompoundTag jt) {
        if (this.scoreboard == null) {
            ScoreboardSaveData.LOGGER.warn("Tried to save scoreboard without having a scoreboard...");
            return jt;
        }
        jt.put("Objectives", this.saveObjectives());
        jt.put("PlayerScores", this.scoreboard.savePlayerScores());
        jt.put("Teams", this.saveTeams());
        this.saveDisplaySlots(jt);
        return jt;
    }
    
    protected ListTag saveTeams() {
        final ListTag jz2 = new ListTag();
        final Collection<PlayerTeam> collection3 = this.scoreboard.getPlayerTeams();
        for (final PlayerTeam cwg5 : collection3) {
            final CompoundTag jt6 = new CompoundTag();
            jt6.putString("Name", cwg5.getName());
            jt6.putString("DisplayName", Component.Serializer.toJson(cwg5.getDisplayName()));
            if (cwg5.getColor().getId() >= 0) {
                jt6.putString("TeamColor", cwg5.getColor().getName());
            }
            jt6.putBoolean("AllowFriendlyFire", cwg5.isAllowFriendlyFire());
            jt6.putBoolean("SeeFriendlyInvisibles", cwg5.canSeeFriendlyInvisibles());
            jt6.putString("MemberNamePrefix", Component.Serializer.toJson(cwg5.getPlayerPrefix()));
            jt6.putString("MemberNameSuffix", Component.Serializer.toJson(cwg5.getPlayerSuffix()));
            jt6.putString("NameTagVisibility", cwg5.getNameTagVisibility().name);
            jt6.putString("DeathMessageVisibility", cwg5.getDeathMessageVisibility().name);
            jt6.putString("CollisionRule", cwg5.getCollisionRule().name);
            final ListTag jz3 = new ListTag();
            for (final String string9 : cwg5.getPlayers()) {
                ((AbstractList<StringTag>)jz3).add(StringTag.valueOf(string9));
            }
            jt6.put("Players", jz3);
            ((AbstractList<CompoundTag>)jz2).add(jt6);
        }
        return jz2;
    }
    
    protected void saveDisplaySlots(final CompoundTag jt) {
        final CompoundTag jt2 = new CompoundTag();
        boolean boolean4 = false;
        for (int integer5 = 0; integer5 < 19; ++integer5) {
            final Objective cwf6 = this.scoreboard.getDisplayObjective(integer5);
            if (cwf6 != null) {
                jt2.putString("slot_" + integer5, cwf6.getName());
                boolean4 = true;
            }
        }
        if (boolean4) {
            jt.put("DisplaySlots", jt2);
        }
    }
    
    protected ListTag saveObjectives() {
        final ListTag jz2 = new ListTag();
        final Collection<Objective> collection3 = this.scoreboard.getObjectives();
        for (final Objective cwf5 : collection3) {
            if (cwf5.getCriteria() == null) {
                continue;
            }
            final CompoundTag jt6 = new CompoundTag();
            jt6.putString("Name", cwf5.getName());
            jt6.putString("CriteriaName", cwf5.getCriteria().getName());
            jt6.putString("DisplayName", Component.Serializer.toJson(cwf5.getDisplayName()));
            jt6.putString("RenderType", cwf5.getRenderType().getId());
            ((AbstractList<CompoundTag>)jz2).add(jt6);
        }
        return jz2;
    }
    
    static {
        LOGGER = LogManager.getLogger();
    }
}
